#include "../src/zeusmp.def"
c-----------------------------------------------------------------------
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
       subroutine diff
c
c ZEUS-MP Post-processor: DIFF.
c
c PURPOSE:  This program reads two USERDUMPS from a ZEUS-MP run
c           and assembles them into a single file whose entries
c           are the difference of the two corresponding values.
c           Assumes the input files have
c           data for the entire physical mesh (run SPLICER first).
c
c LAST MODIFIED: RAF, 11/21/96.
c
c.......................................................................
c
       implicit NONE
#include "../src/param.h"
c
       integer  irestart
       REAL     tdump,dtdump
       character id*2, resfile*15
c
       integer  ntiles(3)
       logical  periodic(3)
c
       integer  nbl,igrid,imin,imax,jmin,jmax,kmin,kmax,i,j,k
       logical  lgrid
c
       integer n, indx, icmb, ndump
       integer nx1z, nx2z, nx3z
c
       integer i1bmx, j1bmx, k1bmx, i2bmx, j2bmx, k2bmx, i3bmx, j3bmx
     &       , k3bmx, idmax, jdmax, kdmax, iemax, jemax, kemax, i1amx
     &       , j1amx, k1amx, i2amx, j2amx, k2amx, i3amx, j3amx, k3amx
     &       , iv1mx, jv1mx, kv1mx, iv2mx, jv2mx, kv2mx, iv3mx, jv3mx
     &       , kv3mx
       REAL x1bmx, x2bmx, x3bmx, ddmax, demax, x1amx, x2amx, x3amx
     &    , v1max, v2max, v3max
       REAL x1b, x2b, x3b, v1, v2, v3, x1a, x2a, x3a, dd, de
c
       REAL     x1min,x1max,x1rat,dx1min,dfndx1r,x1r,deltx1r,errx1r
     &         ,x2min,x2max,x2rat,dx2min,dfndx2r,x2r,deltx2r,errx2r
     &         ,x3min,x3max,x3rat,dx3min,dfndx3r,x3r,deltx3r,errx3r
       REAL     x1,x2,x3,y1,y2,y3,d,e,w,den,dlo,edn,eng,u,v
c
c       REAL     rtol,atol
       REAL     tol
c
       character*8 phrase
       character*5 root1, root2
       character*9 usr(64), cmb
       character*120 line
c
       namelist /rescon/ irestart,tdump,dtdump,id,resfile
       namelist /mpitop/ ntiles, periodic
       namelist /ggen1/  nbl,x1min,x1max,igrid,x1rat,dx1min,lgrid
       namelist /ggen2/  nbl,x2min,x2max,igrid,x2rat,dx2min,lgrid
       namelist /ggen3/  nbl,x3min,x3max,igrid,x3rat,dx3min,lgrid
#ifdef SINGLE_PRECISION
c       data rtol,atol / 1.0e-03, 1.0e-04 /
       data tol / 1.0e-06 /
#else
cc       data rtol,atol / 1.0d-06, 1.0d-07 /
c       data rtol,atol / 1.0d-12, 1.0d-13 /
       data tol / 1.0e-09 /
#endif
c
c.......................................................................
c
c Read ZEUS-MP input file "zmp_inp" to determine the number of zones
c in each direction in the physical mesh (nx1z,nx2z,nx3z), and the 
c number of tiles in each direction "ntiles".
c
       open(1,file='zmp_inp',status='old')
c
c   Look for and read the "mpitop" and "ggen*" namelists.
c
       do 10 n=1,100
         read(1,"(a8)") phrase
         if (phrase(3:8) .eq. 'mpitop') goto 20
   10  continue
       write(*,"(/'DIFF: ERROR -- Did not find mpitop namelist.')")
       stop
c
   20  continue
       backspace (1)
       read(1,mpitop)
       write(*,"('DIFF: ntiles is ',3i6)") ntiles(1),ntiles(2)
     & , ntiles(3)
c
c   Read the "rescon" namelist to find the run's "id".
c
       irestart = 0
        tdump = 0.0
       dtdump = 0.0
       id     = 'aa'
       resfile= 'resaa000000.000'
       read (1,rescon)
       write(*,"(/'DIFF: id is ',a)") id
c
       imax  = 3
       nbl   = 1
       lgrid = .false.
c
   30  continue
       read(1,ggen1)
       imin = imax
       imax = imax + nbl
       if (.not. lgrid) go to 30
c
c       nx1z = ( imax - imin ) / ntiles(1)
       nx1z = imax - imin
c
       jmax  = 3
       nbl   = 1
       lgrid = .false.
c
   40  continue
       read(1,ggen2)
       jmin = jmax
       jmax = jmax + nbl
       if (.not. lgrid) go to 40
c
c       nx2z = ( jmax - jmin ) / ntiles(2)
       nx2z = jmax - jmin
c
       kmax  = 3
       nbl   = 1
       lgrid = .false.
c
   50  continue
       read(1,ggen3)
       kmin = kmax
       kmax = kmax + nbl
       if (.not. lgrid) go to 40
c
c       nx3z = ( kmax - kmin ) / ntiles(3)
       nx3z = kmax - kmin
       write(*,"('DIFF: nx1z,nx2z,nx3z are ',3i6)") nx1z,nx2z,nx3z
       close (1)
c.......................................................................
c
c Determine USERDUMP file names and try to open them.
c
       write(*,"(/'DIFF: File names must be of the form ccccc.nnn')")
       write(*,"(/'DIFF: Enter root name of first file (5 chars)'/)")
       read(*,"(a)") root1
       write(*,"(/'DIFF: Enter root name of second file (5 chars)'/)")
       read(*,"(a)") root2
c
c   Ask for the dump number.  Loop back to this point when finished
c   with this dump number.  Negative input dump number terminates
c   execution.
c
       indx = 0
       icmb = 0
   60  continue
c
c Clean up any opened files.
c
       do 65 i=1,indx
         close (i)
   65  continue
       if (icmb .ne. 0) close (icmb)
c
       write(*,"(/'DIFF: Enter the desired dump number (<0 quits)'/)")
       read(*,*) ndump
       if (ndump .lt. 0) then
         return
       endif
c
       indx = 1
       write(usr(indx),"(a5,'.',i3.3)") root1,ndump
       open(indx,file=usr(indx),status='old',err=999)
c
       indx = indx + 1
       write(usr(indx),"(a5,'.',i3.3)") root2,ndump
       open(indx,file=usr(indx),status='old',err=999)
c.......................................................................
c
c   Name the combined file and open it.
c
       icmb = indx + 1
       write(cmb,"(a3,a2,'.',i3.3)") 'dif',id,ndump
       open(icmb,file=cmb)
c
c   For the first input file only, copy the first 3 lines (header) into
c   the combined file.   For the other, just skip the header.
c
       do 100 indx=1,2
             do 100 i=1,3
               if (indx .eq. 1) then
                 do 93 n=1,120
                   line(n:n) = ' '
   93            continue
                 read(indx,"(a)") line
                 do 95 n=120,1,-1
                   if (line(n:n) .ne. ' ') goto 96
   95            continue
                 n = 0
   96            continue
                 if (n .eq. 0) then
                   write(icmb,"(a)") ''
                 else
                   write(icmb,"(a)") line(1:n)
                 endif
               else
                 read(indx,"(a)")
               endif
  100  continue
c
  101  format(5e16.9)
  102  format(5e16.9)
  103  format(4e16.9)
  104  format(4e16.9)
c
c   Copy the zone centered quantities d and e.
c
c   We can do all k,j values in one loop.
c
       x1bmx = 0.
       i1bmx = 1
       j1bmx = 1
       k1bmx = 1
       x2bmx = 0.
       i2bmx = 1
       j2bmx = 1
       k2bmx = 1
       x3bmx = 0.
       i3bmx = 1
       j3bmx = 1
       k3bmx = 1
       ddmax = 0.
       idmax = 1
       jdmax = 1
       kdmax = 1
       demax = 0.
       iemax = 1
       jemax = 1
       kemax = 1
       x1amx = 0.
       i1amx = 1
       j1amx = 1
       k1amx = 1
       x2amx = 0.
       i2amx = 1
       j2amx = 1
       k2amx = 1
       x3amx = 0.
       i3amx = 1
       j3amx = 1
       k3amx = 1
       v1max = 0.
       iv1mx = 1
       jv1mx = 1
       kv1mx = 1
       v2max = 0.
       iv2mx = 1
       jv2mx = 1
       kv2mx = 1
       v3max = 0.
       iv3mx = 1
       jv3mx = 1
       kv3mx = 1
c
       do 190 k=1,nx3z
         do 140 j=1,nx2z
           do 130 i=1,nx1z
c
c   For each file, copy nx1z rows at a time into the combined file.
c
             read(1,101) x1,x2,x3,dlo,eng
             read(2,101) y1,y2,y3,den,edn
             d = den - dlo
             e = edn - eng
c
c   Do the coordinates match?
c
c             if (x1.ne.y1 .or. x2.ne.y2 .or. x3.ne.y3) then
c               write(*,"(/'DIFF: Different coordinates; zone-ctr.')")
c               write(*,"('DIFF: i,j,k are ',3i6/)") i,j,k
c             endif
c             x1b = abs(y1-x1)/max(x1,y1,tol)
             x1b = y1 - x1
             if (x1b.gt.x1bmx) then
               x1bmx = x1b
               i1bmx = i
               j1bmx = j
               k1bmx = k
             endif
c             x2b = abs(y2-x2)/max(x2,y2,tol)
             x2b = y2 - x2
             if (x2b.gt.x2bmx) then
               x2bmx = x2b
               i2bmx = i
               j2bmx = j
               k2bmx = k
             endif
c             x3b = abs(y3-x3)/max(x3,y3,tol)
             x3b = y3 - x3
             if (x3b.gt.x3bmx) then
               x3bmx = x3b
               i3bmx = i
               j3bmx = j
               k3bmx = k
             endif
c
c   Are the differences within tolerance rtol?  Issue warnings if so.
c
c             if (max(den,dlo).gt.atol .and. 
c     1           abs(d)/max(den,dlo,tiny).gt.rtol) then 
c               write(*,"(/'DIFF: Difference in d exceeds ',e9.2)") rtol
c               write(*,"('DIFF: d1 = ',e16.9,'  d2 = ',e16.9)") dlo,den
c               write(*,"('DIFF: i,j,k are ',3i6/)") i,j,k
c             endif
c             if (max(edn,eng).gt.atol .and. 
c     1           abs(e)/max(edn,eng,tiny).gt.rtol) then 
c               write(*,"(/'DIFF: Difference in e exceeds ',e9.2)") rtol
c               write(*,"('DIFF: e1 = ',e16.9,'  e2 = ',e16.9)") eng,edn
c               write(*,"('DIFF: i,j,k are ',3i6/)") i,j,k
c             endif
c
c             dd = abs(d)/max(den,dlo,tol)
             dd = d
             if (dd.gt.ddmax) then
               ddmax = dd
               idmax = i
               jdmax = j
               kdmax = k
             endif
c             de = abs(e)/max(edn,eng,tol)
             de = e
             if (de.gt.demax) then
               demax = de
               iemax = i
               jemax = j
               kemax = k
             endif
c
             write(icmb,102) x1,x2,x3,d,e
  130      continue
  140    continue
  190  continue
c
       write(*,"(/'DIFF: Largest difference in x1b is ',1pe11.4)") x1bmx
       write(*,"('DIFF: i,j,k are ',3i6/)") i1bmx,j1bmx,k1bmx
       write(*,"(/'DIFF: Largest difference in x2b is ',1pe11.4)") x2bmx
       write(*,"('DIFF: i,j,k are ',3i6/)") i2bmx,j2bmx,k2bmx
       write(*,"(/'DIFF: Largest difference in x3b is ',1pe11.4)") x3bmx
       write(*,"('DIFF: i,j,k are ',3i6/)") i3bmx,j3bmx,k3bmx
       write(*,"(/'DIFF: Largest difference in   d is ',1pe11.4)") ddmax
       write(*,"('DIFF: i,j,k are ',3i6/)") idmax,jdmax,kdmax
       write(*,"(/'DIFF: Largest difference in   e is ',1pe11.4)") demax
       write(*,"('DIFF: i,j,k are ',3i6/)") iemax,jemax,kemax
#ifndef RAD_EXP
c
c   Copy the 1-face centered quantities v1 (and b1).
c
c   For the first tile only, copy the next 2 lines (header) into
c   the combined file.   For the others, just skip the header.
c
       do 200 indx=1,2
             do 200 i=1,2
               if (indx .eq. 1) then
                 do 193 n=1,120
                   line(n:n) = ' '
  193            continue
                 read(indx,"(a)") line
                 do 195 n=120,1,-1
                   if (line(n:n) .ne. ' ') goto 196
  195            continue
                 n = 0
  196            continue
                 if (n .eq. 0) then
                   write(icmb,"(a)") ''
                 else
                   write(icmb,"(a)") line(1:n)
                 endif
               else
                 read(indx,"(a)")
               endif
  200  continue
c
c   We can do all k,j,i values in one loop.
c
       do 290 k=1,nx3z
         do 240 j=1,nx2z
           do 230 i=1,nx1z+1
             read(1,103) x1,x2,x3,v
             read(2,103) y1,y2,y3,u
             w = u - v
c             if (x1.ne.y1 .or. x2.ne.y2 .or. x3.ne.y3) then
c               write(*,"(/'DIFF: Different coordinates; 1-face ctr.')")
c               write(*,"('DIFF: i,j,k are ',3i6/)") i,j,k
c             endif
c             if (max(abs(u),abs(v)).gt.atol .and. 
c     1           abs(w)/max(abs(u),abs(v),tiny).gt.rtol) then 
c               write(*,"(/'DIFF: Difference in v1 exceeds ',e9.2)") rtol
c               write(*,"('DIFF: v11 = ',e16.9,'  v12 = ',e16.9)") v,u
c               write(*,"('DIFF: i,j,k are ',3i6/)") i,j,k
c             endif
c             x1a = abs(y1-x1)/max(x1,y1,tol)
             x1a = y1 - x1
             if (x1a.gt.x1amx) then
               x1amx = x1a
               i1amx = i
               j1amx = j
               k1amx = k
             endif
c             v1 = abs(w)/max(u,v,tol)
             v1 = w
             if (v1.gt.v1max) then
               v1max = v1
               iv1mx = i
               jv1mx = j
               kv1mx = k
             endif
             write(icmb,104) x1,x2,x3,w
  230      continue
  240    continue
  290  continue
       write(*,"(/'DIFF: Largest difference in x1a is ',1pe11.4)") x1amx
       write(*,"('DIFF: i,j,k are ',3i6/)") i1amx,j1amx,k1amx
       write(*,"(/'DIFF: Largest difference in  v1 is ',1pe11.4)") v1max
       write(*,"('DIFF: i,j,k are ',3i6/)") iv1mx,jv1mx,kv1mx
c
c   Copy the 2-face centered quantities v2 (and b2).
c
c   For the first tile only, copy the next 2 lines (header) into
c   the combined file.   For the others, just skip the header.
c
           do 300 indx=1,2
             do 300 i=1,2
               if (indx .eq. 1) then
                 do 293 n=1,120
                   line(n:n) = ' '
  293            continue
                 read(indx,"(a)") line
                 do 295 n=120,1,-1
                   if (line(n:n) .ne. ' ') goto 296
  295            continue
                 n = 0
  296            continue
                 if (n .eq. 0) then
                   write(icmb,"(a)") ''
                 else
                   write(icmb,"(a)") line(1:n)
                 endif
               else
                 read(indx,"(a)")
               endif
  300  continue
c
c   We can do all k,j,i values in one loop.
c
       do 390 k=1,nx3z
         do 340 j=1,nx2z+1
c
c     For each file, copy nx1z rows at a time into the combined file.
c
           do 330 i=1,nx1z
             read(1,103) x1,x2,x3,v
             read(2,103) y1,y2,y3,u
             w = u - v
c             if (x1.ne.y1 .or. x2.ne.y2 .or. x3.ne.y3) then
c               write(*,"(/'DIFF: Different coordinates; 2-face ctr.')")
c               write(*,"('DIFF: i,j,k are ',3i6/)") i,j,k
c             endif
c             if (max(abs(u),abs(v)).gt.atol .and. 
c     1           abs(w)/max(abs(u),abs(v),tiny).gt.rtol) then 
c               write(*,"(/'DIFF: Difference in v2 exceeds ',e9.2)") rtol
c               write(*,"('DIFF: v21 = ',e16.9,'  v22 = ',e16.9)") v,u
c               write(*,"('DIFF: i,j,k are ',3i6/)") i,j,k
c             endif
c             x2a = abs(y2-x2)/max(x2,y2,tol)
             x2a = y2 - x2
             if (x2a.gt.x2amx) then
               x2amx = x2a
               i2amx = i
               j2amx = j
               k2amx = k
             endif
c             v2 = abs(w)/max(u,v,tol)
             v2 = w
             if (v2.gt.v2max) then
               v2max = v2
               iv2mx = i
               jv2mx = j
               kv2mx = k
             endif
             write(icmb,104) x1,x2,x3,w
  330      continue
  340    continue
  390  continue
       write(*,"(/'DIFF: Largest difference in x2a is ',1pe11.4)") x2amx
       write(*,"('DIFF: i,j,k are ',3i6/)") i2amx,j2amx,k2amx
       write(*,"(/'DIFF: Largest difference in  v2 is ',1pe11.4)") v2max
       write(*,"('DIFF: i,j,k are ',3i6/)") iv2mx,jv2mx,kv2mx
c
c   Copy the 3-face centered quantities v3 (and b3).
c
c   For the first tile only, copy the next 2 lines (header) into
c   the combined file.   For the others, just skip the header.
c
           do 400 indx=1,2
             do 400 i=1,2
               if (indx .eq. 1) then
                 do 393 n=1,120
                   line(n:n) = ' '
  393            continue
                 read(indx,"(a)") line
                 do 395 n=120,1,-1
                   if (line(n:n) .ne. ' ') goto 396
  395            continue
                 n = 0
  396            continue
                 if (n .eq. 0) then
                   write(icmb,"(a)") ''
                 else
                   write(icmb,"(a)") line(1:n)
                 endif
               else
                 read(indx,"(a)")
               endif
  400  continue
c
c   We can do all k,j,i values in one loop.
c
       do 490 k=1,nx3z+1
         do 440 j=1,nx2z
           do 430 i=1,nx1z
             read(1,103) x1,x2,x3,v
             read(2,103) y1,y2,y3,u
             w = u - v
c             if (x1.ne.y1 .or. x2.ne.y2 .or. x3.ne.y3) then
c               write(*,"(/'DIFF: Different coordinates; 3-face ctr.')")
c               write(*,"('DIFF: i,j,k are ',3i6/)") i,j,k
c             endif
c             if (max(abs(u),abs(v)).gt.atol .and. 
c     1           abs(w)/max(abs(u),abs(v),tiny).gt.rtol) then 
c               write(*,"(/'DIFF: Difference in v3 exceeds ',e9.2)") rtol
c               write(*,"('DIFF: v31 = ',e16.9,'  v32 = ',e16.9)") v,u
c               write(*,"('DIFF: i,j,k are ',3i6/)") i,j,k
c             endif
c             x3a = abs(y3-x3)/max(x3,y3,tol)
             x3a = y3 - x3
             if (x3a.gt.x3amx) then
               x3amx = x3a
               i3amx = i
               j3amx = j
               k3amx = k
             endif
c             v3 = abs(w)/max(u,v,tol)
             v3 = w
             if (v3.gt.v3max) then
               v3max = v3
               iv3mx = i
               jv3mx = j
               kv3mx = k
             endif
             write(icmb,104) x1,x2,x3,w
  430      continue
  440    continue
  490  continue
       write(*,"(/'DIFF: Largest difference in x3a is ',1pe11.4)") x3amx
       write(*,"('DIFF: i,j,k are ',3i6/)") i3amx,j3amx,k3amx
       write(*,"(/'DIFF: Largest difference in  v3 is ',1pe11.4)") v3max
       write(*,"('DIFF: i,j,k are ',3i6/)") iv3mx,jv3mx,kv3mx
#endif /* not RAD_EXP */
c
c.......................................................................
c
c   Loop back up for the next dump.
c
       goto 60
c.......................................................................
c
c Error messages
c
  999  continue
       write(*,"(/'DIFF: ERROR -- could not open file ',a9
     & , ' on unit ',i4)") usr(indx),indx
c
       do 965 i=1,indx
         close (i)
  965  continue
       if (icmb .ne. 0) close (icmb)
       return
       end
