#include "../src/zeusmp.def"
c-----------------------------------------------------------------------
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
       subroutine splice
c
c ZEUS-MP Post-processor: SPLICE.
c
c PURPOSE:  This program reads all USERDUMPS from a ZEUS-MP run
c           and assembles them into a single huge file which looks
c           like one from a single-processor job with the same
c           physical mesh.
c
c LAST MODIFIED: RAF, 9/09/96.
c
c.......................................................................
c
       implicit NONE
#include "../src/param.h"
#undef MPI_USED
#include "../src/mpicom.h"
#include "../src/grid.h"
#include "../src/root.h"
c
       integer  irestart
c
       integer  nbl,igrid,imin,imax,jmin,jmax,kmin,kmax,i,j,k
       logical  lgrid
c
       integer n, indx, icmb, ndump
       integer it,jt,kt
       REAL     x1min,x1max,x1rat,dx1min,dfndx1r,x1r,deltx1r,errx1r
     &         ,x2min,x2max,x2rat,dx2min,dfndx2r,x2r,deltx2r,errx2r
     &         ,x3min,x3max,x3rat,dx3min,dfndx3r,x3r,deltx3r,errx3r
       REAL     x2bj(in),x3bk(in),den(in),edn(in),ern(in)
     &         ,ve(in)
       character*8 phrase
       character*15 usr(64), cmb
       character*120 line
       character*64 dataline
c
       namelist /rescon/ irestart,tdump,dtdump,id,resfile
       namelist /mpitop/ ntiles, periodic
       namelist /ggen1/  nbl,x1min,x1max,igrid,x1rat,dx1min,lgrid
       namelist /ggen2/  nbl,x2min,x2max,igrid,x2rat,dx2min,lgrid
       namelist /ggen3/  nbl,x3min,x3max,igrid,x3rat,dx3min,lgrid
c
c.......................................................................
c
c Read ZEUS-MP input file "zmp_inp" to determine the number of zones
c in each direction in the physical mesh (nx1z,nx2z,nx3z), and the 
c number of tiles in each direction "ntiles".
c
       open(1,file='zmp_inp',status='old')
c
c   Look for and read the "mpitop" and "ggen*" namelists.
c
       do 10 n=1,100
         read(1,"(a8)") phrase
         if (phrase(3:8) .eq. 'mpitop') goto 20
   10  continue
       write(*,"(/'SPLICE: ERROR -- Did not find mpitop namelist.')")
       close (1)
       return
c
   20  continue
       backspace (1)
       read(1,mpitop)
       write(*,"('SPLICE: ntiles is ',3i6)") ntiles(1),ntiles(2)
     & , ntiles(3)
c
c   Read the "rescon" namelist to find the run's "id".
c
       irestart = 0
        tdump = 0.0
       dtdump = 0.0
       id     = 'aa'
       resfile= 'resaa000000.000'
       read (1,rescon)
       write(*,"(/'SPLICE: id is ',a)") id
c
       imax  = 3
       nbl   = 1
       lgrid = .false.
c
   30  continue
       read(1,ggen1)
       imin = imax
       imax = imax + nbl
       if (.not. lgrid) go to 30
c
       nx1z = ( imax - imin ) / ntiles(1)
c
       jmax  = 3
       nbl   = 1
       lgrid = .false.
c
   40  continue
       read(1,ggen2)
       jmin = jmax
       jmax = jmax + nbl
       if (.not. lgrid) go to 40
c
       nx2z = ( jmax - jmin ) / ntiles(2)
c
       kmax  = 3
       nbl   = 1
       lgrid = .false.
c
   50  continue
       read(1,ggen3)
       kmin = kmax
       kmax = kmax + nbl
       if (.not. lgrid) go to 40
c
       nx3z = ( kmax - kmin ) / ntiles(3)
       write(*,"('SPLICE: nx1z,nx2z,nx3z are ',3i6)") nx1z,nx2z,nx3z
       close(1)
c.......................................................................
c
c Determine USERDUMP file names and try to open them.
c
c   Ask for the dump number.  Loop back to this point when finished
c   with this dump number.  Negative input dump number terminates
c   execution.
c
       indx = 0
       icmb = 0
   60  continue
c
c Clean up any opened files.
c
       do 65 i=7,indx
         close (i)
   65  continue
       if (icmb .ne. 0) close (icmb)
c
       write(*,"(/'SPLICE: Enter the desired dump number; <0 quits:'/)")
       read(*,*) ndump
       if (ndump .lt. 0) then
         return
       endif
c
c  Use logical units 7 through 7 + nprocs.
c
       indx = 6
       do 90 kt=0,ntiles(3)-1
         do 80 jt=0,ntiles(2)-1
           do 70 it=0,ntiles(1)-1
             indx = indx + 1
             write(usr(indx),"(a3,a2,3i2.2,'.',i3.3)") 'usr',id
     &       , it,jt,kt,ndump
             open(indx,file=usr(indx),status='old',err=999)
   70      continue
   80    continue
   90  continue
       nprocs = indx - 6
c.......................................................................
c
c Read nx1z rows of data from each tile's dump and write them into
c the combined file 'cmb' ("usr<id>.<ndump>").
c
c   Name the combined file and open it.
c
       icmb = indx + 1
       write(cmb,"(a3,a2,'.',i3.3)") 'usr',id,ndump
       open(icmb,file=cmb)
c
c   For the first tile only, copy the first 3 lines (header) into
c   the combined file.   For the others, just skip the header.
c
       do 100 kt=0,ntiles(3)-1
         do 100 jt=0,ntiles(2)-1
           do 100 it=0,ntiles(1)-1
             do 100 i=1,3
               indx = 7 + it + jt * ntiles(1)
     &              + kt * ntiles(1) * ntiles(2)
               if (indx .eq. 7) then
                 do 93 n=1,120
                   line(n:n) = ' '
   93            continue
                 read(indx,"(a)") line
                 do 95 n=120,1,-1
                   if (line(n:n) .ne. ' ') goto 96
   95            continue
                 n = 0
   96            continue
                 if (n .eq. 0) then
                   write(icmb,"(a)") ''
                 else
                   write(icmb,"(a)") line(1:n)
                 endif
               else
                 read(indx,"(a)")
               endif
  100  continue
c
c 101  format(1p5e16.8)
c 102  format(1p5e16.8)
  101  format(1p6e16.8)
  102  format(1p6e16.8)
  103  format(1p4e16.8)
  104  format(1p4e16.8)
c
c   Copy the zone centered quantities d and e.
c
c   We can do all k values in one loop.
c
       do 190 kt=0,ntiles(3)-1
         do 190 k=1,nx3z
c
c     For a given k, we must sweep through all tiles to cover all
c     the j values.
c
           do 140 jt=0,ntiles(2)-1
             do 140 j=1,nx2z
c
c   For each tile, copy nx1z rows at a time into the combined file.
c
                do 130 it=0,ntiles(1)-1
                  indx = 7 + it + jt * ntiles(1)
     &                 + kt * ntiles(1) * ntiles(2)
                  read(indx,101) (x1b(i),x2bj(i),x3bk(i),edn(i)
     &            , ern(i),den(i),i=1,nx1z)
                  write(icmb,102) (x1b(i),x2bj(i),x3bk(i),edn(i)
     &            , ern(i),den(i),i=1,nx1z)
  130      continue
  140    continue
  190  continue
c#ifndef RAD_EXP
c
c   Copy the 1-face centered quantities v1 (and b1).
c
c   For the first tile only, copy the next 2 lines (header) into
c   the combined file.   For the others, just skip the header.
c
       do 200 kt=0,ntiles(3)-1
         do 200 jt=0,ntiles(2)-1
           do 200 it=0,ntiles(1)-1
             do 200 i=1,2
               indx = 7 + it + jt * ntiles(1)
     &              + kt * ntiles(1) * ntiles(2)
               if (indx .eq. 7) then
                 do 193 n=1,120
                   line(n:n) = ' '
  193            continue
                 read(indx,"(a)") line
                 do 195 n=120,1,-1
                   if (line(n:n) .ne. ' ') goto 196
  195            continue
                 n = 0
  196            continue
                 if (n .eq. 0) then
                   write(icmb,"(a)") ''
                 else
                   write(icmb,"(a)") line(1:n)
                 endif
               else
                 read(indx,"(a)")
               endif
  200  continue
c
c   We can do all k values in one loop.
c
       do 290 kt=0,ntiles(3)-1
         do 290 k=1,nx3z
c
c     For a given k, we must sweep through all tiles to cover all
c     the j values.
c
           do 240 jt=0,ntiles(2)-1
             do 240 j=1,nx2z
c
c     For each tile, copy nx1z rows at a time into the combined file.
c
                do 230 it=0,ntiles(1)-1
                  indx = 7 + it + jt * ntiles(1)
     &                 + kt * ntiles(1) * ntiles(2)
                  read(indx,103) (x1a(i),x2bj(i),x3bk(i),ve(i)
     &            , i=1,nx1z)
                  write(icmb,104) (x1a(i),x2bj(i),x3bk(i),ve(i)
     &            , i=1,nx1z)
  230      continue
  240    continue
  290  continue
c
c   Copy the 2-face centered quantities v2 (and b2).
c
c   For the first tile only, copy the next 2 lines (header) into
c   the combined file.   For the others, just skip the header.
c
       do 300 kt=0,ntiles(3)-1
         do 300 jt=0,ntiles(2)-1
           do 300 it=0,ntiles(1)-1
             do 300 i=1,2
               indx = 7 + it + jt * ntiles(1)
     &              + kt * ntiles(1) * ntiles(2)
               if (indx .eq. 7) then
                 do 293 n=1,120
                   line(n:n) = ' '
  293            continue
                 read(indx,"(a)") line
                 do 295 n=120,1,-1
                   if (line(n:n) .ne. ' ') goto 296
  295            continue
                 n = 0
  296            continue
                 if (n .eq. 0) then
                   write(icmb,"(a)") ''
                 else
                   write(icmb,"(a)") line(1:n)
                 endif
               else
                 read(indx,"(a)")
               endif
  300  continue
c
c   We can do all k values in one loop.
c
       do 390 kt=0,ntiles(3)-1
         do 390 k=1,nx3z
c
c     For a given k, we must sweep through all tiles to cover all
c     the j values.
c
           do 340 jt=0,ntiles(2)-1
             do 340 j=1,nx2z
c
c     For each tile, copy nx1z rows at a time into the combined file.
c
                do 330 it=0,ntiles(1)-1
                  indx = 7 + it + jt * ntiles(1)
     &                 + kt * ntiles(1) * ntiles(2)
                  read(indx,103) (x1b(i),x2bj(i),x3bk(i),ve(i)
     &            , i=1,nx1z)
                  write(icmb,104) (x1b(i),x2bj(i),x3bk(i),ve(i)
     &            , i=1,nx1z)
  330      continue
  340    continue
  390  continue
c
c   Copy the 3-face centered quantities v3 (and b3).
c
c   For the first tile only, copy the next 2 lines (header) into
c   the combined file.   For the others, just skip the header.
c
       do 400 kt=0,ntiles(3)-1
         do 400 jt=0,ntiles(2)-1
           do 400 it=0,ntiles(1)-1
             do 400 i=1,2
               indx = 7 + it + jt * ntiles(1)
     &              + kt * ntiles(1) * ntiles(2)
               if (indx .eq. 7) then
                 do 393 n=1,120
                   line(n:n) = ' '
  393            continue
                 read(indx,"(a)") line
                 do 395 n=120,1,-1
                   if (line(n:n) .ne. ' ') goto 396
  395            continue
                 n = 0
  396            continue
                 if (n .eq. 0) then
                   write(icmb,"(a)") ''
                 else
                   write(icmb,"(a)") line(1:n)
                 endif
               else
                 read(indx,"(a)")
               endif
  400  continue
c
c   We can do all k values in one loop.
c
       do 490 kt=0,ntiles(3)-1
         do 490 k=1,nx3z
c
c     For a given k, we must sweep through all tiles to cover all
c     the j values.
c
           do 440 jt=0,ntiles(2)-1
             do 440 j=1,nx2z
c
c     For each tile, copy nx1z rows at a time into the combined file.
c
                do 430 it=0,ntiles(1)-1
                  indx = 7 + it + jt * ntiles(1)
     &                 + kt * ntiles(1) * ntiles(2)
                  read(indx,103) (x1b(i),x2bj(i),x3bk(i),ve(i)
     &            , i=1,nx1z)
                  write(icmb,104) (x1b(i),x2bj(i),x3bk(i),ve(i)
     &            , i=1,nx1z)
  430      continue
  440    continue
  490  continue
c#endif /* not RAD_EXP */
c
c.......................................................................
c
c   Loop back up for the next dump.
c
       goto 60
c.......................................................................
c
c Error messages
c
  999  continue
       write(*,"(/'SPLICE: ERROR -- could not open file ',a15
     & , ' on unit ',i4)") usr(indx),indx
c
       do 965 i=7,indx
         close (i)
  965  continue
       if (icmb .ne. 0) close (icmb)
       return
       end
