#include "zeusmp.def"
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                 A D V X 1                 \\\\\\\\\\
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c    
c=======================================================================
c
       subroutine advx1 (dlo,den
#ifndef ISO
     &                  ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &                  ,ero,ern
#endif /* RAD || RAD_EXP */
     &                  ,mflx,s1,s2,s3)
c
c    RAF, 2/17/97
c
c  PURPOSE: 
c    Controls the update of density, energy, and momenta
c    from the advection terms in the 1-direction.
c
c  INPUT:
c    dlo         Mass            density prior to update
c    eod         Specific energy density prior to update
c
c  OUTPUT:
c    den         Mass            density    after update
c    edn         Specific energy density    after update
c
c  I/O:
c    s1,s2,s3    Momentum density components (get updated)
c
c  LOCAL:
c    mflx        Mass flux in the 1-direction at zone faces 
c
c  EXTERNALS:
c    BVALV1  , BVALV2  , BVALV3
c    BVALD   , BVALE   , BVALER
c
c-----------------------------------------------------------------------
       implicit NONE
#include "mod_names.h"
#include "param.h"
#include "root.h"
#include "field.h"
#include "bndry.h"
#include "grid.h"
#include "mpicom.h"
#ifdef PROFILING
#ifdef MPE_USED
#else
#include "clocks.h"
#endif /* MPE_USED */
#endif /* PROFILING */
c......................................................................
c
       integer k1,k2,i
#ifdef ISO
       integer j,k
#endif
c
       REAL  p1
       REAL  dlo(in,jn,kn), den(in,jn,kn), mflx(in,jn,kn)
     &     , s1 (in,jn,kn), s2 (in,jn,kn), s3  (in,jn,kn)
#ifndef ISO
     &     , eod(in,jn,kn), edn(in,jn,kn)
#endif
#if defined RAD || defined RAD_EXP
     &     , ero(in,jn,kn), ern(in,jn,kn)
#endif /* RAD || RAD_EXP */
c
c      External statements
c
       external      tranx1  , momx1   , bvald
     &             , bvalv1  , bvalv2  , bvalv3
#ifndef ISO
     &             , bvale
#endif
#if defined RAD || defined RAD_EXP
     &             , bvalert
#endif /* RAD || RAD_EXP */
c
c      Tunable data
c
       data p1 / 0.9 /
c
c-----------------------------------------------------------------------
c
#ifdef DEBUG
       write(*,"(/'ADVX1: ',i2,' calling tranx1 and momx1.')") myid
#endif /* DEBUG */
       nseq = nseq + 1        !   nseq indicates the sweep sequence.
       k1   = int( real( nx3z ) * p1 ) + ks   !  The lion's share...
       k1   = min( k1, ke - 2 )               !  but not too much!
       k2   = ( ke + k1 ) / 2                 !  Half the remainder.

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal1 = real(wclock)
       cpu1 = real(cputime)
#endif /* MPE_USED */
#endif /* PROFILING */

c......................................................................
c
c i boundaries
c
c    1) Post sends and receives. 
c       By exchanging the i, j, and k boundary data in three separate 
c       steps, we ensure that the corner and edge data are correctly
c       updated.
c
       nreq = 0
       nsub = nsub + 1
c
c TRANX1 and MOMX1 together need all 3 velocities at is-2 through ie+2.
c
       call bvalv1 (3,3,0,0,0,0,v1 )
       call bvalv2 (3,3,0,0,0,0,v2 )
       call bvalv3 (3,3,0,0,0,0,v3 )

#if defined RAD || defined RAD_EXP
c
c We need all slabs of er/d.
c
       call bvalert(3,3,0,0,0,0,ero)
#endif /* RAD || RAD_EXP */
#ifndef ISO
c
c We need all slabs of eod.
c
       call bvale  (3,3,0,0,0,0,eod)
c

#else /* not ISO */
c
       if (nseq .eq. 1) then
c
c We need to make a copy of the density, since we skipped pdv.
c
         do 30 k=ks-2,ke+2
           do 20 j=js-2,je+2
             do 10 i=is-2,ie+2
               dlo(i,j,k) = den(i,j,k)
10           continue
20         continue
30       continue
       endif
c
#endif /* not ISO */
c
c We need all the density slabs.
c
       call bvald  (3,3,0,0,0,0,dlo)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(nsubstps,1,2,2) = timers(nsubstps,1,2,2) + wal1  - wal0
       timers(nsubstps,1,2,1) = timers(nsubstps,1,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
c    2) Do first portion of the interior points.
c
       call tranx1 (is+3,ie-2,js+1,je  ,ks+1,k1,dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(4,1,2,2) = timers(4,1,2,2) + wal1  - wal0
       timers(4,1,2,1) = timers(4,1,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx1  (is+4,ie-3,js+2,je  ,ks+2,k1,s1,s2,s3,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(5,1,2,2) = timers(5,1,2,2) + wal1  - wal0
       timers(5,1,2,1) = timers(5,1,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#ifdef MPI_USED
c
c    3) Wait for communications to complete.
c
#ifdef DEBUG
       write(*,"(/'ADVX1: Process ',i2,' waiting on ',i2,' requests.')")
     &   myid, nreq
#endif /* DEBUG */
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )
#ifdef DEBUG
       write(*,"(/'ADVX1: Process ',i2,' sent/received these msgs.:',/
     & ,'        SOURCE         TAG')") myid
       write(*,"(2i12)") (stat(MPI_SOURCE,i),stat(MPI_TAG,i),i=1,nreq)
       write(*,"(/'ADVX1: Process ',i2,' continuing.')") myid
#endif /* DEBUG */

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(5,1,1,2) = timers(5,1,1,2) + wal1  - wal0
       timers(5,1,1,1) = timers(5,1,1,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#endif /* MPI_USED */
c......................................................................
c
c j boundaries
c
c    1) Post sends and receives.
c
       nreq = 0
       nsub = nsub + 1
       call bvald  (0,0,1,0,0,0,dlo)
       call bvalv1 (0,0,1,0,0,0,v1 )
#ifdef TOTAL_ENERGY
c       call bvalv2 (0,0,0,1,0,0,v2 )     !   Did this already in pdv.
#endif

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(nsubstps,2,2,2) = timers(nsubstps,2,2,2) + wal1  - wal0
       timers(nsubstps,2,2,1) = timers(nsubstps,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
c    2) Do second portion of the interior points, plus some on borders.
c
       call tranx1 (is  ,is+2,js+1,je  ,ks+1,k1,dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(4,2,2,2) = timers(4,2,2,2) + wal1  - wal0
       timers(4,2,2,1) = timers(4,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx1  (is  ,is+3,js+2,je  ,ks+2,k1,s1,s2,s3,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(5,2,2,2) = timers(5,2,2,2) + wal1  - wal0
       timers(5,2,2,1) = timers(5,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
       call tranx1 (ie-1,ie  ,js+1,je  ,ks+1,k1,dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(4,2,2,2) = timers(4,2,2,2) + wal1  - wal0
       timers(4,2,2,1) = timers(4,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx1  (ie-2,ie  ,js+2,je  ,ks+2,k1,s1,s2,s3,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(5,2,2,2) = timers(5,2,2,2) + wal1  - wal0
       timers(5,2,2,1) = timers(5,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
       call tranx1 (is  ,ie  ,js+1,je  ,k1+1,k2,dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(4,2,2,2) = timers(4,2,2,2) + wal1  - wal0
       timers(4,2,2,1) = timers(4,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx1  (is  ,ie  ,js+2,je  ,k1+1,k2,s1,s2,s3,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(5,2,2,2) = timers(5,2,2,2) + wal1  - wal0
       timers(5,2,2,1) = timers(5,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#ifdef MPI_USED
c
c    3) Wait for communications to complete.
c
#ifdef DEBUG
       write(*,"(/'ADVX1: Process ',i2,' waiting on ',i2,' requests.')")
     &   myid, nreq
#endif /* DEBUG */
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(5,2,1,2) = timers(5,2,1,2) + wal1  - wal0
       timers(5,2,1,1) = timers(5,2,1,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#endif /* MPI_USED */
c......................................................................
c
c k boundaries
c
c    1) Post sends and receives.
c
       nreq = 0
       nsub = nsub + 1
       call bvald  (0,0,0,0,1,0,dlo)
       call bvalv1 (0,0,0,0,1,0,v1 )
#ifdef TOTAL_ENERGY
c       call bvalv3 (0,0,0,0,0,1,v3 )     !   Did this already in pdv.
#endif

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(nsubstps,3,2,2) = timers(nsubstps,3,2,2) + wal1  - wal0
       timers(nsubstps,3,2,1) = timers(nsubstps,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
c    2) Do last portion of the interior points, plus some on borders.
c
       call tranx1 (is  ,ie  ,js  ,js  ,ks+1,k2,dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(4,3,2,2) = timers(4,3,2,2) + wal1  - wal0
       timers(4,3,2,1) = timers(4,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx1  (is  ,ie  ,js  ,js+1,ks+2,k2,s1,s2,s3,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(5,3,2,2) = timers(5,3,2,2) + wal1  - wal0
       timers(5,3,2,1) = timers(5,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
       call tranx1 (is  ,ie  ,js  ,je  ,k2+1,ke  ,dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(4,3,2,2) = timers(4,3,2,2) + wal1  - wal0
       timers(4,3,2,1) = timers(4,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx1  (is  ,ie  ,js  ,je  ,k2+1,ke  ,s1,s2,s3,mflx)
c
c Mark d and e/d (e) boundary values out of date.
c
       do 40 i=1,6
         bvstat(i,1) = 0  !  d
#ifndef ISO
         bvstat(i,2) = 0  !  e or e/d
#endif
#if defined RAD || defined RAD_EXP
         bvstat(i,6) = 0  !  er
#endif /* RAD || RAD_EXP */
40     continue
c

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(5,3,2,2) = timers(5,3,2,2) + wal1  - wal0
       timers(5,3,2,1) = timers(5,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#ifdef MPI_USED
c
c    3) Wait for communications to complete.
c
#ifdef DEBUG
       write(*,"(/'ADVX1: Process ',i2,' waiting on ',i2,' requests.')")
     &   myid, nreq
#endif /* DEBUG */
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(5,3,1,2) = timers(5,3,1,2) + wal1  - wal0
       timers(5,3,1,1) = timers(5,3,1,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#endif /* MPI_USED */
c......................................................................
c
c Finally, do the remaining border zones.
c
       call tranx1 (is  ,ie  ,js  ,je  ,ks, ks, dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(4,4,2,2) = timers(4,4,2,2) + wal1  - wal0
       timers(4,4,2,1) = timers(4,4,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx1  (is  ,ie  ,js  ,je  ,ks, ks+1, s1,s2,s3,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(5,4,2,2) = timers(5,4,2,2) + wal1  - wal0
       timers(5,4,2,1) = timers(5,4,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
       return
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////                 A D V X 1                 \\\\\\\\\\
c
c=======================================================================
c
c
