#include "zeusmp.def"
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                 A D V X 2                 \\\\\\\\\\
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
c=======================================================================
c
       subroutine advx2 (dlo,den
#ifndef ISO
     &                  ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &                  ,ero,ern
#endif /* RAD || RAD_EXP */
     &                  ,mflx,s1,s2,s3)
c
c    RAF, 2/17/97
c
c  PURPOSE: 
c    Controls the update of density, energy, and momenta
c    from the advection terms in the 2-direction.
c
c  INPUT:
c    dlo         Mass            density prior to update
c    eod         Specific energy density prior to update
c
c  OUTPUT:
c    den         Mass            density    after update
c    edn         Specific energy density    after update
c
c  I/O:
c    s1,s2,s3    Momentum density components (get updated)
c
c  LOCAL:
c    mflx        Mass flux in the 2-direction at zone faces 
c
c  EXTERNALS:
c    BVALV1  , BVALV2  , BVALV3
c    BVALD   , BVALE
c
c-----------------------------------------------------------------------
       implicit NONE
#include "mod_names.h"
#include "param.h"
#include "root.h"
#include "field.h"
#include "bndry.h"
#include "grid.h"
#include "mpicom.h"

#ifdef PROFILING
#ifdef MPE_USED
#else
#include "clocks.h"
#endif /* MPE_USED */
#endif /* PROFILING */

c
       integer k1,k2,i
#ifdef ISO
       integer j,k
#endif
       REAL    p2
c
       REAL  dlo(in,jn,kn), den(in,jn,kn), mflx(in,jn,kn)
     &     , s1 (in,jn,kn), s2 (in,jn,kn), s3  (in,jn,kn)
#ifndef ISO
     &     , eod(in,jn,kn), edn(in,jn,kn)
#endif
#if defined RAD || defined RAD_EXP
     &     , ero(in,jn,kn), ern(in,jn,kn)
#endif /* RAD || RAD_EXP */
c
c
c      External statements
c
       external      tranx2  , momx2   , bvald
     &             , bvalv1  , bvalv2  , bvalv3
#ifndef ISO
     &             , bvale
#endif
#if defined RAD || defined RAD_EXP
     &             , bvalert
#endif /* RAD || RAD_EXP */
c
c      Tunable data
c
       data p2 / 0.9 /
c
c-----------------------------------------------------------------------
c
#ifdef DEBUG
       write(*,"(/'ADVX2: ',i2,' calling tranx2 and momx2.')") myid
#endif /* DEBUG */
       nseq = nseq + 1        !   nseq indicates the sweep sequence.
       k1   = nint( ( real( ke + ks ) - p2 * nx3z ) * haf )
       k1   = max( k1, ks + 3)
       k2   = ke + ks - k1
       k2   = min( k2, ke - 1 )

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal1 = real(wclock)
       cpu1 = real(cputime)
#endif /* MPE_USED */
#endif /* PROFILING */

c......................................................................
c
c i boundaries
c
c    1) Post sends and receives. 
c       By exchanging the i, j, and k boundary data in three separate 
c       steps, we ensure that the corner and edge data are correctly
c       updated.
c
       nreq = 0
       nsub = nsub + 1
#ifdef TOTAL_ENERGY
c       call bvalv1 (0,1,0,0,0,0,v1 )     !   Did this already in pdv.
#endif
       call bvalv2 (1,0,0,0,0,0,v2 )
c
#ifdef ISO
       if (nseq .eq. 1) then
c
c We need to make a copy of the density, since we skipped pdv.
c
         do 30 k=ks-2,ke+2
           do 20 j=js-2,je+2
             do 10 i=is-2,ie+2
               dlo(i,j,k) = den(i,j,k)
10           continue
20         continue
30       continue
       endif
c
#endif /* ISO */
       call bvald  (1,0,0,0,0,0,dlo)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(nsubstps,1,2,2) = timers(nsubstps,1,2,2) + wal1  - wal0
       timers(nsubstps,1,2,1) = timers(nsubstps,1,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
c    2) Do first portion of the interior points.
c
       call tranx2 (is+1,ie,js+3,je-2,ks+1,k1,dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(6,1,2,2) = timers(6,1,2,2) + wal1  - wal0
       timers(6,1,2,1) = timers(6,1,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx2  (is+2,ie,js+4,je-3,ks+2,k1,s1,s2,s3,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(7,1,2,2) = timers(7,1,2,2) + wal1  - wal0
       timers(7,1,2,1) = timers(7,1,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#ifdef MPI_USED
c
c    3) Wait for communications to complete.
c
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )
#ifdef DEBUG
       write(*,"(/'ADVX2: Process ',i2,' sent/received these msgs.:',/
     & ,'        SOURCE         TAG')") myid
       write(*,"(2i12)") (stat(MPI_SOURCE,i),stat(MPI_TAG,i),i=1,nreq)
#endif /* DEBUG */

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(7,1,1,2) = timers(7,1,1,2) + wal1  - wal0
       timers(7,1,1,1) = timers(7,1,1,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#endif
c......................................................................
c
c j boundaries
c
c    1) Post sends and receives.
c
       nreq = 0
       nsub = nsub + 1
c
c We need all the density slabs.
c
       call bvald  (0,0,3,3,0,0,dlo)
c
#ifndef ISO
c
c We need all slabs of eod.
c
       call bvale  (0,0,3,3,0,0,eod)
c
#endif /* not ISO */
#if defined RAD || defined RAD_EXP
       call bvalert(0,0,3,3,0,0,ero)
#endif /* RAD || RAD_EXP */
c
c TRANX2 and MOMX2 together need all 3 velocities at js-2 through je+2.
c
       call bvalv1 (0,0,3,3,0,0,v1 )
       call bvalv2 (0,0,3,3,0,0,v2 )
       call bvalv3 (0,0,3,3,0,0,v3 )

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(nsubstps,2,2,2) = timers(nsubstps,2,2,2) + wal1  - wal0
       timers(nsubstps,2,2,1) = timers(nsubstps,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
c    2) Do middle portion of the interior points, plus some on borders.
c
       call tranx2 (is  ,is  ,js+3,je-2,ks+1,k1,dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(6,2,2,2) = timers(6,2,2,2) + wal1  - wal0
       timers(6,2,2,1) = timers(6,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx2  (is  ,is+1,js+4,je-3,ks+2,k1,s1,s2,s3,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(7,2,2,2) = timers(7,2,2,2) + wal1  - wal0
       timers(7,2,2,1) = timers(7,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
       call tranx2 (is  ,ie  ,js+3,je-2,k1+1,k2,dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(6,2,2,2) = timers(6,2,2,2) + wal1  - wal0
       timers(6,2,2,1) = timers(6,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx2  (is  ,ie  ,js+4,je-3,k1+1,k2,s1,s2,s3,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(7,2,2,2) = timers(7,2,2,2) + wal1  - wal0
       timers(7,2,2,1) = timers(7,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#ifdef MPI_USED
c
c    3) Wait for communications to complete.
c
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(7,2,1,2) = timers(7,2,1,2) + wal1  - wal0
       timers(7,2,1,1) = timers(7,2,1,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#endif
c......................................................................
c
c k boundaries
c
c    1) Post sends and receives.
c
       nreq = 0
       nsub = nsub + 1
       call bvald  (0,0,0,0,1,0,dlo)
       call bvalv2 (0,0,0,0,1,0,v2 )
#ifdef TOTAL_ENERGY
c       call bvalv3 (0,0,0,0,0,1,v3 )     !   Did this already in pdv.
#endif

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(nsubstps,3,2,2) = timers(nsubstps,3,2,2) + wal1  - wal0
       timers(nsubstps,3,2,1) = timers(nsubstps,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
c    2) Do last portion of the interior points, plus some on borders.
c
       call tranx2 (is  ,ie  ,js  ,js+2,ks+1,k2,dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(6,3,2,2) = timers(6,3,2,2) + wal1  - wal0
       timers(6,3,2,1) = timers(6,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx2  (is  ,ie  ,js  ,js+3,ks+2,k2,s1,s2,s3,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(7,3,2,2) = timers(7,3,2,2) + wal1  - wal0
       timers(7,3,2,1) = timers(7,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
       call tranx2 (is  ,ie  ,je-1,je  ,ks+1,k2,dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(6,3,2,2) = timers(6,3,2,2) + wal1  - wal0
       timers(6,3,2,1) = timers(6,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx2  (is  ,ie  ,je-2,je  ,ks+2,k2,s1,s2,s3,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(7,3,2,2) = timers(7,3,2,2) + wal1  - wal0
       timers(7,3,2,1) = timers(7,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
       call tranx2 (is  ,ie  ,js  ,je  ,k2+1,ke,dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(6,3,2,2) = timers(6,3,2,2) + wal1  - wal0
       timers(6,3,2,1) = timers(6,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx2  (is  ,ie  ,js  ,je  ,k2+1,ke,s1,s2,s3,mflx)
c
c Mark d and e/d (e) boundary values out of date.
c
       do 40 i=1,6
         bvstat(i,1) = 0  !  d
#ifndef ISO
         bvstat(i,2) = 0  !  e or e/d
#endif
#if defined RAD || defined RAD_EXP
         bvstat(i,6) = 0  !  er
#endif /* RAD || RAD_EXP */
40     continue

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(7,3,2,2) = timers(7,3,2,2) + wal1  - wal0
       timers(7,3,2,1) = timers(7,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#ifdef MPI_USED
c
c    3) Wait for communications to complete.
c
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(7,3,1,2) = timers(7,3,1,2) + wal1  - wal0
       timers(7,3,1,1) = timers(7,3,1,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#endif
c......................................................................
c
c Finally, do the remaining border zones.
c
       call tranx2 (is  ,ie  ,js  ,je  ,ks, ks, dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(6,4,2,2) = timers(6,4,2,2) + wal1  - wal0
       timers(6,4,2,1) = timers(6,4,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c      write(*,"('ADVX2 before: s2 =',1pd13.5)")s2(3,3,3)
       call momx2  (is  ,ie  ,js  ,je  ,ks, ks+1, s1,s2,s3,mflx)
c      write(*,"('ADVX2 after: s2 =',1pd13.5)")s2(3,3,3)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(7,4,2,2) = timers(7,4,2,2) + wal1  - wal0
       timers(7,4,2,1) = timers(7,4,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
       return
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////                 A D V X 2                 \\\\\\\\\\
c
c=======================================================================
c
c
