#include "zeusmp.def"
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                 A D V X 3                 \\\\\\\\\\
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
c=======================================================================
c
       subroutine advx3 (dlo,den
#ifndef ISO
     &                  ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &                  ,ero,ern
#endif /* RAD || RAD_EXP */
     &                  ,mflx,s1,s2,s3)
c
c    RAF, 2/19/97
c
c  PURPOSE: 
c    Controls the update of density, energy, and momenta
c    from the advection terms in the 3-direction.
c
c  INPUT:
c    dlo         Mass            density prior to update
c    eod         Specific energy density prior to update
c    ero         Specific radiation energy density prior to update
c
c  OUTPUT:
c    den         Mass            density    after update
c    edn         Specific energy density    after update
c    ern         Specific radiation energy density    after update
c
c  I/O:
c    s1,s2,s3    Momentum density components (get updated)
c
c  LOCAL:
c    mflx        Mass flux in the 3-direction at zone faces 
c
c  EXTERNALS:
c    BVALV1  , BVALV2  , BVALV3
c    BVALD   , BVALE   , BVALER
c
c-----------------------------------------------------------------------
       implicit NONE
#include "mod_names.h"
#include "param.h"
#include "root.h"
#include "field.h"
#include "bndry.h"
#include "grid.h"
#include "mpicom.h"

#ifdef PROFILING
#ifdef MPE_USED
#else
#include "clocks.h"
#endif /* MPE_USED */
#endif /* PROFILING */

c
       integer k1,k2,i
       integer kbeg, kend, krange, kblocks, kskip, ktlb, kblk, kpage
#ifdef ISO
       integer j,k
#endif
       REAL    p3
c
       REAL  dlo(in,jn,kn), den(in,jn,kn), mflx(in,jn,kn)
     &     , s1 (in,jn,kn), s2 (in,jn,kn), s3  (in,jn,kn)
#ifndef ISO
     &     , eod(in,jn,kn), edn(in,jn,kn)
#endif
#if defined RAD || defined RAD_EXP
     &     , ero(in,jn,kn), ern(in,jn,kn)
#endif /* RAD || RAD_EXP */
c
c      External statements
c
       external      tranx3  , momx3   , bvald
     &             , bvalv1  , bvalv2  , bvalv3
#ifndef ISO
     &             , bvale
#endif
#if defined RAD || defined RAD_EXP
     &             , bvalert
#endif /* RAD || RAD_EXP */
c
c      Tunable data
c
       data p3 / 0.9 /   ! Fraction of interior points to do 3rd stage
c
c Set blocking factor for 3rd stage k-loops.  The number of iterations
c in a block should be small enough so that all the data fits on the
c number of pages that the TLB can hold.  Thus,
c
c iterations = <TLB entries * page size> / <2-D arrays * word size>
c
c In tranx3 and momx3, 11 different 2-D arrays of data are used.  
c Assume 16kB pages and 8B words.  The number of iterations per
c block is forced to be at least 5.
c
#ifdef TLB
       data ktlb / TLB /  ! Get TLB size in MB from cpp -DTLB=n
#else /* TLB */
       data ktlb / 9999 /    ! Assume many TLB entries
#endif /* TLB */
#ifdef BLK_MIN
       data kblk / BLK_MIN /  ! Get min block size from cpp -DBLK_MIN=n
#else /* BLK_MIN */
       data kblk / 5 /        ! Default min block size is this many
#endif /* BLK_MIN */
#ifdef PAGE
       data kpage / PAGE /    ! Get PAGE size in bytes from cpp -DPAGE=n
#else /* PAGE */
       data kpage / 16384 /   ! Assume this many bytes per page
#endif /* PAGE */
c
c-----------------------------------------------------------------------
c
c Divide up the work.  Since we must have
c
c   ks+4 < k1-1
c   k1   < k2-1
c   k2   < ke-3
c
c   ke - ks .ge. 12   --- this is the smallest allowable k range.
c
       nseq = nseq + 1        !   nseq indicates the sweep sequence.
       k2   = ke - int( p3 * nx3z )
       k2   = min( k2, ke - 3 )
       k2   = max( k2, ks + 6 )
       k1   = ( k2 + ks ) / 2
       k1   = max( k1, ks + 5 )

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal1 = real(wclock)
       cpu1 = real(cputime)
#endif /* MPE_USED */
#endif /* PROFILING */

c......................................................................
c
c i boundaries
c
c    1) Post sends and receives. 
c       By exchanging the i, j, and k boundary data in three separate 
c       steps, we ensure that the corner and edge data are correctly
c       updated.
c
#ifdef DEBUG
       write(*,"(/'ADVX3: ',i2,' calling tranx3 and momx3.')") myid
#endif /* DEBUG */
       nreq = 0
       nsub = nsub + 1
#ifdef TOTAL_ENERGY
c       call bvalv1 (0,1,0,0,0,0,v1 )     !   Did this already in pdv.
#endif
       call bvalv3 (1,0,0,0,0,0,v3 )
c
#ifdef ISO
       if (nseq .eq. 1) then
c
c We need to make a copy of the density, since we skipped pdv.
c
         do 30 k=ks-2,ke+2
           do 20 j=js-2,je+2
             do 10 i=is-2,ie+2
               dlo(i,j,k) = den(i,j,k)
10           continue
20         continue
30       continue
       endif
c
#endif /* ISO */
       call bvald  (1,0,0,0,0,0,dlo)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(nsubstps,1,2,2) = timers(nsubstps,1,2,2) + wal1  - wal0
       timers(nsubstps,1,2,1) = timers(nsubstps,1,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
c    2) Do first portion of the interior points.
c
       call tranx3 (is+1,ie,js+1,je,ks+3,k1,dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(8,1,2,2) = timers(8,1,2,2) + wal1  - wal0
       timers(8,1,2,1) = timers(8,1,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx3  (is+2,ie,js+2,je,ks+4,k1-1,s1,s2,s3,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(9,1,2,2) = timers(9,1,2,2) + wal1  - wal0
       timers(9,1,2,1) = timers(9,1,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#ifdef MPI_USED
c
c    3) Wait for communications to complete.
c
#ifdef DEBUG
       write(*,"(/'ADVX3: Process ',i2,' waiting on ',i2,' requests.')")
     &  myid, nreq
#endif /* DEBUG */
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )
#ifdef DEBUG
       write(*,"(/'ADVX3: Process ',i2,' sent/received these msgs.:',/
     & ,'        SOURCE         TAG')") myid
       write(*,"(2i12)") (stat(MPI_SOURCE,i),stat(MPI_TAG,i),i=1,nreq)
#endif /* DEBUG */

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(9,1,1,2) = timers(9,1,1,2) + wal1  - wal0
       timers(9,1,1,1) = timers(9,1,1,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#endif
c......................................................................
c
c j boundaries
c
c    1) Post sends and receives.
c
       nreq = 0
       nsub = nsub + 1
       call bvald  (0,0,1,0,0,0,dlo)
#ifdef TOTAL_ENERGY
c       call bvalv2 (0,0,0,1,0,0,v2 )     !   Did this already in pdv.
#endif
       call bvalv3 (0,0,1,0,0,0,v3 )

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(nsubstps,2,2,2) = timers(nsubstps,2,2,2) + wal1  - wal0
       timers(nsubstps,2,2,1) = timers(nsubstps,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
c    2) Do second portion of the interior points, plus some on borders.
c
       call tranx3 (is  ,is  ,js+1,je,ks+3,k1,dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(8,2,2,2) = timers(8,2,2,2) + wal1  - wal0
       timers(8,2,2,1) = timers(8,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx3  (is  ,is+1,js+2,je,ks+4,k1-1,s1,s2,s3,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(9,2,2,2) = timers(9,2,2,2) + wal1  - wal0
       timers(9,2,2,1) = timers(9,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
       call tranx3 (is  ,ie  ,js+1,je,k1+1,k2,dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(8,2,2,2) = timers(8,2,2,2) + wal1  - wal0
       timers(8,2,2,1) = timers(8,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx3  (is  ,ie  ,js+2,je,k1  ,k2-1,s1,s2,s3,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(9,2,2,2) = timers(9,2,2,2) + wal1  - wal0
       timers(9,2,2,1) = timers(9,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#ifdef MPI_USED
c
c    3) Wait for communications to complete.
c
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(9,2,1,2) = timers(9,2,1,2) + wal1  - wal0
       timers(9,2,1,1) = timers(9,2,1,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#endif
c......................................................................
c
c k boundaries
c
c    1) Post sends and receives.
c
       nreq = 0
       nsub = nsub + 1
c
c We need all the density slabs.
c
       call bvald  (0,0,0,0,3,3,dlo)
c
#ifndef ISO
c
c We need all slabs of eod.
c
       call bvale  (0,0,0,0,3,3,eod)
c
#endif /* not ISO */
#if defined RAD || defined RAD_EXP
       call bvalert(0,0,0,0,3,3,ero)
#endif /* RAD || RAD_EXP */
c
c TRANX3 and MOMX3 together need all 3 velocities at ks-2 through ke+2.
c
       call bvalv1 (0,0,0,0,3,3,v1 )
       call bvalv2 (0,0,0,0,3,3,v2 )
       call bvalv3 (0,0,0,0,3,3,v3 )

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(nsubstps,3,2,2) = timers(nsubstps,3,2,2) + wal1  - wal0
       timers(nsubstps,3,2,1) = timers(nsubstps,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
c    2) Do last portion of the interior points, plus some on borders.
c
       call tranx3 (is  ,ie  ,js  ,js  ,ks+3,k2,dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(8,3,2,2) = timers(8,3,2,2) + wal1  - wal0
       timers(8,3,2,1) = timers(8,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx3  (is  ,ie  ,js  ,js+1,ks+4,k2-1,s1,s2,s3,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(9,3,2,2) = timers(9,3,2,2) + wal1  - wal0
       timers(9,3,2,1) = timers(9,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
c Block the k loop to reduce TLB misses; the k ranges for the earlier
c stages above should be small enough already.
c
       kskip   = max( ktlb * kpage / (11*8*in*jn), kblk)
       krange = ke-2 - (k2+1) + 1
       kblocks = krange / kskip
       if ( mod(krange,kskip) .eq. 0) kblocks = max(kblocks-1,0)
c
       do 35 kbeg = k2+1, k2+1 + kblocks*kskip, kskip 
         kend = min( kbeg + kskip - 1, ke-2 )
c       call tranx3 (is  ,ie  ,js  ,je  ,k2+1,ke-2,dlo,den
         call tranx3 (is  ,ie  ,js  ,je  ,kbeg,kend,dlo,den
#ifndef ISO
     &               ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &               ,ero,ern
#endif /* RAD || RAD_EXP */
     &               ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
         call MPE ...
#else
         call clocks (cputime, wclock)
         wal0 = wal1
         cpu0 = cpu1
         wal1 = real(wclock)
         cpu1 = real(cputime)
         timers(8,3,2,2) = timers(8,3,2,2) + wal1  - wal0
         timers(8,3,2,1) = timers(8,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c       call momx3  (is  ,ie  ,js  ,je  ,k2  ,ke-3,s1,s2,s3,mflx)
       call momx3  (is  ,ie  ,js  ,je  ,kbeg-1,kend-1,s1,s2,s3,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(9,3,2,2) = timers(9,3,2,2) + wal1  - wal0
       timers(9,3,2,1) = timers(9,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

35     continue
c
c Mark d and e/d (e) boundary values out of date.
c
       do 40 i=1,6
         bvstat(i,1) = 0  !  d
#ifndef ISO
         bvstat(i,2) = 0  !  e or e/d
#endif
#if defined RAD || defined RAD_EXP
         bvstat(i,6) = 0  !  er
#endif /* RAD || RAD_EXP */
40     continue
c

#ifdef MPI_USED
c
c    3) Wait for communications to complete.
c
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(9,3,1,2) = timers(9,3,1,2) + wal1  - wal0
       timers(9,3,1,1) = timers(9,3,1,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#endif
c......................................................................
c
c Finally, do the remaining border zones.
c
       call tranx3 (is  ,ie  ,js  ,je  ,ks, ks+2, dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(8,4,2,2) = timers(8,4,2,2) + wal1  - wal0
       timers(8,4,2,1) = timers(8,4,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx3  (is  ,ie  ,js  ,je  ,ks, ks+3, s1,s2,s3,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(9,4,2,2) = timers(9,4,2,2) + wal1  - wal0
       timers(9,4,2,1) = timers(9,4,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
       call tranx3 (is  ,ie  ,js  ,je  ,ke-1, ke, dlo,den
#ifndef ISO
     &             ,eod,edn
#endif
#if defined RAD || defined RAD_EXP
     &             ,ero,ern
#endif /* RAD || RAD_EXP */
     &             ,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(8,4,2,2) = timers(8,4,2,2) + wal1  - wal0
       timers(8,4,2,1) = timers(8,4,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       call momx3  (is  ,ie  ,js  ,je  ,ke-2, ke, s1,s2,s3,mflx)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(9,4,2,2) = timers(9,4,2,2) + wal1  - wal0
       timers(9,4,2,1) = timers(9,4,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
#ifdef DEBUG
       write(*,"(/'ADVX3: Process ',i2,' exiting.')") myid
#endif /* DEBUG */
       return
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////                 A D V X 3                 \\\\\\\\\\
c
c=======================================================================
c
c
