#include "zeusmp.def"
c=======================================================================
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
       subroutine clocks (cputime, wclock)
c
c This routine obtains the CPU and wall-clock times in seconds for the 
c calling process since the times (cputime0, wclock0) at which the 
c clocks were initalized (passed through common block /clocks/). 
c
c With MPI, MPI_WALL time is used except on the EXEMPLAR.
c
c For systems without a CPU timer, cputime is set to zero.
c
c Written by RAF, last modified 3/25/96.
c......................................................................
c
      implicit NONE
#include "param.h"
#include "mod_names.h"
#include "clocks.h"
#ifdef MPI_USED
#ifdef UNICOS
      real MPI_WTIME, wall
#else
#ifndef EXEMPLAR
      real*8 MPI_WTIME, wall
#endif /* EXEMPLAR */
#endif /* UNICOS */
      external MPI_WTIME
#endif /* MPI_USED */
#ifdef EXEMPLAR
      integer ifirst
      save    ifirst
      data    ifirst / 1 /
#endif /* not EXEMPLAR */
c
#ifdef MPI_USED
#ifndef EXEMPLAR
c
c Wall clock time is easy to get with MPI:
c
      wall   = MPI_WTIME()
      wclock = real(wall) - wclock0
#endif /* not EXEMPLAR */
#endif /* MPI_USED */
c
c Get the CPU time for this process/thread.
c
#if defined UNICOS || defined CONVEXOS || defined EXEMPLAR
c
c On some systems, times are best determined by routines
c other than the standard UNIX etime/itime pair.
c 
#ifdef UNICOS
#ifndef MPI_USED
      wclock = 0.001 * timef  () 
#endif
      cputime = second () - cputime0
#endif /* UNICOS */
c
#ifdef CONVEXOS
#ifndef MPI_USED
      zero4 = 0.0
      wclock = secnds  ( zero4 ) - wclock0
#endif
      zero4 = 0.0
      cputime = cputime ( zero4 ) - cputime0
#endif /* CONVEXOS */
c
#ifdef EXEMPLAR
      zero4  = 0.0
      wclock = walltime_ ( zero4 ) - wclock0
c
c On the Exemplar, use RAF's FORTRAN interface to the thread timer
c routines.  On the first call, the first argument must be set to
c zero to initalize the descriptor 'descript'; on subsequent calls
c set it to a non-zero value.
c 
      if (ifirst .eq. 1) then
        call ttimer ( 0, tarray(1), descript )
        ifirst = 0
      else
        call ttimer ( 1, tarray(1), descript )
      endif
      cputime = tarray(1) - cputime0
#endif /* EXEMPLAR */
c
#else /* defined UNICOS || defined CONVEXOS || defined EXEMPLAR */
c
c For systems with the standard UNIX etime/itime routines.  Note that
c the output from itime is an integer array with (hours,minutes,seconds)
c and therefor is accurate only to the nearest second.  The output from
c function etime itself is the sum of user plus system times, which can
c be significantly longer than the user time alone (stored in 
c tarray(1)). 
c
#ifndef MPI_USED
      call ITIME ( iarray )
      wclock = iarray(1) * 3600. + iarray(2) * 60. + iarray(3) - wclock0
#endif
c
#ifdef NO_CPU_TIMER
      cputime = 0.0
#else
      cputime = ETIME ( tarray ) - cputime0
#endif /* NO_CPU_TIMER */
c
#endif /* defined UNICOS || defined CONVEXOS || defined EXEMPLAR */
c
      end
