#!/usr/bin/perl -w
use strict;
use Sys::Hostname;

# This script adjusts the Makefile
# It is much easier to set the library paths and compilers
# here than to do it in the makefile itself.
# I could probably do this with automake, but it seemed like overkill
# for so few options.
# It make some attempt to setup for known systems.
# John C. Vernaleo (vernaleo@astro.umd.edu)
# initially added 09/28/2004
# last modified 04/20/2007
# Copyright (c) 2006-2007 John C. Vernaleo
# This file is released under the terms of the 3 clause BSD license

#######################KNOWN SYSTEMS#################################
#set true if compiling on borg
my $borg;
#set true for locutus
my $locutus;
#set true for UMD astronomy dept. computers
my $umdastro;
#set true for UMD OIT cluster (deepthought)
my $oitcluster;
#set true if compiling on ganon (this is an up-to-date
#gentoo system with all libraries from the system).
my $ganon;
#set true for OSX (Darwin) assuming you install libraries with fink
my $osx;
#set true for misc. testing sytems (no hdf)
my $testing;
#######################COMPILERS#################################
#set true for gfortran
my $gfortran;
#set true for intel compilers, false for portland group.
my $intel;
#set true for gnu compilers (g77), false for anything else
my $gnu;
#set true for portland group compilers
#This is for legacy reasons as the pg fortran compilers
#DO NOT WORK
my $pgc;

my $host = hostname;
if($host=~"deepthought")
{
    $oitcluster=1;
    #$intel=1;
    $gnu=1;
    print "Setup for UMD OIT cluster deepthought\n";
}
if(($host=~"astro")&&($host=~"umd")&&($host!~"borg")&&($host!~"locutus"))
{
    $umdastro=1;
    $gnu=1;
    print "Setup for umd astronomy computer\n";
}
if($host=~"borg")
{
    $borg=1;
    $gnu=1;
    print "Setup for the borg\n";
}
if($host=~"locutus")
{
    $locutus=1;
    $gfortran=1;
    print "Setup for the locutus\n";
}
if($host=~"ganon")
{
    $ganon=1;
    $gfortran=1;
    print "Setup for the ganon\n";
}
#kirby is my osx laptop
if($host=~"kirby")
{
    $osx=1;
    $gnu=1;
    print "Setup for the kirby\n";
}
################################################################
#SET ONE SYSTEM AND ONE COMPILER TO TRUE HERE###################
#ONLY USE THIS FOR SYTEMS THAT ARE NOT AUTOMATICALLY DETECETED##
################################################################
#For example for a mac
#$osx=1;
#$gnu=1;

#set true for lam, false for mpich (mpich may not work).
my $lam=1;

my $opt;
my $optpp;
my $fc;
my $cc;
my $mpipath;
my $mpiopts;
my $mpichpath;
my $hdfpath;
my $hdfopts="-lmfhdf -ldf -ljpeg -lz";
my $lampath;
my $laminc;
my $mpichinc;
my $mpiinc;
my $mpilib;
my $lamopts="-llamf77mpi -lmpi -llam";
my $mpichopts="-lpmpich -lmpich -lpmpich -lmpich";
my $extra=" -lnsl -lutil";
my $sixtyfourbit="";
#Set this as -ff2c if you are linking against
#a library compiled with g77
my $g77compat="";

#######################KNOWN SYSTEMS#################################
if($borg){
    $mpichpath="/home/vernaleo/mpich_intel";
    $lampath="/net/lam";
    $laminc=" ";
    $hdfpath="-L/home/vernaleo/hdf/lib";
}
if($umdastro){
    $mpichpath="/n/thales2/vernaleo/mpich_intel";
    $lampath=`echo \$LAMPATH`;
    chomp($lampath);
    $laminc="-I$lampath"."/include";
    $hdfpath=`echo \$HDF4PATH`;
    chomp($hdfpath);
    $hdfpath="-L$hdfpath"."/lib";
}
if($oitcluster){
    $mpichpath=`echo \$MPI_ROOT`;
    chomp($mpichpath);
    $lampath=`echo \$MPI_ROOT`;
    chomp($lampath);
    $laminc="-I$lampath"."/include";
    $hdfpath=`echo \$HDFLIB`;
    chomp($hdfpath);
    $hdfpath="-L$hdfpath";
}
if($ganon){
    $mpichpath="";
    $lampath="";
    $laminc="";
    $hdfpath="";
}
if($locutus){
    $sixtyfourbit="-m64 -mcmodel=medium";
    $mpichpath="";
    $lampath="/home/vernaleo/lam64";
    $laminc="-I$lampath"."/include";
    $hdfpath="-L/usr/lib64/hdf/ ";
}
if($osx){
    $lampath="/sw";
    $laminc="-I$lampath"."/include";
    $hdfpath="-L/sw/lib";
    $extra="";
}
if($testing){
    $lampath="/home/teuben/manybody/manybody/opt";
    $hdfpath=" ";
    $hdfopts=" ";
}

############################COMPILERS############################
if($intel){
    $fc="ifort ";
    $cc="icc";
    $opt="-O2 -axW";
    $optpp="-DLINUX";
    if($oitcluster){
	#only need -DLINUX if lam is compiled with gnu and not intel
	$optpp="";
    }
}
if($pgc){
    $fc="pgf77";
    $cc="pgcc";
    $opt="-O2";
    $optpp="-DLINUX";
}
if($gnu){
    $fc="g77 -Wno-globals ";
    $cc="gcc";
    $opt="-O2";
    $optpp="";
}
if($gfortran){
    $fc="gfortran -ffixed-form $g77compat $sixtyfourbit";
    $cc="gcc $sixtyfourbit";
    $opt="-O2";
    $optpp="";
}
#############################################################

if($lam){
    $mpipath=$lampath;
    $mpiopts=$lamopts;
    $mpiinc=$laminc;
    $mpilib="-L$lampath"."/lib";
}else{
    $mpipath=$mpichpath;
    $mpiopts=$mpichopts;
    $mpiinc=$mpichinc;
    $mpilib="-L$mpichpath"."/lib";
}

@ARGV="Makefile";
$^I="~";

while(<>){
    s/ZMP_FC[\s]+=.*/ZMP_FC     = $fc/;
    s/ZMP_CC[\s]+=.*/ZMP_CC     = $cc/;
    s/ZMP_CPP[\s]+=.*/ZMP_CPP    = \${ZMP_FC} $optpp -DTLB=128 -DUNROLL_I=4 -DUNROLL_J=4 -DUNROLL_K=4/;
    s/ZMP_LIB[\s]+=.*/ZMP_LIB    = $hdfpath $mpilib -ldl$extra $hdfopts $mpiopts -lpthread/;
    s/ZMP_OPTS3[\s]+=.*/ZMP_OPTS3  = -c $opt -I\/usr\/include $mpiinc/;
    s/ZMP_OPTS0[\s]+=.*/ZMP_OPTS0  = -c -O0 -I\/usr\/include $mpiinc/;
    s/ZMP_LDR[\s]+=.*/ZMP_LDR    = \${ZMP_FC} $opt $mpiinc/;
    print;
}
