#include "zeusmp.def"

c JCV 7/24/2006 a tilted, precessing jet.
c JCV 9/11/2006 Combine with feedback.
c JCV 9/26/2006 fixed bug that this shared with delayed.F
c JCV 4/25/2007 fixed bug for np in x1 > 1

c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                 FEED_PREC                 \\\\\\\\\\
c
c=======================================================================
c
       subroutine feed_prec

       implicit NONE
#include "mod_names.h"
#include "param.h"
#include "field.h"
#include "bndry.h"
#include "grid.h"
#include "root.h"
#include "scratch.h"
#include "mpicom.h"
#include "cons.h"

       integer i,j,k

       integer thetamax
       REAL alphar,beta
       REAL vx,vy,vz,vr,vt,vp

       REAL mass,dm,totmass,area
       REAL vjet
c      variabels for units
c Scaling factors real/code units
       REAL lscale,tiscale,Tescale,mscale,escale
c Real units kbolt[eV K-1],muu mean moleculare weight
       REAL kbolt,muu
       REAL jetarea
ccccccccccccccccc
       REAL mjetprint

c Must manually set mdota size (in scratch.h) to the same value as arraymax 
c (or higher) because fortran 77 lacks malloc like function.

       integer arraymax
       REAL tres,tdelay

       namelist /jet/ djet,v1jet,jetsize,efficiency,mcutoff
       namelist /prec/ palpha,pomega

c delay stuff

       tdelay=2.0
       tres=0.0001
       arraymax=tdelay/tres + 1

       mjetprint=0.0

       if(time.eq.0)then
c         20001 is set in scratch.h
          if(arraymax.gt.20001)then
             write(*,*)"Need to set bigger array in scratch.h"
#ifdef MPI_USED
             call MPI_FINALIZE
#endif /* MPI_USED */
             stop
          endif
          counter=1
          counter2=1
          tcounter=0.0
          mjet=0.0
       endif
       tcounter=tcounter+dt
c       if(myid.eq.0)then
c          write(*,*)'tcounter = ',tcounter
c       endif
       if(counter.gt.arraymax)then
          counter=1
       endif
       if(counter2.gt.arraymax)then
          counter2=1
       endif
ccccccccccccccccccccccccccccccccccccccccccccccc


c This is alpha in radians
       alphar=(palpha/180.0)*pi
       if(pomega.eq.0.0)then
          beta=0.0
       else
          beta=(time/(2.0*pomega))*pi
       endif
       thetamax=J_ZONES*ntiles(2)/jetsize
c Uncomment the next line if there is only one jet
       thetamax=thetamax*2.0

c there should be a less hardwired way to do this
c pi*(rin*tan(theta_jet)**2)
       jetarea=pi*(rin*tan(jettheta)**2)

       muu=1.27
       kbolt=1.4861E-9

       lscale=rcore*1000*cmpc/2
       tiscale=lscale/(cism*1000*100)
       Tescale=mp*muu*((cism*1000*100)**2)/(gamma*kbolt*3.0/5.0)
       mscale=muu*mp*ncent*(lscale**3)
       escale=(lscale**2)*mscale/(tiscale**2)

       i=3

c check to make sure this is a tile that touches inner boundary
c currently I do not have more than 1 tile in x1, but should anyway
c NOTE if there are too many processors (~10), this will no longer work
       mass=0.0
       if (x1b(is) .LE. 1) then
          do k=ks,ke
             do j=js,je
                area=dvl2a(j)*dvl3a(k)
                dm=d(i,j,k)*v1(i,j,k)*area
                if(v1(i,j,k).le.0.0)then
                   mass=mass+dm
                endif
             enddo
          enddo
          mass=-1.0*mass
       endif
       
       call MPI_ALLREDUCE(mass,totmass,1,MPI_FLOAT,MPI_SUM
     &      ,comm3d,ierr)
       
          if(myid.eq.0)then
             write(*,*)"total mass across inner boundary = ",totmass*
     &            mscale*secyr/(tiscale*msol)
          endif

ccccccccccccccccccccccccccccccccccccccc
          if(time.lt.tdelay)then
             if(tcounter.gt.tres)then
                mdota(counter)=totmass
                counter=counter+1
                tcounter=0.0
                if(myid.eq.0)then
                   write(*,*)'counter= ',counter
                endif
             endif
          else
             if(tcounter.gt.tres)then
                mdota(counter)=totmass
                counter=counter+1
                tcounter=0.0
                mjet=mdota(counter2)
                counter2=counter2+1
                if(myid.eq.0)then
                   write(*,*)'counter= ',counter
                   write(*,*)'counter2= ',counter2
                endif
             endif
          endif

c calculate vjet here
          vjet=((2*efficiency*mjet*(clight*tiscale/lscale)**2)
     &         /(jetarea*djet))**(1.0/3.0)

          mjetprint=mjet*mscale*secyr/(tiscale*msol)
          if(mjetprint.le.mcutoff)then
             vjet=0.0
             mjet=0.0
             mjetprint=0.0
          endif

          if(myid.eq.0)then
             write(*,*)"mdot used in jet = ",mjetprint
          endif

c Only look for jet near inner edge
       if (x1b(is) .LE. 1) then
c Do first jet if t direction starts close to 0
          if (x2b(js) .LE. 0.1) then

c 1st jet
            do j=js-2,js+thetamax
               do k=ks-2,ke+2
                  
                  vz=vjet*cos(alphar)
                  vy=vjet*sin(alphar)*cos(beta)
                  vx=vjet*sin(alphar)*sin(beta)

                  vr=   vx*(cos(x3a(k))*sin(x2a(j)))
     &                 +vy*(sin(x3a(k))*sin(x2a(j)))
     &                 +vz*(cos(x2a(j)))

                  vt=   vx*(cos(x3a(k))*cos(x2a(j)))
     &                 +vy*(sin(x3a(k))*cos(x2a(j)))
     &                 +vz*(-1.0*sin(x2a(j)))

                  vp=   vx*(-1.0*sin(x3a(k)))
     &                 +vy*cos(x3a(k))

                  v1iib(j,k,1)=vr
                  v1iib(j,k,2)=vr
                  v2iib(j,k,1)=vt
                  v2iib(j,k,2)=vt
                  v3iib(j,k,1)=vp
                  v3iib(j,k,2)=vp
               enddo
            enddo

c end loop for 1st jet
         endif
c end loop for both jets
      endif
ccccccccccc

       return
       end
