#include "zeusmp.def"

c JCV 10/27/03
c subroutine to update boundaries and turn jet off
c based on my subroutine from zeus3d
c JCV 11/22/04
c calculate m dot across inner boundary for feedback
c JCV 12/13/04
c use this to set jet power based on some feedback scheme
c JCV 02/28/05
c get mdot on inner edge, not ghost zones
c JCV 08/01/05
c fixed bug that prevented multiple mpi tiles in x1

c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////              FEEDBACK                     \\\\\\\\\\
c
c=======================================================================
c

       subroutine feedback

       implicit NONE

#include "mod_names.h"
#include "param.h"
#include "field.h"
#include "bndry.h"
#include "grid.h"
#include "root.h"
#include "scratch.h"
#include "mpicom.h"
#include "cons.h"

       integer i,j,k
       REAL mass,dm,totmass,area
       REAL vjet
       integer thetamax
c      variabels for units
c Scaling factors real/code units
       REAL lscale,tiscale,Tescale,mscale,escale
c Real units kbolt[eV K-1],muu mean moleculare weight
       REAL kbolt,muu
       REAL jetarea
ccccccccccccccccc

      thetamax=J_ZONES*ntiles(2)/jetsize
c Uncomment the next line if there is only one jet
      thetamax=thetamax*2.0       

      rin=x1a(3)
      jettheta=x2b(thetamax+2)

c there should be a less hardwired way to do this
c pi*(rin*tan(theta_jet)**2)
       jetarea=pi*(rin*tan(jettheta)**2)

       muu=1.27
       kbolt=1.4861E-9

       lscale=rcore*1000*3.086E18/2
       tiscale=lscale/(cism*1000*100)
       Tescale=mp*muu*((cism*1000*100)**2)/(gamma*kbolt*3.0/5.0)
       mscale=muu*mp*ncent*(lscale**3)
       escale=(lscale**2)*mscale/(tiscale**2)

       i=3

c check to make sure this is a tile that touches inner boundary
c currently I do not have more than 1 tile in x1, but should anyway
c NOTE if there are too many processors (~10), this will no longer work
       mass=0.0
       if (x1b(is) .LE. 1) then
          do k=ks,ke
             do j=js,je
                area=dvl2a(j)*dvl3a(k)
                dm=d(i,j,k)*v1(i,j,k)*area
                if(v1(i,j,k).le.0.0)then
                   mass=mass+dm
                endif
             enddo
          enddo

          mass=-1.0*mass
       endif

       call MPI_ALLREDUCE(mass,totmass,1,MPI_FLOAT,MPI_SUM
     &      ,comm3d,ierr)

       if (x1b(is) .LE. 1) then
c calculate vjet here
          vjet=((2*efficiency*totmass*(clight*tiscale/lscale)**2)
     &         /(jetarea*djet))**(1.0/3.0)
c get mdot in normal units
          totmass=totmass*mscale*secyr/(tiscale*msol)

          if(myid.eq.0)then
             write(*,*)"total mass across inner boundary = ",totmass
c             write(*,*)"vjet=",vjet
          endif

       endif

c Check to see is mass is above threshold.
c Should do nothing if mcutoff is 0 (or not definted in input deck
       if(totmass.le.mcutoff)then
          vjet=0.0
       endif

c Only look for jet near inner edge
       if (x1b(is) .LE. 1) then
c Do first jet if t direction starts close to 0
          if (x2b(js) .LE. 0.1) then

c 1st jet
             do j=js-2,js+thetamax
                do k=ks-2,ke+2
                   v1iib(j,k,1)=vjet
                   v1iib(j,k,2)=vjet
                enddo
             enddo

c end loop for 1st jet
          endif
ccc

c only do 2nd jet if t direction ends at almost 3.14
          if (x2b(je) .GE. 3.0) then
         
c jet 2
             do j=je-thetamax,je+2
                do k=ks-2,ke+2
                   v1iib(j,k,1)=vjet
                   v1iib(j,k,2)=vjet
                enddo
             enddo

c end loop for 2nd jet
          endif
ccccc

c end loop for both jets
       endif

       return
       end
