#include "zeusmp.def"
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////            F F T G R A V                  \\\\\\\\\\
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
c=======================================================================
c
       subroutine fftgrav
#if alias GRAVITY.eq.fftgrav
c
c    ab:zeus3d.fftgrav  <-------------- gravity solver for 3D periodic BC's XYZ
c                                                        june 1997
c    from:
c
c    written by: Andi Burkert, modified for ZEUS by M-MML
c    modified1:
c
c  PURPOSE:  Takes the density field and returns the gravitational potential
c    at all points.  Note that EDITOR doesn't recognize complex*16 as a
c    data type, and so doesn't issue correct parallelization directives. 
c    I've entered the correct directives for IRIX; others will have to be
c    included some other time.
c
c  INPUT PARAMETERS: 
c
c  LOCAL VARIABLES :
c
c  COMMON BLOCKS   : common /tabcom/ table
c                    common /greendat/ green
c
c  EXTERNALS       : cfft
c
c-----------------------------------------------------------------------
c
      integer n1,n2,n3
      integer ntable,nwork,i,j,k
      integer isys
      parameter (ntable=90+2*(in+jn+kn))
      parameter (nwork=2*in*jn*kn)
      REAL green(0:in-1,0:jn-1,0:kn-1)
      REAL table(ntable),work(nwork),error

c assume that all zones are used (ie-is+1 = in-5)
      complex*16  x(0:in-6,0:jn-6,0:kn-6)
#ifdef SOLARIS
      double complex  x(0:in-6,0:jn-6,0:kn-6)
#endif /* SOLARIS */

      common /tabcom/ table
      common /greendat/ green

      equivalence (x(0,0,0), wa3d(1,1,1)), 
     &            (work(1),  wc3d(1,1,1))

c-----------------------------------------------------------------------

#ifdef DBGZEUS_FFTGRAV
       print*,'  fftgrav : called'
#endif /* DBGZEUS_FFTGRAV */

c     set up constants
      isys = 0

      n1 = ie - is + 1
      n2 = je - js + 1
      n3 = ke - ks + 1

      if (nhy .eq. 0) 
     *  call zzfft3d(0,n1,n2,n3,1d0,
     *            x,n1,n2,x,n1,n2,table,work,isys)
c
c     initialize input array 
c
c$doacross   local   ( k, j, i),
c$&          share   ( x, n3, n2, n1, d, is, js, ks )
      do k=1,n3-1
        do j=1,n2-1
          do i=1,n1-1
            x(i,j,k) = cmplx(d(i+is-1,j+js-1,k+ks-1),0.)
          enddo
        enddo
      enddo
c$doacross   local   ( k, j),
c$&          share   ( x, n3, n2, d, n1, is, js, ks )
      do k=1,n3-1
        do j=1,n2-1
          x(0,j,k)=cmplx(d(n1+is-1,j+js-1,k+ks-1),0.)
        enddo
      enddo
c$doacross   local   ( k, i),
c$&          share   ( x, n3, n1, d, is, n2, js, ks )
      do k=1,n3-1
        do i=1,n1-1
          x(i,0,k)=cmplx(d(i+is-1,n2+js-1,k+ks-1),0.)
        enddo
      enddo
c$doacross   local   ( j, i),
c$&          share   ( x, n2, n1, d, is, js, n3, ks )
      do j=1,n2-1
        do i=1,n1-1
          x(i,j,0)=cmplx(d(i+is-1,j+js-1,n3+ks-1),0.)
        enddo
      enddo
      do k=1,n3-1
        x(0,0,k)=cmplx(d(n1+is-1,n2+js-1,k+ks-1),0.)
      enddo
      do j=1,n2-1
        x(0,j,0)=cmplx(d(n1+is-1,j+js-1,n3+ks-1),0.)
      enddo
      do i=1,n1-1
        x(i,0,0)=cmplx(d(i+is-1,n2+js-1,n3+ks-1),0.)
      enddo
      x(0,0,0)=cmplx(d(n1+is-1,n2+js-1,n3+ks-1),0.)

#ifdef DBGZEUS_FFTGRAV
      print*,'  fftgrav :   call (1) ??fft3d'
#endif /* DBGZEUS_FFTGRAV */
      call zzfft3d(+1,n1,n2,n3,1d0,
     *            x,n1,n2,x,n1,n2,table,work,isys)
c$doacross   local   ( k, j, i),
c$&          share   ( x, n3, n2, n1, green )
      do 23 k=0,n3-1
        do 22 j=0,n2-1
          do 21 i=0,n1-1
            x(i,j,k)=x(i,j,k)*green(i,j,k)
21        continue
22      continue
23    continue

#ifdef DBGZEUS_FFTGRAV
      print*,'  fftgrav :   call (2) ??fft3d'
#endif /* DBGZEUS_FFTGRAV */
      call zzfft3d(-1,n1,n2,n3,1d0/(n1*n2*n3),
     &            x,n1,n2,x,n1,n2,table,work,isys)
c
c note that ZEUS uses the negative of the normally defined potential!
c
c$doacross   local   ( k, j, i),
c$&          share   ( x, n3, n2, n1, gp, is, js, ks )
      do k=1,n3-1
        do j=1,n2-1
          do i=1,n1-1
            gp(i+is-1,j+js-1,k+ks-1) = -real( x(i,j,k) )
          enddo
        enddo
      enddo
c$doacross   local   ( k, j),
c$&          share   ( x, n3, n2, gp, n1, is, js, ks )
      do k=1,n3-1
        do j=1,n2-1
          gp(n1+is-1,j+js-1,k+ks-1)=-real(x(0,j,k))
        enddo
      enddo
c$doacross   local   ( k, i),
c$&          share   ( x, n3, n1, gp, is, n2, js, ks )
      do k=1,n3-1
        do i=1,n1-1
          gp(i+is-1,n2+js-1,k+ks-1)=-real(x(i,0,k))
        enddo
      enddo
c$doacross   local   ( j, i),
c$&          share   ( x, n2, n1, gp, is, js, n3, ks )
      do j=1,n2-1
        do i=1,n1-1
          gp(i+is-1,j+js-1,n3+ks-1)=-real(x(i,j,0))
        enddo
      enddo
      do k=1,n3-1
        gp(n1+is-1,n2+js-1,k+ks-1)=-real(x(0,0,k))
      enddo
      do j=1,n2-1
        gp(n1+is-1,j+js-1,n3+ks-1)=-real(x(0,j,0))
      enddo
      do i=1,n1-1
        gp(i+is-1,n2+js-1,n3+ks-1)=-real(x(i,0,0))
      enddo
      gp(n1+is-1,n2+js-1,n3+ks-1)=-real(x(0,0,0))
c
c  now fill in the periodic boundary zones
c
       do j=js,je
         do i=is,ie
           gp(i,j,ksm1) = gp(i,j,ke)
         enddo
       enddo
       do j=js,je
         do i=is,ie
           gp(i,j,kep1) = gp(i,j,ks)
         enddo
       enddo

       do k=ks,ke
         do j=js,je
           gp(ism1,j,k) = gp(ie,j,k)
         enddo
       enddo
       do k=ks,ke
         do j=js,je
           gp(iep1,j,k) = gp(is,j,k)
         enddo
       enddo

       do k=ks,ke
         do i=is,ie
           gp(i,jsm1,k) = gp(i,je,k)
         enddo
       enddo
       do k=ks,ke
         do i=is,ie
           gp(i,jep1,k) = gp(i,js,k)
         enddo
       enddo


#ifdef DBGZEUS_FFTGRAV
       print*,'  fftgrav : finished'
#endif /* DBGZEUS_FFTGRAV */

#endif GRAVITY.eq.fftgrav
       return
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////              F F T G R A V                \\\\\\\\\\
c
c=======================================================================
