#include "zeusmp.def"
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                F O R C E S                \\\\\\\\\\
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
c=======================================================================
c
       subroutine forces (ibeg,iend,jbeg,jend,kbeg,kend
     &                   ,u1,u2,u3,w1,w2,w3)
#ifndef PNEWT
#ifndef NO_FORCES
c
c
c Computes Pressure, Gravity, and Pseudo-Rotational Forces.
#ifdef MHD
c   and includes magnetic pressure forces (longitudinal Lorentz forces)
#endif /* MHD */
c
c Arrays u1 , u2 , u3  hold the old velocity values, while
c        w1 , w2 , w3  receive the updated values.
c
c BOUNDARY VALUES USED:
c
c  Macro defined  var   ii    oi    ij    oj    ik    ok
c  -------------  ---  ----  ----  ----  ----  ----  ----
c                  d   is-1        js-1        ks-1
c  ZRP             d   is-1        js-1        ks-1  ke+1
c  RTP             d   is-1        js-1  je+1  ks-1  ke+1
c                  e   is-1        js-1        ks-1
c  TOTAL_ENERGY    u1  is-1  ie+1  js-1        ks-1
c  RTP             u2  is-1              je+1
c  TOTAL_ENERGY    u2  is-1        js-1  je+1  ks-1
c  TOTAL_ENERGY    u3  is-1        js-1        ks-1  ke+1
c  ZRP             u3              js-1              ke+1
c  RTP             u3  is-1        js-1              ke+1
c  RAD || RAD_EXP  er  is-1        js-1        ks-1
c  MHD             b1  is-1  ie+1  js-1  je+1  ks-1  ke+1
c  MHD             b2  is-1  ie+1  js-1  je+1  ks-1  ke+1
c  MHD             b3  is-1  ie+1  js-1  je+1  ks-1  ke+1
c Written by RAF; modified 3/13/97 (JCH); 
c     modified 2 Mar 1998 to add MHD (M-MML)
c     modified 3 1/19/2007 jcv
c              fixes from zeusmpv2 for bug om v2 for rotational
c              pseudo-forces (very old bug from zeus3d)
c              and bugs for point masses in rtp.
c
c-----------------------------------------------------------------------
c
       implicit NONE
#include "param.h"
#include "root.h"
#include "field.h"
#include "grid.h"
#include "scratch.h"
#include "gravity.h"
#include "mpicom.h"
#include "cons.h"
       integer ibeg,iend,jbeg,jend,kbeg,kend
       integer i,j,k
       REAL u1 (in,jn,kn),u2 (in,jn,kn),u3 (in,jn,kn)
     &     ,w1 (in,jn,kn),w2 (in,jn,kn),w3 (in,jn,kn)
       REAL poo(in),pmo(in),pom(in)
       REAL st,rhoi
#ifdef RTP
       REAL s2oo(in),s2po(in)
#endif /* RTP */
#if defined ZRP || defined RTP
       REAL s3oo(in),s3mo(in),s3op(in),s3mp(in)
#endif /* ZRP || RTP */
#if defined GRAV || defined GRAV_FFT || defined PT_MASS
       REAL r2,r2i,prpq,pzpq,prpt,pzpt
#endif /* GRAV || GRAV_FFT */
#ifdef MHD
       REAL d1b2oo(in), d1b3oo(in), d2b1oo(in), d2b3oo(in)
     &     ,d3b1oo(in), d3b2oo(in)
       REAL d1b2po(in), d1b3op(in), d2b1po(in), d2b3op(in)
     &     ,d3b1po(in), d3b2op(in)
#endif /* MHD */
#if defined  RAD || defined RAD_EXP
       REAL derdx1, derdx2, derdx3
     &    , flx_lim, grad_er, big_R, ros_mfp, t_max, rdmcm,
     .      rdmcp, power, temp
#include "radexp.h"
#endif /* RAD || RAD_EXP */
#if defined RAD || defined RAD_EXP
#include "opac.h"
#endif /* RAD || RAD_EXP */
#ifndef UNROLL_I
#define UNROLL_I
#endif
c
c-----------------------------------------------------------------------
c
c Main loop for pressure, magnetic pressure, self-gravity, and
c     rotational pseudo-forces.  
c
c Active zones for all velocity components are (is:ie, js:je, ks:ke).
c
c In its most general configuration, this loop needs 1 layer of 
c boundary data for each of d, e, u1, u2, u3, and gp.
c
c If GRAV or GRAV_FFT is not defined, we don't need gp.
c If ISO  is     defined, we don't need e .
c
c For XYZ defined and TOTAL_ENERGY not defined, we don't need u1,u2,u3
c boundary values.  All we do need in this case is the inner boundary 
c data for d (and e).
c
c Of course, TOTAL_ENERGY and ISO are mutually exclusive. 
c
c **> at the moment, TOTAL_ENERGY and MHD also can't be used together
c       (M-MML 4 Mar 98)
c
c#if defined RAD || defined RAD_EXP
c       call opacity(e, d, gamma, kap, sig, dkapdt,
c     .              ibeg-1, iend, jbeg, jend, kbeg, kend)
c#endif /* RAD || RAD_EXP */
c
       do 40 k=kbeg,kend
         do 30 j=jbeg,jend
CDIR$ UNROLL (UNROLL_I)
           do 10 i=ibeg-1,iend
c
c Compute thermal pressure
c
#ifdef TOTAL_ENERGY
#ifndef ISO
               poo(i)         =    e(i  ,j  ,k  ) + efloor
     1                        - ((u1(i  ,j  ,k  ) + u1(i+1,j  ,k  ))**2
     2                        +  (u2(i  ,j  ,k  ) + u2(i  ,j+1,k  ))**2
     3                        +  (u3(i  ,j  ,k  ) + u3(i  ,j  ,k+1))**2) 
     4                        *    d(i  ,j  ,k  ) * 0.125
#if defined GRAV || defined GRAV_FFT
     1                        - gp(i  ,j  ,k  ) * d(i  ,j  ,k  )
#endif /* GRAV || GRAV_FFT */
               pmo(i)         =    e(i  ,j-1,k  ) + efloor
     1                        - ((u1(i  ,j-1,k  ) + u1(i+1,j-1,k  ))**2
     2                        +  (u2(i  ,j-1,k  ) + u2(i  ,j  ,k  ))**2
     3                        +  (u3(i  ,j-1,k  ) + u3(i  ,j-1,k+1))**2) 
     4                        *    d(i  ,j-1,k  ) * 0.125
#if defined GRAV || defined GRAV_FFT
     1                        - gp(i  ,j-1,k  ) * d(i  ,j-1,k  )
#endif /* GRAV || GRAV_FFT */
               pom(i)         =    e(i  ,j  ,k-1) + efloor
     1                        - ((u1(i  ,j  ,k-1) + u1(i+1,j  ,k-1))**2
     2                        +  (u2(i  ,j  ,k-1) + u2(i  ,j+1,k-1))**2
     3                        +  (u3(i  ,j  ,k-1) + u3(i  ,j  ,k  ))**2) 
     4                        *    d(i  ,j  ,k-1) * 0.125
#if defined GRAV || defined GRAV_FFT
     1                        - gp(i  ,j  ,k-1) * d(i  ,j  ,k-1)
#endif /* GRAV || GRAV_FFT */
               poo(i)         = gamm1 * poo(i)
               pmo(i)         = gamm1 * pmo(i)
               pom(i)         = gamm1 * pom(i)
#endif /* not ISO */
#else /* TOTAL_ENERGY */
#ifdef ISO
               poo(i)         = ciso**2 * ( d(i  ,j  ,k  ) + dfloor )
               pmo(i)         = ciso**2 * ( d(i  ,j-1,k  ) + dfloor )
               pom(i)         = ciso**2 * ( d(i  ,j  ,k-1) + dfloor )
#else /* ISO */
               poo(i)         = gamm1 * ( e(i  ,j  ,k  ) + efloor )
               pmo(i)         = gamm1 * ( e(i  ,j-1,k  ) + efloor )
               pom(i)         = gamm1 * ( e(i  ,j  ,k-1) + efloor )

#endif /* ISO */
#endif /* TOTAL_ENERGY */
c.......................................................................
c   
#if defined ZRP || defined RTP
c
c      Construct momentum densities from velocities.
c
              s3oo(i)         = u3(i  ,j  ,k  )*0.5*g31b(i  )*g32b(j  )
     1                        * ( d(i  ,j  ,k-1) + d(i  ,j  ,k  ) )
              s3op(i)         = u3(i  ,j  ,k+1)*0.5*g31b(i  )*g32b(j  )
     1                        * ( d(i  ,j  ,k  ) + d(i  ,j  ,k+1) )
              s3mo(i)         = u3(i  ,j-1,k  )*0.5*g31b(i  )*g32b(j-1)
     1                        * ( d(i  ,j-1,k-1) + d(i  ,j-1,k  ) )
              s3mp(i)         = u3(i  ,j-1,k+1)*0.5*g31b(i  )*g32b(j-1)
     1                        * ( d(i  ,j-1,k  ) + d(i  ,j-1,k+1) )
#endif /* ZRP || RTP */
#ifdef RTP
              s2oo(i)         = u2(i  ,j  ,k  )*0.5*g2b (i  )
     1                        * ( d(i  ,j-1,k  ) + d(i  ,j  ,k  ) )
              s2po(i)         = u2(i  ,j+1,k  )*0.5*g2b (i  )
     1                        * ( d(i  ,j  ,k  ) + d(i  ,j+1,k  ) )
#endif /* RTP */
 10        continue
c.......................................................................
#ifdef MHD
c
c        Compute differences in the squares of the magnetic field
c        components for the longitudinal Lorentz forces.
c
CDIR$ UNROLL (UNROLL_I)
           do 11 i=ibeg-1,iend
              d1b2oo(i) = ( ( g2b (i  ) * b2(i  ,j  ,k  ) )**2
     1                    - ( g2b (i-1) * b2(i-1,j  ,k  ) )**2 )
     2                     * g2ai (i) * g2ai (i)
              d1b2po(i) = ( ( g2b (i  ) * b2(i  ,j+1,k  ) )**2
     1                    - ( g2b (i-1) * b2(i-1,j+1,k  ) )**2 )
     2                     * g2ai (i) * g2ai (i)
              d1b3oo(i) = ( ( g31b(i  ) * b3(i  ,j  ,k  ) )**2
     1                    - ( g31b(i-1) * b3(i-1,j  ,k  ) )**2 )
     2                     * g31ai(i) * g31ai(i)
              d1b3op(i) = ( ( g31b(i  ) * b3(i  ,j  ,k+1) )**2
     1                    - ( g31b(i-1) * b3(i-1,j  ,k+1) )**2 )
     2                     * g31ai(i) * g31ai(i)
              d2b3oo(i) = ( ( g32b(j  ) * b3(i  ,j  ,k  ) )**2
     1                    - ( g32b(j-1) * b3(i  ,j-1,k  ) )**2 )
     2                     * g32ai(j) * g32ai(j)
              d2b3op(i) = ( ( g32b(j  ) * b3(i  ,j  ,k+1) )**2
     1                    - ( g32b(j-1) * b3(i  ,j-1,k+1) )**2 )
     2                     * g32ai(j) * g32ai(j)
              d2b1oo(i) = b1(i  ,j  ,k  ) * b1(i  ,j  ,k  )
     1                  - b1(i  ,j-1,k  ) * b1(i  ,j-1,k  )
              d2b1po(i) = b1(i+1,j  ,k  ) * b1(i+1,j  ,k  )
     1                  - b1(i+1,j-1,k  ) * b1(i+1,j-1,k  )
              d3b1oo(i) = b1(i  ,j  ,k  ) * b1(i  ,j  ,k  )
     1                  - b1(i  ,j  ,k-1) * b1(i  ,j  ,k-1)
              d3b1po(i) = b1(i+1,j  ,k  ) * b1(i+1,j  ,k  )
     1                  - b1(i+1,j  ,k-1) * b1(i+1,j  ,k-1)
              d3b2oo(i) = b2(i  ,j  ,k  ) * b2(i  ,j  ,k  )
     1                  - b2(i  ,j  ,k-1) * b2(i  ,j  ,k-1)
              d3b2op(i) = b2(i  ,j+1,k  ) * b2(i  ,j+1,k  )
     1                  - b2(i  ,j+1,k-1) * b2(i  ,j+1,k-1)
11         continue
#endif /* MHD */

CDIR$ UNROLL (UNROLL_I)
           do 20 i=ibeg,iend
c.......................................................................
c
c Perform an explicit update for v1
c
             rhoi    = 2.0 / ( d(i-1,j,k) + d(i,j,k) )
c
c  1.  pressure gradient
c
             st      = - rhoi
     1               * ( poo(i)    - poo(i-1)    ) * dx1bi(i)
#if defined GRAV || defined GRAV_FFT
c
c  2.  gravitational potential gradient
c
             st      = st 
     1               + ( gp(i,j,k) - gp(i-1,j,k) ) * dx1bi(i)
#endif /* GRAV || GRAV_FFT */
#ifdef RTP
c
c  3.  rotational pseudo-forces
c
             st      = st 
     1               + ( s2oo(i  )     + s2po(i  )
     2                 + s2oo(i-1)     + s2po(i-1)   )
     3               * ( u2(i  ,j,k) + u2(i  ,j+1,k  )
     4                 + u2(i-1,j,k) + u2(i-1,j+1,k  ) ) / 16.0
     5               * rhoi * dg2bd1 (i) * g2a i(i)    * g2a i(i)
             st      = st 
     1               + ( s3oo(i  )     + s3op(i  )
     2                 + s3oo(i-1)     + s3op(i-1)   )
     3               * ( u3(i  ,j,k) + u3(i  ,j  ,k+1)
     4                 + u3(i-1,j,k) + u3(i-1,j  ,k+1) ) / 16.0
     5               * rhoi * dg31bd1(i) * g31ai(i)**2 * g32bi(j)
#endif /* RTP */ 
#ifdef PT_MASS
c
c  4.  gravitational point mass 
c
#ifdef XYZ
             r2        = ( x1a(i) - x1ptm )**2
     1                 + ( x2b(j) - x2ptm )**2
     2                 + ( x3b(k) - x3ptm )**2
             r2i       = ( x1a(i) - x1ptm )
     1                 / ( r2    * sqrt(r2   ) + tiny )
#endif /* XYZ */
#ifdef ZRP
             r2        = ( x1a(i) - x1ptm )**2
     1                 + ( x2b(j) - x2ptm )**2
     2                 + 2.0 * x2b(j) * x2ptm
     3                 * ( 1.0 - cos(x3b(k) - x3ptm) )
             r2i       = ( x1a(i) - x1ptm )
     1                 / ( r2    * sqrt(r2   ) + tiny )
#endif /* ZRP */
#ifdef RTP
c jcv Fix from zeusmpv2
c radial force from pt mass depends on radius only
             if(x1ptm .eq. 0.0)then
                r2i = x1ai(i)**2
             else
                prpq      = x1a(i) * sin(x2b(j))
                pzpq      = x1a(i) * cos(x2b(j))
                prpt      = x1ptm  * sin(x2ptm)
                pzpt      = x1ptm  * cos(x2ptm)
c
                r2        = ( pzpq - pzpt )**2
     1               + ( prpq - prpt )**2
     2               + 2.0 * prpq * prpt
     3               * ( 1.0 - cos(x3b(k) - x3ptm) )
                r2i       = ( ( pzpq - pzpt ) * cos(x2b(j))
     1               + ( prpq - prpt * cos(x3b(k) - x3ptm))
     2               * sin(x2b(j))) / ( r2    * sqrt(r2   ) + tiny )
             endif
#endif /* RTP */
ccc
ccc==============================================
ccc delete vertical component of gravity (CSR)
ccc
ccc             st        = st - g * ptmass * r2i
ccc==============================================
ccc

#endif /* PT_MASS */
#if defined RAD || defined RAD_EXP
c
c  5.  Radiation force
c
c        Compute grad(er) at the 1-faces
c
             derdx1 = ( er     (i  ,j  ,k  ) - er     (i-1,j  ,k  ) )
     1              * dx1bi(i)
#ifdef FLD
c
c        Compute flux limiter at the 1-faces
c
      if(er(i-1,j,k) .gt. 1.0d20) then
       write(*,"('eri- error on PE ',i2,' at i,j,k = ',3i3)")
     .       myid,i,j,k
       write(*,"('eri- = ',1pd13.5)")er(i-1,j,k)
       call mpi_finalize(ierr)
       stop
      endif
      if(er(i-1,j-1,k) .gt. 1.0d20) then
       write(*,"('eri-j- error on PE ',i2,' at i,j,k = ',3i3)")
     .       myid,i,j,k
       call mpi_finalize(ierr)
       stop
      endif
      if(er(i,j-1,k) .gt. 1.0d20) then
       write(*,"('erj- error on PE ',i2,' at i,j,k = ',3i3)")
     .       myid,i,j,k
       call mpi_finalize(ierr)
       stop
      endif
      if(er(i,j,k-1) .gt. 1.0d20) then
       write(*,"('erk- error on PE ',i2,' at i,j,k = ',3i3)")
     .       myid,i,j,k
       call mpi_finalize(ierr)
       stop
      endif
      if(er(i-1,j,k-1) .gt. 1.0d20) then
       write(*,"('eroldi-k- error on PE ',i2,' at i,j,k = ',3i3)")
     .       myid,i,j,k
       call mpi_finalize(ierr)
       stop
      endif
#ifdef ONE_DIMENSIONAL
             derdx2 = 0.0
#else /* ONE_DIMENSIONAL */
             derdx2 = ( er     (i-1,j  ,k  ) - er     (i-1,j-1,k  ) )
     1              * dx2bi(j)
     2              + ( er     (i  ,j  ,k  ) - er     (i  ,j-1,k  ) )
     3              * dx2bi(j)
             derdx2 = derdx2 * 0.5 * g2ai(i)
#endif /* ONE_DIMENSIONAL */
#if defined ONE_DIMENSIONAL || defined TWO_DIMENSIONAL
             derdx3 = 0.0
#else /* ONE_DIMENSIONAL || TWO_DIMENSIONAL */
             derdx3 = ( er     (i-1,j  ,k  ) - er     (i-1,j  ,k-1) )
     1              * dx3bi(k)
     2              + ( er     (i  ,j  ,k  ) - er     (i  ,j  ,k-1) )
     3              * dx3bi(k)
             derdx3 = derdx3 * 0.5 * g31ai(i) * g32bi(j)
#endif /* ONE_DIMENSIONAL || TWO_DIMENSIONAL */
             grad_er    = sqrt( derdx1**2 + derdx2**2 + derdx3**2 )
c
c        Compute the Rosseland mean free path at face centers.
c
c            ros_mfp = 1.0 / (kap(i,j,k)+sig(i,j,k))
             rdmcm   = 1.0 / (kap(i-1,j,k) + sig(i-1,j,k))
             rdmcp   = 1.0 / (kap(i  ,j,k) + sig(i  ,j,k))
             ros_mfp = 0.5 * (rdmcm + rdmcp)
c            ros_mfp = rdmcp
c#ifndef OPACITY_PWR_LAW
c            ros_mfp = 1.0 / (ABSORPTION + SCATTERING)
c#else /* not OPACITY_PWR_LAW */
c             temp    = gamm1*e(i,j,k)*mmw*mh/(d(i,j,k)*boltz)
c             ros_mfp = rmfp0 * (rho0/d(i,j,k))**xnu
c     1               * (temp/t_0)**powr
c            power  = powr/4.0 !CJH
c            t_max  = max(er(i  ,j,k),er(i-1,j,k),tiny)    !  Fix
c            rdmcm  = (d(i-1,j,k)/rho0)**xnu * (t_0/t_max)**power
c            rdmcp  = (d(i  ,j,k)/rho0)**xnu * (t_0/t_max)**power
c            ros_mfp = rmfp0 * 2.0 / (rdmcm + rdmcp)
c#endif /* not OPACITY_PWR_LAW */
c
c        Compute the dimensionless quantity R that the flux limiter 
c        depends on.
c
             big_R  = 2.0 * grad_er * ros_mfp 
     1              / max(er(i  ,j,k)+er(i-1,j,k), tiny)
c
c        Compute the flux limiter flx_lim

#ifdef MINERBO
c
c          Minerbo flux limiter
c
             if (big_R .le. 1.5) then
               flx_lim = 2.0/(3.0 + sqrt(9.0 + 12.0*big_R**2) )
             else
               flx_lim = 1.0/(1.0 + big_R + sqrt(1.0 + 2.0*big_R) )
             endif
#else /* MINERBO */
c
c          LP flux limiter
c
             flx_lim = (2.0 + big_R)/(6.0 + 3.0*big_R + big_R**2)
#endif /* MINERBO */
#else /* FLD */
             flx_lim    = 1.0/3.0
#endif /* FLD */
             st         = st - rhoi * flx_lim * derdx1
#endif /* RAD || RAD_EXP */
c
#ifdef MHD
c
c  6. Magnetic pressure (longitudinal Lorentz force)
c
             st = st - (d1b2oo(i) + d1b2po(i) + d1b3oo(i) + d1b3op(i))
     3                 * rhoi * dx1bi(i) * 0.25

#endif /* MHD */
c
             w1(i,j,k) = u1(i,j,k) + dt * st
c.......................................................................
c
c Perform an explicit update for v2 
c
             rhoi    = 2.0 / ( d(i,j-1,k) + d(i,j,k) )
c
c  1.  pressure gradient
c
             st      = - rhoi
     1               * ( poo(i)     - pmo(i)     ) * dx2bi(j)
     2               * g2bi(i)
#if defined GRAV || defined GRAV_FFT
c
c  2.  gravitational potential gradient
c
             st      = st 
     1               + ( gp(i,j,k) - gp(i,j-1,k) ) * dx2bi(j)
     2               * g2bi(i)
#endif /* GRAV || GRAV_FFT */
#if defined ZRP || defined RTP
c
c  3.  rotational pseudo-force
c
             st      = st 
     1               + ( s3oo(i)     + s3op(i)
     2                 + s3mo(i)     + s3mp(i)      )
     3               * ( u3(i,j  ,k) + u3(i,j  ,k+1)
     4                 + u3(i,j-1,k) + u3(i,j-1,k+1) ) / 16.0
     5               * rhoi       * g2b i(i)
     6               * dg32ad2(j) * g31bi(i) * g32ai(j) * g32ai(j)
c jcv Change dg32bd2 to dg32ad2 following zeusmpv2
#endif /* ZRP || RTP */
#ifdef PT_MASS
c
c  4.  graitational point mass
c
#ifdef XYZ
             r2        = ( x1b(i) - x1ptm )**2
     1                 + ( x2a(j) - x2ptm )**2
     2                 + ( x3b(k) - x3ptm )**2
             r2i       = ( x2a(j) - x2ptm )
     1                 / ( r2    * sqrt(r2   ) + tiny )
#endif /* XYZ */
#ifdef ZRP
             r2        = ( x1b(i) - x1ptm )**2
     1                 + ( x2a(j) - x2ptm )**2
     2                 + 2.0 * x2a(j) * x2ptm
     3                 * ( 1.0 - cos(x3b(k) - x3ptm) )
             r2i       = ( x2a(j) - x2ptm * cos(x3b(k) - x3ptm))
     1                 / ( r2    * sqrt(r2   ) + tiny )
#endif /* ZRP */
#ifdef RTP
c jcv Correction from zeusmpv2
c theta component of zero 0 for mass at origin
             if(x1ptm .eq. 0.0)then
                r2i=0.0
             else
                prpq      = x1b(i) * sin(x2a(j))
                pzpq      = x1b(i) * cos(x2a(j))
                prpt      = x1ptm  * sin(x2ptm)
                pzpt      = x1ptm  * cos(x2ptm)
c
                r2        = ( pzpq - pzpt )**2
     1               + ( prpq - prpt )**2
     2               + 2.0 * prpq * prpt
     3               * ( 1.0 - cos(x3b(k) - x3ptm) )
                r2i       = (-( pzpq - pzpt ) * sin(x2a(j))
     1               + ( prpq - prpt * cos(x3b(k) - x3ptm))
     2               * cos(x2a(j))) / ( r2    * sqrt(r2   ) + tiny )
                endif
#endif /* RTP */
             st        = st - g * ptmass * r2i
#endif /* PT_MASS */
#if defined RAD || defined RAD_EXP
c
c  5.  Radiation force
c
c        Compute grad(er) at 2-faces
c
             derdx2 = ( er     (i  ,j  ,k  ) - er     (i  ,j-1,k  ) )
     1              * dx2bi(j) * g2bi(i)
#ifdef FLD
             derdx1 = ( er     (i  ,j-1,k  ) - er     (i-1,j-1,k  ) )
     1              * dx1bi(i)
     2              + ( er     (i  ,j  ,k  ) - er     (i-1,j  ,k  ) )
     3              * dx1bi(i)
             derdx1 = derdx1 * 0.5
#ifdef TWO_DIMENSIONAL
             derdx3 = 0.0
#else /* TWO_DIMENSIONAL */
             derdx3 = ( er     (i  ,j-1,k  ) - er     (i  ,j-1,k-1) )
     1              * dx3bi(k)
     2              + ( er     (i  ,j  ,k  ) - er     (i  ,j  ,k-1) )
     3              * dx3bi(k)
             derdx3 = derdx3 * 0.5 * g31bi(i) * g32ai(j)
#endif /* TWO_DIMENSIONAL */
             grad_er = sqrt( derdx1**2 + derdx2**2 + derdx3**2 )
c
c        Compute the Rosseland mean free path at face centers.
c
c            ros_mfp = 1.0 / (kap(i,j,k) + sig(i,j,k))
             rdmcm   = 1.0 / (kap(i,j-1,k) + sig(i,j-1,k))
             rdmcp   = 1.0 / (kap(i,j  ,k) + sig(i,j  ,k))
             ros_mfp = 0.5 * (rdmcm + rdmcp)
c            ros_mfp = rdmcp
c#ifndef OPACITY_PWR_LAW
c            ros_mfp = 1.0 / (ABSORPTION + SCATTERING)
c#else /* not OPACITY_PWR_LAW */
c             t_max  = max(er(i,j  ,k),er(i,j-1,k),tiny) ! Fix
c             rdmcm  = (d(i,j-1,k)/rho0)**xnu * (t_0/t_max)**power
c             rdmcp  = (d(i,j  ,k)/rho0)**xnu * (t_0/t_max)**power
c             ros_mfp = rmfp0 * 2.0 / (rdmcm + rdmcp)
c#endif /* not OPACITY_PWR_LAW */
c
c        Compute the dimensionless quantity R that the flux limiter 
c        depends on.
c
             big_R  = 2.0 * grad_er * ros_mfp 
     1              / max(er(i,j  ,k)+er(i,j-1,k), tiny)
c
c Compute the flux limiter flx_lim

#ifdef MINERBO
c
c   Minerbo flux limiter
c
             if (big_R .le. 1.5) then
               flx_lim = 2.0/(3.0 + sqrt(9.0 + 12.0*big_R**2) )
             else
               flx_lim = 1.0/(1.0 + big_R + sqrt(1.0 + 2.0*big_R) )
             endif
#else /* MINERBO */
c
c   LP flux limiter
c
             flx_lim = (2.0 + big_R)/(6.0 + 3.0*big_R + big_R**2)
#endif /* MINERBO */
#else /* FLD */
             flx_lim = 1.0/3.0
#endif /* FLD */
             st        = st - rhoi * flx_lim * derdx2
#endif /* RAD || RAD_EXP */
c
#ifdef MHD
c
c  6. Magnetic pressure (longitudinal Lorentz force)
c
             st = st - (d2b3oo(i) + d2b3op(i) + d2b1oo(i) + d2b1po(i))
     3               * rhoi * dx2bi(j) * g2bi(i) * 0.25

#endif /* MHD */
c
             w2(i,j,k) = u2(i,j,k) + dt * st
c
c.......................................................................
c
c Perform an explicit update for v3
c
             rhoi    = 2.0 / ( d(i,j,k-1) + d(i,j,k) )
c
c  1.  pressure gradient
c
             st      = - rhoi
     1               * ( poo(i)    - pom(i)      ) * dx3bi(k)
     2               * g31bi(i) * g32bi(j)
#if defined GRAV || defined GRAV_FFT
c
c  2.  gravitational potential gradient
c
             st      = st 
     1               + ( gp(i,j,k) - gp(i,j,k-1) ) * dx3bi(k)
     2               * g31bi(i) * g32bi(j)
#endif /* GRAV || GRAV_FFT */
#ifdef PT_MASS
c
c  4.  gravitational point mass
c
#ifdef XYZ
             r2        = ( x1b(i) - x1ptm )**2
     1                 + ( x2b(j) - x2ptm )**2
     2                 + ( x3a(k) - x3ptm )**2
             r2i       = ( x3a(k) - x3ptm )
     1                 / ( r2    * sqrt(r2   ) + tiny )
#endif /* XYZ */
#ifdef ZRP
             r2        = ( x1b(i) - x1ptm )**2
     1                 + ( x2b(j) - x2ptm )**2
     2                 + 2.0 * x2b(j) * x2ptm
     3                 * ( 1.0 - cos(x3a(k) - x3ptm) )
             r2i       = x2ptm * sin(x3a(k) - x3ptm)
     1                 / ( r2    * sqrt(r2   ) + tiny )
#endif /* ZRP */
#ifdef RTP
c jcv fix from zeusmpv2
c phi component should be 0 for mass at origin
             if(x1ptm .eq. 0.0)then
                r2i=0.0
             else
                prpq      = x1b(i) * sin(x2b(j))
                pzpq      = x1b(i) * cos(x2b(j))
                prpt      = x1ptm  * sin(x2ptm)
                pzpt      = x1ptm  * cos(x2ptm)
c
                r2        = ( pzpq - pzpt )**2
     1               + ( prpq - prpt )**2
     2               + 2.0 * prpq * prpt
     3               * ( 1.0 - cos(x3a(k) - x3ptm) )
                r2i       = prpt * sin(x3a(k) - x3ptm)
     1               / ( r2    * sqrt(r2   ) + tiny )
             endif
#endif /* RTP */
             st        = st - g * ptmass * r2i
#endif /* PT_MASS */
#if defined RAD || defined RAD_EXP
c
c  5.  Radiation force
c
c        Compute grad(er) at 3-face centers
c 
             derdx3 = ( er     (i  ,j  ,k  ) - er     (i  ,j  ,k-1) )
     1              * dx3bi(k) * g31bi(i) * g32bi(j)
#ifdef FLD
             derdx1 = ( er     (i  ,j  ,k-1) - er     (i-1,j  ,k-1) )
     1              * dx1bi(i)
     2              + ( er     (i  ,j  ,k  ) - er     (i-1,j  ,k  ) )
     3              * dx1bi(i)
             derdx1 = derdx1 * 0.5
             derdx2 = ( er     (i  ,j  ,k-1) - er     (i  ,j-1,k-1) )
     1              * dx2bi(j)
     2              + ( er     (i  ,j  ,k  ) - er     (i  ,j-1,k  ) )
     3              * dx2bi(j)
             derdx2 = derdx2 * 0.5 * g2bi(i)
             grad_er = sqrt( derdx1**2 + derdx2**2 + derdx3**2 )
c
c        Compute the Rosseland mean free path at face centers.
c
c            ros_mfp = 1.0 / (kap(i,j,k) + sig(i,j,k))
             rdmcm   = 1.0 / (kap(i,j,k-1) + sig(i,j,k-1))
             rdmcp   = 1.0 / (kap(i,j,k  ) + sig(i,j,k  ))
             ros_mfp = 0.5 * (rdmcm + rdmcp)
c            ros_mfp = rdmcp
c#ifndef OPACITY_PWR_LAW
c            ros_mfp = 1.0 / (ABSORPTION + SCATTERING)
c#else /* not OPACITY_PWR_LAW */
c             t_max  = max(er(i,j,k  ),er(i,j,k-1),tiny) ! Fix
c             rdmcm  = (d(i,j,k-1)/rho0)**xnu * (t_0/t_max)**power
c             rdmcp  = (d(i,j,k  )/rho0)**xnu * (t_0/t_max)**power
c             ros_mfp = rmfp0 * 2.0 / (rdmcm + rdmcp)
c#endif /* not OPACITY_PWR_LAW */
c
c        Compute the dimensionless quantity R that the flux limiter 
c        depends on.
c
             big_R  = 2.0 * grad_er * ros_mfp 
     1              / max(er(i,j,k  )+er(i,j,k-1), tiny)
c
c        Compute the flux limiter flx_lim

#ifdef MINERBO
c
c          Minerbo flux limiter
c
             if (big_R .le. 1.5) then
               flx_lim = 2.0/(3.0 + sqrt(9.0 + 12.0*big_R**2) )
             else
               flx_lim = 1.0/(1.0 + big_R + sqrt(1.0 + 2.0*big_R) )
             endif
#else /* MINERBO */
c
c          LP flux limiter
c
             flx_lim = (2.0 + big_R)/(6.0 + 3.0*big_R + big_R**2)
#endif /* MINERBO */
#else /* FLD */
             flx_lim   = 1.0/3.0
#endif /* FLD */
             st        = st - rhoi * flx_lim * derdx3
#endif /* RAD || RAD_EXP */
c
#ifdef MHD
c
c  6. Magnetic pressure (longitudinal Lorentz force)
c
             st = st - (d3b1oo(i) + d3b1po(i) + d3b2oo(i) + d3b2op(i))
     3               * rhoi * dx3bi(k) * g31bi(i) * g32bi(j) * 0.25

#endif /* MHD */
c
             w3(i,j,k) = u3(i,j,k) + dt * st
20         continue
30       continue
40     continue
c
#endif /* !NO_FORCES */
#endif /* !PNEWT */
       return
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////                F O R C E S                \\\\\\\\\\
c
c=======================================================================
c
c
