#include "zeusmp.def"
c=======================================================================
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
      subroutine hdfall(filename)
c
c  PURPOSE: Makes an hdf dump of all the active field variables.  The
c  set of field variables dumped is problem specific (depends on what
c  physics is defined).  Data is written in the Scientific Data Set
c  format to the file zhzXXNNNNNN.MMM.
c  Note that data must be stored column major and contiguously in order
c  to interface correctly to the C hdf routines.  All variables are
c  dumped as zone centered quantities.
c
c  EXTERNALS: HDF library routines
c
c  LOCALS:
c
c  LAST MODIFIED: by JCH; 3/12/97.
c-----------------------------------------------------------------------
      implicit NONE
c
      character*15 filename
#ifdef HDF_DUMPS
      character*16 coordsys
      character*32 string
c
#include "mod_names.h"
#include "param.h"
#include "grid.h"
#include "field.h"
#include "root.h"
#include "scratch.h"
#include "mpicom.h"
#if defined RAD || defined RAD_EXP
#include "cons.h"
#endif /* RAD || RAD_EXP */
      integer  i,j,k,indx
c
c real is correct on PVP Crays, but real*4 is required on T3E.
c This is due to the kind of C floats on the two machines according to
c Albert Cheng of the NCSA HDF group.  The varying integer definitions
c I have decided on by trial and error M-MML 17.5.98
c
#if ! defined T3E && defined UNICOS
      integer rank,shape(3),ret
      real data(in*jn*kn),xscale(in),yscale(jn),zscale(kn)
      integer  dssdims,dssdast,dssdisc,dsadata,dspdata
#elif defined T3E
      integer rank,shape(3),ret
      real*4 data(in*jn*kn),xscale(in),yscale(jn),zscale(kn)
      integer  dssdims,dssdast,dssdisc,dsadata,dspdata
#else
      integer*4 rank,shape(3),ret
      real*4 data(in*jn*kn),xscale(in),yscale(jn),zscale(kn)
      integer*4 dssdims,dssdast,dssdisc,dsadata,dspdata
#endif
      REAL dum
c
c      equivalence (xscale,w1da),(yscale,w1db),(zscale,w1dc)
c      equivalence (data  ,w3df)
c
      external dssdims,dssdast,dssdisc,dsadata,dspdata
c
c\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\///////////////////////////////
c=======================================================================
c
#ifdef RTP
      coordsys = 'spherical polar' // char(0)
#endif
#ifdef ZRP
      coordsys = 'cylindrical' // char(0)
#endif
#ifdef XYZ
      coordsys = 'cartesian' // char(0)
#endif
      do 10 i=is,ie
        xscale(i-is+1) = real(x1b(i))
10    continue
      do 20 j=js,je
        yscale(j-js+1) = real(x2b(j))
20    continue
      do 30 k=ks,ke
        zscale(k-ks+1) = real(x3b(k))
30    continue
c
      rank     = 3
      shape(1) = nx1z
      shape(2) = nx2z
      shape(3) = nx3z
      ret = dssdims(rank,shape)
      ret = dssdisc(1,shape(1),xscale)
      ret = dssdisc(2,shape(2),yscale)
      ret = dssdisc(3,shape(3),zscale)
c
c Put the vector fields first for use with the 4d2 visualization pkg.
c
c  1-velocity
c
#ifdef MPI_USED
      if(myid .EQ. 0)then
#endif /* MPI_USED */
         write(*,*)"writing hdf file ",filename, " at t= ",time
#ifdef MPI_USED
      endif
#endif /* MPI_USED */
      do 120 k=ks,ke
        do 110 j=js,je
          do 100 i=is,ie
            indx = (k-ks)*nx2z*nx1z + (j-js)*nx1z + (i-is) + 1
	    data(indx) = real(0.5*(v1(i,j,k) + v1(i+1,j,k)))
100       continue
110     continue
120   continue
      write(string,"('1-VELOCITY AT TIME=',1pe8.2,'     ')") time
      ret = dssdast(string,' ',' ',coordsys)
      ret = dspdata(filename,rank,shape,data)
c
c  2-velocity
c
      do 220 k=ks,ke
        do 210 j=js,je
          do 200 i=is,ie
            indx = (k-ks)*nx2z*nx1z + (j-js)*nx1z + (i-is) + 1
	    data(indx)  = real(0.5*(v2(i,j,k) + v2(i,j+1,k)))
200       continue
210     continue
220   continue
      write(string,"('2-VELOCITY AT TIME=',1pe8.2,'     ')") time
      ret = dssdast(string,' ',' ',coordsys)
      ret = dsadata(filename,rank,shape,data)
c
c  3-velocity
c
      do 320 k=ks,ke
        do 310 j=js,je
          do 300 i=is,ie
            indx = (k-ks)*nx2z*nx1z + (j-js)*nx1z + (i-is) + 1
	    data(indx)  = real(0.5*(v3(i,j,k) + v3(i,j,k+1)))
300       continue
310     continue
320   continue
      write(string,"('3-VELOCITY AT TIME=',1pe8.2,'     ')") time
      ret = dssdast(string,' ',' ',coordsys)
      ret = dsadata(filename,rank,shape,data)
#ifdef MHD
c
c  1-magnetic field
c
      do 420 k=ks,ke
        do 410 j=js,je
          do 400 i=is,ie
            indx = (k-ks)*nx2z*nx1z + (j-js)*nx1z + (i-is) + 1
            data(indx)  = real(0.5*(b1(i,j,k) + b1(i+1,j,k)))
400       continue
410     continue
420   continue
      write(string,"('1-MAG FIELD AT TIME=',1pe8.2,'    ')") time
      ret = dssdast(string,' ',' ',coordsys)
      ret = dsadata(filename,rank,shape,data)
c
c  2-magnetic field
c
      do 520 k=ks,ke
        do 510 j=js,je
          do 500 i=is,ie
            indx = (k-ks)*nx2z*nx1z + (j-js)*nx1z + (i-is) + 1
	    data(indx)  = real(0.5*(b2(i,j,k) + b2(i,j+1,k)))
500       continue
510     continue
520   continue
      write(string,"('2-MAG FIELD AT TIME=',1pe8.2,'    ')") time
      ret = dssdast(string,' ',' ',coordsys)
      ret = dsadata(filename,rank,shape,data)
c
c  3-magnetic field
c
      do 620 k=ks,ke
        do 610 j=js,je
          do 600 i=is,ie
            indx = (k-ks)*nx2z*nx1z + (j-js)*nx1z + (i-is) + 1
	    data(indx)  = real(0.5*(b3(i,j,k) + b3(i,j,k+1)))
600       continue
610     continue
620   continue
      write(string,"('3-MAG FIELD AT TIME=',1pe8.2,'    ')") time
      ret = dssdast(string,' ',' ',coordsys)
      ret = dsadata(filename,rank,shape,data)
#endif /* MHD */
c
c  density
c
      do 720 k=ks,ke
        do 710 j=js,je
          do 700 i=is,ie
            indx = (k-ks)*nx2z*nx1z + (j-js)*nx1z + (i-is) + 1
            data(indx) = real(d(i,j,k))
700       continue
710     continue
720   continue
      write(string,"('DENSITY AT TIME=',1pe8.2,'        ')") time
      ret = dssdast(string,' ',' ',coordsys)
      ret = dsadata(filename,rank,shape,data)

c
c  internal energy
c
      do 820 k=ks,ke
        do 810 j=js,je
          do 800 i=is,ie
            indx = (k-ks)*nx2z*nx1z + (j-js)*nx1z + (i-is) + 1
c           data(indx) = real(e(i,j,k))
            dum = e(i,j,k)
            if(dum .lt. 0) dum = 1.0e20
            data(indx) = real(dum)
800       continue
810     continue
820   continue
      write(string,"('GAS ENERGY AT TIME=',1pe8.2,'   ')") time
      ret = dssdast(string,' ',' ',coordsys)
      ret = dsadata(filename,rank,shape,data)
#if defined GRAV || defined GRAV_FFT
c
c  gravitational potential
c
      do 920 k=ks,ke
        do 910 j=js,je
          do 900 i=is,ie
            indx = (k-ks)*nx2z*nx1z + (j-js)*nx1z + (i-is) + 1
            data(indx) = real(gp(i,j,k))
900       continue
910     continue
920   continue
      write(string,"('GRAV POT AT TIME=',1pe8.2,'       ')") time
      ret = dssdast(string,' ',' ',coordsys)
      ret = dsadata(filename,rank,shape,data)
#endif /* GRAV || GRAV_FFT */
#if defined RAD || defined RAD_EXP
c
c  radiation internal energy
c
      do 1020 k=ks,ke
        do 1010 j=js,je
          do 1000 i=is,ie
            indx = (k-ks)*nx2z*nx1z + (j-js)*nx1z + (i-is) + 1
#if defined NO_FORCES && defined NO_TRANSPORT && defined NO_COUPLING
c           data(indx) = real(sqrt(sqrt(max(zro,er(i,j,k))/rad_con)))
            if(er(i,j,k) .lt. 0.D0) er(i,j,k) = 1.D20
            data(indx) = real(er(i,j,k))
#else /* NO_FORCES && NO_TRANSPORT && NO_COUPLING */
c           data(indx) = real(sqrt(sqrt(max(zro,er(i,j,k))/rad_con)))
            data(indx) = real(er(i,j,k))
#endif /* NO_FORCES && NO_TRANSPORT && NO_COUPLING */
1000      continue
1010    continue
1020  continue
#ifdef RAD_EXP
      write(string,"('RADIATION T AT TIME=',1pe8.2,'    ')") time
#else /* RAD_EXP */
      write(string,"('RADIATION T AT TIME=',1pe8.2,'    ')") time
#endif /* RAD_EXP */
      ret = dssdast(string,' ',' ',coordsys)
      ret = dsadata(filename,rank,shape,data)
#endif /* RAD || RAD_EXP */
#endif /* HDF_DUMPS */
      return
      end
