#include "zeusmp.def"
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////            I N I T G R A V                \\\\\\\\\\
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
c=======================================================================
c
       subroutine initgrav
#if alias GRAVITY.eq.fftgrav

c
c    ab:zeus3d.initgrav  <-------------- initialization routine for 
c                                        gravity solver fftgrav
c    from:
c
c    written by: Fabian Heitsch
c    modified1:
c
c  PURPOSE:  Declares and initializes parameters and arrays for the
c            gravity solver fftgrav. This routine has to
c            be called in the problem deck. 
c
c  INPUT PARAMETERS: none
c
c  LOCAL VARIABLES: isys, nwork, x, error, n1, n2, n3
c
c  COMMON BLOCKS  : common /grvcom / g0, tgrav
c                   common /tabcom / table
c
c  EXTERNALS:     : fgreen
c  
c
c-----------------------------------------------------------------------
c
      integer n1,n2,n3
      integer ntable,nwork,iseed
      integer isys
      parameter (ntable=90+2*(in+jn+kn))
      parameter (nwork=2*in*jn*kn)
      parameter (iseed=-157)

      REAL table(ntable),work(nwork),error
      REAL green(0:in-1,0:jn-1,0:kn-1)

c assume that all zones are used (ie-is+1 = in-5)
      complex*16  x(0:in-6,0:jn-6,0:kn-6)
#ifdef SOLARIS
      double complex  x(0:in-6,0:jn-6,0:kn-6)
#endif /* SOLARIS */

      common /tabcom / table
      common /greendat/ green

c careful with the scratch arrays!
      equivalence (x(0,0,0), wa3d(1,1,1)),
     &              (work(1),  wc3d(1,1,1))

      external fgreen


c--------------------------------------------------------

#ifdef DBGZEUS_INITGRAV
      print*,'  initgrav : called'
#endif /* DBGZEUS_INITGRAV */

#ifdef DBGZEUS_INITGRAV
      print*,'  initgrav :   setting up fftgrav'
#endif /* DBGZEUS_INITGRAV */

c set the gravitational constant to tiny initially (it will be
c set to g0 after a time tgrav)
c
      if ((tgrav .gt. 0.0).and.(time .le. tgrav)) then
        g = tiny
      else
        g = g0
        tgrav = huge
      endif
       
c set up the Green's functions for our grid
#ifdef DBGZEUS_INITGRAV
      print*,'  initgrav :   call fgreen'
#endif /* DBGZEUS_INITGRAV */
      call fgreen

      if (iresdmp .ne. 0) then
c this section is only to be called if we have a
c restart dump, as in fftgrav, the initialization wouldn't
c be carried out.
c     set up constants to start FFT solver
        isys = 0
        n1 = ie - is + 1
        n2 = je - js + 1
        n3 = ke - ks + 1
#ifdef DBGZEUS_INITGRAV
        print*,'  initgrav :   call ??fft3d'
#endif /* DBGZEUS_INITGRAV */
        call zzfft3d(0,n1,n2,n3,1d0,
     &                x,n1,n2,x,n1,n2,table,work,isys)
      endif

#ifdef DBGZEUS_INITGRAV
      print*,'  initgrav : finished'
#endif /* DBGZEUS_INITGRAV */

#endif GRAVITY.eq.fftgrav
      return
      end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////              I N I T G R A V              \\\\\\\\\\
c
c=======================================================================
