#include "zeusmp.def"

c JCV 10/27/03
c subroutine to update boundaries and turn jet off
c based on my subroutine from zeus3d
c JCV 01/12/05
c start jet after some delay, then stop after another delay

c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                 JETLATE                    \\\\\\\\\\
c
c=======================================================================
c

       subroutine jetlate

       implicit NONE

#include "mod_names.h"
#include "param.h"
#include "field.h"
#include "bndry.h"
#include "grid.h"
#include "root.h"
#include "scratch.h"
#include "mpicom.h"
#include "cons.h"

       character*15 hdfname
       integer i,j,k

       integer thetamax

       REAL offtime,ontime

       REAL mass,dm,totmass,area
c Scaling factors real/code units
       REAL lscale,tiscale,Tescale,mscale,escale
c Real units kbolt[eV K-1],muu mean moleculare weight
       REAL kbolt,muu

c only want inner edge
       i=1
c check to make sure this is a tile that touches inner boundary
c currently I do not have more than 1 tile in x1, but should anyway
c NOTE if there are too many processors (~10), this will no longer work

       muu=1.27
       kbolt=1.4861E-9

       lscale=rcore*1000*cmpc/2
       tiscale=lscale/(cism*1000*100)
       Tescale=mp*muu*((cism*1000*100)**2)/(gamma*kbolt*3.0/5.0)
       mscale=muu*mp*ncent*(lscale**3)
       escale=(lscale**2)*mscale/(tiscale**2)

       if (x1b(is) .LE. 1) then
       
          mass=0
          do k=ks,ke
             do j=js,je
                area=dvl2a(j)*dvl3a(k)
                dm=d(i,j,k)*v1(i,j,k)*area
                if(v1(i,j,k).le.0.0)then
                   mass=mass+dm
                endif
             enddo
          enddo

          mass=-1.0*mass*mscale*secyr/(tiscale*msol)

          call MPI_ALLREDUCE(mass,totmass,1,MPI_FLOAT,MPI_SUM
     &         ,comm3d,ierr)

          if(myid.eq.0)then
             write(*,*)"total mass across i=1 boundary = ",totmass
          endif

       endif


       v1jet   =  66.461

       ontime=0.5
       offtime=1.5

c---Check to see if it is time to turn jet on
       if (time .GT. ontime .AND. time .LT. ontime + dt) then

          if(myid.eq.0)then
             write(6,"(/'Turning jet on')")
          endif

          thetamax=J_ZONES*ntiles(2)/jetsize
          thetamax=thetamax*2.0
          if (myid .eq. 0) then
             write(*,*)'thetamax=',thetamax,' v1jet=',v1jet
             write(6,"(10x,'RGJET  : Initialisation commencing')")
          endif

          if (x1b(is) .LE. 1) then
c Do first jet if t direction starts close to 0
             if (x2b(js) .LE. 0.1) then

c 1st jet
                do j=js-2,js+thetamax
                   do k=ks-2,ke+2
                      niib(j,k)=3
                      v1iib(j,k,1)=v1jet
                      v1iib(j,k,2)=v1jet
                   enddo
                enddo

c end loop for 1st jet
             endif
          endif
ccccccccccc

          call bndyflgs
          write(hdfname,"(a4,3i2.2,'.999')") 'test',coords(1)
     &         ,coords(2),coords(3)
          call hdfall(hdfname)
       endif

c---Check to see if it is time to turn jet off
       if (time .GT. offtime .AND. time .LT. offtime + dt) then

          if(myid.eq.0)then
             write(6,"(/'Turning jet off')")
          endif
c 1st jet
       do j=js-2,je+2
          do k=ks-2,ke+2
             niib(j,k)=2
             v1iib(j,k,1)=0.0
             v1iib(j,k,2)=0.0
          enddo
       enddo


       endif

       return
       end


