#include "zeusmp.def"

c JCV 10/27/03
c subroutine to update boundaries and turn jet off
c based on my subroutine from zeus3d
c JCV 11/22/04
c calculate m dot across inner boundary for feedback
c jcv 02/28/05
c get mdot on inner edge, not ghost zones
c JCV 08/01/05
c fixed bug that prevented multiple mpi tiles in x1

c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                 MDOT                      \\\\\\\\\\
c
c=======================================================================
c

       subroutine mdot

       implicit NONE

#include "mod_names.h"
#include "param.h"
#include "field.h"
#include "bndry.h"
#include "grid.h"
#include "root.h"
#include "scratch.h"
#include "mpicom.h"
#include "cons.h"

       integer i,j,k
       REAL mass,dm,totmass,area
       REAL offtime
c Scaling factors real/code units
       REAL lscale,tiscale,Tescale,mscale,escale
c Real units kbolt[eV K-1],muu mean moleculare weight
       REAL kbolt,muu

c only want inner edge, but not ghost zones
       i=3
c check to make sure this is a tile that touches inner boundary
c currently I do not have more than 1 tile in x1, but should anyway
c NOTE if there are too many processors (~10), this will no longer work

       muu=1.27
       kbolt=1.4861E-9

       lscale=rcore*1000*cmpc/2
       tiscale=lscale/(cism*1000*100)
       Tescale=mp*muu*((cism*1000*100)**2)/(gamma*kbolt*3.0/5.0)
       mscale=muu*mp*ncent*(lscale**3)
       escale=(lscale**2)*mscale/(tiscale**2)
       mass=0.0
       if (x1b(is) .LE. 1) then
          do k=ks,ke
             do j=js,je
                area=dvl2a(j)*dvl3a(k)
                dm=d(i,j,k)*v1(i,j,k)*area
                if(v1(i,j,k).le.0.0)then
                   mass=mass+dm
                endif
             enddo
          enddo
          mass=-1.0*mass*mscale*secyr/(tiscale*msol)
       endif


       call MPI_ALLREDUCE(mass,totmass,1,MPI_FLOAT,MPI_SUM
     &      ,comm3d,ierr)


       if (x1b(is) .LE. 1) then          
          if(myid.eq.0)then
             write(*,*)"total mass across inner boundary = ",totmass
          endif
       endif

       offtime=1.0

c---Check to see if it is time to turn jet off
       if (time .GT. offtime .AND. time .LT. offtime + dt) then

          if(myid.eq.0)then
             write(6,"(/'Turning jet off')")
          endif
c 1st jet       
       do j=js-2,je+2
          do k=ks-2,ke+2
             niib(j,k)=2
             v1iib(j,k,1)=0.0
             v1iib(j,k,2)=0.0
c for XYZ geometry in viscous runs
c           do i=is-2,ie+2
c              v3ikb(i,j,1)=0.0
c              v3ikb(i,j,2)=0.0

          enddo
       enddo


       endif

       return
       end
