#include "zeusmp.def"

c JCV 7/24/2006 a tilted, precessing jet.

c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                 MDOT_PREC                 \\\\\\\\\\
c
c=======================================================================
c
       subroutine mdot_prec

       implicit NONE
#include "mod_names.h"
#include "param.h"
#include "field.h"
#include "bndry.h"
#include "grid.h"
#include "root.h"
#include "scratch.h"
#include "mpicom.h"
#include "cons.h"

       integer i,j,k

       integer thetamax
       REAL alphar,beta
       REAL vx,vy,vz,vr,vt,vp
       REAL offtime
       integer njet
       REAL mass,dm,totmass,area
c Scaling factors real/code units
       REAL lscale,tiscale,Tescale,mscale,escale
c Real units kbolt[eV K-1],muu mean moleculare weight
       REAL kbolt,muu
       namelist /jet/ djet,v1jet,jetsize,efficiency,mcutoff
       namelist /prec/ palpha,pomega

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c This block is to calculate mdot
       i=3
c check to make sure this is a tile that touches inner boundary
c currently I do not have more than 1 tile in x1, but should anyway
c NOTE if there are too many processors (~10), this will no longer work

       muu=1.27
       kbolt=1.4861E-9

       lscale=rcore*1000*cmpc/2
       tiscale=lscale/(cism*1000*100)
       Tescale=mp*muu*((cism*1000*100)**2)/(gamma*kbolt*3.0/5.0)
       mscale=muu*mp*ncent*(lscale**3)
       escale=(lscale**2)*mscale/(tiscale**2)
       mass=0.0
       if (x1b(is) .LE. 1) then
          do k=ks,ke
             do j=js,je
                area=dvl2a(j)*dvl3a(k)
                dm=d(i,j,k)*v1(i,j,k)*area
                if(v1(i,j,k).le.0.0)then
                   mass=mass+dm
                endif
             enddo
          enddo
          mass=-1.0*mass*mscale*secyr/(tiscale*msol)
       endif


       call MPI_ALLREDUCE(mass,totmass,1,MPI_FLOAT,MPI_SUM
     &      ,comm3d,ierr)


       if (x1b(is) .LE. 1) then          
          if(myid.eq.0)then
             write(*,*)"total mass across inner boundary = ",totmass
          endif
       endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

       njet=3
       offtime=1.0

c This is alpha in radians
       alphar=(palpha/180.0)*pi
       if(pomega.eq.0.0)then
          beta=0.0
       else
          beta=(time/(2.0*pomega))*pi
       endif
       thetamax=J_ZONES*ntiles(2)/jetsize
c Uncomment the next line if there is only one jet
          thetamax=thetamax*2.0
c Only look for jet near inner edge
      if (x1b(is) .LE. 1) then

c NOTE if there are too many processors (~10), this will no longer work

c Do first jet if t direction starts close to 0
         if (x2b(js) .LE. 0.1) then

c 1st jet
            do j=js-2,js+thetamax
               do k=ks-2,ke+2
                  diib(j,k,1)=djet
                  diib(j,k,2)=djet
                  eiib(j,k,1)=9.0/10.0
                  eiib(j,k,2)=9.0/10.0
                  
                  vz=v1jet*cos(alphar)
                  vy=v1jet*sin(alphar)*cos(beta)
                  vx=v1jet*sin(alphar)*sin(beta)

#ifdef OFF
      if (time .GT. offtime .AND. time .LT. offtime + dt) then
         
         if(myid.eq.0)then
            write(6,"(/'Turning jet off')")
         endif
         njet=2
         vx=0.0
         vy=0.0
         vz=0.0
      endif
#endif /* OFF */

                  niib(j,k)=njet
                  vr=   vx*(cos(x3a(k))*sin(x2a(j)))
     &                 +vy*(sin(x3a(k))*sin(x2a(j)))
     &                 +vz*(cos(x2a(j)))

                  vt=   vx*(cos(x3a(k))*cos(x2a(j)))
     &                 +vy*(sin(x3a(k))*cos(x2a(j)))
     &                 +vz*(-1.0*sin(x2a(j)))

                  vp=   vx*(-1.0*sin(x3a(k)))
     &                 +vy*cos(x3a(k))

                  v1iib(j,k,1)=vr
                  v1iib(j,k,2)=vr
                  v2iib(j,k,1)=vt
                  v2iib(j,k,2)=vt
                  v3iib(j,k,1)=vp
                  v3iib(j,k,2)=vp

               enddo
            enddo

c end loop for 1st jet
         endif
c end loop for both jets
      endif
ccccccccccc



       return
       end
