c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
C ******************************* 
C *** USER-CONTROLLED DEFINES *** 
C ******************************* 

#define MGRID

C --------------------- 
C Convergence criterion 
C --------------------- 

C Stop when ...
C  1 ||R|| <= eps*(||A|| ||X|| + ||B||)         (Least stringent)
C  2 ||R|| <= eps*||B||                                 :
C  3 |R(i)/B(i)| <= eps for all i               (Most stringent)
C

#define CONV_CRIT 2

C --------------------- 
C Communication defines 
C --------------------- 

#define MPI_USED    /* MPI is to be used */
#define MPI_BLOCKING    /* use MPI blocking calls, otherwise use non-blocking */
C                         calls when available      

C ------------------- 
C Performance defines 
C ------------------- 

C #define PERF_EVAL     /* collect performance data */
#define DOUBLE_PRECISION /* use double precision (64-bit floats) */
C #define PARAGRAPH  /* dump send/receive trace file for use with ParaGraph */

C ------------------------- 
C Debugging-related defines 
C ------------------------- 

C #define DEBUG /* print out debugging information */
C #define DEBUG_MG /* mmMgVcycle debug statements  */
C #define DEBUG_BICG  /* mmBiCGStab debug statements  */
C #define DEBUG_CG  /* mmCG debug statements  */
C #define DEBUG_SMOOTH  /* Smoother debug statements  */
C #define DEBUG_CG2  /* mmCg2 debug statements  */
C #define DEBUG_PROLONG  /* mmProlong* debug statements  */


C #define DEBUG_PROCS /* mmCg2 debug statements  */
C #define PROC_TRACE  /* print some procedure names when entered  */  
C #define MPI_TRACE /* print out *all* calls to MPI routines  */
C #define ASSERT       /* perform assertion computations  */
C                         /*  (see "man assert" for C language)  */
C #define DUMP_INPUTS      /* display input values to some subroutines  */

C ---------------------------------------------
C The following should generally be left alone 
C ---------------------------------------------

#ifdef DOUBLE_PRECISION
#define REAL real*8
#define MM_REAL MPI_REAL8
#define REAL_SIZE 8
#else
#define REAL real*4
#define MM_REAL MPI_REAL4
#define REAL_SIZE 4
#endif

#define ICOMMSTART 200

#define SB_MG3 1
#define SB_MGSOLVE 2 
#define SB_DISTRIBUTE 3
#define SB_BUILD 4  /* levels  */

C Main 3D solvers 
#define SB_MG 10 
#define SB_CGMG 11
#define SB_CG  12 /* levels  */
#define SB_PRECON 18
#define SB_FMG 19

C Smoother 2D solvers 

#define SB_MG2 20   /* levels  */
#define SB_CG2 30  /* levels  */

C 3D linear algebra 

#define SB_MAT_DIAG 39
#define SB_MATVEC 40  /* levels  */
#define SB_RESIDUAL 50  /* levels  */
#define SB_ZAXPY 60  /* levels  */
#define SB_DOTPROD 70  /* levels  */
#define SB_SUM 80  /* levels  */

C 2D linear algebra  
#define SB_MATVEC2 90 /* levels  */
#define SB_RESIDUAL2 100  /* levels  */
#define SB_ZAXPY2  110 /* levels  */
#define SB_DOTPROD2 120  /* levels  */

C 3D multigrid 

#define SB_RESTRICT 130 /* levels  */
#define SB_PROLONG 140  /* levels  */
#define SB_SMOOTH 150  /* levels  */
#define SB_CORRECT 160  /* levels  */

C 2D multigrid 

#define SB_RESTRICT2 170  /* levels  */
#define SB_PROLONG2 180  /* levels  */


C Communication 

#define SB_GHOST 190  /* levels  */
#define SB_GHOST2 200  /* levels  */
#define SB_IGHOST 210  /* levels  */
#define SB_MATVEC_WAIT 220
#define SB_SMOOTH_WAIT 221
#define SB_RESIDUAL_WAIT 222
#define SB_REFRESH 230  /* levels  */

C convergence tests 

#define SB_CONVERGE 240  /* 2   */

#define SB_NORMALIZE 250  /* levels  */
#define SB_SHIFT 260  /* levels  */

#define SB_DRIVER 270

C  NOTE: highest value must be lower than NTIMER in mgmpi.h 



