#include "zeusmp.def"
c=======================================================================
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
      program msgt
c
c Main program to determine the interconnection network bandwidth and 
c latency.
c
c METHOD
c
c   The communication time for a message is the sum of the latency
c   plus the message length times the time per Byte:
c
c       T_total  =  Latency  +  Length / Bandwidth
c
c   In this program, we time the transmission of messages of two
c   different lengths, and then solve a system of 2 equations in
c   two unknowns to get the latency and bandwidth.
c
c   We time itrips of each length of message for better accuracy.
c
c Robert Fiedler, 11/18/96.
c.......................................................................
c
c DECLARATIONS
c
      implicit NONE
#include "param.h"
#include "mpicom.h"
#include "mod_names.h"
#include "clocks.h"
c
      integer i,j,k,s1,s2
      integer ilarge,ismall,itrips
      parameter(ismall=32+5,ilarge=128+5,itrips=100)
      REAL cw, su, tw1, tw2, tc1, tc2
      REAL a(ismall*ismall*ismall), b(ilarge*ilarge*ilarge)
c
c Initialize MPI
c
      call MPI_INIT( ierr )
      call MPI_ERRHANDLER_SET(MPI_COMM_WORLD,MPI_ERRORS_RETURN,ierr)
      call MPI_COMM_RANK( MPI_COMM_WORLD, myid_w  , ierr )
      call MPI_COMM_SIZE( MPI_COMM_WORLD, nprocs_w, ierr )
#ifdef DEBUG
      write(*,"('Process ',i3,'  of ',i3,'  + 1 initialized')") 
     1  myid_w, nprocs_w - 1 
#endif /* DEBUG */
c
      s1 = ismall**2
      s2 = ilarge**2
c
c Send messages back and forth to determine bandwidth and latency.
c
      do 17 k = 1, nprocs_w-1
      do 10 i=1,s1
        a(i              ) =  real(k)
        a(i+s1*(ismall-1)) = -real(k)
10    continue
c
c Pass one message to prime the pump.
c
      if (myid_w.eq.0) then
c
c Send first message
c
#ifdef DEBUG
        write(*,"('Sending first message')")
#endif
        call MPI_SEND(a,s1,MPI_FLOAT,k,1,MPI_COMM_WORLD,ierr)
        call MPI_RECV(a(s1*(ismall-1)),s1,MPI_FLOAT,k,1
     1      ,MPI_COMM_WORLD,stat,ierr)
      else if (myid_w .eq. k) then
c
c Receive first message
c
#ifdef DEBUG
        write(*,"('Receiveing first message')")
#endif
        call MPI_RECV(a(s1*(ismall-1)),s1,MPI_FLOAT,0,1
     1      ,MPI_COMM_WORLD,stat,ierr)
        call MPI_SEND(a,s1,MPI_FLOAT,0,1,MPI_COMM_WORLD,ierr)
      endif
      call MPI_BARRIER(MPI_COMM_WORLD,ierr)
c
c  Initialize cpu and wall clocks.  The quantities "cputime" and
c  "wclock" are the CPU and wall clock times (in seconds).
c
        wclock0 = 0.0
        cputime0 = 0.0
        call clocks (cputime, wclock)
#ifdef DEBUG
        write(*,"(/'MSGT: clocks returned cputime = ',g12.5)") cputime
#endif /* DEBUG */
        wclock0 = wclock
        cputime0 = cputime
c
      do 15 j=1,itrips
      if (myid_w.eq.0) then
c
c Send first message
c
#ifdef DEBUG
        write(*,"('Sending first message')")
#endif
        call MPI_SEND(a,s1,MPI_FLOAT,k,1,MPI_COMM_WORLD,ierr)
        call MPI_RECV(a(s1*(ismall-1)),s1,MPI_FLOAT,k,1
     1      ,MPI_COMM_WORLD,stat,ierr)
      else if (myid_w .eq. k) then
c
c Receive first message
c
#ifdef DEBUG
        write(*,"('Receiveing first message')")
#endif
        call MPI_RECV(a(s1*(ismall-1)),s1,MPI_FLOAT,0,1
     1      ,MPI_COMM_WORLD,stat,ierr)
        call MPI_SEND(a,s1,MPI_FLOAT,0,1,MPI_COMM_WORLD,ierr)
      endif
15    continue
      if (myid_w.eq.k) then
c
c Record the time
c
        call clocks (cputime, wclock)
        tc1 = 0.5*cputime/real(itrips)
        tw1 = 0.5*wclock/real(itrips)
      endif
c
c Ready the second message.
c
      do 20 i=1,s2
        b(i              ) =  real(k)
        b(i+s2*(ilarge-1)) = -real(k)
20    continue
c
c Pass one message to prime the pump
c
      if (myid_w.eq.0) then
c
c Send second message
c
#ifdef DEBUG
        write(*,"('Sending second message')")
#endif
        call MPI_SEND(b,s2,MPI_FLOAT,k,2,MPI_COMM_WORLD,ierr)
        call MPI_RECV(b(s2*(ilarge-1)),s2,MPI_FLOAT,k,2
     1      ,MPI_COMM_WORLD,stat,ierr)
      else if (myid_w .eq. k) then
c
c Receive second message
c
#ifdef DEBUG
        write(*,"('Receiveing second message')")
#endif
        call MPI_RECV(b(s2*(ilarge-1)),s2,MPI_FLOAT,0,2
     1      ,MPI_COMM_WORLD,stat,ierr)
        call MPI_SEND(b,s2,MPI_FLOAT,0,2,MPI_COMM_WORLD,ierr)
      endif
      call MPI_BARRIER(MPI_COMM_WORLD,ierr)
c
c  Reset cpu and wall clocks.
c
        wclock0 = 0.0
        cputime0 = 0.0
        call clocks (cputime, wclock)
#ifdef DEBUG
        write(*,"(/'MSGT: clocks returned wclock = ',g12.5)") wclock
#endif /* DEBUG */
        wclock0 = wclock
        cputime0 = cputime
c
      do 25 j=1,itrips
      if (myid_w.eq.0) then
c
c Send second message
c
#ifdef DEBUG
        write(*,"('Sending second message')")
#endif
        call MPI_SEND(b,s2,MPI_FLOAT,k,2,MPI_COMM_WORLD,ierr)
        call MPI_RECV(b(s2*(ilarge-1)),s2,MPI_FLOAT,k,2
     1      ,MPI_COMM_WORLD,stat,ierr)
      else if (myid_w .eq. k) then
c
c Receive second message
c
#ifdef DEBUG
        write(*,"('Receiveing second message')")
#endif
        call MPI_RECV(b(s2*(ilarge-1)),s2,MPI_FLOAT,0,2
     1      ,MPI_COMM_WORLD,stat,ierr)
        call MPI_SEND(b,s2,MPI_FLOAT,0,2,MPI_COMM_WORLD,ierr)
      endif
25    continue
c
      if (myid_w .eq. k) then
c
c Record the time
c
        call clocks (cputime, wclock)
        tc2 = 0.5*cputime/real(itrips)
        tw2 = 0.5*wclock/real(itrips)
c
c Now figure out the latency and bandwidth
c
        cw = (tc2 - tc1)/(s2 - s1)
c        su = tc2 - s2 * cw
        su = tc1 - s1 * cw
        write(*,"()")
        write(*,"('Processor ',i3)") k
        write(*,"('t1 = ',1pe15.6,' t2 = ',1pe15.6)") tc1,tc2
        write(*,"('BW = ',1pe15.6,' SU = ',1pe15.6)") 8.0/cw, su
        write(*,"()")
      endif
17    continue
c
c Turn off MPI
c
      call MPI_FINALIZE ( ierr )
      end
