#include "zeusmp.def"
c=======================================================================
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
      subroutine mstart
c
c  PURPOSE:  Starts a run.
c
c  EXTERNALS: SETUP, MGET, RESTART
c
c  LOCALS:
c
c  LAST MODIFIED: 7/20/01 by PSLi
c-----------------------------------------------------------------------
      implicit NONE
#include "mod_names.h"
#include "param.h"
#include "root.h"
#include "mpicom.h"
      integer irestart
c
      REAL t_out(nbuff-8)
      common / op_times / t_out
      integer incr,strtoi
#ifdef MPI_USED
#ifdef DEBUG
      integer i, k, pnamel, kcoords(3)
      character*(MPI_MAX_PROCESSOR_NAME) pname, pnames(0:63)
#endif /* DEBUG */
#endif /* MPI_USED */
c switched to new restart routines 7/08/04 jcv
      external setup, mget2, restart, strtoi
      namelist /mpitop/ ntiles, periodic
      namelist /rescon/ irestart,tdump,dtdump,id,resfile
      namelist /iocon/ thdf,dthdf,thist,dthist,tusr,dtusr,t_out
     .                ,ttsl,dttsl
#ifdef SHELLS
     .                ,tshl,dtshl
#endif /* SHELLS */
c added 7/19/04 jcv to save proper thdf after restart
      REAL oldthdf
c\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\///////////////////////////////
c=======================================================================
c
#ifdef DEBUG
      write(*,"(/'Process ',i2,' entered mstart.')") myid_w
#endif /* DEBUG */
c
c Open input and log files -- master thread only.
c
      if (myid_w .eq. 0) then
        open(unit=1,file='zmp_inp',status='old')
        open(unit=2,file='zmp_log',status='unknown')
      endif
CJH
c------------------------  MPI TOPOLOGY  -------------------------------c
c  ntiles:   elements equal the number of tiles in each direction.
c  periodic: elements are true if grid is periodic in that direction --
c            we check hydro BC flags for defaults, but can override.
c
      if (myid_w .eq. 0) then
        ntiles(1) = 1
        ntiles(2) = 1
        ntiles(3) = 1
        periodic(1) = .false.
        periodic(2) = .false.
        periodic(3) = .false.
        read (1,mpitop)
        write(2,mpitop)
      endif
c
#ifdef MPI_USED
c
c Tell the others what the master has read.
c
      call MPI_BCAST(ntiles  , 3,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(periodic, 3,MPI_LOGICAL,0,MPI_COMM_WORLD,ierr)
c
c Quit if the number of processors indicated on the command line
c differs from the number of tiles specified in the input file.
c
      if (nprocs_w .ne. ntiles(1)*ntiles(2)*ntiles(3)) then
        if (myid_w .eq. 0)
     1  write(*,"(/'MSTART: The number of threads ',i3,' does not match',
     2            /'MSTART: the number input via ntiles ',i3,' in',
     3            /'MSTART: input file zmp_inp; aborting the run...')")
     4    nprocs_w, ntiles(1)*ntiles(2)*ntiles(3)
        call MPI_FINALIZE( ierr )
        stop
      endif
c
c Create a virtual Cartesian topology for the domain decomposition.
c
      call MPI_CART_CREATE( MPI_COMM_WORLD, 3, ntiles, periodic
     &                    , reorder, comm3d, ierr )
      call MPI_COMM_RANK( comm3d, myid,     ierr )
      call MPI_COMM_SIZE( comm3d, nprocs,   ierr )
c
c Find the ranks of my neighbors; find my virtual Cartesian coords.
c
      call MPI_CART_SHIFT( comm3d, 0, 1, n1m, n1p, ierr )
      call MPI_CART_SHIFT( comm3d, 1, 1, n2m, n2p, ierr )
      call MPI_CART_SHIFT( comm3d, 2, 1, n3m, n3p, ierr )
c
      call MPI_CART_COORDS( comm3d, myid, 3, coords, ierr )
#ifdef DEBUG
c
c Identify this processor.
c
#ifdef DEC
      pname(1:MPI_MAX_PROCESSOR_NAME) = ' '
#else /* DEC */
      pname(1:MPI_MAX_PROCESSOR_NAME) = ""
#endif /* DEC */
      call MPI_GET_PROCESSOR_NAME( pname, pnamel, ierr )
c
c Send processor names to the root process.
c
      call MPI_GATHER( pname,MPI_MAX_PROCESSOR_NAME
     &,  MPI_CHARACTER
     &,               pnames,MPI_MAX_PROCESSOR_NAME
     &,  MPI_CHARACTER
     &,  0, comm3d, ierr)
      if (myid .eq. 0) then
        do 25 k=0,min(nprocs-1,63)
          kcoords(3) = k / ( ntiles(1) * ntiles(2) )
          i          = k - kcoords(3) * ( ntiles(1) * ntiles(2) )
          kcoords(2) = i / ntiles(1)
          kcoords(1) = i - kcoords(2) * ntiles(1)
          call MPI_CART_RANK( comm3d, kcoords, i, ierr )
          write(*,"(' Process at (',i2.2,',',i2.2,',',i2.2,') is ',
     1      'named ',a)") kcoords(1),kcoords(2),kcoords(3),
     2      pnames(i)(1:MPI_MAX_PROCESSOR_NAME)
25      continue
      endif
#endif /* DEBUG */
c
c      call MPI_BUFFER_ATTACH ( passit, npassit, ierr )
#endif /* MPI_USED */
c
c------------------------  RESTART CONTROL  ----------------------------
c
c  irestart: set to one for calculation restarted from restart dump
c  tdump: time of last restart dump
c  dtdump: time between restart dumps
c  id: character*2 tag attended to filenames to identify run
c  resfile: name of restart file to restart from
c
      if (myid_w .eq. 0) then
        irestart = 0
        tdump = 0.0
        dtdump = 0.0
        id     = 'aa'
        resfile= 'resaa000000.000'
        read (1,rescon)
        write(2,rescon)
      endif
#ifdef MPI_USED
c
c Tell the others what the master has read (do tdump, dtdump later). 
c
      call MPI_BCAST(irestart, 1,MPI_INTEGER  ,0,MPI_COMM_WORLD,ierr)
#ifndef T3E
      call MPI_BCAST(id      , 2,MPI_CHARACTER,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(resfile ,15,MPI_CHARACTER,0,MPI_COMM_WORLD,ierr)
#else /* T3E */
      id = 'aa'
      resfile = 'resaa000000.000'
#endif /* T3E */
#endif /* MPI_USED */
c
c Read the remaining namelists (except for iocon) in setup/restart.
c
      if (irestart .eq. 0) then
#ifdef DEBUG
        write(*,"(/'Process ',i2,' calling setup.')") myid_w
#endif /* DEBUG */
        call setup
        time = 0.0D0
      else
#ifdef MPI_USED
        incr = strtoi(resfile,13,15)
        write(resfile,"(a3,a2,3i2.2,'.',i3.3)") 'res',id,coords(1)
     &                                      ,coords(2),coords(3),incr
#endif /* MPI_USED */
c switched to new restart routines 7/08/04 jcv
        call mget2(resfile)
        nwarn = 0
        ifsen(1) = 0
c removed by jcv 7/12/04 not needed with new restart stuff
c        call restart
c        call nudt
      endif
c
c------------------------  I/O CONTROL ---------------------------------
c
c  thdf: time of last HDF dump
c  dthdf: time between HDF dumps
c  ttsl: time of last tslice dump
c  dttsl: time between tslice dumps
c  thist: time of last history dump
c  dthist: time between history dumps
c  tusr: time of last user dump
c  dtusr: time between user dumps
#ifdef SHELLS
c  tshl: time of last shells dump
c  dtshl: time between shells dumps
#endif /* SHELLS */
c
      if (myid_w .eq. 0) then
        if (irestart .eq. 0) then
           thdf  = 0.0
          dthdf  = 0.0
           ttsl  = 0.0
          dttsl  = 0.0
           thist = 0.0
          dthist = 0.0
           tusr  = 0.0
          dtusr  = 0.0
#ifdef SHELLS
           tshl  = 0.0
          dtshl  = 0.0
#endif /* SHELLS */
          do incr=1,nbuff-8
            t_out(incr) = 0.0
          enddo ! incr
        endif
c in case of restart, save the thdf read from file
        if(irestart.ne.0)then
           oldthdf = thdf
        endif
        read (1,iocon)
        write(2,iocon)
c restore correct thdf and set it back by one dthdf so it numbers correctly
        if(irestart.ne.0)then
           thdf = oldthdf - dthdf
        endif
      endif
#ifdef MPI_USED
      if (myid_w .eq. 0) then
        buf_in(1) = thdf
        buf_in(2) = dthdf
        buf_in(3) = thist
        buf_in(4) = dthist
        buf_in(5) = tusr
        buf_in(6) = dtusr
        buf_in(7) = tdump
        buf_in(8) = dtdump
#ifdef SHELLS
        buf_in(9) = tshl
        buf_in(10) = dtshl
#else /* SHELLS */
        buf_in(9) = 0.0
        buf_in(10) = 0.0
#endif /* SHELLS */
        buf_in(11) = ttsl
        buf_in(12) = dttsl
        do incr=13,nbuff
          buf_in(incr) = t_out(incr-12)
        enddo ! incr
      endif
c
c Tell the others what the master has read.  An array
c is used to pass just one message.
c
      call MPI_BCAST(buf_in,nbuff,MPI_FLOAT,0,MPI_COMM_WORLD
     &              ,ierr)
      if (myid_w .ne. 0) then
          thdf  = buf_in(1)
         dthdf  = buf_in(2)
          thist = buf_in(3)
         dthist = buf_in(4)
          tusr  = buf_in(5)
         dtusr  = buf_in(6)
         tdump  = buf_in(7)
        dtdump  = buf_in(8)
#ifdef SHELLS
          tshl  = buf_in(9)
         dtshl  = buf_in(10)
#endif /* SHELLS */
        ttsl   = buf_in(11)
        dttsl  = buf_in(12)
        do incr=1,nbuff-12
          t_out(incr) = buf_in(incr+12)
        enddo ! incr
      endif
#endif
c
c Output file names are of the form "hdfidccc.n" for ease of
c use with graphics packages that can process a series of files.
c
      if (irestart .eq. 0) then
        incr = 0
        ifsen(2) = 1
        ifsen(3) = 1
        ifsen(4) = 1
        ifsen(5) = 1
        ifsen(6) = 1

        write(tslfile,"(a3,i3.3,a2)") 'tsl',incr,id

        write(resfile,"(a3,a2,3i2.2,'.',i3.3)") 'res',id,coords(1)
     &                                      ,coords(2),coords(3),incr
        write(hdffile,"(a3,a2,3i2.2,'.',i3.3)") 'hdf',id,coords(1)
     &                                      ,coords(2),coords(3),incr
        write(hstfile,"(a3,a2,3i2.2,'.',i3.3)") 'hst',id,coords(1)
     &                                      ,coords(2),coords(3),incr
        write(usrfile,"(a3,a2,3i2.2,'.',i3.3)") 'usr',id,coords(1)
     &                                      ,coords(2),coords(3),incr

#ifdef SHELLS
        ifsen(7) = 1
        write(shlfile,"(a3,a2,3i2.2,'.',i3.3)") 'shl',id,coords(1)
     &                                      ,coords(2),coords(3),incr
#endif /* SHELLS */
      else
        ifsen(2) = 0
        ifsen(3) = 0
        ifsen(4) = 0
        ifsen(5) = 0
        ifsen(6) = 0

        incr = strtoi(tslfile,4,6) + 1
        write(tslfile,"(a3,i3.3,a2)") 'tsl',incr,id 

        incr = strtoi(resfile,13,15)
        write(resfile,"(a3,a2,3i2.2,'.',i3.3)") 'res',id,coords(1)
     &                                      ,coords(2),coords(3),incr
        incr = strtoi(hdffile,13,15)
        write(hdffile,"(a3,a2,3i2.2,'.',i3.3)") 'hdf',id,coords(1)
     &                                      ,coords(2),coords(3),incr
        incr = strtoi(hstfile,13,15)
        write(hstfile,"(a3,a2,3i2.2,'.',i3.3)") 'hst',id,coords(1)
     &                                      ,coords(2),coords(3),incr
        incr = strtoi(usrfile,13,15)
        write(usrfile,"(a3,a2,3i2.2,'.',i3.3)") 'usr',id,coords(1)
     &                                      ,coords(2),coords(3),incr
#ifdef SHELLS
        ifsen(7) = 0
        incr = strtoi(shlfile,13,15)
        write(usrfile,"(a3,a2,3i2.2,'.',i3.3)") 'shl',id,coords(1)
     &                                      ,coords(2),coords(3),incr
#endif /* SHELLS */
      endif
#ifdef HST_DUMPS
c
c Open history dump file.
c
       open (unit=3,file=hstfile,status='unknown')
#endif /* HST_DUMPS */
#ifdef TSL_DUMPS
       if (myid_w .eq. 0) then
         open (unit=31,file=tslfile,status='unknown')
       endif
#endif /* TSL_DUMPS */ 
#ifdef SHELLS
c
c Open shells dump file.
c
       open (unit=30,file=shlfile,status='unknown')
#endif /* SHELLS */
      if (incr+1 .le. nbuff-10) then
c
c Adjust dump times to hit the next t_out, if defined non-zero.
c
        if (t_out(incr+1) .gt. 0.0) then
          tusr = t_out(incr+1) - dtusr
          thdf = t_out(incr+1) - dthdf
        endif
      endif
c
c  Close unit=1 (input deck).  Unit=2 (zeuslp) is kept open throughout
c  entire run to accept warning messages.  It is closed in zeusmp.
c  1/17/2007 jcv Check myid before closing (from zeusmpv2)
c
      if (myid_w .eq. 0 )then
         close(unit=1)
      endif
c
c Write out initial data dumps.
c
      if(irestart .eq. 0) then
c      call dataio( iswres, iswhdf, iswhst, iswusr
#ifdef SHELLS
c    .              iswshl
#endif /* SHELLS */
c    .            )
       call dataio( ifsen(2), ifsen(3), ifsen(4), ifsen(5), ifsen(6)
#ifdef SHELLS
     .             ,ifsen(7)
#endif /* SHELLS */
     .            )
       nhy = 0
      else
       nhy = 0
      endif
c
      return
      end
