#include "zeusmp.def"
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                 B L O C K                 \\\\\\\\\\
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
c=======================================================================
c
       subroutine block
c
c Sets up radiation diffusion problem like Frank's.
c
c VARIABLES IN PGEN NAMELIST
c    d0          background density
c    temp0       background temperature or radiation energy density
c    x10, x11    beginning/end of block in 1-direction
c    x20, x21    beginning/end of block in 2-direction
c    x30, x31    beginning/end of block in 3-direction
c
c    The value of temp0 is raised to the 4th power to convert to
c    radiation energy density if xnu and powr are non-zero.
c
c RAF, 2/18/97
c-----------------------------------------------------------------------
c
       implicit NONE
#include "param.h"
#include "field.h"
#include "bndry.h"
#include "grid.h"
#include "root.h"
#include "scratch.h"
#include "radexp.h"
#include "mod_names.h"
#include "cons.h"
#include "mpicom.h"
c
       integer       i       , j       , k       
     1             , i1      , i2
     1             , j1      , j2
     1             , k1      , k2
       REAL          d0      , d1      , temp0
     1             , x10     , x20     , x30
     2             , x11     , x21     , x31
c
       REAL          e0
     1             , v10     , v20     , v30
c
       REAL          dmc_max , t_max, dx_min
c
       namelist / pgen     /
     1               d0      , d1      , temp0
     2             , x10     , x20     , x30
     3             , x11     , x21     , x31
c
c-----------------------------------------------------------------------
c
       d0   = 1.0
       d1   = 1.0
       temp0= 1.0
       x10  = 0.0
       x20  = 0.0
       x30  = 0.0
       x11  = 0.0
       x21  = 0.0
       x31  = 0.0
c
       if (myid .eq. 0) then
         read (1, pgen)
         write (2, pgen)
#ifdef MPI_USED
         buf_in( 1) = d0   
         buf_in( 2) = d1
         buf_in( 3) = temp0
         buf_in( 4) = x10 
         buf_in( 5) = x20 
         buf_in( 6) = x30 
         buf_in( 7) = x11 
         buf_in( 8) = x21 
         buf_in( 9) = x31 
       endif
       call MPI_BCAST( buf_in, 9, MPI_FLOAT
     &               , 0, comm3d, ierr )
       if (myid .ne. 0) then
         d0   = buf_in( 1)
         d1   = buf_in( 2)
         temp0= buf_in( 3)
         x10  = buf_in( 4)
         x20  = buf_in( 5)
         x30  = buf_in( 6)
         x11  = buf_in( 7)
         x21  = buf_in( 8)
         x31  = buf_in( 9)
#endif /* MPI_USED */
       endif
c
c      Set up field variables.
c
       v10 = 0.
       v20 = 0.
       v30 = 0.
       e0  = 1.e3 * (0.0001) * everg/(mh*(gamma-1)) ! 0.
       do 30 k=ks,ke
         do 20 j=js,je
           do 10 i=is,ie
             if ((x1a(i).ge.x10 .and. x1a(i).lt.x11) .and.
     1           (x2a(j).ge.x20 .and. x2a(j).lt.x21) .and.
     2           (x3a(k).ge.x30 .and. x3a(k).lt.x31)) then
c
c Inside the block
c
               d (i,j,k) = d1
             else
c
c Outside the block
c
               d (i,j,k) = d0
             endif
             if (xnu.eq.0.0 .and. powr.eq.0.0) then
c
c er is the temperature -- linear diffusion problem.
c
               er(i,j,k) = temp0
             else
c
c er is the radiation energy denisty rad_con * T**4, where 
c "rad_con" is the Radiation constant in erg cm^-3 K^-4
c and T is in K.  
c
c It would be nice to input "temperature" in keV and convert to K,
c but for now just input erg/cm^3.
c
c               er(i,j,k) = rad_con * (temp0*1000.0*everg/boltz)**4
               er(i,j,k) = temp0**4
             endif
             v1(i,j,k) = v10
             v2(i,j,k) = v20
             v3(i,j,k) = v30
#ifndef ISO
cjh          e (i,j,k) = e0
             e (i,j,k) = e0 * d(i,j,k)
#endif
#ifdef MHD
             b1(i,j,k) = b10
             b2(i,j,k) = b20
             b3(i,j,k) = b30
#endif
10         continue
20       continue
30     continue
c
c One more thing -- compute the initial time step from the Courant
c number specified in hycon.  If that is zero, use the default.
c
       if (rcourno.ne.0.0) then
         if (xnu.eq.0.0 .and. powr.eq.0.0) then
           dmc_max = rmfp0 * clight
         else
c
c Assume the maximum temperature lies along one of the boundaries.
c
           t_max   = max(fiis(12),fois(12)
     1                  ,fijs(12),fojs(12)
     2                  ,fiks(12),foks(12))
           dmc_max = clight * rmfp0 * (rho0/d0)**xnu
     1             * (t_max/t_0)**(powr/4.0) / 3.0
         endif
c
c Assume uniform grid in each direction.
c
         if (t_max.eq.fiis(12) .or. t_max.eq.fois(12)) then
           dx_min = dx1a(is+1)
         else if (t_max.eq.fijs(12) .or. t_max.eq.fojs(12)) then
           dx_min = dx2a(js+1)
         else
           dx_min = dx3a(ks+1)
         endif
c
c         if (dmc_max .gt. 0.) dt = courno * dx_min**2 / dmc_max
         if (dmc_max .gt. 0.) dtrdi2 = (dmc_max/rcourno*dx_min**2)**2
       endif
c
       return
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////                 B L O C K                 \\\\\\\\\\
c
c=======================================================================
c
