#include "zeusmp.def"
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                   P D V                   \\\\\\\\\\
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
c=======================================================================
c
       subroutine pdv 
#ifndef ISO
     &                (ibeg,iend,jbeg,jend,kbeg,kend,dlo,eod)
c
c Updates the energy with the PdV compressional work term.  Also
c computes the specific internal energy density eod for the transport
c step.
c
c BOUNDARY VALUES USED:
c
c  Macro defined  var   ii    oi    ij    oj    ik    ok
c  -------------  ---  ----  ----  ----  ----  ----  ----
c                  u1        ie+1
c                  u2                    je+1
c                  u3                                ke+1
c
c OUTPUT
c    dlo    Mass density (copy)                      for transport step.
c    eod    Specific energy density e/d [or (e+p)/d] for transport step.
c
c Written by RAF 2/12/96.
c......................................................................
       implicit NONE
#include "param.h"
#include "root.h"
#include "grid.h"
#include "field.h"
       integer i,j,k,ibeg,iend,jbeg,jend,kbeg,kend
       REAL dlo(in,jn,kn),eod(in,jn,kn)
#ifndef TOTAL_ENERGY
       REAL q1,q2
       q1 = 0.5 * dt * gamm1
#endif /* not TOTAL_ENERGY */
#ifndef UNROLL_I
#define UNROLL_I
#endif
c
       do 30 k=kbeg,kend
         do 20 j=jbeg,jend
CDIR$ UNROLL (UNROLL_I)
           do 10 i=ibeg,iend
#ifndef TOTAL_ENERGY
c
c      Compute divergence of velocity field and add pdv source term 
c      to energy density.
c
             q2         = ( g2a(i+1) * g31a(i+1) * v1(i+1,j,k)
     1                    - g2a(i  ) * g31a(i  ) * v1(i  ,j,k) )
     2                  *                         dvl1ai(i)
     3                  + ( g32a(j+1) * v2(i,j+1,k)
     4                    - g32a(j  ) * v2(i,j  ,k) )
     5                  *   g2bi(i)             * dvl2ai(j)
     6                  + ( v3(i,j,k+1) - v3(i,j,k) )
     7                  *   g31bi(i) * g32bi(j) * dvl3ai(k)
             q2         = q2 * q1
             e(i,j,k)   = ( 1.0 - q2 ) / ( 1.0 + q2 ) * e(i,j,k)
#endif /* not TOTAL_ENERGY */
c
c      Compute the specific internal energy density and save a copy of
c      the mass denisity for the transport step.
c
             dlo(i,j,k) = d(i,j,k)
#ifndef TOTAL_ENERGY
             eod(i,j,k) = e(i,j,k) / d(i,j,k)
#else /* not TOTAL_ENERGY */
             eod(i,j,k) = gamma * e(i,j,k) / d(i,j,k)
     1                  - gamm1 * ( ( v1(i,j,k) + v1(i+1,j  ,k  ) )**2
     2                            + ( v2(i,j,k) + v2(i  ,j+1,k  ) )**2
     3                            + ( v3(i,j,k) + v3(i  ,j  ,k+1) )**2 )
     4                          * 0.125
#if defined GRAV || defined GRAV_FFT
     5                  - gamm1 * gp(i,j,k)
#endif /* GRAV || GRAV_FFT */
#endif /* not TOTAL_ENERGY */
10         continue
20       continue
30     continue
c
#endif /* not ISO */
       return
       end
