#include "zeusmp.def"
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                 P D V _ D                 \\\\\\\\\\
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
c=======================================================================
c
       subroutine pdv_d
#if defined RAD || defined RAD_EXP
#else /* RAD || RAD_EXP */
#ifndef ISO
     1            (dcopy, eod)
c
c  PURPOSE
c    Compute the compressional work term (PdV) in the material
c    energy equation (for no radiation).
c
c  AUTHOR
c    Robert A. Fiedler
c
c  LAST MODIFIED
c    01/21/97
c
c  INPUT
c    (none)
c
c  OUTPUT
c    dcopy     A copy of the density.
c    eod       e/d for the transport step.
c
c  We need just 1 "p" layer of updated boundary data for v1, v2, and v3,
c  but none for d and e.  
c
c  Routine pdv also saves the density and e/d in (dcopy,eod) for use
c  in the transport step.  This is why it is being called even when
c  TOTAL_ENERGY is defined.
c
c  EXTERNALS:
c    BVALV1  , BVALV2  , BVALV3
c    PDV
c.......................................................................
c
       implicit NONE
#include "mod_names.h"
#include "param.h"
#include "root.h"
#include "field.h"
#include "bndry.h"
#include "grid.h"
#include "mpicom.h"
#ifdef PROFILING
#ifdef MPE_USED
#else
#include "clocks.h"
#endif /* MPE_USED */
#endif /* PROFILING */
c
       REAL dcopy(in,jn,kn), eod(in,jn,kn)
c
       integer i, k1, k2, imax, jmax, kmax
       integer j, k
c
c      External statements
c
       external      pdv
     1             , bvalv1  , bvalv2  , bvalv3
c
c-----------------------------------------------------------------------
c
c Divide the computational volume into three equal pieces.  We must
c have at least 5 active zones in the 3-direction.
c
       k1 = int( real( ke - ks + 1 ) / 3.0 ) + ks
       k2 = int( real( ke - ks + 1 ) / 3.0 ) + k1
c
c i boundaries
c
c    1) Post sends and receives.
c
#ifdef DEBUG
       write(*,"(/'PDV_D: ',i2,' calling pdv.')") myid
#endif /* DEBUG */
       nreq = 0
       nsub = nsub + 1
       call bvalv1 (0,1,0,0,0,0,v1)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(nsubstps,1,2,2) = timers(nsubstps,1,2,2) + wal1  - wal0
       timers(nsubstps,1,2,1) = timers(nsubstps,1,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
c    2) Do first portion of the interior points.
c
c      Copy density boundary values.
c
       do k=ks-2,ks-1
         do j=js-2,je+2
           do i=is-2,ie+2
             dcopy(i,j,k) = d(i,j,k)
           enddo ! i
         enddo ! j
       enddo ! k
c
       call pdv (is,ie-1,js,je-1,ks  ,k1
     &          ,dcopy ,eod )
c
c       subroutine pdv (ibeg,iend,jbeg,jend,kbeg,kend
c     &                ,dlo,eod)
cc
cc    dlo   Mass density (copy)                      for transport step.
cc    eod   Specific energy density e/d (or (e+p)/d] for transport step.
c
c
c      Copy density boundary values.
c
       do k=ks,ke
         do j=js-2,js-1
           do i=is-2,ie+2
             dcopy(i,j,k) = d(i,j,k)
           enddo ! i
         enddo ! j
       enddo ! k

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(3,1,2,2) = timers(3,1,2,2) + wal1  - wal0
       timers(3,1,2,1) = timers(3,1,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#ifdef MPI_USED
c
c    3) Wait for communications to complete.
c
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(3,1,1,2) = timers(3,1,1,2) + wal1  - wal0
       timers(3,1,1,1) = timers(3,1,1,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#endif
c......................................................................
c
c j boundaries
c
c    1) Post sends and receives.
c
       nreq = 0
       nsub = nsub + 1
       call bvalv2 (0,0,0,1,0,0,v2)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(nsubstps,2,2,2) = timers(nsubstps,2,2,2) + wal1  - wal0
       timers(nsubstps,2,2,1) = timers(nsubstps,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
c    2) Do middle 1/3 of the interior points, and some on borders.
c
       call pdv (ie  ,ie  ,js,je-1,ks  ,k1
     &          ,dcopy ,eod )
       call pdv (is  ,ie  ,js,je-1,k1+1,k2
     &          ,dcopy ,eod )
c
c      Copy density boundary values.
c
       do k=ks,ke
         do j=je+1,je+2
           do i=is-2,ie+2
             dcopy(i,j,k) = d(i,j,k)
           enddo ! i
         enddo ! j
       enddo ! k
       do k=ks,ke
         do j=js,je
           do i=is-2,is-1
             dcopy(i,j,k) = d(i,j,k)
           enddo ! i
         enddo ! j
       enddo ! k

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(3,2,2,2) = timers(3,2,2,2) + wal1  - wal0
       timers(3,2,2,1) = timers(3,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#ifdef MPI_USED
c
c    3) Wait for communications to complete.
c
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(3,2,1,2) = timers(3,2,1,2) + wal1  - wal0
       timers(3,2,1,1) = timers(3,2,1,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#endif
c......................................................................
c
c k boundaries
c
c    1) Post sends and receives.
c
       nreq = 0
       nsub = nsub + 1
       call bvalv3 (0,0,0,0,0,1,v3)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(nsubstps,3,2,2) = timers(nsubstps,3,2,2) + wal1  - wal0
       timers(nsubstps,3,2,1) = timers(nsubstps,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
c    2) Do last 1/3 of the interior points, and some on borders.
c
       call pdv (is  ,ie  ,je  ,je  ,ks  ,k2
     &          ,dcopy ,eod )
       call pdv (is  ,ie  ,js  ,je  ,k2+1,ke-1   
     &          ,dcopy ,eod )
c
c      Mark the boundary values out of date.
c
       do 70 i = 1,6
         bvstat(i,2) = 0      !  e
70     continue
c
c      Copy density boundary values.
c
       do k=ke+1,ke+2
         do j=js-2,je+2
           do i=is-2,ie+2
             dcopy(i,j,k) = d(i,j,k)
           enddo ! i
         enddo ! j
       enddo ! k
       do k=ks,ke
         do j=js,je
           do i=ie+1,ie+2
             dcopy(i,j,k) = d(i,j,k)
           enddo ! i
         enddo ! j
       enddo ! k

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(3,3,2,2) = timers(3,3,2,2) + wal1  - wal0
       timers(3,3,2,1) = timers(3,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#ifdef MPI_USED
c
c    3) Wait for communications to complete. 
c
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(3,3,1,2) = timers(3,3,1,2) + wal1  - wal0
       timers(3,3,1,1) = timers(3,3,1,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#endif
c......................................................................
c
c Finally, do the remaining border.
c
       call pdv (is  ,ie  ,js  ,je  ,ke  ,ke
     &          ,dcopy ,eod )

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(3,4,2,2) = timers(3,4,2,2) + wal1  - wal0
       timers(3,4,2,1) = timers(3,4,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#endif /* not ISO */
#endif /* RAD || RAD_EXP */
       return
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////                 P D V _ D                 \\\\\\\\\\
c
c=======================================================================
c
