#include "zeusmp.def"
c=======================================================================
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
      subroutine printd
c
c  FORMATTED WRITE OF SELECTED VARIABLES
c
c     written by: Jim Stone
c     date:       February,1993
c     modified1:  DAC from ZEUS-3D
c     modified2:  RAF 1/12/96 for ZEUS-MP
c     modified3:  RAF 3/12/97, for ablation test problem
c     modified4:  JCH 4/17/97, cons.h included even when RAD_EXP
c                 not defined
c     modified5:  M-MML 5/18/98, expanded to include the full ZEUS-3D 
c         suite, including max and min value, as well as things like 
c         rms velocity, and to run under MPI
c
c  PURPOSE:  Dumps scalar "history" variables in a formatted write
c  for analysis.  Currently implemented variables are:
c
c   scal( 1) = time
c   scal( 2) = dt
c   scal( 3) = mass
c   scal( 4) = total energy
c   scal( 5) = kinetic energy
c   scal( 6) = magnetic energy
c   scal( 7) = internal energy
c   scal( 8) = radiation energy
c   scal( 9) = gravitational potential energy
c   scal(10) = 0.5*d*v1**2
c   scal(11) = 0.5*d*v2**2
c   scal(12) = 0.5*d*v3**2
c   scal(13) = b1**2
c   scal(14) = b2**2
c   scal(15) = b3**2
c   scal(16) = b1 flux
c   scal(17) = b2 flux
c   scal(18) = b3 flux
c   scal(19) = 1 angular momentum
c   scal(20) = 2 angular momentum
c   scal(21) = 3 angular momentum
c   scal(22) = 1 center of mass
c   scal(23) = 2 center of mass
c   scal(24) = 3 center of mass
c
c
c   scal(25) =    dmin   - minimum density
c   scal(26) =    dmax   - maximum density
c   scal(27) =    emin   - minimum specific energy
c   scal(28) =    emax   - maximum specific energy
c   scal(29) =    pmin   - minimum pressure
c   scal(30) =    pmax   - maximum pressure
c   scal(31) =    v1min  - minimum velocity in 1-direction
c   scal(32) =    v1max  - maximum velocity in 1-direction
c   scal(33) =    v2min  - minimum velocity in 2-direction
c   scal(34) =    v2max  - maximum velocity in 2-direction
c   scal(35) =    v3min  - minimum velocity in 3-direction
c   scal(36) =    v3max  - maximum velocity in 3-direction
c   scal(37) =    vmin   - minimum speed (magnitude of velocity vector)
c   scal(38) =    vmax   - maximum speed (magnitude of velocity vector)
c   scal(39) =    b1min  - minimum mag. field in 1-direction
c   scal(40) =    b1max  - maximum mag. field in 1-direction
c   scal(41) =    b2min  - minimum mag. field in 2-direction
c   scal(42) =    b2max  - maximum mag. field in 2-direction
c   scal(43) =    b3min  - minimum mag. field in 3-direction
c   scal(44) =    b3max  - maximum mag. field in 3-direction
c   scal(45) =    bmin   - minimum mag. field strength (magnitude of B-vector)
c   scal(46) =    bmax   - maximum mag. field strength (magnitude of B-vector)
c   scal(47) =    dvbmin - minimum div(B) (normalised)
c   scal(48) =    dvbmax - maximum div(B) (normalised)
c   scal(49) =    dtcs   - sound speed time stop
c      dt**   - timestep limit caused by ** (eg., dtcs, dtqq, etc.)
c  LOCALS:
c-----------------------------------------------------------------------
      implicit NONE
#include "param.h"
#include "grid.h"
#include "field.h"
#include "root.h"
#include "gravity.h"
#include "cons.h"
#if defined RAD || defined RAD_EXP
#include "radexp.h"
#endif /* RAD || RAD_EXP */
#include "mpicom.h"
      integer i,j,k,nscal
      integer id_ctr
      parameter(nscal=24)
      REAL scal(nscal)
      REAL mass,x1cm,x2cm,x3cm,eint,ek1,ek2,ek3,epot
     .   ,emag1,emag2,emag3,b1flx,b2flx,b3flx
     .   ,l1,l2,l3,erad,dvb
       REAL        dmin    , dmax    , semin   , semax   , pmin
     1             , pmax    , v1min   , v1max   , v2min   , v2max
     2             , v3min   , v3max   , vsqrmin , vsqrmax , vmin
     3             , vmax    , b1min   , b1max   , b2min   , b2max
     4             , b3min   , b3max   , bsqrmin , bsqrmax , bmin
     5             , bmax    , dvbmin  , dvbmax
       integer       imin    (  12), jmin    (  12), kmin    (  12)
     1             , imax    (  12), jmax    (  12), kmax    (  12)
      REAL rv,dvola,dm,g3b,q1,q2,q3,dar1,dar2,dar3
     .   , v1av,v2av,v3av,b1av,b2av,b3av
c\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\///////////////////////////////
c=======================================================================
c
c
c  Integrate quantities
c
c      Zero totals.
c
       id_ctr = 99999
       mass  = tiny
       x1cm  = 0.0
       x2cm  = 0.0
       x3cm  = 0.0
       eint  = 0.0
       ek1   = 0.0
       ek2   = 0.0
       ek3   = 0.0
       epot  = 0.0
       emag1 = 0.0
       emag2 = 0.0
       emag3 = 0.0
       b1flx = 0.0
       b2flx = 0.0
       b3flx = 0.0
       l1    = 0.0
       l2    = 0.0
       l3    = 0.0
       erad  = 0.0
c
c      Compute totals over k-sweeps
c
       do 40 k=ks,ke
         do 30 j=js,je
           do 20 i=is,ie
             dvola     = dvl1a(i) * dvl2a(j) * dvl3a(k)
             dm        = d(i,j,k) * dvola
             g3b       = g31b(i) * g32b(j)
             q1        = x1b(i)
             q2        = x2b(j) * g2b(i)
             q3        = x3b(k) * g3b
             dar1      = g2a(i) * dx2b(j) * g3b    * dx3b(k)
             dar2      = g3b    * dx3b(k)          * dx1b(i)
             dar3      =          dx1b(i) * g2a(i) * dx2b(j)
             v1av      = v1(i,j,k) + v1(i+1,j  ,k  )
             v2av      = v2(i,j,k) + v2(i  ,j+1,k  )
             v3av      = v3(i,j,k) + v3(i  ,j  ,k+1)
#ifdef MHD
             b1av      = b1(i,j,k) + b1(i+1,j  ,k  )
             b2av      = b2(i,j,k) + b2(i  ,j+1,k  )
             b3av      = b3(i,j,k) + b3(i  ,j  ,k+1)
#else
             b1av      = 0.0
             b2av      = 0.0
             b3av      = 0.0
#endif
             mass      = mass      + dm
             x1cm      = x1cm      + dm * q1
             x2cm      = x2cm      + dm * q2
             x3cm      = x3cm      + dm * q3
             ek1       = ek1       + dm * v1av**2
             ek2       = ek2       + dm * v2av**2
             ek3       = ek3       + dm * v3av**2
#ifndef ISO
CJH
#ifndef TOTAL_ENERGY
             eint      = eint      + e(i,j,k) * dvola
#else /* NOT TOTAL_ENERGY */
             eint      = eint      + dvola *
     .                   ( e(i,j,k) - 0.125 * d(i,j,k) *
     .                                (v1av**2 + v2av**2 + v3av**2) )
#endif /* NOT TOTAL_ENERGY */
#endif /* NOT ISO */
#if defined RAD || defined RAD_EXP
             erad      = erad      + er(i,j,k) * dvola
#endif /* RAD || RAD_EXP */
#if defined GRAV || defined GRAV_FFT
             epot      = epot      - dm * gp(i,j,k)
#endif /* GRAV || GRAV_FFT */
#ifdef XYZ
             rv        = sqrt ( ( x1b(i) - x1ptm )**2
     1                        + ( x2b(j) - x2ptm )**2
     2                        + ( x3b(k) - x3ptm )**2 )
#endif
#ifdef ZRP
             rv        = sqrt ( ( x1b(i) - x1ptm )**2
     1                        + ( x2b(j) - x2ptm )**2
     2                        + 2.0 * x2b(j) * x2ptm
     3                        * ( 1.0 - cos(x3b(k) - x3ptm) ) )
#endif
#ifdef RTP
             rv        = ( x1b(i)*cos(x2b(j)) - x1ptm*cos(x2ptm) )**2
     1                 + ( x1b(i)*sin(x2b(j)) - x1ptm*sin(x2ptm) )**2
     2                 + 2.0 * x1b(i)*sin(x2b(j)) * x1ptm*sin(x2ptm)
     3                 * ( 1.0 - cos(x3b(k) - x3ptm) )
             rv        = sqrt ( rv )
#endif
             epot      = epot      - dm * g * ptmass / rv
#ifdef MHD
             emag1     = emag1     + b1av**2 * dvola
             emag2     = emag2     + b2av**2 * dvola
             emag3     = emag3     + b3av**2 * dvola
             b1flx     = b1flx     + b1(i,j,k) * dar1
             b2flx     = b2flx     + b2(i,j,k) * dar2
             b3flx     = b3flx     + b3(i,j,k) * dar3
#endif
             l1        = l1        + dm * ( q2 * v3av - q3 * v2av )
             l2        = l2        + dm * ( q3 * v1av - q1 * v3av )
             l3        = l3        + dm * ( q1 * v2av - q2 * v1av )

#if defined RAD_EXP || defined RAD
             if (  abs(x1a(i)-zro).lt.0.25*dx1a(i) 
     1       .and. abs(x2a(j)-zro).lt.0.25*dx2a(j)
     2       .and. abs(x3a(k)-zro).lt.0.25*dx3a(k)) then
               scal(19) = d (i,j,k)
               scal(20) = e (i,j,k)
               scal(21) = er(i,j,k)
               id_ctr = myid
             endif
#endif /* RAD_EXP || RAD */
c
20         continue
30       continue
40     continue
c
       scal(1)  = time
       scal(2)  = dt
       scal(3)  = mass
c
       scal(7)  = eint
       scal(8)  = erad
       scal(9)  = epot
       scal(10) = ek1   / 8.0
       scal(11) = ek2   / 8.0
       scal(12) = ek3   / 8.0
       scal(13) = emag1 / 8.0
       scal(14) = emag2 / 8.0
       scal(15) = emag3 / 8.0
       scal(16) = b1flx
       scal(17) = b2flx
       scal(18) = b3flx
c
#if defined RAD_EXP || defined RAD
c......................................................................
c
c Special output for ablation problem:
c
      if(myid .eq. id_ctr) then
       scal(22) = gamm1 * scal(20) * 1.0e-12   !  pressure in Megabars
       scal(23) = scal(20) * (gamm1*mmw*mh) / (scal(19) * everg) ! T_gas
       scal(24) = sqrt(sqrt(scal(21)/rad_con)) * boltz/everg     ! T_rad
      endif
c
c......................................................................
c
#else /* RAD_EXP || RAD */
c
       scal(19) = l1    / 2.0
       scal(20) = l2    / 2.0
       scal(21) = l3    / 2.0
c
       scal(22) = x1cm  / mass
       scal(23) = x2cm  / mass
       scal(24) = x3cm  / mass
c
#endif /* RAD_EXP || RAD */
c
c Total up various forms of energy.
c
       scal(5)  = scal(10) + scal(11) + scal(12)             ! kinetic
       scal(6)  = scal(13) + scal(14) + scal(15)             ! magnetic
       scal(4)  = scal( 5) + scal( 6) + scal( 7) + scal( 8) + scal( 9)
c
c  Write out variables to file connected to unit 3 opened in MAIN
c  program unit (zeusmp.src)
c
#if defined RAD_EXP || defined RAD
      if (myid.eq.id_ctr) write(3,2001) (scal(i),i=1,nscal)
#else /* RAD_EXP || RAD */
      write(3,2001) (scal(i),i=1,nscal)
#endif /* RAD_EXP || RAD */
2001  format(1p6e13.5/1p6e13.5/1p6e13.5/1p6e13.5/)
c
      return
      end
