#include "zeusmp.def"
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////           R A D _ I M P _ D T             \\\\\\\\\\
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
c=======================================================================
c
       subroutine rad_imp_dt(erold,ernew) 
c      subroutine rad_imp_dt(egold,egnew,erold,ernew) 
#ifdef RAD
c
c  INPUT
c    erold   radiation energy density at old time step
c    ernew   updated radiation energy density
c
c  OUTPUT
c    dtimrdi  proposed new timestep
c
c  EXTERNALS:
c
c  AUTHOR
c    Robert Fiedler and John Hayes
c
c  LAST MODIFIED
c    4 June 1997
c-----------------------------------------------------------------------
c
       implicit NONE
#include "param.h"
#include "field.h"
#include "grid.h"
#include "root.h"
#include "radexp.h"
#include "radiation.h"
#include "scratch.h"
#include "cons.h"
#include "mpicom.h"
#include "foo.h"

#ifdef PROFILING
#ifdef MPE_USED
#else
#include "clocks.h"
#endif /* MPE_USED */
#endif /* PROFILING */

c
       integer       i       ,  j      , k
     1             , imax    ,  jmax   , kmax
     &             , iamax   ,  jamax  , kamax
#ifdef PROFILING
#ifdef MPE_USED
#else
     &             , nt
#endif /* MPE_USED */
#endif /* PROFILING */
c
       REAL        erold(in,jn,kn) , ernew(in,jn,kn)
       REAL        egold(in,jn,kn) , egnew(in,jn,kn)
       REAL        dtec, er_max, ader, ader_max, fac_dt,
     .                   eg_max, adeg, adeg_max, ade_max
c
       REAL        delta_t_max
       REAL        inp(2), out(2)
       REAL        trnew, trold
c
c      External statements
c
c      DATA
c
       data fac_dt /1.26/
c-----------------------------------------------------------------------
c
c Find the minimum time step required by the Courant condition for
c this tile.
c
#ifdef DEBUG
       write(*,"(/'RAD_IMP_DT: Process ',i2,' entering.')") myid
#endif /* DEBUG */

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       nt = nsubstps - 1  !  New dt always next to last substep.
       call clocks (cputime, wclock)
       wal1 = real(wclock)
       cpu1 = real(cputime)
#endif /* MPE_USED */
#endif /* PROFILING */

c
c-----------------------------------------------------------------------
c
c Find max new rad energy and max abs rad energy change for this tile.
c
       iamax = is
       jamax = js
       kamax = ks
       ader_max = abs((ernew(is,js,ks)-erold(is,js,ks))/
     .                 (erold(is,js,ks)+tiny))
       do k=ks,ke
         do j=js,je
           do i=is,ie
             ader = abs((ernew(i,j,k) - erold(i,j,k))/
     .                   (erold(i,j,k)+tiny))
             if (ader .gt. ader_max) then
               ader_max = ader
               iamax = i
               jamax = j
               kamax = k
             endif
           enddo ! i
         enddo ! j
       enddo ! k

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(nt,4,2,2) = timers(nt,4,2,2) + wal1  - wal0
       timers(nt,4,2,1) = timers(nt,4,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#ifdef MPI_USED
c
c Now find the largest e_max and ade_max among all tiles, and send the 
c results to all in buf_out.
c
       buf_in(1) = ader_max
       buf_in(2) = float(myid)
       call MPI_ALLREDUCE(buf_in, buf_out, 1, MPI_2DOUBLE_PRECISION,
     .                    MPI_MAXLOC, comm3d, ierr)
       if(.false.) then
        if(mod(nhy,100) .eq. 0) then
         if(myid .eq. int(buf_out(2))) then
           write(*,"('RAD_IMP_DT: max dE/E = ',1pd12.4,' at i,j,k =',
     .               3i3,' on node ',i2)")
     .           buf_out(1),iamax,jamax,kamax,int(buf_out(2))
           write(*,"('erold, ernew = ',1p2d12.4)")
     .         erold(iamax,jamax,kamax),
     .         ernew(iamax,jamax,kamax)
         endif
        endif
       endif
       ader_max = buf_out(1)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(nt,4,1,2) = timers(nt,4,1,2) + wal1  - wal0
       timers(nt,4,1,1) = timers(nt,4,1,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#else /* MPI_USED */
       if(.false.) then
        if(mod(nhy,10) .eq. 0) then
           write(*,"('RAD_IMP_DT: max dE/E = ',1pd12.4,' at i,j,k =',
     .               3i3)")
     .           ader_max,iamax,jamax,kamax
         write(*,"('erold, ernew = ',1p2d12.4)")
     .         erold(iamax,jamax,kamax),
     .         ernew(iamax,jamax,kamax)
        endif
       endif
#endif /* MPI_USED */
c
c Convert dtec from input dt
c
       dtec = 1.0/max(sqrt(dtimrdi2),tiny)
c
c Error test
c
       ade_max = ader_max
       if (ade_max .le. epsmaxd) then
c
c Error criterion is satisfied.  The timestep may be increased to
c speed up the calculation.
c
c        dtec = dtec * fac_dt
         delta_t_max = 0.02D0 * sqrt(2.0D0) * tlim
         dtec = min (dtec * fac_dt, delta_t_max)
       else
c
c The error test failed!  We really should do the step over
c with a smaller dt.  For now, follow Frank and just reduce dt
c for the next step.
c
c        dtec = dtec * epsmaxd / ade_max
         dtec = dtec / fac_dt
       endif
c
       dtimrdi2 = 1.0/max(dtec,tiny)**2
#endif /* RAD */
       return
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////            R A D _ P F _ D T              \\\\\\\\\\
c
c=======================================================================
c
c
