#include "zeusmp.def"
c
c     JCV 2/05/04
c     subroutine to calculate radiative energy loss
c     5/02/04  added physical units and realistic cooling law
c     2/15/06  added substeps to resolve cooling better and rk4
c
      subroutine rad_loss
      implicit NONE
#ifdef RAD_LOSS
c
c     standard includes
#include "param.h"
#include "root.h"
#include "field.h"
#include "grid.h"
#include "scratch.h"
#include "gravity.h"
#include "mpicom.h"
#include "cons.h"
#include "mod_names.h"

      integer i,j,k
cc  coefficients from cooling formula taken from Ruszkowski & Begelman 2002
      REAL radalpha,beta,c1,c2,c3,normalize
ccccccccccccccccccccccccccc
c Scaling factors real/code units
      REAL lscale,tiscale,Tescale,mscale,escale
c Scaling for cooling law
      REAL cscale
c Real units kbolt[eV K-1],muu mean moleculare weight
      REAL kbolt,muu
      REAL delta
      REAL cool
      REAL tkev
      integer tflag ,tflag2
      REAL rk1,rktotal,dtsub
c      REAL rk2,rk3,rk4
      integer tcount,ntcount
      REAL dtcool,coolt
      common / radc / dtcool

      character*15 filename
      write(filename,"(a4,3i2.2,'.','hdf')") 'last',coords(1)
     &     ,coords(2),coords(3)

cc These values from Ruszkowski & Begelman 2002
      radalpha=-1.7
      beta=0.5
      c1=8.6E-3
      c2=5.8E-2
      c3=6.4E-2
      normalize=10E-22
cccccccccccccccccccccc

      muu=1.27
      kbolt=1.4861E-9

      lscale=rcore*1000*cmpc/2
      tiscale=lscale/(cism*1000*100)
      Tescale=mp*muu*((cism*1000*100)**2)
     &     /(gamma*kbolt*3.0/5.0)
      mscale=muu*mp*ncent*(lscale**3)
      escale=(lscale**2)*mscale/(tiscale**2)
      cscale=(escale*(lscale**3))/(tiscale)

cccccccccccccc for perseusccccccccccccc
      c1=1.13
      c2=5.3
      c3=6.3
      Tescale=Tescale*0.625
      normalize=10E-24
ccccccccccccccccccccccccccccccccccccccc

c set dtrcool2 equal to something large
      dtcool=1000.0

      delta=0
      cool=0
      tflag=0
      tflag2=0

      do k=ks,ke
         do j=js,je
            do i=is,ie

c Calculate cooling timescale
               tkev=Tescale*(gamma-1)*e(i,j,k)/d(i,j,k)
               delta=((c1*(tkev**radalpha))+(c2*(tkev**beta))+
     &              c3)*normalize
               cool=(delta/cscale)*0.704*
     &              ((mscale*d(i,j,k)/mp)**2)
               coolt=(0.1*e(i,j,k))/cool
               if(coolt .le. dt)then
                  ntcount=(dt/coolt)+1
                  tflag2=1
               else
                  ntcount=1
               endif

               dtsub=dt/ntcount

               do tcount=1,ntcount
                  tkev=Tescale*(gamma-1)*e(i,j,k)/d(i,j,k)
                  if(e(i,j,k).le.0.0)then
                     write(*,*)'Negative energy before cooling 
     &                    at',i,j,k,myid
                     call hdfall(filename)
#ifdef MPI_USED
                     call MPI_FINALIZE
#endif /* MPI_USED */
                     stop
                  endif

                  if (tkev .GE. tmin) then
c     calculate loss
                     tkev=Tescale*(gamma-1)*e(i,j,k)/d(i,j,k)
                     delta=((c1*(tkev**radalpha))+(c2*(tkev**beta))+
     &                    c3)*normalize
                     cool=(delta/cscale)*0.704*
     &                    ((mscale*d(i,j,k)/mp)**2)
                     rk1=dtsub*cool
                     
c$$$                     tkev=Tescale*(gamma-1)*(e(i,j,k)+(rk1/2.0))/
c$$$     &                    d(i,j,k)
c$$$                     delta=((c1*(tkev**radalpha))+(c2*(tkev**beta))+
c$$$     &                    c3)*normalize
c$$$                     cool=(delta/cscale)*0.704*
c$$$     &                    ((mscale*d(i,j,k)/mp)**2)
c$$$                     rk2=dtsub*cool
c$$$
c$$$                     tkev=Tescale*(gamma-1)*(e(i,j,k)+(rk2/2.0))/
c$$$     &                    d(i,j,k)
c$$$                     delta=((c1*(tkev**radalpha))+(c2*(tkev**beta))+
c$$$     &                    c3)*normalize
c$$$                     cool=(delta/cscale)*0.704*
c$$$     &                    ((mscale*d(i,j,k)/mp)**2)
c$$$                     rk3=dtsub*cool
c$$$
c$$$                     tkev=Tescale*(gamma-1)*(e(i,j,k)+rk3)/d(i,j,k)
c$$$                     delta=((c1*(tkev**radalpha))+(c2*(tkev**beta))+
c$$$     &                    c3)*normalize
c$$$                     cool=(delta/cscale)*0.704*
c$$$     &                    ((mscale*d(i,j,k)/mp)**2)
c$$$                     rk4=dtsub*cool
c$$$                     
c$$$                     rktotal=3.0*((rk1/2.0)+rk2+rk3+(rk4/2.0))
                     rktotal=rk1
c update energy
                     e(i,j,k)=e(i,j,k)-rktotal

                     
c     newdt
                     coolt=0.1*(e(i,j,k)/(rktotal))

                     if (dtcool .GT. coolt) then
                        dtcool = coolt
                        tflag=1
                     endif
                  else
                     tflag=1
                  endif
      
                  if(e(i,j,k).le.0.0)then
                     write(*,*)'Negative energy in cooling 
     &                    at',i,j,k,myid
                     call hdfall(filename)
#ifdef MPI_USED
                     call MPI_FINALIZE
#endif /* MPI_USED */
                     stop
                  endif
            
               enddo
            
            enddo
         enddo
      enddo

      if (tflag.GT.0)then
         write(2,*)'Temperature below ',tmin,' at t= ',time
      endif

      if (tflag2.GT.0)then
         write(2,*)'Cooling substeps at t= ',time,' proc. = ',myid
      endif

#endif /* RAD_LOSS */
      return
      end
