#include "zeusmp.def"
c=======================================================================
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
      subroutine restart
c
c  PURPOSE:  Sets up restarted run
c
c  EXTERNALS:
c     PROBRES -- macroname which is defined to be the user-supplied
c                subroutine name which re-initializes user-defined
c                variables for the problem to be restarted.
c                PROBRES is undefined by default for backward 
c                compatiability with earlier versions of ZEUS-2D.
c     nudt    -- computes timestep
c
c  LOCALS:
c
c  LAST MODIFIED: 12/10/04 jcv
c  LAST MODIFIED: Jan. 7, 1999 by JCH for ZEUS-MP
c  LAST MODIFIED : Mar     1999 by efh, including debug
c-----------------------------------------------------------------------
      implicit NONE
#include "param.h"
#include "root.h"
      external nudt
      namelist /pcon/ nlim,tlim,cpulim,tsave,mbatch
c
#ifdef PROBRES
c
#include "grid.h"
#include "field.h"
#include "bndry.h"
#include "scratch.h"
#if defined GRAV || defined GRAV_FFT
#include "gravity.h"
#endif /* GRAV || GRAV_FFT */
C#ifdef RAD
#include "radiation.h"
C#endif /* RAD */
C#ifdef RAD_EXP
#include "radexp.h"
C#endif RAD_EXP
#include "opac.h"
#include "cons.h"
#include "mpicom.h"
#ifdef GRAV
#include "mgmpi.h"
#endif /* GRAV */
c
      external PROBRES
c
      integer  i,j,k
      integer  iord,istp
      REAL     dtrat
      namelist /hycon/
     &  qcon,qlin,courno,dtrat,iord,istp
     & ,iordd,iorde,iords1,iords2,iords3,iordb1,iordb2,iordb3,iorder
     & ,istpd,istpe,istps1,istps2,istps3,istpb1,istpb2,istpb3,istper
     & ,dfloor,efloor,v1floor,v2floor,v3floor,b1floor,b2floor,b3floor
     & ,emf1floor,emf2floor,emf3floor,erfloor,gpfloor
c
      integer  nbl,igrid
      REAL     x1min,x1max,x1rat,dx1min
     &        ,x2min,x2max,x2rat,dx2min
     &        ,x3min,x3max,x3rat,dx3min
      logical  lgrid
c
      character*2 x2units,x3units
      namelist /ggen1/ nbl,x1min,x1max,igrid,x1rat,dx1min,lgrid
      namelist /ggen2/ nbl,x2min,x2max,igrid,x2rat,dx2min,x2units,lgrid
      namelist /ggen3/ nbl,x3min,x3max,igrid,x3rat,dx3min,x2units,lgrid
c
c      namelist /iib/ ii, niib, miib, liib, b3iib
c     &,  diim1,  eiim1, v1ii  , v2iim1, v3iim1, b3iim1, emfii
c     &,  diim2,  eiim2, v1iim1, v2iim2, v3iim2, b3iim2, emfiim1, emfiim2
c     &, eriim1, eriim2
c      namelist /oib/ io, noib, moib, loib, b3oib
c     &,  diop1,  eiop1, v1iop1, v2iop1, v3iop1, b3iop1, emfiop1
c     &,  diop2,  eiop2, v1iop2, v2iop2, v3iop2, b3iop2, emfiop2, emfiop3
c     &, eriop1, eriop2
c      namelist /ijb/     nijb, mijb, lijb, b3ijb
c     &,  djim1,  ejim1, v1jim1, v2ji  , v3jim1, b3jim1, emfji
c     &,  djim2,  ejim2, v1jim2, v2jim1, v3jim2, b3jim2, emfjim1, emfjim2
c     &, erjim1, erjim2
c      namelist /ojb/     nojb, mojb, lojb, b3ojb
c     &,  djop1,  ejop1, v1jop1, v2jop1, v3jop1, b3jop1, emfjop1
c     &,  djop2,  ejop2, v1jop2, v2jop2, v3jop2, b3jop2, emfjop2, emfjop3
c     &, erjop1, erjop2
      namelist /iib/     niis, fiis
      namelist /oib/     nois, fois
      namelist /ijb/     nijs, fijs
      namelist /ojb/     nojs, fojs
      namelist /ikb/     niks, fiks
      namelist /okb/     noks, foks
c
      namelist /mpitop/ ntiles, periodic
#if defined GRAV || defined GRAV_FFT
      namelist /grvcon/ g,tgrav,ptmass,x1ptm,x2ptm,x3ptm,
     &                  rt_fn_grv,rt_co_grv,wgt_grv,
     &                  mi_fn_grv,mi_co_grv,m_grv,nu_pr_grv,nu_po_grv,
     &                  vv_grv,sl_fn_grv,sl_co_grv,smth_grv
#endif /* GRAV || GRAV_FFT */
#ifdef RAD 
      namelist /radcon/ c,ifld,epsme,demax,dermax,dtotmax,nmeiter,radth
     &, epsrad,maxrad,ks0rad,iorad,cnvcrit,ernom,ennom,epsmaxd,epsmaxc
     &,                 rt_fn_rad,rt_co_rad,wgt_rad
     &,                 mi_fn_rad,mi_co_rad,m_rad,nu_pr_rad,nu_po_rad
     &,                 vv_rad,sl_fn_rad,sl_co_rad,smth_rad
     &,                 kpfrac
#endif
#ifdef RAD_EXP
      namelist /radexp/ epsmin,epsmax,rmfp0,xnu,powr,rho0,t_0,rcourno
#endif
      namelist /eqos/ gamma, ciso
      namelist /gcon/ x1fac,x2fac,x3fac,ia,ja,ka,igcon
c
c=======================================================================
c\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\///////////////////////////////////
c
c------------------------  PROB CONTROL  -------------------------------
c
c   nlim   = cycles to run
c   tlim   = physical (problem) time to stop calculation
c cpulim   = CPU time in seconds to stop the calculation (default  3.6M)
c  tsave   = CPU time to reserve for terminating the run (default 30.0s)
c mbatch   = 0 interactive mode                          (default     0)
c          = 1 batch mode (does not scan for keyboard input)
c
      nlim   = 1 000 000
      tlim   = 0.0
      cpulim = 3 600 000.0
      tsave  = 30.0
      mbatch = 0
c
      nred   = 0
      if (myid_w .eq. 0) then
       read (1,pcon)
       write(2,pcon)
      endif
c
c------------------------  HYDRO CONTROL  ------------------------------
c
c  qcon   = quadratic artificial viscosity (q) constant
c  qlin   = linear    artificial viscosity (q) constant
c  courno = courant number
c  dtrat  = ratio of initial dt to dtmin (used to compute dtmin below)
c  iord   = default order of advection scheme for all variables
c  istp   = default steepening flag for all variables
c  iord** = order of advection scheme to be used for variable **
c  iostp**= steepening flag for 3rd order advection.  When istp**=1,
c           use the discontinuity detection to steepen shocks during
c           interpolation for variable ** in X1INT,X1INTFC,X2INT,X2INTFC
c  **floor = smallest value desired for variable ** on grid
c            Can also be used to set a default value for initialization.
c            Note that no attempt is made to ensure that actual values
c            stay above the floor values.
c
      if (myid_w .eq. 0) then
        qcon   = 2.0
        qlin   = 0.0
        courno = 0.5
        dtrat  = 1.0e-3
        iord   = 2
        iordd  = 0
        iorde  = 0
        iords1 = 0
        iords2 = 0
        iords3 = 0
        iordb1 = 0
        iordb2 = 0
        iordb3 = 0
        iorder = 0
        istp   = 0
        istpd  = 2
        istpe  = 2
        istps1 = 2
        istps2 = 2
        istps3 = 2
        istpb1 = 2
        istpb2 = 2
        istpb3 = 2
        istper = 2
        d floor = tiny
        e floor = tiny
        v1floor = 0.0
        v2floor = 0.0
        v3floor = 0.0
        b1floor = 0.0
        b2floor = 0.0
        b3floor = 0.0
        emf1floor= 0.0
        emf2floor= 0.0
        emf3floor= 0.0
        erfloor = tiny
        gpfloor = 0.0
c
      read (1,hycon)
      write(2,hycon)
c
c Set flags to default values unless they were set in the input deck.
c
        if(iordd  .eq. 0) iordd  = iord
        if(iorde  .eq. 0) iorde  = iord
        if(iords1 .eq. 0) iords1 = iord
        if(iords2 .eq. 0) iords2 = iord
        if(iords3 .eq. 0) iords3 = iord
        if(iordb1 .eq. 0) iordb1 = iord
        if(iordb2 .eq. 0) iordb2 = iord
        if(iordb3 .eq. 0) iordb3 = iord
        if(iorder .eq. 0) iorder = iord
c
        if(istpd  .eq. 2) istpd  = istp
        if(istpe  .eq. 2) istpe  = istp
        if(istps1 .eq. 2) istps1 = istp
        if(istps2 .eq. 2) istps2 = istp
        if(istps3 .eq. 2) istps3 = istp
        if(istps1 .eq. 2) istps1 = istp
        if(istps2 .eq. 2) istps2 = istp
        if(istps3 .eq. 2) istps3 = istp
        if(istper .eq. 2) istper = istp
c
c copy input flags to a buffer for later use and broadcasting.
c
#ifdef MPI_USED
         buf_in( 1) = qcon
         buf_in( 2) = qlin
         buf_in( 3) = courno
         buf_in( 4) = dtrat
         buf_in( 5) = dfloor
         buf_in( 6) = efloor
         buf_in( 7) = v1floor
         buf_in( 8) = v2floor
         buf_in( 9) = v3floor
         buf_in(10) = b1floor
         buf_in(11) = b2floor
         buf_in(12) = b3floor
         buf_in(13) = emf1floor
         buf_in(14) = emf2floor
         buf_in(15) = emf3floor
         buf_in(16) = erfloor
         buf_in(17) = gpfloor
         buf_in(18) = tlim
         buf_in(19) = cpulim
         buf_in(20) = tsave
c
        ibuf_in( 1) = iordd
        ibuf_in( 2) = iorde
        ibuf_in( 3) = iords1
        ibuf_in( 4) = iords2
        ibuf_in( 5) = iords3
        ibuf_in( 6) = iordb1
        ibuf_in( 7) = iordb2
        ibuf_in( 8) = iordb3
        ibuf_in( 9) = iorder
        ibuf_in(10) = istpd
        ibuf_in(11) = istpe
        ibuf_in(12) = istps1
        ibuf_in(13) = istps2
        ibuf_in(14) = istps3
        ibuf_in(15) = istpb1
        ibuf_in(16) = istpb2
        ibuf_in(17) = istpb3
        ibuf_in(18) = istper
        ibuf_in(19) = nlim
        ibuf_in(20) = mbatch
        ibuf_in(21) = idebug
#endif /* MPI_USED */
      endif
c
c Broadcast pcon and hycon to the others (use arrays).
c
#ifdef MPI_USED
      call MPI_BCAST( buf_in,20,MPI_FLOAT  ,0
     &                                       ,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(ibuf_in,21,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
      if (myid_w .ne. 0) then
        qcon    =  buf_in( 1)
        qlin    =  buf_in( 2)
        courno  =  buf_in( 3)
        dtrat   =  buf_in( 4)
        d floor =  buf_in( 5)
        e floor =  buf_in( 6)
        v1floor =  buf_in( 7)
        v2floor =  buf_in( 8)
        v3floor =  buf_in( 9)
        b1floor =  buf_in(10)
        b2floor =  buf_in(11)
        b3floor =  buf_in(12)
        emf1floor=  buf_in(13)
        emf2floor=  buf_in(14)
        emf3floor=  buf_in(15)
        erfloor =  buf_in(16)
        gpfloor =  buf_in(17)
        tlim    =  buf_in(18)
        cpulim  =  buf_in(19)
        tsave   =  buf_in(20)
c
        iordd   = ibuf_in( 1)
        iorde   = ibuf_in( 2)
        iords1  = ibuf_in( 3)
        iords2  = ibuf_in( 4)
        iords3  = ibuf_in( 5)
        iordb1  = ibuf_in( 6)
        iordb2  = ibuf_in( 7)
        iordb3  = ibuf_in( 8)
        iorder  = ibuf_in( 9)
        istpd   = ibuf_in(10)
        istpe   = ibuf_in(11)
        istps1  = ibuf_in(12)
        istps2  = ibuf_in(13)
        istps3  = ibuf_in(14)
        istpb1  = ibuf_in(15)
        istpb2  = ibuf_in(16)
        istpb3  = ibuf_in(17)
        istper  = ibuf_in(18)
        nlim    = ibuf_in(19)
#ifdef DEBUG
        write(*,"(/'Process ',i2,' has nlim = ',i2)") myid_w,nlim
#endif /* DEBUG */
        mbatch  = ibuf_in(20)
        idebug  = ibuf_in(21)
      endif
#endif /* MPI_USED */
c
c------------------------  BOUNDARY CONTROL ----------------------------
c
CJH   --- Data taken from RESTART ---
c
c-----------------  SET BOUNDARY TYPES FOR MULTIGRID    ---------------c
#ifdef MGRID
c
c     Set multigrid B.C. flags correctly
c
      IF (niis(2).EQ.1) ixmb = MM_DIRICHLET
      IF (nois(2).EQ.1) ixpb = MM_DIRICHLET
      IF (nijs(2).EQ.1) iymb = MM_DIRICHLET
      IF (nojs(2).EQ.1) iypb = MM_DIRICHLET
      IF (niks(2).EQ.1) izmb = MM_DIRICHLET
      IF (noks(2).EQ.1) izpb = MM_DIRICHLET
c
      IF (niis(2).EQ.3) ixmb = MM_DIRICHLET
      IF (nois(2).EQ.3) ixpb = MM_DIRICHLET
      IF (nijs(2).EQ.3) iymb = MM_DIRICHLET
      IF (nojs(2).EQ.3) iypb = MM_DIRICHLET
      IF (niks(2).EQ.3) izmb = MM_DIRICHLET
      IF (noks(2).EQ.3) izpb = MM_DIRICHLET
c
      IF (niis(2).EQ.4) ixmb = MM_PERIODIC
      IF (nois(2).EQ.4) ixpb = MM_PERIODIC
      IF (nijs(2).EQ.4) iymb = MM_PERIODIC
      IF (nojs(2).EQ.4) iypb = MM_PERIODIC
      IF (niks(2).EQ.4) izmb = MM_PERIODIC
      IF (noks(2).EQ.4) izpb = MM_PERIODIC
#endif /* MGRID */
c
c-------------------------  SET MPI DATATYPES  -------------------------c
c Define MPI Derived Datatypes for passing 2-D slices of 3-D arrays.
c
#ifdef MPI_USED
#ifdef DEBUG
      write(*,"(/'SETUP: Define i_slices as ',i4,' blocks of size '
     &  ,i4,' strided by ',i4)") jn*kn,1,in
#endif /* DEBUG */
      call MPI_TYPE_VECTOR (  jn*kn,1 ,in
     &, MPI_FLOAT, i_slice,ierr)
      call MPI_TYPE_COMMIT (i_slice,ierr)
c
      call MPI_TYPE_VECTOR (  jn*kn,1*neqm ,in*neqm
     &, MPI_FLOAT, ils_slice,ierr)
      call MPI_TYPE_COMMIT (ils_slice,ierr)
      call MPI_TYPE_VECTOR (  jn*kn,1*neqm*neqm ,in*neqm*neqm
     &, MPI_FLOAT, ilsm_slice,ierr)
      call MPI_TYPE_COMMIT (ilsm_slice,ierr)
#ifdef DEBUG
      write(*,"(/'SETUP: Define j_slices as ',i4,' blocks of size '
     &  ,i4,' strided by ',i4)") kn,in,in*jn
#endif /* DEBUG */
      call MPI_TYPE_VECTOR (    kn  ,in,in*jn,MPI_FLOAT
     &                                 ,j_slice,ierr)
      call MPI_TYPE_COMMIT (j_slice,ierr)
      call MPI_TYPE_VECTOR (    kn  ,in*neqm,in*jn*neqm,
     .                      MPI_FLOAT
     &                                 ,jls_slice,ierr)
      call MPI_TYPE_COMMIT (jls_slice,ierr)
      call MPI_TYPE_VECTOR (    kn  ,in*neqm*neqm,in*jn*neqm*neqm,
     .                      MPI_FLOAT
     &                                 ,jlsm_slice,ierr)
      call MPI_TYPE_COMMIT (jlsm_slice,ierr)
#ifdef DEBUG
      write(*,"(/'SETUP: Define k_slices as ',i4,' blocks of size '
     &  ,i4,' strided by ',i4)") 1,in*jn,1
#endif /* DEBUG */
      call MPI_TYPE_VECTOR (1,in*jn ,1    ,MPI_FLOAT
     &                                 ,k_slice,ierr)
      call MPI_TYPE_COMMIT (k_slice,ierr)
      call MPI_TYPE_VECTOR (1,in*jn*neqm ,1    ,
     .                      MPI_FLOAT
     &                                 ,kls_slice,ierr)
      call MPI_TYPE_COMMIT (kls_slice,ierr)
      call MPI_TYPE_VECTOR (1,in*jn*neqm*neqm ,1    ,
     .                      MPI_FLOAT
     &                                 ,klsm_slice,ierr)
      call MPI_TYPE_COMMIT (klsm_slice,ierr)
#endif /* MPI_USED */
c
c-------------------------  READ GRID NAMELISTS  -----------------------
c
CJH   --- GRID data handled in RESTART ---
c
c------------------------  GRAVITY CONTROL  ----------------------------
c
c  Gravitational self-potentials can be included in both 1-D and 2-D
c  problems by solving the Poisson equation in the GRAVITY module.
c  Point mass potentials are included directly in the momentum eqn
c  by using a non-zero value for the variable ptmass.  Point mass
c  potentials do not require defining GRAV, do not call the GRAVITY
c  module, and are not included in the array phi but are explicitely
c  added to the momentum eqn terms in the routines STV1 and STV2.
c     g      = gravitational constant
c     tgrav  = time when gravitation is switched on
c     ptmass = fixed central point mass object
c     izero  = i index of Z=0 (x1=0) for odd symmetry case
c              (cylindrical geometry only)
c     igrijb = ijb flag (0 for     symmetric (Dirichlet) boundary      )
c                       (1 for non-symmetric             boundary whose
c                        value is calculated using multipole expansion )
c     igrojb = ojb flag ("  "      "          "         "              )
c     epsgrv = error limit              for ICCGAF
c     maxgrv = maximum iteration count  for ICCGAF
c     ks0grv = level of cyclic reduction in ICCGAF
c     phibverr = error criteria for multipole moments in PHIBV
c     phibvnmx = max number of moments to be taken in PHIBV
c     graverr = error criteria for computing a soln in GRAVITY
c
      if(myid_w .eq. 0) then
        g         = 6.667e-8
C        g0        = 6.667e-8   ! efh 99/05/14
        tgrav     = 0.0        ! efh 99/05/14
        ptmass    = 0.0
        x1ptm     = x1a(is)
        x2ptm     = x2a(js)
        x3ptm     = x3a(ks)

        wgt_grv   = 0.9
        rt_fn_grv = 1.0E-7
        rt_co_grv = 1.0E-7
        m_grv     = 4
        nu_pr_grv = 3
        nu_po_grv = 3
        sl_fn_grv = 2
        sl_co_grv = 1
        mi_fn_grv = 10
        mi_co_grv = 100
        vv_grv    = 1
        smth_grv  = 0
c
       read (1,grvcon)
       write(2,grvcon)
      endif
c
cc-------------------------  READ GRID NAMELISTS  -----------------------
cc
cc  Read in blocks of x1 grid zones.  Note we loop over read statement
cc  until all blocks are read (signalled by reading in lgrid = .true.).
cc  We can zone within each block completely independently of the others,
cc  however we must ensure the starting position of one block (x1min) is
cc  the same as the ending position (x1max) of the previous.
cc   nbl    is number of active zones in block read in
cc   x1min  is x1a(imin) ; bottom position of block
cc   x1max  is x1a(imax) ; top    position of block
cc   igrid  selects zoning type; we solve the zoning equation:
cc            x1max = x1min + SUM OVER N[dx1min*x1rat**n]  ; so we must 
cc          input either dx1min or x1rat (the other is calculated)
cc     igrid = 1 uses input ratios          => x1rat  = dx1a(i+1)/dx1a(i)
cc     igrid = 2 uses input inner zone size => dx1min = dx1a(imin)
cc   lgrid  logical flag for additional blocks ( =.true. reads another)
cc   imax,imin  are indices of top and bottom of block
cc
c      lgrid = .false.
c10    continue
c      read (1,ggen1)
c      write(2,ggen1)
c      if (.not. lgrid) go to 10
cc
c      lgrid =.false.
c110   continue
c      read (1,ggen2)
c      write(2,ggen2)
c      if (.not. lgrid) go to 110
c
cc------------------------  BOUNDARY CONTROL ----------------------------
cc
cc  The following points describe how boundaries are handled:
cc
cc*1)  We may specify any of 4 fluid boundary conditions independently at
cc  every zone on the problem boundary.  The problem i boundaries may be
cc  irregular to treat steps and cavities (resulting in irregular j
cc  boundaries as well).
cc     ii(j) is  i  index of first active zone for sweep  j
cc     io(j) is  i  index of last  active zone for sweep  j
cc     ji(i) is  j  index of first active zone for sweep  i # calculated
cc     jo(i) is  j  index of last  active zone for sweep  i # from ii,io
cc  The boundary type is specified by nflo, where
cc     nflo = 1   reflecting
cc          =-1   reflecting with inversion of 3-velocity
cc          = 2   flow out 
cc          = 3   flow in 
cc          = 4   periodic
cc   then,    niib(j) is nflo of inner i boundary on sweep j
cc            noib(j) is nflo of outer i boundary on sweep j
cc            nijb(i) is nflo of inner j boundary on sweep i
cc            nojb(i) is nflo of outer j boundary on sweep i
cc
cc*2) Since the mhd boundary types may differ from the fluid boundary
cc  types, we may specify the former independently of the latter.  Thus,
cc  the mhd boundary types are specified by mflo,where
cc     mflo = 1   reflecting
cc          =-1   reflecting with inversion of 3-magnetic field
cc          = 2   flow out
cc          = 3   flow in
cc          = 4   periodic
cc  Values for mflo are stored in miib,moib,mijb,mojb.
cc
cc*3) Since the radiation boundary types may differ from both the fluid
cc  and mhd boundary types,we may specify the former independent of the
cc  others.  The boundary flag values follow the above conventions.
cc
cc*4) To be completely general, we allow the boundary flag for each
cc  variable to be set independent of all others.  This can be done by
cc  setting values for the flags in the namelists, eg:
cc   diib,doib,dijb,dojb = flags on boundaries for d
cc  If no values are input for each individual variable, then the flags
cc  are initialized using the values for nflo,mflo above.
cc
cc*5) For flow in boundaries, boundary values of d,e,v1,v2,[v3],...
cc  must be input, for example:
cc      diim1(j) is inner i boundary density for sweep j at iim1(j)
cc      diim2(j) is inner i boundary density for sweep j at iim2(j)
cc      diop1(j) is outer i boundary density for sweep j
cc      djim1(i) is inner j boundary density for sweep i
cc      djop1(i) is outer j boundary density for sweep i
cc
cc*6) Since the corner zones [(ii-1,ji-1),(io+1,ji-1),etc] are set across
cc  the ijb or ojb, then all i boundary values must be specified for
cc  j=js,je and all j boundary values must be specified for i=is-2,ie+2
cc
cc  set boundaries, order is iib, oib, ijb, ojb
cc
c      read (1,iib)
c      write(2,iib)
cc
c      read (1,oib)
c      write(2,oib)
cc
c      read (1,ijb)
c      write(2,ijb)
cc
c      read (1,ojb)
c      write(2,ojb)
cc
cc------------------------  GRAVITY CONTROL  ----------------------------
cc
cc  Gravitational self-potentials can be included in both 1-D and 2-D
cc  problems by solving the Poisson equation in the GRAVITY module.
cc  Point mass potentials are included directly in the momentum eqn
cc  by using a non-zero value for the variable ptmass.  Point mass
cc  potentials do not require defining GRAV, do not call the GRAVITY
cc  module, and are not included in the array phi but are explicitely
cc  added to the momentum eqn terms in the routines STV1 and STV2.
cc     g      = gravitational constant
cc     ptmass = fixed central point mass object
cc     izero  = i index of Z=0 (x1=0) for odd symmetry case
cc              (cylindrical geometry only)
cc     igrijb = ijb flag (0 for     symmetric (Dirichlet) boundary      )
cc                       (1 for non-symmetric             boundary whose
cc                        value is calculated using multipole expansion )
cc     igrojb = ojb flag ("  "      "          "         "              )
cc     epsgrv = error limit              for ICCGAF
cc     maxgrv = maximum iteration count  for ICCGAF
cc     ks0grv = level of cyclic reduction in ICCGAF
cc     phibverr = error criteria for multipole moments in PHIBV
cc     phibvnmx = max number of moments to be taken in PHIBV
cc      graverr = error criteria for computing a soln in GRAVITY
cc
c      read (1,grvcon)
c      write(2,grvcon)
cc
c------------------------  RADIATION CONTROL  --------------------------
c
c   c      = speed of light
c   ifld   = choice of flux limiters
c   epsme  = error criterion for NR convergence (WH80s uses 1.0e-5)
c demax,dermax = max relative change allowed in e,er in a single NR iter
c   dtotmax= max total relative change allowed over all NR iterations
c   nmeiter= max number of NR iterations allowed (WH80s uses 20)
c   radth  = "theta" time centering parameter
c     epsrad = error limit              for ICCGAF
c     maxrad = maximum iteration count  for ICCGAF
c     ks0rad = level of cyclic reduction in ICCGAF
c   iorad = 1 to dump convergence data for NR iterations
c
#ifdef RAD || defined RAD_EXP
      if(myid_w .eq. 0) then
        c      = clight
        ifld   = 1
        epsme  = 1.0e-08
        de  max= 0.2
        der max= 0.2
        dtotmax= 0.2
        nmeiter= 20
        radth  = 0.55
        epsrad = 1.0e-12
        maxrad = 200
        ks0rad = 99
        iorad = 0
c
        wgt_rad   = 0.9
        rt_fn_rad = 1.0E-7
        rt_co_rad = 1.0E-7
        m_rad     = 4
        nu_pr_rad = 3
        nu_po_rad = 3
        sl_fn_rad = 2
        sl_co_rad = 1
        mi_fn_rad = 10
        mi_co_rad = 100
        vv_rad    = 1
c
        read (1,radcon)
        write(2,radcon)
      endif
      if(myid_w .eq. 0) then
        epsmin  = 0.001
        epsmax  = 0.01
        rmfp0   = 1.0
        xnu     = 1.9
        powr    = 5.6/4.0
        rho0    = 1.0
        t_0     = 1.0
        rcourno = 1.0
c
        read (1,radexp)
        write(2,radexp)
      endif
#endif /* RAD || defined RAD_EXP */
c
c------------------------  EQUATION OF STATE  --------------------------
c
c      gamma = ratio of specific heats
c      ciso  = isothermal sound speed
c
      if(myid_w .eq. 0) then
        gamma = 0.0
        ciso  = 0.0
c
        read (1,eqos)
        write(2,eqos)
      endif
#ifdef MPI_USED
c
c Broadcast grvcon, radcon, radexp, and eqos to the others.
c
      if (myid_w .eq. 0) then
         buf_in( 1) = g
         buf_in( 2) = tgrav
         buf_in( 3) = ptmass
         buf_in( 4) = x1ptm
         buf_in( 5) = x2ptm
         buf_in( 6) = x3ptm
         buf_in( 7) = wgt_grv
         buf_in( 8) = rt_fn_grv
         buf_in( 9) = rt_co_grv
         buf_in(10) = c
         buf_in(11) = epsme
         buf_in(12) = de  max
         buf_in(13) = der max
         buf_in(14) = dtotmax
         buf_in(15) = radth
         buf_in(16) = epsrad
         buf_in(17) = epsmin
         buf_in(18) = epsmax
         buf_in(19) = rmfp0
         buf_in(20) = xnu
         buf_in(21) = powr
         buf_in(22) = rho0
         buf_in(23) = t_0
         buf_in(24) = rcourno
         buf_in(25) = gamma
         buf_in(26) = ciso
         buf_in(27) = epsmaxd
         buf_in(28) = epsmaxc
         buf_in(29)  = wgt_rad
         buf_in(30)  = rt_fn_rad
         buf_in(31)  = rt_co_rad
         buf_in(32) = ernom
         buf_in(33) = ennom
         buf_in(34) = kpfrac
        ibuf_in(1)  = ifld
        ibuf_in(2)  = nmeiter
        ibuf_in(3)  = maxrad
        ibuf_in(4)  = ks0rad
        ibuf_in(5)  = iorad
        ibuf_in(6)  = m_grv
        ibuf_in(7)  = nu_pr_grv
        ibuf_in(8)  = nu_po_grv
        ibuf_in(9)  = sl_fn_grv
        ibuf_in(10) = sl_co_grv
        ibuf_in(11) = mi_fn_grv
        ibuf_in(12) = mi_co_grv
        ibuf_in(13) = vv_rad
        ibuf_in(14)  = m_rad
        ibuf_in(15)  = nu_pr_rad
        ibuf_in(16)  = nu_po_rad
        ibuf_in(17)  = sl_fn_rad
        ibuf_in(18) = sl_co_rad
        ibuf_in(19) = mi_fn_rad
        ibuf_in(20) = mi_co_rad
        ibuf_in(21) = vv_rad
        ibuf_in(22) = cnvcrit
      endif
      call MPI_BCAST( buf_in,34,MPI_FLOAT  ,0,comm3d,ierr)
      call MPI_BCAST(ibuf_in,22,MPI_INTEGER,0,comm3d,ierr)

      if (myid_w .ne. 0) then
        g          =  buf_in( 1)
        tgrav      =  buf_in( 2)
        ptmass     =  buf_in( 3)
        x1ptm      =  buf_in( 4)
        x2ptm      =  buf_in( 5)
        x3ptm      =  buf_in( 6)
        wgt_grv    =  buf_in( 7)
        rt_fn_grv  =  buf_in( 8)
        rt_co_grv  =  buf_in( 9)
        c          =  clight
        epsme      =  buf_in(11)
        de  max    =  buf_in(12)
        der max    =  buf_in(13)
        dtotmax    =  buf_in(14)
        radth      =  buf_in(15)
        epsrad     =  buf_in(16)
        epsmin     =  buf_in(17)
        epsmax     =  buf_in(18)
        rmfp0      =  buf_in(19)
        xnu        =  buf_in(20)
        powr       =  buf_in(21)
        rho0       =  buf_in(22)
        t_0        =  buf_in(23)
        rcourno    =  buf_in(24)
        gamma      =  buf_in(25)
        ciso       =  buf_in(26)
        epsmaxd    =  buf_in(27)
        epsmaxc    =  buf_in(28)
        wgt_rad    =  buf_in(29)
        rt_fn_rad  =  buf_in(30)
        rt_co_rad  =  buf_in(31)
        ernom      =  buf_in(32)
        ennom      =  buf_in(33)
        kpfrac     =  buf_in(34)
        ifld       = ibuf_in(1)
        nmeiter    = ibuf_in(2)
        maxrad     = ibuf_in(3)
        ks0rad     = ibuf_in(4)
        iorad      = ibuf_in(5)
        m_grv      = ibuf_in(6)
        nu_pr_grv  = ibuf_in(7)
        nu_po_grv  = ibuf_in(8)
        sl_fn_grv  = ibuf_in(9)
        sl_co_grv  = ibuf_in(10)
        mi_fn_grv  = ibuf_in(11)
        mi_co_grv  = ibuf_in(12)
        vv_grv     = ibuf_in(13)
        m_rad      = ibuf_in(14)
        nu_pr_rad  = ibuf_in(15)
        nu_po_rad  = ibuf_in(16)
        sl_fn_rad  = ibuf_in(17)
        sl_co_rad  = ibuf_in(18)
        mi_fn_rad  = ibuf_in(19)
        mi_co_rad  = ibuf_in(20)
        vv_rad     = ibuf_in(21)
        cnvcrit    = ibuf_in(22)
      endif
#endif /* MPI_USED */
      gamm1 = gamma - 1.0
c
c--------------RADIATION TILE BOUNDARY ARRAYS -------------------------
c
c  For the loops below, set the boundary flags to zero for internal
c  boundaries.  Copy the constant values into the 2-D arrays.
c
c----------------------  Constant values for IIB  ---------------------
c
#if defined RAD || defined RAD_EXP
       do 40 k=ks-2,ke+3
         do 30 j=js-2,je+3
           if (coords(1) .eq. 0) then
             liib    (j,k) = niis(2)
           else
             liib    (j,k) = 0
           endif
30       continue
40     continue
c
c-----------------------  Constant values for OIB  ---------------------
c
       do 60 k=ks-2,ke+3
         do 50 j=js-2,je+3
           if (coords(1) .eq. ntiles(1) - 1) then
             loib    (j,k) = nois(2)
           else
             loib    (j,k) = 0
           endif
50       continue
60     continue
c
c-----------------------  Constant values for IJB  ---------------------
c
       do 80 k=ks-2,ke+3
         do 70 i=is-2,ie+3
           if (coords(2) .eq. 0) then
             lijb    (i,k) = nijs(2)
           else
             lijb    (i,k) = 0
           endif
70       continue
80     continue
c
c-----------------------  Constant values for OJB  ---------------------
c
       do 100 k=ks-2,ke+3
         do 90 i=is-2,ie+3
           if (coords(2) .eq. ntiles(2) - 1) then
             lojb    (i,k) = nojs(2)
           else
             lojb    (i,k) = 0
           endif
90       continue
100    continue
c
c-----------------------  Constant values for IKB  ---------------------
c
       do 120 j=js-2,je+3
         do 110 i=is-2,ie+3
           if (coords(3) .eq. 0) then
             likb    (i,j) = niks(2)
           else
             likb    (i,j) = 0
           endif
110      continue
120    continue
c
c-----------------------  Constant values for OKB  ---------------------
c
       do 140 j=js-2,je+3
         do 130 i=is-2,ie+3
           if (coords(3) .eq. ntiles(3) - 1) then
             lokb    (i,j) = noks(2)
           else
             lokb    (i,j) = 0
           endif
130      continue
140    continue
#endif /* defined RAD || defined RAD_EXP */
c
c-------------------------  PROBLEM RESTART  ---------------------------
c
c  PROBRES is a user defined cpp macroname representing a subroutine
c  which re-intializes all user-defined variables for the particular 
c  problem to be restarted.  PROBRES should re-initialize only the
c  variables defined by the user in PROBLEM which are not in any
c  ZEUS-2D common blocks and are therefore not saved in the restart
c  dump.  PROBRES must read the same namelist(s) as PROBLEM.
c
      call PROBRES
c
c-------------------------  GRID MOTION CONTROL  -----------------------
c
c        igcon = 1 and x1fac < 0 gives "lagrangian" tracking in x1 lines
c        igcon = 1 and x2fac < 0 gives "lagrangian" tracking in x2 lines
c        igcon = 2 for input grid boundary speeds
c                vg1(io) = x1fac * central soundspeed
c                vg2(jo) = x2fac * central soundspeed
c        igcon = 3 for constant motion at x1[2]fac
c
      if(myid_w .eq. 0) then
        read (1,gcon)
        write(2,gcon)
      endif
c
c----------------------  COMPUTE TIMESTEP  -----------------------------
       do 370 k=ks,ke
        do 360 j=js,je
         do 350 i=is,ie
          w3da(i,j,k) = v1(i,j,k) * 0.5 * (d(i-1,j  ,k  ) + d(i,j,k))
          w3db(i,j,k) = v2(i,j,k) * 0.5 * (d(i  ,j-1,k  ) + d(i,j,k))
     1                   * g2b(i)
          w3dc(i,j,k) = v3(i,j,k) * 0.5 * (d(i  ,j  ,k-1) + d(i,j,k))
     1                   * g31b(i) * g32b(j)
#if defined RAD || defined RAD_EXP
#ifdef NO_TRANSPORT
          w3dh(i,j,k) = er(i,j,k) / d(i,j,k)
#else /* NO_TRANSPORT */
          w3dh(i,j,k) = sqrt(sqrt(er(i,j,k)/rad_con)) ! / d(i,j,k)
#endif /* NO_TRANSPORT */
#endif /* RAD || RAD_EXP */
350      continue
360     continue
370    continue
c
c     Do I need this?  It was removed in zeusmpv2
      call nudt
#endif /* PROBRES */
c
      return
      end
