#include "zeusmp.def"

C JCV - 10/16/03  based on CSR bubble routine and z3d rgjet from jcv
C JCV - 10/06/05  oscillating cluster

c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                  RG_OSC                    \\\\\\\\\\
c
c=======================================================================
c
       subroutine rg_osc

       implicit NONE
#include "mod_names.h"
#include "param.h"
#include "field.h"
#include "bndry.h"
#include "grid.h"
#include "root.h"
#include "scratch.h"
#include "mpicom.h"

       integer i,j,k

       REAL fact
       integer thetamax
       REAL buffer(5)
       REAL buff2(2)
       namelist /jet/ djet,v1jet,jetsize,efficiency,mcutoff
c       external      bndyflgs, bndyall
       fact=0.3
       

c set defaults
       djet    =  0.01
       v1jet   =  105.5
       jetsize =  8.33
       efficiency = 0.0
       mcutoff = 0.0

#ifdef MPI_USED
       if (myid .eq. 0) then
#endif /*MPI_USED*/
       rewind(1)
       read (1,jet)
       write(2,jet)
       rewind(1)
       
#ifdef MPI_USED
       buffer(1)=djet
       buffer(2)=v1jet
       buffer(3)=jetsize
       buffer(4)=efficiency
       buffer(5)= mcutoff
       endif
       call MPI_BCAST(buffer,5,MPI_FLOAT,0,MPI_COMM_WORLD
     &              ,ierr)

       if (myid .ne. 0) then
          djet   = buffer(1)
          v1jet  = buffer(2)
          jetsize = buffer(3)
          efficiency = buffer(4)
          mcutoff = buffer(5)
       endif
       if (myid .eq. 0) then
          close(14)
       endif
#endif /*MPI_USED*/

       thetamax=J_ZONES*ntiles(2)/jetsize
c Uncomment the next line if there is only one jet
       thetamax=thetamax*2.0
      if (myid .eq. 0) then
         write(*,*)'thetamax=',thetamax,' v1jet=',v1jet
         write(6,"(10x,'RG_OSC  : Initialisation commencing')")
         write(*,*)gamma
      endif

      rin=x1a(3)
      jettheta=x2b(thetamax+2)

      if(myid .eq. 0)then
         if ((x1b(is) .LE. 1).and.(x2b(js) .LE. 0.1))then
            buff2(1)=rin
            buff2(2)=jettheta
         else
            write(*,*)'Node 0 does not contain jet base. Fix rgjet.F'
#ifdef MPI_USED
            call MPI_FINALIZE
#endif /* MPI_USED */
            stop
         endif
      endif

      call MPI_BCAST(buff2,2,MPI_FLOAT,0,MPI_COMM_WORLD
     &     ,ierr)

         
      if(myid .ne. 0)then
         rin=buff2(1)
         jettheta=buff2(2)
      endif

c make sure jet fits on one processor.  Necessary to make setup easy
       if (thetamax .GE. je) then
       write(*,*)'jet is too big to fit on one processor. Need to fix.'
#ifdef MPI_USED
       call MPI_FINALIZE
#endif /* MPI_USED */
       stop
       endif
cccccccccccccccc
      do i=is-2,ie+2
         do j=js-2,je+2
            do k=ks-2,ke+2
c
c     set up undisturbed atmosphere
c
c changed beta and rcore
               d(i,j,k)=(1.0+(x1b(i)/1.0)**2.0)**(-0.46)
               e(i,j,k)=9.0*d(i,j,k)/10.0
               v1(i,j,k)=0.0
               v2(i,j,k)=0.0
               v3(i,j,k)=0.0
               gp(i,j,k)=(3.0/5.0)*log(d(i,j,k))

               d(i,j,k)=d(i,j,k)*(1.0+fact*(cos(x2b(j))**2))
               e(i,j,k)=e(i,j,k)*
     &             (1+0+fact*(cos(x2b(j))**2)**(gamma+1.0))

            enddo
         enddo
      enddo

c
c     set inflow boundaries
c

c Set all boundaries equal to no jet first
         do j=js-2,je+2
            do k=ks-2,ke+2
               diib(j,k,1)=djet
               diib(j,k,2)=djet
               eiib(j,k,1)=9.0/10.0
               eiib(j,k,2)=9.0/10.0
               v1iib(j,k,1)=0.0
               v1iib(j,k,2)=0.0
               v2iib(j,k,1)=0.0
               v2iib(j,k,2)=0.0
               v3iib(j,k,1)=0.0
               v3iib(j,k,2)=0.0
            enddo
         enddo

c Only look for jet near inner edge
      if (x1b(is) .LE. 1) then

c NOTE if there are too many processors (~10), this will no longer work

c Do first jet if t direction starts close to 0
      if (x2b(js) .LE. 0.1) then

c 1st jet
         do j=js-2,js+thetamax
            do k=ks-2,ke+2
               niib(j,k)=3
               diib(j,k,1)=djet
               diib(j,k,2)=djet
               eiib(j,k,1)=9.0/10.0
               eiib(j,k,2)=9.0/10.0
               v1iib(j,k,1)=v1jet
               v1iib(j,k,2)=v1jet
               v2iib(j,k,1)=0.0
               v2iib(j,k,2)=0.0
               v3iib(j,k,1)=0.0
               v3iib(j,k,2)=0.0
            enddo
         enddo

c end loop for 1st jet
      endif
ccc

c only do 2nd jet if t direction ends at almost 3.14
      if (x2b(je) .GE. 3.0) then
         
c jet 2
         do j=je-thetamax,je+2
            do k=ks-2,ke+2
               niib(j,k)=3
               diib(j,k,1)=djet
               diib(j,k,2)=djet
               eiib(j,k,1)=9.0/10.0
               eiib(j,k,2)=9.0/10.0
               v1iib(j,k,1)=v1jet
               v1iib(j,k,2)=v1jet
               v2iib(j,k,1)=0.0
               v2iib(j,k,2)=0.0
               v3iib(j,k,1)=0.0
               v3iib(j,k,2)=0.0
            enddo
         enddo

c end loop for 2nd jet
      endif
ccccc

c end loop for both jets
      endif
ccccccccccc

      call bndyflgs


c---All done

c       write (iotty, 2010)
c       write (iolog, 2010)
c2010   format('MYPROB  : Initialisation complete.')
c

       return
       end
