#include "zeusmp.def"
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////               R H D _ P F                 \\\\\\\\\\
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
c=======================================================================
c
       subroutine rhd_pf
c
c  AUTHOR
c    Robert Fiedler
c
c  LAST MODIFIED
c    12/16/96
c
c  PURPOSE
c    Driver routine for implicit update of coupled e and er eqns.
c    in the Product Formula RHD algorithm.
c
c
c  EXTERNALS:
c    BVALV1  , BVALV2  , BVALV3
c    BVALER  ,
c    NR2X2
c
c-----------------------------------------------------------------------
       implicit NONE
#include "param.h"
#include "root.h"
#include "field.h"
#include "bndry.h"
#include "grid.h"
#include "scratch.h"
#include "mpicom.h"
#ifdef PROFILING
#ifdef MPE_USED
#else
#include "clocks.h"
#endif /* MPE_USED */
#endif /* PROFILING */
c
       integer i, ko3
c
c      External statements
c
       external      nr2x2
c
       ko3 = int( real( ke - ks + 1 ) / 3.0 )
#ifdef DEBUG
       write(*,"(/'RHD_PF: ',i2,' calling nr_pf.')") myid
#endif /* DEBUG */
c
c i boundaries
c
c    1) Post sends and receives.
c
       nreq = 0
       nsub = nsub + 1
       call bvalv1 (0,1,0,0,0,0,v1)
       call bvalv2 (1,1,0,0,0,0,v2)
       call bvalv3 (1,1,0,0,0,0,v3)
       call bvaler (1,1,0,0,0,0,er)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(nsubstps,1,2,2) = timers(nsubstps,1,2,2) + wal1  - wal0
       timers(nsubstps,1,2,1) = timers(nsubstps,1,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
c    2) Do first 1/3 of the interior points.
c
       call nr_pf (is+1,ie-1,js+1,je-1,ks+1 ,ko3
     &            ,e,w3de,er,w3df)
c
c      subroutine nr_pf (ibeg,iend,jbeg,jend,kbeg,kend
c     1                 ,eold,enew,erold,ernew)
c

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(13,1,2,2) = timers(13,1,2,2) + wal1  - wal0
       timers(13,1,2,1) = timers(13,1,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#ifdef MPI_USED
c
c    3) Wait for communications to complete.
c
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(13,1,1,2) = timers(13,1,1,2) + wal1  - wal0
       timers(13,1,1,1) = timers(13,1,1,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#endif
c......................................................................
c
c j boundaries
c
c    1) Post sends and receives.
c
       nreq = 0
       nsub = nsub + 1
       call bvalv1 (0,0,1,1,0,0,v1)
       call bvalv2 (0,0,0,1,0,0,v2)
       call bvalv3 (0,0,1,1,0,0,v3)
       call bvaler (0,0,1,1,0,0,er)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(nsubstps,2,2,2) = timers(nsubstps,2,2,2) + wal1  - wal0
       timers(nsubstps,2,2,1) = timers(nsubstps,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
c    2) Do middle 1/3 of the interior points, and some on borders.
c
       call nr_pf (is  ,is  ,js+1,je-1,ks+1 ,ko3
     &            ,e,w3de,er,w3df)
       call nr_pf (ie  ,ie  ,js+1,je-1,ks+1 ,ko3
     &            ,e,w3de,er,w3df)
       call nr_pf (is  ,ie  ,js+1,je-1,ko3+1 ,ko3+ko3
     &            ,e,w3de,er,w3df)
c
c      subroutine nr_pf (ibeg,iend,jbeg,jend,kbeg,kend
c     1                 ,eold,enew,erold,ernew)
c

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(13,2,2,2) = timers(13,2,2,2) + wal1  - wal0
       timers(13,2,2,1) = timers(13,2,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#ifdef MPI_USED
c
c    3) Wait for communications to complete.
c
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(13,2,1,2) = timers(13,2,1,2) + wal1  - wal0
       timers(13,2,1,1) = timers(13,2,1,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#endif
c......................................................................
c
c k boundaries
c
c    1) Post sends and receives.
c
       nreq = 0
       nsub = nsub + 1
       call bvalv1 (0,0,0,0,1,1,v1)
       call bvalv2 (0,0,0,0,1,1,v2)
       call bvalv3 (0,0,0,0,0,1,v3)
       call bvaler (0,0,0,0,1,1,er)

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(nsubstps,3,2,2) = timers(nsubstps,3,2,2) + wal1  - wal0
       timers(nsubstps,3,2,1) = timers(nsubstps,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

c
c    2) Do last 1/3 of the interior points, and some on borders.
c
       call nr_pf (is  ,ie  ,js  ,js  ,ks+1 ,ko3+ko3
     &            ,e,w3de,er,w3df)
       call nr_pf (is  ,ie  ,je  ,je  ,ks+1 ,ko3_ko3
     &            ,e,w3de,er,w3df)
       call nr_pf (is  ,ie  ,js  ,je  ,ko3+ko3+1 ,ke-1
     &            ,e,w3de,er,w3df)
c
c      subroutine nr_pf (ibeg,iend,jbeg,jend,kbeg,kend
c     1                 ,eold,enew,erold,ernew)
c
c      Mark the boundary values out of date.
c
       do 20 i = 1,6
         bvstat(i,3) = 0      !  v1
         bvstat(i,4) = 0      !  v2
         bvstat(i,5) = 0      !  v3
         bvstat(i,6) = 0      !  er
20     continue

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(13,3,2,2) = timers(13,3,2,2) + wal1  - wal0
       timers(13,3,2,1) = timers(13,3,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#ifdef MPI_USED
c
c    3) Wait for communications to complete.
c
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(13,3,1,2) = timers(13,3,1,2) + wal1  - wal0
       timers(13,3,1,1) = timers(13,3,1,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

#endif
c......................................................................
c
c Do the remaining border zones.
c
       call nr_pf (is  ,is  ,js  ,je  ,ks   ,ks
     &            ,e,w3de,er,w3df)
       call nr_pf (is  ,ie  ,js  ,je  ,ke   ,ke
     &            ,e,w3de,er,w3df)
c

#ifdef PROFILING
#ifdef MPE_USED
       call MPE ...
#else
       call clocks (cputime, wclock)
       wal0 = wal1
       cpu0 = cpu1
       wal1 = real(wclock)
       cpu1 = real(cputime)
       timers(13,4,2,2) = timers(13,4,2,2) + wal1  - wal0
       timers(13,4,2,1) = timers(13,4,2,1) + cpu1  - cpu0
#endif /* MPE_USED */
#endif /* PROFILING */

       return
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////               R H D _ P F                 \\\\\\\\\\
c
c=======================================================================
c
c
