#include "zeusmp.def"
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////               S H K T U B E               \\\\\\\\\\
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
c=======================================================================
c
       subroutine shktube
c
c    mln:zeus3d.shkset <---------------- sod shock tube in any direction
c    from mln:zeus04.shksetz,shksetr                           may, 1986
c
c    written by: Mike Norman
c    modified 1: May, 1986 by David Clarke; adapted for mhd
c    modified 2: October, 1987 by Mike Norman; reworked for covariant
c                formulation
c    modified 3: July, 1988 by Jim Stone; incorporated into ZEUS2D
c    modified 4: February, 1990 by David Clarke; incorporated into
c                ZEUS3D
c    modified 5: December, 1999 by PSLi; miniaturized to cylindrical
c                coordinate systems for ZeusMP.
c
c  PURPOSE:  Sets up a Sod shock tube problem in the z-direction.
c
c  LOCAL VARIABLES:
c    n0        Number of zones to be initialised.  The namelist is read
c              from 1 until a total of ie-is+1 [je-js+1, ke-ks+1]
c              zones are initialised.
c    d0        input density
c    e0        input specific internal energy ( = e/d )
c    v10       input velocity in 1 direction
c    v20       input velocity in 2 direction
c    v30       input velocity in 3 direction
c
c  EXTERNALS:
c    BNDYALL
c    BSETMAG
c
c-----------------------------------------------------------------------
c
       implicit NONE
#include "param.h"
#include "field.h"
#include "bndry.h"
#include "grid.h"
#include "root.h"
#include "scratch.h"
#include "mod_names.h"
#include "mpicom.h"
c
       integer       n0      , i       , j       , k
       REAL          d0      , e0      , v10     , v20     , v30
     1             , x10     , x20     , v11     , v21     , v31
     2             , x30     , x11     , x21     , x31     , drat
     3             , prat    , p0      , p1      , d1      , e1
c
c      External statements
c
c-----------------------------------------------------------------------
c
       namelist / pgen     /
     1               d0      , e0      , v10     , v20     , v30
     2             , v11     , v21     , v31     , p0
     3             , x10     , x20     , x30     , x11     , x21
     4             , x31     , drat    , prat
c
c      Default values
c
       d0      = dfloor
       e0      = 0.0
       p0      = 0.6
       x10     = 0.0
       x20     = 0.0
       x30     = 0.0
       x11     = 0.5
       x21     = 0.0
       x31     = 0.0
       v10     = v1floor
       v20     = v2floor
       v30     = v3floor
       v11     = v1floor
       v21     = v2floor
       v31     = v3floor
       drat    = 1.0
       prat    = 1.0
c
       if (myid .eq. 0) then
         read (1, pgen)
         write (2, pgen)
#ifdef MPI_USED
         buf_in( 1) = x10 
         buf_in( 2) = x20 
         buf_in( 3) = x30 
         buf_in( 4) = x11 
         buf_in( 5) = x21 
         buf_in( 6) = x31 
         buf_in( 7) = drat
         buf_in( 8) = prat
         buf_in( 9) = d0  
         buf_in(10) = p0  
         buf_in(11) = e0  
         buf_in(12) = v10 
         buf_in(13) = v20 
         buf_in(14) = v30 
         buf_in(15) = v11 
         buf_in(16) = v21 
         buf_in(17) = v31 
       endif
       call MPI_BCAST( buf_in, 17, MPI_FLOAT
     &               , 0, comm3d, ierr )
       if (myid .ne. 0) then
         x10  = buf_in( 1)
         x20  = buf_in( 2)
         x30  = buf_in( 3)
         x11  = buf_in( 4)
         x21  = buf_in( 5)
         x31  = buf_in( 6)
         drat = buf_in( 7)
         prat = buf_in( 8)
         d0   = buf_in( 9)
         p0   = buf_in(10)
         e0   = buf_in(11)
         v10  = buf_in(12)
         v20  = buf_in(13)
         v30  = buf_in(14)
         v11  = buf_in(15)
         v21  = buf_in(16)
         v31  = buf_in(17)
#endif /* MPI_USED */
       endif
c
c      Set up atmosphere.
c
       if (e0 .ne. 0.0) p0 = e0 * gamm1
       if (e0 .eq. 0.0) e0 = p0 / gamm1
       do 30 k=ks,ke
         do 20 j=js,je
           do 10 i=is,ie
             d (i,j,k) = d0
             v1(i,j,k) = v10
             v2(i,j,k) = v20
             v3(i,j,k) = v30
#ifndef ISO
             e (i,j,k) = e0
#endif
10         continue
20       continue
30     continue
c
c      Shock tube in the z-direction.
c
       if(x1a(is).ge.x11) then
         d1 = d0 * drat
         p1 = p0 * prat
         e1 = p1 / gamm1
         do 40 k=ks,ke
           do 50 j=js,je
             do 60 i=is,ie
               if(x1a(i).ge.x11) then
                 d (i,j,k) = d1
                 v1(i,j,k) = v11
                 v2(i,j,k) = v21
                 v3(i,j,k) = v31
#ifndef ISO
                 e (i,j,k) = e1
#endif -ISO
               endif
 60          continue
 50        continue
 40      continue
       endif
c     
       return
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////               S H K T U B E               \\\\\\\\\\
c
c=======================================================================
