#include "zeusmp.def"

c JCV 04/05/2006
c subroutine to update boundaries and turn jet off
c based on my subroutine from zeus3d
c Modified version of jetoff to turn jet off slowly
c because about cutoff seems to introduce trouble for conduction

c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                 SLOWOFF                   \\\\\\\\\\
c
c=======================================================================
c

       subroutine slowoff

       implicit NONE

#include "mod_names.h"
#include "param.h"
#include "field.h"
#include "bndry.h"
#include "grid.h"
#include "root.h"
#include "scratch.h"
#include "mpicom.h"
#include "cons.h"

       integer i,j,k

       REAL offtime,offspan,dv1,vsmall
       integer thetamax

       REAL mass,dm,totmass,area
c Scaling factors real/code units
       REAL lscale,tiscale,Tescale,mscale,escale
c Real units kbolt[eV K-1],muu mean moleculare weight
       REAL kbolt,muu,msun

c only want inner edge, but not ghost zones
       i=3
c check to make sure this is a tile that touches inner boundary
c currently I do not have more than 1 tile in x1, but should anyway
c NOTE if there are too many processors (~10), this will no longer work

       muu=1.27
       kbolt=1.4861E-9
       lscale=rcore*1000*cmpc/2
       tiscale=lscale/(cism*1000*100)
       Tescale=mp*muu*((cism*1000*100)**2)/(gamma*kbolt*3.0/5.0)
       mscale=muu*mp*ncent*(lscale**3)
       escale=(lscale**2)*mscale/(tiscale**2)
       mass=0.0
       if (x1b(is) .LE. 1) then
          do k=ks,ke
             do j=js,je
                area=dvl2a(j)*dvl3a(k)
                dm=d(i,j,k)*v1(i,j,k)*area
                if(v1(i,j,k).le.0.0)then
                   mass=mass+dm
                endif
             enddo
          enddo
          mass=-1.0*mass*mscale*secyr/(tiscale*msol)
       endif


       call MPI_ALLREDUCE(mass,totmass,1,MPI_FLOAT,MPI_SUM
     &      ,comm3d,ierr)


       if (x1b(is) .LE. 1) then          
          if(myid.eq.0)then
             write(*,*)"total mass across inner boundary = ",totmass
          endif
       endif

c here is where we lower jet velocity

       offtime=1.0
       offspan=0.1

       dv1=v1jet/(offspan/dt)
       vsmall=v1jet/200

       thetamax=J_ZONES*ntiles(2)/jetsize
       thetamax=thetamax*2.0

       if(time.GT.offtime.AND.time.LE.offtime+offspan)then
          if(myid.eq.0)then
             write(6,"(/'Slowly turning jet off')")
          endif
          if (x1b(is) .LE. 1) then
             if (x2b(js) .LE. 0.1) then
                do j=js-2,jS+thetamax
                   do k=ks-2,ke+2
c                      if(v1iib(j,k,1).ge.dv1)then
                      if(v1iib(j,k,1).ge.vsmall)then
                         v1iib(j,k,1)=v1iib(j,k,1)-dv1
                         v1iib(j,k,2)=v1iib(j,k,2)-dv1
                      endif
                   enddo
                enddo
             endif
          endif
          if(myid.eq.0)then
             write(*,*)'vjet now = ',v1iib(2,2,1)
          endif
       endif

       if (time.GT.offtime+offspan.AND.time.LT.offtime+offspan+dt) then
          if(myid.eq.0)then
c             write(6,"(/'Finished turning jet off')")
             write(6,"(/'Leave jet at very low level')")
          endif
c 1st jet       
          do j=js-2,je+2
             do k=ks-2,ke+2
                niib(j,k)=2
                v1iib(j,k,1)=vsmall
                v1iib(j,k,2)=vsmall
             enddo
          enddo
       endif

       return
       end
