#include "zeusmp.def"
c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////               S R C S T E P               \\\\\\\\\\
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
c=======================================================================
c
       subroutine srcstep
c
c    jms:zeus2d.srcstep <------------------------ source step controller
c                                                          october, 1987
c
c    written by: Jim Stone
c    modified 1: June, 1988 by Jim Stone; incorporated into ZEUS2D
c    modified 2: Spring, 1989 by Jim Stone; rewritten
c    modified 3: February, 1990 by David Clarke; incorporated into
c                ZEUS3D
c    modified 4: July, 1990 by David Clarke; because current densities
c                are not needed to compute source terms in the new CT
c                algorithm (MOCCT), workers can be used to store the
c                momenta, thereby saving redundant computations in STV1
c                and STV2.
c    modified 5: June 1992, by David Clarke; added the total energy
c                option originally designed by Byung-IL Jun.
c    modified 6: RAF, 3/27/96, completely rewritten for ZEUS-MP.
c    modified 7: RAF, 1/2/97, added radiation force, radiation
c                diffusion, and radiation-matter coupling terms.
c    modified 8: RAF, 1/22/97, moved modules into driver routines.
c    modified 9: RAF, 2/18/97, added PF and N-R timestep controllers.
c    modified 10: PSLi, 12/30/99, added subcycle of artificial viscosity.
c
c
c  PURPOSE: Controls the update of velocities (v1, v2, v3) and internal
c  energy (e) from source terms in the equation of motion and energy
c  equations respectively.
c
c  LOCAL VARIABLES:      
c    w3da     scratch 1-momentum denisty
c    w3db     scratch 2-momentum denisty
c    w3dc     scratch 3-momentum denisty
c    w3dd     scratch 1-velocity ; scratch density (pdv)
c    w3de     scratch 2-velocity ; scratch e/d     (pdv)
c    w3df     scratch 3-velocity
c    j1       1-current density
c    j2       2-current density
c    j3       3-current density
c
c  EXTERNALS:
c    AVISC_D  ,  FORCES_D  , PDV_D
c    NR_PF_D  ,  RAD_PF
c
c-----------------------------------------------------------------------
       implicit NONE
#include "param.h"
#include "root.h"
#include "field.h"
#include "grid.h"
#include "bndry.h"
#include "scratch.h"
#include "mpicom.h"
#if defined RAD || defined RAD_EXP
#include "radexp.h"
#include "radiation.h"
#include "opac.h"
#endif /* RAD || RAD_EXP */
#ifdef PROFILING
#ifdef MPE_USED
#else
#include "clocks.h"
#endif /* MPE_USED */
#endif /* PROFILING */
#include "cons.h"
CJH
      REAL t1, t2, t3, t4
#ifdef SUB_AV
         , subdt
#endif /* SUB_AV */
c
       integer i,j,k
#ifdef SUB_AV
     1        ,index
#endif /* SUB_AV */
c
c      External statements
c
       external      forces_d  , avisc_d
#if defined RAD || defined RAD_EXP
#ifdef RAD_EXP
     4             , nr_pf   , rad_pf_d
#else /* RAD_EXP */
     4             , imp_fld_d
#endif /* RAD_EXP */
#else  /* RAD || RAD_EXP */
     3             , pdv
#endif /* RAD || RAD_EXP */
c
c-----------------------------------------------------------------------
c
c FORCES
c
c Thermal and MHD Pressure, Gravity, and Rotational Pseudo-Forces
c (including gravitational point mass)
c
c Routine "forces" updates the three velocity components.  The arrays
c v1, v2, and v3 save the old values, while w3dd, w3de, w3df get the 
c new ones.
c
#if defined RAD || defined RAD_EXP
#ifdef OPACITY_PWR_LAW
       do i = 1, 6
        bvstat(i,1) = 0 ! d
        bvstat(i,2) = 0 ! e
       enddo
       nreq = 0
       nsub = nsub + 1
       call bvald(3,3,0,0,0,0,d)
       call bvale(3,3,0,0,0,0,e)
#endif /* OPACITY_PWR_LAW */
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           is, ie, js, je, ks, ke)
#if defined MPI_USED && defined OPACITY_PWR_LAW
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )
#endif /* MPI_USED && OPACITY_PWR_LAW */
#ifdef ONE_DIMENSIONAL
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           is-2, is-1, js, je, ks, ke)
       call opacity(e, d, gamma, kap, sig, dkapdt, 
     .           ie+1, ie+2, js, je, ks, ke)
#endif /* ONE_DIMENSIONAL */
#ifndef ONE_DIMENSIONAL
#ifdef OPACITY_PWR_LAW
       nreq = 0
       nsub = nsub + 1
       call bvald(0,0,3,3,0,0,d)
       call bvale(0,0,3,3,0,0,e)
#endif /* OPACITY_PWR_LAW */
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           is-2, is-1, js, je, ks, ke)
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           ie+1, ie+2, js, je, ks, ke)
#if defined MPI_USED && defined OPACITY_PWR_LAW
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )
#endif /* MPI_USED && OPACITY_PWR_LAW */
#ifdef TWO_DIMENSIONAL
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           is, ie, js-2, js-1, ks, ke)
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           is, ie, je+1, je+2, ks, ke)
#endif /* TWO_DIMENSIONAL */
#ifndef TWO_DIMENSIONAL
#ifdef OPACITY_PWR_LAW
       nreq = 0
       nsub = nsub + 1
       call bvald(0,0,0,0,3,3,d)
       call bvale(0,0,0,0,3,3,e)
#endif /* OPACITY_PWR_LAW */
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           is, ie, js-2, js-1, ks, ke)
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           is, ie, je+1, je+2, ks, ke)
#if defined MPI_USED && defined OPACITY_PWR_LAW
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )
#endif /* MPI_USED && OPACITY_PWR_LAW */
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           is, ie, js, je, ks-2, ks-1)
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           is, ie, js, je, ke+1, ke+2)
#endif /* NOT TWO_DIMENSIONAL */
#endif /* NOT ONE_DIMENSIONAL */
#endif /* RAD || RAD_EXP */
       call forces_d (v1,v2,v3,w3dd,w3de,w3df)
c
c       subroutine forces_d
c     1 (v1old, v2old, v3old, v1new, v2new, v3new)
c
c.......................................................................
c
c ARTIFICIAL VISCOSITY
c
c  Update velocity and e from the artificial viscosity source terms.
c  We need 1 layer of updated boundary data for the velocity.
c  We use just the "m" layer of d and, unless ISO is defined, e -- they
c  should be up to date already, since we used them in "forces" but did
c  not update them.  We don't need e BVs with no "linear" viscosity.
c
c  Arrays w3dd, w3de, w3df save the old velcoity values, while v1, v2, 
c  v3 get the updated ones.
c
c  The artificial viscosity routine must also compute momentum densities
c  w3da, w3db, w3dc from v1, v2, v3 for use in the transport step.
c
CPS
C  Subcycle of artificial viscosity calculation
C
#ifdef SUB_AV
       index=0
       subdt=dt
       avisc_dt=courno / (SQRT(dtqqi2)+tiny)
       if(dtqqi2.eq.0.0) avisc_dt=dt
#ifdef MPI_USED
          buf_in(1) = avisc_dt
          call MPI_ALLREDUCE( buf_in(1), buf_out(1), 1
     &                      , MPI_2DOUBLE_PRECISION
     &                      , MPI_MINLOC, comm3d, ierr)
          avisc_dt  =   buf_out(1)
#endif
       DO WHILE (subdt .GT. 0.0)
          IF(subdt.GT.avisc_dt) THEN
             subdt=subdt-avisc_dt
          ELSE
             avisc_dt=subdt
             subdt=-1.0
          ENDIF
          index=index+1
          IF(mod(index,2).GT.0) THEN
             call avisc_d (w3dd,w3de,w3df,v1,v2,v3,w3da,w3db,w3dc)
          ELSE
             call avisc_d (v1,v2,v3,w3dd,w3de,w3df,w3da,w3db,w3dc)
          ENDIF
          avisc_dt=courno / (SQRT(dtqqi2)+tiny)
#ifdef MPI_USED
          buf_in(1) = avisc_dt
          call MPI_ALLREDUCE( buf_in(1), buf_out(1), 1
     &                      , MPI_2DOUBLE_PRECISION
     &                      , MPI_MINLOC, comm3d, ierr)
          avisc_dt  =   buf_out(1)
#endif
       ENDDO
C
C  Update velocity arrays if necessary.
C
       IF(mod(index,2).EQ.0) THEN
          DO k=ks,ke
             DO j=js,je
                DO i=is,ie
                   v1(i,j,k)=w3dd(i,j,k)
                   v2(i,j,k)=w3de(i,j,k)
                   v3(i,j,k)=w3df(i,j,k)
                ENDDO
             ENDDO
          ENDDO
       ENDIF
#else
       call avisc_d (w3dd,w3de,w3df,v1,v2,v3,w3da,w3db,w3dc)

#endif /* SUB_AV */
c
c      subroutine avisc_d
c     1            (v1old,v2old,v3old,v1new,v2new,v3new,s1,s2,s3)
c
c......................................................................
#if defined RAD || defined RAD_EXP
c
c RADIATION ENERGY UPDATES
c
#ifdef RAD_EXP
c
c Unconditionally stable explicit algorithm based on the Product
c Formula code supplied by Frank Graziani at LLNL.
c
c I) RADIATION DIFFUSION
c
c    Use Product Formula Method to perform explicit radiation diffusion.
c    This will put the updated radiation energy density into w3dh.
c    The Courant limit radiation diffusion time step (actually 1/dtrd) 
c    will be computed in rad_pf_l and stored in dtrdi2 (/radexp/).  
c    We will do a global min on this in nudt.
c
c
       call rad_pf_d (er, w3dh)
c
c       subroutine rad_pf_d
c     1                   (erold, ernew)
c
#ifndef NO_DIFFUSION
       if (rcourno .eq. 0.) then
c
c Use an accuracy criterion to compute the next radiation diffusion time
c step, rather than the Courant limit.
c
         call rad_pf_dt (er, w3dh)
       else
c
c Courant limit
c
         dtrdi2 = (dtrdi2/rcourno)**2
       endif
#endif /* NO_DIFFUSION */
c
c II) RADIATION-MATTER COUPLING
c
c    Apply Newton-Raphson method to implicitly solve 2 by 2 systems for 
c    the new gas and radiation energy densities in each zone.
c    This will put the updated radiation energy into er (actually E/d)
c    and           the updated       gas energy into w3de (really e/d).
c
       call nr_pf_d (w3dd,e,w3de,w3dh,er)
c
c       subroutine nr_pf_d
c     1       (dcopy,eold,eod,erold,erod)
c
#ifndef NO_COUPLING
       call rad_nr_dt (w3dh, er)
#endif /* not NO_COUPLING */

#else /* RAD_EXP */
c
c Fully implicit flux-limited diffusion radiation driver goes here.
c
#ifdef IMP_COUP
       call imp_fld_d(w3dd,e,w3de,w3dh,er)
c
c     subroutine imp_fld_d(dcopy,eold,eod,erold,erod)
c
       call rad_imp_dt(w3dh,er)
#endif /* IMP_COUP */
#ifdef IMP_SPLIT
       nreq = 0
       nsub = nsub + 1
       call bvald(3,3,0,0,0,0,d)
       call bvale(3,3,0,0,0,0,e)
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           is, ie, js, je, ks, ke)
#ifdef MPI_USED
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )
#endif /* MPI_USED */
#ifdef ONE_DIMENSIONAL
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           is-2, is-1, js, je, ks, ke)
       call opacity(e, d, gamma, kap, sig, dkapdt, 
     .           ie+1, ie+2, js, je, ks, ke)
#endif /* ONE_DIMENSIONAL */
#ifndef ONE_DIMENSIONAL
       nreq = 0
       nsub = nsub + 1
       call bvald(0,0,3,3,0,0,d)
       call bvale(0,0,3,3,0,0,e)
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           is-2, is-1, js, je, ks, ke)
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           ie+1, ie+2, js, je, ks, ke)
#ifdef MPI_USED
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )
#endif /* MPI_USED */
#ifdef TWO_DIMENSIONAL
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           is, ie, js-2, js-1, ks, ke)
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           is, ie, je+1, je+2, ks, ke)
#endif /* TWO_DIMENSIONAL */
#ifndef TWO_DIMENSIONAL
       nreq = 0
       nsub = nsub + 1
       call bvald(0,0,0,0,3,3,d)
       call bvale(0,0,0,0,3,3,e)
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           is, ie, js-2, js-1, ks, ke)
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           is, ie, je+1, je+2, ks, ke)
#ifdef MPI_USED
       if (nreq .ne. 0) call MPI_WAITALL ( nreq, req, stat, ierr )
#endif /* MPI_USED */
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           is, ie, js, je, ks-2, ks-1)
       call opacity(e, d, gamma, kap, sig, dkapdt,
     .           is, ie, js, je, ke+1, ke+2)
#endif /* NOT TWO_DIMENSIONAL */
#endif /* NOT ONE_DIMENSIONAL */
c
       call imp_ddf_d(er,w3dh)
c
c     subroutine imp_ddf_d(erold,ernew)
c
      call rad_imp_dt(er,w3dh)
c
c     subroutine rad_imp_dt(erold, ernew)
c
       call nr_pf_d (w3dd,e,w3de,w3dh,er)
c
c       subroutine nr_pf_d
c     1       (dcopy,eold,eod,erold,erod)
c
#endif /* IMP_SPLIT */
#endif /* RAD_EXP */

#else /* RAD || RAD_EXP */

#ifdef RAD_LOSS
       call rad_loss
#endif /* RAD_LOSS */

#ifdef T_CONDUCT
       call tconduct
#endif /* T_CONDUCT */

#ifdef PHYS_VISC
       call kin_visc
#endif /* PHYS_VISC */

#ifndef ISO
c
c No radiation, not isothermal.
c
c  COMPRESSIONAL WORK TERM (PdV).
c
c  Finally, update the energy with the PdV source term
c  only if the internal energy equation is being solved.
c
c  We need just 1 "p" layer of updated boundary data for v1, v2, and v3,
c  but none for d and e.  
c
c  Routine pdv also saves the density and e/d in (w3dd,w3de) for use
c  in the transport step.  This is why it is being called even when
c  TOTAL_ENERGY is defined.
c
       call pdv_d (w3dd ,w3de )
c
c       subroutine pdv_d
c     1            (dcopy, eod)
c

#endif /* not ISO */

#endif /* RAD_EXP */
       return
       end
c
c=======================================================================
c
c    \\\\\\\\\\        E N D   S U B R O U T I N E        //////////
c    //////////               S R C S T E P               \\\\\\\\\\
c
c=======================================================================
c
