#include "zeusmp.def"
c
c     JCV 5/18/04
c     subroutine to calculate thermal conduction
c
c
      subroutine tconduct
      implicit NONE
#ifdef T_CONDUCT
c
c     standard includes
#include "param.h"
#include "root.h"
#include "field.h"
#include "grid.h"
#include "scratch.h"
#include "gravity.h"
#include "mpicom.h"
#include "cons.h"
#include "mod_names.h"

      integer i,j,k
      REAL lscale,tiscale,Tescale,mscale,escale
      REAL de
      REAL kcond(in,jn,kn)
      REAL oldT(in,jn,kn)
      REAL nonspitz
      REAL kbolt,muu
      integer tflag
      REAL dtcond,condt
      common / tcondr / dtcond

      integer power
      REAL Tmax

c     External statements
      REAL kappa,conduction
      external kappa,conduction

      character*15 filename
      write(filename,"(a4,3i2.2,'.','hdf')") 'last',coords(1)
     &     ,coords(2),coords(3)

      dtcond=1000.0

      Tmax=1000.0
      power=4

      nonspitz=0.1

      muu=1.27
      kbolt=1.4861E-9

      lscale=rcore*1000*cmpc/2
      tiscale=lscale/(cism*1000*100)
c     0.11605 is conversion from Kev to 10E7 Kelvin
      Tescale=(mp*muu*((cism*1000*100)**2)/
     &     (gamma*kbolt*3.0/5.0))*0.11605
      mscale=muu*mp*ncent*(lscale**3)
      escale=(lscale**2)*mscale/(tiscale**2)

      tflag=0

      do k=ks,ke
         do j=js,je
            do i=is,ie
               oldT(i,j,k)=e(i,j,k)/d(i,j,k)
               kcond(i,j,k)=kappa(i,j,k,nonspitz,Tescale,tiscale,
     &              lscale,escale,oldT)
     &              *((Tmax**power)/
     &              ((Tmax**power)+(e(i,j,k)/d(i,j,k))**power))
            enddo
         enddo
      enddo

      do k=ks+2,ke-2
         do j=js+2,je-2
            do i=is+2,ie-2
               if(e(i,j,k).le.0.0)then
                  write(*,*)'Negative energy before conduction
     &                 at',i,j,k,myid
                  call hdfall(filename)
                  call MPI_FINALIZE
                  stop
               endif

               de=conduction(i,j,k,kcond,oldT)
               e(i,j,k)=e(i,j,k)+de*dt

               condt=0.1*d(i,j,k)*(min(dx1b(i),dx2b(j),
     &              dx3b(k)))**2.0/(nonspitz*kcond(i,j,k))
               if(dtcond .GE. condt) then
                  dtcond = condt
                  tflag=1
               endif

               if(e(i,j,k).le.0.0)then
                  write(*,*)'Negative energy after conduction 
     &                 at',i,j,k,myid
                  call hdfall(filename)
                  call MPI_FINALIZE
                  stop
               endif
               
            enddo
         enddo
      enddo


         if (tflag.GT.0)then
            if(myid.eq.0)then
               write(2,*)'Using conduction time step at t= ',time
            endif
         endif

#endif /* T_CONDUCT */
      return
      end

#include "zeusmp.def"
#ifdef T_CONDUCT

c===============================================
c
c     FUNCTION kappa
c
c===============================================

      REAL function kappa(i,j,k,nonspitz,Tescale,tiscale,
     &lscale,escale,oldT)

#include "param.h"
#include "field.h"

      integer i,j,k
      REAL nonspitz,Tescale,tiscale,lscale,escale
      REAL lcoulomb
      REAL oldT(in,jn,kn)

      lcoulomb=37

c      lcoulomb=32.0+log((d(i,j,k)**-0.5)*
c     &     Tescale*(e(i,j,k)/d(i,j,k))*0.11605)

      kappa=(nonspitz*1.84E-5*(Tescale*
c     &     (oldT(i,j,k)/d(i,j,k))**(5.0/2.0))/lcoulomb)*
     &     oldT(i,j,k)**(5.0/2.0))/lcoulomb)*
     &     tiscale*Tescale*(lscale**2.0)/escale

      kappa=4.95E-6
c      kappa=0.0
c      write(*,*)'kappa= ',kappa

      return
      end

c===============================================
c
c     FUNCTION conduction
c
c===============================================

      REAL function conduction(i,j,k,kcond,oldT)

#include "param.h"
#include "field.h"
#include "grid.h"

      integer i,j,k,h1
      REAL kcond(in,jn,kn)
      REAL oldT(in,jn,kn)
      h1=1

      conduction=
     &     (1.0/(g2a(i)*g31a(i)*g32a(j)))*
     &     (
     &     (1.0/(2.0*dx1a(i)))*
     &     (
     &     (g2a(i+1)*g31a(i+1)*g32a(j)*
     &     ((kcond(i+2,j,k)+kcond(i+1,j,k))/2.0)*
     &     (oldT(i+2,j,k)-oldT(i,j,k))/(2.0*dx1b(i+1)))
     &     -
     &     (g2a(i-1)*g31a(i-1)*g32a(j)*
     &     ((kcond(i-1,j,k)+kcond(i-2,j,k))/2.0)*
     &     (oldT(i,j,k)-oldT(i-2,j,k))/(2.0*dx1b(i)))
     &     )
     &     +
     &     (1.0/(2.0*g2b(i)*dx2a(j)))*
     &     (
     &     (g31a(i)*g32a(j+1)*
     &     ((kcond(i,j+2,k)+kcond(i,j+1,k))/2.0)*
     &     (oldT(i,j+2,k)-oldT(i,j,k))/(2.0*dx2b(j+1)))
     &     -
     &     (g31a(i)*g32a(j-1)*
     &     ((kcond(i,j-1,k)+kcond(i,j-2,k))/2.0)*
     &     (oldT(i,j,k)-oldT(i,j-2,k))/(2.0*dx2b(j)))
     &     )
     &     +
     &     (1.0/(2.0*g32b(i)*g32b(j)*dx3a(k)))*
     &     (
     &     (g2a(i)*
     &     ((kcond(i,j,k+2)+kcond(i,j,k+1))/2.0)*
     &     (oldT(i,j,k+2)-oldT(i,j,k))/(2.0*dx3b(k+1)))
     &     -
     &     (g2a(i)*
     &     ((kcond(i,j,k-1)+kcond(i,j,k-2))/2.0)*
     &     (oldT(i,j,k)-oldT(i,j,k-2))/(2.0*dx3b(k)))
     &     )
     &     )

      return
      end

#endif /* T_CONDUCT */
