#include "zeusmp.def"
c=======================================================================
c/////////////////////////  EXAMPLE USERDUMP  \\\\\\\\\\\\\\\\\\\\\\\\\\
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
      subroutine textdmp
c
c  written by:   Robert Fiedler
c  Last modified: Feb. 18, 1997
c
c  PURPOSE:  Example USERDUMP routine to generate ASCII output
c            files during a ZEUS-MP run.
c
c  USAGE:  To use this routine, include "-u textdmp" on the
c          Make_zeusmp command line.
c
c  DESCRIPTION:  This example USERDUMP routine prints out the
c                coordinates and the density, energy density,
c                and velocity components for a 3-D
c                hydrodynamics simulation.  It must be modified
c                to list magnetic field components, 
c                gravitational potential, radiation energy density, 
c                etc.  
c
c-----------------------------------------------------------------------
      implicit NONE
#include "param.h"
#include "field.h"
#include "grid.h"
#include "root.h"
#include "cons.h"
#ifdef RAD_EXP
#include "scratch.h"
#include "radexp.h"
       integer niter
       data niter / 1 999 /
#endif /* RAD_EXP */
       integer  i, j, k
#if defined GRAV || defined GRAV_FFT
       REAL mass
       REAL xgp(in,jn,kn)
       common /tdmp/ mass, xgp
#endif /* GRAV || GRAV_FFT */
#ifdef SHELLS
      REAL ocoef  (5), ofac(in,jn,kn)
      common /shellkap/ ocoef, ofac
#endif /* SHELLS */
c\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\///////////////////////////////////
c=======================================================================
c
c Write out the coordinates and the density for each zone in the
c computational domain to unit 12 (usrfile).
c
      open(12,file=usrfile)
      write(12,"('ZEUS-MP field variables at time = ',1pe15.8)") time
      write(12,"(a)")
12    format(1p5e16.8)
13    format(3i4,1p6e16.8)
c12    format(1p6e12.4)
#ifdef RAD
#ifdef ONE_DIMENSIONAL
c       write(12,"('         x1b             x2b             x3b',
c    1             '           gas e     radiation e')")
c       write(12,12) ( ( ( x1b(i), x2b(j), x3b(k), e(i,j,k),
c    .                 er(i,j,k)
c    1               , i=is,ie ), j=js,je ), k=ks,ke)
        write(12,"('         x1b             x2b             x3b',
     1             '           gas T     radiation T')")
        write(12,12) ( ( ( x1b(i), x2b(j), x3b(k),
     .                 mmw*mh*gamm1*e(i,j,k)/(d(i,j,k)*1000*everg),
     .                 boltz*sqrt(sqrt(er(i,j,k)/7.56d-15))/(1000*everg)
     1               , i=is,ie ), j=js,je ), k=ks,ke)
#else /* ONE_DIMENSIONAL */
c       write(12,"('         x1b             x2b             x3b',
c    1             '           gas e     radiation e')")
c       write(12,12) ( ( ( x1b(i), x2b(j), x3b(k), e(i,j,k),
c    .                 er(i,j,k)
c    1               , i=is,ie ), j=js,je ), k=ks,ke)
        write(12,"('         x1b             x2b             x3b',
     1             '           gas T     radiation T')")
        write(12,12) ( ( ( x1b(i), x2b(j), x3b(k),
     .                 mmw*mh*gamm1*e(i,j,k)/(d(i,j,k)*1000*everg),
     .                 boltz*sqrt(sqrt(er(i,j,k)/7.56d-15))/(1000*everg)
     1               , i=is,ie ), j=js,je ), k=ks,ke)
c      write(12,"('         x1b             x2b             x3b',
c     1           '         density    ',' temperature')")
c      write(12,12) ( ( ( x1b(i), x2b(j), x3b(k), d(i,j,k)
c     2             , sqrt(sqrt(er(i,j,k)))
c     1             , i=is,ie ), j=js,je ), k=ks,ke)
      write(12,"(a)")
      write(12,"('         x1a             x2b             x3b',
     1 '          v1')")
      write(12,22) ( ( ( x1a(i), x2b(j), x3b(k), v1(i,j,k),
     1                 i=is,ie ), j=js,je ), k=ks,ke )
22    format(1p4e16.8)
      write(12,"(a)")
      write(12,"('         x1b             x2a             x3b',
     1 '          v2')")
      write(12,32) ( ( ( x1b(i), x2a(j), x3b(k), v2(i,j,k),
     1                 i=is,ie ), j=js,je ), k=ks,ke )
32    format(1p4e16.8)
      write(12,"(a)")
      write(12,"('         x1b             x2b             x3a',
     1 '          v3')")
      write(12,42) ( ( ( x1b(i), x2b(j), x3a(k), v3(i,j,k),
     1                 i=is,ie ), j=js,je ), k=ks,ke )
42    format(1p4e16.8)
#endif /* ONE_DIMENSIONAL */
#endif /* RAD */
#ifndef RAD
#ifndef RAD_EXP
#ifndef GRAV
      write(12,"('         x1b             x2b             x3b',
     1           '         density       ener dens')")
      write(12,12) ( ( ( x1b(i), x2b(j), x3b(k), d (i,j,k), e (i,j,k)
     1           , i=is,ie ), j=js,je ), k=ks,ke)
#else /* GRAV */
cps      write(12,"('         x1b             x2b             x3b',
c     1           '        exact gp       grav pot ')")
c      write(12,12) ( ( ( x1b(i), x2b(j), x3b(k), xgp(i,j,k), gp(i,j,k)
c     1           , i=is,ie ), j=js,je ), k=ks,ke)
#endif /* GRAV */
      write(12,"(a)")
      write(12,"('         x1b             x2b             x3b',
     1           '         density    ')")
      write(12,14) ( ( ( x1b(i), x2b(j), x3b(k), d(i,j,k)
     1             , i=is,ie ), j=js,je ), k=ks,ke)
      write(12,"(a)")
 14   format(1p4e16.8)
c      write(12,"('         x1a             x2b             x3b',
c     1 '          v1')")
c      write(12,22) ( ( ( x1a(i), x2b(j), x3b(k), v1(i,j,k), 
c     1                 i=is,ie ), j=js,je ), k=ks,ke )
c22    format(1p4e16.8)
c      write(12,"(a)")
c      write(12,"('         x1b             x2a             x3b',
c     1 '          v2')")
c      write(12,32) ( ( ( x1b(i), x2a(j), x3b(k), v2(i,j,k),
c     1                 i=is,ie ), j=js,je ), k=ks,ke )
c32    format(1p4e16.8)
c      write(12,"(a)")
c      write(12,"('         x1b             x2b             x3a',
c     1 '          v3')")
c      write(12,42) ( ( ( x1b(i), x2b(j), x3a(k), v3(i,j,k),
c     1                 i=is,ie ), j=js,je ), k=ks,ke )
cps42    format(1p4e16.8)
#else /* RAD_EXP */
#ifdef ONE_DIMENSIONAL
#if defined NO_FORCES && defined NO_TRANSPORT && defined ONE_DIMENSIONAL && defined NO_COUPLING
c     if (xnu.eq.0. .and. powr.eq.0.) then
c       call xact (w1da,niter)
c       write(12,"('         x1b             x2b             x3b',
c    1             '         series ',i4.4,' temperature')") niter
c       write(12,12) ( ( ( x1b(i), x2b(j), x3b(k), w1da(i), er(i,j,k)
c    1               , i=is,ie ), j=js,je ), k=ks,ke)
c     else
c       call marshak (w1da)
c       write(12,"('         x1b             x2b             x3b',
c    1             '         marshak    ',' temperature')")
c       write(12,12) ( ( ( x1b(i), x2b(j), x3b(k), w1da(i)
c    2               , sqrt(sqrt(er(i,j,k)))
c    1               , i=is,ie ), j=js,je ), k=ks,ke)
c     endif
        write(12,"('         x1b             x2b             x3b',
     1             '           gas e     radiation e')")
        write(12,12) ( ( ( x1b(i), x2b(j), x3b(k), e(i,j,k), er(i,j,k)
     1               , i=is,ie ), j=js,je ), k=ks,ke)
#else /* NO_FORCES && NO_TRANSPORT && ONE_DIMENSIONAL && NO_COUPLING */
        write(12,"('         x1b             x2b             x3b',
     1             '           gas e     radiation e')") 
        write(12,12) ( ( ( x1b(i), x2b(j), x3b(k), e(i,j,k), er(i,j,k)
     1               , i=is,ie ), j=js,je ), k=ks,ke)
#endif /* NO_FORCES && NO_TRANSPORT && ONE_DIMENSIONAL && NO_COUPLING */
#else /* ONE_DIMENSIONAL */
c     write(12,"(12x,'         x1b             x2b             x3b',
c    1           '         density    ','     er     ')")
c     write(12,"(12x,'      x1b       ','      x2b       ',
c    .               '      x3b       ',
c    1           '     density    ','   gas energy   ',
c    .           '   rad energy   ')")
c     write(12,13) ( ( ( i,j,k,x1b(i), x2b(j), x3b(k), d(i,j,k)
c    2             , e(i,j,k), er(i,j,k)
c    1             , i=is-2,ie+2 ), j=js,je ), k=ks,ke)
        write(12,"('         x1b             x2b             x3b',
     1             '           gas T     radiation T')")
        write(12,12) ( ( ( x1b(i), x2b(j), x3b(k),
     .                 mmw*mh*gamm1*e(i,j,k)/(d(i,j,k)*1000*everg),
     .                 boltz*sqrt(sqrt(er(i,j,k)/7.56d-15))/(1000*everg)
     1               , i=is,ie ), j=js,je ), k=ks,ke)
#endif /* ONE_DIMENSIONAL */
#endif /* RAD_EXP */
#endif /* RAD */
c
      close(12)
c
c Make a note of this dump in the log file.
c
c      write(2,"(/1x,'user dump written at time=',1pe12.5,' cycle='
c     & ,i6)") time,nhy
c
      end
#include "zeusmp.def"
c-----------------------------------------------------------------------
      subroutine marshak
#ifdef RAD_EXP
     1 (soln)
c
c
      implicit NONE
#include "param.h"
#include "field.h"
#include "bndry.h"
#include "grid.h"
#include "root.h"
#include "radexp.h"
        integer i
        REAL    soln(ijkn)
        REAL    p, const, xi, t0, xj, pp
c
        const = (rho0/d(is,js,ks))**xnu / (3.0 * t_0**powr)
        t0 = sqrt(sqrt(eriib(js,ks,1)))
c
        p  = 1.0/powr
        pp = 1.0/(powr+4)
c        xi = 2.0 * const * t0**powr * p**2 * (powr+4) * 4 * time 
c From PF code
        xi = 2.0 * const * t0**p * time 
        xi = sqrt(xi)
c
        if (xi .lt. x1b(ie+1)) then
          do i=is,ie
            xj = 1.0 - x1b(i)/max(xi,tiny)
            if (xj .gt. 0.0) then
              soln(i) = t0 * xj**p
            else
              soln(i) = 0.0
            endif
          enddo ! i
        else
          do i=is,ie
            soln(i) = t0 * ( 1.0 - ( x1b(i   ) - x1b(is-1) )
     1                   /         ( x1b(ie+1) - x1b(is-1) ) )**pp
          enddo ! i
        endif
#endif /* RAD_EXP */
      return
      end
#include "zeusmp.def"
c-----------------------------------------------------------------------
      subroutine xact
#ifdef RAD_EXP
     1 (soln,niter)
c
c Exact solution to 1-D heat diffusion problem.  See
c Ritger and Rose, "Differential Equations with Applications", p 463,
c Problem 2.
c
      implicit NONE
#include "param.h"
#include "bndry.h"
#include "grid.h"
#include "root.h"
        integer i, j, niter
        REAL    a, bfinal, b1, b2, b3, xd, xl
        REAL    soln(ijkn)
c
      do i = is, ie
        xd = x1b(i)    - x1b(is-1)
        xl = x1b(ie+1) - x1b(is-1)
        a = eriib(js,ks,1) + ( eroib(js,ks,1) - eriib(js,ks,1) ) * xd/xl
        bfinal = 0.
c
        do j = 1, niter    
          b1 = ( (-1)**j * eroib(js,ks,1) - eriib(js,ks,1) ) / real(j)
          b2 = b1 * exp( -j**2 * pi**2 * time / xl**2 )
          b3 = b2 * sin( j * pi * xd/xl )
          bfinal = bfinal + b3
        enddo
c
        bfinal = (2.0/pi) * bfinal
        soln(i) = a + bfinal
c
      enddo
#endif /* RAD_EXP */
      return
      end
