#include "zeusmp.def"

C BMcK 01/16/04

c=======================================================================
c
c    \\\\\\\\\\      B E G I N   S U B R O U T I N E      //////////
c    //////////                   TORUS                   \\\\\\\\\\
c
c
c=======================================================================
c
       subroutine torus
#ifdef MHD
c      v1 BMcK 05/13/04 No MHD. No viscosity. Just set up torus geometry and 
c                       undisturbed atmosphere a la 
c                       Hawley & Krolik (2001) and 
c                       cylindrical (ZRP) co-ords. 
c                       G=1,ptmass=1, pt mass located at origin.       
c                                     
               

       implicit NONE
#include "mod_names.h"
#include "param.h"
#include "field.h"
#include "bndry.h"
#include "grid.h"
#include "root.h"
#include "scratch.h"
#include "mpicom.h"

       integer i,j,k
       integer nintq,nrayq,rintq,rrayq,imid


       REAL r0,rho0
       REAL n,rho_max
       REAL gamma1,q1,r01
       REAL rad,r_in
       REAL rad1,q,Asqd,C
       REAL z_init,z_init1
       REAL beta
       REAL d_min,d_floor
       REAL d_cntr(in+4,jn+4,kn+4)
       REAL a_phi(in+4,jn+4,kn+4)
       common /output/nintq,nrayq,rintq,rrayq

c removed unused bndyall from external jcv
       external      bndyflgs

       r0=10.0
       rho0=1.0e-5
       rho_max=34.
       n=1.0/(gamma-1.0)
       r_in=4.0
       q=1.68
       beta=6.0e4
       d_min=0.3*rho_max

       r01=(r0-1.0)
       q1=2.0*(q-1.0)
       gamma1=(gamma-1.0)/gamma

c      special output integers
       
       nintq=100
       nrayq=100
       rintq=0
       rrayq=0

       C=-(1.0/(r_in-1.0))
     &      +(
     &        (1.0/q1)
     &        *(r0**(-1.0+2.0*q))
     &        /((r01**2.0)*(r_in**q1))
     &       ) 

       Asqd=gamma1
     &       *(rho_max**(1.0-gamma))
     &        *(
     &          C
     &          +(1.0/r01)
     &          -(
     &            (1.0/q1)
     &            *r0**(-1.0+2.0*q)
     &            /((r01**2.0)*(r0**(q1)))
     &           )
     &          )

c       write (iotty, 2011)
c       write (iolog, 2011)
c2011   format('MYPROB  : Initialisation commencing')
       if(myid.eq.0)then
          write(6,"(10x,'MYPROB  : Initialisation commencing')")
          write(*,*)gamma
       endif

      do k=ks-2,ke+2
         do j=js-2,je+2
            do i=is-2,ie+2

             rad=sqrt((x1b(i)**2.0)+(x2b(j)**2.0))

c     undisturbed atmosphere
c     ======================

c      In ZRP co-ords, x1b=z,x2b=R, x3b=Phi
c      b in above denotes 'b' grid, ie co-ords centered on face(s) of cell
c      as opposed to 'a' grid which has co-ords centered in middle of cell
c      Radial vector in Cylindrical co-ords is
c      Rad = sqrt(R^2+z^2) =sqrt(x2b^2+x1b^2) in ZRP ZEUS co-ords       

             d(i,j,k)=rho0
             e(i,j,k)=d(i,j,k)
     &               /((rad-1.0)*(gamma-1.0))

             v1(i,j,k)=0.0
             v2(i,j,k)=0.0
             v3(i,j,k)=0.0
             b1(i,j,k)=0.0
             b2(i,j,k)=0.0
             b3(i,j,k)=0.0

c     Thin torus
c     ==========

             rad1=(rad-1.0)               
             z_init1=(-C
     &                +(
     &                  (r0**(-1.0+2.0*q))
     &                 /(q1*(r01**2)*(x2b(j)**q1))
     &                 )
     &                )
    
             z_init=(1.0
     &                +(1.0/z_init1)
     &               )

             if (x2b(j) .ge. r_in .and.
     &           x1b(i)**2.0 .le. 
     &           (z_init**2.0-x2b(j)**2.0)) then

                 d(i,j,k)=(
     &                     1.0/((n+1)*Asqd)
     &                     *(
     &                       -z_init1
     &                       +(1.0/rad1) 
     &                       )
     &                     )**n
                 e(i,j,k)=Asqd*(d(i,j,k)**gamma)/(gamma-1.0)

                 v1(i,j,k)=0.0                 
                 v2(i,j,k)=0.0                
                 v3(i,j,k)=(x2b(j)/r0)**(1.0-q)*sqrt(r0)/r01

c      B-field setup for torus
c      Poloidal field:
c      B-field A_phi is non-zero, A_r,z=0
c      From Hawley and Krolik(HK01): Pick A=(rho(r,z,phi)-rho_min)^2
c        where rho_min=5.e-4 so
c      A_phi=d(i,j,k)-d_min where d>d_min else A_phi=0
c      explicit differencing of A for B_z,r components, B_phi=0

c      Want a_phi at node centers, by averaging 
c      d(i,j),d(i-1,j), d(i,j-1) and d(i-1,j-1) 
c      d is face-centered.
                  d_cntr(i,j,k)=(d(i,j,k)
     &                           +d(i-1,j,k)
     &                           +d(i,j-1,k)
     &                           +d(i-1,j-1,k)      
     &                               )/4.0               
                  if (d_cntr(i,j,k) .gt. d_min) then
                      a_phi(i,j,k)=(
     &                            (1.0/beta) 
     &                         *(
     &                           (d_cntr(i,j,k)-d_min)**2.0)
     &                          )
                  endif
             endif    
            enddo 
         enddo
      enddo

      do k=ks-2,ke+2
         do j=js-2,je+2
            do i=is-2,ie+2

                 b1(i,j,k)=(
     &                     1/(x2b(j)*dx2a(j))  
     &                     *(
     &                       ( 
     &                        x2a(j+1)*a_phi(i,j+1,k)
     &                        -(
     &                          x2a(j)*a_phi(i,j,k)
     &                         )
     &                        )
     &                      )
     &                     )
                 b2(i,j,k)=-(
     &                     1/(x2b(j)*dx1a(i))  
     &                     *(
     &                       ( 
     &                        x2a(j)*a_phi(i+1,j,k)
     &                        -(
     &                          x2a(j)*a_phi(i,j,k)
     &                          )
     &                        )
     &                       )
     &                     ) 
                 b3(i,j,k)=0.0

            enddo
         enddo
      enddo

       call bndyflgs


c---All done

c       write (iotty, 2010)
c       write (iolog, 2010)
c2010   format('MYPROB  : Initialisation complete.')
c
#endif /* MHD */
       return
       end
