#include "zeusmp.def"

      subroutine toruswrite

#include "cons.h"
#include "mod_names.h"
#include "options.h"
#include "param.h"
#include "field.h"
#include "grid.h"
#include "root.h"
#include "scratch.h"
#include "mpicom.h"
#include "mod_names.h"
#include "clocks.h"
#include "impsoln.h"
#include "gravity.h"
#include "radiation.h"
#include "bndry.h"

      integer i,j,k,nintq,nrayq,rintq,rrayq
      integer fileex
      REAL mdotin,mdotout,mdotvert
      REAL intstress,bremms,ohmic
      REAL totmdotin,totmdotout,totmdotvert
      REAL totintstress,totbremms,totohmic
      common /output/nintq,nrayq,rintq,rrayq


       REAL d_floor,e_floor
c      02/19/04 BMcK
c      Change d_floor to 1.e-4 from 1.e-5 to make calc. run quicker
c      Prevent numerical vacuum by setting a floor on the density (1.e-5)
c      11/18/04 Put e_floor into code at 1.e-7 to make calc. run quicker.  
c      NOTE: FOUR HARDWIRED ASSUMPTIONS IN THIS CODE. CHECK zmp_inp
c      AND torus.F to see what needs to be changed.

c     11/29/04 Note: A density floor introduces matter into the 
c              simulation. Conservation of Angular momentum requires
c              that this matter be introduced with *zero* velocity.
      d_floor=1.e-5
      e_floor=1.e-7
      do k=ks-2,ke+2
         do j=js-2,je+2
            do i=is-2,ie+2
               if (d(i,j,k) .le. d_floor) then
                   d(i,j,k)=d_floor
c     Removed by Barry in recent version 3/30/05
c                   v1(i,j,k)=0.0
c                   v2(i,j,k)=0.0
c                   v3(i,j,k)=0.0
               endif
               if (e(i,j,k) .le. e_floor) then
                   e(i,j,k)=e_floor
               endif
            enddo
         enddo
      enddo

      rintq=rintq+1
      if (rintq .eq. nintq) then
        rintq=0
        mdotin=0.0
        mdotout=0.0
        mdotvert=0.0
        intstress=0.0
        bremms=0.0
        ohmic=0.0
c       ASSUMPTION 1: phi= 0-> pi/2
c       IF only covering 0->2pi, then *4.0 throughout
        do k=ks-2,ke+2
           do i=is-2,ie+2
c       INTEGRATE OVER processors at inner boundary ONLY for mdotin  
             if (x2b(js) .lt. 4.0) then
               mdotin= mdotin + 
     &             4.0*dx1b(i)*dx3b(k)*x2b(js)*d(i,js,k)*(-v2(i,js,k))
             else
               mdotin= mdotin + 0.0
             endif 
c       ASSUMPTION 2: Outer boundary=21.5 
c       INTEGRATE OVER processors at outer bd. ONLY for mdotout
             if (x2b(je) .gt. 20.0) then
               mdotout= mdotout + 
     &             4.0*dx1b(i)*dx3b(k)*x2b(je)*d(i,je,k)*v2(i,je,k)
             else
               mdotout= mdotout + 0.0
             endif 
           enddo
        enddo
        do k=ks-2,ke+2
           do j=js-2,je+2
c       ASSUMPTION 3: Vertical boundary=+/- 5.0 
c       INTEGRATE OVER processors at outer bd. ONLY for mdotvert
             if (x1b(ie) .lt. -2.0) then
               mdotvert= mdotvert + 
     &             4.0*dx2b(j)*dx3b(k)*x2b(j)*
     &             (
     &              x1b(ie)*d(ie,j,k)*v1(ie,j,k)
     &             )
             else
               mdotvert= mdotvert +
     &             4.0*dx2b(j)*dx3b(k)*x2b(j)*
     &             (
     &             -x1b(is)*d(is,j,k)*v1(is,j,k)
     &              )
             endif
           enddo
        enddo
        do k=ks-2,ke+2
           do j=js-2,je+2
              do i=is-2,ie+2
                  intstress= intstress -
     &              4.0*dx1b(i)*dx2b(j)*dx3b(k)*x2b(j)
     &              *b2(i,j,k)*b3(i,j,k)

                  bremms= bremms -
     &              4.0*dx1b(i)*dx2b(j)*dx3b(k)*x2b(j)
     &              *d(i,j,k)**2.0 *
     &              dsqrt(e(i,j,k)/d(i,j,k))

                  ohmic= ohmic +
     &              ( 
     &               4.0*x2b(j)*dx1a(i)*dx2a(j)*dx3a(k)*
     &                (
     &                   (  
     &                    (1.0/x2b(j))*
     &                     (
     &                      (
     &                       (x2a(j)*b3(i,j,k)
     &                       -x2a(j-1)*b3(i,j-1,k)
     &                        )/dx2a(j)
     &                       )-
     &                      (
     &                       b2(i,j,k)-b2(i,j,k-1)
     &                       )/dx3a(k)
     &                      )
     &                    )
     &                  +
     &                   (  
     &                    (1.0/x2b(j))*
     &                     (
     &                      (
     &                       (b1(i,j,k)-b1(i,j,k-1)
     &                        )/dx3a(k)
     &                       )-
     &                      (
     &                       (x2a(j)*b3(i,j,k)
     &                       -x2a(j)*b3(i-1,j,k)
     &                       )/dx1a(i)
     &                      )
     &                     )
     &                   ) 
     &                  + 
     &                   (  
     &                      (
     &                       (b2(i,j,k)-b2(i-1,j,k)
     &                        )/dx1a(i)
     &                       )-
     &                      (
     &                       (b1(i,j,k)-b1(i,j-1,k)
     &                       )/dx2a(j)
     &                      )
     &                   )
     &                )**2.0
     &              )
 
              enddo 
           enddo
        enddo
        call MPI_REDUCE(mdotin,totmdotin, 1, 
     &               MPI_FLOAT, MPI_SUM, 0, comm3d, ierr)
        call MPI_REDUCE(mdotout,totmdotout, 1, 
     &               MPI_FLOAT, MPI_SUM, 0, comm3d, ierr)
        call MPI_REDUCE(mdotvert,totmdotvert, 1, 
     &               MPI_FLOAT, MPI_SUM, 0, comm3d, ierr)
        call MPI_REDUCE(intstress,totintstress, 1, 
     &               MPI_FLOAT, MPI_SUM, 0, comm3d, ierr)
        call MPI_REDUCE(bremms,totbremms, 1, 
     &               MPI_FLOAT, MPI_SUM, 0, comm3d, ierr)
        call MPI_REDUCE(ohmic,totohmic, 1, 
     &               MPI_FLOAT, MPI_SUM, 0, comm3d, ierr)
        if (myid .eq. 0) then
          inquire(file='integrated_quan',exist=fileex)
          if(fileex)then
             open(8,file='integrated_quan',status='old',access='append')
          else
             open(8,file='integrated_quan',status='new')
          endif

          write(8,1099)time,totmdotin,totmdotout,totmdotvert,
     &          totintstress,totbremms,totohmic
 1099     format(7d15.5)
          close(8)
        endif
        rrayq=0
        inquire(file='ray_quan',exist=fileex)
        if(fileex)then
           open(9,file='ray_quan',status='old',access='append')
        else
           open(9,file='ray_quan',status='new')
        endif

c        write(*,*)'You made it to the ray file'
        do j=js,je           
           if (x3b(ks) .lt. 0.1 .and. x1b(ie) .lt. 1.0) then
c            write(*,*)'Did you make it this far in ray file?'
            write(9,1100)time,x1b(ie),x2b(j),x3b(ks),
     &       d(ie,j,ks),e(ie,j,ks),
     &       v1(ie,j,ks),v2(ie,j,ks),v3(ie,j,ks),
     &       b1(ie,j,ks),b2(ie,j,ks),b3(ie,j,ks)
 1100       format(12d15.5)
           endif
        enddo
        close(9) 
c       ASSUMPTION 4: Ro=10.0 at about j=85 of 200 
c       INTEGRATE O
        inquire(file='ang_quan',exist=fileex)
        if(fileex)then
           open(10,file='ang_quan',status='old',access='append')
        else
           open(10,file='ang_quan',status='new')
        endif

        do k=ks,ke
           if (x2b(je) .lt. 15.0 .and. x2b(js) .gt. 6.0 .and. 
     &         x1b(ie) .lt. 1.0) then
            write(10,1101)time,x1b(ie),x2b(28),x3b(k),
     &       d(ie,28,k),e(ie,28,k),
     &       v1(ie,28,k),v2(ie,28,k),v3(ie,28,k),
     &       b1(ie,28,k),b2(ie,28,k),b3(ie,28,k)
 1101       format(12d15.5)
           endif
        enddo
        close(10) 
      endif
      if(myid.eq.0)then
         write(*,*)time,dt
      endif
      return
      end
