#include "zeusmp.def"
c=======================================================================
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
      subroutine tslice
c
c  FORMATTED WRITE OF SELECTED VARIABLES
c
c     written by: Jim Stone
c     date:       February,1993
c     modified1:  DAC from ZEUS-3D
c     modified2:  RAF 1/12/96 for ZEUS-MP
c     modified3:  RAF 3/12/97, for ablation test problem
c     modified4:  JCH 4/17/97, cons.h included even when RAD_EXP
c                 not defined
c     modified5:  M-MML 5/18/98, expanded to include the full ZEUS-3D 
c         suite, including max and min value, as well as things like 
c         rms velocity, and to run under MPI
c     modified6:  efh 04/15/99, see modified5. Control mechanism modified
c                 as well (intchk, dataio), as printd works for each tile.
c     modified7:  PSLi 11/19.99, correct a mistake in getting extrema by
c                 the root process.
c     modified8:  CSR - include output of viscous dissipation
c     modified9:  9/17/06 CSR tslice fix
c
c  PURPOSE:  Dumps global scalar "history" variables in a formatted write
c  for analysis.  
c  Currently implemented variables are:
c
c "sum up" variables
c   sumscal( 1) = mass
c   sumscal( 2) = total energy
c   sumscal( 3) = kinetic energy
c   sumscal( 4) = magnetic energy
c   sumscal( 5) = internal energy
c   sumscal( 6) = radiation energy
c   sumscal( 7) = gravitational potential energy
c   sumscal( 8) = 0.5*d*v1**2
c   sumscal( 9) = 0.5*d*v2**2
c   sumscal(10) = 0.5*d*v3**2
c   sumscal(11) = b1**2
c   sumscal(12) = b2**2
c   sumscal(13) = b3**2
c   sumscal(14) = b1 flux
c   sumscal(15) = b2 flux
c   sumscal(16) = b3 flux
c   sumscal(17) = 1 angular momentum
c   sumscal(18) = 2 angular momentum
c   sumscal(19) = 3 angular momentum
c   sumscal(20) = 1 center of mass
c   sumscal(21) = 2 center of mass
c   sumscal(22) = 3 center of mass
c   sumscal(13) = 1 rms velocity
c   sumscal(14) = 2 rms velocity
c   sumscal(15) = 3 rms velocity
c   sumscal(16) = rms velocity
c   sumscal(28) = time-integrated viscous dissipation
c
c "minmax" variables
c   minscal(1)  =    dmin   - minimum density
c   maxscal(1)  =    dmax   - maximum density
c   minscal(2)  =    emin   - minimum specific energy
c   maxscal(2)  =    emax   - maximum specific energy
c   minscal(3)  =    pmin   - minimum pressure
c   maxscal(3)  =    pmax   - maximum pressure
c   minscal(4)  =    v1min  - minimum velocity in 1-direction
c   maxscal(4)  =    v1max  - maximum velocity in 1-direction
c   minscal(5)  =    v2min  - minimum velocity in 2-direction
c   maxscal(5)  =    v2max  - maximum velocity in 2-direction
c   minscal(6)  =    v3min  - minimum velocity in 3-direction
c   maxscal(6)  =    v3max  - maximum velocity in 3-direction
c   minscal(7)  =    vmin   - minimum speed (magnitude of velocity vector)
c   maxscal(7)  =    vmax   - maximum speed (magnitude of velocity vector)
c   minscal(8)  =    b1min  - minimum mag. field in 1-direction
c   maxscal(8)  =    b1max  - maximum mag. field in 1-direction
c   minscal(9)  =    b2min  - minimum mag. field in 2-direction
c   maxscal(9)  =    b2max  - maximum mag. field in 2-direction
c   minscal(10) =    b3min  - minimum mag. field in 3-direction
c   maxscal(10) =    b3max  - maximum mag. field in 3-direction
c   minscal(11) =    bmin   - minimum mag. field strength (magnitude of B-vector)
c   maxscal(11) =    bmax   - maximum mag. field strength (magnitude of B-vector)
c   minscal(12) =    dvbmin - minimum div(B) (normalised)
c   maxscal(12) =    dvbmax - maximum div(B) (normalised)
c
c
c  LOCALS:  REAL sumscal(nscal)     sendbuf in MPI_Reduce
c           REAL totsumscal(nscal)  recvbuf in MPI_Reduce 
c 
c-----------------------------------------------------------------------
      implicit NONE
#include "mod_names.h"
#include "param.h"
#include "grid.h"
#include "field.h"
#include "scratch.h"
#include "root.h"
#include "gravity.h"
#include "cons.h"
#if defined RAD || defined RAD_EXP
#include "radexp.h"
#endif /* RAD || RAD_EXP */
#include "mpicom.h"

      integer i,j,k,ip1,jp1,kp1,
     &        nsumscal,nminscal,nmaxscal
      integer id_ctr, my_rank
      parameter(nsumscal=28)
#ifdef MHD
      parameter(nminscal=12)
      parameter(nmaxscal=12)
#else /* MHD */
      parameter(nminscal=7)
      parameter(nmaxscal=7)
#endif /* MHD */

      REAL sumscal(nsumscal)    
     &    ,totsumscal(nsumscal)
     &    ,minscal(2,nminscal)     ! 2d for process rank used in MAXLOC
     &    ,maxscal(2,nmaxscal)    
     &    ,totminscal(2,nminscal)
     &    ,totmaxscal(2,nmaxscal)
      integer
c     &    ,totxmin(nprocs,nminscal) ! for the coordinates of extrema
c     &    ,totxmax(nprocs,nmaxscal)
     &     totxmin(nprocs*nminscal) ! for the coordinates of extrema
     &    ,totxmax(nprocs*nmaxscal)
      integer  
     &         imin(nminscal)    , imax(nmaxscal)
     &       , jmin(nminscal)    , jmax(nmaxscal)
     &       , kmin(nminscal)    , kmax(nmaxscal)

      REAL mass,x1cm,x2cm,x3cm,eint,ek1,ek2,ek3,epot
     .   ,emag1,emag2,emag3,b1flx,b2flx,b3flx
     .   ,l1,l2,l3,erad,dvb,totdvb
CPS
#ifdef MHD
     .   ,mdtal
#endif
#ifdef RAD_EXP
     .   ,mdtrd
#endif
     .   ,mdtcs,mdtv1,mdtv2,mdtv3,mdtqqi2,mdtqq
C
       REAL        dmin    , dmax    , semin   , semax   , pmin
     &             , pmax    , v1min   , v1max   , v2min   , v2max
     &             , v3min   , v3max   , vsqrmin , vsqrmax , vmin
     &             , vmax    , b1min   , b1max   , b2min   , b2max
     &             , b3min   , b3max   , bsqrmin , bsqrmax , bmin
     &             , bmax    , dvbmin  , dvbmax
     &             , vrms1   , vrms2   , vrms3   , vrms
      REAL rv,dvola,vola,dm,g3b,q1,q2,q3,dar1,dar2,dar3
     .   , v1av,v2av,v3av,b1av,b2av,b3av
c\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\///////////////////////////////
c=======================================================================
c
c
c  Integrate quantities for each tile
c
c      Zero totals.
c
c
c  CSR - include viscous dissipation information
c
#ifdef PHYS_VISC
      REAL visc_diss
      common/dissipation/visc_diss
#endif /* PHYS_VISC */

#ifdef DBGZMP_TSLICE
       write(*,"('TSLICE: Process ',i2,' entering')") myid_w
       write(*,"('TSLICE: Process ',i2,' : integrated quantities ')") 
     &         myid_w
       call flush(0)
#endif /* DBGZMP_TSLICE */
       id_ctr = 99999
       mass  = tiny
       vola  = 0.0
       x1cm  = 0.0
       x2cm  = 0.0
       x3cm  = 0.0
       eint  = 0.0
       ek1   = 0.0
       ek2   = 0.0
       ek3   = 0.0
       epot  = 0.0
       emag1 = 0.0
       emag2 = 0.0
       emag3 = 0.0
       b1flx = 0.0
       b2flx = 0.0
       b3flx = 0.0
       l1    = 0.0
       l2    = 0.0
       l3    = 0.0
       vrms1 = 0.0
       vrms2 = 0.0
       vrms3 = 0.0
       vrms  = 0.0
       erad  = 0.0
c
c      Compute totals over k-sweeps
c
       do k=ks,ke
         kp1 = k + 1
         do j=js,je
           jp1 = j + 1
           do i=is,ie
             ip1 = i + 1
             dvola     = dvl1a(i) * dvl2a(j) * dvl3a(k)
             vola      = vola + dvola
             dm        = d(i,j,k) * dvola
             g3b       = g31b(i) * g32b(j)
             q1        = x1b(i)
             q2        = x2b(j) * g2b(i)
             q3        = x3b(k) * g3b
             dar1      = g2a(i) * dx2b(j) * g3b    * dx3b(k)
             dar2      = g3b    * dx3b(k)          * dx1b(i)
             dar3      =          dx1b(i) * g2a(i) * dx2b(j)
             v1av      = v1(i,j,k) + v1(ip1,j  ,k  )
             v2av      = v2(i,j,k) + v2(i  ,jp1,k  )
             v3av      = v3(i,j,k) + v3(i  ,j  ,kp1)
#ifdef MHD
             b1av      = b1(i,j,k) + b1(ip1,j  ,k  )
             b2av      = b2(i,j,k) + b2(i  ,jp1,k  )
             b3av      = b3(i,j,k) + b3(i  ,j  ,kp1)
#else /* MHD */
             b1av      = 0.0
             b2av      = 0.0
             b3av      = 0.0
#endif /* MHD */
             mass      = mass      + dm
             x1cm      = x1cm      + dm * q1
             x2cm      = x2cm      + dm * q2
             x3cm      = x3cm      + dm * q3
             ek1       = ek1       + dm * v1av**2
             ek2       = ek2       + dm * v2av**2
             ek3       = ek3       + dm * v3av**2
#ifndef ISO
CJH
#ifndef TOTAL_ENERGY
             eint      = eint      + e(i,j,k) * dvola
#else /* NOT TOTAL_ENERGY */
             eint      = eint      + dvola *
     .                   ( e(i,j,k) - 0.125 * d(i,j,k) *
     .                                (v1av**2 + v2av**2 + v3av**2) )
#endif /* NOT TOTAL_ENERGY */
#endif /* NOT ISO */
#if defined RAD || defined RAD_EXP
             erad      = erad      + er(i,j,k) * dvola
#endif /* RAD || RAD_EXP */
#if defined GRAV || defined GRAV_FFT
             epot      = epot      - dm * gp(i,j,k)
#endif
#ifdef XYZ
             rv        = sqrt ( ( x1b(i) - x1ptm )**2
     1                        + ( x2b(j) - x2ptm )**2
     2                        + ( x3b(k) - x3ptm )**2 )
#endif
#ifdef ZRP
             rv        = sqrt ( ( x1b(i) - x1ptm )**2
     1                        + ( x2b(j) - x2ptm )**2
     2                        + 2.0 * x2b(j) * x2ptm
     3                        * ( 1.0 - cos(x3b(k) - x3ptm) ) )
#endif
#ifdef RTP
             rv        = ( x1b(i)*cos(x2b(j)) - x1ptm*cos(x2ptm) )**2
     1                 + ( x1b(i)*sin(x2b(j)) - x1ptm*sin(x2ptm) )**2
     2                 + 2.0 * x1b(i)*sin(x2b(j)) * x1ptm*sin(x2ptm)
     3                 * ( 1.0 - cos(x3b(k) - x3ptm) )
             rv        = sqrt ( rv )
#endif
             epot      = epot      - dm * g * ptmass / rv
#ifdef MHD
             emag1     = emag1     + b1av**2 * dvola
             emag2     = emag2     + b2av**2 * dvola
             emag3     = emag3     + b3av**2 * dvola
             b1flx     = b1flx     + b1(i,j,k) * dar1
             b2flx     = b2flx     + b2(i,j,k) * dar2
             b3flx     = b3flx     + b3(i,j,k) * dar3
#endif
             l1        = l1        + dm * ( q2 * v3av - q3 * v2av )
             l2        = l2        + dm * ( q3 * v1av - q1 * v3av )
             l3        = l3        + dm * ( q1 * v2av - q2 * v1av )
             
             vrms1     = vrms1     + dvola*v1(i,j,k)**2
             vrms2     = vrms2     + dvola*v2(i,j,k)**2
             vrms3     = vrms3     + dvola*v3(i,j,k)**2
             vrms      = vrms      + dvola
     &                             *(   v1(i,j,k)**2
     &                                + v2(i,j,k)**2
     &                                + v3(i,j,k)**2)

#if defined RAD_EXP || defined RAD
             if (  abs(x1a(i)-zro).lt.0.25*dx1a(i) 
     1       .and. abs(x2a(j)-zro).lt.0.25*dx2a(j)
     2       .and. abs(x3a(k)-zro).lt.0.25*dx3a(k)) then
               sumscal(17) = d (i,j,k)
               sumscal(18) = e (i,j,k)
               sumscal(19) = er(i,j,k)
               id_ctr = myid
             endif
#endif /* RAD_EXP || RAD */

           enddo
         enddo
       enddo
#ifdef DBGZMP_TSLICE 
       if (idebug .gt. 9) then
         write(*,"('TSLICE: Process ',i2,' loop finished.')") myid
         call flush(0)
       endif
#endif /* DBGZMP_TSLICE */

       sumscal(1)  = mass
       sumscal(5)  = eint
       sumscal(6)  = erad
       sumscal(7)  = epot
       sumscal(8)  = ek1   / 8.0
       sumscal(9)  = ek2   / 8.0
       sumscal(10) = ek3   / 8.0
       sumscal(11) = emag1 / 8.0
       sumscal(12) = emag2 / 8.0
       sumscal(13) = emag3 / 8.0
       sumscal(14) = b1flx
       sumscal(15) = b2flx
       sumscal(16) = b3flx
c
#if defined RAD_EXP || defined RAD
c......................................................................
c
c Special output for ablation problem:
c
       if(myid .eq. id_ctr) then
         sumscal(20) = gamm1 * scal(18) * 1.0e-12   !  pressure in Megabars
         sumscal(21) = scal(18) * (gamm1*mmw*mh) / (scal(17) * everg) ! T_gas
         sumscal(22) = sqrt(sqrt(scal(19)/rad_con)) * boltz/everg     ! T_rad
       endif
c
c......................................................................
c
#else /* RAD_EXP || RAD */

       sumscal(17) = l1    / 2.0
       sumscal(18) = l2    / 2.0
       sumscal(19) = l3    / 2.0

       sumscal(20) = x1cm  ! no division by mass because of 
       sumscal(21) = x2cm  ! other tiles
       sumscal(22) = x3cm

       sumscal(23) = vrms1 
       sumscal(24) = vrms2
       sumscal(25) = vrms3
       sumscal(26) = vrms 
       sumscal(27) = vola
c
c      assign viscous dissipation
c
#ifdef PHYS_VISC
       sumscal(28) = visc_diss
#endif /* PHYS_VISC */

#endif /* RAD_EXP || RAD */

c
c Total up various forms of energy.
c
       sumscal(3)  = sumscal(8)  + sumscal(9)  + sumscal(10)    ! kinetic
       sumscal(4)  = sumscal(11) + sumscal(12) + sumscal(13)    ! magnetic
       sumscal(2)  = sumscal(3)  + sumscal(4)  + sumscal(5)     ! total
     &             + sumscal(6) + sumscal(7)

c
c.......................................................................
c  Now reduce all tiles to root process
c
#ifdef DBGZMP_TSLICE
       if (idebug .gt. 9) then
         write(*,"('TSLICE: Process ',i2,' reducing sums.')") myid
         call flush(0)
       endif
#endif /* DBGZMP_TSLICE */
      call MPI_Reduce(sumscal, totsumscal, nsumscal, 
     &                MPI_FLOAT, MPI_SUM, 0, comm3d, ierr)

      if (myid .eq. 0) then 
#ifdef DBGZMP_TSLICE
        if (idebug .gt. 11) then
          write(*,"('TSLICE: Process ',i2,' tss1  = ',e13.4)")
     &              myid, totsumscal(1)
          write(*,"('TSLICE: Process ',i2,' tss27 = ',e13.4)") 
     &              myid, totsumscal(27)
          call flush(0)
        endif
#endif /* DBGZMP_TSLICE */
        totsumscal(20) = totsumscal(20) / totsumscal(1)         ! center of mass
        totsumscal(21) = totsumscal(21) / totsumscal(1)
        totsumscal(22) = totsumscal(22) / totsumscal(1)  
        totsumscal(23) = sqrt(totsumscal(23) / totsumscal(27))  ! rms-velocities
        totsumscal(24) = sqrt(totsumscal(24) / totsumscal(27))
        totsumscal(25) = sqrt(totsumscal(25) / totsumscal(27))
        totsumscal(26) = sqrt(totsumscal(26) / totsumscal(27))
      endif
        
c
c.......................................................................
c Calculate extrema for each tile.
c
#ifdef DBGZMP_TSLICE
       if (idebug .ge. 2) then
         write(*,"('TSLICE: Process ',i2,' : extrema')") 
     &           myid_w
         call flush(0)
       endif
       if (idebug .ge. 4) then
         write(*,"('TSLICE: Process ',i2,' :   density')")
     &         myid_w
         call flush(0)
       endif
#endif /* DBGZMP_TSLICE */
c-- 1.  density --------------------------------------------------------
c
       call mnmx ( d, is, js, ks, ie, je, ke
     &           , minscal(1,1), imin(1), jmin(1), kmin(1)
     &           , maxscal(1,1), imax(1), jmax(1), kmax(1) )
       
#ifndef ISO
c
c-- 2.  specific internal energy (proportional to T) -------------------
c
#ifdef DBGZMP_TSLICE
       if (idebug .ge. 4) then
         write(*,"('TSLICE: Process ',i2,' :   spec. energy')")
     &           myid_w
         call flush(0)
       endif
#endif
       call spenergy ( is, ie, js, je, ks, ke, w3da )
       call mnmx ( w3da, is, js, ks, ie, je, ke
     1           , minscal(1,2), imin(2), jmin(2), kmin(2)
     2           , maxscal(1,2), imax(2), jmax(2), kmax(2) )
C#else
C       minscal(1,2)=0.0
C       maxscal(1,2)=0.0
C       imin(2)=0
C       imax(2)=0
C       jmin(2)=0
C       jmax(2)=0
C       kmin(2)=0
C       kmax(2)=0
#endif /* !ISO */
c
c-- 3.  pressure -------------------------------------------------------
c
#ifdef DBGZMP_TSLICE
       if (idebug .ge. 4) then
         write(*,"('TSLICE: Process ',i2,' :   pressure')")
     &           myid_w
         call flush(0)
       endif
#endif /* DBGZMP_TSLICE */
       call pressure ( w3da, 1 )
       call mnmx ( w3da, is, js, ks, ie, je, ke
     1           , minscal(1,3), imin(3), jmin(3), kmin(3)
     2           , maxscal(1,3), imax(3), jmax(3), kmax(3) )
c
c-- 4.  1-velocity -----------------------------------------------------
c
#ifdef DBGZMP_TSLICE
       if (idebug .ge. 4) then
         write(*,"('TSLICE: Process ',i2,' :   velocities')")
     &           myid_w
         call flush(0)
       endif
#endif /* DBZMP_TSLICE */
       call mnmx ( v1, is+1, js, ks, ie, je, ke
     1           , minscal(1,4), imin(4), jmin(4), kmin(4)
     2           , maxscal(1,4), imax(4), jmax(4), kmax(4) )
c
c-- 5.  2-velocity -----------------------------------------------------
c
       call mnmx ( v2, is, js+1, ks, ie, je, ke
     1           , minscal(1,5), imin(5), jmin(5), kmin(5)
     2           , maxscal(1,5), imax(5), jmax(5), kmax(5) )
c
c-- 6.  3-velocity -----------------------------------------------------
c
       call mnmx ( v3, is, js, ks+1, ie, je, ke
     1           , minscal(1,6), imin(6), jmin(6), kmin(6)
     2           , maxscal(1,6), imax(6), jmax(6), kmax(6) )
c
c-- 7.  speed (magnitude of velocity) ----------------------------------
c
#ifndef KSYM
       do k=ks,ke
#endif /* !KSYM */
#ifndef JSYM
         do j=js,je
#endif /* !JSYM */
#ifndef ISYM
           do i=is,ie
#endif /* !ISYM */
             w3da(i,j,k) = 0.25 * ( v1(i+1,j  ,k  ) + v1(i,j,k) )**2
     1                   + 0.25 * ( v2(i  ,j+1,k  ) + v2(i,j,k) )**2
     2                   + 0.25 * ( v3(i  ,j  ,k+1) + v3(i,j,k) )**2
           enddo
         enddo
#ifndef KSYM
       enddo
#endif /* !KSYM */
       call mnmx ( w3da, is, js, ks, ie, je, ke
     1           , vsqrmin, imin(7), jmin(7), kmin(7)
     2           , vsqrmax, imax(7), jmax(7), kmax(7) )
       minscal(1,7) = sqrt ( vsqrmin )
       maxscal(1,7) = sqrt ( vsqrmax )
#ifdef MHD
c
c-- 8.  1-magnetic field -----------------------------------------------
c
#ifdef DBGZMP_TSLICE
       if (idebug .ge. 4) then
         write(*,"('TSLICE: Process ',i2,' :   mag. fields')")
     &           myid_w
         call flush(0)
       endif
#endif
       call mnmx ( b1, is+1, js, ks, ie, je, ke
     1           , minscal(1,8), imin(8), jmin(8), kmin(8)
     2           , maxscal(1,8), imax(8), jmax(8), kmax(8) )
c
c-- 9. 2-magnetic field ------------------------------------------------
c
       call mnmx ( b2, is, js+1, ks, ie, je, ke
     1           , minscal(1,9), imin(9), jmin(9), kmin(9)
     2           , maxscal(1,9), imax(9), jmax(9), kmax(9) )
c
c-- 10. 3-magnetic field -----------------------------------------------
c
       call mnmx ( b3, is, js, ks+1, ie, je, ke
     1           , minscal(1,10), imin(10), jmin(10), kmin(10)
     2           , maxscal(1,10), imax(10), jmax(10), kmax(10) )
c
c-- 11. magnetic field strength (magnitude of magnetic field vector) ---
c
       call pressure ( w3da, 2 )
       call mnmx ( w3da, is, js, ks, ie, je, ke
     1           , bsqrmin, imin(11), jmin(11), kmin(11)
     2           , bsqrmax, imax(11), jmax(11), kmax(11) )
       minscal(1,11) = sqrt ( 2.0 * bsqrmin )
       maxscal(1,11) = sqrt ( 2.0 * bsqrmax )
c
c-- 12.  div(B) --------------------------------------------------------
c
       call diverg ( b1, b2, b3, 1, 1, w3da, dvb )
       call mnmx ( w3da, is+1, js+1, ks+1, ie-1, je-1, ke-1
     1           , minscal(1,12), imin(12), jmin(12), kmin(12)
     2           , maxscal(1,12), imax(12), jmax(12), kmax(12) )
c we need this extra call for dvb
      call MPI_Reduce(dvb, totdvb, 1, 
     &                MPI_FLOAT, MPI_SUM, 0, comm3d, ierr)


#endif /* MHD */
c
c get the extrema and their coordinates
c   tot[min/max]scal contains the processID (2,i) and the
c   extemum value of that process (1,i) for all i=1,n[min/max]scal 
c
#ifdef DBGZMP_TSLICE
       if (idebug .ge. 4) then
         write(*,"('TSLICE: Process ',i2,' :   gather all extrema')")
     &           myid_w
         call flush(0)
       endif
#endif /* DBGZMP_TSLICE */
      do i=1,nminscal
        minscal(2,i) = myid
        maxscal(2,i) = myid
      enddo

      call MPI_Reduce(minscal, totminscal, nminscal, 
     &                MPI_2FLOAT, MPI_MINLOC, 
     &                0, comm3d, ierr)
      call MPI_Reduce(maxscal, totmaxscal, nmaxscal, 
     &                MPI_2FLOAT, MPI_MAXLOC, 
     &                0, comm3d, ierr)
#ifdef DBGZMP_TSLICE
      if (idebug .ge. 9 .and. myid .eq. 0) then
        write(*,"('TSLICE: Process root:   contents of arrays')")
        do i=1,nminscal
          write(*,"('TSLICE:   i',i2,', pID',i2,', min',e13.4)")
     &         i, int(totminscal(2,i)),totminscal(1,i)
          write(*,"('TSLICE:   i',i2,', pID',i2,', max',e13.4)")
     &         i, int(totmaxscal(2,i)),totmaxscal(1,i)
        enddo
      endif
#endif /* DBGZMP_TSLICE */
c
c we still need the coordinates ...
c These can be stored in [i-k][min-max] of process 0, as
c 0 does the output.

#ifdef DBGZMP_TSLICE
       if (idebug .ge. 4) then
         write(*,"('TSLICE: Process ',i2,' :   gather coordinates')")
     &           myid_w
         write(*,"('TSLICE: Proces  ',i2,' :   number of procs :',i2)")
     &           myid_w, nprocs
         call flush(0)
       endif
#endif /* DBGZMP_TSLICE */

c x-coord
#ifdef DBGZMP_TSLICE
       if (idebug .ge. 6) then
         write(*,"('TSLICE: Process ',i2,' :     x-coord')")
     &           myid_w
         call flush(0)
       endif
#endif /* DBGZMP_TSLICE */
      call MPI_Gather(imin   , nminscal, MPI_INTEGER, 
     &                totxmin, nminscal, MPI_INTEGER, 
     &                0, comm3d, ierr)
      call MPI_Gather(imax   , nmaxscal, MPI_INTEGER, 
     &                totxmax, nmaxscal, MPI_INTEGER, 
     &                0, comm3d, ierr)

      if (myid .eq. 0) then  ! read out the coordinates 
        do i=1,nminscal
#ifdef DBGZMP_TSLICE
          if (idebug .ge. 9) then
            write(*,"('TSLICE: Process ',i2,
     &                ' :     i',i2,
     &                ', pID(min)',i2,
     &                ', min',e13.4)")
     &           myid_w, i, 
     &           int(totminscal(2,i)), 
     &           totminscal(1,i)
            write(*,"('TSLICE: Process ',i2,
     &                ' :     i',i2,
     &                ', pID(max)',i2,
     &                ', max',e13.4)")
     &           myid_w, i,
     &           int(totmaxscal(2,i)),
     &           totmaxscal(1,i)
            call flush(0)
            write(*,"('TSLICE: Process ',i2,
     &                ' :     i',i2,
     &                ', totimin ',i2)")
     &           myid_w, i,
     &           totxmin(int(totminscal(2,i))*nprocs+i)
            write(*,"('TSLICE: Process ',i2,
     &                ' :     i',i2,
     &                ', totimax ',i2)")
     &           myid_w, i,
     &           totxmax(int(totmaxscal(2,i))*nprocs+i)
            call flush(0)
          endif
#endif /* DBGZMP_TSLICE */
          imin(i) = totxmin(int(totminscal(2,i))*nminscal+i)
          imax(i) = totxmax(int(totmaxscal(2,i))*nmaxscal+i)
c          imin(i) = totxmin(int(totminscal(2,i)),i)
c          imax(i) = totxmax(int(totmaxscal(2,i)),i)
        enddo
      endif  
c y-coord
#ifdef DBGZMP_TSLICE
       if (idebug .ge. 6) then
         write(*,"('TSLICE: Process ',i2,' :     y-coord')")
     &           myid_w
         call flush(0)
       endif
#endif /* DBGZMP_TSLICE */
      call MPI_Gather(jmin   , nminscal, MPI_INTEGER, 
     &                totxmin, nminscal, MPI_INTEGER, 
     &                0, comm3d, ierr)
      call MPI_Gather(jmax   , nmaxscal, MPI_INTEGER, 
     &                totxmax, nmaxscal, MPI_INTEGER, 
     &                0, comm3d, ierr)

      if (myid .eq. 0) then 
        do j=1,nminscal
c          jmin(j) = totxmin(int(totminscal(2,j))+1,j)
c          jmax(j) = totxmax(int(totmaxscal(2,j))+1,j)
          jmin(j) = totxmin(int(totminscal(2,j))*nminscal+j)
          jmax(j) = totxmax(int(totmaxscal(2,j))*nmaxscal+j)
        enddo
      endif  
c z-coord
#ifdef DBGZMP_TSLICE
       if (idebug .ge. 6) then
         write(*,"('TSLICE: Process ',i2,' :     z-coord')")
     &           myid_w
         call flush(0)
       endif
#endif /* DBGZMP_TSLICE */
      call MPI_Gather(kmin   , nminscal, MPI_INTEGER, 
     &                totxmin, nminscal, MPI_INTEGER, 
     &                0, comm3d, ierr)
      call MPI_Gather(kmax   , nmaxscal, MPI_INTEGER, 
     &                totxmax, nmaxscal, MPI_INTEGER, 
     &                0, comm3d, ierr)

      if (myid .eq. 0) then 
        do k=1,nminscal
c          kmin(k) = totxmin(int(totminscal(2,k))+1,k)
c          kmax(k) = totxmax(int(totmaxscal(2,k))+1,k)
          kmin(k) = totxmin(int(totminscal(2,k))*nminscal+k)
          kmax(k) = totxmax(int(totmaxscal(2,k))*nmaxscal+k)
        enddo
      endif  
CPS
#ifdef MPI_USED
C  Get the maximum dt contributors from all processors
C
#ifdef MHD
       call MPI_REDUCE(dtal,mdtal,1,MPI_FLOAT,MPI_MAX,0,
     &      comm3d,ierr)
#endif /* MHD */
#ifdef RAD_EXP
       call MPI_REDUCE(dtrd,mdtrd,1,MPI_FLOAT,MPI_MAX,0,
     &      comm3d,ierr)
#endif /* RAD_EXP */
       call MPI_REDUCE(dtcs,mdtcs,1,MPI_FLOAT,MPI_MAX,0,
     &      comm3d,ierr)
       call MPI_REDUCE(dtv1,mdtv1,1,MPI_FLOAT,MPI_MAX,0,
     &      comm3d,ierr)
       call MPI_REDUCE(dtv2,mdtv2,1,MPI_FLOAT,MPI_MAX,0,
     &      comm3d,ierr)
       call MPI_REDUCE(dtv3,mdtv3,1,MPI_FLOAT,MPI_MAX,0,
     &      comm3d,ierr)
       call MPI_REDUCE(dtqqi2,mdtqqi2,1,MPI_FLOAT,MPI_MAX,0,
     &      comm3d,ierr)
#endif /* MPI_USED */
c
c  Write out variables to file connected to unit iotsl opened in MAIN
c  program unit (zeusmp.f)
#ifdef DBGZMP_TSLICE
       write(*,"('TSLICE: Process ',i2,': dump to file ')") 
     &         myid_w
       call flush(0)
#endif /* DBGZMP_TSLICE */

c-----------------------------------------------------------------------
c------------------------- DUMP TO ASCII FILE --------------------------
c-----------------------------------------------------------------------
c

      if (myid .eq. 0) then
         write (31, 2040) 'time = ', time, ', nhy = ', nhy,
     &         ', dt = ', dt
         write (31, 2050) 'total mass                      =', 
     &         totsumscal(1)
#if !defined RAD_EXP && !defined RAD
         write (31, 2050) 'centre of mass in 1-direction   =', 
     &         totsumscal(20)
         write (31, 2050) 'centre of mass in 2-direction   =', 
     &         totsumscal(21)
         write (31, 2050) 'centre of mass in 3-direction   =', 
     &         totsumscal(22)
#endif /* !RAD_EXP && !RAD */
         write (31, 2050) 'total internal energy           =', 
     &         totsumscal(5)
         write (31, 2050) 'kinetic energy in 1-direction   =', 
     &         totsumscal(8)
         write (31, 2050) 'kinetic energy in 2-direction   =', 
     &         totsumscal(9)
         write (31, 2050) 'kinetic energy in 3-direction   =', 
     &         totsumscal(10)
         write (31, 2050) 'total kinetic energy            =', 
     &         totsumscal(3)
         write (31, 2050) 'gravitational potential energy  =', 
     &         totsumscal(7)
#ifdef MHD
         write (31, 2050) 'magnetic energy of 1-component  =', 
     &         totsumscal(11)
         write (31, 2050) 'magnetic energy of 2-component  =', 
     &         totsumscal(12)
         write (31, 2050) 'magnetic energy of 3-component  =', 
     &         totsumscal(13)
         write (31, 2050) 'total magnetic energy           =', 
     &         totsumscal(4)
         write (31, 2050) 'magnetic flux of 1-component    =', 
     &         totsumscal(14)
         write (31, 2050) 'magnetic flux of 2-component    =', 
     &         totsumscal(15)
         write (31, 2050) 'magnetic flux of 3-component    =', 
     &         totsumscal(16)
         write (31, 2050) 'averaged and normalised div(B)  =', 
     &         dvb
#endif /* MHD */
         write (31, 2050) 'total energy                    =', 
     &         totsumscal(2)
#if defined RAD_EXP || defined RAD
         write (31, 2050) 'total radiation energy          =', 
     &         totsumscal(6)
         write (31, 2050) 'radiative pressure              =', 
     &         totsumscal(20)
         write (31, 2050) 'temperature of gas              =', 
     &         totsumscal(21)
         write (31, 2050) 'radiative temperature           =', 
     &         totsumscal(22)
#else  /*  RAD_EXP || defined RAD */        
         write (31, 2050) '1-component of angular momentum =', 
     &         totsumscal(17)
         write (31, 2050) '2-component of angular momentum =', 
     &         totsumscal(18)
         write (31, 2050) '3-component of angular momentum =', 
     &         totsumscal(19)
#endif /*  RAD_EXP || defined RAD */
         write (31, 2050) 'rms velocity in 1-direction     =',
     &         totsumscal(23)
         write (31, 2050) 'rms velocity in 2-direction     =',
     &         totsumscal(24)
         write (31, 2050) 'rms velocity in 3-direction     =',
     &         totsumscal(25)
         write (31, 2050) 'rms velocity                    =',
     &         totsumscal(26)
         write (31, 2060) 'minimum density                 =', 
     &         totminscal(1,1), imin(1), jmin(1), kmin(1)
         write (31, 2060) 'maximum density                 =', 
     &         totmaxscal(1,1), imax(1), jmax(1), kmax(1)
#ifndef ISO
         write (31, 2060) 'minimum internal energy density =', 
     &         totminscal(1,2), imin(2), jmin(2), kmin(2)
         write (31, 2060) 'maximum internal energy density =', 
     &         totmaxscal(1,2), imax(2), jmax(2), kmax(2)
#endif /* not ISO */
c         write (31, 2060) 'minimum thermal pressure        =', 
c     &         totminscal(1,3), imin(3), jmin(3), kmin(3)
c         write (31, 2060) 'maximum thermal pressure        =', 
c     &         totmaxscal(1,3), imax(3), jmax(3), kmax(3)
         write (31, 2060) 'minimum 1-velocity              =', 
     &         totminscal(1,4), imin(4), jmin(4), kmin(4)
         write (31, 2060) 'maximum 1-velocity              =', 
     &         totmaxscal(1,4), imax(4), jmax(4), kmax(4)
         write (31, 2060) 'minimum 2-velocity              =', 
     &         totminscal(1,5), imin(5), jmin(5), kmin(5)
         write (31, 2060) 'maximum 2-velocity              =', 
     &         totmaxscal(1,5), imax(5), jmax(5), kmax(5)
         write (31, 2060) 'minimum 3-velocity              =', 
     &         totminscal(1,6), imin(6), jmin(6), kmin(6)
         write (31, 2060) 'maximum 3-velocity              =', 
     &         totmaxscal(1,6), imax(6), jmax(6), kmax(6)
         write (31, 2060) 'minimum speed (1-2 plane)       =', 
     &         totminscal(1,7), imin(7), jmin(7), kmin(7)
         write (31, 2060) 'maximum speed (1-2 plane)       =', 
     &         totmaxscal(1,7), imax(7), jmax(7), kmax(7)
#ifdef MHD
         write (31, 2060) 'minimum 1-magnetic field        =', 
     &         totminscal(1,8), imin(8), jmin(8), kmin(8)
         write (31, 2060) 'maximum 1-magnetic field        =', 
     &         totmaxscal(1,8), imax(8), jmax(8), kmax(8)
         write (31, 2060) 'minimum 2-magnetic field        =', 
     &         totminscal(1,9), imin(9), jmin(9), kmin(9)
         write (31, 2060) 'maximum 2-magnetic field        =', 
     &         totmaxscal(1,9), imax(9), jmax(9), kmax(9)
         write (31, 2060) 'minimum 3-magnetic field        =', 
     &         totminscal(1,10), imin(10), jmin(10), kmin(10)
         write (31, 2060) 'maximum 3-magnetic field        =', 
     &         totmaxscal(1,10), imax(10), jmax(10), kmax(10)
         write (31, 2060) 'minimum magnetic field strength =', 
     &         totminscal(1,11), imin(11), jmin(11), kmin(11)
         write (31, 2060) 'maximum magnetic field strength =', 
     &         totmaxscal(1,11), imax(11), jmax(11), kmax(11)
         write (31, 2060) 'minimum normalised div(B)       =', 
     &         totminscal(1,12), imin(12), jmin(12), kmin(12)
         write (31, 2060) 'maximum normalised div(B)       =', 
     &         totmaxscal(1,12), imax(12), jmax(12), kmax(12)
         write (31, 2050) 'Alfven time step                =', mdtal
#endif /* MHD */
#ifdef PHYS_VISC
         write (31, 2050) 't-integrated viscous dissip.    =',
     &         totsumscal(28)        
#endif /* PHYS_VISC */
#ifdef RAD_EXP
         write (31, 2050) 'radiation time step             =', mdtrd
#endif /* RAD_EXP */
         write (31, 2050) 'sound speed time step           =', mdtcs
         write (31, 2050) 'v1 time step                    =', mdtv1
         write (31, 2050) 'v2 time step                    =', mdtv2
         write (31, 2050) 'v3 time step                    =', mdtv3
c
c  CSR - added in actual computation of mdtqq (before the code was just
c  outputing dtqqi2 and NOT the actual artificial viscosity timestep
c
         mdtqq = 1.0 / sqrt ( mdtqqi2 + tiny )
         write (31, 2050) 'artificial viscosity time step  =', mdtqq
         write (31, 2050) 'time step for next cycle        =', dtnew
      endif
c
c-----------------------------------------------------------------------
c----------------------- Write format statements -----------------------
c-----------------------------------------------------------------------
c

2040   format('TSLICE  :',/
     1       ,'TSLICE  : ----------------------------------------------'
     2       ,'------------------------',/
     3       ,'TSLICE  : ------- ',a7,1pe12.5,a10,i5,a7,e12.5
     4       ,' --------',/
     5       ,'TSLICE  : ----------------------------------------------'
     6       ,'------------------------')
2050   format('TSLICE  : ',a33,1pe12.5)
2060   format('TSLICE  : ',a33,1pe12.5,' at i=',i4,' j=',i4,' k=',i4)
2070   format('TSLICE  : Data appended to file ',a8,'  at time ='
     1       ,1pe12.5,', nhy =',i6)
2080   format('TSLICE  : **** WARNING **** >',i6,' plot entries '
     1       ,'requested at time =',1pe12.5,/
     2       ,'TSLICE  : nhy =',i6,'.  Data thinned, resetting dttslp '
     3       ,'to ',e12.5,'.')
2090   format('TSLICE  : Data appended to plot arrays    at time ='
     1       ,1pe12.5,', nhy =',i6)

c
c-----------------------------------------------------------------------
c

#ifdef DBGZMP_TSLICE
       if (idebug .gt. 0) then 
         write(*,"('TSLICE: Process ',i2,' finished')") myid_w
         call flush(0)
       endif
#endif

      return
      end
