#include "zeusmp.def"
c=======================================================================
c
c    \\\\\\\\\\        B E G I N   P R O G R A M          //////////
c    //////////               Z E U S M P                 \\\\\\\\\\
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
c
c=======================================================================
c
      program zeusmp
c
c PURPOSE
c   Main program for 3-D MPI version of ZEUS.
c
c AUTHOR
c   Robert A. Fiedler
c
c LAST MODIFIED by John C. Vernaleo
c   04/30/2007
c.......................................................................
c
c DECLARATIONS
c
      implicit NONE
#include "mod_names.h"
#include "options.h"
#include "param.h"
#include "field.h"
#include "grid.h"
#include "root.h"
#include "scratch.h"
#include "mpicom.h"
#include "clocks.h"
#include "impsoln.h"
#include "gravity.h"
#include "radiation.h"
c
      REAL     zcs
      integer i,j,k

#ifdef PROFILING
#ifdef MPE_USED
#else
      integer i,j,k,l,m,n
      integer iu
      integer itimers(nsubstps)
      REAL tcum, tnorm
      character*6 ctimers(nsubstps)
      character*13 prof_fl
      character*13 ctype(2)
      character*4  ttype(2)
      external sort_tim
      integer kcoords(3)
      REAL    atimers(nsubstps,4,2,2,0:26)
      REAL    twcpu, tccpu

#endif /* MPE_USED */
#endif /* PROFILING */
CJH
      REAL t1, t2

#ifdef UNICOS
      real       cpuall
#else
      real*8     cpuall
#endif /* UNICOS */

#ifdef GRAV_FFT
      integer nx,ny,nz,snz,maxn
      external create_plan_
      common /fftw/ maxn
#endif /* GRAV_FFT */
      external mstart,dataio,srcstep,nudt,newgrid,intchk,SPECIAL
c
c DATA
c
      data nhy, time, nsub / 0, 0.0, 0 /
      data zcs / 0. /
#ifndef SHELLS
      data ifsen / 0, 0, 1, 1, 1, 1/
#else
      data ifsen / 0, 0, 1, 1, 1, 1, 1/
#endif /* NOT SHELLS */
      data myid_w, myid, nprocs_w, nprocs / 0, 0, 1, 1 /
      data coords / 0, 0, 0 /
      data reorder  / .true. /

#ifdef PROFILING
#ifdef MPE_USED
#else
      data ctype   /'COMMUNICATION','COMPUTATION  '/
      data ttype   /'CPU '         ,'WALL'/
      data ctimers /'forces','avisc ','pdv   '
     & ,            'tranx1','momx1 '
     & ,            'tranx2','momx2 '
     & ,            'tranx3','momx3 '
     & ,            'radpf1','radpf2','radpf3'
#ifdef RAD_EXP
     & ,            'nr_pf ','newdt ','bval3d'/
#else /* RAD_EXP */
     & ,            'nr_mg ','newdt ','bval3d'/
#endif /* RAD_EXP */
c        timers ( nsubstps, 4, 2, 2 )
c                    .      .  .  .
c                   .      .   .   .
c                  .      .    .    .
c                 .      .     .     .
c                .      .      .      .
c               .      .       .       .
c          substeps  i,j,k,r  comm/work cpu/wall
c
c***** MHD needs to be added to this scheme if it's used M-MML 17.3.98
c
c  SUBSTEPS
c  1) Forces
c  2) Artificial Viscosity
c  3) PdV
c  4) tranx1
c  5) momx1
c  6) tranx2
c  7) momx2
c  8) tranx3
c  9) momx3
c 10) radpf1
c 11) radpf2
c 12) radpf3
c 13) nr_pf or nr_mg
c 14) newdt
c 15) BCs
c
#endif /* MPE_USED */
#endif /* PROFILING */

#ifdef MPI_USED
c
c Initialize MPI
c
      call MPI_INIT( ierr )
      call MPI_COMM_RANK( MPI_COMM_WORLD, myid_w  , ierr )
      call MPI_COMM_SIZE( MPI_COMM_WORLD, nprocs_w, ierr )
#ifdef DEBUG
      write(*,"('Process ',i3,'  of ',i3,'  + 1 initialized')") 
     1  myid_w, nprocs_w - 1 
#endif /* DEBUG */
#endif /* MPI_USED */
c
c  Master writes greeting.
c
#ifdef MPI_USED
      if (myid_w .eq. 0) then
#endif /* MPI_USED */
        write(6,"(///10x,'ZZZZZ EEEEE U   U  SSSS     M   M PPPP ')")
        write(6,   "(10x,'   Z  E     U   U S         MM MM P   P')")
        write(6,   "(10x,'  Z   EEEE  U   U  SSS  === M M M PPPP ')")
        write(6,   "(10x,' Z    E     U   U     S     M   M P    ')")
        write(6,   "(10x,'ZZZZZ EEEEE  UUU  SSSS      M   M P    ')")
        write(6,"()")
        write(6,"()")
        write(6,"(10x,'       DEVELOPED BY ROBERT A. FIEDLER')")
        write(6,"(10x,'       Maintained by John C. Vernaleo')")
        write(6,"(10x,'         ZEUS-MP V1.5.13 - 04/30/2007')")
        write(6,"()")
        write(6,"(10x,'RUNNING AS ',i4,' PROCESS(ES)')") nprocs_w
        write(6,"(10x,'WITH THE FOLLOWING CONFIGURATION:')")
        write(6,"()")
        write(6, GEOMETRY_OPTION  )
        write(6, GRID_OPTION  )
        write(6, PT_MASS_OPTION   )
        write(6, GRAVITY_OPTION   )
        write(6, SELF_GRAVITY_OPTION   )
#ifdef GRAV_FFT
        write(6, PS_OPTION   )
#endif
        write(6, MHD_OPTION       )
        write(6, RAD_IMP_OPTION )
        write(6, RAD_EXP_OPTION )
        write(6, PRECISION_OPTION )
        write(6, CPU_TIMER_OPTION )
        write(6, PROFILING_OPTION )
        write(6, MPI_OPTION )
        write(6, MPE_OPTION )
        write(6, DEBUG_OPTION )
        write(6, RES_DUMPS_OPTION )
        write(6, HDF_DUMPS_OPTION )
        write(6, TSL_DUMPS_OPTION )
        write(6, HST_DUMPS_OPTION )
        write(6, TXT_DUMPS_OPTION )
        write(6, FORCE_OPTION )
        write(6, ALFLIMIT_OPTION )
        write(6, RAD_LOSS_OPTION )
        write(6, VISC_OPTION )
        write(6, CONDUCT_OPTION )
        write(6,"()")
#ifdef MPI_USED
      endif
#endif /* MPI_USED */
c
c Set up the problem: read input deck and possibly a restart file.
c
      call mstart
c
c Write out initial data dumps.
c
c      call dataio( iswres, iswhdf, iswhst, iswusr 
#ifdef SHELLS
c    .              iswshl
#endif /* SHELLS */
c    .                                             )
      call dataio( ifsen(2), ifsen(3), ifsen(4), ifsen(5), ifsen(6)
#ifdef SHELLS
     .            ,ifsen(7)
#endif /* SHELLS */
     .                                                   )
c
c  Initialize cpu and wall clocks.  The quantities "cputime" and
c  "wclock" are the CPU and wall clock times (in seconds) in the main 
c  loop.
c
        wclock0 = 0.0
        cputime0 = 0.0
        call clocks (cputime, wclock)
#ifdef DEBUG
        write(*,"(/'ZEUSMP: clocks returned wclock = ',g12.5)") wclock
#endif /* DEBUG */
        wclock0 = wclock
        cputime0 = cputime

#ifdef PROFILING
#ifdef MPE_USED
#else
c
c Initialize timing variables
c
        do 10 l=1,2
          do 10 k=1,2
            do 10 j=1,4
              do 10 i=1,nsubstps
                timers(i,j,k,l) = 0.0
10      continue
#endif /* MPE_USED */
#endif /* PROFILING */

      if (myid .eq. 0)then
         write(6,"(/,' Set-up complete with ',i2,' warning(s):'
     &        ,' entering main loop...')") nwarn
      endif

CPS
#ifdef GRAV_FFT
C
C Create best calculating plans for FFTW to perform forward
C and backward FFT.
C
      nx=ie-is+1
      ny=je-js+1
      nz=ke-ks+1
      maxn=nx
      if(maxn.LT.ny) maxn=ny
      if(maxn.LT.nz) maxn=nz

      if(ntiles(1).GT.1) call create_plan(nx*nprocs_w,ny,nz)
      if(ntiles(2).GT.1) call create_plan(nx,ny*nprocs_w,nz)
      if(ntiles(3).GT.1) call create_plan(nx,ny,nz*nprocs_w)

#endif /* GRAV_FFT */
c
c--------------------------  start of main loop  -----------------------
c
c Execution ends when INTCHK returns a value of 1 for ifsen(1).
c
#ifdef RAD
cjh   initial N-R and lssolve iteration counters for implicit radiation
cjh   module
      totlsit = 0.D0
      totnrit = 0.D0
#endif /* RAD */
1000  continue

      nsub = 1

CPS
#if defined GRAV && defined GRAV_FFT
c
c Solve Poisson's equation for the gravitational potential.
c
c
c      turn off self-gravity (CSR)
c      turned back on, but in an ifdef now (jcv)
      if (time .gt. tgrav) call gravity
#endif /* GRAV && GRAV_FFT */
C
c
c Call optional boundary update step
c 10/18/03 jcv
c based on similar thing from zeus3d
c
        call BNDYUPDATE
c
c Evaluate all non-advective terms in the evolution equations.
c
        call srcstep
c
c Compute the advection of all field variables.
c
        call transprt
c
c Optional call user-supplied module.
c
        call SPECIAL
c
c Update the step counter and current time value.
c
        nhy   = nhy   + 1
        time  = time  + dt
c
c Check the CPU time, number of steps, output times to determine if
c a stopping criterion has been met or output is desired.
c Also check for keyboard input, depending on the value of mbatch.
c
c        call intchk( iswres, iswhdf, iswhst, iswusr )
        call intchk( ifsen(2), ifsen(3), ifsen(4), ifsen(5), ifsen(6) )
c
c Compute new timestep
c
c        call nantest
        call nudt
#ifdef MOVING_GRID
c
c Update the grid and related quantites.
c
        call newgrid
#endif
c
c Write out any desired output files now that everything has been
c updated.
c Skip dataio if the run is being terminated to avoid duplicate output.
c
        if (ifsen(1) .eq. 1) goto 2000
        call dataio( ifsen(2), ifsen(3), ifsen(4), ifsen(5), ifsen(6)
#ifdef SHELLS
     .              ,ifsen(7)
#endif /* SHELLS */
     .                                                     )
c
      goto 1000  !  Loop back to begin the next time step.
c
c--------------------------  end of main loop  -------------------------
c
c Terminate the run by making final dumps, write goodbyes
c
2000  continue
      call clocks (cputime, wclock)
#ifdef DEBUG
      write(*,"(/'ZEUSMP: clocks returned wclock = ',g12.5)") wclock
#endif /* DEBUG */
#ifdef NO_CPU_TIMER
      tused = real(wclock)
#else
      tused = real(cputime)
#endif /* NO_CPU_TIMER */
      ifsen(2) = 1
      ifsen(3) = 1
      ifsen(4) = 1
      ifsen(5) = 1
      ifsen(6) = 1
#ifdef SHELLS
      ifsen(7) = 1
#endif /* SHELLS */
      call dataio( ifsen(2), ifsen(3), ifsen(4), ifsen(5) , ifsen(6)
#ifdef SHELLS
     .            ,ifsen(7)
#endif /* SHELLS */
     .                                                   )
#ifdef MPI_USED
c
c Sum everyone's cputime (stored in tused) to get CPU time for all 
c processes.
c
      call MPI_REDUCE(tused, cpuall, 1, MPI_FLOAT,
     &                MPI_SUM, 0, comm3d, ierr )
#else /* MPI_USED */
      cpuall = tused
#endif /* MPI_USED */
      if (myid .eq. 0) then      
c
c Let's assume tused is user + system time on the master thread.
c One would also like to keep track of wall-clock time and the sum
c of CPU times used by each processor.
c
        zcs = real(nprocs_w*nhy*nx1z*nx2z*nx3z)/(tused+tiny)
        write(6,"(/' Execution terminated with ',i4,' warning(s)')") 
     &     nwarn
        write(6,"(/' Performance summary:')")
        write(6,"('  zone-cycles per cpu second =',1pe12.5)") zcs
        write(6,"('  Master CPU                 =',1pe12.5, ' sec')") 
     &     tused
        write(6,"('  Average CPU/node           =',1pe12.5, ' sec')") 
     &     cpuall/real(nprocs_w)
        write(6,"('  Wall Clock                 =',1pe12.5, ' sec')") 
     &     wclock
        write(6,"()")
        close(unit=2)
        close(unit=3)
        close(unit=30)
#ifdef TSL_DUMPS
        close(unit=31)
#endif /* TSL_DUMPS */
      endif

#ifdef PROFILING
#ifdef MPE_USED
        call MPE ...
#else
c
c Get timing data from other processors
c
#ifdef MPI_USED
      call MPI_GATHER( timers,nsubstps*16
     &,  MPI_FLOAT
     &,               atimers,nsubstps*16
     &,  MPI_FLOAT
     &,  0, comm3d, ierr)
#else /* MPI_USED */
        do 11 l=1,2
          do 11 k=1,2
            do 11 j=1,4
              do 11 i=1,nsubstps
                atimers(i,j,k,l,0) = timers(i,j,k,l)
11      continue
#endif /* MPI_USED */
c
      if (myid .eq. 0) then
        iu = 30
        open(iu,file='profile')
        write(iu
     &  ,  "( '              PROFILING REPORT')")
        write(iu,"()")
        write(iu
     &  ,  "( '              EXECUTIVE SUMMARY')")
        write(iu,"()")
c
c Find the total times spent on computation and communicaion.
c
        twcpu = 0.0
        tccpu = 0.0
        do 12 k=0,nprocs-1
          do 12 j=1,4
            do 12 i=1,nsubstps
              twcpu = twcpu + atimers(i,j,2,1,k)
              tccpu = tccpu + atimers(i,j,1,1,k)
12      continue
        write(iu,14) 
        write(iu,19) nprocs_w, wclock, cpuall/real(nprocs_w)
     &             , 100.0*twcpu/cpuall, 100.0*tccpu/cpuall
14      format(    ' THREADS     WALL      CPU    %COMP    %COMM')
19      format(2x,i3,3x,4f9.3)
        write(iu,"()")
        write(iu,"()")
c
15      format(    ' ROUTINE          TIME    %TIME'
     &  ,'      SUM     %SUM')
20      format(2x,a6,5x,4f9.3)
16      format(    ' ROUTINE  PHASE   TIME    %TIME'
     &  ,'      SUM     %SUM')
21      format(2x,a6,4x,i1,4f9.3)
c
        do 500 n=1,2      ! Loop over CPU, WALL
        do 500 m=2,1,-1   ! Loop over COMP, COMM
c
c Sort routines by CPU time spent working.
c Note that the relevant "timers" array elements are reordered
c
c**
c Average over processors instead of reporting just master thread times.
c
          do 490 k=1,nprocs-1
            do 490 j=1,4
              do 490 i=1,nsubstps
                timers(i,j,m,n) = timers(i,j,m,n) + atimers(i,j,m,n,k)
490       continue
c**
          call sort_tim(timers,nsubstps,m,n,itimers)
c
          write(iu
     &    ,  "( '              ',a4,' TIME ON ',a13)") ttype(n)
     &    ,  ctype(m)
          write(iu,"()")
          write(iu
     &    ,  "( '              BREAKDOWN BY ROUTINE')")
          write(iu,"()")
          write(iu,15)
c
c Write average thread's data to table
c
          tcum = 0.0
          if (n .eq. 1) then
            tnorm = 100.0 / ( tused  + tiny ) / real(nprocs_w)
          else
            tnorm = 100.0 / ( wclock + tiny ) / real(nprocs_w)
          endif
          do 30 i=1,nsubstps
            cpu1 = 0.0
            do 25 j=1,4
              cpu1 = cpu1 + timers(i,j,m,n)
25          continue
            tcum = tcum + cpu1
            write(iu,20) ctimers(itimers(i)),cpu1/real(nprocs_w)
     &      ,  cpu1*tnorm, tcum/real(nprocs_w), tcum*tnorm
30        continue
c
          write(iu,"()")
          write(iu,"()")
c
          write(iu,"( '              BREAKDOWN BY SUBSTEP      ')")
          write(iu,"()")
          write(iu,16)
c
c For each thread, write data for each routine and substep in the order
c dictated by that for the average thread.
c
          tcum = 0.0
          if (n .eq. 1) then
            tnorm = 100.0 / ( cpuall          + tiny )
          else
            tnorm = 100.0 / ( nprocs * wclock + tiny )
          endif
          do 35 i=1,nsubstps
            do 35 j=1,4
              cpu1 = 0.0
              do 34 l=0,nprocs-1
                cpu1 = cpu1 + atimers(itimers(i),j,m,n,l)
34            continue
              tcum = tcum + cpu1
              write(iu,21) ctimers(itimers(i)), j, cpu1/real(nprocs_w)
     &                   , cpu1*tnorm, tcum/real(nprocs_w), tcum*tnorm
35        continue
          write(iu,"()")
          write(iu,"()")
500     continue
        close (iu)
      endif
#endif /* MPE_USED */
#endif /* PROFILING */

c
#ifdef RAD
cjh   write out implicit radiation iteration summary
#ifdef MPI_USED
      if(myid_w .eq. 0) then
#endif /* MPI_USED */
         nrpert = totnrit / nhy
         lspert = totlsit / nhy
         print *,'Average LSSOLVE iter. per tstep: ',lspert
         print *,'Average imp. rad. NR iter. per tstep: ',nrpert
#ifdef MPI_USED
      endif
#endif /* MPI_USED */
#endif /* RAD */
#ifdef MPI_USED
c
c Turn off MPI
c
      call MPI_FINALIZE ( ierr )
#endif /* MPI_USED */
c
c=======================================================================
c
c    \\\\\\\\\\          E N D  P R O G R A M             //////////
c    //////////               Z E U S M P                 \\\\\\\\\\
c
c=======================================================================
c
      end
