C=======================================================================
c
c                            Developed by
c                Laboratory of Computational Astrophysics
c               University of Illinois at Urbana-Champaign
C     
C     Module:  SUBROUTINE mg3ns 
C     
C     Purpose: User-callable nonsymmetric solver
C
C     Algorithm: 
C
C     Inputs: 
C
C     Outputs: 
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      



































































































































































































































































































































































































































































#include "mgmpi.cpp"

      SUBROUTINE mg3ns (
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,
     &     XI,FI,ifng,icoords,
     &     ixmb,ixpb,iymb,iypb,izmb,izpb,
     &     XMB,XPB,YMB,YPB,ZMB,ZPB,
     &     nxd2,nyd2,nzd2,
     &     iGridTypeX,iGridTypeY,iGridTypeZ,
     &     xp,yp,zp,
     &     npxold,npyold,npzold,ipxold,ipyold,ipzold,ipold,
     &     nxi,nyi,nzi,
     &     comm,nbr,
     &     maxItFine,maxItCoarse,
     &     resTolFine,resTolCoarse,
     &     m,wgt,numPreSmooth,numPostSmooth,iSmoothIt,
     &     iFineSolver,iCoarseSolver,iSmoother,
     &     ioutput,
     &     numItFine,resFine,resRedFine,istat,totalTime,
     &     Z,nzl)

* ======================================================================
*     See 'README' for descriptions of parameters
* ======================================================================

      IMPLICIT NONE
      INTEGER nxd2,nyd2,nzd2
      REAL AFD(nxd2,nyd2,nzd2),AF1(nxd2,nyd2,nzd2)
      REAL AF2(nxd2,nyd2,nzd2),AF3(nxd2,nyd2,nzd2)
      REAL AF4(nxd2,nyd2,nzd2),AF5(nxd2,nyd2,nzd2)
      REAL AF6(nxd2,nyd2,nzd2)
*
      REAL XI(nxd2,nyd2,nzd2),FI(nxd2,nyd2,nzd2)
      INTEGER ifng,icoords
      INTEGER ixmb,ixpb,iymb,iypb,izmb,izpb
      REAL XMB(nyd2,nzd2),XPB(nyd2,nzd2)
      REAL YMB(nxd2,nzd2),YPB(nxd2,nzd2)
      REAL ZMB(nxd2,nyd2),ZPB(nxd2,nyd2)
*
      INTEGER iGridTypeX,iGridTypeY,iGridTypeZ
      REAL xp(*),yp(*),zp(*)
*
      INTEGER nxi,nyi,nzi,ix1,iy1,iz1
      INTEGER comm, nbr(6)
*
      INTEGER maxItFine,maxItCoarse
      REAL resTolFine,resTolCoarse
      INTEGER m,numPreSmooth,numPostSmooth,iSmoothIt
      REAL wgt
      INTEGER iFineSolver,iCoarseSolver,iSmoother
*
      INTEGER numItFine,istat
      REAL resFine, resRedFine,totalTime
*
      INTEGER ioutput
      INTEGER nzl
      REAL Z(nzl)

      INTEGER npxold,npyold,npzold,ipxold,ipyold,ipzold,ipold

#ifdef MGRID

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

* ----------------------------------------------------------------------

      INTEGER nxg(MLIM),nyg(MLIM),nzg(MLIM)
      INTEGER nx(MLIM),ny(MLIM),nz(MLIM)
      INTEGER nxd(MLIM),nyd(MLIM),nzd(MLIM)
      INTEGER indh(MLIM),indx(MLIM),inda(MLIM)
      INTEGER bnd(3,MLIM),bnd2(3)
      INTEGER i,ierr,ix,iy,iz
 
* ----------------------------------------------------------------------
*     Offsets into Z array  
* ----------------------------------------------------------------------

      INTEGER i_AD,i_A1,i_A2,i_A3,i_A4,i_A5,i_A6
      INTEGER i_XC,i_YSM,i_BC,i_R,i_W,i_RMG,i_ZMG
      INTEGER i_RCG,i_PCG,i_WCG,i_HX,i_HY,i_HZ,i_XP,i_YP,i_ZP
      INTEGER i_RSM,i_PSM,i_WSM
      INTEGER i_RT,i_V,i_P,i_PT,i_S,i_ST,i_T

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      INTEGER isub

      isub = SB_MG3

#ifdef PERF_EVAL
      CALL mmInitTimers
      CALL mmInitFlops
#endif

      npx = npxold
      npy = npyold
      npz = npzold
      np = npx*npy*npz
      ipx = ipxold
      ipy = ipyold
      ipz = ipzold
      ip = ipold

C     (Don't use macro since this should be called even if PERF_EVAL isn't

      gtimestart(isub) = MPI_WTIME()

      IF (ip.EQ.0 .AND. ioutput.GT.0) THEN
          print*, '==============================================='
          print*, 'BEGIN mg3ns: MGMPI Nonsymmetric Elliptic Solver'
          print*, '==============================================='
      END IF

* ----------------------------------------------------------------------
*     Make sure certain input variables are equal on all processors
* ----------------------------------------------------------------------

#ifdef MPI_USED
      CALL MPI_BCAST(ifng,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(icoords,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iGridTypeX,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iGridTypeY,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iGridTypeZ,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(npx,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(npy,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(npz,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(maxItFine,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(maxItCoarse,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(resTolFine,1,MM_REAL,0,comm,ierr)
      CALL MPI_BCAST(resTolCoarse,1,MM_REAL,0,comm,ierr)
      CALL MPI_BCAST(m,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(wgt,1,MM_REAL,0,comm,ierr)
      CALL MPI_BCAST(numPreSmooth,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(numPostSmooth,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iSmoothIt,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iFineSolver,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iCoarseSolver,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iSmoother,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(ioutput,1,MPI_INTEGER,0,comm,ierr)
#endif

      gisymmetric = 0
      gismoother = iSmoother
      gimatfree = 0

      DO i=1,nzl
         Z(i)=0.0D0
      END DO

#ifdef DUMP_INPUTS
      DO i=0,npx*npy*npz-1
         IF (ip.EQ.i) THEN
            PRINT*, 'Input parameters: ip=',ip
            print*, '----------------------------------'
            PRINT*, 'AFD corners: ',AFD(2,2,2),AFD(nxi-1,nyi-1,nzi-1)
            PRINT*, 'AF1 corners: ',AF1(1,1,1),AF1(nxi-1,nyi-1,nzi-1)
            PRINT*, 'AF2 corners: ',AF2(1,1,1),AF2(nxi-1,nyi-1,nzi-1)
            PRINT*, 'AF3 corners: ',AF3(1,1,1),AF3(nxi-1,nyi-1,nzi-1)
            PRINT*, 'AF4 corners: ',AF4(1,1,1),AF4(nxi-1,nyi-1,nzi-1)
            PRINT*, 'AF5 corners: ',AF5(1,1,1),AF5(nxi-1,nyi-1,nzi-1)
            PRINT*, 'AF6 corners: ',AF6(1,1,1),AF6(nxi-1,nyi-1,nzi-1)
            print*, 'XI corners: ',XI(2,2,2),XI(nxi-1,nyi-1,nzi-1)
            print*, 'FI corners: ',FI(2,2,2),FI(nxi-1,nyi-1,nzi-1)
            print*, 'ifng = ',ifng
            print*, 'icoords = ',icoords
            print*, 'ixmb,ixpb,etc. = ',ixmb,ixpb,iymb,iypb,izmb,izpb
            IF (ixmb.NE.MM_PERIODIC) THEN
               print*, 'XMB corners = ',XMB(2,2),XMB(nyi-1,nzi-1)
               print*, 'XPB corners = ',XPB(2,2),XPB(nyi-1,nzi-1)
            END IF
            IF (iymb.NE.MM_PERIODIC) THEN
               print*, 'YMB corners = ',YMB(2,2),YMB(nxi-1,nzi-1)
               print*, 'YPB corners = ',YPB(2,2),YPB(nxi-1,nzi-1)
            END IF
            IF (izmb.NE.MM_PERIODIC) THEN
               print*, 'ZMB corners = ',ZMB(2,2),ZMB(nxi-1,nyi-1)
               print*, 'ZPB corners = ',ZPB(2,2),ZPB(nxi-1,nyi-1)
            END IF
            PRINT*, 'nxd2,nyd2,nzd2 = ',nxd2,nyd2,nzd2
            PRINT*, 'iGridTypeX,iGridTypeY,iGridTypeZ = ',
     &           iGridTypeX,iGridTypeY,iGridTypeZ
            PRINT*, 'xp ends = ',xp(1),xp(nxi)
            PRINT*, 'yp ends = ',yp(1),yp(nyi)
            PRINT*, 'zp ends = ',zp(1),zp(nzi)
            PRINT*, 'npx,npy,npz = ',npx,npy,npz
            PRINT*, 'ip,ipx,ipy,ipz = ',ip,ipx,ipy,ipz
            PRINT*, 'nxi,nyi,nzi = ',nxi,nyi,nzi
#ifdef MPI_USED
            PRINT*, 'comm = ',comm
            PRINT*, 'nbr = ',nbr
#endif
            PRINT*, 'maxItFine = ',maxItFine
            PRINT*, 'maxItCoarse = ',maxItCoarse
            PRINT*, 'resTolFine = ',resTolFine
            PRINT*, 'resTolCoarse = ',resTolCoarse
            PRINT*, 'levels m = ',m
            PRINT*, 'weight wgt = ',wgt
            PRINT*, 'numPreSmooth = ',numPreSmooth
            PRINT*, 'numPostSmooth = ',numPostSmooth
            PRINT*, 'iSmoothIt = ',iSmoothIt
            PRINT*, 'iFineSolver = ',iFineSolver
            PRINT*, 'iCoarseSolver = ',iCoarseSolver
            PRINT*, 'iSmoother = ',gismoother
            PRINT*, 'ioutput = ',ioutput
            PRINT*, 'nzl = ',nzl
         END IF
#ifdef MPI_USED
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,'mg3ns: MPI_Barrier')")  ip
#endif
         CALL MPI_Barrier (comm,ierr)
#endif
      END DO
#endif

      icommtag = ICOMMSTART

* ----------------------------------------------------------------------
*     Get global and local problem sizes
* ----------------------------------------------------------------------

      nx(1)=nxi
      ny(1)=nyi
      nz(1)=nzi

      CALL mmGetGlobalSize (nxg,nyg,nzg,nx,ny,nz,
     &     comm,nbr,bnd
     &     )

* ----------------------------------------------------------------------
*     Verify correctness of input parameters
* ----------------------------------------------------------------------

      CALL mmCheckParams (
     &     nxg,nyg,nzg,m,
     &     ixmb,iymb,izmb,ixpb,iypb,izpb,comm)

* ----------------------------------------------------------------------
*     Distribute the domain across processors
* ----------------------------------------------------------------------

      CALL mmDistribute (
     &     nxg,nyg,nzg,nxd2,nyd2,nzd2,
     &     nxd,nyd,nzd,nx,ny,nz,m,
     &     indh,inda,indx
     &     ,
     &     comm,nbr,bnd,bnd2
     &     ,isub)

* ----------------------------------------------------------------------
*     Initialize level-independent global variables
* ----------------------------------------------------------------------

#ifdef DEBUG
      if (ip.EQ.0) print*, ip, 'mmMgSolve2: set globals'
#endif

      gicoords = icoords

      IF ( iGridTypeX.EQ.MM_IS_UNIFORM .AND.
     &     iGridTypeY.EQ.MM_IS_UNIFORM .AND.
     &     iGridTypeZ.EQ.MM_IS_UNIFORM) THEN
         giGridType = MM_IS_UNIFORM 
      ELSE
         giGridType = MM_IS_NONUNIFORM
      END IF

      CALL mmInitCoords (nx(1),ny(1),nz(1),xp(1),yp(1),zp(1),
     &     icoords,1)

* ======================================================================
*     Determine offsets into work array
* ======================================================================

      CALL mmAllocate(nxd,nyd,nzd,nxg,nyg,nzg,m,ioutput,
     &     iFineSolver,iCoarseSolver,iSmoother,
     &     i_AD,i_A1,i_A2,i_A3,i_A4,i_A5,i_A6,
     &     i_XC,i_YSM,i_BC,i_R,i_P,i_W,i_RMG,i_ZMG,
     &     i_RCG,i_PCG,i_WCG,i_HX,i_HY,i_HZ,i_XP,i_YP,i_ZP,
     &     i_RSM,i_PSM,i_WSM,
     &     i_RT,i_V,i_PT,i_S,i_ST,i_T,comm,
     &     Z,nzl,gistorage)

* ----------------------------------------------------------------------
*     Copy mesh coordinates to fine grid portion of extended array
* ----------------------------------------------------------------------

      DO i=1,nxd(1)
         Z(i_XP+i-1) = xp(i)
      END DO
      DO i=1,nyd(1)
         Z(i_YP+i-1) = yp(i)
      END DO
      DO i=1,nzd(1)
         Z(i_ZP+i-1) = zp(i)
      END DO

* ----------------------------------------------------------------------
*     Clear matrix values linking boundary nodes
* ----------------------------------------------------------------------

      IF (ipx.EQ.0) THEN
         DO iz=1,nzi
            DO iy=1,nyi
               AF2(1,iy,iz) = 0.0D0
               AF5(1,iy,iz) = 0.0D0
               AF3(1,iy,iz) = 0.0D0
               AF6(1,iy,iz) = 0.0D0
            END DO
         END DO
      END IF
      IF (ipx.EQ.npx-1) THEN
         DO iz=1,nzi
            DO iy=1,nyi
               AF2(nxi,iy,iz) = 0.0D0
               AF5(nxi,iy,iz) = 0.0D0
               AF3(nxi,iy,iz) = 0.0D0
               AF6(nxi,iy,iz) = 0.0D0
            END DO
         END DO
      END IF
      IF (ipy.EQ.0) THEN
         DO iz=1,nzi
            DO ix=1,nxi
               AF3(ix,1,iz) = 0.0D0
               AF6(ix,1,iz) = 0.0D0
               AF1(ix,1,iz) = 0.0D0
               AF4(ix,1,iz) = 0.0D0
            END DO
         END DO
      END IF
      IF (ipy.EQ.npy-1) THEN
         DO iz=1,nzi
            DO ix=1,nxi
               AF3(ix,nyi,iz) = 0.0D0
               AF6(ix,nyi,iz) = 0.0D0
               AF1(ix,nyi,iz) = 0.0D0
               AF4(ix,nyi,iz) = 0.0D0
            END DO
         END DO
      END IF
      IF (ipz.EQ.0) THEN
         DO iy=1,nyi
            DO ix=1,nxi
               AF1(ix,iy,1) = 0.0D0
               AF4(ix,iy,1) = 0.0D0
               AF2(ix,iy,1) = 0.0D0
               AF5(ix,iy,1) = 0.0D0
            END DO
         END DO
      END IF
      IF (ipz.EQ.npz-1) THEN
         DO iy=1,nyi
            DO ix=1,nxi
               AF1(ix,iy,nzi) = 0.0D0
               AF4(ix,iy,nzi) = 0.0D0
               AF2(ix,iy,nzi) = 0.0D0
               AF5(ix,iy,nzi) = 0.0D0
            END DO
         END DO
      END IF

*-----------------------------------------------------------------------
*     Get fine-grid matrix elements for lower ghost zones (for ProlongGen)
*-----------------------------------------------------------------------

CCCCC @@ Macro dependent parameters
CCCCC @@ (bnd in COMM ignored--uses bnd2)

      CALL mmGetY1Ghost (AF1,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,
     &     comm,nbr,bnd
     &     ,isub)
      CALL mmGetZ1Ghost (AF1,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,
     &     comm,nbr,bnd
     &     ,isub)
      CALL mmGetX1Ghost (AF2,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,
     &     comm,nbr,bnd
     &     ,isub)
      CALL mmGetZ1Ghost (AF2,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,
     &     comm,nbr,bnd
     &     ,isub)
      CALL mmGetX1Ghost (AF3,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,
     &     comm,nbr,bnd
     &     ,isub)
      CALL mmGetY1Ghost (AF3,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,
     &     comm,nbr,bnd
     &     ,isub)
      CALL mmGetY1Ghost (AF4,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,
     &     comm,nbr,bnd
     &     ,isub)
      CALL mmGetZ1Ghost (AF4,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,
     &     comm,nbr,bnd
     &     ,isub)
      CALL mmGetX1Ghost (AF5,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,
     &     comm,nbr,bnd
     &     ,isub)
      CALL mmGetZ1Ghost (AF5,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,
     &     comm,nbr,bnd
     &     ,isub)
      CALL mmGetX1Ghost (AF6,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,
     &     comm,nbr,bnd
     &     ,isub)
      CALL mmGetY1Ghost (AF6,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,
     &     comm,nbr,bnd
     &     ,isub)
         
* ----------------------------------------------------------------------
*     Split work array and call mgsolve2
* ----------------------------------------------------------------------

CCCCC @@ Macro dependent parameters

      CALL mmMgSolve2 (
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxd2,nyd2,nzd2,bnd2,
     &     nx,ny,nz,nxg,nyg,nzg,
     &     indh,inda,indx
     &     ,
     &     iGridTypeX,iGridTypeY,iGridTypeZ,
     &     comm,nbr,bnd,bnd2
     &     ,icoords,
     &     m,wgt,numPreSmooth,numPostSmooth,iSmoothIt,
     &     iFineSolver,iCoarseSolver,
     &     maxItFine,maxItCoarse,
     &     resTolFine,resTolCoarse,totalTime,
     &     ioutput,
     &     numItFine,resFine,resRedFine,istat,
     &     XI,nxd2,nyd2,nzd2,bnd2,
     &     FI,nxd2,nyd2,nzd2,bnd2,
     &     ifng,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxd2,nyd2,nzd2,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb,
     &     Z(i_AD),Z(i_A1),Z(i_A2),Z(i_A3),Z(i_A4),Z(i_A5),Z(i_A6),
     &     nxd,nyd,nzd,bnd,
     &     Z(i_XC),nxd,nyd,nzd,bnd,
     &     Z(i_YSM),nxd,nyd,nzd,bnd,
     &     Z(i_BC),nxd,nyd,nzd,bnd,
     &     Z(i_R),nxd,nyd,nzd,bnd,
     &     Z(i_P),nxd,nyd,nzd,bnd,
     &     Z(i_W),nxd,nyd,nzd,bnd,
     &     Z(i_RMG),nxd,nyd,nzd,bnd,
     &     Z(i_ZMG),nxd,nyd,nzd,bnd,
     &     Z(i_RCG),nxd(m),nyd(m),nzd(m),bnd(1,m),
     &     Z(i_PCG),nxd(m),nyd(m),nzd(m),bnd(1,m),
     &     Z(i_WCG),nxd(m),nyd(m),nzd(m),bnd(1,m),
     &     Z(i_RSM),nxd,nyd,nzd,bnd,
     &     Z(i_PSM),nxd,nyd,nzd,bnd,
     &     Z(i_WSM),nxd,nyd,nzd,bnd,
     &     Z(i_RT),nxd,nyd,nzd,bnd,
     &     Z(i_V),nxd,nyd,nzd,bnd,
     &     Z(i_PT),nxd,nyd,nzd,bnd,
     &     Z(i_S),nxd,nyd,nzd,bnd,
     &     Z(i_ST),nxd,nyd,nzd,bnd,
     &     Z(i_T),nxd,nyd,nzd,bnd,
     &     Z(i_XP),Z(i_YP),Z(i_ZP),Z(i_HX),Z(i_HY),Z(i_HZ),
     &     nzl,isub)

#ifdef MPI_USED
      CALL MPI_BCAST(numItFine,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(resFine,1,MM_REAL,0,comm,ierr)
      CALL MPI_BCAST(resRedFine,1,MM_REAL,0,comm,ierr)
      CALL MPI_BCAST(istat,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(totalTime,1,MM_REAL,0,comm,ierr)
#endif
      
      gtime(isub) = gtime(isub) + (MPI_WTIME() - gtimestart(isub))
      gtimecount(isub) = gtimecount(isub) + 1
      totalTime = gtime(isub)

      IF (ip.EQ.0) THEN
         CALL mmOutput (ifng,icoords,
     &        ixmb,ixpb,iymb,iypb,izmb,izpb,
     &        nxd2,nyd2,nzd2,nx,ny,nz,nxg,nyg,nzg,
     &        iGridTypeX,iGridTypeY,iGridTypeZ,
     &        maxItFine,maxItCoarse,resTolFine,resTolCoarse,m,
     &        wgt,numPreSmooth,numPostSmooth,iSmoothIt,
     &        NONSYMMETRIC,iFineSolver,iCoarseSolver,
     &        ioutput,numItFine,resFine,resRedFine,istat,
     &        totalTime,nzl)
      END IF

#ifdef PERF_EVAL
      IF (ip.EQ.0) THEN
         CALL mmWriteTimes
         CALL mmWritePercents
         CALL mmWriteFlops
         CALL mmWriteFlopRate
      END IF
      CALL mmWriteTotalFlopRate (comm)
#endif

      IF (ip.EQ.0 .AND. ioutput.GT.0) THEN
          print*, '============================================='
          print*, 'END mg3ns: MGMPI Nonsymmetric Elliptic Solver'
          print*, '============================================='
      END IF

#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  mgmpi
C     
C     Purpose: User-callable functions and subroutines
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"


C=======================================================================
C     Initialization
C=======================================================================

      SUBROUTINE mmInitMgmpi
C     MODULE:  mmInitMgmpi
C     PURPOSE: Initialize default parameters
      IMPLICIT NONE
#include "mgmpi_internal.h"
      CALL mmSetSolver (MM_MG_SOLVER)
      CALL mmSetCoarsening (.TRUE.,.TRUE.,.TRUE.)
      CALL mmSetCollapsed (.FALSE.)
      RETURN
      END

C=======================================================================
C     Problem
C=======================================================================

      SUBROUTINE mmSetRhsType (irhs)
      INTEGER irhs
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      INTEGER FUNCTION mmGetRhsType ()
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      SUBROUTINE mmSetBcType (idir, ibctype)
      INTEGER idir, ibctype
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      INTEGER FUNCTION mmGetBcType (idir)
      INTEGER idir
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      SUBROUTINE mmSetGeometry (igeom)
      INTEGER igeom
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      INTEGER FUNCTION mmGetGeometry ()
      IMPLICIT NONE
      RETURN
      END

C=======================================================================
C     Discretization
C=======================================================================

      SUBROUTINE mmSetGridType (igridtype)
      INTEGER igridtype
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      INTEGER FUNCTION mmGetGridType  ()
      IMPLICIT NONE
      RETURN
      END

C=======================================================================
C     Solver
C=======================================================================

      SUBROUTINE mmSetSolver (isolver)
C     MODULE:  mmSetSolverMG
C     PURPOSE: Set MGMPI to use the specified solver
      IMPLICIT NONE
      INTEGER isolver
#include "mgmpi_internal.h"
      gFineSolver = isolver
      RETURN
      END
C-----------------------------------------------------------------------

      INTEGER FUNCTION mmGetSolver ()
C     MODULE: FUNCTION mmGetSolver 
C     PURPOSE:
      IMPLICIT NONE
#include "mgmpi_internal.h"
      mmGetSolver = gFineSolver
      RETURN
      END

C-----------------------------------------------------------------------

      SUBROUTINE mmSetIterLimit (maxItFine)
      IMPLICIT NONE
      INTEGER maxItFine
#include "mgmpi_internal.h"
      gMaxItFine = maxItFine
      RETURN
      END

C-----------------------------------------------------------------------

      INTEGER FUNCTION mmGetIterLimit ()
      IMPLICIT NONE
#include "mgmpi_internal.h"
      mmGetIterLimit = gMaxItFine
      RETURN
      END

C-----------------------------------------------------------------------

      SUBROUTINE mmSetTolerance (rtol)
      REAL rtol
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      REAL FUNCTION mmGetTolerance  ()
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      SUBROUTINE mmSetStoppingCriteria (istopcrit)
      IMPLICIT NONE
      INTEGER istopcrit
      RETURN
      END

C-----------------------------------------------------------------------

      INTEGER FUNCTION mmGetStoppingCriteria ()
      IMPLICIT NONE
      RETURN
      END

C=======================================================================
C     Multigrid
C=======================================================================

      SUBROUTINE mmSetCoarsening (lc1,lc2,lc3)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      LOGICAL lc1,lc2,lc3
      gmmlc1 = lc1
      gmmlc2 = lc2
      gmmlc3 = lc3
      RETURN
      END

C-----------------------------------------------------------------------

      SUBROUTINE mmGetCoarsening (lc1,lc2,lc3)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      LOGICAL lc1,lc2,lc3
      lc1 = gmmlc1
      lc2 = gmmlc2
      lc3 = gmmlc3
      RETURN
      END

C-----------------------------------------------------------------------

      SUBROUTINE mmSetCollapsed (isCollapsed)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      LOGICAL isCollapsed
      gmmIsCollapsed = isCollapsed
      IF (isCollapsed) THEN
         PRINT*, '-- MGMPI warning --: mmSetCollapsed(.TRUE.) called;'
         PRINT*, '-- MGMPI warning --:   Collapsed coarse-grid solves'
         PRINT*, '-- MGMPI warning --:   not implemented yet: setting'
         PRINT*, '-- MGMPI warning --:   gmmIsCollapsed = .FALSE.'
         gmmIsCollapsed = .FALSE.
      END IF
      RETURN
      END

C-----------------------------------------------------------------------

      LOGICAL FUNCTION mmGetCollapsed ()
      IMPLICIT NONE
#include "mgmpi_internal.h"
      mmGetCollapsed = gmmIsCollapsed
      RETURN
      END

C-----------------------------------------------------------------------

      SUBROUTINE mmSetCoarseSolver (icoarsesolver)
      INTEGER icoarsesolver
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      INTEGER FUNCTION mmGetCoarseSolver  ()
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      SUBROUTINE mmSetGridLevels (ilevels)
      INTEGER ilevels
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      INTEGER FUNCTION mmGetGridLevels  ()
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      SUBROUTINE mmSetCoarseIterLimit (maxiter)
      INTEGER maxiter
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      INTEGER FUNCTION mmGetCoarseIterLimit ()
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      SUBROUTINE mmSetCoarseTolerance (rtol)
      REAL rtol
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      REAL FUNCTION mmGetCoarseTolerance ()
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      SUBROUTINE mmSetSmoother (ismoother)
      INTEGER ismoother
C     MODULE:  mmSetSmoother
C     PURPOSE:
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      INTEGER FUNCTION mmGetSmoother  ()
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      SUBROUTINE mmSetNumSmoothings (ipre, ipost)
      INTEGER ipre, ipost
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      SUBROUTINE mmGetNumPreSmooths (ipre, ipost)
      INTEGER ipre, ipost
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      SUBROUTINE mmSetSmootherWeight (wgt)
      REAL wgt
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      REAL FUNCTION mmGetSmootherWeight  ()
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      SUBROUTINE mmSetSmootherIterLimit (iter)
      INTEGER iter
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      INTEGER FUNCTION mmGetSmootherIterLimit ()
      IMPLICIT NONE
      RETURN
      END

C=======================================================================
C     Output
C=======================================================================

      SUBROUTINE mmSetOutputLevel (ioutput)
      INTEGER ioutput
C     MODULE:  mmSetOutputLevel
C     PURPOSE:
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      INTEGER FUNCTION mmGetOutputLevel  ()
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      INTEGER FUNCTION mmGetIter ()
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      REAL FUNCTION mmGetResidual ()
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      REAL FUNCTION mmGetResidReduction ()
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      INTEGER FUNCTION mmGetStatus ()
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      REAL FUNCTION mmGetTime ()
      IMPLICIT NONE
      RETURN
      END

C-----------------------------------------------------------------------

      INTEGER FUNCTION mmGetStorage ()
      RETURN
      END

C=======================================================================

      SUBROUTINE MGMPIp (
     &     XI,FI,ifng,icoords,
     &     ixmb,ixpb,iymb,iypb,izmb,izpb,
     &     XMB,XPB,YMB,YPB,ZMB,ZPB,
     &     nxd2,nyd2,nzd2,
     &     iGridTypeX,iGridTypeY,iGridTypeZ,
     &     xp,yp,zp,
     &     npxold,npyold,npzold,ipxold,ipyold,ipzold,ipold,
     &     nxi,nyi,nzi,
     &     comm,nbr,
     &     maxItFine,maxItCoarse,
     &     resTolFine,resTolCoarse,
     &     m,wgt,numPreSmooth,numPostSmooth,iSmoothIt,
     &     iFineSolver,iCoarseSolver,iSmoother,
     &     ioutput,
     &     numItFine,resFine,resRedFine,istat,totalTime,
     &     Z,nzl)
C     MODULE:  MGMPI
C     PURPOSE: Main solver 
      IMPLICIT NONE
      INTEGER nxd2,nyd2,nzd2
*
      REAL XI(nxd2,nyd2,nzd2),FI(nxd2,nyd2,nzd2)
      INTEGER ifng,icoords
      INTEGER ixmb,ixpb,iymb,iypb,izmb,izpb
      REAL XMB(nyd2,nzd2),XPB(nyd2,nzd2)
      REAL YMB(nxd2,nzd2),YPB(nxd2,nzd2)
      REAL ZMB(nxd2,nyd2),ZPB(nxd2,nyd2)
*
      INTEGER iGridTypeX,iGridTypeY,iGridTypeZ
      REAL xp(*),yp(*),zp(*)
*
      INTEGER nxi,nyi,nzi
      INTEGER comm, nbr(6)
*
      INTEGER maxItFine,maxItCoarse
      REAL resTolFine,resTolCoarse
      INTEGER m,numPreSmooth,numPostSmooth,iSmoothIt
      REAL wgt
      INTEGER iFineSolver,iCoarseSolver,iSmoother
*
      INTEGER numItFine,istat
      REAL resFine, resRedFine,totalTime
*
      INTEGER ioutput
      INTEGER nzl
      REAL Z(nzl)

      INTEGER npxold,npyold,npzold,ipxold,ipyold,ipzold,ipold

#include "mgmpi_internal.h"

      INTEGER maxdims,dims(3),coords(3),ierr
      LOGICAL periods(3)

      maxdims = 3
      
#ifdef MPI_USED
      CALL MPI_Cart_Get (comm,maxdims,dims,periods,coords,ierr)
      CALL MPI_Cart_Rank (comm,coords,ip,ierr)

      npx = dims(1)
      npy = dims(2)
      npz = dims(3)
      np = npx*npy*npz
      ipx = coords(1)
      ipy = coords(2)
      ipz = coords(3)
#else
      npx = 1
      npy = 1
      npz = 1
      np = 1
      ipx = 0
      ipy = 0
      ipz = 0
#endif

      CALL mg3p (
     &     XI,FI,ifng,icoords,
     &     ixmb,ixpb,iymb,iypb,izmb,izpb,
     &     XMB,XPB,YMB,YPB,ZMB,ZPB,
     &     nxd2,nyd2,nzd2,
     &     iGridTypeX,iGridTypeY,iGridTypeZ,
     &     xp,yp,zp,
     &     npxold,npyold,npzold,ipxold,ipyold,ipzold,ipold,
     &     nxi,nyi,nzi,
     &     comm,nbr,
     &     maxItFine,maxItCoarse,
     &     resTolFine,resTolCoarse,
     &     m,wgt,numPreSmooth,numPostSmooth,iSmoothIt,
     &     gFineSolver,iCoarseSolver,iSmoother,
     &     ioutput,
     &     numItFine,resFine,resRedFine,istat,totalTime,
     &     Z,nzl)

      RETURN
      END

C-----------------------------------------------------------------------

      SUBROUTINE MGMPIs (
     &     AFD,AF1,AF2,AF3,
     &     XI,FI,ifng,icoords,
     &     ixmb,ixpb,iymb,iypb,izmb,izpb,
     &     XMB,XPB,YMB,YPB,ZMB,ZPB,
     &     nxd2,nyd2,nzd2,
     &     iGridTypeX,iGridTypeY,iGridTypeZ,
     &     xp,yp,zp,
     &     npxold,npyold,npzold,ipxold,ipyold,ipzold,ipold,
     &     nxi,nyi,nzi,
     &     comm,nbr,
     &     maxItFine,maxItCoarse,
     &     resTolFine,resTolCoarse,
     &     m,wgt,numPreSmooth,numPostSmooth,iSmoothIt,
     &     iFineSolver,iCoarseSolver,iSmoother,
     &     ioutput,
     &     numItFine,resFine,resRedFine,istat,totalTime,
     &     Z,nzl)
C     MODULE:  MGMPI
C     PURPOSE: Main solver 
      IMPLICIT NONE
      INTEGER nxd2,nyd2,nzd2
      REAL AFD(nxd2,nyd2,nzd2),AF1(nxd2,nyd2,nzd2)
      REAL AF2(nxd2,nyd2,nzd2),AF3(nxd2,nyd2,nzd2)
*
      REAL XI(nxd2,nyd2,nzd2),FI(nxd2,nyd2,nzd2)
      INTEGER ifng,icoords
      INTEGER ixmb,ixpb,iymb,iypb,izmb,izpb
      REAL XMB(nyd2,nzd2),XPB(nyd2,nzd2)
      REAL YMB(nxd2,nzd2),YPB(nxd2,nzd2)
      REAL ZMB(nxd2,nyd2),ZPB(nxd2,nyd2)
*
      INTEGER iGridTypeX,iGridTypeY,iGridTypeZ
      REAL xp(*),yp(*),zp(*)
*
      INTEGER nxi,nyi,nzi
      INTEGER comm, nbr(6)
*
      INTEGER maxItFine,maxItCoarse
      REAL resTolFine,resTolCoarse
      INTEGER m,numPreSmooth,numPostSmooth,iSmoothIt
      REAL wgt
      INTEGER iFineSolver,iCoarseSolver,iSmoother
*
      INTEGER numItFine,istat
      REAL resFine, resRedFine,totalTime
*
      INTEGER ioutput
      INTEGER nzl
      REAL Z(nzl)

      INTEGER npxold,npyold,npzold,ipxold,ipyold,ipzold,ipold

#include "mgmpi_internal.h"

      CALL mg3s (
     &     AFD,AF1,AF2,AF3,
     &     XI,FI,ifng,icoords,
     &     ixmb,ixpb,iymb,iypb,izmb,izpb,
     &     XMB,XPB,YMB,YPB,ZMB,ZPB,
     &     nxd2,nyd2,nzd2,
     &     iGridTypeX,iGridTypeY,iGridTypeZ,
     &     xp,yp,zp,
     &     npxold,npyold,npzold,ipxold,ipyold,ipzold,ipold,
     &     nxi,nyi,nzi,
     &     comm,nbr,
     &     maxItFine,maxItCoarse,
     &     resTolFine,resTolCoarse,
     &     m,wgt,numPreSmooth,numPostSmooth,iSmoothIt,
     &     gFineSolver,iCoarseSolver,iSmoother,
     &     ioutput,
     &     numItFine,resFine,resRedFine,istat,totalTime,
     &     Z,nzl)

      RETURN
      END

C-----------------------------------------------------------------------

      SUBROUTINE MGMPIns (
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,
     &     XI,FI,ifng,icoords,
     &     ixmb,ixpb,iymb,iypb,izmb,izpb,
     &     XMB,XPB,YMB,YPB,ZMB,ZPB,
     &     nxd2,nyd2,nzd2,
     &     iGridTypeX,iGridTypeY,iGridTypeZ,
     &     xp,yp,zp,
     &     npxold,npyold,npzold,ipxold,ipyold,ipzold,ipold,
     &     nxi,nyi,nzi,
     &     comm,nbr,
     &     maxItFine,maxItCoarse,
     &     resTolFine,resTolCoarse,
     &     m,wgt,numPreSmooth,numPostSmooth,iSmoothIt,
     &     iFineSolver,iCoarseSolver,iSmoother,
     &     ioutput,
     &     numItFine,resFine,resRedFine,istat,totalTime,
     &     Z,nzl)
C     MODULE:  MGMPI
C     PURPOSE: Main solver 
      IMPLICIT NONE
      INTEGER nxd2,nyd2,nzd2
      REAL AFD(nxd2,nyd2,nzd2),AF1(nxd2,nyd2,nzd2)
      REAL AF2(nxd2,nyd2,nzd2),AF3(nxd2,nyd2,nzd2)
      REAL AF4(nxd2,nyd2,nzd2),AF5(nxd2,nyd2,nzd2)
      REAL AF6(nxd2,nyd2,nzd2)
*
      REAL XI(nxd2,nyd2,nzd2),FI(nxd2,nyd2,nzd2)
      INTEGER ifng,icoords
      INTEGER ixmb,ixpb,iymb,iypb,izmb,izpb
      REAL XMB(nyd2,nzd2),XPB(nyd2,nzd2)
      REAL YMB(nxd2,nzd2),YPB(nxd2,nzd2)
      REAL ZMB(nxd2,nyd2),ZPB(nxd2,nyd2)
*
      INTEGER iGridTypeX,iGridTypeY,iGridTypeZ
      REAL xp(*),yp(*),zp(*)
*
      INTEGER nxi,nyi,nzi
      INTEGER comm, nbr(6)
*
      INTEGER maxItFine,maxItCoarse
      REAL resTolFine,resTolCoarse
      INTEGER m,numPreSmooth,numPostSmooth,iSmoothIt
      REAL wgt
      INTEGER iFineSolver,iCoarseSolver,iSmoother
*
      INTEGER numItFine,istat
      REAL resFine, resRedFine,totalTime
*
      INTEGER ioutput
      INTEGER nzl
      REAL Z(nzl)

      INTEGER npxold,npyold,npzold,ipxold,ipyold,ipzold,ipold

#include "mgmpi_internal.h"

      CALL mg3ns (
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,
     &     XI,FI,ifng,icoords,
     &     ixmb,ixpb,iymb,iypb,izmb,izpb,
     &     XMB,XPB,YMB,YPB,ZMB,ZPB,
     &     nxd2,nyd2,nzd2,
     &     iGridTypeX,iGridTypeY,iGridTypeZ,
     &     xp,yp,zp,
     &     npxold,npyold,npzold,ipxold,ipyold,ipzold,ipold,
     &     nxi,nyi,nzi,
     &     comm,nbr,
     &     maxItFine,maxItCoarse,
     &     resTolFine,resTolCoarse,
     &     m,wgt,numPreSmooth,numPostSmooth,iSmoothIt,
     &     gFineSolver,iCoarseSolver,iSmoother,
     &     ioutput,
     &     numItFine,resFine,resRedFine,istat,totalTime,
     &     Z,nzl)

      RETURN
      END

C=======================================================================
C     
C     Module:  SUBROUTINE mg3s 
C     
C     Purpose: User-callable symmetric solver
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mg3s (
     &     AFD,AF1,AF2,AF3,
     &     XI,FI,ifng,icoords,
     &     ixmb,ixpb,iymb,iypb,izmb,izpb,
     &     XMB,XPB,YMB,YPB,ZMB,ZPB,
     &     nxd2,nyd2,nzd2,
     &     iGridTypeX,iGridTypeY,iGridTypeZ,
     &     xp,yp,zp,
     &     npxold,npyold,npzold,ipxold,ipyold,ipzold,ipold,
     &     nxi,nyi,nzi,
     &     comm,nbr,
     &     maxItFine,maxItCoarse,
     &     resTolFine,resTolCoarse,
     &     m,wgt,numPreSmooth,numPostSmooth,iSmoothIt,
     &     iFineSolver,iCoarseSolver,iSmoother,
     &     ioutput,
     &     numItFine,resFine,resRedFine,istat,totalTime,
     &     Z,nzl)

* ======================================================================
*     See 'README' for descriptions of parameters
* ======================================================================

      IMPLICIT NONE
      INTEGER nxd2,nyd2,nzd2
      REAL AFD(nxd2,nyd2,nzd2),AF1(nxd2,nyd2,nzd2)
      REAL AF2(nxd2,nyd2,nzd2),AF3(nxd2,nyd2,nzd2)
*
      REAL XI(nxd2,nyd2,nzd2),FI(nxd2,nyd2,nzd2)
      INTEGER ifng,icoords
      INTEGER ixmb,ixpb,iymb,iypb,izmb,izpb
      REAL XMB(nyd2,nzd2),XPB(nyd2,nzd2)
      REAL YMB(nxd2,nzd2),YPB(nxd2,nzd2)
      REAL ZMB(nxd2,nyd2),ZPB(nxd2,nyd2)
*
      INTEGER iGridTypeX,iGridTypeY,iGridTypeZ
      REAL xp(*),yp(*),zp(*)
*
      INTEGER nxi,nyi,nzi
      INTEGER comm, nbr(6)
*
      INTEGER maxItFine,maxItCoarse
      REAL resTolFine,resTolCoarse
      INTEGER m,numPreSmooth,numPostSmooth,iSmoothIt
      REAL wgt
      INTEGER iFineSolver,iCoarseSolver,iSmoother
*
      INTEGER numItFine,istat
      REAL resFine, resRedFine,totalTime
*
      INTEGER ioutput
      INTEGER nzl
      REAL Z(nzl)

      INTEGER npxold,npyold,npzold,ipxold,ipyold,ipzold,ipold

#ifdef MGRID

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

* ----------------------------------------------------------------------

      INTEGER nxg(MLIM),nyg(MLIM),nzg(MLIM)
      INTEGER nx(MLIM),ny(MLIM),nz(MLIM)
      INTEGER nxd(MLIM),nyd(MLIM),nzd(MLIM)
      INTEGER indh(MLIM),indx(MLIM),inda(MLIM)
      INTEGER bnd(3,MLIM),bnd2(3)
      INTEGER i,ierr,ix,iy,iz
 
* ----------------------------------------------------------------------
*     Offsets into Z array  
* ----------------------------------------------------------------------

      INTEGER i_AD,i_A1,i_A2,i_A3,i_A4,i_A5,i_A6
      INTEGER i_XC,i_YSM,i_BC,i_R,i_W,i_RMG,i_ZMG
      INTEGER i_RCG,i_PCG,i_WCG,i_HX,i_HY,i_HZ,i_XP,i_YP,i_ZP
      INTEGER i_RSM,i_PSM,i_WSM
      INTEGER i_RT,i_V,i_P,i_PT,i_S,i_ST,i_T

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      INTEGER isub

      isub = SB_MG3

#ifdef PERF_EVAL
      CALL mmInitTimers
      CALL mmInitFlops
#endif

C     (Don't use macro since this should be called even if PERF_EVAL isn't

      npx = npxold
      npy = npyold
      npz = npzold
      np = npx*npy*npz
      ipx = ipxold
      ipy = ipyold
      ipz = ipzold
      ip = ipold

      gtimestart(isub) = MPI_WTIME()

      IF (ip.EQ.0 .AND. ioutput.GT.0) THEN
          print*, '==========================================='
          print*, 'BEGIN mg3s: MGMPI Symmetric Elliptic Solver'
          print*, '==========================================='
      END IF

* ----------------------------------------------------------------------
*     Make sure certain input variables are equal on all processors
* ----------------------------------------------------------------------

#ifdef MPI_USED
      CALL MPI_BCAST(ifng,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(icoords,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iGridTypeX,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iGridTypeY,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iGridTypeZ,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(npx,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(npy,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(npz,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(maxItFine,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(maxItCoarse,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(resTolFine,1,MM_REAL,0,comm,ierr)
      CALL MPI_BCAST(resTolCoarse,1,MM_REAL,0,comm,ierr)
      CALL MPI_BCAST(m,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(wgt,1,MM_REAL,0,comm,ierr)
      CALL MPI_BCAST(numPreSmooth,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(numPostSmooth,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iSmoothIt,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iFineSolver,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iCoarseSolver,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iSmoother,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(ioutput,1,MPI_INTEGER,0,comm,ierr)
#endif

      gisymmetric = 1
      gismoother = iSmoother
      gimatfree = 0

#ifdef DEBUG
      IF (ip.EQ.0) print*, 'Calling VectorIsNan(XI)'
      CALL mmVectorIsNan (XI,nxd2,nyd2,nzd2,bnd2,nxi,nyi,nzi,comm)
      IF (ip.EQ.0) print*, 'Calling VectorIsNan(FI)'
      CALL mmVectorIsNan (FI,nxd2,nyd2,nzd2,bnd2,nxi,nyi,nzi,comm)
      IF (ip.EQ.0) print*, 'Calling VectorIsNan(AFD)'
      CALL mmVectorIsNan (AFD,nxd2,nyd2,nzd2,bnd2,nxi,nyi,nzi,comm)
      IF (ip.EQ.0) print*, 'Calling VectorIsNan(AF1)'
      CALL mmVectorIsNan (AF1,nxd2,nyd2,nzd2,bnd2,nxi,nyi,nzi,comm)
      IF (ip.EQ.0) print*, 'Calling VectorIsNan(AF2)'
      CALL mmVectorIsNan (AF2,nxd2,nyd2,nzd2,bnd2,nxi,nyi,nzi,comm)
      IF (ip.EQ.0) print*, 'Calling VectorIsNan(AF3)'
      CALL mmVectorIsNan (AF3,nxd2,nyd2,nzd2,bnd2,nxi,nyi,nzi,comm)
#endif

C@@@@@@@@@
C      DO i=1,nzl
C         Z(i)=0.0D0
C      END DO
C@@@@@@@@@

#ifdef DUMP_INPUTS
      DO i=0,npx*npy*npz-1
         IF (ip.EQ.i) THEN
            PRINT*, 'Input parameters: ip=',ip
            print*, '----------------------------------'
            PRINT*, 'AFD corners: ',AFD(2,2,2),AFD(nxi-1,nyi-1,nzi-1)
            PRINT*, 'AF1 corners: ',AF1(1,1,1),AF1(nxi-1,nyi-1,nzi-1)
            PRINT*, 'AF2 corners: ',AF2(1,1,1),AF2(nxi-1,nyi-1,nzi-1)
            PRINT*, 'AF3 corners: ',AF3(1,1,1),AF3(nxi-1,nyi-1,nzi-1)
            print*, 'XI corners: ',XI(2,2,2),XI(nxi-1,nyi-1,nzi-1)
            print*, 'FI corners: ',FI(2,2,2),FI(nxi-1,nyi-1,nzi-1)
            print*, 'ifng = ',ifng
            print*, 'icoords = ',icoords
            print*, 'ixmb,ixpb,etc. = ',ixmb,ixpb,iymb,iypb,izmb,izpb
            IF (ixmb.NE.MM_PERIODIC) THEN
               print*, 'XMB corners = ',XMB(2,2),XMB(nyi-1,nzi-1)
               print*, 'XPB corners = ',XPB(2,2),XPB(nyi-1,nzi-1)
            END IF
            IF (iymb.NE.MM_PERIODIC) THEN
               print*, 'YMB corners = ',YMB(2,2),YMB(nxi-1,nzi-1)
               print*, 'YPB corners = ',YPB(2,2),YPB(nxi-1,nzi-1)
            END IF
            IF (izmb.NE.MM_PERIODIC) THEN
               print*, 'ZMB corners = ',ZMB(2,2),ZMB(nxi-1,nyi-1)
               print*, 'ZPB corners = ',ZPB(2,2),ZPB(nxi-1,nyi-1)
            END IF
            PRINT*, 'nxd2,nyd2,nzd2 = ',nxd2,nyd2,nzd2
            PRINT*, 'iGridTypeX,iGridTypeY,iGridTypeZ = ',
     &           iGridTypeX,iGridTypeY,iGridTypeZ
            PRINT*, 'xp ends = ',xp(1),xp(nxi)
            PRINT*, 'yp ends = ',yp(1),yp(nyi)
            PRINT*, 'zp ends = ',zp(1),zp(nzi)
            PRINT*, 'npx,npy,npz = ',npx,npy,npz
            PRINT*, 'ip,ipx,ipy,ipz = ',ip,ipx,ipy,ipz
            PRINT*, 'nxi,nyi,nzi = ',nxi,nyi,nzi
#ifdef MPI_USED
            PRINT*, 'comm = ',comm
            PRINT*, 'nbr = ',nbr
#endif
            PRINT*, 'maxItFine = ',maxItFine
            PRINT*, 'maxItCoarse = ',maxItCoarse
            PRINT*, 'resTolFine = ',resTolFine
            PRINT*, 'resTolCoarse = ',resTolCoarse
            PRINT*, 'levels m = ',m
            PRINT*, 'weight wgt = ',wgt
            PRINT*, 'numPreSmooth = ',numPreSmooth
            PRINT*, 'numPostSmooth = ',numPostSmooth
            PRINT*, 'iSmoothIt = ',iSmoothIt
            PRINT*, 'iFineSolver = ',iFineSolver
            PRINT*, 'iCoarseSolver = ',iCoarseSolver
            PRINT*, 'iSmoother = ',gismoother
            PRINT*, 'ioutput = ',ioutput
            PRINT*, 'nzl = ',nzl
         END IF
#ifdef MPI_USED
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,'mg3s: MPI_Barrier')")  ip
#endif
         CALL MPI_Barrier (comm,ierr)
#endif
      END DO
#endif 


#ifdef DEBUG
      IF (ip.EQ.0) print*, 'Input vectors'
CCCCC @@ Macro dependent parameters
      CALL mmVectorInfo ('XI  ',XI,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,comm)
CCCCC @@ Macro dependent parameters
      CALL mmVectorInfo ('FI  ',FI,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,comm)
CCCCC @@ Macro dependent parameters
      CALL mmVectorInfo ('AFD ',AFD,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,comm)
CCCCC @@ Macro dependent parameters
      CALL mmVectorInfo ('AF1 ',AF1,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,comm)
CCCCC @@ Macro dependent parameters
      CALL mmVectorInfo ('AF2 ',AF2,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,comm)
CCCCC @@ Macro dependent parameters
      CALL mmVectorInfo ('AF3 ',AF3,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,comm)
#endif

      icommtag = ICOMMSTART

* ----------------------------------------------------------------------
*     Get global and local problem sizes
* ----------------------------------------------------------------------

      nx(1)=nxi
      ny(1)=nyi
      nz(1)=nzi

      CALL mmGetGlobalSize (nxg,nyg,nzg,nx,ny,nz,
     &     comm,nbr,bnd
     &     )

* ----------------------------------------------------------------------
*     Verify correctness of input parameters
* ----------------------------------------------------------------------

      CALL mmCheckParams (
     &     nxg,nyg,nzg,m,
     &     ixmb,iymb,izmb,ixpb,iypb,izpb,comm)

* ----------------------------------------------------------------------
*     Distribute the domain across processors
* ----------------------------------------------------------------------

      CALL mmDistribute (
     &     nxg,nyg,nzg,nxd2,nyd2,nzd2,
     &     nxd,nyd,nzd,nx,ny,nz,m,
     &     indh,inda,indx
     &     ,
     &     comm,nbr,bnd,bnd2
     &     ,isub)

* ----------------------------------------------------------------------
*     Initialize level-independent global variables
* ----------------------------------------------------------------------

#ifdef DEBUG
      if (ip.EQ.0) print*, ip, 'mmMgSolve2: set globals'
#endif

      gicoords = icoords

      IF ( iGridTypeX.EQ.MM_IS_UNIFORM .AND.
     &     iGridTypeY.EQ.MM_IS_UNIFORM .AND.
     &     iGridTypeZ.EQ.MM_IS_UNIFORM) THEN
         giGridType = MM_IS_UNIFORM 
      ELSE
         giGridType = MM_IS_NONUNIFORM
      END IF

      CALL mmInitCoords (nx(1),ny(1),nz(1),xp(1),yp(1),zp(1),
     &     icoords,1)

* ======================================================================
*     Determine offsets into work array
* ======================================================================

      CALL mmAllocate(nxd,nyd,nzd,nxg,nyg,nzg,m,ioutput,
     &     iFineSolver,iCoarseSolver,iSmoother,
     &     i_AD,i_A1,i_A2,i_A3,i_A4,i_A5,i_A6,
     &     i_XC,i_YSM,i_BC,i_R,i_P,i_W,i_RMG,i_ZMG,
     &     i_RCG,i_PCG,i_WCG,i_HX,i_HY,i_HZ,i_XP,i_YP,i_ZP,
     &     i_RSM,i_PSM,i_WSM,
     &     i_RT,i_V,i_PT,i_S,i_ST,i_T,comm,
     &     Z,nzl,gistorage)

* ----------------------------------------------------------------------
*     Copy mesh coordinates to fine grid portion of extended array
* ----------------------------------------------------------------------

      DO i=1,nxd(1)
         Z(i_XP+i-1) = xp(i)
      END DO
      DO i=1,nyd(1)
         Z(i_YP+i-1) = yp(i)
      END DO
      DO i=1,nzd(1)
         Z(i_ZP+i-1) = zp(i)
      END DO

* ----------------------------------------------------------------------
*     Clear matrix values linking boundary nodes
* ----------------------------------------------------------------------

      IF (ipx.EQ.0) THEN
         DO iz=1,nzi
            DO iy=1,nyi
               AF2(1,iy,iz) = 0.0D0
               AF3(1,iy,iz) = 0.0D0
            END DO
         END DO
      END IF
      IF (ipx.EQ.npx-1) THEN
         DO iz=1,nzi
            DO iy=1,nyi
               AF2(nxi,iy,iz) = 0.0D0
               AF3(nxi,iy,iz) = 0.0D0
            END DO
         END DO
      END IF
      IF (ipy.EQ.0) THEN
         DO iz=1,nzi
            DO ix=1,nxi
               AF3(ix,1,iz) = 0.0D0
               AF1(ix,1,iz) = 0.0D0
            END DO
         END DO
      END IF
      IF (ipy.EQ.npy-1) THEN
         DO iz=1,nzi
            DO ix=1,nxi
               AF3(ix,nyi,iz) = 0.0D0
               AF1(ix,nyi,iz) = 0.0D0
            END DO
         END DO
      END IF
      IF (ipz.EQ.0) THEN
         DO iy=1,nyi
            DO ix=1,nxi
               AF1(ix,iy,1) = 0.0D0
               AF2(ix,iy,1) = 0.0D0
            END DO
         END DO
      END IF
      IF (ipz.EQ.npz-1) THEN
         DO iy=1,nyi
            DO ix=1,nxi
               AF1(ix,iy,nzi) = 0.0D0
               AF2(ix,iy,nzi) = 0.0D0
            END DO
         END DO
      END IF

*-----------------------------------------------------------------------
*     Get fine-grid matrix elements for lower ghost zones (for ProlongGen)
*-----------------------------------------------------------------------

CCCCC @@ Macro dependent parameters
CCCCC @@ (bnd in COMM ignored--uses bnd2)

      CALL mmGetY1Ghost (AF1,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,
     &     comm,nbr,bnd
     &     ,isub)
      CALL mmGetZ1Ghost (AF1,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,
     &     comm,nbr,bnd
     &     ,isub)
      CALL mmGetX1Ghost (AF2,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,
     &     comm,nbr,bnd
     &     ,isub)
      CALL mmGetZ1Ghost (AF2,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,
     &     comm,nbr,bnd
     &     ,isub)
      CALL mmGetX1Ghost (AF3,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,
     &     comm,nbr,bnd
     &     ,isub)
      CALL mmGetY1Ghost (AF3,nxd2,nyd2,nzd2,bnd2,
     &     nxi,nyi,nzi,
     &     comm,nbr,bnd
     &     ,isub)
         
* ----------------------------------------------------------------------
*     Split work array and call mgsolve2
* ----------------------------------------------------------------------

CCCCC @@ Macro dependent parameters

      CALL mmMgSolve2 (
     &     AFD,AF1,AF2,AF3,AF1,AF2,AF3,nxd2,nyd2,nzd2,bnd2,
     &     nx,ny,nz,nxg,nyg,nzg,
     &     indh,inda,indx
     &     ,
     &     iGridTypeX,iGridTypeY,iGridTypeZ,
     &     comm,nbr,bnd,bnd2
     &     ,
     &     icoords,
     &     m,wgt,numPreSmooth,numPostSmooth,iSmoothIt,
     &     iFineSolver,iCoarseSolver,
     &     maxItFine,maxItCoarse,
     &     resTolFine,resTolCoarse,totalTime,
     &     ioutput,
     &     numItFine,resFine,resRedFine,istat,
     &     XI,nxd2,nyd2,nzd2,bnd2,
     &     FI,nxd2,nyd2,nzd2,bnd2,
     &     ifng,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxd2,nyd2,nzd2,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb,
     &     Z(i_AD),Z(i_A1),Z(i_A2),Z(i_A3),Z(i_A4),Z(i_A5),Z(i_A6),
     &     nxd,nyd,nzd,bnd,
     &     Z(i_XC),nxd,nyd,nzd,bnd,
     &     Z(i_YSM),nxd,nyd,nzd,bnd,
     &     Z(i_BC),nxd,nyd,nzd,bnd,
     &     Z(i_R),nxd,nyd,nzd,bnd,
     &     Z(i_P),nxd,nyd,nzd,bnd,
     &     Z(i_W),nxd,nyd,nzd,bnd,
     &     Z(i_RMG),nxd,nyd,nzd,bnd,
     &     Z(i_ZMG),nxd,nyd,nzd,bnd,
     &     Z(i_RCG),nxd(m),nyd(m),nzd(m),bnd(1,m),
     &     Z(i_PCG),nxd(m),nyd(m),nzd(m),bnd(1,m),
     &     Z(i_WCG),nxd(m),nyd(m),nzd(m),bnd(1,m),
     &     Z(i_RSM),nxd,nyd,nzd,bnd,
     &     Z(i_PSM),nxd,nyd,nzd,bnd,
     &     Z(i_WSM),nxd,nyd,nzd,bnd,
     &     Z(i_RT),nxd,nyd,nzd,bnd,
     &     Z(i_V),nxd,nyd,nzd,bnd,
     &     Z(i_PT),nxd,nyd,nzd,bnd,
     &     Z(i_S),nxd,nyd,nzd,bnd,
     &     Z(i_ST),nxd,nyd,nzd,bnd,
     &     Z(i_T),nxd,nyd,nzd,bnd,
     &     Z(i_XP),Z(i_YP),Z(i_ZP),Z(i_HX),Z(i_HY),Z(i_HZ),
     &     nzl,isub)

#ifdef MPI_USED
      CALL MPI_BCAST(numItFine,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(resFine,1,MM_REAL,0,comm,ierr)
      CALL MPI_BCAST(resRedFine,1,MM_REAL,0,comm,ierr)
      CALL MPI_BCAST(istat,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(totalTime,1,MM_REAL,0,comm,ierr)
#endif
      
      gtime(isub) = gtime(isub) + (MPI_WTIME() - gtimestart(isub))
      gtimecount(isub) = gtimecount(isub) + 1
      totalTime = gtime(isub)

      IF (ip.EQ.0) THEN
         CALL mmOutput (ifng,icoords,
     &        ixmb,ixpb,iymb,iypb,izmb,izpb,
     &        nxd2,nyd2,nzd2,nx,ny,nz,nxg,nyg,nzg,
     &        iGridTypeX,iGridTypeY,iGridTypeZ,
     &        maxItFine,maxItCoarse,resTolFine,resTolCoarse,m,
     &        wgt,numPreSmooth,numPostSmooth,iSmoothIt,
     &        SYMMETRIC,iFineSolver,iCoarseSolver,
     &        ioutput,numItFine,resFine,resRedFine,istat,
     &        totalTime,nzl)
      END IF

#ifdef PERF_EVAL
      IF (ip.EQ.0) THEN
         CALL mmWriteTimes
         CALL mmWritePercents
         CALL mmWriteFlops
         CALL mmWriteFlopRate
      END IF
      CALL mmWriteTotalFlopRate (comm)
#endif
#ifdef DEBUG
      IF (ip.EQ.0) print*, 'Calling VectorIsNan(XI)'
      CALL mmVectorIsNan (XI,nxd2,nyd2,nzd2,bnd2,nxi,nyi,nzi,comm)
      IF (ip.EQ.0) print*, 'Calling VectorIsNan(FI)'
      CALL mmVectorIsNan (FI,nxd2,nyd2,nzd2,bnd2,nxi,nyi,nzi,comm)
      IF (ip.EQ.0) print*, 'Calling VectorIsNan(AFD)'
      CALL mmVectorIsNan (AFD,nxd2,nyd2,nzd2,bnd2,nxi,nyi,nzi,comm)
      IF (ip.EQ.0) print*, 'Calling VectorIsNan(AF1)'
      CALL mmVectorIsNan (AF1,nxd2,nyd2,nzd2,bnd2,nxi,nyi,nzi,comm)
      IF (ip.EQ.0) print*, 'Calling VectorIsNan(AF2)'
      CALL mmVectorIsNan (AF2,nxd2,nyd2,nzd2,bnd2,nxi,nyi,nzi,comm)
      IF (ip.EQ.0) print*, 'Calling VectorIsNan(AF3)'
      CALL mmVectorIsNan (AF3,nxd2,nyd2,nzd2,bnd2,nxi,nyi,nzi,comm)
#endif

      IF (ip.EQ.0 .AND. ioutput.GT.0) THEN
          print*, '========================================='
          print*, 'END mg3s: MGMPI Symmetric Elliptic Solver'
          print*, '========================================='
      END IF

#endif 

      RETURN
      END

C=======================================================================
C     
C     Module:  SUBROUTINE mg3p 
C     
C     Purpose: User-callable Poisson solver
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C               Sep 9 -- Changed i?1(k) to gmmi?(k,ip)
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mg3p (
     &     XI,FI,ifng,icoords,
     &     ixmb,ixpb,iymb,iypb,izmb,izpb,
     &     XMB,XPB,YMB,YPB,ZMB,ZPB,
     &     nxd2,nyd2,nzd2,
     &     iGridTypeX,iGridTypeY,iGridTypeZ,
     &     xp,yp,zp,
     &     npxold,npyold,npzold,ipxold,ipyold,ipzold,ipold,
     &     nxi,nyi,nzi,
     &     comm,nbr,
     &     maxItFine,maxItCoarse,
     &     resTolFine,resTolCoarse,
     &     m,wgt,numPreSmooth,numPostSmooth,iSmoothIt,
     &     iFineSolver,iCoarseSolver,iSmoother,
     &     ioutput,
     &     numItFine,resFine,resRedFine,istat,totalTime,
     &     Z,nzl)

* ======================================================================
*     See 'README' for descriptions of parameters
* ======================================================================

      IMPLICIT NONE
      INTEGER nxd2,nyd2,nzd2
      REAL XI(nxd2,nyd2,nzd2),FI(nxd2,nyd2,nzd2)
      INTEGER ifng,icoords
      INTEGER ixmb,ixpb,iymb,iypb,izmb,izpb
      REAL XMB(nyd2,nzd2),XPB(nyd2,nzd2)
      REAL YMB(nxd2,nzd2),YPB(nxd2,nzd2)
      REAL ZMB(nxd2,nyd2),ZPB(nxd2,nyd2)
*
      INTEGER iGridTypeX,iGridTypeY,iGridTypeZ
      REAL xp(*),yp(*),zp(*)
*
      INTEGER nxi,nyi,nzi
      INTEGER comm, nbr(6)
*
      INTEGER maxItFine,maxItCoarse
      REAL resTolFine,resTolCoarse
      INTEGER m,numPreSmooth,numPostSmooth,iSmoothIt
      REAL wgt
      INTEGER iFineSolver,iCoarseSolver,iSmoother
*
      INTEGER numItFine,istat
      REAL resFine, resRedFine,totalTime
*
      INTEGER ioutput
      INTEGER nzl
      REAL Z(nzl)

      INTEGER npxold,npyold,npzold,ipxold,ipyold,ipzold,ipold

#ifdef MGRID

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

* ----------------------------------------------------------------------

      INTEGER nxg(MLIM),nyg(MLIM),nzg(MLIM)
      INTEGER nx(MLIM),ny(MLIM),nz(MLIM)
      INTEGER nxd(MLIM),nyd(MLIM),nzd(MLIM)
      INTEGER indh(MLIM),indx(MLIM),inda(MLIM)
      INTEGER bnd(3,MLIM),bnd2(3)
      INTEGER i,ierr
 
* ----------------------------------------------------------------------
*     Offsets into Z array  
* ----------------------------------------------------------------------

      INTEGER i_AFD,i_AF1,i_AF2,i_AF3
      INTEGER i_AD,i_A1,i_A2,i_A3,i_A4,i_A5,i_A6
      INTEGER i_XC,i_YSM,i_BC,i_R,i_W,i_RMG,i_ZMG
      INTEGER i_RCG,i_PCG,i_WCG,i_HX,i_HY,i_HZ,i_XP,i_YP,i_ZP
      INTEGER i_RSM,i_PSM,i_WSM
      INTEGER i_RT,i_V,i_P,i_PT,i_S,i_ST,i_T

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      INTEGER isub

      isub = SB_MG3

#ifdef PERF_EVAL
      CALL mmInitTimers
      CALL mmInitFlops
#endif

      npx = npxold
      npy = npyold
      npz = npzold
      np = npx*npy*npz
      ipx = ipxold
      ipy = ipyold
      ipz = ipzold
      ip = ipold

C     (Don't use macro since this should be called even if PERF_EVAL isn't

      gtimestart(isub) = MPI_WTIME()

      IF (ip.EQ.0 .AND. ioutput.GT.0) THEN
          print*, '================================'
          print*, 'BEGIN mg3p: MGMPI Poisson Solver'
          print*, '================================'
      END IF

* ----------------------------------------------------------------------
*     Make sure certain input variables are equal on all processors
* ----------------------------------------------------------------------

#ifdef MPI_USED
      CALL MPI_BCAST(ifng,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(icoords,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iGridTypeX,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iGridTypeY,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iGridTypeZ,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(npx,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(npy,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(npz,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(maxItFine,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(maxItCoarse,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(resTolFine,1,MM_REAL,0,comm,ierr)
      CALL MPI_BCAST(resTolCoarse,1,MM_REAL,0,comm,ierr)
      CALL MPI_BCAST(m,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(wgt,1,MM_REAL,0,comm,ierr)
      CALL MPI_BCAST(numPreSmooth,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(numPostSmooth,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iSmoothIt,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iFineSolver,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iCoarseSolver,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(iSmoother,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(ioutput,1,MPI_INTEGER,0,comm,ierr)
#endif

      gisymmetric = 1
      gismoother = iSmoother
      gimatfree = 1

C@@@@@@@@@
C      DO i=1,nzl
C         Z(i)=0.0D0
C      END DO
C@@@@@@@@@

#ifdef DUMP_INPUTS
      DO i=0,npx*npy*npz-1
         IF (ip.EQ.i) THEN
            PRINT*, 'Input parameters: ip=',ip
            print*, '----------------------------------'
            print*, 'XI corners: ',XI(2,2,2),XI(nxi-1,nyi-1,nzi-1)
            print*, 'FI corners: ',FI(2,2,2),FI(nxi-1,nyi-1,nzi-1)
            print*, 'ifng = ',ifng
            print*, 'icoords = ',icoords
            print*, 'ixmb,ixpb,etc. = ',ixmb,ixpb,iymb,iypb,izmb,izpb
            IF (ixmb.NE.MM_PERIODIC) THEN
               print*, 'XMB corners = ',XMB(2,2),XMB(nyi-1,nzi-1)
               print*, 'XPB corners = ',XPB(2,2),XPB(nyi-1,nzi-1)
            END IF
            IF (iymb.NE.MM_PERIODIC) THEN
               print*, 'YMB corners = ',YMB(2,2),YMB(nxi-1,nzi-1)
               print*, 'YPB corners = ',YPB(2,2),YPB(nxi-1,nzi-1)
            END IF
            IF (izmb.NE.MM_PERIODIC) THEN
               print*, 'ZMB corners = ',ZMB(2,2),ZMB(nxi-1,nyi-1)
               print*, 'ZPB corners = ',ZPB(2,2),ZPB(nxi-1,nyi-1)
            END IF
            PRINT*, 'nxd2,nyd2,nzd2 = ',nxd2,nyd2,nzd2
            PRINT*, 'iGridTypeX,iGridTypeY,iGridTypeZ = ',
     &           iGridTypeX,iGridTypeY,iGridTypeZ
            PRINT*, 'xp ends = ',xp(1),xp(nxi)
            PRINT*, 'yp ends = ',yp(1),yp(nyi)
            PRINT*, 'zp ends = ',zp(1),zp(nzi)
            PRINT*, 'npx,npy,npz = ',npx,npy,npz
            PRINT*, 'ip,ipx,ipy,ipz = ',ip,ipx,ipy,ipz
            PRINT*, 'nxi,nyi,nzi = ',nxi,nyi,nzi
#ifdef MPI_USED
            PRINT*, 'comm = ',comm
            PRINT*, 'nbr = ',nbr
#endif
            PRINT*, 'maxItFine = ',maxItFine
            PRINT*, 'maxItCoarse = ',maxItCoarse
            PRINT*, 'resTolFine = ',resTolFine
            PRINT*, 'resTolCoarse = ',resTolCoarse
            PRINT*, 'levels m = ',m
            PRINT*, 'weight wgt = ',wgt
            PRINT*, 'numPreSmooth = ',numPreSmooth
            PRINT*, 'numPostSmooth = ',numPostSmooth
            PRINT*, 'iSmoothIt = ',iSmoothIt
            PRINT*, 'iFineSolver = ',iFineSolver
            PRINT*, 'iCoarseSolver = ',iCoarseSolver
            PRINT*, 'iSmoother = ',gismoother
            PRINT*, 'ioutput = ',ioutput
            PRINT*, 'nzl = ',nzl
         END IF
#ifdef MPI_USED
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,'mg3p: MPI_Barrier')")  ip
#endif
         CALL MPI_Barrier (comm,ierr)
#endif
      END DO
#endif

      icommtag = ICOMMSTART

* ----------------------------------------------------------------------
*     Get global and local problem sizes
* ----------------------------------------------------------------------

      nx(1)=nxi
      ny(1)=nyi
      nz(1)=nzi

      CALL mmGetGlobalSize (nxg,nyg,nzg,nx,ny,nz,
     &     comm,nbr,bnd
     &     )

* ----------------------------------------------------------------------
*     Verify correctness of input parameters
* ----------------------------------------------------------------------

      CALL mmCheckParams (
     &     nxg,nyg,nzg,m,
     &     ixmb,iymb,izmb,ixpb,iypb,izpb,comm)

* ----------------------------------------------------------------------
*     Distribute the domain across processors
* ----------------------------------------------------------------------

      CALL mmDistribute (
     &     nxg,nyg,nzg,nxd2,nyd2,nzd2,
     &     nxd,nyd,nzd,nx,ny,nz,m,
     &     indh,inda,indx
     &     ,
     &     comm,nbr,bnd,bnd2
     &     ,isub)

* ----------------------------------------------------------------------
*     Initialize level-independent global variables
* ----------------------------------------------------------------------

#ifdef DEBUG
      if (ip.EQ.0) print*, ip, 'mmMgSolve2: set globals'
#endif

      gicoords = icoords

      IF ( iGridTypeX.EQ.MM_IS_UNIFORM .AND.
     &     iGridTypeY.EQ.MM_IS_UNIFORM .AND.
     &     iGridTypeZ.EQ.MM_IS_UNIFORM) THEN
         giGridType = MM_IS_UNIFORM 
      ELSE
         giGridType = MM_IS_NONUNIFORM
      END IF

      CALL mmInitCoords (nx(1),ny(1),nz(1),xp(1),yp(1),zp(1),
     &     icoords,1)

* ======================================================================
*     Determine offsets into work array
* ======================================================================

      CALL mmAllocate(nxd,nyd,nzd,nxg,nyg,nzg,m,ioutput,
     &     iFineSolver,iCoarseSolver,iSmoother,
     &     i_AD,i_A1,i_A2,i_A3,i_A4,i_A5,i_A6,
     &     i_XC,i_YSM,i_BC,i_R,i_P,i_W,i_RMG,i_ZMG,
     &     i_RCG,i_PCG,i_WCG,i_HX,i_HY,i_HZ,i_XP,i_YP,i_ZP,
     &     i_RSM,i_PSM,i_WSM,
     &     i_RT,i_V,i_PT,i_S,i_ST,i_T,comm,
     &     Z,nzl,gistorage)

* ----------------------------------------------------------------------
*     Scale right-hand side (uses mmBuildFine to do that)
* ----------------------------------------------------------------------
      
CCCCC @@ Macro dependent parameters

c$$$      CALL mmBuildFine (
c$$$     &     Z(i_AFD),Z(i_AF1),Z(i_AF2),Z(i_AF3),
c$$$     &     Z(i_AF1),Z(i_AF2),Z(i_AF3),nxd2,nyd2,nzd2,bnd2,
c$$$     &     FI,nxd2,nyd2,nzd2,bnd2,nx(1),ny(1),nz(1),
c$$$     &     xp,yp,zp,comm,isub)

CCCCC @@ Macro dependent parameters

      CALL mmScaleRHS (FI,nxd2,nyd2,nzd2,bnd2,nx(1),ny(1),nz(1),1)

* ----------------------------------------------------------------------
*     Copy mesh coordinates to fine grid portion of extended array
* ----------------------------------------------------------------------

      DO i=1,nxd(1)
         Z(i_XP+i-1) = xp(i)
      END DO
      DO i=1,nyd(1)
         Z(i_YP+i-1) = yp(i)
      END DO
      DO i=1,nzd(1)
         Z(i_ZP+i-1) = zp(i)
      END DO

* ----------------------------------------------------------------------
*     Split work array and call mgsolve2
* ----------------------------------------------------------------------

CCCCC @@ Macro dependent parameters

      CALL mmMgSolve2 (
     &     Z(i_AFD),Z(i_AF1),Z(i_AF2),Z(i_AF3),
     &     Z(i_AF1),Z(i_AF2),Z(i_AF3),1,1,1,bnd2,
     &     nx,ny,nz,nxg,nyg,nzg,
     &     indh,inda,indx
     &     ,
     &     iGridTypeX,iGridTypeY,iGridTypeZ,
     &     comm,nbr,bnd,bnd2
     &     ,
     &     icoords,
     &     m,wgt,numPreSmooth,numPostSmooth,iSmoothIt,
     &     iFineSolver,iCoarseSolver,
     &     maxItFine,maxItCoarse,
     &     resTolFine,resTolCoarse,totalTime,
     &     ioutput,
     &     numItFine,resFine,resRedFine,istat,
     &     XI,nxd2,nyd2,nzd2,bnd2,
     &     FI,nxd2,nyd2,nzd2,bnd2,
     &     ifng,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxd2,nyd2,nzd2,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb,
     &     Z(i_AD),Z(i_A1),Z(i_A2),Z(i_A3),Z(i_A4),Z(i_A5),Z(i_A6),
     &     nxd,nyd,nzd,bnd,
     &     Z(i_XC),nxd,nyd,nzd,bnd,
     &     Z(i_YSM),nxd,nyd,nzd,bnd,
     &     Z(i_BC),nxd,nyd,nzd,bnd,
     &     Z(i_R),nxd,nyd,nzd,bnd,
     &     Z(i_P),nxd,nyd,nzd,bnd,
     &     Z(i_W),nxd,nyd,nzd,bnd,
     &     Z(i_RMG),nxd,nyd,nzd,bnd,
     &     Z(i_ZMG),nxd,nyd,nzd,bnd,
     &     Z(i_RCG),nxd(m),nyd(m),nzd(m),bnd(1,m),
     &     Z(i_PCG),nxd(m),nyd(m),nzd(m),bnd(1,m),
     &     Z(i_WCG),nxd(m),nyd(m),nzd(m),bnd(1,m),
     &     Z(i_RSM),nxd,nyd,nzd,bnd,
     &     Z(i_PSM),nxd,nyd,nzd,bnd,
     &     Z(i_WSM),nxd,nyd,nzd,bnd,
     &     Z(i_RT),nxd,nyd,nzd,bnd,
     &     Z(i_V),nxd,nyd,nzd,bnd,
     &     Z(i_PT),nxd,nyd,nzd,bnd,
     &     Z(i_S),nxd,nyd,nzd,bnd,
     &     Z(i_ST),nxd,nyd,nzd,bnd,
     &     Z(i_T),nxd,nyd,nzd,bnd,
     &     Z(i_XP),Z(i_YP),Z(i_ZP),Z(i_HX),Z(i_HY),Z(i_HZ),
     &     nzl,isub)

      
CCCCC @@ Macro dependent parameters

      CALL mmScaleRHS (FI,nxd2,nyd2,nzd2,bnd2,nx(1),ny(1),nz(1),-1)

#ifdef MPI_USED
      CALL MPI_BCAST(numItFine,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(resFine,1,MM_REAL,0,comm,ierr)
      CALL MPI_BCAST(resRedFine,1,MM_REAL,0,comm,ierr)
      CALL MPI_BCAST(istat,1,MPI_INTEGER,0,comm,ierr)
      CALL MPI_BCAST(totalTime,1,MM_REAL,0,comm,ierr)
#endif
      
      gtime(isub) = gtime(isub) + (MPI_WTIME() - gtimestart(isub))
      gtimecount(isub) = gtimecount(isub) + 1
      totalTime = gtime(isub)

      IF (ip.EQ.0) THEN
         CALL mmOutput (ifng,icoords,
     &        ixmb,ixpb,iymb,iypb,izmb,izpb,
     &        nxd2,nyd2,nzd2,nx,ny,nz,nxg,nyg,nzg,
     &        iGridTypeX,iGridTypeY,iGridTypeZ,
     &        maxItFine,maxItCoarse,resTolFine,resTolCoarse,m,
     &        wgt,numPreSmooth,numPostSmooth,iSmoothIt,
     &        POISSON,iFineSolver,iCoarseSolver,
     &        ioutput,numItFine,resFine,resRedFine,istat,
     &        totalTime,nzl)
      END IF

#ifdef PERF_EVAL
      IF (ip.EQ.0) THEN
         CALL mmWriteTimes
         CALL mmWritePercents
         CALL mmWriteFlops
         CALL mmWriteFlopRate
      END IF
      CALL mmWriteTotalFlopRate (comm)
#endif

C@@@@@@@
C WRITE SOLUTION TO FILES X.out.<ipx><ipy><ipz>
C@@@@@@@
      IF (ioutput.GE.10) THEN
         CALL mmWriteVector ('X.out     ',XI,
     &        nxd2,nyd2,nzd2,nxi,nyi,nzi,
     &        gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
     &        comm,nbr)
         CALL mmTerminate(comm)
      END IF
         
      IF (ip.EQ.0 .AND. ioutput.GT.0) THEN
          print*, '=============================='
          print*, 'END mg3p: MGMPI Poisson Solver'
          print*, '=============================='
      END IF

#endif 

      RETURN
      END
C=======================================================================
C     
C     Module:  MPI_WTIME
C     
C     Purpose: Timer for compiling without MPI
C
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C 
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

#ifndef MPI_USED
      DOUBLE PRECISION FUNCTION MPI_WTIME ()
#include "mgmpi_internal.h"
      MPI_WTIME = etime(mmtarray)
      RETURN
      END
#endif
C=======================================================================
C
C     Module:  SUBROUTINE mmAllocate
C
C     Purpose: Allocate vectors from Z input array
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:
C
C     Communication estimate:
C
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history:
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmAllocate (nxd,nyd,nzd,nxg,nyg,nzg,m,ioutput,
     &     iFineSolver,iCoarseSolver,iSmoother,
     &     i_AD,i_A1,i_A2,i_A3,i_A4,i_A5,i_A6,
     &     i_XC,i_YSM,i_BC,i_R,i_P,i_W,i_RMG,i_ZMG,
     &     i_RCG,i_PCG,i_WCG,i_HX,i_HY,i_HZ,i_XP,i_YP,i_ZP,
     &     i_RSM,i_PSM,i_WSM,
     &     i_RT,i_V,i_PT,i_S,i_ST,i_T,comm,
     &     Z,nzl,JM)

      IMPLICIT NONE

      INTEGER i_AD,i_A1,i_A2,i_A3,i_A4,i_A5,i_A6
      INTEGER i_XC,i_YSM,i_BC,i_R,i_W,i_RMG,i_ZMG
      INTEGER i_RCG,i_PCG,i_WCG,i_HX,i_HY,i_HZ,i_XP,i_YP,i_ZP
      INTEGER i_RSM,i_PSM,i_WSM
      INTEGER i_RT,i_V,i_P,i_PT,i_S,i_ST,i_T
      INTEGER nxd(*),nyd(*),nzd(*),nxg(*),nyg(*),nzg(*),ioutput,comm
      INTEGER m,iFineSolver,iCoarseSolver,iSmoother

      INTEGER nzl,JM
      REAL Z(nzl)

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      INTEGER k,nd,ndh,nd1,ndm,nd2m,ndmc,JM0,ierrors,ierr
      LOGICAL isBiCg,isMg,isCg,l0,l1,l2

      isBiCg = (iFineSolver.EQ.MM_BICG_SOLVER
     &     .OR. iFineSolver.EQ.MM_BICGMG_SOLVER)

      isMg = (  iFineSolver.EQ.MM_MG_SOLVER
     &     .OR. iFineSolver.EQ.MM_BICGMG_SOLVER
     &     .OR. iFineSolver.EQ.MM_CGMG_SOLVER
     &     .OR. iFineSolver.EQ.MM_FMG_SOLVER)

      isCg = (  iFineSolver.EQ.MM_CG_SOLVER
     &     .OR. iFineSolver.EQ.MM_CGMG_SOLVER)

      l0 = isMg .AND. iCoarseSolver.EQ.MM_CG_COARSE_SOLVER
      l1 = iSmoother.EQ.MM_LOCAL_LINE_JACOBI
      l2 = iSmoother.EQ.MM_PLANE_JACOBI

      nd = 0
      ndh = 0
      DO k=1,m
         nd = nd + nxd(k)*nyd(k)*nzd(k)
         ndh = ndh + max(max(nxd(k),nyd(k)),nzd(k))
      END DO
      nd1 = nxd(1)*nyd(1)*nzd(1)
      ndm = nxd(m)*nyd(m)*nzd(m)
      nd2m = nd - nd1
      ndmc = nxg(m)*nyg(m)*nzg(m)  ! collapsed coarse-grid vector

      JM = 1

C-----------------------------------------------------------------------
C     Coarse matrix storage AC
C-----------------------------------------------------------------------

      JM0 = JM

      IF (isMg) THEN
         i_AD  = JM
         JM    = JM + nd2m
         i_A1  = JM
         JM    = JM + nd2m
         i_A2  = JM
         JM    = JM + nd2m
         i_A3  = JM
         JM    = JM + nd2m
         IF (gisymmetric.EQ.1) THEN
            i_A4  = i_A1
            i_A5  = i_A2
            i_A6  = i_A3
         ELSE
            i_A4  = JM
            JM    = JM + nd2m
            i_A5  = JM
            JM    = JM + nd2m
            i_A6  = JM
            JM    = JM + nd2m
         END IF
      ELSE
         i_AD = JM
         i_A1 = JM
         i_A2 = JM
         i_A3 = JM
         i_A4 = JM
         i_A5 = JM
         i_A6 = JM
      END IF

      IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &     print*, 'AC* storage = ',1.0*(JM-JM0)/nd1

C-----------------------------------------------------------------------
C     XC and BC storage
C-----------------------------------------------------------------------

      i_XC  = JM
      JM0 = JM
      IF (isMg) JM = JM + nd2m
      IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &     print*, 'XC storage = ',1.0*(JM-JM0)/nd1

      i_BC  = JM
      JM0 = JM
      IF (isMg) JM = JM + nd2m
      IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &     print*, 'BC storage = ',1.0*(JM-JM0)/nd1

* ----------------------------------------------------------------------
*     Residual R storage
* ----------------------------------------------------------------------

      i_R = JM
      JM0 = JM
      IF (isMg)       JM  = JM + nd
      IF (.NOT. isMg) JM  = JM + nd1

      IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &     print*, 'R storage = ',1.0*(JM-JM0)/nd1

* ----------------------------------------------------------------------
*     CG coarse solver work vectors RCG, PCG and WCG for MG and BiCGStabMG
* ----------------------------------------------------------------------

      IF (gmmIsCollapsed) THEN
         i_RCG = JM
         JM0 = JM
         IF (l0) JM = JM + ndmc
         IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &        print*, 'RCG storage = ',1.0*(JM-JM0)/nd1

         i_PCG = JM
         JM0 = JM
         IF (l0) JM = JM + ndmc
         IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &        print*, 'PCG storage = ',1.0*(JM-JM0)/nd1

         i_WCG = JM
         JM0 = JM
         IF (l0) JM = JM + ndmc
         IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &        print*, 'WCG storage = ',1.0*(JM-JM0)/nd1
      ELSE
         i_RCG = JM
         JM0 = JM
         IF (l0) JM = JM + ndm
         IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &        print*, 'RCG storage = ',1.0*(JM-JM0)/nd1

         i_PCG = JM
         JM0 = JM
         IF (l0) JM = JM + ndm
         IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &        print*, 'PCG storage = ',1.0*(JM-JM0)/nd1

         i_WCG = JM
         JM0 = JM
         IF (l0) JM = JM + ndm
         IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &        print*, 'WCG storage = ',1.0*(JM-JM0)/nd1
      END IF
* ----------------------------------------------------------------------
*     Smoother work arrays
* ----------------------------------------------------------------------

      i_YSM = JM
      JM0 = JM
      IF (l1) JM = JM + nd1
      IF (l2) JM = JM + nd1
      IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &     print*, 'YSM storage = ',1.0*(JM-JM0)/nd1

      i_RSM = JM
      JM0 = JM
      IF (l2) JM = JM + nd1
      IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &     print*, 'RSM storage = ',1.0*(JM-JM0)/nd1

      i_PSM = JM
      JM0 = JM
      IF (l2) JM = JM + nd1
      IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &     print*, 'PSM storage = ',1.0*(JM-JM0)/nd1

      i_WSM = JM
      JM0 = JM
      IF (l2) JM = JM + nd1
      IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &     print*, 'WSM storage = ',1.0*(JM-JM0)/nd1

* ----------------------------------------------------------------------
*     CG and CGMG work vectors P and W
*     BICG-STAB work vectors RT,V,P,S,T
*     BCGMG work vectors PT,ST
* ----------------------------------------------------------------------

      i_P = JM
      JM0 = JM
      IF (isCg .OR. isBiCg) JM = JM + nd
      IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &     print*, 'P storage = ',1.0*(JM-JM0)/nd1

C     *** W is allocated as nd (= nd1+nd2m) not nd1 for cgmg since 
C     *** W is used in place of R for mmMgPrecon (since R is B 
C     *** for mmMgPrecon)

      i_W = JM
      JM0 = JM
      IF (isCg)          JM = JM + nd
      IF (isCg.AND.isMg) JM = JM + nd2m
      IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &     print*, 'W storage = ',1.0*(JM-JM0)/nd1

      i_RT = JM
      JM0 = JM
      IF (isBiCg) JM = JM + nd1
      IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &     print*, 'RT storage = ',1.0*(JM-JM0)/nd1

      i_V = JM
      JM0 = JM
      IF (isBiCg) JM = JM + nd1
      IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &     print*, 'V storage = ',1.0*(JM-JM0)/nd1

      i_S = JM
      JM0 = JM
      IF (isBiCg) JM = JM + nd1
      IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &     print*, 'S storage = ',1.0*(JM-JM0)/nd1

      i_T = JM
      JM0 = JM
      IF (isBiCg) JM = JM + nd1
      IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &     print*, 'T storage = ',1.0*(JM-JM0)/nd1

      i_PT = JM
      JM0 = JM
      IF (isBiCg.AND.isMg) JM = JM + nd1
      IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &     print*, 'PT storage = ',1.0*(JM-JM0)/nd1

      i_ST = JM
      JM0 = JM
      IF (isBiCg.AND.isMg) JM = JM + nd1
      IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &     print*, 'ST storage = ',1.0*(JM-JM0)/nd1

* ----------------------------------------------------------------------
*     CgMg array Z
* ----------------------------------------------------------------------

      i_ZMG = JM
      JM0 = JM
      IF (isCg .AND. isMg) JM = JM + nd1
      IF (ip.EQ.0 .AND. ioutput.GT.0 .AND. JM .GT. JM0)
     &     print*, 'ZMG storage = ',1.0*(JM-JM0)/nd1

C      IF (iFineSolver.EQ.MM_CG_SOLVER) THEN
C     AF XF BF R P W
C      ELSE IF (iFineSolver.EQ.MM_MG_SOLVER) THEN
C     AF XF BF AC XC Y BC R RCG PCG WCG RSM PSM WSM
C      ELSE IF (iFineSolver.EQ.MM_BICG_SOLVER) THEN
C     AF XF BF R RT P PT S ST V T
C      ELSE IF (iFineSolver.EQ.MM_BICGMG_SOLVER) THEN
C     AF XF BF AC XC BC Y R RT P PT S ST V T RCG PCG WCG RSM PSM WSM
C      ELSE IF (iFineSolver.EQ.MM_FMG_SOLVER) THEN
C     AF XF BF AC XC Y BC R RCG PCG WCG RSM PSM WSM
C      ELSE IF (iFineSolver.EQ.MM_CGMG_SOLVER) THEN
C     AF XF BF AC XC BC Y R P W ZMG RCG PCG WCG RSM PSM WSM
C      END IF

* ----------------------------------------------------------------------
*     HX, HY, HZ, XP, YP, ZP
* ----------------------------------------------------------------------

      JM0 = JM

      i_HX = JM
      JM = JM + ndh
      i_HY = JM
      JM = JM + ndh
      i_HZ = JM
      JM = JM + ndh
      i_XP = JM
      JM = JM + ndh
      i_YP = JM
      JM = JM + ndh
      i_ZP = JM
      JM = JM + ndh

      if (ip.EQ.0 .AND. ioutput.GT.0)
     &     print*, 'HX HY HZ XP YP ZP vectors = ',1.0*(JM-JM0)/nd1

* ----------------------------------------------------------------------
*     Verify that work array is big enough
* ----------------------------------------------------------------------

      IF (ip.EQ.0 .AND. ioutput.GT.0) 
     &     print*, 'Storage available (vectors): ',1.0*nzl/nd1
      IF (ip.EQ.0 .AND. ioutput.GT.0) 
     &     print*, 'Storage required (vectors):  ',1.0*(JM-1)/nd1
      IF (ip.EQ.0 .AND. ioutput.GT.0) 
     &     print*, 'Storage available (floats): ', nzl
      IF (ip.EQ.0 .AND. ioutput.GT.0) 
     &     print*, 'Storage required (floats):  ', (JM-1)

      ierrors = 0
      IF (ip.EQ.0 .AND. JM-1.GT.nzl) THEN
         print*,'== mgmpi ERROR == nzl must be increased from'
         print*,'                  ',nzl,' to ',JM-1
         ierrors = ierrors + 1
      END IF

#ifdef MPI_USED
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,'mg3s: MPI_BCAST')")  ip
#endif
      CALL MPI_BCAST (ierrors,1,MPI_INTEGER,0,comm,ierr)
      if (ierr.NE.0) print*, 'MPI Error in check_params: ',ierr
#endif

      IF (ierrors.GT.0) THEN
         IF (ip.EQ.0)
     &      print*, '== mgmpi ERROR == Terminating because of errors'

#ifdef MPI_USED
         CALL mmTerminate (comm)
#else
         STOP
#endif

      END IF

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmBuildCoarse 
C     
C     Purpose: Build a coarse-level matrix given the next-finer one
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBuildCoarse (
     &     ACD,AC1,AC2,AC3,AC4,AC5,AC6,nxdac,nydac,nzdac,bndac
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,nbr,bndc,bndf,levelC,icall)

      IMPLICIT NONE

      
      INTEGER nxdac,nydac,nzdac
      INTEGER bndac(3)
      REAL ACD(nxdac,nydac,nzdac)
      REAL AC1(nxdac,nydac,nzdac)
      REAL AC2(nxdac,nydac,nzdac)
      REAL AC3(nxdac,nydac,nzdac)
      REAL AC4(nxdac,nydac,nzdac)
      REAL AC5(nxdac,nydac,nzdac)
      REAL AC6(nxdac,nydac,nzdac)
      
      INTEGER nxdaf,nydaf,nzdaf
      INTEGER bndaf(3)  
      REAL AFD(nxdaf,nydaf,nzdaf)
      REAL AF1(nxdaf,nydaf,nzdaf)
      REAL AF2(nxdaf,nydaf,nzdaf)
      REAL AF3(nxdaf,nydaf,nzdaf)
      REAL AF4(nxdaf,nydaf,nzdaf)
      REAL AF5(nxdaf,nydaf,nzdaf)
      REAL AF6(nxdaf,nydaf,nzdaf)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)

      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      INTEGER comm,nbr(6),bndc(3),bndf(3),levelC

      INTEGER idim

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_BUILD+levelC
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmBuildCoarse'
#endif

      idim = 0

      IF (gmmlc1) idim = idim + 1
      IF (gmmlc2) idim = idim + 1
      IF (gmmlc3) idim = idim + 1

      IF (idim .EQ. 3) THEN
         CALL mmBuildCoarseGen (
     &     ACD,AC1,AC2,AC3,AC4,AC5,AC6,nxdac,nydac,nzdac,bndac
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,nbr,bndc,bndf,levelC,isub)
      ELSE
         CALL mmBuildCoarseSCGen (
     &     ACD,AC1,AC2,AC3,AC4,AC5,AC6,nxdac,nydac,nzdac,bndac
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,nbr,bndc,bndf,levelC,isub)
      END IF         

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmBuildCoarse'
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmBuildCoarseU
C     
C     Purpose: Build a coarse-level matrix given the next-finer one
C     Purpose: for regular problems
C
C     Algorithm: 
C
C     Inputs:  
C
C     Outputs: 
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     
C
C     Communication estimate:  
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBuildCoarseU (
     &     ACD,AC1,AC2,AC3,AC4,AC5,AC6,nxdac,nydac,nzdac,bndac
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,nbr,bndc,bndf,levelC,icall)

      IMPLICIT NONE

      
      INTEGER nxdac,nydac,nzdac
      INTEGER bndac(3)
      REAL ACD(nxdac,nydac,nzdac)
      REAL AC1(nxdac,nydac,nzdac)
      REAL AC2(nxdac,nydac,nzdac)
      REAL AC3(nxdac,nydac,nzdac)
      REAL AC4(nxdac,nydac,nzdac)
      REAL AC5(nxdac,nydac,nzdac)
      REAL AC6(nxdac,nydac,nzdac)
      
      INTEGER nxdaf,nydaf,nzdaf
      INTEGER bndaf(3)  
      REAL AFD(nxdaf,nydaf,nzdaf)
      REAL AF1(nxdaf,nydaf,nzdaf)
      REAL AF2(nxdaf,nydaf,nzdaf)
      REAL AF3(nxdaf,nydaf,nzdaf)
      REAL AF4(nxdaf,nydaf,nzdaf)
      REAL AF5(nxdaf,nydaf,nzdaf)
      REAL AF6(nxdaf,nydaf,nzdaf)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)

      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      INTEGER comm,nbr(6),bndc(3),bndf(3),levelC

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#else 
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      INTEGER ixc,iyc,izc,ixf,iyf,izf
      REAL hrm,a,b,x,y,z
      REAL dx,dy,dz,dxi,dyi,dzi
      INTEGER CX1,CY1,CZ1,CX2,CY2,CZ2
      INTEGER MX1,MY1,MZ1,MX2,MY2,MZ2
      INTEGER levelF

#ifdef DEBUG
      REAL s,mmVectorSum
#endif

      
      INTEGER isub,icall

      hrm (a,b) = (2.0D0)*a*b/(a+b)

      
#ifdef PERF_EVAL
      isub = SB_BUILD+levelC
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      print*, ip, '>>> mmBuildCoarseGen enter', levelF
#endif

      levelF = levelC-1

      CX1 = 2
      CY1 = 2
      CZ1 = 2
      CX2 = nxc-1
      CY2 = nyc-1
      CZ2 = nzc-1
      MX1 = 1
      MY1 = 1
      MZ1 = 1
      IF (ipx.LT.npx-1) MX2 = nxc-2
      IF (ipx.EQ.npx-1) MX2 = nxc-1
      IF (ipy.LT.npy-1) MY2 = nyc-2
      IF (ipy.EQ.npy-1) MY2 = nyc-1
      IF (ipz.LT.npz-1) MZ2 = nzc-2
      IF (ipz.EQ.npz-1) MZ2 = nzc-1

*-----------------------------------------------------------------------
*     Compute the positive X-diagonal
*-----------------------------------------------------------------------

      IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,1,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
      ELSE
         CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AF1,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf)
      END IF

      CALL mmGetYnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
      CALL mmGetZnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

      dx = gdx(2,levelF)
      dy = gdy(2,levelF)
      dz = gdz(2,levelF)
      dxi = gdxi(2,levelF)
      dyi = gdyi(2,levelF)
      dzi = gdzi(2,levelF)

      DO izc=CZ1,CZ2
         izf = 2*(izc-1)+1
         z = zp(izf)
         DO iyc=CY1,CY2
            iyf = 2*(iyc-1)+1
            y = yp(iyf)
            DO ixc=MX1,MX2
               ixf = 2*(ixc-1)+1
               x = xp(ixf)
               AC1(ixc,iyc,izc) = o4
     &              *(o2*hrm(R(ixf,  iyf,izf),  R(ixf+1,iyf,izf))
     &              +o2*(hrm(R(ixf,  iyf-1,izf),R(ixf+1,iyf-1,izf))*dyi
     &              +    hrm(R(ixf,  iyf+1,izf),R(ixf+1,iyf+1,izf))*dyi
     &              +    hrm(R(ixf,  iyf,izf-1),R(ixf+1,iyf,izf-1))*dzi
     &              +    hrm(R(ixf,  iyf,izf+1),R(ixf+1,iyf,izf+1))*dzi)
     &              / (dyi+dyi+dzi+dzi))
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (5*7+16)*(MX2-MX1+1)*(CY2-CY1+1)*(CZ2-CZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (5*7+16)*(MX2-MX1+1)*(CY2-CY1+1)*(CZ2-CZ1+1)
#endif

CCCCC @@ Macro dependent parameters
      CALL mmSendX1Ghost (AC1,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
      CALL mmGetY1Ghost (AC1,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
      CALL mmGetZ1Ghost (AC1,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

*-----------------------------------------------------------------------
*     Compute the positive Y-diagonal
*-----------------------------------------------------------------------

      IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,2,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
      ELSE
         CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AF2,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf)
      END IF

      CALL mmGetZnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
      CALL mmGetXnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

      DO izc=CZ1,CZ2
         izf = 2*(izc-1)+1
         z = zp(izf)
         DO ixc=CX1,CX2
            ixf = 2*(ixc-1)+1
            x = xp(ixf)
            DO iyc=MY1,MY2
               iyf = 2*(iyc-1)+1
               y = yp(iyf)
               AC2(ixc,iyc,izc) = o4
     &              *(o2 *hrm(R(ixf,iyf,  izf),
     &              R(ixf,iyf+1,izf))
     &              +o2*(hrm(R(ixf-1,iyf,  izf),R(ixf-1,iyf+1,izf))*dxi
     &              +    hrm(R(ixf+1,iyf,  izf),R(ixf+1,iyf+1,izf))*dxi
     &              +    hrm(R(ixf,iyf,  izf-1),R(ixf,iyf+1,izf-1))*dzi
     &              +    hrm(R(ixf,iyf,  izf+1),R(ixf,iyf+1,izf+1))*dzi)
     &              /(dxi+dxi+dzi+dzi))
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (5*7+16)*(CX2-CX1+1)*(MY2-MY1+1)*(CZ2-CZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (5*7+16)*(CX2-CX1+1)*(MY2-MY1+1)*(CZ2-CZ1+1)
#endif

CCCCC @@ Macro dependent parameters
      CALL mmGetX1Ghost (AC2,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
      CALL mmSendY1Ghost (AC2,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
      CALL mmGetZ1Ghost (AC2,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

*-----------------------------------------------------------------------
*     Compute the positive Z-diagonal
*-----------------------------------------------------------------------

      IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,3,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
      ELSE
         CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AF3,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf)
      END IF

      CALL mmGetXnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
      CALL mmGetYnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

      DO iyc=CY1,CY2
         iyf = 2*(iyc-1)+1
         y = yp(iyf)
         DO ixc=CX1,CX2
            ixf = 2*(ixc-1)+1
            x = xp(ixf)
            DO izc=MZ1,MZ2
               izf = 2*(izc-1)+1
               z = zp(izf)
               AC3(ixc,iyc,izc) = o4
     &              *(o2 *hrm(R(ixf,iyf,izf),
     &              R(ixf,iyf,izf+1))
     &              + o2*(hrm(R(ixf-1,iyf,izf),R(ixf-1,iyf,izf+1))*dxi
     &              +     hrm(R(ixf+1,iyf,izf),R(ixf+1,iyf,izf+1))*dxi
     &              +     hrm(R(ixf,iyf-1,izf),R(ixf,iyf-1,izf+1))*dyi
     &              +     hrm(R(ixf,iyf+1,izf),R(ixf,iyf+1,izf+1))*dyi)
     &              /(dxi+dxi+dyi+dyi))
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (5*7+16)*(CX2-CX1+1)*(CY2-CY1+1)*(MZ2-MZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (5*7+16)*(CX2-CX1+1)*(CY2-CY1+1)*(MZ2-MZ1+1)
#endif

CCCCC @@ Macro dependent parameters
      CALL mmGetX1Ghost (AC3,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
      CALL mmGetY1Ghost (AC3,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
      CALL mmSendZ1Ghost (AC3,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

*-----------------------------------------------------------------------
*     Compute the negative X-diagonal
*-----------------------------------------------------------------------

      IF (gisymmetric.EQ.0) THEN

         IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,4,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         ELSE
            CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AF4,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf)
         END IF

         CALL mmGetYnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
         CALL mmGetZnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

         DO izc=CZ1,CZ2
            izf = 2*(izc-1)+1
            z = zp(izf)
            DO iyc=CY1,CY2
               iyf = 2*(iyc-1)+1
               y = yp(iyf)
               DO ixc=MX1,MX2
                  ixf = 2*(ixc-1)+1
                  x = xp(ixf)
                  AC4(ixc,iyc,izc) = o4
     &                 *(o2* hrm(R(ixf,  iyf,izf),  
     &                 R(ixf+1,iyf,izf))
     &                 +o2*(hrm(R(ixf,iyf-1,izf),R(ixf+1,iyf-1,izf))*dyi
     &                 +   hrm(R(ixf,iyf+1,izf),R(ixf+1,iyf+1,izf))*dyi
     &                 +   hrm(R(ixf,iyf,izf-1),R(ixf+1,iyf,izf-1))*dzi
     &                 +   hrm(R(ixf,iyf,izf+1),R(ixf+1,iyf,izf+1))*dzi)
     &                 /(dyi+dyi+dzi+dzi))
               END DO
            END DO
         END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (5*7+16)*(MX2-MX1+1)*(CY2-CY1+1)*(CZ2-CZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (5*7+16)*(MX2-MX1+1)*(CY2-CY1+1)*(CZ2-CZ1+1)
#endif

CCCCC @@ Macro dependent parameters
         CALL mmSendX1Ghost (AC4,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
         CALL mmGetY1Ghost (AC4,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
         CALL mmGetZ1Ghost (AC4,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

*-----------------------------------------------------------------------
*     Compute the negative Y-diagonal
*-----------------------------------------------------------------------

         IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,5,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         ELSE
            CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AF5,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf)
         END IF

         CALL mmGetZnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
         CALL mmGetXnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

         DO izc=CZ1,CZ2
            izf = 2*(izc-1)+1
            z = zp(izf)
            DO ixc=CX1,CX2
               ixf = 2*(ixc-1)+1
               x = xp(ixf)
               DO iyc=MY1,MY2
                  iyf = 2*(iyc-1)+1
                  y = yp(iyf)
                  AC5(ixc,iyc,izc) = o4
     &                 *(o2 *hrm(R(ixf,iyf,  izf),
     &                 R(ixf,iyf+1,izf))
     &                 +o2*(hrm(R(ixf-1,iyf,izf),R(ixf-1,iyf+1,izf))*dxi
     &                 +   hrm(R(ixf+1,iyf,izf),R(ixf+1,iyf+1,izf))*dxi
     &                 +   hrm(R(ixf,iyf,izf-1),R(ixf,iyf+1,izf-1))*dzi
     &                 +   hrm(R(ixf,iyf,izf+1),R(ixf,iyf+1,izf+1))*dzi)
     &                 /(dxi+dxi+dzi+dzi))
               END DO
            END DO
         END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (5*7+16)*(CX2-CX1+1)*(MY2-MY1+1)*(CZ2-CZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (5*7+16)*(CX2-CX1+1)*(MY2-MY1+1)*(CZ2-CZ1+1)
#endif

CCCCC @@ Macro dependent parameters
         CALL mmGetX1Ghost (AC5,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
         CALL mmSendY1Ghost (AC5,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
         CALL mmGetZ1Ghost (AC5,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

*-----------------------------------------------------------------------
*     Compute the negative Z-diagonal
*-----------------------------------------------------------------------

         IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,6,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         ELSE
            CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AF6,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf)
         END IF

         CALL mmGetXnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
         CALL mmGetYnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

         DO iyc=CY1,CY2
            iyf = 2*(iyc-1)+1
            y = yp(iyf)
            DO ixc=CX1,CX2
               ixf = 2*(ixc-1)+1
               x = xp(ixf)
               DO izc=MZ1,MZ2
                  izf = 2*(izc-1)+1
                  z = zp(izf)
                  AC6(ixc,iyc,izc) = o4
     &                 *(o2 *hrm(R(ixf,iyf,izf),
     &                 R(ixf,iyf,izf+1))
     &                 +o2*(hrm(R(ixf-1,iyf,izf),R(ixf-1,iyf,izf+1))*dxi
     &                 +   hrm(R(ixf+1,iyf,izf),R(ixf+1,iyf,izf+1))*dxi
     &                 +   hrm(R(ixf,iyf-1,izf),R(ixf,iyf-1,izf+1))*dyi
     &                 +   hrm(R(ixf,iyf+1,izf),R(ixf,iyf+1,izf+1))*dyi)
     &                 /(dxi+dxi+dyi+dyi))
               END DO
            END DO
         END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (5*7+16)*(CX2-CX1+1)*(CY2-CY1+1)*(MZ2-MZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (5*7+16)*(CX2-CX1+1)*(CY2-CY1+1)*(MZ2-MZ1+1)
#endif

CCCCC @@ Macro dependent parameters
         CALL mmGetX1Ghost (AC6,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
         CALL mmGetY1Ghost (AC6,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
         CALL mmSendZ1Ghost (AC6,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

      END IF

*-----------------------------------------------------------------------
*     Compute the main diagonal of the matrix and the right-hand side
*-----------------------------------------------------------------------

      IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,0,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
      ELSE
         CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AFD,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf)
      END IF

      DO izc=CZ1,CZ2
         izf = 2*(izc-1)+1
         DO iyc=CY1,CY2
            iyf = 2*(iyc-1)+1
            DO ixc=CX1,CX2
               ixf = 2*(ixc-1)+1
               ACD(ixc,iyc,izc) = o4*R(ixf,iyf,izf)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (CX2-CX1+1)*(CY2-CY1+1)*(CZ2-CZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (CX2-CX1+1)*(CY2-CY1+1)*(CZ2-CZ1+1)
#endif

#ifdef DEBUG
      s = mmVectorSum (ACD,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
      if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(ACD) = ',s
      s = mmVectorSum (AC1,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
      if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AC1) = ',s
      s = mmVectorSum (AC2,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
      if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AC2) = ',s
      s = mmVectorSum (AC3,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
      if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AC3) = ',s
      IF (gisymmetric.EQ.0) THEN
         s = mmVectorSum (AC4,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AC4) = ',s
         s = mmVectorSum (AC5,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AC5) = ',s
         s = mmVectorSum (AC6,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AC6) = ',s
      END IF 

      IF (gimatfree.EQ.0.OR.levelC.GT.2) THEN

         s = mmVectorSum (AFD,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf,comm,
     &     isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AFD) = ',s
         s = mmVectorSum (AF1,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf,comm,
     &     isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF1) = ',s
         s = mmVectorSum (AF2,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf,comm,
     &     isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF2) = ',s
         s = mmVectorSum (AF3,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf,comm,
     &     isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF3) = ',s
         IF (gisymmetric.EQ.0) THEN
            s = mmVectorSum (AF4,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF4) = ',s
            s = mmVectorSum (AF5,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF5) = ',s
            s = mmVectorSum (AF6,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF6) = ',s
         END IF
      ELSE
         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,0,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AFD) = ',s

         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,1,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF1) = ',s

         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,2,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF2) = ',s

         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,3,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF3) = ',s

         IF (gisymmetric.EQ.0) THEN
            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,4,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
            s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF4) = ',s

            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,5,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
            s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF5) = ',s

            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,6,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
            s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF6) = ',s
         END IF
      END IF 
#endif

#ifdef PROC_TRACE
      print*, ip, 'exiting mmBuildCoarseGen'
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      print*, ip, '<<< mmBuildCoarseGen enter', levelF
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmBuildCoarseGen 
C     
C     Purpose: Build a coarse-level matrix given the next-finer one
C     Purpose: for general problems
C
C     Algorithm: 
C
C     Inputs: 
C
C     Outputs: 
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate: 
C
C     Communication estimate: 
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBuildCoarseGen (
     &     ACD,AC1,AC2,AC3,AC4,AC5,AC6,nxdac,nydac,nzdac,bndac
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,nbr,bndc,bndf,levelC,icall)

      IMPLICIT NONE

      
      INTEGER nxdac,nydac,nzdac
      INTEGER bndac(3)
      REAL ACD(nxdac,nydac,nzdac)
      REAL AC1(nxdac,nydac,nzdac)
      REAL AC2(nxdac,nydac,nzdac)
      REAL AC3(nxdac,nydac,nzdac)
      REAL AC4(nxdac,nydac,nzdac)
      REAL AC5(nxdac,nydac,nzdac)
      REAL AC6(nxdac,nydac,nzdac)
      
      INTEGER nxdaf,nydaf,nzdaf
      INTEGER bndaf(3)  
      REAL AFD(nxdaf,nydaf,nzdaf)
      REAL AF1(nxdaf,nydaf,nzdaf)
      REAL AF2(nxdaf,nydaf,nzdaf)
      REAL AF3(nxdaf,nydaf,nzdaf)
      REAL AF4(nxdaf,nydaf,nzdaf)
      REAL AF5(nxdaf,nydaf,nzdaf)
      REAL AF6(nxdaf,nydaf,nzdaf)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)

      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      INTEGER comm,nbr(6),bndc(3),bndf(3),levelC

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#else 
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      INTEGER ixc,iyc,izc,ixf,iyf,izf
      REAL hrm,hrmh,a,b,ha,hb
      REAL dxm,dxp,dym,dyp,dzm,dzp
      REAL dxmi,dxpi,dymi,dypi,dzmi,dzpi
      INTEGER CX1,CY1,CZ1,CX2,CY2,CZ2
      INTEGER MX1,MY1,MZ1,MX2,MY2,MZ2
      INTEGER levelF

#ifdef DEBUG
      REAL s,mmVectorSum
#endif

      
      INTEGER isub,icall

      hrm (a,b) = (2.0D0)*a*b/(a+b)
      hrmh (a,b,ha,hb) = (ha+hb)*a*b/(ha*a+hb*b)

      
#ifdef PERF_EVAL
      isub = SB_BUILD+levelC
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      print*, ip, '>>> mmBuildCoarseGen enter', levelF
#endif

      levelF = levelC-1

      CX1 = 2
      CY1 = 2
      CZ1 = 2
      CX2 = nxc-1
      CY2 = nyc-1
      CZ2 = nzc-1
      MX1 = 1
      MY1 = 1
      MZ1 = 1
      IF (ipx.LT.npx-1) MX2 = nxc-2
      IF (ipx.EQ.npx-1) MX2 = nxc-1
      IF (ipy.LT.npy-1) MY2 = nyc-2
      IF (ipy.EQ.npy-1) MY2 = nyc-1
      IF (ipz.LT.npz-1) MZ2 = nzc-2
      IF (ipz.EQ.npz-1) MZ2 = nzc-1

*-----------------------------------------------------------------------
*     Compute the positive X-diagonal
*-----------------------------------------------------------------------

      IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,1,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
      ELSE
         CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AF1,nxdaf,nydaf,nzdaf,bndaf,
     &        nxf,nyf,nzf)
      END IF

      CALL mmGetYnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
      CALL mmGetZnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

      DO izc=CZ1,CZ2
         izf = 2*(izc-1)+1
         dzmi = gdzi(izf-1,levelF)
         dzpi = gdzi(izf,levelF)
         DO iyc=CY1,CY2
            iyf = 2*(iyc-1)+1
            dymi = gdyi(iyf-1,levelF)
            dypi = gdyi(iyf,levelF)
            DO ixc=MX1,MX2
               ixf = 2*(ixc-1)+1
               dxm = gdx(ixf,levelF)
               dxp = gdx(ixf+1,levelF)
               AC1(ixc,iyc,izc) = o4
     &              *(o2* hrmh(R(ixf,  iyf,izf),  
     &              R(ixf+1,iyf,izf),  dxm,dxp)
     &              + o2*(hrmh(R(ixf,  iyf-1,izf),
     &              R(ixf+1,iyf-1,izf),dxm,dxp)*dymi
     &              +     hrmh(R(ixf,  iyf+1,izf),
     &              R(ixf+1,iyf+1,izf),dxm,dxp)*dypi
     &              +     hrmh(R(ixf,  iyf,izf-1),
     &              R(ixf+1,iyf,izf-1),dxm,dxp)*dzmi
     &              +     hrmh(R(ixf,  iyf,izf+1),
     &              R(ixf+1,iyf,izf+1),dxm,dxp)*dzpi)
     &              /(dymi+dypi+dzmi+dzpi))
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (5*7+15)*(MX2-MX1+1)*(CY2-CY1+1)*(CZ2-CZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (5*7+15)*(MX2-MX1+1)*(CY2-CY1+1)*(CZ2-CZ1+1)
#endif

CCCCC @@ Macro dependent parameters
      CALL mmSendX1Ghost (AC1,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
      CALL mmGetY1Ghost (AC1,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
      CALL mmGetZ1Ghost (AC1,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

*-----------------------------------------------------------------------
*     Compute the positive Y-diagonal
*-----------------------------------------------------------------------

      IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,2,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
      ELSE
         CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AF2,nxdaf,nydaf,nzdaf,bndaf,
     &        nxf,nyf,nzf)
      END IF

      CALL mmGetZnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
      CALL mmGetXnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

      DO izc=CZ1,CZ2
         izf = 2*(izc-1)+1
         dzmi = gdzi(izf-1,levelF)
         dzpi = gdzi(izf,levelF)
         DO iyc=MY1,MY2
            iyf = 2*(iyc-1)+1
            dym = gdy(iyf,levelF)
            dyp = gdy(iyf+1,levelF)
            DO ixc=CX1,CX2
               ixf = 2*(ixc-1)+1
               dxmi = gdxi(ixf-1,levelF)
               dxpi = gdxi(ixf,levelF)
               AC2(ixc,iyc,izc) = o4
     &              *(o2 *hrmh(R(ixf,iyf,  izf),
     &              R(ixf,iyf+1,izf),dym,dyp)
     &              + o2*(hrmh(R(ixf-1,iyf,  izf),
     &              R(ixf-1,iyf+1,izf),dym,dyp)*dxmi
     &              +     hrmh(R(ixf+1,iyf,  izf),
     &              R(ixf+1,iyf+1,izf),dym,dyp)*dxpi
     &              +     hrmh(R(ixf,iyf,  izf-1),
     &              R(ixf,iyf+1,izf-1),dym,dyp)*dzmi
     &              +     hrmh(R(ixf,iyf,  izf+1),
     &              R(ixf,iyf+1,izf+1),dym,dyp)*dzpi)
     &              /(dxmi+dxpi+dzmi+dzpi))
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (5*7+15)*(CX2-CX1+1)*(MY2-MY1+1)*(CZ2-CZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (5*7+15)*(CX2-CX1+1)*(MY2-MY1+1)*(CZ2-CZ1+1)
#endif

CCCCC @@ Macro dependent parameters
      CALL mmGetX1Ghost (AC2,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
      CALL mmSendY1Ghost (AC2,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
      CALL mmGetZ1Ghost (AC2,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

*-----------------------------------------------------------------------
*     Compute the positive Z-diagonal
*-----------------------------------------------------------------------

      IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,3,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
      ELSE
         CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AF3,nxdaf,nydaf,nzdaf,bndaf,
     &        nxf,nyf,nzf)
      END IF

      CALL mmGetXnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
      CALL mmGetYnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

      DO izc=MZ1,MZ2
         izf = 2*(izc-1)+1
         dzm = gdz(izf,levelF)
         dzp = gdz(izf+1,levelF)
         DO iyc=CY1,CY2
            iyf = 2*(iyc-1)+1
            dymi = gdyi(iyf-1,levelF)
            dypi = gdyi(iyf,levelF)
            DO ixc=CX1,CX2
               ixf = 2*(ixc-1)+1
               dxmi = gdxi(ixf-1,levelF)
               dxpi = gdxi(ixf,levelF)
               AC3(ixc,iyc,izc) = o4
     &              *(o2 *hrmh(R(ixf,iyf,izf),
     &              R(ixf,iyf,izf+1),dzm,dzp)
     &              + o2*(hrmh(R(ixf-1,iyf,izf),
     &              R(ixf-1,iyf,izf+1),dzm,dzp)*dxmi
     &              +     hrmh(R(ixf+1,iyf,izf),
     &              R(ixf+1,iyf,izf+1),dzm,dzp)*dxpi
     &              +     hrmh(R(ixf,iyf-1,izf),
     &              R(ixf,iyf-1,izf+1),dzm,dzp)*dymi
     &              +     hrmh(R(ixf,iyf+1,izf),
     &              R(ixf,iyf+1,izf+1),dzm,dzp)*dypi)
     &              /(dxmi+dxpi+dymi+dypi))
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (5*7+15)*(CX2-CX1+1)*(CY2-CY1+1)*(MZ2-MZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (5*7+15)*(CX2-CX1+1)*(CY2-CY1+1)*(MZ2-MZ1+1)
#endif

CCCCC @@ Macro dependent parameters
      CALL mmGetX1Ghost (AC3,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
      CALL mmGetY1Ghost (AC3,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
      CALL mmSendZ1Ghost (AC3,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

*-----------------------------------------------------------------------
*     Compute the negative X-diagonal
*-----------------------------------------------------------------------

      IF (gisymmetric.EQ.0) THEN
         IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,4,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         ELSE
            CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AF4,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf)
         END IF

         CALL mmGetYnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
         CALL mmGetZnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

         DO izc=CZ1,CZ2
            izf = 2*(izc-1)+1
            dzmi = gdzi(izf-1,levelF)
            dzpi = gdzi(izf,levelF)
            DO iyc=CY1,CY2
               iyf = 2*(iyc-1)+1
               dymi = gdyi(iyf-1,levelF)
               dypi = gdyi(iyf,levelF)
               DO ixc=MX1,MX2
                  ixf = 2*(ixc-1)+1
                  dxm = gdx(ixf,levelF)
                  dxp = gdx(ixf+1,levelF)
                  AC4(ixc,iyc,izc) = o4
     &                 *(o2* hrmh(R(ixf,  iyf,izf),  
     &                 R(ixf+1,iyf,izf),dxm,dxp)
     &                 + o2*(hrmh(R(ixf,  iyf-1,izf),
     &                 R(ixf+1,iyf-1,izf),dxm,dxp)*dymi
     &                 +     hrmh(R(ixf,  iyf+1,izf),
     &                 R(ixf+1,iyf+1,izf),dxm,dxp)*dypi
     &                 +     hrmh(R(ixf,  iyf,izf-1),
     &                 R(ixf+1,iyf,izf-1),dxm,dxp)*dzmi
     &                 +     hrmh(R(ixf,  iyf,izf+1),
     &                 R(ixf+1,iyf,izf+1),dxm,dxp)*dzpi)
     &                 /(dymi+dypi+dzmi+dzpi))
               END DO
            END DO
         END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (5*7+15)*(MX2-MX1+1)*(CY2-CY1+1)*(CZ2-CZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (5*7+15)*(MX2-MX1+1)*(CY2-CY1+1)*(CZ2-CZ1+1)
#endif

CCCCC @@ Macro dependent parameters
         CALL mmSendX1Ghost (AC4,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
         CALL mmGetY1Ghost (AC4,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
         CALL mmGetZ1Ghost (AC4,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

*-----------------------------------------------------------------------
*     Compute the negative Y-diagonal
*-----------------------------------------------------------------------

         IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,5,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         ELSE
            CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AF5,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf)
         END IF

         CALL mmGetZnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
         CALL mmGetXnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

         DO izc=CZ1,CZ2
            izf = 2*(izc-1)+1
            dzmi = gdzi(izf-1,levelF)
            dzpi = gdzi(izf,levelF)
            DO iyc=MY1,MY2
               iyf = 2*(iyc-1)+1
               dym = gdy(iyf,levelF)
               dyp = gdy(iyf+1,levelF)
               DO ixc=CX1,CX2
                  ixf = 2*(ixc-1)+1
                  dxmi = gdxi(ixf-1,levelF)
                  dxpi = gdxi(ixf,levelF)
                  AC5(ixc,iyc,izc) = o4
     &                 *(o2 *hrmh(R(ixf,iyf,  izf),
     &                 R(ixf,iyf+1,izf),dym,dyp)
     &                 + o2*(hrmh(R(ixf-1,iyf,  izf),
     &                 R(ixf-1,iyf+1,izf),dym,dyp)*dxmi
     &                 +     hrmh(R(ixf+1,iyf,  izf),
     &                 R(ixf+1,iyf+1,izf),dym,dyp)*dxpi
     &                 +     hrmh(R(ixf,iyf,  izf-1),
     &                 R(ixf,iyf+1,izf-1),dym,dyp)*dzmi
     &                 +     hrmh(R(ixf,iyf,  izf+1),
     &                 R(ixf,iyf+1,izf+1),dym,dyp)*dzpi)
     &                 /(dxmi+dxpi+dzmi+dzpi))
               END DO
            END DO
         END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (5*7+15)*(CX2-CX1+1)*(MY2-MY1+1)*(CZ2-CZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (5*7+15)*(CX2-CX1+1)*(MY2-MY1+1)*(CZ2-CZ1+1)
#endif

CCCCC @@ Macro dependent parameters
         CALL mmGetX1Ghost (AC5,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
         CALL mmSendY1Ghost (AC5,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
         CALL mmGetZ1Ghost (AC5,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

*-----------------------------------------------------------------------
*     Compute the negative Z-diagonal
*-----------------------------------------------------------------------

         IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,6,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         ELSE
            CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AF6,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf)
         END IF

         CALL mmGetXnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
         CALL mmGetYnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

         DO izc=MZ1,MZ2
            izf = 2*(izc-1)+1
            dzm = gdz(izf,levelF)
            dzp = gdz(izf+1,levelF)
            DO iyc=CY1,CY2
               iyf = 2*(iyc-1)+1
               dymi = gdyi(iyf-1,levelF)
               dypi = gdyi(iyf,levelF)
               DO ixc=CX1,CX2
                  ixf = 2*(ixc-1)+1
                  dxmi = gdxi(ixf-1,levelF)
                  dxpi = gdxi(ixf,levelF)
                  AC6(ixc,iyc,izc) = o4
     &                 *(o2 *hrmh(R(ixf,iyf,izf),
     &                 R(ixf,iyf,izf+1),dzm,dzp)
     &                 + o2*(hrmh(R(ixf-1,iyf,izf),
     &                 R(ixf-1,iyf,izf+1),dzm,dzp)*dxmi
     &                 +     hrmh(R(ixf+1,iyf,izf),
     &                 R(ixf+1,iyf,izf+1),dzm,dzp)*dxpi
     &                 +     hrmh(R(ixf,iyf-1,izf),
     &                 R(ixf,iyf-1,izf+1),dzm,dzp)*dymi
     &                 +     hrmh(R(ixf,iyf+1,izf),
     &                 R(ixf,iyf+1,izf+1),dzm,dzp)*dypi)
     &                 /(dxmi+dxpi+dymi+dypi))
               END DO
            END DO
         END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (5*7+15)*(CX2-CX1+1)*(CY2-CY1+1)*(MZ2-MZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (5*7+15)*(CX2-CX1+1)*(CY2-CY1+1)*(MZ2-MZ1+1)
#endif

CCCCC @@ Macro dependent parameters
         CALL mmGetX1Ghost (AC6,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
         CALL mmGetY1Ghost (AC6,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
         CALL mmSendZ1Ghost (AC6,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

      END IF

*-----------------------------------------------------------------------
*     Compute the main diagonal of the matrix and the right-hand side
*-----------------------------------------------------------------------

      IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,0,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
      ELSE
         CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AFD,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf)
      END IF

      DO izc=CZ1,CZ2
         izf = 2*(izc-1)+1
         DO iyc=CY1,CY2
            iyf = 2*(iyc-1)+1
            DO ixc=CX1,CX2
               ixf = 2*(ixc-1)+1
               ACD(ixc,iyc,izc) = o4*R(ixf,iyf,izf)
CCC ( ACD = -(AC1 + ... + AC6) is not guaranteed for general problems! )
c$$$               ACD(ixc,iyc,izc) = 
c$$$     &              - AC1(ixc-1,iyc,izc) 
c$$$     &              - AC2(ixc,iyc-1,izc)
c$$$     &              - AC3(ixc,iyc,izc-1)
c$$$     &              - AC4(ixc,iyc,izc)
c$$$     &              - AC5(ixc,iyc,izc)
c$$$     &              - AC6(ixc,iyc,izc)
            END DO
         END DO
      END DO


      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (CX2-CX1+1)*(CY2-CY1+1)*(CZ2-CZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (CX2-CX1+1)*(CY2-CY1+1)*(CZ2-CZ1+1)
#endif

#ifdef DEBUG
      s = mmVectorSum (ACD,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
      if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(ACD) = ',s
      s = mmVectorSum (AC1,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
      if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AC1) = ',s
      s = mmVectorSum (AC2,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
      if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AC2) = ',s
      s = mmVectorSum (AC3,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
      if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AC3) = ',s
      IF (gisymmetric.EQ.0) THEN
         s = mmVectorSum (AC4,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AC4) = ',s
         s = mmVectorSum (AC5,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AC5) = ',s
         s = mmVectorSum (AC6,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AC6) = ',s
      END IF 

      IF (gimatfree.EQ.0.OR.levelC.GT.2) THEN

         s = mmVectorSum (AFD,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf,comm,
     &     isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AFD) = ',s
         s = mmVectorSum (AF1,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf,comm,
     &     isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF1) = ',s
         s = mmVectorSum (AF2,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf,comm,
     &     isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF2) = ',s
         s = mmVectorSum (AF3,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf,comm,
     &     isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF3) = ',s
         IF (gisymmetric.EQ.0) THEN
            s = mmVectorSum (AF4,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF4) = ',s
            s = mmVectorSum (AF5,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF5) = ',s
            s = mmVectorSum (AF6,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF6) = ',s
         END IF
      ELSE
         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,0,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AFD) = ',s

         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,1,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF1) = ',s

         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,2,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF2) = ',s

         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,3,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF3) = ',s

         IF (gisymmetric.EQ.0) THEN
            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,4,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
            s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF4) = ',s

            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,5,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
            s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF5) = ',s

            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,6,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
            s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseGen sum(AF6) = ',s
         END IF
      END IF 
#endif

#ifdef PROC_TRACE
      print*, ip, 'exiting mmBuildCoarseGen'
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      print*, ip, '<<< mmBuildCoarseGen enter', levelF
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmBuildCoarseSCGen 
C     
C     Purpose: Build a coarse-level matrix given the next-finer one
C     Purpose: for general problems
C     
C     Algorithm: 
C     
C     Inputs: 
C     
C     Outputs: 
C     
C     Globals accessed:
C     
C     Globals modified:
C     
C     Flop count estimate: 
C     
C     Communication estimate: 
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C     
C     Modification history: 
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBuildCoarseSCGen (
     &     ACD,AC1,AC2,AC3,AC4,AC5,AC6,nxdac,nydac,nzdac,bndac
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,nbr,bndc,bndf,levelC,icall)

      IMPLICIT NONE

      
      INTEGER nxdac,nydac,nzdac
      INTEGER bndac(3)
      REAL ACD(nxdac,nydac,nzdac)
      REAL AC1(nxdac,nydac,nzdac)
      REAL AC2(nxdac,nydac,nzdac)
      REAL AC3(nxdac,nydac,nzdac)
      REAL AC4(nxdac,nydac,nzdac)
      REAL AC5(nxdac,nydac,nzdac)
      REAL AC6(nxdac,nydac,nzdac)
      
      INTEGER nxdaf,nydaf,nzdaf
      INTEGER bndaf(3)  
      REAL AFD(nxdaf,nydaf,nzdaf)
      REAL AF1(nxdaf,nydaf,nzdaf)
      REAL AF2(nxdaf,nydaf,nzdaf)
      REAL AF3(nxdaf,nydaf,nzdaf)
      REAL AF4(nxdaf,nydaf,nzdaf)
      REAL AF5(nxdaf,nydaf,nzdaf)
      REAL AF6(nxdaf,nydaf,nzdaf)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)

      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      INTEGER comm,nbr(6),bndc(3),bndf(3),levelC

#ifdef DEBUG
      REAL s,mmVectorSum
      EXTERNAL mmVectorSum
#endif

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#else 
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif


      INTEGER ixc,iyc,izc,ixf,iyf,izf
      REAL hrm,hrmh,a,b,ha,hb
      REAL dxm,dxp,dym,dyp,dzm,dzp
      REAL dxmi,dxpi,dymi,dypi,dzmi,dzpi
      INTEGER CX1,CY1,CZ1,CX2,CY2,CZ2
      INTEGER MX1,MY1,MZ1,MX2,MY2,MZ2
      INTEGER levelF
      INTEGER idx,idy,idz,idim
      REAL wgt

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_BUILD+levelC
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      print*, ip, '>>> mmBuildCoarseSCGen enter', levelF
#endif

      idx = 1
      idy = 1
      idz = 1
      IF (gmmlc1) idx = 2 
      IF (gmmlc2) idy = 2 
      IF (gmmlc3) idz = 2
      idim = 0
      
      IF (gmmlc1) idim = idim + 1
      IF (gmmlc2) idim = idim + 1
      IF (gmmlc3) idim = idim + 1
      
      levelF = levelC-1

      CX1 = 2
      CY1 = 2
      CZ1 = 2
      CX2 = nxc-1
      CY2 = nyc-1
      CZ2 = nzc-1
      MX1 = 1
      MY1 = 1
      MZ1 = 1
      MX2 = nxc-1
      MY2 = nyc-1
      MZ2 = nzc-1
      IF (ipx.LT.npx-1 .AND. gmmlc1) MX2 = nxc-2
      IF (ipy.LT.npy-1 .AND. gmmlc2) MY2 = nyc-2
      IF (ipz.LT.npz-1 .AND. gmmlc3) MZ2 = nzc-2

#ifdef DEBUG
      if (ip.EQ.0) PRINT*, 'mmBuildCoarseSCGen 1'
#endif
      
      IF (idim.EQ.3) THEN
         
         PRINT*, '*** MGMPI ***: mmBuildCoarseSCGen called as full'
         PRINT*, '               coarsening build-coarse!'
         PRINT*, '               Calling mmBuildCoarseSCGen...'
         CALL mmBuildCoarseGen (
     &     ACD,AC1,AC2,AC3,AC4,AC5,AC6,nxdac,nydac,nzdac,bndac
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,nbr,bndc,bndf,levelC,icall)
         
         GOTO 10
         
      END IF

#ifdef DEBUG
      if (ip.EQ.0) PRINT*, 'mmBuildCoarseSCGen 123'
#endif
      CALL mmBuildCoarseSCGen123 (
     &     ACD,AC1,AC2,AC3,AC4,AC5,AC6,nxdac,nydac,nzdac,bndac
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     nxc,nyc,nzc,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,nbr,bndc,bndf,levelC,isub)
#ifdef DEBUG
      if (ip.EQ.0) PRINT*, 'mmBuildCoarseSCGen 456'
#endif
      CALL mmBuildCoarseSCGen456 (
     &     ACD,AC1,AC2,AC3,AC4,AC5,AC6,nxdac,nydac,nzdac,bndac
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     nxc,nyc,nzc,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,nbr,bndc,bndf,levelC,isub)

#ifdef DEBUG
      if (ip.EQ.0) PRINT*, 'mmBuildCoarseSCGen main diag'
#endif
*-----------------------------------------------------------------------
*     Compute the main diagonal of the matrix and the right-hand side
*-----------------------------------------------------------------------

      IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,0,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
      ELSE
         CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AFD,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf)
      END IF

      IF (idim.EQ.1) wgt=9.0D0 / 16.0D0
      IF (idim.EQ.2) wgt=o2

      IF (ip.EQ.0) THEN
         PRINT*, '*** MGMPI ***: In mmBuildCoarseSCGen, diagonal'
         PRINT*, '               is computed as sum of off-diagonals:'
         PRINT*, '               May not be accurate for non-Poisson'
         PRINT*, '               problems'
      END IF

      DO izc=CZ1,CZ2
         izf = 2*(izc-1)+1
         DO iyc=CY1,CY2
            iyf = 2*(iyc-1)+1
            DO ixc=CX1,CX2
               ixf = 2*(ixc-1)+1
CCC   ( ACD = -(AC1 + ... + AC6) is not guaranteed for general problems! )
C               ACD(ixc,iyc,izc) = wgt * R(ixf,iyf,izf)
               ACD(ixc,iyc,izc) = -(AC1(ixc,iyc,izc)
     &              + AC2(ixc,iyc,izc)
     &              + AC3(ixc,iyc,izc)
     &              + AC4(ixc,iyc,izc)
     &              + AC5(ixc,iyc,izc)
     &              + AC6(ixc,iyc,izc))
            END DO
         END DO
      END DO

#ifdef DEBUG
      if (ip.EQ.0) PRINT*, 'mmBuildCoarseSCGen exit'
#endif

 10   CONTINUE

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      print*, ip, '<<< mmBuildCoarseSCGen', levelF
#endif

      RETURN
      END


      SUBROUTINE mmBuildCoarseSCGen123 (
     &     ACD,AC1,AC2,AC3,AC4,AC5,AC6,nxdac,nydac,nzdac,bndac
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     nxc,nyc,nzc,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,nbr,bndc,bndf,levelC,icall)

      IMPLICIT NONE

      
      INTEGER nxdac,nydac,nzdac
      INTEGER bndac(3)
      REAL ACD(nxdac,nydac,nzdac)
      REAL AC1(nxdac,nydac,nzdac)
      REAL AC2(nxdac,nydac,nzdac)
      REAL AC3(nxdac,nydac,nzdac)
      REAL AC4(nxdac,nydac,nzdac)
      REAL AC5(nxdac,nydac,nzdac)
      REAL AC6(nxdac,nydac,nzdac)
      
      INTEGER nxdaf,nydaf,nzdaf
      INTEGER bndaf(3)  
      REAL AFD(nxdaf,nydaf,nzdaf)
      REAL AF1(nxdaf,nydaf,nzdaf)
      REAL AF2(nxdaf,nydaf,nzdaf)
      REAL AF3(nxdaf,nydaf,nzdaf)
      REAL AF4(nxdaf,nydaf,nzdaf)
      REAL AF5(nxdaf,nydaf,nzdaf)
      REAL AF6(nxdaf,nydaf,nzdaf)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)

      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      INTEGER comm,nbr(6),bndc(3),bndf(3),levelC

#ifdef DEBUG
      REAL s,mmVectorSum
      EXTERNAL mmVectorSum
#endif

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#else 
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      INTEGER ixc,iyc,izc,ixf,iyf,izf
      REAL hrm,hrmh,a,b,ha,hb
      REAL dxm,dxp,dym,dyp,dzm,dzp
      REAL dxmi,dxpi,dymi,dypi,dzmi,dzpi
      INTEGER CX1,CY1,CZ1,CX2,CY2,CZ2
      INTEGER MX1,MY1,MZ1,MX2,MY2,MZ2
      INTEGER levelF
      INTEGER idx,idy,idz,idim
      REAL wgt

      
      INTEGER isub,icall

      hrm (a,b) = (2.0D0)*a*b/(a+b)
      hrmh (a,b,ha,hb) = (ha+hb)*a*b/(ha*a+hb*b)

      
#ifdef PERF_EVAL
      isub = SB_BUILD+levelC
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      print*, ip, '>>> mmBuildCoarseSCGen enter', levelF
#endif

      idx = 1
      idy = 1
      idz = 1
      IF (gmmlc1) idx = 2 
      IF (gmmlc2) idy = 2 
      IF (gmmlc3) idz = 2
      idim = 0
      
      IF (gmmlc1) idim = idim + 1
      IF (gmmlc2) idim = idim + 1
      IF (gmmlc3) idim = idim + 1
      
      levelF = levelC-1

      CX1 = 2
      CY1 = 2
      CZ1 = 2
      CX2 = nxc-1
      CY2 = nyc-1
      CZ2 = nzc-1
      MX1 = 1
      MY1 = 1
      MZ1 = 1
      MX2 = nxc-1
      MY2 = nyc-1
      MZ2 = nzc-1
      IF (ipx.LT.npx-1 .AND. gmmlc1) MX2 = nxc-2
      IF (ipy.LT.npy-1 .AND. gmmlc2) MY2 = nyc-2
      IF (ipz.LT.npz-1 .AND. gmmlc3) MZ2 = nzc-2

*-----------------------------------------------------------------------
*     Compute the negative X-diagonal
*-----------------------------------------------------------------------

      IF (gisymmetric.EQ.0) THEN

         IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,4,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         ELSE
            CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AF4,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf)
         END IF

         IF (gmmlc2) CALL mmGetYnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
         IF (gmmlc3) CALL mmGetZnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

         IF (gmmlc1) THEN
            IF (.NOT.gmmlc2) THEN
               DO izf=1,nzf
                  DO ixf=1,nxf
                     R(ixf,  1,izf) = R(ixf,    2,izf)
                     R(ixf,nyf,izf) = R(ixf,nyf-1,izf)
                  END DO
               END DO
            END IF
            IF (.NOT.gmmlc3) THEN
               DO iyf=1,nyf
                  DO ixf=1,nxf
                     R(ixf,iyf,  1) = R(ixf,iyf,    2)
                     R(ixf,iyf,nzf) = R(ixf,iyf,nzf-1)
                  END DO
               END DO
            END IF
            DO izc=CZ1,CZ2
               izf = (izc-1)*idz+1
               dzmi = gdzi(izf-1,levelF)
               dzpi = gdzi(izf,levelF)
               DO iyc=CY1,CY2
                  iyf = (iyc-1)*idy+1
                  dymi = gdyi(iyf-1,levelF)
                  dypi = gdyi(iyf,levelF)
                  DO ixc=MX1,MX2
                     ixf = (ixc-1)*idx+1
                     dxm = gdx(ixf,levelF)
                     dxp = gdx(ixf+1,levelF)
                     AC4(ixc,iyc,izc) = o4
     &                    *(o2* hrmh(R(ixf,  iyf,izf),  
     &                    R(ixf+1,iyf,izf),dxm,dxp)
     &                    + o2*(hrmh(R(ixf,  iyf-1,izf),
     &                    R(ixf+1,iyf-1,izf),dxm,dxp)*dymi
     &                    +     hrmh(R(ixf,  iyf+1,izf),
     &                    R(ixf+1,iyf+1,izf),dxm,dxp)*dypi
     &                    +     hrmh(R(ixf,  iyf,izf-1),
     &                    R(ixf+1,iyf,izf-1),dxm,dxp)*dzmi
     &                    +     hrmh(R(ixf,  iyf,izf+1),
     &                    R(ixf+1,iyf,izf+1),dxm,dxp)*dzpi)
     &                    /(dymi+dypi+dzmi+dzpi))
                  END DO
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (5*7+15)*(MX2-MX1+1)*(CY2-CY1+1)*(CZ2-CZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (5*7+15)*(MX2-MX1+1)*(CY2-CY1+1)*(CZ2-CZ1+1)
#endif
         ELSE
            DO izc=CZ1,CZ2
               izf = (izc-1)*idz+1
               DO iyc=CY1,CY2
                  iyf = (iyc-1)*idy+1
                  DO ixc=MX1,MX2
                     ixf = (ixc-1)*idx+1
                     AC4(ixc,iyc,izc) = R(ixf,  iyf,izf)
                  END DO
               END DO
            END DO
         END IF


CCCCC @@ Macro dependent parameters
         IF (gmmlc1) CALL mmSendX1Ghost (AC4,nxdac,nydac,nzdac,bndac,
     &        nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
         IF (gmmlc2) CALL mmGetY1Ghost (AC4,nxdac,nydac,nzdac,bndac,
     &        nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
         IF (gmmlc3) CALL mmGetZ1Ghost (AC4,nxdac,nydac,nzdac,bndac,
     &        nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

      END IF

*-----------------------------------------------------------------------
*     Compute the negative Y-diagonal
*-----------------------------------------------------------------------

      IF (gisymmetric.EQ.0) THEN

         IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,5,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         ELSE
            CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AF5,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf)
         END IF

         IF (gmmlc3) CALL mmGetZnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
         IF (gmmlc1) CALL mmGetXnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

         IF (gmmlc2) THEN
            IF (.NOT.gmmlc1) THEN
               DO izf=1,nzf
                  DO iyf=1,nyf
                     R(  1,iyf,izf) = R(    2,iyf,izf)
                     R(nxf,iyf,izf) = R(nxf-1,iyf,izf)
                  END DO
               END DO
            END IF
            IF (.NOT.gmmlc3) THEN
               DO iyf=1,nyf
                  DO ixf=1,nxf
                     R(ixf,iyf,  1) = R(ixf,iyf,    2)
                     R(ixf,iyf,nzf) = R(ixf,iyf,nzf-1)
                  END DO
               END DO
            END IF
            DO izc=CZ1,CZ2
               izf = (izc-1)*idz+1
               dzmi = gdzi(izf-1,levelF)
               dzpi = gdzi(izf,levelF)
               DO iyc=MY1,MY2
                  iyf = (iyc-1)*idy+1
                  dym = gdy(iyf,levelF)
                  dyp = gdy(iyf+1,levelF)
                  DO ixc=CX1,CX2
                     ixf = (ixc-1)*idx+1
                     dxmi = gdxi(ixf-1,levelF)
                     dxpi = gdxi(ixf,levelF)
                     AC5(ixc,iyc,izc) = o4
     &                    *(o2 *hrmh(R(ixf,iyf,  izf),
     &                    R(ixf,iyf+1,izf),dym,dyp)
     &                    + o2*(hrmh(R(ixf-1,iyf,  izf),
     &                    R(ixf-1,iyf+1,izf),dym,dyp)*dxmi
     &                    +     hrmh(R(ixf+1,iyf,  izf),
     &                    R(ixf+1,iyf+1,izf),dym,dyp)*dxpi
     &                    +     hrmh(R(ixf,iyf,  izf-1),
     &                    R(ixf,iyf+1,izf-1),dym,dyp)*dzmi
     &                    +     hrmh(R(ixf,iyf,  izf+1),
     &                    R(ixf,iyf+1,izf+1),dym,dyp)*dzpi)
     &                    /(dxmi+dxpi+dzmi+dzpi))
                  END DO
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (5*7+15)*(CX2-CX1+1)*(MY2-MY1+1)*(CZ2-CZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (5*7+15)*(CX2-CX1+1)*(MY2-MY1+1)*(CZ2-CZ1+1)
#endif
         ELSE
            DO izc=CZ1,CZ2
               izf = (izc-1)*idz+1
               DO iyc=MY1,MY2
                  iyf = (iyc-1)*idy+1
                  DO ixc=CX1,CX2
                     ixf = (ixc-1)*idx+1
                     AC5(ixc,iyc,izc) = R(ixf,  iyf,izf)
                  END DO
               END DO
            END DO
         END IF


CCCCC @@ Macro dependent parameters
         IF (gmmlc1) CALL mmGetX1Ghost (AC5,nxdac,nydac,nzdac,bndac,
     &        nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
         IF (gmmlc2) CALL mmSendY1Ghost (AC5,nxdac,nydac,nzdac,bndac,
     &        nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
         IF (gmmlc3) CALL mmGetZ1Ghost (AC5,nxdac,nydac,nzdac,bndac,
     &        nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

      END IF

*-----------------------------------------------------------------------
*     Compute the negative Z-diagonal
*-----------------------------------------------------------------------

      IF (gisymmetric.EQ.0) THEN

         IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,6,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         ELSE
            CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AF6,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf)
         END IF

         IF (gmmlc1) CALL mmGetXnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
         IF (gmmlc2) CALL mmGetYnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

         IF (gmmlc3) THEN
            IF (.NOT.gmmlc1) THEN
               DO izf=1,nzf
                  DO iyf=1,nyf
                     R(  1,iyf,izf) = R(    2,iyf,izf)
                     R(nxf,iyf,izf) = R(nxf-1,iyf,izf)
                  END DO
               END DO
            END IF
            IF (.NOT.gmmlc2) THEN
               DO izf=1,nzf
                  DO ixf=1,nxf
                     R(ixf,  1,izf) = R(ixf,    2,izf)
                     R(ixf,nyf,izf) = R(ixf,nyf-1,izf)
                  END DO
               END DO
            END IF
            DO izc=MZ1,MZ2
               izf = (izc-1)*idz+1
               dzm = gdz(izf,levelF)
               dzp = gdz(izf+1,levelF)
               DO iyc=CY1,CY2
                  iyf = (iyc-1)*idy+1
                  dymi = gdyi(iyf-1,levelF)
                  dypi = gdyi(iyf,levelF)
                  DO ixc=CX1,CX2
                     ixf = (ixc-1)*idx+1
                     dxmi = gdxi(ixf-1,levelF)
                     dxpi = gdxi(ixf,levelF)
                     AC6(ixc,iyc,izc) = o4
     &                    *(o2 *hrmh(R(ixf,iyf,izf),
     &                    R(ixf,iyf,izf+1),dzm,dzp)
     &                    + o2*(hrmh(R(ixf-1,iyf,izf),
     &                    R(ixf-1,iyf,izf+1),dzm,dzp)*dxmi
     &                    +     hrmh(R(ixf+1,iyf,izf),
     &                    R(ixf+1,iyf,izf+1),dzm,dzp)*dxpi
     &                    +     hrmh(R(ixf,iyf-1,izf),
     &                    R(ixf,iyf-1,izf+1),dzm,dzp)*dymi
     &                    +     hrmh(R(ixf,iyf+1,izf),
     &                    R(ixf,iyf+1,izf+1),dzm,dzp)*dypi)
     &                    /(dxmi+dxpi+dymi+dypi))
                  END DO
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (5*7+15)*(CX2-CX1+1)*(CY2-CY1+1)*(MZ2-MZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (5*7+15)*(CX2-CX1+1)*(CY2-CY1+1)*(MZ2-MZ1+1)
#endif
         ELSE
            DO izc=MZ1,MZ2
               izf = (izc-1)*idz+1
               DO iyc=CY1,CY2
                  iyf = (iyc-1)*idy+1
                  DO ixc=CX1,CX2
                     ixf = (ixc-1)*idx+1
                     AC6(ixc,iyc,izc) = R(ixf,  iyf,izf)
                  END DO
               END DO
            END DO
         END IF


CCCCC @@ Macro dependent parameters
         IF (gmmlc1) CALL mmGetX1Ghost (AC6,nxdac,nydac,nzdac,bndac,
     &        nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
         IF (gmmlc2) CALL mmGetY1Ghost (AC6,nxdac,nydac,nzdac,bndac,
     &        nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
         IF (gmmlc3) CALL mmSendZ1Ghost (AC6,nxdac,nydac,nzdac,bndac,
     &        nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

      END IF

#ifdef DEBUG
      s = mmVectorSum (ACD,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
      if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(ACD) = ',s
      s = mmVectorSum (AC1,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
      if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AC1) = ',s
      s = mmVectorSum (AC2,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
      if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AC2) = ',s
      s = mmVectorSum (AC3,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
      if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AC3) = ',s
      IF (gisymmetric.EQ.0) THEN
         s = mmVectorSum (AC4,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &        isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AC4) = ',s
         s = mmVectorSum (AC5,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &        isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AC5) = ',s
         s = mmVectorSum (AC6,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &        isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AC6) = ',s
      END IF 

      IF (gimatfree.EQ.0.OR.levelC.GT.2) THEN

         s = mmVectorSum (AFD,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf,comm,
     &        isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AFD) = ',s
         s = mmVectorSum (AF1,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf,comm,
     &        isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AF1) = ',s
         s = mmVectorSum (AF2,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf,comm,
     &        isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AF2) = ',s
         s = mmVectorSum (AF3,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf,comm,
     &        isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AF3) = ',s
         IF (gisymmetric.EQ.0) THEN
            s = mmVectorSum (AF4,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,
     &           'mmBuildCoarseSCGen sum(AF4) = ',s
            s = mmVectorSum (AF5,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,
     &           'mmBuildCoarseSCGen sum(AF5) = ',s
            s = mmVectorSum (AF6,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,
     &           'mmBuildCoarseSCGen sum(AF6) = ',s
         END IF
      ELSE
         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,0,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AFD) = ',s

         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,1,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AF1) = ',s

         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,2,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AF2) = ',s

         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,3,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AF3) = ',s

         IF (gisymmetric.EQ.0) THEN
            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,4,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
            s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,
     &           'mmBuildCoarseSCGen sum(AF4) = ',s

            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,5,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
            s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,
     &           'mmBuildCoarseSCGen sum(AF5) = ',s

            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,6,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
            s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,
     &           'mmBuildCoarseSCGen sum(AF6) = ',s
         END IF
      END IF 
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END


      SUBROUTINE mmBuildCoarseSCGen456 (
     &     ACD,AC1,AC2,AC3,AC4,AC5,AC6,nxdac,nydac,nzdac,bndac
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     nxc,nyc,nzc,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,nbr,bndc,bndf,levelC,icall)

      IMPLICIT NONE

      
      INTEGER nxdac,nydac,nzdac
      INTEGER bndac(3)
      REAL ACD(nxdac,nydac,nzdac)
      REAL AC1(nxdac,nydac,nzdac)
      REAL AC2(nxdac,nydac,nzdac)
      REAL AC3(nxdac,nydac,nzdac)
      REAL AC4(nxdac,nydac,nzdac)
      REAL AC5(nxdac,nydac,nzdac)
      REAL AC6(nxdac,nydac,nzdac)
      
      INTEGER nxdaf,nydaf,nzdaf
      INTEGER bndaf(3)  
      REAL AFD(nxdaf,nydaf,nzdaf)
      REAL AF1(nxdaf,nydaf,nzdaf)
      REAL AF2(nxdaf,nydaf,nzdaf)
      REAL AF3(nxdaf,nydaf,nzdaf)
      REAL AF4(nxdaf,nydaf,nzdaf)
      REAL AF5(nxdaf,nydaf,nzdaf)
      REAL AF6(nxdaf,nydaf,nzdaf)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)

      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      INTEGER comm,nbr(6),bndc(3),bndf(3),levelC

#ifdef DEBUG
      REAL s,mmVectorSum
      EXTERNAL mmVectorSum
#endif

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#else 
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      INTEGER ixc,iyc,izc,ixf,iyf,izf
      REAL hrm,hrmh,a,b,ha,hb
      REAL dxm,dxp,dym,dyp,dzm,dzp
      REAL dxmi,dxpi,dymi,dypi,dzmi,dzpi
      INTEGER CX1,CY1,CZ1,CX2,CY2,CZ2
      INTEGER MX1,MY1,MZ1,MX2,MY2,MZ2
      INTEGER levelF
      INTEGER idx,idy,idz,idim
      REAL wgt

      
      INTEGER isub,icall

      hrm (a,b) = (2.0D0)*a*b/(a+b)
      hrmh (a,b,ha,hb) = (ha+hb)*a*b/(ha*a+hb*b)

      
#ifdef PERF_EVAL
      isub = SB_BUILD+levelC
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      print*, ip, '>>> mmBuildCoarseSCGen enter', levelF
#endif

      idx = 1
      idy = 1
      idz = 1
      IF (gmmlc1) idx = 2 
      IF (gmmlc2) idy = 2 
      IF (gmmlc3) idz = 2
      idim = 0
      
      IF (gmmlc1) idim = idim + 1
      IF (gmmlc2) idim = idim + 1
      IF (gmmlc3) idim = idim + 1
      
      levelF = levelC-1

      CX1 = 2
      CY1 = 2
      CZ1 = 2
      CX2 = nxc-1
      CY2 = nyc-1
      CZ2 = nzc-1
      MX1 = 1
      MY1 = 1
      MZ1 = 1
      MX2 = nxc-1
      MY2 = nyc-1
      MZ2 = nzc-1
      IF (ipx.LT.npx-1 .AND. gmmlc1) MX2 = nxc-2
      IF (ipy.LT.npy-1 .AND. gmmlc2) MY2 = nyc-2
      IF (ipz.LT.npz-1 .AND. gmmlc3) MZ2 = nzc-2

*-----------------------------------------------------------------------
*     Compute the positive X-diagonal
*-----------------------------------------------------------------------

      IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,1,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
      ELSE
         CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AF1,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf)
      END IF

      IF (gmmlc2) CALL mmGetYnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
      IF (gmmlc3) CALL mmGetZnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

      IF (gmmlc1) THEN
         IF (.NOT.gmmlc2) THEN
            DO izf=1,nzf
               DO ixf=1,nxf
                  R(ixf,  1,izf) = R(ixf,    2,izf)
                  R(ixf,nyf,izf) = R(ixf,nyf-1,izf)
               END DO
            END DO
         END IF
         IF (.NOT.gmmlc3) THEN
            DO iyf=1,nyf
               DO ixf=1,nxf
                  R(ixf,iyf,  1) = R(ixf,iyf,    2)
                  R(ixf,iyf,nzf) = R(ixf,iyf,nzf-1)
               END DO
            END DO
         END IF
         DO izc=CZ1,CZ2
            izf = (izc-1)*idz+1
            dzmi = gdzi(izf-1,levelF)
            dzpi = gdzi(izf,levelF)
            DO iyc=CY1,CY2
               iyf = (iyc-1)*idy+1
               dymi = gdyi(iyf-1,levelF)
               dypi = gdyi(iyf,levelF)
               DO ixc=MX1,MX2
                  ixf = (ixc-1)*idx+1
                  dxm = gdx(ixf,levelF)
                  dxp = gdx(ixf+1,levelF)
                  AC1(ixc,iyc,izc) = o4
     &                 *(o2* hrmh(R(ixf,  iyf,izf),  
     &                 R(ixf+1,iyf,izf),  dxm,dxp)
     &                 + o2*(hrmh(R(ixf,  iyf-1,izf),
     &                 R(ixf+1,iyf-1,izf),dxm,dxp)*dymi
     &                 +     hrmh(R(ixf,  iyf+1,izf),
     &                 R(ixf+1,iyf+1,izf),dxm,dxp)*dypi
     &                 +     hrmh(R(ixf,  iyf,izf-1),
     &                 R(ixf+1,iyf,izf-1),dxm,dxp)*dzmi
     &                 +     hrmh(R(ixf,  iyf,izf+1),
     &                 R(ixf+1,iyf,izf+1),dxm,dxp)*dzpi)
     &                 / (dymi+dypi+dzmi+dzpi))
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (5*7+15)*(MX2-MX1+1)*(CY2-CY1+1)*(CZ2-CZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (5*7+15)*(MX2-MX1+1)*(CY2-CY1+1)*(CZ2-CZ1+1)
#endif
      ELSE
         DO izc=CZ1,CZ2
            izf = (izc-1)*idz+1
            DO iyc=CY1,CY2
               iyf = (iyc-1)*idy+1
               DO ixc=MX1,MX2
                  ixf = (ixc-1)*idx+1
                  AC1(ixc,iyc,izc) = R(ixf,  iyf,izf)
               END DO
            END DO
         END DO
      END IF


CCCCC @@ Macro dependent parameters
      IF (gmmlc1) CALL mmSendX1Ghost (AC1,nxdac,nydac,nzdac,bndac,
     &     nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
      IF (gmmlc2) CALL mmGetY1Ghost (AC1,nxdac,nydac,nzdac,bndac,
     &     nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
      IF (gmmlc3) CALL mmGetZ1Ghost (AC1,nxdac,nydac,nzdac,bndac,
     &     nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

*-----------------------------------------------------------------------
*     Compute the positive Y-diagonal
*-----------------------------------------------------------------------

      IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,2,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
      ELSE
         CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AF2,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf)
      END IF

      IF (gmmlc3) CALL mmGetZnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
      IF (gmmlc1) CALL mmGetXnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

      IF (gmmlc2) THEN
         IF (.NOT.gmmlc1) THEN
            DO izf=1,nzf
               DO iyf=1,nyf
                  R(  1,iyf,izf) = R(    2,iyf,izf)
                  R(nxf,iyf,izf) = R(nxf-1,iyf,izf)
               END DO
            END DO
         END IF
         IF (.NOT.gmmlc3) THEN
            DO iyf=1,nyf
               DO ixf=1,nxf
                  R(ixf,iyf,  1) = R(ixf,iyf,    2)
                  R(ixf,iyf,nzf) = R(ixf,iyf,nzf-1)
               END DO
            END DO
         END IF
         DO izc=CZ1,CZ2
            izf = (izc-1)*idz+1
            dzmi = gdzi(izf-1,levelF)
            dzpi = gdzi(izf,levelF)
            DO iyc=MY1,MY2
               iyf = (iyc-1)*idy+1
               dym = gdy(iyf,levelF)
               dyp = gdy(iyf+1,levelF)
               DO ixc=CX1,CX2
                  ixf = (ixc-1)*idx+1
                  dxmi = gdxi(ixf-1,levelF)
                  dxpi = gdxi(ixf,levelF)
                  AC2(ixc,iyc,izc) = o4
     &                 *(o2 *hrmh(R(ixf,iyf,  izf),
     &                 R(ixf,iyf+1,izf),dym,dyp)
     &                 + o2*(hrmh(R(ixf-1,iyf,  izf),
     &                 R(ixf-1,iyf+1,izf),dym,dyp)*dxmi
     &                 +     hrmh(R(ixf+1,iyf,  izf),
     &                 R(ixf+1,iyf+1,izf),dym,dyp)*dxpi
     &                 +     hrmh(R(ixf,iyf,  izf-1),
     &                 R(ixf,iyf+1,izf-1),dym,dyp)*dzmi
     &                 +     hrmh(R(ixf,iyf,  izf+1),
     &                 R(ixf,iyf+1,izf+1),dym,dyp)*dzpi)
     &                 /(dxmi+dxpi+dzmi+dzpi))
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (5*7+15)*(CX2-CX1+1)*(MY2-MY1+1)*(CZ2-CZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (5*7+15)*(CX2-CX1+1)*(MY2-MY1+1)*(CZ2-CZ1+1)
#endif
      ELSE
         DO izc=CZ1,CZ2
            izf = (izc-1)*idz+1
            DO iyc=MY1,MY2
               iyf = (iyc-1)*idy+1
               DO ixc=CX1,CX2
                  ixf = (ixc-1)*idx+1
                  AC2(ixc,iyc,izc) = R(ixf,  iyf,izf)
               END DO
            END DO
         END DO
      END IF


CCCCC @@ Macro dependent parameters
      IF (gmmlc1) CALL mmGetX1Ghost (AC2,nxdac,nydac,nzdac,bndac,
     &     nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
      IF (gmmlc2) CALL mmSendY1Ghost (AC2,nxdac,nydac,nzdac,bndac,
     &     nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
      IF (gmmlc3) CALL mmGetZ1Ghost (AC2,nxdac,nydac,nzdac,bndac,
     &     nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

*-----------------------------------------------------------------------
*     Compute the positive Z-diagonal
*-----------------------------------------------------------------------

      IF (gimatfree.EQ.1 .AND. levelF.EQ.1) THEN
         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,3,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
      ELSE
         CALL mmVectorCopy (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,AF3,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf)
      END IF

      IF (gmmlc1) CALL mmGetXnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
      IF (gmmlc2) CALL mmGetYnGhost (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

      IF (gmmlc3) THEN
         IF (.NOT.gmmlc1) THEN
            DO izf=1,nzf
               DO iyf=1,nyf
                  R(  1,iyf,izf) = R(    2,iyf,izf)
                  R(nxf,iyf,izf) = R(nxf-1,iyf,izf)
               END DO
            END DO
         END IF
         IF (.NOT.gmmlc2) THEN
            DO izf=1,nzf
               DO ixf=1,nxf
                  R(ixf,  1,izf) = R(ixf,    2,izf)
                  R(ixf,nyf,izf) = R(ixf,nyf-1,izf)
               END DO
            END DO
         END IF
         DO izc=MZ1,MZ2
            izf = (izc-1)*idz+1
            dzm = gdz(izf,levelF)
            dzp = gdz(izf+1,levelF)
            DO iyc=CY1,CY2
               iyf = (iyc-1)*idy+1
               dymi = gdyi(iyf-1,levelF)
               dypi = gdyi(iyf,levelF)
               DO ixc=CX1,CX2
                  ixf = (ixc-1)*idx+1
                  dxmi = gdxi(ixf-1,levelF)
                  dxpi = gdxi(ixf,levelF)
                  AC3(ixc,iyc,izc) = o4
     &                 *(o2 *hrmh(R(ixf,iyf,izf),
     &                 R(ixf,iyf,izf+1),dzm,dzp)
     &                 + o2*(hrmh(R(ixf-1,iyf,izf),
     &                 R(ixf-1,iyf,izf+1),dzm,dzp)*dxmi
     &                 +     hrmh(R(ixf+1,iyf,izf),
     &                 R(ixf+1,iyf,izf+1),dzm,dzp)*dxpi
     &                 +     hrmh(R(ixf,iyf-1,izf),
     &                 R(ixf,iyf-1,izf+1),dzm,dzp)*dymi
     &                 +     hrmh(R(ixf,iyf+1,izf),
     &                 R(ixf,iyf+1,izf+1),dzm,dzp)*dypi)
     &                 /(dxmi+dxpi+dymi+dypi))
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (5*7+15)*(CX2-CX1+1)*(CY2-CY1+1)*(MZ2-MZ1+1)
      gmflops(isub)=gmflops(isub) + 
     & (5*7+15)*(CX2-CX1+1)*(CY2-CY1+1)*(MZ2-MZ1+1)
#endif
      ELSE
         DO izc=MZ1,MZ2
            izf = (izc-1)*idz+1
            DO iyc=CY1,CY2
               iyf = (iyc-1)*idy+1
               DO ixc=CX1,CX2
                  ixf = (ixc-1)*idx+1
                  AC3(ixc,iyc,izc) = R(ixf,  iyf,izf)
               END DO
            END DO
         END DO
      END IF


CCCCC @@ Macro dependent parameters
      IF (gmmlc1) CALL mmGetX1Ghost (AC3,nxdac,nydac,nzdac,bndac,
     &     nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
      IF (gmmlc2) CALL mmGetY1Ghost (AC3,nxdac,nydac,nzdac,bndac,
     &     nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
CCCCC @@ Macro dependent parameters
      IF (gmmlc3) CALL mmSendZ1Ghost (AC3,nxdac,nydac,nzdac,bndac,
     &     nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

#ifdef DEBUG
      s = mmVectorSum (ACD,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
      if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(ACD) = ',s
      s = mmVectorSum (AC1,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
      if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AC1) = ',s
      s = mmVectorSum (AC2,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
      if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AC2) = ',s
      s = mmVectorSum (AC3,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &     isub)
      if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AC3) = ',s
      IF (gisymmetric.EQ.0) THEN
         s = mmVectorSum (AC4,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &        isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AC4) = ',s
         s = mmVectorSum (AC5,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &        isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AC5) = ',s
         s = mmVectorSum (AC6,nxdac,nydac,nzdac,bndac,nxc,nyc,nzc,comm,
     &        isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AC6) = ',s
      END IF 

      IF (gimatfree.EQ.0.OR.levelC.GT.2) THEN

         s = mmVectorSum (AFD,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf,comm,
     &        isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AFD) = ',s
         s = mmVectorSum (AF1,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf,comm,
     &        isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AF1) = ',s
         s = mmVectorSum (AF2,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf,comm,
     &        isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AF2) = ',s
         s = mmVectorSum (AF3,nxdaf,nydaf,nzdaf,bndaf,nxf,nyf,nzf,comm,
     &        isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AF3) = ',s
         IF (gisymmetric.EQ.0) THEN
            s = mmVectorSum (AF4,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,
     &           'mmBuildCoarseSCGen sum(AF4) = ',s
            s = mmVectorSum (AF5,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,
     &           'mmBuildCoarseSCGen sum(AF5) = ',s
            s = mmVectorSum (AF6,nxdaf,nydaf,nzdaf,bndaf,
     &           nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,
     &           'mmBuildCoarseSCGen sum(AF6) = ',s
         END IF
      ELSE
         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,0,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AFD) = ',s

         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,1,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AF1) = ',s

         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,2,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AF2) = ',s

         CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,3,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
         s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
         if (ip.EQ.0) PRINT*, levelC,'mmBuildCoarseSCGen sum(AF3) = ',s

         IF (gisymmetric.EQ.0) THEN
            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,4,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
            s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,
     &           'mmBuildCoarseSCGen sum(AF4) = ',s

            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,5,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
            s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,
     &           'mmBuildCoarseSCGen sum(AF5) = ',s

            CALL mmMatDiag (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,6,nxf,nyf,nzf,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
            s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nxf,nyf,nzf,comm,isub)
            if (ip.EQ.0) PRINT*, levelC,
     &           'mmBuildCoarseSCGen sum(AF6) = ',s
         END IF
      END IF 
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C
C     MODULE:  SUBROUTINE mmBuildFine 
C
C     PURPOSE: Build fine-level matrix for Poisson problem
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBuildFine (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,xp,yp,zp,comm,icall)

      IMPLICIT NONE

      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      REAL xp(*),yp(*),zp(*)
      INTEGER comm

      INTEGER nx,ny,nz
      REAL mmVectorSum,s

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz
      REAL hxmi,hymi,hzmi,hxpi,hypi,hzpi
      REAL xm2,ym2,zm2,xp2,yp2,zp2
      REAL dxs,dys,dzs
      REAL x,x2,y,y2,z,z2

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_BUILD + 1
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      IF (gimatfree.EQ.0) THEN
         CALL mmMatrixClear (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,nx,ny,nz)
      END IF


*-----------------------------------------------------------------------
*     Compute the fine-grid matrix entries and right-hand side
*-----------------------------------------------------------------------

      IF (gimatfree.EQ.0) THEN

*---------------------------------------------
*     Upper X-diagonal matrix entries ...
*---------------------------------------------

         DO iz=2,nz-1
            dzs = gdz(iz-1,1)+gdz(iz,1)
            z=zp(iz)
            z2 = o2*(zp(iz)+zp(iz+1))
            DO iy=2,ny-1
               dys=gdy(iy-1,1)+gdy(iy,1)
               y=yp(iy)
               y2 = o2*(yp(iy)+yp(iy+1))
               DO ix=1,nx-1
                  hxpi=o1/(xp(ix+1)-xp(ix))
                  x=xp(ix)
                  x2 = o2*(xp(ix)+xp(ix+1))
                  A1(ix,iy,iz)=o4*dys*dzs*hxpi
     &                 *gg2off(ix,1)*gg31off(ix,1)
               END DO
            END DO
         END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 8*(nx-1)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 8*(nx-1)*(ny-2)*(nz-2)
#endif

*---------------------------------------------
*     Upper Y-diagonal matrix entries ...
*---------------------------------------------

         DO iz=2,nz-1
            dzs = gdz(iz-1,1)+gdz(iz,1)
            z=zp(iz)
            z2 = o2*(zp(iz)+zp(iz+1))
            DO iy=1,ny-1
               hypi=o1/(yp(iy+1)-yp(iy))
               y=yp(iy)
               y2 = o2*(yp(iy)+yp(iy+1))
               DO ix=2,nx-1
                  dxs=gdx(ix-1,1)+gdx(ix,1)
                  x=xp(ix)
                  x2 = o2*(xp(ix)+xp(ix+1))
                  A2(ix,iy,iz)=o4*dzs*dxs*hypi
     &                 * gg32off(iy,1)/gg2on(ix,1)**2
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 10*(nx-2)*(ny-1)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 10*(nx-2)*(ny-1)*(nz-2)
#endif

*---------------------------------------------
*     Upper Z-diagonal matrix entries ...
*---------------------------------------------

         DO iz=1,nz-1
            hzpi=o1/(zp(iz+1)-zp(iz))
            z=zp(iz)
            z2 = o2*(zp(iz)+zp(iz+1))
            DO iy=2,ny-1
               dys=gdy(iy-1,1)+gdy(iy,1)
               y=yp(iy)
               y2 = o2*(yp(iy)+yp(iy+1))
               DO ix=2,nx-1
                  dxs=gdx(ix-1,1)+gdx(ix,1)
                  x=xp(ix)
                  x2 = o2*(xp(ix)+xp(ix+1))
                  A3(ix,iy,iz)=o4*dxs*dys*hzpi/
     &                 (gg31on(ix,1)**2*gg32on(iy,1)**2) 
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 11*(nx-2)*(ny-2)*(nz-1)
      gmflops(isub)=gmflops(isub) + 
     & 11*(nx-2)*(ny-2)*(nz-1)
#endif

         IF (gisymmetric.EQ.0) THEN

*---------------------------------------------
*     Lower X-diagonal matrix entries ...
*---------------------------------------------

            DO iz=2,nz-1
               dzs = gdz(iz-1,1)+gdz(iz,1)
               z=zp(iz)
               z2 = o2*(zp(iz)+zp(iz+1))
               DO iy=2,ny-1
                  dys=gdy(iy-1,1)+gdy(iy,1)
                  y=yp(iy)
                  y2 = o2*(yp(iy)+yp(iy+1))
                  DO ix=1,nx-1
                     hxpi=o1/(xp(ix+1)-xp(ix))
                     x=xp(ix)
                     x2 = o2*(xp(ix)+xp(ix+1))
                     A4(ix,iy,iz)=o4*dys*dzs*hxpi
     &                    * gg2off(ix,1)*gg31off(ix,1)
                  END DO
               END DO
            END DO

            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 8*(nx-1)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 8*(nx-1)*(ny-2)*(nz-2)
#endif

*---------------------------------------------
*     Lower Y-diagonal matrix entries ...
*---------------------------------------------

            DO iz=2,nz-1
               dzs = gdz(iz-1,1)+gdz(iz,1)
               z=zp(iz)
               z2 = o2*(zp(iz)+zp(iz+1))
               DO iy=1,ny-1
                  hypi=o1/(yp(iy+1)-yp(iy))
                  y=yp(iy)
                  y2 = o2*(yp(iy)+yp(iy+1))
                  DO ix=2,nx-1
                     dxs=gdx(ix-1,1)+gdx(ix,1)
                     x=xp(ix)
                     x2 = o2*(xp(ix)+xp(ix+1))
                     A5(ix,iy,iz)=o4*dzs*dxs*hypi
     &                    * gg32off(iy,1)/gg2on(ix,1)**2
                  END DO
               END DO
            END DO

            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 10*(nx-2)*(ny-1)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 10*(nx-2)*(ny-1)*(nz-2)
#endif

*---------------------------------------------
*     Lower Z-diagonal matrix entries ...
*---------------------------------------------

            DO iz=1,nz-1
               hzpi=o1/(zp(iz+1)-zp(iz))
               z=zp(iz)
               z2 = o2*(zp(iz)+zp(iz+1))
               DO iy=2,ny-1
                  dys=gdy(iy-1,1)+gdy(iy,1)
                  y=yp(iy)
                  y2 = o2*(yp(iy)+yp(iy+1))
                  DO ix=2,nx-1
                     dxs=gdx(ix-1,1)+gdx(ix,1)
                     x=xp(ix)
                     x2 = o2*(xp(ix)+xp(ix+1))
                     A6(ix,iy,iz)=o4*dxs*dys*hzpi/
     &                    (gg31on(ix,1)**2*gg32on(iy,1)**2) 
                  END DO
               END DO
            END DO

            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 11*(nx-2)*(ny-2)*(nz-1)
      gmflops(isub)=gmflops(isub) + 
     & 11*(nx-2)*(ny-2)*(nz-1)
#endif

         END IF ! gisymmetric .EQ. 0

*---------------------------------------------
*     Main diagonal matrix entries ...
*---------------------------------------------

         DO iz=2,nz-1
            dzs = gdz(iz-1,1)+gdz(iz,1)
            hzmi=o1/(zp(iz)-zp(iz-1))
            hzpi=o1/(zp(iz+1)-zp(iz))
            z=zp(iz)
            zm2 = o2*(zp(iz-1)+zp(iz))
            zp2 = o2*(zp(iz)+zp(iz+1))
            DO iy=2,ny-1
               dys=gdy(iy-1,1)+gdy(iy,1)
               hymi=o1/(yp(iy)-yp(iy-1))
               hypi=o1/(yp(iy+1)-yp(iy))
               y=yp(iy)
               ym2 = o2*(yp(iy-1)+yp(iy))
               yp2 = o2*(yp(iy)+yp(iy+1))
               DO ix=2,nx-1
                  dxs=gdx(ix-1,1)+gdx(ix,1)
                  hxmi=o1/(xp(ix)-xp(ix-1))
                  hxpi=o1/(xp(ix+1)-xp(ix))
                  x=xp(ix)
                  xm2 = o2*(xp(ix-1)+xp(ix))
                  xp2 = o2*(xp(ix)+xp(ix+1))
                  AD(ix,iy,iz) = -o4*
     &                 (dys*dzs*(gg2off(ix,1)*gg31off(ix,1)*hxpi
     &                 +gg2off(ix-1,1)*gg31off(ix-1,1)*hxmi)
     &                 +dzs*dxs*((gg32off(iy,1)/gg2on(ix,1)**2)*hypi
     &                 +(gg32off(iy-1,1)/gg2on(ix,1)**2)*hymi)
     &                 +dxs*dys*(o1/
     &                   ((gg31on(ix,1)**2*gg32on(iy,1)**2)))*
     &                 (hzpi + hzmi))
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 36*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 36*(nx-2)*(ny-2)*(nz-2)
#endif

      END IF ! gimatfree .EQ. 0

*------------------------------------
C     Right-hand side
*------------------------------------

      DO iz=2,nz-1
         dzs = gdz(iz-1,1)+gdz(iz,1)
         DO iy=2,ny-1
            dys = gdy(iy-1,1)+gdy(iy,1)
            DO ix=2,nx-1
               dxs = gdx(ix-1,1)+gdx(ix,1)
               B(ix,iy,iz) = o8*dxs*dys*dzs*B(ix,iy,iz)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 5*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 5*(nx-2)*(ny-2)*(nz-2)
#endif

#ifdef DEBUG
      if (ip.EQ.0) print*, 'gimatfree = ',gimatfree
      IF (gimatfree.EQ.0) THEN
         s = mmVectorSum (AD,nxda,nyda,nzda,bnda,nx,ny,nz,comm,isub)
         if (ip.EQ.0) PRINT*, 'mmBuildFine sum(AD) = ',s
         s = mmVectorSum (A1,nxda,nyda,nzda,bnda,nx,ny,nz,comm,isub)
         if (ip.EQ.0) PRINT*, 'mmBuildFine sum(A1) = ',s
         s = mmVectorSum (A2,nxda,nyda,nzda,bnda,nx,ny,nz,comm,isub)
         if (ip.EQ.0) PRINT*, 'mmBuildFine sum(A2) = ',s
         s = mmVectorSum (A3,nxda,nyda,nzda,bnda,nx,ny,nz,comm,isub)
         if (ip.EQ.0) PRINT*, 'mmBuildFine sum(A3) = ',s
         IF (gisymmetric.EQ.0) THEN
            s = mmVectorSum (A4,nxda,nyda,nzda,bnda,nx,ny,nz,comm,isub)
            if (ip.EQ.0) PRINT*, 'mmBuildFine sum(A4) = ',s
            s = mmVectorSum (A5,nxda,nyda,nzda,bnda,nx,ny,nz,comm,isub)
            if (ip.EQ.0) PRINT*, 'mmBuildFine sum(A5) = ',s
            s = mmVectorSum (A6,nxda,nyda,nzda,bnda,nx,ny,nz,comm,isub)
            if (ip.EQ.0) PRINT*, 'mmBuildFine sum(A6) = ',s
         ENDIF
      ENDIF
      s = mmVectorSum (
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, 'mmBuildFine sum(B) = ',s
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmCg 
C     
C     Purpose: 3D Conjugate gradient method 
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmCg (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     P,nxdp,nydp,nzdp,bndp
     &     ,
     &     W,nxdw,nydw,nzdw,bndw
     &     ,
     &     nx,ny,nz,nxg,nyg,nzg,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,ibc,
     &     numIterCg,maxIterCg,rtol,outputLevel,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,level,rnorm,resRedFine,istat,icall)

      IMPLICIT NONE

      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxdp,nydp,nzdp
      INTEGER bndp(3)	
      REAL P(nxdp,nydp,nzdp)
      
      INTEGER nxdw,nydw,nzdw
      INTEGER bndw(3)	
      REAL W(nxdw,nydw,nzdw)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz,level,ibc
      INTEGER nxg,nyg,nzg
      INTEGER numIterCg,maxIterCg,outputLevel
      REAL rtol
      LOGICAL mmIsConverged,isConverged
      REAL scaleA, scaleB
      INTEGER istat
      REAL resRedFine,s
      INTEGER mmIsNan
      EXTERNAL mmIsNan      

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

#ifdef DEBUG_CG
      REAL mmVectorSum,s
#endif

      INTEGER ix,iy,iz,iter,ierr
      REAL alpha,beta,mmDotProd,mmVectorMax
      REAL anorm,bnorm,rnorm,rnormFirst,rdotFirst,rdot,rdotPrev
      REAL rtol2

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_CG+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

C      CALL mmVectorClear (__X,nx,ny,nz)

      istat = 0

*-----------------------------------------------------------------------
*     Fix for singular problems with null(A) = ones(n,1)
*-----------------------------------------------------------------------

      CALL mmShiftRhs (
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,nxg,nyg,nzg,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,comm,isub)

*-----------------------------------------------------------------------
*     Initialize Dirichlet b.c.'s
*-----------------------------------------------------------------------

      IF (ibc.EQ.1) THEN
         CALL mmDirichletBC (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,scaleA,scaleB,level,.FALSE.)
      ELSE
         CALL mmDirichletBC (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,scaleA,scaleB,level,.TRUE.)
      END IF

*-----------------------------------------------------------------------
*     Compute residual R = B - A*X
*-----------------------------------------------------------------------

#ifdef DEBUG_CG
      s = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, level,'mmCg sum(X) = ',s
      s = mmVectorSum (
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, level,'mmCg sum(B) = ',s
#endif

      CALL mmClearBoundary (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz)

      CALL mmResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,0,level,isub)

#ifdef DEBUG_CG

CCCCC @@ Macro dependent parameters
      s = mmVectorSum (AD,nxda,nyda,nzda,bnda,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, level,'mmCg sum(AD) = ',s
CCCCC @@ Macro dependent parameters
      s = mmVectorSum (A1,nxda,nyda,nzda,bnda,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, level,'mmCg sum(A1) = ',s
CCCCC @@ Macro dependent parameters
      s = mmVectorSum (A2,nxda,nyda,nzda,bnda,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, level,'mmCg sum(A2) = ',s
CCCCC @@ Macro dependent parameters
      s = mmVectorSum (A3,nxda,nyda,nzda,bnda,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, level,'mmCg sum(A3) = ',s
CCCCC @@ Macro dependent parameters
      s = mmVectorSum (A4,nxda,nyda,nzda,bnda,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, level,'mmCg sum(A4) = ',s
CCCCC @@ Macro dependent parameters
      s = mmVectorSum (A5,nxda,nyda,nzda,bnda,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, level,'mmCg sum(A5) = ',s
CCCCC @@ Macro dependent parameters
      s = mmVectorSum (A6,nxda,nyda,nzda,bnda,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, level,'mmCg sum(A6) = ',s
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, level,'mmCg sum(R) = ',s

#endif

*-----------------------------------------------------------------------
*     Compute norms for convergence test
*-----------------------------------------------------------------------

      IF (gimatfree.EQ.0) THEN
CCCCC @@ Macro dependent parameters
         anorm = mmVectorMax (AD,nxda,nyda,nzda,bnda,nx,ny,nz,comm)
      ELSE
         CALL mmMatDiag (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,0,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
CCCCC @@ Macro dependent parameters
         anorm = mmVectorMax (P,nxdp,nydp,nzdp,bndp,nx,ny,nz,comm)
      END IF

      rdot = mmDotProd (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,level,isub)
      rdotFirst = rdot
      rnorm = DSQRT(rdot)
      rnormFirst = rnorm
      bnorm = DSQRT(mmDotProd (
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,comm,level,isub))

      DO iz=1,nz
         DO iy=1,ny
            DO ix=1,nx
               P(ix,iy,iz) = R(ix,iy,iz)
            END DO
         END DO
      END DO


#ifdef DEBUG_CG
      IF (ip.EQ.0) print*, 'mmCg: a,b,r = ',anorm,bnorm,rnorm
#endif

      iter = 0

*-----------------------------------------------------------------------
*     Check for convergence
*-----------------------------------------------------------------------

      rtol2 = rtol*rtol

 10   CONTINUE

*-----------------------------------------------------------------------
*     Fine-level convergence test
*-----------------------------------------------------------------------

      IF (level.EQ.1) THEN

         isConverged = .FALSE.

         isConverged = mmIsConverged(istat,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,
     &        anorm,bnorm,rnorm,rnormFirst,rtol,iter,maxIterCg,
     &        
     &     comm,nbr,bnd
     &     ,1,isub)

         IF (isConverged) GO TO 20

      ELSE

         IF (iter.GE.maxIterCg .OR. rdot/rdotFirst.LT.rtol2) GOTO 20 

      END IF


#ifdef DEBUG_CG
      IF (ip.EQ.0) print*, 'mmCg calling mmMatvec'
#endif

      CALL mmRefreshBC (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,level,isub)
      CALL mmMatvec (
     &     W,nxdw,nydw,nzdw,bndw
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,level,isub)

#ifdef DEBUG_CG
      IF (ip.EQ.0) print*, 'mmCg exiting from mmMatvec'
#endif

#ifdef DEBUG_CG
      s = mmVectorSum (
     &     W,nxdw,nydw,nzdw,bndw
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, level,'CG: sum(W) = ',s
      s = mmVectorSum (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, level,'CG: sum(P) = ',s
#endif

      alpha = rdot / mmDotProd (
     &     W,nxdw,nydw,nzdw,bndw
     &     ,
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,comm,level,isub)

      
      rdotPrev = rdot
      s = 0.0
      DO iz=2,nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               X(ix,iy,iz) = alpha * P(ix,iy,iz) + X(ix,iy,iz)
               R(ix,iy,iz) = -alpha * W(ix,iy,iz) + R(ix,iy,iz)
               s = s + R(ix,iy,iz)*R(ix,iy,iz)
            END DO
         END DO
      END DO

      
#ifdef MPI_USED
      CALL MPI_Allreduce(s,rdot,1,MM_REAL,MPI_SUM,comm,ierr)
#else
      rdot = s
#endif

      rnorm=DSQRT(rdot)
      beta = rdot/rdotPrev
      
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 6*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 6*(nx-2)*(ny-2)*(nz-2)
#endif

#ifdef DEBUG_CG
      s = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, level,'CG: sum(X) = ',s
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, level,'CG: sum(R) = ',s
#endif


      DO iz=2,nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               P(ix,iy,iz) = beta * P(ix,iy,iz) + R(ix,iy,iz)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif

#ifdef DEBUG_CG
      s = mmVectorSum (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, level,'CG: sum(P) = ',s
#endif

      iter = iter + 1
      numIterCg = numIterCg + 1

      IF (outputLevel.GT.1 .AND. ip.EQ.0)
     &     WRITE(*,1000) iter, MPI_WTIME() - gtimestart(SB_MG3),
     &     DSQRT(rdot/rdotFirst)

      GO TO 10

*-----------------------------------------------------------------------
*     Exit subroutine, printing warning message if applicable
*-----------------------------------------------------------------------

 20   CONTINUE

      IF (ip.EQ.0.AND.outputLevel.GT.0) THEN
         IF (istat.EQ.1) print*,
     &        '== mgmpi ERROR == aborting CG: apparently diverging'
         IF (istat.EQ.2) print*,
     &        '== mgmpi ERROR == aborting CG: residual norm is NaN'
         IF (istat.EQ.3) print*,
     &        '-- mgmpi warning -- exiting CG: iteration limit reached'
      END IF

      IF (outputLevel.EQ.1 .AND. ip.EQ.0)
     &     print*, 'CG:',iter, DSQRT(rdot/rdotFirst)


      resRedFine=DSQRT(rdot/rdotFirst)

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN

 1000 FORMAT (1x,i4,f8.2,e14.3)

      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmCgMg
C     
C     Purpose: Conjugate gradient method preconditioned
C     Purpose: with MG V-cycle
C
C     Algorithm: 
C
C     Inputs: 
C
C     Outputs: 
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate: 
C
C     Communication estimate: 
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C               Sep 9 -- Changed i?1(k) to gmmi?(k,ip)
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmCgMg (
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     ACD,AC1,AC2,AC3,AC4,AC5,AC6,nxdac,nydac,nzdac,bndac
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     BC,nxdbc,nydbc,nzdbc,bndbc
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     P,nxdp,nydp,nzdp,bndp
     &     ,
     &     W,nxdw,nydw,nzdw,bndw
     &     ,
     &     Z,nxdz,nydz,nzdz,bndz
     &     ,
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &     indh,inda,indx
     &     ,
     &     m,nx,ny,nz,
     &     nxg,nyg,nzg,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver,
     &     wgt,numIterCg,iterCoarse,maxIterCg,maxItCoarse,
     &     resTolCgMg,resTolCoarse,outputLevelCg,
     &     comm,nbr,bnd,bnd2
     &     ,scaleA,scaleB,rnorm,resRedFine,istat,icall)
      
      IMPLICIT NONE
      
      
      INTEGER nxdaf,nydaf,nzdaf
      INTEGER bndaf(3)  
      REAL AFD(nxdaf,nydaf,nzdaf)
      REAL AF1(nxdaf,nydaf,nzdaf)
      REAL AF2(nxdaf,nydaf,nzdaf)
      REAL AF3(nxdaf,nydaf,nzdaf)
      REAL AF4(nxdaf,nydaf,nzdaf)
      REAL AF5(nxdaf,nydaf,nzdaf)
      REAL AF6(nxdaf,nydaf,nzdaf)
      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      
      INTEGER nxdbf,nydbf,nzdbf
      INTEGER bndbf(3)	
      REAL BF(nxdbf,nydbf,nzdbf)
      
      INTEGER nxdac(*),nydac(*),nzdac(*)
      INTEGER bndac(3,*) 
      REAL ACD(*)
      REAL AC1(*)
      REAL AC2(*)
      REAL AC3(*)
      REAL AC4(*)
      REAL AC5(*)
      REAL AC6(*)
      
      INTEGER nxdxc(*),nydxc(*),nzdxc(*)
      INTEGER bndxc(3,*)  
      REAL XC(*)
      
      INTEGER nxdbc(*),nydbc(*),nzdbc(*)
      INTEGER bndbc(3,*)
      REAL BC(*)
      
      INTEGER nxdy(*),nydy(*),nzdy(*)
      INTEGER bndy(3,*)
      REAL Y(*)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxdp,nydp,nzdp
      INTEGER bndp(3)	
      REAL P(nxdp,nydp,nzdp)
      
      INTEGER nxdw,nydw,nzdw
      INTEGER bndw(3)	
      REAL W(nxdw,nydw,nzdw)
      
      INTEGER nxdz,nydz,nzdz
      INTEGER bndz(3)
      REAL Z(nxdz,nydz,nzdz)
      
      INTEGER nxdrcg,nydrcg,nzdrcg
      INTEGER bndrcg(3)
      REAL RCG(nxdrcg,nydrcg,nzdrcg)
      
      INTEGER nxdpcg,nydpcg,nzdpcg
      INTEGER bndpcg(3)
      REAL PCG(nxdpcg,nydpcg,nzdpcg)
      
      INTEGER nxdwcg,nydwcg,nzdwcg
      INTEGER bndwcg(3)
      REAL WCG(nxdwcg,nydwcg,nzdwcg)
      
      INTEGER nxdrsm,nydrsm,nzdrsm
      INTEGER bndrsm(3)
      REAL RSM(nxdrsm,nydrsm,nzdrsm)
      
      INTEGER nxdpsm,nydpsm,nzdpsm
      INTEGER bndpsm(3)
      REAL PSM(nxdpsm,nydpsm,nzdpsm)
      
      INTEGER nxdwsm,nydwsm,nzdwsm
      INTEGER bndzsm(3)
      REAL WSM(nxdwsm,nydwsm,nzdwsm)

      INTEGER indh(*),inda(*),indx(*)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3,*),bnd2(3)

      INTEGER m
      INTEGER nx(m),ny(m),nz(m)
      INTEGER nxg(m),nyg(m),nzg(m)
      INTEGER numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver
      REAL resTolFine,resTolCoarse,wgt
      INTEGER iterFine,iterCoarse,outputLevelMg
      INTEGER maxItFine,maxItCoarse
      REAL scaleA(*), scaleB(*),rnorm,rnormMg,rtz,rtzPrev
      REAL resRedFine,resRedFineMg
      INTEGER istat,istatMg
      INTEGER numIterCg,maxIterCg,outputLevelCg
      REAL resTolCgMg

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

#ifdef DEBUG_CG
      REAL s,mmVectorSum,tmp
#endif

      INTEGER ix,iy,iz,iter
      REAL alpha,beta,mmVectorMax
      REAL anorm,bnorm,rdotFirst,rdot,rtol2
      REAL rmin,mmDotProd,rnormFirst
      LOGICAL mmIsConverged,isConverged,preconOn

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_CGMG
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      istat = 0
C-----------------------------------------------------------------------
C     Preconditioner parameters
C-----------------------------------------------------------------------

      preconOn = .TRUE.
      iterFine = 0
      maxItFine = 1
      resTolFine = 0.0D0
      outputLevelMg = 0
      rnormMg = 0.0D0
      resRedFineMg = 0.0D0
      istatMg = 0

      rtol2 = resTolCgMg*resTolCgMg

*-----------------------------------------------------------------------
*     Fix for singular problems with null(A) = ones(n,1)
*-----------------------------------------------------------------------

      CALL mmShiftRhs (
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,nxg,nyg,nzg,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,comm,isub)

*-----------------------------------------------------------------------
*     Initialize Dirichlet b.c.'s
*-----------------------------------------------------------------------

      CALL mmDirichletBC (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,scaleA,scaleB,1,.FALSE.)

*-----------------------------------------------------------------------
*     Compute residual R = B - A*X
*-----------------------------------------------------------------------

#ifdef DEBUG_CG
      s = mmVectorSum (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, 'mmCgMg sum(X) = ',s
      s = mmVectorSum (
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, 'mmCgMg sum(B) = ',s
#endif

      CALL mmClearBoundary (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz)

      CALL mmResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,0,1,isub)

#ifdef DEBUG_CG
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, 'mmCgMg sum(R) = ',s
#endif

      rdot = mmDotProd (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,1,isub)
      rnorm = DSQRT(rdot)

*-----------------------------------------------------------------------
*     Compute norms for convergence test
*-----------------------------------------------------------------------

      IF (gimatfree.EQ.0) THEN
CCCCC @@ Macro dependent parameters
         anorm = mmVectorMax (AFD,nxdaf,nydaf,nzdaf,bndaf,nx,ny,nz,comm)
      ELSE
         CALL mmMatDiag (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,0,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
CCCCC @@ Macro dependent parameters
         anorm = mmVectorMax (P,nxdp,nydp,nzdp,bndp,nx,ny,nz,comm)
      END IF

      bnorm = mmDotProd (
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,comm,1,isub)
      bnorm = DSQRT(bnorm)

C-----------------------------------------------------------------------
C     if (preconOn) then SOLVE A PT = P; 
C     (otherwise PT === P and nothing is done)
C-----------------------------------------------------------------------

      IF (preconOn) THEN
         DO iz=1,nz(1)
            DO iy=1,ny(1)
               DO ix=1,nx(1)
                  Z(ix,iy,iz) = 0.0D0
               END DO
            END DO
         END DO
#ifdef DEBUG_CG
         IF (ip.EQ.0) PRINT*, 'mmCgMg MgPrecon 1'
#endif

C@@@@ Clear boundary of P???

#ifdef DEBUG_CG
         tmp = mmVectorSum (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,comm,isub)
         IF (ip.EQ.0) PRINT*, '  sum(P) = ',tmp
#endif
         CALL mmMgPrecon (
     &        
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     Z,nxdz,nydz,nzdz,bndz
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     ACD,AC1,AC2,AC3,AC4,AC5,AC6,nxdac,nydac,nzdac,bndac
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     BC,nxdbc,nydbc,nzdbc,bndbc
     &     ,
     &     W,nxdw,nydw,nzdw,bndw
     &     ,
     &        
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &        
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &        
     &     indh,inda,indx
     &     ,
     &        m,nx,ny,nz,
     &        nxg,nyg,nzg,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver,
     &        wgt,iterFine,iterCoarse,
     &        maxItFine,maxItCoarse,
     &        resTolFine,resTolCoarse,outputLevelMg,
     &        
     &     comm,nbr,bnd,bnd2
     &     ,
     &        scaleA,scaleB,rnormMg,resRedFineMg,istatMg,isub)
#ifdef DEBUG_CG
         IF (ip.EQ.0) PRINT*, 'mmCgMg MgPrecon 1 status:',istatMg
#endif
      ELSE
         DO iz=1,nz(1)
            DO iy=1,ny(1)
               DO ix=1,nx(1)
                  Z(ix,iy,iz) = R(ix,iy,iz)
               END DO
            END DO
         END DO
      END IF

#ifdef DEBUG_CG
      s = mmVectorSum (
     &     Z,nxdz,nydz,nzdz,bndz
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, 'mmCgMg sum(Z) = ',s
#endif

      rtz = mmDotProd (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     Z,nxdz,nydz,nzdz,bndz
     &     ,nx,ny,nz,comm,1,isub)
      
#ifdef DEBUG_CG
      IF (ip.EQ.0) PRINT*, 'mmCgMg Matvec 1'
#endif

      DO iz=1,nz(1)
         DO iy=1,ny(1)
            DO ix=1,nx(1)
               P(ix,iy,iz) = Z(ix,iy,iz)
            END DO
         END DO
      END DO


#ifdef DEBUG_CG
      IF (ip.EQ.0) print*, 'mmCg: a,b,r = ',anorm,bnorm,rnorm
#endif
      rdotFirst = rdot
      rnormFirst = DSQRT(rdot)

      rmin = rdot

      iter = 0

*-----------------------------------------------------------------------
*     Check for convergence
*-----------------------------------------------------------------------

 10   CONTINUE

*-----------------------------------------------------------------------
*     Fine-level convergence test
*-----------------------------------------------------------------------

      isConverged = .FALSE.

      isConverged = mmIsConverged(istat,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,
     &     anorm,bnorm,rnorm,rnormFirst,resTolCgMg,iter,maxIterCg,
     &     comm,nbr,bnd
     &     ,1,isub)

      if (isConverged) GO TO 20


      CALL mmRefreshBC (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,1,isub)

#ifdef DEBUG_CG
      IF (ip.EQ.0) print*, 'mmCgMg calling mmMatvec'
#endif

      CALL mmMatvec (
     &     W,nxdw,nydw,nzdw,bndw
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,1,isub)

#ifdef DEBUG_CG
      IF (ip.EQ.0) print*, 'mmCgMg exiting from mmMatvec'
#endif

#ifdef DEBUG_CG
      s = mmVectorSum (
     &     W,nxdw,nydw,nzdw,bndw
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, 'CG: sum(W) = ',s
CCCCC @@ Macro dependent parameters
c$$$      CALL mmWriteVector ('P         ',P,
c$$$     &     nxdp,nydp,nzdp,nx,ny,nz,
c$$$     &     gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
c$$$     &     comm,nbr)
c$$$      CALL mmTerminate(comm)
      s = mmVectorSum (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, 'CG: sum(P) = ',s
#endif

      alpha = rtz / mmDotProd (
     &     W,nxdw,nydw,nzdw,bndw
     &     ,
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,comm,1,isub)


      DO iz=2,nz(1)-1
         DO iy=2,ny(1)-1
            DO ix=2,nx(1)-1
               XF(ix,iy,iz) = alpha * P(ix,iy,iz) + XF(ix,iy,iz)
               R(ix,iy,iz) = -alpha * W(ix,iy,iz) + R(ix,iy,iz)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 4*(nx(1)-2)*(ny(1)-2)*(nz(1)-2)
      gmflops(isub)=gmflops(isub) + 
     & 4*(nx(1)-2)*(ny(1)-2)*(nz(1)-2)
#endif

#ifdef DEBUG_CG
      s = mmVectorSum (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, 'CG: sum(X) = ',s
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, 'CG: sum(R) = ',s
#endif

C-----------------------------------------------------------------------
C     if (preconOn) then SOLVE A PT = P; 
C     (otherwise PT === P and nothing is done)
C-----------------------------------------------------------------------

      IF (preconOn) THEN
         DO iz=1,nz(1)
            DO iy=1,ny(1)
               DO ix=1,nx(1)
                  Z(ix,iy,iz) = 0.0D0
               END DO
            END DO
         END DO
#ifdef DEBUG_CG
         IF (ip.EQ.0) PRINT*, 'mmCgMg MgPrecon 1'
#endif

C@@@@ Clear boundary of P???

#ifdef DEBUG_CG
         tmp = mmVectorSum (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,comm,isub)
         IF (ip.EQ.0) PRINT*, '  sum(P) = ',tmp
#endif
         CALL mmMgPrecon (
     &        
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     Z,nxdz,nydz,nzdz,bndz
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     ACD,AC1,AC2,AC3,AC4,AC5,AC6,nxdac,nydac,nzdac,bndac
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     BC,nxdbc,nydbc,nzdbc,bndbc
     &     ,
     &     W,nxdw,nydw,nzdw,bndw
     &     ,
     &        
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &        
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &        
     &     indh,inda,indx
     &     ,
     &        m,nx,ny,nz,
     &        nxg,nyg,nzg,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver,
     &        wgt,iterFine,iterCoarse,
     &        maxItFine,maxItCoarse,
     &        resTolFine,resTolCoarse,outputLevelMg,
     &        
     &     comm,nbr,bnd,bnd2
     &     ,
     &        scaleA,scaleB,rnormMg,resRedFineMg,istatMg,isub)
#ifdef DEBUG_CG
         IF (ip.EQ.0) PRINT*, 'mmCgMg MgPrecon 1 status:',istatMg
#endif
      ELSE
         DO iz=1,nz(1)
            DO iy=1,ny(1)
               DO ix=1,nx(1)
                  Z(ix,iy,iz) = R(ix,iy,iz)
               END DO
            END DO
         END DO
      END IF

      rtzPrev = rtz
      rtz = mmDotProd (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     Z,nxdz,nydz,nzdz,bndz
     &     ,nx,ny,nz,comm,1,isub)

      rdot = mmDotProd (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,1,isub)
      rnorm=DSQRT(rdot)
      beta = rtz/rtzPrev

      DO iz=2,nz(1)-1
         DO iy=2,ny(1)-1
            DO ix=2,nx(1)-1
               P(ix,iy,iz) = Z(ix,iy,iz) + beta * P(ix,iy,iz)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx(1)-2)*(ny(1)-2)*(nz(1)-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx(1)-2)*(ny(1)-2)*(nz(1)-2)
#endif

#ifdef DEBUG_CG
      s = mmVectorSum (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, 'CG: sum(P) = ',s
#endif

      iter = iter + 1
      numIterCg = numIterCg + 1

      IF (outputLevelCg.GT.1 .AND. ip.EQ.0)
     &     WRITE(*,1000) iter, MPI_WTIME() - gtimestart(SB_MG3),
     &     DSQRT(rdot/rdotFirst)

      GO TO 10

*-----------------------------------------------------------------------
*     Exit subroutine, printing warning message if applicable
*-----------------------------------------------------------------------

 20   CONTINUE

      IF (ip.EQ.0.AND.outputLevelCg.GT.0) THEN
         IF (istat.EQ.1) print*,
     &        '== mgmpi ERROR == aborting CG: apparently diverging'
         IF (istat.EQ.2) print*,
     &        '== mgmpi ERROR == aborting CG: residual norm is NaN'
         IF (istat.EQ.3) print*,
     &        '-- mgmpi warning -- exiting CG: iteration limit reached'
      END IF

      IF (outputLevelCg.EQ.1 .AND. ip.EQ.0)
     &     print*, 'CG:',iter, DSQRT(rdot/rdotFirst)


      resRedFine=DSQRT(rdot/rdotFirst)

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

 1000 FORMAT (1x,i4,f8.2,e14.3)

      RETURN
      END
C=======================================================================
C     
C     Module:  mmBiCgStab
C     
C     Purpose: Solve nonsymmetric sparse linear problems
C
C     Algorithm: BiCG-STAB nonsymmetric Krylov subspace solver
C
C     Inputs:  A,X,B: Linear system and initial solution estimate
C     Inputs:  R,RT,P,S,V,T 
C
C     Outputs: X:     resulting solution estimate
C     Outputs: istat: solver status  
C
C     Globals accessed: gimatfree, gScaleA, gtimestart
C
C     Flop count estimate:     
C
C     Communication estimate:  
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBiCgStab (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     RT,nxdrt,nydrt,nzdrt,bndrt
     &     ,
     &     P,nxdp,nydp,nzdp,bndp
     &     ,
     &     S,nxds,nyds,nzds,bnds
     &     ,
     &     V,nxdv,nydv,nzdv,bndv
     &     ,
     &     T,nxdt,nydt,nzdt,bndt
     &     ,
     &     nx,ny,nz,nxg,nyg,nzg,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     numIterCg,maxIterCg,rtol,outputLevel,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,level,rnorm,resRedFine,istat,icall)

      IMPLICIT NONE

      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxdrt,nydrt,nzdrt
      INTEGER bndrt(3)	
      REAL RT(nxdrt,nydrt,nzdrt)
      
      INTEGER nxdp,nydp,nzdp
      INTEGER bndp(3)	
      REAL P(nxdp,nydp,nzdp)
      
      INTEGER nxds,nyds,nzds
      INTEGER bnds(3)	
      REAL S(nxds,nyds,nzds)
      
      INTEGER nxdv,nydv,nzdv
      INTEGER bndv(3)	
      REAL V(nxdv,nydv,nzdv)
      
      INTEGER nxdt,nydt,nzdt
      INTEGER bndt(3)	
      REAL T(nxdt,nydt,nzdt)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz,level
      INTEGER nxg,nyg,nzg
      INTEGER numIterCg,maxIterCg,outputLevel
      REAL rtol
      REAL scaleA, scaleB
      REAL resRedFine
      INTEGER istat
      INTEGER mmIsNan
      EXTERNAL mmIsNan      

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

#ifdef DEBUG_BICG
      REAL tmp,mmVectorSum
#endif

      INTEGER ix,iy,iz,iter
      REAL alpha,beta,mmVectorMax
      REAL anorm,bnorm,rnorm,rdotFirst,rdot,rdotPrev,rtol2
      REAL rtr,rtr2,omega,mmDotProd,rnormFirst
      LOGICAL mmIsConverged,isConverged

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_CG+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      istat = 0

*-----------------------------------------------------------------------
*     Fix for singular problems with null(A) = ones(n,1)
*-----------------------------------------------------------------------

      CALL mmShiftRhs (
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,nxg,nyg,nzg,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,comm,icall)

*-----------------------------------------------------------------------
*     Initialize Dirichlet b.c.'s
*-----------------------------------------------------------------------

#ifdef DEBUG_BICG
      if (ip.EQ.0) PRINT*, 'mmBiCgStab DirichletBC'
#endif 

      IF (level.EQ.1) THEN
         CALL mmDirichletBC (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,scaleA,scaleB,level,.FALSE.)
      ELSE
         CALL mmDirichletBC (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,scaleA,scaleB,level,.TRUE.)
      END IF

      CALL mmResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,0,1,isub)

*-----------------------------------------------------------------------
*     Compute norms for convergence test
*-----------------------------------------------------------------------

      IF (gimatfree.EQ.0) THEN
CCCCC @@ Macro dependent parameters
         anorm = mmVectorMax (AD,nxda,nyda,nzda,bnda,nx,ny,nz,comm)
      ELSE
         CALL mmMatDiag (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,0,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
CCCCC @@ Macro dependent parameters
         anorm = mmVectorMax (P,nxdp,nydp,nzdp,bndp,nx,ny,nz,comm)
      END IF

      bnorm = mmDotProd (
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,comm,level,isub)
      bnorm = DSQRT (bnorm)
      rdot  = mmDotProd (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,level,isub)
      rnorm = DSQRT(rdot)
      rdotFirst = rdot
      rnormFirst = rnorm

#ifdef DEBUG_BICG
      print*, 'mmBiCgStab: anorm = ',anorm
      print*, 'mmBiCgStab: bnorm = ',bnorm
      print*, 'mmBiCgStab: rnorm = ',rnorm
#endif
      
      DO iz=1,nz
         DO iy=1,ny
            DO ix=1,nx
               RT(ix,iy,iz) = R(ix,iy,iz)
               P(ix,iy,iz) = R(ix,iy,iz)
            END DO
         END DO
      END DO

      rtr = mmDotProd (
     &     RT,nxdrt,nydrt,nzdrt,bndrt
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,level,isub)

      iter = 0

 10   CONTINUE

*-----------------------------------------------------------------------
*     Fine-level convergence test
*-----------------------------------------------------------------------

      IF (level.EQ.1) THEN

         isConverged = .FALSE.

         isConverged = mmIsConverged(istat,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,
     &        anorm,bnorm,rnorm,rnormFirst,rtol,iter,
     &        maxIterCg,
     &     comm,nbr,bnd
     &     ,1,isub)

         if (isConverged) GO TO 20

      ELSE

         IF (iter .GE. maxIterCg .OR. rdot/rdotFirst .LT. rtol2) GOTO 20 

      END IF
      
#ifdef DEBUG_BICG
      if (ip.EQ.0) PRINT*, 'mmBiCgStab Matvec 1'
#endif 
*-----------------------------------------------------------------------
      CALL mmMatvec (
     &     V,nxdv,nydv,nzdv,bndv
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,level,isub)
*-----------------------------------------------------------------------
#ifdef DEBUG_BICG
      tmp = mmVectorSum (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(P) = ',tmp
      tmp = mmVectorSum (
     &     V,nxdv,nydv,nzdv,bndv
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(V) = ',tmp
#endif

*-----------------------------------------------------------------------
      alpha = rtr / mmDotProd (
     &     RT,nxdrt,nydrt,nzdrt,bndrt
     &     ,
     &     V,nxdv,nydv,nzdv,bndv
     &     ,nx,ny,nz,comm,level,isub)
*-----------------------------------------------------------------------
#ifdef DEBUG_BICG
      if (ip.EQ.0) PRINT*, '  alpha = ',alpha
#endif

#ifdef DEBUG_BICG
      if (ip.EQ.0) PRINT*, 'mmBiCgStab Zaxpy 1'
#endif 
*-----------------------------------------------------------------------

      DO iz=2,nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               S(ix,iy,iz) = -alpha * V(ix,iy,iz) + R(ix,iy,iz)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif
*-----------------------------------------------------------------------
#ifdef DEBUG_BICG
      tmp = mmVectorSum (
     &     V,nxdv,nydv,nzdv,bndv
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(V) = ',tmp
#endif

*-----------------------------------------------------------------------
      CALL mmMatvec (
     &     T,nxdt,nydt,nzdt,bndt
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     S,nxds,nyds,nzds,bnds
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,level,isub)
*-----------------------------------------------------------------------
#ifdef DEBUG_BICG
      tmp = mmVectorSum (
     &     T,nxdt,nydt,nzdt,bndt
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(T) = ',tmp
#endif

*-----------------------------------------------------------------------
      omega = mmDotProd (
     &     T,nxdt,nydt,nzdt,bndt
     &     ,
     &     S,nxds,nyds,nzds,bnds
     &     ,nx,ny,nz,comm,level,isub) / 
     &     mmDotProd (
     &     T,nxdt,nydt,nzdt,bndt
     &     ,
     &     T,nxdt,nydt,nzdt,bndt
     &     ,nx,ny,nz,comm,level,isub)
*-----------------------------------------------------------------------
#ifdef DEBUG_BICG
      if (ip.EQ.0) PRINT*, '  omega = ',omega
#endif
      
#ifdef DEBUG_BICG
      if (ip.EQ.0) PRINT*, 'mmBiCgStab Zaxpy 2'
#endif 
*-----------------------------------------------------------------------

      DO iz=2,nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               X(ix,iy,iz) = alpha * P(ix,iy,iz) + X(ix,iy,iz)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif
*-----------------------------------------------------------------------
#ifdef DEBUG_BICG
      tmp = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*,  ' sum(X) = ',tmp
#endif

#ifdef DEBUG_BICG
      if (ip.EQ.0) PRINT*, 'mmBiCgStab Zaxpy 3'
#endif 
*-----------------------------------------------------------------------

      DO iz=2,nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               X(ix,iy,iz) = omega * S(ix,iy,iz) + X(ix,iy,iz)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif
*-----------------------------------------------------------------------
#ifdef DEBUG_BICG
      tmp = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(X) = ',tmp
#endif

#ifdef DEBUG_BICG
      if (ip.EQ.0) PRINT*, 'mmBiCgStab Zaxpy 4'
#endif 
*-----------------------------------------------------------------------

      DO iz=2,nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               R(ix,iy,iz) = -omega * T(ix,iy,iz) + S(ix,iy,iz)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif
*-----------------------------------------------------------------------
#ifdef DEBUG_BICG
      tmp = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(R) = ',tmp
#endif

*-----------------------------------------------------------------------
      rtr2 = mmDotProd (
     &     RT,nxdrt,nydrt,nzdrt,bndrt
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,level,isub)
*-----------------------------------------------------------------------
#ifdef DEBUG_BICG
      if (ip.EQ.0) PRINT*, '    rtr2 = ',rtr2
#endif 
      beta = rtr2/rtr*(alpha/omega)
      rtr = rtr2

      DO iz=1,nz
         DO iy=1,ny
            DO ix=1,nx
               P(ix,iy,iz) = R(ix,iy,iz) + beta
     &              *(P(ix,iy,iz) - omega*V(ix,iy,iz))
            END DO
         END DO
      END DO
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 4*nx*ny*nz
      gmflops(isub)=gmflops(isub) + 
     & 4*nx*ny*nz
#endif

#ifdef DEBUG_BICG
      tmp = mmVectorSum (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(P) = ',tmp
#endif

      rdotPrev = rdot
      rdot = mmDotProd (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,level,isub)
      rnorm = DSQRT(rdot)

#ifdef DEBUG_BICG
      if (ip.EQ.0) PRINT*, '    rdot = ',rdot
#endif 

C       R=B
C       RT = R
C       P=R
C       rtr = RT*R
C       iter = 0
C       IF (mmIsConverged(@)) GOTO 10
C       V = A*P
C       alpha = rtr / (RT*V)
C       S = R - alpha*V
C       T = A*S
C 
C       omega=S*T/(T*T)
C       X=X+alpha*P + omega*S
C       R=S-omega*T
C 
C       rtr2=RT*R
C       beta=rtr2/rtr
C       rtr = rtr2
C 
C       P=R+(beta*alpha/omega)*(P-omega*V)
C 
C       iter = iter + 1

      iter = iter + 1
      numIterCg = numIterCg + 1

      IF (outputLevel.GT.1 .AND. ip.EQ.0)
     &     WRITE(*,1000) iter, MPI_WTIME() - gtimestart(SB_MG3),
     &     DSQRT(rdot/rdotFirst)

      GO TO 10

*-----------------------------------------------------------------------
*     Exit subroutine, printing warning message if applicable
*-----------------------------------------------------------------------

 20   CONTINUE

      IF (ip.EQ.0.AND.outputLevel.GT.0) THEN
         IF (istat.EQ.1) print*,
     &        '== mgmpi ERROR == aborting BiCG-S: apparently diverging'
         IF (istat.EQ.2) print*,
     &        '== mgmpi ERROR == aborting BiCG-S: residual norm is NaN'
         IF (istat.EQ.3) print*,
     &        '-- mgmpi warning -- ',
     &        'exiting BiCG-S: iteration limit reached'
      END IF

      IF (outputLevel.GT.1 .AND. ip.EQ.0)
     &     WRITE(*,1000) iter, MPI_WTIME() - gtimestart(SB_MG3),
     &     DSQRT(rdot/rdotFirst)

      rnorm=DSQRT(rdot)
      resRedFine=DSQRT(rdot/rdotFirst)

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN

 1000 FORMAT (1x,i4,f8.2,e14.3)

      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmBiCgStabMg
C     
C     Purpose: Bi Conjugate gradient-Stabalized method preconditioned
C     Purpose: with MG V-cycle
C     
C     Algorithm: R=B
C     Algorithm: RT = R
C     Algorithm: P=R
C     Algorithm: rtr = RT*R
C     Algorithm: iter = 0
C     Algorithm: IF (mmIsConverged(@)) GOTO 10
C     Algorithm: V = A*P
C     Algorithm: alpha = rtr / (RT*V)
C     Algorithm: S = R - alpha*V
C     Algorithm: T = A*S
C     Algorithm:  
C     Algorithm: omega=S*T/(T*T)
C     Algorithm: X=X+alpha*P + omega*S
C     Algorithm: R=S-omega*T
C     Algorithm:  
C     Algorithm: rtr2=RT*R
C     Algorithm: beta=rtr2/rtr
C     Algorithm: rtr = rtr2
C     Algorithm:  
C     Algorithm: P=R+(beta*alpha/omega)*(P-omega*V)
C     Algorithm:  
C     Algorithm: iter = iter + 1
C     
C     Inputs:  XF: fine grid function
C     
C     Outputs: XC: coarse grid function
C     
C     Globals accessed:
C     
C     Globals modified:
C     
C     Flop count estimate:     nxf nyf nzf
C     
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C     
C     Modification history: 
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBiCgStabMg (
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     ACD,AC1,AC2,AC3,AC4,AC5,AC6,nxdac,nydac,nzdac,bndac
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     BC,nxdbc,nydbc,nzdbc,bndbc
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     RT,nxdrt,nydrt,nzdrt,bndrt
     &     ,
     &     P,nxdp,nydp,nzdp,bndp
     &     ,
     &     PT,nxdpt,nydpt,nzdpt,bndpt
     &     ,
     &     S,nxds,nyds,nzds,bnds
     &     ,
     &     ST,nxdst,nydst,nzdst,bndst
     &     ,
     &     V,nxdv,nydv,nzdv,bndv
     &     ,
     &     T,nxdt,nydt,nzdt,bndt
     &     ,
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &     indh,inda,indx
     &     ,
     &     m,nx,ny,nz,
     &     nxg,nyg,nzg,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver,
     &     wgt,numIterBiCg,iterCoarse,maxIterBiCg,maxItCoarse,
     &     resTolBiCg,resTolCoarse,outputLevelBiCg,
     &     comm,nbr,bnd,bnd2
     &     ,scaleA,scaleB,rnorm,resRedFine,istat,icall)
      
      IMPLICIT NONE
      
      
      INTEGER nxdaf,nydaf,nzdaf
      INTEGER bndaf(3)  
      REAL AFD(nxdaf,nydaf,nzdaf)
      REAL AF1(nxdaf,nydaf,nzdaf)
      REAL AF2(nxdaf,nydaf,nzdaf)
      REAL AF3(nxdaf,nydaf,nzdaf)
      REAL AF4(nxdaf,nydaf,nzdaf)
      REAL AF5(nxdaf,nydaf,nzdaf)
      REAL AF6(nxdaf,nydaf,nzdaf)
      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      
      INTEGER nxdbf,nydbf,nzdbf
      INTEGER bndbf(3)	
      REAL BF(nxdbf,nydbf,nzdbf)
      
      INTEGER nxdac(*),nydac(*),nzdac(*)
      INTEGER bndac(3,*) 
      REAL ACD(*)
      REAL AC1(*)
      REAL AC2(*)
      REAL AC3(*)
      REAL AC4(*)
      REAL AC5(*)
      REAL AC6(*)
      
      INTEGER nxdxc(*),nydxc(*),nzdxc(*)
      INTEGER bndxc(3,*)  
      REAL XC(*)
      
      INTEGER nxdbc(*),nydbc(*),nzdbc(*)
      INTEGER bndbc(3,*)
      REAL BC(*)
      
      INTEGER nxdy(*),nydy(*),nzdy(*)
      INTEGER bndy(3,*)
      REAL Y(*)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxdrt,nydrt,nzdrt
      INTEGER bndrt(3)	
      REAL RT(nxdrt,nydrt,nzdrt)
      
      INTEGER nxdp,nydp,nzdp
      INTEGER bndp(3)	
      REAL P(nxdp,nydp,nzdp)
      
      INTEGER nxdpt,nydpt,nzdpt
      INTEGER bndpt(3)	
      REAL PT(nxdpt,nydpt,nzdpt)
      
      INTEGER nxds,nyds,nzds
      INTEGER bnds(3)	
      REAL S(nxds,nyds,nzds)
      
      INTEGER nxdst,nydst,nzdst
      INTEGER bndst(3)	
      REAL ST(nxdst,nydst,nzdst)
      
      INTEGER nxdv,nydv,nzdv
      INTEGER bndv(3)	
      REAL V(nxdv,nydv,nzdv)
      
      INTEGER nxdt,nydt,nzdt
      INTEGER bndt(3)	
      REAL T(nxdt,nydt,nzdt)
      
      INTEGER nxdrcg,nydrcg,nzdrcg
      INTEGER bndrcg(3)
      REAL RCG(nxdrcg,nydrcg,nzdrcg)
      
      INTEGER nxdpcg,nydpcg,nzdpcg
      INTEGER bndpcg(3)
      REAL PCG(nxdpcg,nydpcg,nzdpcg)
      
      INTEGER nxdwcg,nydwcg,nzdwcg
      INTEGER bndwcg(3)
      REAL WCG(nxdwcg,nydwcg,nzdwcg)
      
      INTEGER nxdrsm,nydrsm,nzdrsm
      INTEGER bndrsm(3)
      REAL RSM(nxdrsm,nydrsm,nzdrsm)
      
      INTEGER nxdpsm,nydpsm,nzdpsm
      INTEGER bndpsm(3)
      REAL PSM(nxdpsm,nydpsm,nzdpsm)
      
      INTEGER nxdwsm,nydwsm,nzdwsm
      INTEGER bndzsm(3)
      REAL WSM(nxdwsm,nydwsm,nzdwsm)

      INTEGER indh(*),inda(*),indx(*)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3,*),bnd2(3)

      INTEGER m
      INTEGER nx(m),ny(m),nz(m)
      INTEGER nxg(m),nyg(m),nzg(m)
      INTEGER numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver
      REAL resTolFine,resTolCoarse,wgt
      INTEGER iterFine,iterCoarse,outputLevelMg
      INTEGER maxItFine,maxItCoarse
      REAL scaleA(*), scaleB(*),rnorm,rnormMg
      REAL resRedFine,resRedFineMg
      INTEGER istat,istatMg
      INTEGER numIterBiCg,maxIterBiCg,outputLevelBiCg
      REAL resTolBiCg

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

#ifdef DEBUG_BICG
      REAL mmVectorSum,tmp
      EXTERNAL mmVectorSum
#endif

      INTEGER ix,iy,iz,iter
      REAL alpha,beta,mmVectorMax
      REAL anorm,bnorm,rdotFirst,rdot,rdotPrev,rtol2
      REAL rtr,rtr2,omega,mmDotProd,rnormFirst
      LOGICAL mmIsConverged,isConverged,preconOn

      EXTERNAL mmDotProd

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_CGMG
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

C-----------------------------------------------------------------------
C     Preconditioner parameters
C-----------------------------------------------------------------------

      preconOn = .TRUE.
      iterFine = 0
      maxItFine = 1
      resTolFine = 0.0D0
      outputLevelMg = 0
      rnormMg = 0.0D0
      resRedFineMg = 0.0D0
      istatMg = 0

      istat = 0
      rtol2 = resTolBiCg*resTolBiCg

C-----------------------------------------------------------------------
C     Fix for singular problems with null(A) = ones(n,1)
C-----------------------------------------------------------------------

      CALL mmShiftRhs (
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,nxg,nyg,nzg,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,comm,isub)

C-----------------------------------------------------------------------
C     Initialize Dirichlet b.c.'s
C-----------------------------------------------------------------------

#ifdef DEBUG_BICG
      if (ip.EQ.0) PRINT*, 'mmBiCgStabMg DirichletBC'
#endif

      CALL mmDirichletBC (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,scaleA,scaleB,1,.FALSE.)

*-----------------------------------------------------------------------
*     Compute residual R = B - A*X
*-----------------------------------------------------------------------

      CALL mmResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,0,1,isub)

C-----------------------------------------------------------------------
C     Compute norms for convergence test
C-----------------------------------------------------------------------

      IF (gimatfree.EQ.0) THEN
CCCCC @@ Macro dependent parameters
         anorm = mmVectorMax (AFD,nxdaf,nydaf,nzdaf,bndaf,nx,ny,nz,
     &        comm)
      ELSE
         CALL mmMatDiag (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,0,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,gScaleA,isub)
CCCCC @@ Macro dependent parameters
         anorm = mmVectorMax (P,nxdp,nydp,nzdp,bndp,nx,ny,nz,comm)
      END IF

      bnorm = mmDotProd (
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,comm,1,isub)
      bnorm = DSQRT (bnorm)
      rdot  = mmDotProd (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,1,isub)
      rnorm = DSQRT(rdot)
      rdotFirst = rdot
      rnormFirst = rnorm

#ifdef DEBUG_BICG
      print*, 'mmBiCgStabMg: anorm = ',anorm
      print*, 'mmBiCgStabMg: bnorm = ',bnorm
      print*, 'mmBiCgStabMg: rnorm = ',rnorm
#endif
      
      DO iz=1,nz(1)
         DO iy=1,ny(1)
            DO ix=1,nx(1)
               RT(ix,iy,iz) = R(ix,iy,iz)
               P(ix,iy,iz) = R(ix,iy,iz)
            END DO
         END DO
      END DO

#ifdef DEBUG_BICG
      tmp = mmVectorSum (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(P) = ',tmp
#endif

      rtr = mmDotProd (
     &     RT,nxdrt,nydrt,nzdrt,bndrt
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,1,isub)

      iter = 0

 10   CONTINUE

C-----------------------------------------------------------------------
C     Fine-level convergence test
C-----------------------------------------------------------------------

      isConverged = .FALSE.

      isConverged = mmIsConverged(istat,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,
     &     anorm,bnorm,rnorm,rnormFirst,resTolBiCg,iter,
     &     maxIterBiCg,
     &     comm,nbr,bnd
     &     ,1,isub)

      IF (istat.EQ.1) isConverged=.FALSE.
      IF (isConverged) GO TO 20

      
C-----------------------------------------------------------------------
C     if (preconOn) then SOLVE A PT = P; 
C     (otherwise PT === P and nothing is done)
C-----------------------------------------------------------------------

      IF (preconOn) THEN
         DO iz=1,nz(1)
            DO iy=1,ny(1)
               DO ix=1,nx(1)
                  PT(ix,iy,iz) = 0.0D0
               END DO
            END DO
         END DO
#ifdef DEBUG_BICG
         IF (ip.EQ.0) PRINT*, 'mmBiCgStabMg MgPrecon 1'
#endif

C@@@@ Clear boundary of P???

#ifdef DEBUG_BICG
         tmp = mmVectorSum (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,comm,isub)
         IF (ip.EQ.0) PRINT*, '  sum(P) = ',tmp
#endif
         CALL mmMgPrecon (
     &        
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     PT,nxdpt,nydpt,nzdpt,bndpt
     &     ,
     &     P,nxdp,nydp,nzdp,bndp
     &     ,
     &     ACD,AC1,AC2,AC3,AC4,AC5,AC6,nxdac,nydac,nzdac,bndac
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     BC,nxdbc,nydbc,nzdbc,bndbc
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &        
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &        
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &        
     &     indh,inda,indx
     &     ,
     &        m,nx,ny,nz,
     &        nxg,nyg,nzg,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver,
     &        wgt,iterFine,iterCoarse,
     &        maxItFine,maxItCoarse,
     &        resTolFine,resTolCoarse,outputLevelMg,
     &        
     &     comm,nbr,bnd,bnd2
     &     ,
     &        scaleA,scaleB,rnormMg,resRedFineMg,istatMg,isub)
#ifdef DEBUG_BICG
         IF (ip.EQ.0) PRINT*, 'mmBiCgStabMg MgPrecon 1 status:',istatMg
#endif
         CALL mmResidual (
     &        
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,0,1,isub)
      ELSE
         DO iz=1,nz(1)
            DO iy=1,ny(1)
               DO ix=1,nx(1)
                  PT(ix,iy,iz) = P(ix,iy,iz)
               END DO
            END DO
         END DO
      END IF

      
#ifdef DEBUG_BICG
      IF (ip.EQ.0) PRINT*, 'mmBiCgStabMg Matvec 1'
#endif
C-----------------------------------------------------------------------
      CALL mmMatvec (
     &     V,nxdv,nydv,nzdv,bndv
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     PT,nxdpt,nydpt,nzdpt,bndpt
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,1,isub)
C-----------------------------------------------------------------------
#ifdef DEBUG_BICG
      tmp = mmVectorSum (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, '  sum(PT) = ',tmp
      tmp = mmVectorSum (
     &     V,nxdv,nydv,nzdv,bndv
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, '  sum(V) = ',tmp
#endif

C-----------------------------------------------------------------------
      alpha = rtr / mmDotProd (
     &     RT,nxdrt,nydrt,nzdrt,bndrt
     &     ,
     &     V,nxdv,nydv,nzdv,bndv
     &     ,nx,ny,nz,comm,1,isub)
C-----------------------------------------------------------------------
#ifdef DEBUG_BICG
      IF (ip.EQ.0) PRINT*, '  alpha = ',alpha
#endif

#ifdef DEBUG_BICG
      IF (ip.EQ.0) PRINT*, 'mmBiCgStabMg Zaxpy 1'
#endif
C-----------------------------------------------------------------------

      DO iz=2,nz(1)-1
         DO iy=2,ny(1)-1
            DO ix=2,nx(1)-1
               S(ix,iy,iz) = -alpha * V(ix,iy,iz) + R(ix,iy,iz)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx(1)-2)*(ny(1)-2)*(nz(1)-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx(1)-2)*(ny(1)-2)*(nz(1)-2)
#endif
C-----------------------------------------------------------------------
#ifdef DEBUG_BICG
      tmp = mmVectorSum (
     &     V,nxdv,nydv,nzdv,bndv
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, '  sum(V) = ',tmp
#endif

C-----------------------------------------------------------------------
C     C SOLVE A ST = S; otherwise ST === S and nothing is done
C-----------------------------------------------------------------------

      IF (preconOn) THEN
         DO iz=1,nz(1)
            DO iy=1,ny(1)
               DO ix=1,nx(1)
                  ST(ix,iy,iz) = 0.0D0
               END DO
            END DO
         END DO
#ifdef DEBUG_BICG
         IF (ip.EQ.0) PRINT*, 'mmBiCgStabMg MgPrecon 2'
#endif
#ifdef DEBUG_BICG
         tmp = mmVectorSum (
     &     S,nxds,nyds,nzds,bnds
     &     ,nx,ny,nz,comm,isub)
         IF (ip.EQ.0) PRINT*, '  sum(S) = ',tmp
#endif
         CALL mmMgPrecon (
     &        
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     ST,nxdst,nydst,nzdst,bndst
     &     ,
     &     S,nxds,nyds,nzds,bnds
     &     ,
     &     ACD,AC1,AC2,AC3,AC4,AC5,AC6,nxdac,nydac,nzdac,bndac
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     BC,nxdbc,nydbc,nzdbc,bndbc
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &        
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &        
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &        
     &     indh,inda,indx
     &     ,
     &        m,nx,ny,nz,
     &        nxg,nyg,nzg,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver,
     &        wgt,iterFine,iterCoarse,
     &        maxItFine,maxItCoarse,
     &        resTolFine,resTolCoarse,outputLevelMg,
     &        
     &     comm,nbr,bnd,bnd2
     &     ,
     &        scaleA,scaleB,rnormMg,resRedFineMg,istatMg,isub)
#ifdef DEBUG_BICG
         IF (ip.EQ.0) PRINT*, 'mmBiCgStabMg MgPrecon 2 status:',istatMg
#endif
         CALL mmResidual (
     &        
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,0,1,isub)
      ELSE
         DO iz=1,nz(1)
            DO iy=1,ny(1)
               DO ix=1,nx(1)
                  ST(ix,iy,iz) = S(ix,iy,iz)
               END DO
            END DO
         END DO
      END IF

#ifdef DEBUG_BICG
      IF (ip.EQ.0) PRINT*, 'mmBiCgStabMg Matvec 2'
#endif
C-----------------------------------------------------------------------
      CALL mmMatvec (
     &     T,nxdt,nydt,nzdt,bndt
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     ST,nxdst,nydst,nzdst,bndst
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,1,isub)
C-----------------------------------------------------------------------
#ifdef DEBUG_BICG
      tmp = mmVectorSum (
     &     T,nxdt,nydt,nzdt,bndt
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, '  sum(T) = ',tmp
#endif

C-----------------------------------------------------------------------
      omega = mmDotProd (
     &     T,nxdt,nydt,nzdt,bndt
     &     ,
     &     S,nxds,nyds,nzds,bnds
     &     ,nx,ny,nz,comm,1,isub) / 
     &     mmDotProd (
     &     T,nxdt,nydt,nzdt,bndt
     &     ,
     &     T,nxdt,nydt,nzdt,bndt
     &     ,nx,ny,nz,comm,1,isub)
C-----------------------------------------------------------------------
#ifdef DEBUG_BICG
      IF (ip.EQ.0) PRINT*, '  omega = ',omega
#endif
      
#ifdef DEBUG_BICG
      IF (ip.EQ.0) PRINT*, 'mmBiCgStabMg Zaxpy 2'
#endif
C-----------------------------------------------------------------------

      DO iz=2,nz(1)-1
         DO iy=2,ny(1)-1
            DO ix=2,nx(1)-1
               XF(ix,iy,iz) = XF(ix,iy,iz) + alpha * PT(ix,iy,iz) 
     &              + omega * ST(ix,iy,iz)
               R(ix,iy,iz) =  S(ix,iy,iz) - omega * T(ix,iy,iz)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 6*(nx(1)-2)*(ny(1)-2)*(nz(1)-2)
      gmflops(isub)=gmflops(isub) + 
     & 6*(nx(1)-2)*(ny(1)-2)*(nz(1)-2)
#endif

C-----------------------------------------------------------------------
#ifdef DEBUG_BICG
      tmp = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, '  sum(R) = ',tmp
#endif

C-----------------------------------------------------------------------
      rtr2 = mmDotProd (
     &     RT,nxdrt,nydrt,nzdrt,bndrt
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,1,isub)
C-----------------------------------------------------------------------
#ifdef DEBUG_BICG
      IF (ip.EQ.0) PRINT*, '    rtr2 = ',rtr2
#endif
      beta = rtr2/rtr*(alpha/omega)
      rtr = rtr2

      DO iz=1,nz(1)
         DO iy=1,ny(1)
            DO ix=1,nx(1)
               P(ix,iy,iz) = R(ix,iy,iz) + beta
     &              *(P(ix,iy,iz) - omega*V(ix,iy,iz))
            END DO
         END DO
      END DO
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 4*nx(1)*ny(1)*nz(1)
      gmflops(isub)=gmflops(isub) + 
     & 4*nx(1)*ny(1)*nz(1)
#endif

#ifdef DEBUG_BICG
      tmp = mmVectorSum (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, '  sum(P) = ',tmp
#endif

      rdotPrev = rdot
      rdot = mmDotProd (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,1,isub)
      rnorm = DSQRT(rdot)

#ifdef DEBUG_BICG
      IF (ip.EQ.0) PRINT*, '    rdot = ',rdot
#endif

      iter = iter + 1
      numIterBiCg = numIterBiCg + 1

      IF (outputLevelBiCg.GT.1 .AND. ip.EQ.0)
     &     WRITE(*,1000) iter, MPI_WTIME() - gtimestart(SB_MG3),
     &     DSQRT(rdot/rdotFirst)

      GO TO 10

C-----------------------------------------------------------------------
C     Exit subroutine, printing warning message if applicable
C-----------------------------------------------------------------------

 20   CONTINUE

      IF (ip.EQ.0.AND.outputLevelBiCg.GT.0) THEN
C     IF (istat.EQ.1) print*,
C     &        '== mgmpi ERROR == aborting BiCG-S: apparently diverging'
         IF (istat.EQ.2) print*,
     &        '== mgmpi ERROR == aborting BiCG-S: residual norm is NaN'
         IF (istat.EQ.3) print*,
     &        '-- mgmpi warning -- ',
     &        'exiting BiCG-S: iteration limit reached'
      END IF

      IF (outputLevelBiCg.GT.1 .AND. ip.EQ.0)
     &     WRITE(*,1000) iter, MPI_WTIME() - gtimestart(SB_MG3),
     &     DSQRT(rdot/rdotFirst)

      rnorm=DSQRT(rdot)
      resRedFine=DSQRT(rdot/rdotFirst)

      CALL mmDirichletBC (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,scaleA,scaleB,1,.FALSE.)

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

 1000 FORMAT (1x,i4,f8.2,e10.3)

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmCg2
C     
C     PURPOSE: Planar (2D) Conjugate gradient method 
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmCg2 (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     P,nxdp,nydp,nzdp,bndp
     &     ,
     &     W,nxdw,nydw,nzdw,bndw
     &     ,nx,ny,nz,nxg,nyg,nzg,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     numIterCg,maxIterCg,rtol,outputLevel,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,lx,ly,lz,mask,level,rnorm,resRedFine,istat,
     &     icall)

      IMPLICIT NONE

      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxdp,nydp,nzdp
      INTEGER bndp(3)	
      REAL P(nxdp,nydp,nzdp)
      
      INTEGER nxdw,nydw,nzdw
      INTEGER bndw(3)	
      REAL W(nxdw,nydw,nzdw)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz,level
      INTEGER nxg,nyg,nzg
      INTEGER numIterCg,maxIterCg,outputLevel
      REAL rtol
      LOGICAL lx,ly,lz
      REAL scaleA, scaleB
      INTEGER istat
      REAL resRedFine,rnorm(*)

#ifdef DEBUG_CG
      REAL s,mmVectorSum
#endif

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER mmIsNan
      EXTERNAL mmIsNan      

      INTEGER iter,i,n,ng
      REAL alpha(NMAXLIM),beta(NMAXLIM),t(NMAXLIM)
      REAL bnorm(NMAXLIM)
      REAL rdotFirst(NMAXLIM),rdot(NMAXLIM),rdotPrev(NMAXLIM),rtol2
      REAL rmin(NMAXLIM)
      LOGICAL mask(*),isConverged
      INTEGER iConvergeCount,iDivergeCount,iProblemCount

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

#ifdef DEBUG_CG2
      REAL s,ierr,tnorm(NMAXLIM)
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_CG2+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      CALL mmVectorClear (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz)

#ifdef ASSERT
      IF (.NOT.(((.NOT.lx).AND.ly.AND.lz) .OR.
     &     (lx.AND.(.NOT.ly).AND.lz) .OR.
     &     (lx.AND.ly.AND.(.NOT.lz)))) THEN
         PRINT*, 'mmCg2 ASSERTION FAILED!'
         PRINT*, 'lx ly lz =',lx,ly,lz
      END IF
#endif

#ifdef DUMP_INPUTS
      DO i=0,npx*npy*npz-1
         IF (ip.EQ.i) THEN
            print*, 'CG enter:'
            print*, 'AD(2,2,2)=',AD(2,2,2)
            print*, 'nxda,nyda,nzda=',nxda,nyda,nzda
            print*, 'nxdx,nydx,nzdx=',nxdx,nydx,nzdx
            print*, 'nxdb,nydb,nzdb=',nxdb,nydb,nzdb
            print*, 'nxdr,nydr,nzdr=',nxdr,nydr,nzdr
            print*, 'nxdp,nydp,nzdp=',nxdp,nydp,nzdp
            print*, 'nxdw,nydw,nzdw=',nxdw,nydw,nzdw
            print*, 'nx,ny,nz=',nx,ny,nz
            print*, 'nxg,nyg,nzg=',nxg,nyg,nzg
            print*, 'ip,ipx,ipy,ipz=',ip,ipx,ipy,ipz
            print*, 'npx,npy,npz=',npx,npy,npz
            print*, 'xp(2),xp(nx-1)=',xp(2),xp(nx-1)
            print*, 'yp(2),yp(ny-1)=',yp(2),yp(ny-1)
            print*, 'zp(2),zp(nz-1)=',zp(2),zp(nz-1)
            print*, 'hx(2),hx(nx-1)=',hx(2),hx(nx-1)
            print*, 'hy(2),hy(ny-1)=',hy(2),hy(ny-1)
            print*, 'hz(2),hz(nz-1)=',hz(2),hz(nz-1)
            print*, 'nxdbnd,nydbnd,nzdbnd=',nxdbnd,nydbnd,nzdbnd
            print*, 'ixpb,ixmb=',ixpb,ixmb
            print*, 'iypb,iymb=',iypb,iymb
            print*, 'izpb,izmb=',izpb,izmb
            print*, 'numIterCg=',numIterCg
            print*, 'maxIterCg=',maxIterCg
            print*, 'rtol=',rtol
            print*, 'outputLevel=',outputLevel
            print*, 'comm=',comm
            print*, 'nbr=',nbr
            print*, 'bnd=',bnd
            print*, 'scaleA=',scaleA
            print*, 'scaleB=',scaleB
            print*, 'lx=',lx
            print*, 'ly=',ly
            print*, 'lz=',lz
            print*, 'level=',level
C     print*, 'rnorm=',rnorm(2),rnorm(3),rnorm(4)
C     print*, 'resRedFine=',resRedFine
C     print*, 'istat=',istat
         END IF
      END DO
#endif

      IF (.NOT.lx) THEN
         n = nx
         ng = (nyg-2)*(nzg-2)
      ELSE IF (.NOT.ly) THEN
         n = ny
         ng = (nxg-2)*(nzg-2)
      ELSE IF (.NOT.lz) THEN
         n = nz
         ng = (nxg-2)*(nyg-2)
      ELSE 
         PRINT*, 'MPI Error in mmCg2: lx,ly,lz=',lx,ly,lz
         CALL mmTerminate(comm)
      END IF

      istat = 0

      iProblemCount = 0
      iConvergeCount = 0
      iDivergeCount = 0

      DO i=1,n
         IF (mask(i)) iProblemCount=iProblemCount+1
      END DO

*-----------------------------------------------------------------------
*     Fix for singular problems with null(A) = ones(n,1)
*-----------------------------------------------------------------------

      IF ( ABS(ixmb).NE.MM_DIRICHLET .AND.
     &     ABS(iymb).NE.MM_DIRICHLET .AND. 
     &     ABS(izmb).NE.MM_DIRICHLET .AND.
     &     ABS(ixpb).NE.MM_DIRICHLET .AND.
     &     ABS(iypb).NE.MM_DIRICHLET .AND.
     &     ABS(izpb).NE.MM_DIRICHLET) THEN

         CALL mmVectorSum2 (t,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,comm,lx,ly,lz,mask,isub) 
         DO i=1,n
            IF (mask(i)) t(i)=t(i) / ng
         END DO

         CALL mmVectorShift2 (
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,t,comm,lx,ly,lz,mask,isub)

      END IF

*-----------------------------------------------------------------------
*     Initialize Dirichlet b.c.'s
*-----------------------------------------------------------------------

C     IF (level.EQ.1) THEN
C     CALL mmDirichletBC2 (__X,nx,ny,nz,
C     &     __GRID,__BOUNDARY__CONDITIONS,
C     &     scaleA,scaleB,lx,ly,lz,1,.FALSE.)
C     ELSE
C     CALL mmDirichletBC2 (__X,nx,ny,nz,
C     &     __GRID,__BOUNDARY__CONDITIONS,
C     &     scaleA,scaleB,lx,ly,lz,level,.TRUE.)
C     END IF

C     Solving residual equation, so Dirichlet values should be 0

      CALL mmClearBoundary2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,lx,ly,lz)

*-----------------------------------------------------------------------
*     Compute residual R = B - A*X
*-----------------------------------------------------------------------

#ifdef DEBUG_CG2
      if (ip.EQ.0) WRITE(*,"('mmCg2 Residual2',i2)") level
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,' mmCg2 Residual2')") ip
#endif
#endif
      CALL mmResidual2 (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,0,lx,ly,lz,mask,level,isub)
      CALL mmClearBoundary2 (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,lx,ly,lz)
#ifdef DEBUG_CG2
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(R) = ',s
      s = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(X) = ',s
#endif

#ifdef DEBUG_CG2
      if (ip.EQ.0) WRITE(*,"('mmCg2 DotProd2',i2)") level
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,' mmCg2 DotProd2-1')") ip
#endif
#endif
      CALL mmDotProd2 (rnorm,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,lx,ly,lz,mask,level,
     &     isub)
#ifdef DEBUG_CG2
      if (ip.EQ.0) PRINT*, '  rnorm(2) = ',rnorm(2)
#endif

      CALL mmVectorCopy2 (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,lx,ly,lz,mask)

*-----------------------------------------------------------------------
*     Compute norms for convergence test
*-----------------------------------------------------------------------

#ifdef DEBUG_CG2
      if (ip.EQ.0) WRITE(*,"('mmCg2 DotProd2',i2)") level
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,' mmCg2 DotProd2-2')") ip
#endif
#endif
      CALL mmDotProd2 (bnorm,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,comm,lx,ly,lz,mask,level,
     &     isub)
#ifdef DEBUG_CG2
      if (ip.EQ.0) PRINT*, '  bnorm(2) = ',bnorm(2)
#endif

#ifdef DEBUG_CG2
      if (ip.EQ.0) WRITE(*,"('mmCg2 DotProd2',i2)") level
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,' mmCg2 DotProd2-3')") ip
#endif
#endif
      CALL mmDotProd2 (rdot,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,lx,ly,lz,mask,level,
     &     isub)

      iter = 0

      DO i=1,n
         IF (mask(i)) THEN
            IF (DABS(rdot(i)).LT.mmSmall2) THEN
               mask(i)=.FALSE.
               iConvergeCount = iConvergeCount + 1
            ELSE 
               bnorm(i) = DSQRT(bnorm(i))
               rdotFirst(i) = rdot(i)
               rmin(i) = rdot(i)
            END IF
         END IF
      END DO

C     CALL mmVectorCopy2 (__B,__X,nx,ny,nz,lx,ly,lz,mask)

      IF (outputLevel.GT.0 .AND. ip.EQ.0) THEN
         DO i=1,n
            IF (mask(i)) THEN
               IF (DABS(rdotFirst(i)).GT.mmSmall2) THEN
                  print*, 'CG2: ',iter,i,DSQRT(rdot(i)/rdotFirst(i))
               ELSE
                  print*, 'CG2(r): ',iter,i,DSQRT(rdot(i))
               END IF
            END IF
         END DO
      END IF

*-----------------------------------------------------------------------
*     Check for convergence
*-----------------------------------------------------------------------

      rtol2 = rtol*rtol

 10   CONTINUE

*-----------------------------------------------------------------------
*     Fine-level convergence test
*-----------------------------------------------------------------------

      IF (iter .GE. maxIterCg) GOTO 20

*-------------------------------------------
*     "Check" for divergence
*-------------------------------------------

      DO i=1,n
         IF (mask(i)) THEN
            IF (rdot(i).GT.rdotFirst(i)*1.0D6) THEN
               IF (outputLevel.GT.1 .AND. ip.EQ.0)
     &              PRINT*, 'cg2:(',lx,ly,lz,') level ',i,
     &              'apparently diverging'
               mask(i)=.FALSE.
               iDivergeCount = iDivergeCount + 1
            END IF
         END IF
      END DO

*-------------------------------------------
*     Check for NaN's
*-------------------------------------------

      DO i=1,n
         IF (mask(i)) THEN
C            IF (mmIsNan(rdot(i)).NE.0) GOTO 40
         END IF
      END DO

*-------------------------------------------
*     ||R|| <= epsilon * ( ||B|| )
*-------------------------------------------

      isConverged = .TRUE.
      DO i=1,n
         IF (mask(i)) THEN
            IF  ((level.EQ.1.AND.DSQRT(rdot(i)).GT.rtol*bnorm(i)) .OR.
     &           (level.GT.1.AND.rdot(i).GT.rtol2*rdotFirst(i))) THEN
               isConverged = .FALSE.
            ELSE
               mask(i) = .FALSE.
               iConvergeCount = iConvergeCount + 1
            END IF
         END IF
      END DO
      IF (isConverged) GOTO 20

*-----------------------------------------------------------------------
*     Start multigrid
*-----------------------------------------------------------------------

      CALL mmRefreshBC2 (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,lx,ly,lz,level,isub)


#ifdef DEBUG_CG2
      if (ip.EQ.0) WRITE(*,"('mmCg2 Matvec2',i2)") level
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,' mmCg2 Matvec2 W=A*P')") ip
#endif
#endif
      CALL mmMatvec2 (
     &     W,nxdw,nydw,nzdw,bndw
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     lx,ly,lz,mask,level,isub)
#ifdef DEBUG_CG2
      s = mmVectorSum (
     &     W,nxdw,nydw,nzdw,bndw
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(W) = ',s
      s = mmVectorSum (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(P) = ',s
#endif

#ifdef DEBUG_CG2
      if (ip.EQ.0) WRITE(*,"('mmCg2 DotProd2',i2)") level
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,' mmCg2 DotProd2-5')") ip
#endif
#endif
      CALL mmDotProd2(alpha,
     &     W,nxdw,nydw,nzdw,bndw
     &     ,
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,comm,lx,ly,lz,mask,level,
     &     isub)
#ifdef DEBUG_CG2
      if (ip.EQ.0) PRINT*, '  alpha(2) = ',alpha(2)
#endif

      DO i=1,n
         IF (mask(i)) THEN
            alpha(i) = rdot(i) / alpha(i)
         END IF
      END DO

#ifdef DEBUG_CG2
      if (ip.EQ.0) WRITE(*,"('mmCg2 Zaxpy2',i2)") level
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,' mmCg2 Zaxpy2-1')") ip
#endif
#endif
      CALL mmZaxpy2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,alpha,
     &     P,nxdp,nydp,nzdp,bndp
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,lx,ly,lz,mask,level,isub)

#ifdef DEBUG_CG2
      s = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(X) = ',s
#endif

#ifdef DEBUG_CG2
      if (ip.EQ.0) WRITE(*,"('mmCg2 Zmaxpy2',i2)") level
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,' mmCg2 Zmaxpy2')") ip
#endif
#endif
      CALL mmZmaxpy2 (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,alpha,
     &     W,nxdw,nydw,nzdw,bndw
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,lx,ly,lz,mask,level,isub)
#ifdef DEBUG_CG2
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(R) = ',s
#endif

      DO i=1,n
         IF (mask(i)) THEN
            rdotPrev(i) = rdot(i)
         END IF
      END DO

#ifdef DEBUG_CG2
      if (ip.EQ.0) WRITE(*,"('mmCg2 DotProd2',i2)") level
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,' mmCg2 DotProd2-6')") ip
#endif
#endif
      CALL mmDotProd2 (rdot,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,lx,ly,lz,mask,level,
     &     isub)
#ifdef DEBUG_CG2
      if (ip.EQ.0) PRINT*, '  rdot(2) = ',rdot(2)
#endif

      DO i=1,n
         IF (mask(i)) THEN
            beta(i) = rdot(i)/rdotPrev(i)
         END IF
      END DO

*-------------------------------------------
*     Save best solution in B
*-------------------------------------------
*     IF (rdot.LT.rmin) THEN
*     rmin = rdot
*     DO iz=1,nz
*     DO iy=1,ny
*     DO ix=1,nx
*     B(ix,iy,iz)=X(ix,iy,iz)
*     END DO
*     END DO
*     END DO
*     END IF

#ifdef DEBUG_CG2
      if (ip.EQ.0) WRITE(*,"('mmCg2 Zaxpy2',i2)") level
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,' mmCg2 Zaxpy2-2')") ip
#endif
#endif
      CALL mmZaxpy2 (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,beta,
     &     P,nxdp,nydp,nzdp,bndp
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,lx,ly,lz,mask,level,isub)

#ifdef DEBUG_CG2
      s = mmVectorSum (
     &     P,nxdp,nydp,nzdp,bndp
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(P) = ',s
#endif

      iter = iter + 1
      numIterCg = numIterCg + 1

      IF (outputLevel.GT.1 .AND. ip.EQ.0) THEN
         DO i=1,n
            IF (mask(i)) THEN
               IF (DABS(rdotFirst(i)).GT.mmSmall2) THEN
                  print*, 'CG2: ',iter,i,DSQRT(rdot(i)/rdotFirst(i))
               ELSE
                  print*, 'CG2(r): ',iter,i,DSQRT(rdot(i))
               END IF
            END IF
         END DO
      END IF

      GO TO 10

*-----------------------------------------------------------------------
*     "Handle" apparent divergence
*-----------------------------------------------------------------------

 30   CONTINUE
      IF (ip.EQ.0) print*,
     &     '== mgmpi MESSAGE == aborting CG2: apparently diverging'
      istat = 1
      GO TO 20

 40   CONTINUE
      IF (ip.EQ.0) print*,
     &     '== mgmpi ERROR == aborting CG2: residual norm is NaN'
      istat = 1
      GO TO 20

 20   CONTINUE


*-------------------------------------------
*     Use saved best solution in B
*-------------------------------------------

*     IF (rdot.NE.rmin) THEN
*     DO iz=1,nz
*     DO iy=1,ny
*     DO ix=1,nx
*     X(ix,iy,iz)=B(ix,iy,iz)
*     END DO
*     END DO
*     END DO
*     END IF

C     Compute AVERAGE residual reduction in resRedFine
C     (assumes mask(2:n-1)=.TRUE. on input)

      resRedFine=0.0D0
      DO i=2,n-1
         rnorm(i)=DSQRT(rdot(i))
         resRedFine=resRedFine+rnorm(i)
      END DO
      resRedFine=resRedFine/(n-2)

#ifdef DUMP_INPUTS
      DO i=0,npx*npy*npz-1
         IF (ip.EQ.i) THEN
            print*, 'CG exit'
            print*, 'nxda,nyda,nzda=',nxda,nyda,nzda
            print*, 'nxdx,nydx,nzdx=',nxdx,nydx,nzdx
            print*, 'nxdb,nydb,nzdb=',nxdb,nydb,nzdb
            print*, 'nxdr,nydr,nzdr=',nxdr,nydr,nzdr
            print*, 'nxdp,nydp,nzdp=',nxdp,nydp,nzdp
            print*, 'nxdw,nydw,nzdw=',nxdw,nydw,nzdw
            print*, 'nx,ny,nz=',nx,ny,nz
            print*, 'nxg,nyg,nzg=',nxg,nyg,nzg
            print*, 'ip,ipx,ipy,ipz=',ip,ipx,ipy,ipz
            print*, 'npx,npy,npz=',npx,npy,npz
            print*, 'xp(2),xp(nx-1)=',xp(2),xp(nx-1)
            print*, 'yp(2),yp(ny-1)=',yp(2),yp(ny-1)
            print*, 'zp(2),zp(nz-1)=',zp(2),zp(nz-1)
            print*, 'hx(2),hx(nx-1)=',hx(2),hx(nx-1)
            print*, 'hy(2),hy(ny-1)=',hy(2),hy(ny-1)
            print*, 'hz(2),hz(nz-1)=',hz(2),hz(nz-1)
            print*, 'nxdbnd,nydbnd,nzdbnd=',nxdbnd,nydbnd,nzdbnd
            print*, 'ixpb,ixmb=',ixpb,ixmb
            print*, 'iypb,iymb=',iypb,iymb
            print*, 'izpb,izmb=',izpb,izmb
            print*, 'numIterCg=',numIterCg
            print*, 'maxIterCg=',maxIterCg
            print*, 'rtol=',rtol
            print*, 'outputLevel=',outputLevel
            print*, 'comm=',comm
            print*, 'nbr=',nbr
            print*, 'bnd=',bnd
            print*, 'scaleA=',scaleA
            print*, 'scaleB=',scaleB
            print*, 'lx=',lx
            print*, 'ly=',ly
            print*, 'lz=',lz
            print*, 'level=',level
            print*, 'rnorm=',rnorm(2),rnorm(3),rnorm(4)
            print*, 'resRedFine=',resRedFine
            print*, 'istat=',istat
         END IF
      END DO
#endif
      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmCheckParams 
C     
C     Purpose: Check some parameters for errors and output warnings or errors
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmCheckParams (
     &     nxg,nyg,nzg,m,
     &     ixmb,iymb,izmb,ixpb,iypb,izpb,comm)

      IMPLICIT NONE

*-----------------------------------------------------------------------
*
*----------------------------------------
*     The following conditions must hold:
*----------------------------------------
*
*     discretization
*     --------------
*     2^(m-1) evenly divides nxg-1
*     2^(m-1) evenly divides nyg-1
*     2^(m-1) evenly divides nzg-1
*
*     processor distribution
*     ----------------------
*     npx <= nxg(m)-1
*     npy <= nyg(m)-1
*     npz <= nzg(m)-1
*
*     number of processors
*     --------------------
*     npx*npy*npz=np
*
*     storage allocation
*     --------------------
*     nxg*nyg*nzg/np <= NMAXLIM*NMAXLIM*NMAXLIM
*     m <= MLIM
*
*------------------------------------------
*     The following conditions should hold:
*------------------------------------------
*
*     load balancing
*     --------------
*     npx evenly divides nxg(m)-1
*     npy evenly divides nyg(m)-1
*     npz evenly divides nzg(m)-1
*
*-----------------------------------------------------------------------

      INTEGER nxg,nyg,nzg,m
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb,comm

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      INTEGER ierrors,iwarnings,m2,nxm,nym,nzm,ierr

#ifdef PROC_TRACE
      IF (ip.EQ.0) print*, '>>> mmCheckParams'
#endif
      ierrors = 0
      iwarnings = 0
      m2 = 2**(m-1)

#ifdef MPI_USED
      ierr = 0
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,'mmCheckParams: MPI_Comm_rank')")  ip
#endif
      CALL MPI_Comm_rank (comm,ip,ierr)
      if (ierr.NE.0) print*, 'MPI Error in check_params: ',ierr
#else
      ip = 0
#endif

      IF (ip.eq.0) THEN

*-----------------------------------------------------------------------
*     Check validity of discretization sizes
*-----------------------------------------------------------------------

      IF (((nxg-1)/m2)*m2.NE.nxg-1) THEN
         print*,'== mgmpi ERROR == nxg-1 must be divisible by 2**(m-1)'
         print*, nxg-1,m2
         ierrors = ierrors + 1
      END IF
      IF (((nyg-1)/m2)*m2.NE.nyg-1) THEN
         print*,'== mgmpi ERROR == nyg-1 must be divisible by 2**(m-1)'
         print*, nyg-1,m2
         ierrors = ierrors + 1
      END IF
      IF (((nzg-1)/m2)*m2.NE.nzg-1) THEN
         print*,'== mgmpi ERROR == nzg-1 must be divisible by 2**(m-1)'
         print*, nzg-1,m2
         ierrors = ierrors + 1
      END IF

*-----------------------------------------------------------------------
*     Check validity of processor distribution
*-----------------------------------------------------------------------

      nxm = (nxg-1)/m2
      nym = (nyg-1)/m2
      nzm = (nzg-1)/m2

      IF (npx.GT.nxm) THEN
         print*,'== mgmpi ERROR == npx must be less than nxm'
         print*,'== mgmpi ERROR == ',m,npx,nxm
         ierrors = ierrors + 1
      END IF
      IF (npy.GT.nym) THEN
         print*,'== mgmpi ERROR == npy must be less than nyg(m)'
         print*,'== mgmpi ERROR == ',m,npy,nym
         ierrors = ierrors + 1
      END IF
      IF (npz.GT.nzm) THEN
         print*,'== mgmpi ERROR == npz must be less than nzg(m)'
         print*,'== mgmpi ERROR == ',m,npz,nzm
         ierrors = ierrors + 1
      END IF

*-----------------------------------------------------------------------
*     Check storage allocation
*-----------------------------------------------------------------------

      IF (m.GT.MLIM) THEN
         print*,'== mgmpi ERROR == MLIM must be increased from'
         print*,MLIM,' to ',m
         ierrors = ierrors + 1
      END IF

*-----------------------------------------------------------------------
*     Check for load balancing problems
*-----------------------------------------------------------------------

      IF ((nxm/npx)*npx.NE.nxm) THEN
         print*,'-- mgmpi warning -- possible load-balancing problem:'
         print*,'                    npx should evenly divide nxg(m)-1'
         iwarnings = iwarnings + 1
      END IF
      IF ((nym/npy)*npy.NE.nym) THEN
         print*,'-- mgmpi warning -- possible load-balancing problem:'
         print*,'                    npy should evenly divide nyg(m)-1'
         iwarnings = iwarnings + 1
      END IF
      IF ((nzm/npz)*npz.NE.nzm) THEN
         print*,'-- mgmpi warning -- possible load-balancing problem:'
         print*,'                    npz should evenly divide nzg(m)-1'
         iwarnings = iwarnings + 1
      END IF

*-----------------------------------------------------------------------
*     Check number of processors
*-----------------------------------------------------------------------

      ierr = 0
#ifdef MPI_USED
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,'mmCheckParams: MPI_Comm_size')")  ip
#endif
      CALL MPI_Comm_size (comm,np,ierr)
#else
      np = 1
#endif
      if (ierr.NE.0) print*, 'MPI Error in check_params: ',ierr
      IF (npx*npy*npz .NE. np) THEN
         print*,'== mgmpi ERROR == npx*npy*npz must equal np'
         ierrors = ierrors + 1
      END IF

*-----------------------------------------------------------------------
*     Check boundary types
*-----------------------------------------------------------------------

      IF (ipx.EQ.0 .AND. 
     &     (ixmb.LT.-2 .OR. ixmb.EQ.0 .OR. ixmb.GT.3)) THEN
         print*,'== mgmpi ERROR == Illegal boundary type ixmb=',ixmb
         ierrors = ierrors + 1
      END IF
      IF (ipx.EQ.npx-1 .AND. 
     &     (ixpb.LT.-2 .OR. ixpb.EQ.0 .OR. ixpb.GT.3)) THEN
         print*,'== mgmpi ERROR == Illegal boundary type ixpb=',ixpb
         ierrors = ierrors + 1
      END IF
      IF (ipy.EQ.0 .AND. 
     &     (iymb.LT.-2 .OR. iymb.EQ.0 .OR. iymb.GT.3)) THEN
         print*,'== mgmpi ERROR == Illegal boundary type iymb=',iymb
         ierrors = ierrors + 1
      END IF
      IF (ipy.EQ.npy-1 .AND. 
     &     (iypb.LT.-2 .OR. iypb.EQ.0 .OR. iypb.GT.3)) THEN
         print*,'== mgmpi ERROR == Illegal boundary type iypb=',iypb
         ierrors = ierrors + 1
      END IF
      IF (ipz.EQ.0 .AND. 
     &     (izmb.LT.-2 .OR. izmb.EQ.0 .OR. izmb.GT.3)) THEN
         print*,'== mgmpi ERROR == Illegal boundary type izmb=',izmb
         ierrors = ierrors + 1
      END IF
      IF (ipz.EQ.npz-1 .AND. 
     &     (izpb.LT.-2 .OR. izpb.EQ.0 .OR. izpb.GT.3)) THEN
         print*,'== mgmpi ERROR == Illegal boundary type izpb=',izpb
         ierrors = ierrors + 1
      END IF

*-----------------------------------------------------------------------
*     Summarize results and quit if any errors
*-----------------------------------------------------------------------

*C      print*
*C      print*, '== MGMPI == errors:',ierrors
*C      print*, '== MGMPI == warnings:',iwarnings
*C      print*

      END IF

#ifdef MPI_USED
      ierr = 0
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,'mmCheckParams: MPI_BCAST')")  ip
#endif
      CALL MPI_BCAST (ierrors,1,MPI_INTEGER,0,comm,ierr)
      if (ierr.NE.0) print*, 'MPI Error in check_params: ',ierr
#endif

      IF (ierrors.GT.0) THEN
         IF (ip.EQ.0) print*, 'Terminating because of errors'
#ifdef MPI_USED
         CALL mmTerminate (comm)
#else
         STOP
#endif
      END IF

#ifdef PROC_TRACE
      IF (ip.EQ.0) print*, '<<< mmCheckParams'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C
C     MODULE:  SUBROUTINE mmClearBoundary 
C
C     PURPOSE: Set nodes in the physical boundary to 0
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmClearBoundary (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      
#include "mgmpi_internal.h"

      INTEGER ix,iy,iz

*--------------------------
*     Lower X-boundary
*--------------------------

      IF (ipx.eq.0) THEN
         DO iz=1,nz
            DO iy=1,ny
               X(1,iy,iz)=(0.0D0)
            END DO
         END DO
      END IF

*--------------------------
*     Upper X-boundary
*--------------------------

      IF (ipx.eq.npx-1) THEN
         DO iz=1,nz
            DO iy=1,ny
               X(nx,iy,iz)=(0.0D0)
            END DO
         END DO
      END IF

*--------------------------
*     Lower Y-boundary
*--------------------------

      IF (ipy.eq.0) THEN
         DO iz=1,nz
            DO ix=1,nx
               X(ix,1,iz)=(0.0D0)
            END DO
         END DO
      END IF

*--------------------------
*     Upper Y-boundary
*--------------------------

      IF (ipy.eq.npy-1) THEN
         DO iz=1,nz
            DO ix=1,nx
               X(ix,ny,iz)=(0.0D0)
            END DO
         END DO
      END IF

*--------------------------
*     Lower Z-boundary
*--------------------------

      IF (ipz.eq.0) THEN
         DO iy=1,ny
            DO ix=1,nx
               X(ix,iy,1)=(0.0D0)
            END DO
         END DO
      END IF

*--------------------------
*     Upper Z-boundary
*--------------------------

      IF (ipz.eq.npz-1) THEN
         DO iy=1,ny
            DO ix=1,nx
               X(ix,iy,nz)=(0.0D0)
            END DO
         END DO
      END IF

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C
C     MODULE:  SUBROUTINE mmClearBoundary 
C
C     PURPOSE: Set nodes in the physical boundary to 0
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmClearBoundary2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,lx,ly,lz)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      LOGICAL lx,ly,lz
      
#include "mgmpi_internal.h"

      INTEGER ix,iy,iz

*--------------------------
*     Lower X-boundary
*--------------------------

      IF (ipx.eq.0.AND.lx) THEN
         DO iz=1,nz
            DO iy=1,ny
               X(1,iy,iz)=(0.0D0)
            END DO
         END DO
      END IF

*--------------------------
*     Upper X-boundary
*--------------------------

      IF (ipx.eq.npx-1.AND.lx) THEN
         DO iz=1,nz
            DO iy=1,ny
               X(nx,iy,iz)=(0.0D0)
            END DO
         END DO
      END IF

*--------------------------
*     Lower Y-boundary
*--------------------------

      IF (ipy.eq.0.AND.ly) THEN
         DO iz=1,nz
            DO ix=1,nx
               X(ix,1,iz)=(0.0D0)
            END DO
         END DO
      END IF

*--------------------------
*     Upper Y-boundary
*--------------------------

      IF (ipy.eq.npy-1.AND.ly) THEN
         DO iz=1,nz
            DO ix=1,nx
               X(ix,ny,iz)=(0.0D0)
            END DO
         END DO
      END IF

*--------------------------
*     Lower Z-boundary
*--------------------------

      IF (ipz.eq.0.AND.lz) THEN
         DO iy=1,ny
            DO ix=1,nx
               X(ix,iy,1)=(0.0D0)
            END DO
         END DO
      END IF

*--------------------------
*     Upper Z-boundary
*--------------------------

      IF (ipz.eq.npz-1.AND.lz) THEN
         DO iy=1,ny
            DO ix=1,nx
               X(ix,iy,nz)=(0.0D0)
            END DO
         END DO
      END IF

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C
C     MODULE:  mm<Coords>
C
C     PURPOSE: Compute transformations for coordinate different systems
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      REAL FUNCTION mmDxA (x1,x2)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      REAL x1,x2
      mmDxA = x2-x1
      RETURN
      END
      REAL FUNCTION mmDxC(z1,z2)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      REAL z1,z2
      mmDxC = z2-z1
      RETURN
      END
      REAL FUNCTION mmDxS(r1,r2)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      REAL r1,r2
*      mmDxS = (r2*r2*r2-r1*r1*r1)/3.0D0
#ifdef DOUBLE_PRECISION
      mmDxS = (r2-r1)*((r1+r2)**2-r1*r2)*0.3333333333333333D0
#else
      mmDxS = (r2-r1)*((r1+r2)**2-r1*r2)*0.33333333E0
#endif
      RETURN
      END
      REAL FUNCTION mmDyA(y1,y2) 
      IMPLICIT NONE
#include "mgmpi_internal.h"
      REAL y1,y2
      mmDyA = y2-y1
      RETURN
      END
      REAL FUNCTION mmDyC(r1,r2)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      REAL r1,r2
*      mmDyC = 0.5D0*(r2*r2-r1*r1)
#ifdef DOUBLE_PRECISION
      mmDyC = 0.5D0*(r2-r1)*(r2+r1)
#else
      mmDyC = 0.5E0*(r2-r1)*(r2+r1)
#endif      
      RETURN
      END
      REAL FUNCTION mmDyS(t1,t2)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      REAL t1,t2
#ifdef DOUBLE_PRECISION
      mmDyS = -DCOS(t2) + DCOS(t1)
#else
      mmDyS = -ECOS(t2) + ECOS(t1)
#endif
      RETURN
      END
      REAL FUNCTION mmDzA(z1,z2) 
      IMPLICIT NONE
#include "mgmpi_internal.h"
      REAL z1,z2
      mmDzA = z2-z1
      RETURN
      END
      REAL FUNCTION mmDzC(p1,p2)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      REAL p1,p2
      mmDzC = p2-p1
      RETURN
      END
      REAL FUNCTION mmDzS(p1,p2)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      REAL p1,p2
      mmDzS = p2-p1
      RETURN
      END
      REAL FUNCTION mmG2A(x)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      REAL x
      mmG2A = 1.0D0
      RETURN
      END
      REAL FUNCTION mmG2C(z)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      REAL z
      mmG2C = 1.0D0
      RETURN
      END
      REAL FUNCTION mmG2S(r)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      REAL r
      mmG2S = r
      RETURN
      END
      REAL FUNCTION mmG31A(x)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      REAL x
      mmG31A = 1.0D0
      RETURN
      END
      REAL FUNCTION mmG31C(z)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      REAL z
      mmG31C = 1.0D0
      RETURN
      END
      REAL FUNCTION mmG31S(r)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      REAL r
      mmG31S = r
      RETURN
      END
      REAL FUNCTION mmG32A(y)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      REAL y
      mmG32A = 1.0D0
      RETURN
      END
      REAL FUNCTION mmG32C(r)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      REAL r
      mmG32C = r
      RETURN
      END
      REAL FUNCTION mmG32S(t)
      IMPLICIT NONE
#include "mgmpi_internal.h"
      REAL t
      mmG32S = DSIN (t)
      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C
C     MODULE:  SUBROUTINE mmCorrect 
C
C     PURPOSE: Add one vector to another
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmCorrect (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)

      INTEGER nx,ny,nz

#ifdef MPI_USED
      include "mpif.h"
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

#include "mgmpi_internal.h"

      INTEGER ix,iy,iz

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_CORRECT
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      DO iz=2,nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               X(ix,iy,iz) = X(ix,iy,iz) + R(ix,iy,iz)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & (nx-2)*(ny-2)*(nz-2)
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C
C     MODULE:  SUBROUTINE mmCorrect2
C
C     PURPOSE: Add planar vectors
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmCorrect2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,lx,ly,lz,mask,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)

      INTEGER nx,ny,nz
      LOGICAL lx,ly,lz,mask(*)

#ifdef MPI_USED
      include "mpif.h"
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

#include "mgmpi_internal.h"

      INTEGER ix,iy,iz

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_CORRECT
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      IF (.NOT.lx) THEN
         DO ix=2,nx-1
            IF (mask(ix)) THEN
               DO iz=2,nz-1
                  DO iy=2,ny-1
                     X(ix,iy,iz) = X(ix,iy,iz) + R(ix,iy,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & (ny-2)*(nz-2)
#endif
            END IF
         END DO
      ELSE IF (.NOT.lz) THEN
         DO iy=2,ny-1
            IF (mask(iy)) THEN
               DO iz=2,nz-1
                  DO ix=2,nx-1
                     X(ix,iy,iz) = X(ix,iy,iz) + R(ix,iy,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & (nx-2)*(nz-2)
#endif
            END IF
         END DO
      ELSE IF (.NOT.lz) THEN
         DO iz=2,nz-1
            IF (mask(iz)) THEN
               DO iy=2,ny-1
                  DO ix=2,ny-1
                     X(ix,iy,iz) = X(ix,iy,iz) + R(ix,iy,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (nx-2)*(ny-2)
      gmflops(isub)=gmflops(isub) + 
     & (nx-2)*(ny-2)
#endif
            END IF
         END DO
      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C
C     MODULE:  SUBROUTINE mmDirichletBC 
C
C     PURPOSE: Enforce Dirichlet boundary conditions for a vector
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmDirichletBC (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,scaleA,scaleB,level,IsHomogenious)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz,level
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      REAL scaleA, scaleB
      LOGICAL IsHomogenious

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz,ixf,iyf,izf,istride
      REAL scale,sxm,sxp,sym,syp,szm,szp

      scale = scaleB / scaleA

      istride = 2**(level-1)

      IF (ixmb.EQ.MM_DIRICHLET) sxm = scale
      IF (ixmb.EQ.-MM_DIRICHLET) sxm = -scale
      IF (ixpb.EQ.MM_DIRICHLET) sxp = scale
      IF (ixpb.EQ.-MM_DIRICHLET) sxp = -scale
      IF (iymb.EQ.MM_DIRICHLET) sym = scale
      IF (iymb.EQ.-MM_DIRICHLET) sym = -scale
      IF (iypb.EQ.MM_DIRICHLET) syp = scale
      IF (iypb.EQ.-MM_DIRICHLET) syp = -scale
      IF (izmb.EQ.MM_DIRICHLET) szm = scale
      IF (izmb.EQ.-MM_DIRICHLET) szm = -scale
      IF (izpb.EQ.MM_DIRICHLET) szp = scale
      IF (izpb.EQ.-MM_DIRICHLET) szp = -scale

*----------------------------------------
*     Lower X-boundary Dirichlet BC's
*----------------------------------------

      IF (ipx.EQ.0) THEN
         IF (ABS(ixmb).EQ.MM_DIRICHLET .AND. isHomogenious) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(1,iy,iz) = 0.0D0
               END DO
            END DO
         ELSE IF (abs(ixmb).EQ.MM_DIRICHLET) THEN
            IF (level.EQ.1) THEN
               DO iz=1,nz
                  DO iy=1,ny
                     X(1,iy,iz) = sxm*XMB (iy,iz)
                  END DO
               END DO
            ELSE
               DO iz=1,nz
                  izf = istride*(iz-1)+1
                  DO iy=1,ny
                     iyf = istride*(iy-1)+1
                     X(1,iy,iz) = sxm*XMB (iyf,izf)
                  END DO
               END DO
            END IF
         END IF
      END IF

*----------------------------------------
*     Upper X-boundary Dirichlet BC's
*----------------------------------------

      IF (ipx.EQ.npx-1) THEN
         IF (ABS(ixpb).EQ.MM_DIRICHLET .AND. isHomogenious) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(nx,iy,iz) = 0.0D0
               END DO
            END DO
         ELSE IF (abs(ixpb).EQ.MM_DIRICHLET) THEN
            IF (level.EQ.1) THEN
               DO iz=1,nz
                  DO iy=1,ny
                     X(nx,iy,iz) = sxp*XPB (iy,iz)
                  END DO
               END DO
            ELSE
               DO iz=1,nz
                  izf = istride*(iz-1)+1
                  DO iy=1,ny
                     iyf = istride*(iy-1)+1
                     X(nx,iy,iz) = sxp*XPB (iyf,izf)
                  END DO
               END DO
            END IF
         END IF
      END IF

*----------------------------------------
*     Lower Y-boundary Dirichlet BC's
*----------------------------------------

      IF (ipy.EQ.0) THEN
         IF (ABS(iymb).EQ.MM_DIRICHLET .AND. isHomogenious) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,1,iz) = 0.0D0
               END DO
            END DO
         ELSE IF (abs(iymb).EQ.MM_DIRICHLET) THEN
            IF (level.EQ.1) THEN
               DO iz=1,nz
                  DO ix=1,nx
                     X(ix,1,iz) = sym*YMB (ix,iz)
                  END DO
               END DO
            ELSE
               DO iz=1,nz
                  izf = istride*(iz-1)+1
                  DO ix=1,nx
                     ixf = istride*(ix-1)+1
                     X(ix,1,iz) = sym*YMB (ixf,izf)
                  END DO
               END DO
            END IF
         END IF
      END IF

*----------------------------------------
*     Upper Y-boundary Dirichlet BC's
*----------------------------------------

      IF (ipy.EQ.npy-1) THEN
         IF (ABS(iypb).EQ.MM_DIRICHLET .AND. isHomogenious) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,ny,iz) = 0.0D0
               END DO
            END DO
         ELSE IF (abs(iypb).EQ.MM_DIRICHLET) THEN
            IF (level.EQ.1) THEN
               DO iz=1,nz
                  DO ix=1,nx
                     X(ix,ny,iz) = syp*YPB (ix,iz)
                  END DO
               END DO
            ELSE
               DO iz=1,nz
                  izf = istride*(iz-1)+1
                  DO ix=1,nx
                     ixf = istride*(ix-1)+1
                     X(ix,ny,iz) = syp*YPB (ixf,izf)
                  END DO
               END DO
            END IF
         END IF
      END IF

*----------------------------------------
*     Lower Z-boundary Dirichlet BC's
*----------------------------------------

      IF (ipz.EQ.0) THEN
         IF (ABS(izmb).EQ.MM_DIRICHLET .AND. isHomogenious) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,1) = 0.0D0
               END DO
            END DO
         ELSE IF (abs(izmb).EQ.MM_DIRICHLET) THEN
            IF (level.EQ.1) THEN
               DO iy=1,ny
                  DO ix=1,nx
                     X(ix,iy,1) = szm*ZMB (ix,iy)
                  END DO
               END DO
            ELSE
               DO iy=1,ny
                  iyf = istride*(iy-1)+1
                  DO ix=1,nx
                     ixf = istride*(ix-1)+1
                     X(ix,iy,1) = szm*ZMB (ixf,iyf)
                  END DO
               END DO
            END IF
         END IF
      END IF

*----------------------------------------
*     Upper Z-boundary Dirichlet BC's
*----------------------------------------

      IF (ipz.EQ.npz-1) THEN
         IF (ABS(izpb).EQ.MM_DIRICHLET .AND. isHomogenious) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,nz) = 0.0D0
               END DO
            END DO
         ELSE IF (abs(izpb).EQ.MM_DIRICHLET) THEN
            IF (level.EQ.1) THEN
               DO iy=1,ny
                  DO ix=1,nx
                     X(ix,iy,nz) = szp*ZPB (ix,iy)
                  END DO
               END DO
            ELSE
               DO iy=1,ny
                  iyf = istride*(iy-1)+1
                  DO ix=1,nx
                     ixf = istride*(ix-1)+1
                     X(ix,iy,nz) = szp*ZPB (ixf,iyf)
                  END DO
               END DO
            END IF
         END IF
      END IF
      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmDirichletBC 
C     
C     PURPOSE: Enforce Dirichlet boundary conditions for a vector
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmDirichletBC2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,scaleA,scaleB,lx,ly,lz,level,
     &     isHomogenious)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz,level
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      REAL scaleA, scaleB
      LOGICAL lx,ly,lz,isHomogenious

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz
      REAL scale

      scale = scaleB / scaleA

*----------------------------------------
*     Lower X-boundary Dirichlet BC's
*----------------------------------------

      IF (lx.AND.ipx.EQ.0) THEN
         IF (ABS(ixmb).EQ.MM_DIRICHLET .AND. isHomogenious) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(1,iy,iz) = 0.0D0
               END DO
            END DO
         ELSE IF (ixmb.EQ.MM_DIRICHLET) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(1,iy,iz) = scale*XMB (iy,iz)
               END DO
            END DO
         ELSE IF (ixmb.EQ.-MM_DIRICHLET) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(1,iy,iz) = -scale*XMB (iy,iz)
               END DO
            END DO
         END IF
      END IF

*----------------------------------------
*     Upper X-boundary Dirichlet BC's
*----------------------------------------

      IF (lx.AND.ipx.EQ.npx-1) THEN
         IF (ABS(ixpb).EQ.MM_DIRICHLET .AND. isHomogenious) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(nx,iy,iz) = 0.0D0
               END DO
            END DO
         ELSE IF (ixpb.EQ.MM_DIRICHLET) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(nx,iy,iz) = scale*XPB (iy,iz)
               END DO
            END DO
         ELSE IF (ixpb.EQ.-MM_DIRICHLET) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(nx,iy,iz) = -scale*XPB (iy,iz)
               END DO
            END DO
         END IF
      END IF

*----------------------------------------
*     Lower Y-boundary Dirichlet BC's
*----------------------------------------

      IF (ly.AND.ipy.EQ.0) THEN
         IF (ABS(iymb).EQ.MM_DIRICHLET .AND. isHomogenious) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,1,iz) = 0.0D0
               END DO
            END DO
         ELSE IF (iymb.EQ.MM_DIRICHLET) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,1,iz) = scale*YMB (ix,iz)
               END DO
            END DO
         ELSE IF (iymb.EQ.-MM_DIRICHLET) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,1,iz) = -scale*YMB (ix,iz)
               END DO
            END DO
         END IF
      END IF

*----------------------------------------
*     Upper Y-boundary Dirichlet BC's
*----------------------------------------

      IF (ly.AND.ipy.EQ.npy-1) THEN
         IF (ABS(iypb).EQ.MM_DIRICHLET .AND. isHomogenious) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,ny,iz) = 0.0D0
               END DO
            END DO
         ELSE IF (iypb.EQ.MM_DIRICHLET) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,ny,iz) = scale*YPB (ix,iz)
               END DO
            END DO
         ELSE IF (iypb.EQ.-MM_DIRICHLET) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,ny,iz) = -scale*YPB (ix,iz)
               END DO
            END DO
         END IF
      END IF

*----------------------------------------
*     Lower Z-boundary Dirichlet BC's
*----------------------------------------

      IF (lz.AND.ipz.EQ.0) THEN
         IF (ABS(izmb).EQ.MM_DIRICHLET .AND. isHomogenious) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,1) = 0.0D0
               END DO
            END DO
         ELSE IF (izmb.EQ.MM_DIRICHLET) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,1) = scale*ZMB (ix,iy)
               END DO
            END DO
         ELSE IF (izmb.EQ.-MM_DIRICHLET) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,1) = -scale*ZMB (ix,iy)
               END DO
            END DO
         END IF
      END IF

*----------------------------------------
*     Upper Z-boundary Dirichlet BC's
*----------------------------------------

      IF (lz.AND.ipz.EQ.npz-1) THEN
         IF (ABS(izpb).EQ.MM_DIRICHLET .AND. isHomogenious) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,nz) = 0.0D0
               END DO
            END DO
         ELSE IF (izpb.EQ.MM_DIRICHLET) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,nz) = scale*ZPB (ix,iy)
               END DO
            END DO
         ELSE IF (izpb.EQ.-MM_DIRICHLET) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,nz) = -scale*ZPB (ix,iy)
               END DO
            END DO
         END IF
      END IF

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmDistribute 
C     
c     Purpose: Determine values defining the distribution of computation
c     Purpose: across processors
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C        1999 / Jul 6 -- Added support for semicoarsening: lc1,lc2,lc3
C               Aug 7 -- Changed lc? to gmmlc?
C               Sep 9 -- Changed i?1(k) to gmmi?(k,ip)
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmDistribute (
     &     nxg,nyg,nzg,nxd2,nyd2,nzd2,
     &     nxd,nyd,nzd,nx,ny,nz,m,
     &     indh,inda,indx
     &     ,
     &     comm,nbr,bnd,bnd2
     &     ,icall)

      IMPLICIT NONE

      INTEGER m
      INTEGER nxd2,nyd2,nzd2,nxg(m),nyg(m),nzg(m),nxd(m),nyd(m),nzd(m)
      INTEGER nx(m),ny(m),nz(m)
      INTEGER indh(*),inda(*),indx(*)
      
      INTEGER comm,nbr(6),bnd(3,*),bnd2(3)

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

*=======================================================================
*     INPUT
*     
*     nxg(1),nyg(1),nzg(1)           Global fine grid size
*     nx(1),ny(1),nz(1)              Local fine grid size
*     ipx,ipy,ipz,npx,npy,npz        Processor id and array size
*     comm,nbr                       Communicator and neighboring procs
*     m                              Multigrid levels
*     
*     OUTPUT
*     
*     nxd(1:m),nyd(1:m),nzd(1:m)     Global grid sizes
*     nxg(2:m),nyg(2:m),nzg(2:m)     Global coarse grid sizes
*     nx(2:m),ny(2:m),nz(2:m)        Local coarse grid sizes 
*     gmmix(1:m,1:npx),gmmiy(1:m,1:npy),gmmiz(1:m,1:npz)     
*                                    Local--global -1 mappings
*     bnd,bnd2                       Face data types (bnd2 for fine matrix)
*     indh, indx, inda               Offsets for mesh widths, vectors, 
*                                    matrix off-diagonals, matrix diag.
*=======================================================================

      INTEGER i,j,k,ierr,iq(3),ndim(3),nsubdim(3),nsubstart(3)
      LOGICAL DIMS(3)

      LOGICAL isFirstCall
      DATA isFirstCall/.TRUE./

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_DISTRIBUTE
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

*-----------------------------------------------------------------------
*     Find global <--> local grid -1 mapping
*-----------------------------------------------------------------------

      CALL mmGetGlobalIndices (gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
     &     nx,ny,nz,
     &     comm,nbr,bnd
     &     )

*-----------------------------------------------------------------------
*     Determine coarse grid sizes
*-----------------------------------------------------------------------

      DO k=2,m

         IF (gmmlc1) THEN
            gmmix(k,ipx) = gmmix(k-1,ipx) / 2
            nxg(k)=(nxg(k-1)-1) / 2 + 1
            IF (ipx.LT.npx-1) nx(k) = (nx(k-1)-2) / 2 + 2
            IF (ipx.EQ.npx-1) nx(k) = (nx(k-1)-1) / 2 + 1
         ELSE
            gmmix(k,ipx) = gmmix(k-1,ipx)
            nxg(k) = nxg(k-1)
            nx(k) = nx(k-1)
         END IF

         IF (gmmlc2) THEN
            gmmiy(k,ipy) = gmmiy(k-1,ipy) / 2
            nyg(k)=(nyg(k-1)-1) / 2 + 1
            IF (ipy.LT.npy-1) ny(k) = (ny(k-1)-2) / 2 + 2
            IF (ipy.EQ.npy-1) ny(k) = (ny(k-1)-1) / 2 + 1
         ELSE
            gmmiy(k,ipy) = gmmiy(k-1,ipy)
            nyg(k) = nyg(k-1)
            ny(k) = ny(k-1)
         END IF

         IF (gmmlc3) THEN
            gmmiz(k,ipz) = gmmiz(k-1,ipz) / 2
            nzg(k)=(nzg(k-1)-1) / 2 + 1
            IF (ipz.LT.npz-1) nz(k) = (nz(k-1)-2) / 2 + 2
            IF (ipz.EQ.npz-1) nz(k) = (nz(k-1)-1) / 2 + 1
         ELSE
            gmmiz(k,ipz) = gmmiz(k-1,ipz)
            nzg(k) = nzg(k-1)
            nz(k) = nz(k-1)
         END IF

      END DO

#ifdef MPI_USED
      DO i=0,npx-1
         CALL MPI_BCAST (gmmix(1,i),m,MPI_INTEGER,i,comm,ierr)
      END DO
      DO i=0,npy-1
         CALL MPI_BCAST (gmmiy(1,i),m,MPI_INTEGER,i,comm,ierr)
      END DO
      DO i=0,npz-1
         CALL MPI_BCAST (gmmiz(1,i),m,MPI_INTEGER,i,comm,ierr)
      END DO
#endif

      DO k=1,m
         gmmix(k,npx) = nxg(k)-2
         gmmiy(k,npy) = nyg(k)-2
         gmmiz(k,npz) = nzg(k)-2
      END DO

*-----------------------------------------------------------------------
*     Processor's array sizes (nxd,nyd,nzd)--needed for MPI data types
*-----------------------------------------------------------------------

      DO k=1,m

#ifdef MPI_USED
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,'mmDistribute: MPI_AllReduce')")  ip
#endif
         nxd(k)=nx(k) ! (in case allreduce is not called for one processor)
         CALL MPI_AllReduce (nx(k),nxd(k),
     &        1,MPI_INTEGER,MPI_MAX,comm,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmDistribute: ',ierr
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,'mmDistribute: MPI_AllReduce')")  ip
#endif
         nyd(k) = ny(k)  ! (in case allreduce is not called for one processor)
         CALL MPI_AllReduce (ny(k),nyd(k),
     &        1,MPI_INTEGER,MPI_MAX,comm,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmDistribute: ',ierr
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,'mmDistribute: MPI_AllReduce')")  ip
#endif
         nzd(k) = nz(k)  ! (in case allreduce is not called for one processor)
         CALL MPI_AllReduce (nz(k),nzd(k),
     &        1,MPI_INTEGER,MPI_MAX,comm,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmDistribute: ',ierr
#else
         nxd(k) = nx(k)
         nyd(k) = ny(k)
         nzd(k) = nz(k)
#endif
      END DO

*-----------------------------------------------------------------------
*     Define local sub-grid communicators gsubcomm (first call only)
*-----------------------------------------------------------------------

#ifdef MPI_USED
      
      IF (isFirstCall) THEN
         gsubcomm (1,0,0) = MPI_COMM_NULL
         gsubcomm (0,1,0) = MPI_COMM_NULL
         gsubcomm (0,0,1) = MPI_COMM_NULL

         dims(1)=.FALSE.
         dims(2)=.TRUE.
         dims(3)=.TRUE.
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,'mmDistribute: MPI_Cart_sub')")  ip
#endif
         CALL MPI_Cart_sub (comm, dims, gsubcomm (0,1,1),ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmDistribute: ',ierr
         dims(1)=.TRUE.
         dims(2)=.FALSE.
         dims(3)=.TRUE.
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,'mmDistribute: MPI_Cart_sub')")  ip
#endif
         CALL MPI_Cart_sub (comm, dims, gsubcomm (1,0,1),ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmDistribute: ',ierr
         dims(1)=.TRUE.
         dims(2)=.TRUE.
         dims(3)=.FALSE.
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,'mmDistribute: MPI_Cart_sub')")  ip
#endif
         CALL MPI_Cart_sub (comm, dims, gsubcomm (1,1,0),ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmDistribute: ',ierr

*-----------------------------------------------------------------------
*     Define local sub-grid neighbors gsubnbr (first call only)
*-----------------------------------------------------------------------

C     X-direction neighbors

         gsubnbr(0,1,1,XMD) = MPI_PROC_NULL
         gsubnbr(0,1,1,XPD) = MPI_PROC_NULL
         CALL MPI_Cart_shift (gsubcomm(1,0,1),0,1,
     &        gsubnbr(1,0,1,XMD),gsubnbr(1,0,1,XPD),ierr)
         IF (ierr.NE.0) print*, 'MPI ierr=',ierr
         CALL MPI_Cart_shift (gsubcomm(1,1,0),0,1,
     &        gsubnbr(1,1,0,XMD),gsubnbr(1,1,0,XPD),ierr)
         IF (ierr.NE.0) print*, 'MPI ierr=',ierr

C     Y-direction neighbors

         CALL MPI_Cart_shift (gsubcomm(0,1,1),0,1,
     &        gsubnbr(0,1,1,YMD),gsubnbr(0,1,1,YPD),ierr)
         IF (ierr.NE.0) print*, 'MPI ierr=',ierr
         gsubnbr(1,0,1,YMD) = MPI_PROC_NULL
         gsubnbr(1,0,1,YPD) = MPI_PROC_NULL
         CALL MPI_Cart_shift (gsubcomm(1,1,0),1,1,
     &        gsubnbr(1,1,0,YMD),gsubnbr(1,1,0,YPD),ierr)
         IF (ierr.NE.0) print*, 'MPI ierr=',ierr

C     Z-direction neighbors

         CALL MPI_Cart_shift (gsubcomm(0,1,1),1,1,
     &        gsubnbr(0,1,1,ZMD),gsubnbr(0,1,1,ZPD),ierr)
         IF (ierr.NE.0) print*, 'MPI ierr=',ierr
         CALL MPI_Cart_shift (gsubcomm(1,0,1),1,1,
     &        gsubnbr(1,0,1,ZMD),gsubnbr(1,0,1,ZPD),ierr)
         IF (ierr.NE.0) print*, 'MPI ierr=',ierr
         gsubnbr(1,1,0,ZMD) = MPI_PROC_NULL
         gsubnbr(1,1,0,ZPD) = MPI_PROC_NULL

      END IF
#endif


#ifdef MPI_USED

      IF (isFirstCall) THEN

*-----------------------------------------------------------------------
*     Define local k-grid face data-types (first call only)
*-----------------------------------------------------------------------

         DO k=1,m

*     ------------------------------
*     MGMPI vector X-faces
*     ------------------------------

#ifdef MPI_TRACE
            WRITE(*,"(1x,i3,'mmDistribute: MPI_Type_vector')")  ip
#endif
            CALL MPI_Type_vector (nyd(k)*nzd(k),1,nxd(k),
     &           MM_REAL,bnd(XD,k),ierr)
            IF (ierr.NE.0) print*, 'MPI Error in mmDistribute: ',ierr
            CALL MPI_Type_commit (bnd(XD,k),ierr)

*     ------------------------------
*     MGMPI vector Y-faces
*     ------------------------------

#ifdef MPI_TRACE
            WRITE(*,"(1x,i3,'mmDistribute: MPI_Type_vector')")  ip
#endif
            CALL MPI_Type_vector (nzd(k),nxd(k),nxd(k)*nyd(k),
     &           MM_REAL,bnd(YD,k),ierr)
            IF (ierr.NE.0) print*, 'MPI Error in mmDistribute: ',ierr
#ifdef MPI_TRACE
            WRITE(*,"(1x,i3,'mmDistribute: MPI_Type_commit')")  ip
#endif
            CALL MPI_Type_commit (bnd(YD,k),ierr)
            IF (ierr.NE.0) print*, 'MPI Error in mmDistribute: ',ierr

*     ------------------------------
*     MGMPI vector Z-faces
*     ------------------------------

#ifdef MPI_TRACE
            WRITE(*,"(1x,i3,'mmDistribute: MPI_Type_contiguous')")  ip
#endif
            CALL MPI_Type_contiguous (nxd(k)*nyd(k),
     &           MM_REAL,bnd(ZD,k),ierr)
            IF (ierr.NE.0) print*, 'MPI Error in mmDistribute: ',ierr
#ifdef MPI_TRACE
            WRITE(*,"(1x,i3,'mmDistribute: MPI_Type_commit')")  ip
#endif
            CALL MPI_Type_commit (bnd(ZD,k),ierr)
            IF (ierr.NE.0) print*, 'MPI Error in mmDistribute: ',ierr

         END DO

*     ------------------------------
*     Input vector fine X-faces
*     ------------------------------
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,'mmDistribute: MPI_Type_vector')")  ip
#endif
         CALL MPI_Type_vector (nyd2*nzd2,1,nxd2,
     &        MM_REAL,bnd2(XD),ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmDistribute: ',ierr
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,'mmDistribute: MPI_Type_commit')")  ip
#endif
         CALL MPI_Type_commit (bnd2(XD),ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmDistribute: ',ierr
         
*     ------------------------------
*     Input vector fine Y-faces
*     ------------------------------
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmDistribute: MPI_Type_vector')")  ip
#endif
         CALL MPI_Type_vector (nzd2,nxd2,nxd2*nyd2,
     &        MM_REAL,bnd2(YD),ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmDistribute: ',ierr
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmDistribute: MPI_Type_commit')")  ip
#endif
         CALL MPI_Type_commit (bnd2(YD),ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmDistribute: ',ierr

*     ------------------------------
*     Input vector fine Z-faces
*     ------------------------------
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmDistribute: MPI_Type_contiguous')")  ip
#endif
         CALL MPI_Type_contiguous (nxd2*nyd2,
     &        MM_REAL,bnd2(ZD),ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmDistribute: ',ierr
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmDistribute: MPI_Type_commit')")  ip
#endif
         CALL MPI_Type_commit (bnd2(ZD),ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmDistribute: ',ierr

C     ----------------------------------------------
C     Store datatypes in global for subsequent calls
C     ----------------------------------------------

         DO k=1,m
            gbnd(XD,k) = bnd(XD,k)
            gbnd(YD,k) = bnd(YD,k)
            gbnd(ZD,k) = bnd(ZD,k)
         END DO
         gbnd2(XD) = bnd2(XD)
         gbnd2(YD) = bnd2(YD)
         gbnd2(ZD) = bnd2(ZD)
         
      ELSE 
         
C     -----------------------------------------------
C     Restore datatypes from global from initial call
C     -----------------------------------------------

         DO k=1,m
            bnd(XD,k) = gbnd (XD,k)
            bnd(YD,k) = gbnd (YD,k)
            bnd(ZD,k) = gbnd (ZD,k)
         END DO

         bnd2(XD) = gbnd2(XD)
         bnd2(YD) = gbnd2(YD)
         bnd2(ZD) = gbnd2(ZD)

      END IF

      IF (isFirstCall.AND.gmmIsCollapsed) THEN

*-----------------------------------------------------------------------
*     Define entire grid data types (first call only) (coarsest-level only)
*-----------------------------------------------------------------------

         
C-------------------------------------
C     Distributed subvectors gmmgrid
C-------------------------------------

         ndim(1) = nxd(m)
         ndim(2) = nyd(m)
         ndim(3) = nzd(m)
         nsubdim(1) = gmmix(m,ipx+1) - gmmix(m,ipx)
         nsubdim(2) = gmmiy(m,ipy+1) - gmmiy(m,ipy)
         nsubdim(3) = gmmiz(m,ipz+1) - gmmiz(m,ipz)
#ifdef ASSERT
         IF (nsubdim(1).NE.nx(m)-2) THEN
            PRINT*, 'mmDistribute ASSERTION x FAILED!'
            PRINT*, nsubdim(1),nx(m)-2
            ierr = -1
         END IF
         IF (nsubdim(2).NE.ny(m)-2) THEN
            PRINT*, 'mmDistribute ASSERTION y FAILED!'
            PRINT*, nsubdim(2),ny(m)-2
            ierr = -1
         END IF
         IF (nsubdim(3).NE.nz(m)-2) THEN
            PRINT*, 'mmDistribute ASSERTION z FAILED!'
            PRINT*, nsubdim(3),nz(m)-2
            ierr = -1
         END IF
         IF (ierr.EQ.-1) CALL mmTerminate(comm)
#endif
         nsubstart(1) = 2
         nsubstart(2) = 2
         nsubstart(3) = 2
         CALL MPI_Type_Create_Subarray (3,ndim,nsubdim,nsubstart,
     &        MPI_ORDER_FORTRAN,MM_REAL,gmmgrid(m),ierr)

C-------------------------------------
C     Collapsed subvectors gmmgrid
C-------------------------------------

         IF (ip.EQ.0) THEN
            DO j=0, np-1
               ndim(1) = nxg(m)
               ndim(2) = nyg(m)
               ndim(3) = nzg(m)
               CALL MPI_Cart_Coords (comm, j, 3, iq, ierr)
               nsubdim(1) = gmmix(m,iq(1)+1) - gmmix(m,iq(1))
               nsubdim(2) = gmmiy(m,iq(2)+1) - gmmiy(m,iq(2))
               nsubdim(3) = gmmiz(m,iq(3)+1) - gmmiz(m,iq(3))
               nsubstart(1) = gmmix(m,iq(1)) + 2
               nsubstart(2) = gmmiy(m,iq(1)) + 2
               nsubstart(3) = gmmiz(m,iq(1)) + 2
               CALL MPI_Type_Create_Subarray (3,ndim,nsubdim,nsubstart,
     &              MPI_ORDER_FORTRAN,MM_REAL,gmmcgrid(m,j),ierr)
            END DO
         END IF
      END IF

#endif

*-----------------------------------------------------------------------
*     Determine augmented array offsets for grid levels
*-----------------------------------------------------------------------

      indh(1) = 1
      indx(1) = 1
      DO k=2,m
         indh(k)=indh(k-1) +   max(max(nxd(k-1),nyd(k-1)),nzd(k-1))
         indx(k)=indx(k-1) +   nxd(k-1)*nyd(k-1)*nzd(k-1)
      END DO

      inda(1) = -1
      inda(2) = 1
      DO k=3,m
         inda(k) =inda(k-1)  +  nxd(k-1)*nyd(k-1)*nzd(k-1)
      END DO

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      isFirstCall = .FALSE.

      RETURN
      END
C=======================================================================
C     
C     Module:  REAL FUNCTION mmDotProd
C     
C     Purpose: Compute the inner-product of two vectors
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      REAL FUNCTION mmDotProd (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,nx,ny,nz,comm,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      INTEGER nx,ny,nz,comm,level

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      INTEGER ix,iy,iz,ierr
      REAL a,b

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_DOTPROD+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif
      
*-----------------------------------------------------------------------

      a=(0.0D0)
      DO iz=2,nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               a = a + X(ix,iy,iz) * Y(ix,iy,iz)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif

*-----------------------------------------------------------------------
*     Compute global inner-product
*-----------------------------------------------------------------------

#ifdef MPI_USED
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmDotProd: MPI_Allreduce')")  -1
#endif
      b = a ! (in case allreduce is not called for one processor)
      CALL MPI_Allreduce(a,b,1,MM_REAL,MPI_SUM,comm,ierr)
      IF (ierr.NE.0) print*, 'MPI Error in mmDotProd: ',ierr
      mmDotProd = b
#else
      mmDotProd = a
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C
C     MODULE:  REAL FUNCTION mmDotProd2
C
C     PURPOSE: Compute the inner-product of multiple planar (2D) vectors
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmDotProd2 (b,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,nx,ny,nz,comm,lx,ly,lz,mask,level,
     &     icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      INTEGER nx,ny,nz,comm,level
      LOGICAL lx,ly,lz,mask(*)

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      INTEGER ix,iy,iz,ierr
      REAL a,b(*)

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_DOTPROD2+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

*-----------------------------------------------------------------------
*     Compute sequence of 2-D inner-products
*-----------------------------------------------------------------------

      IF (.NOT.lx) THEN
         DO ix = 2,nx-1
            IF (mask(ix)) THEN
               a=(0.0D0)
               DO iz=2,nz-1
                  DO iy=2,ny-1
                     a = a + X(ix,iy,iz) * Y(ix,iy,iz)
                  END DO
               END DO

               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(ny-2)*(nz-2)
#endif

#ifdef MPI_USED
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmDotProd2: MPI_Allreduce')")  -1
#endif
               b(ix)=a ! (in case allreduce is not called for one processor) 
               CALL MPI_Allreduce(a,b(ix),1,MM_REAL,MPI_SUM,
     &              gsubcomm(0,1,1),ierr)
#else 
               b(ix)=a
#endif
            END IF
         END DO
      ELSE IF (.NOT.ly) THEN
         DO iy = 2,ny-1
            IF (mask(iy)) THEN
               a=(0.0D0)
               DO iz=2,nz-1
                  DO ix=2,nx-1
                     a = a + X(ix,iy,iz) * Y(ix,iy,iz)
                  END DO
               END DO

               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(nz-2)
#endif

#ifdef MPI_USED
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmDotProd2: MPI_Allreduce')")  -1
#endif
               b(iy)=a ! (in case allreduce is not called for one processor)
               CALL MPI_Allreduce(a,b(iy),1,MM_REAL,MPI_SUM,
     &              gsubcomm(1,0,1),ierr)
#else
               b(iy)=a
#endif
            END IF
         END DO
      ELSE IF (.NOT.lz) THEN
         DO iz = 2,nz-1
            IF (mask(iz)) THEN
               a=(0.0D0)
               DO iy=2,ny-1
                  DO ix=2,nx-1
                     a = a + X(ix,iy,iz) * Y(ix,iy,iz)
                  END DO
               END DO

               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(ny-2)*(nx-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(ny-2)*(nx-2)
#endif

#ifdef MPI_USED
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmDotProd2: MPI_Allreduce')")  -1
#endif
               b(iz)=a ! (in case allreduce is not called for one processor)
               CALL MPI_Allreduce(a,b(iz),1,MM_REAL,MPI_SUM,
     &              gsubcomm(1,1,0),ierr)
#else
               b(iz)=a
#endif
            END IF
         END DO
      ELSE
         PRINT*, 'MPI Error in mmDotProd2: lx,ly,lz=',lx,ly,lz
         CALL mmTerminate(comm)
      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmGetCoarseMeshPoints 
C     
C     Purpose: Compute the mesh point locations for a coarse mesh given
C     Purpose: ones fore the next finer level
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmGetCoarseMeshPoints (
     &     xpf,ypf,zpf,xpc,ypc,zpc,
     &     nxf,nyf,nzf,nxc,nyc,nzc,
     &     comm,nbr,bnd
     &     , level)

      IMPLICIT NONE
      
      
      INTEGER comm,nbr(6),bnd(3)

      REAL xpf(*),ypf(*),zpf(*)
      REAL xpc(*),ypc(*),zpc(*)
      INTEGER nxf,nyf,nzf,nxc,nyc,nzc
      INTEGER level

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

*=======================================================================
*     Determine coarse-grid pointns xp,yp,zp from fine-grid xp,yp,zp
*=======================================================================

#ifdef MPI_USED
      INTEGER status(MPI_STATUS_SIZE)
      INTEGER ierr
#endif

      INTEGER ixf,iyf,izf,ixc,iyc,izc

      IF (gmmlc1) THEN
         ixf=1
         DO ixc=1,nxc-1
            xpc(ixc)=xpf(ixf)
            ixf=ixf+2
         END DO
      ELSE
         DO ixc=1,nxc-1
            xpc(ixc)=xpf(ixc)
         END DO
      END IF

      IF (gmmlc2) THEN
         iyf=1
         DO iyc=1,nyc-1
            ypc(iyc)=ypf(iyf)
            iyf=iyf+2
         END DO
      ELSE
         DO iyc=1,nyc-1
            ypc(iyc)=ypf(iyc)
         END DO
      END IF

      IF (gmmlc3) THEN
         izf=1
         DO izc=1,nzc-1
            zpc(izc)=zpf(izf)
            izf=izf+2
         END DO
      ELSE
         DO izc=1,nzc-1
            zpc(izc)=zpf(izc)
         END DO
      END IF

      IF (ipx.EQ.npx-1) xpc(nxc)=xpf(nxf)
      IF (ipy.EQ.npy-1) ypc(nyc)=ypf(nyf)
      IF (ipz.EQ.npz-1) zpc(nzc)=zpf(nzf)

#ifdef MPI_USED
C       ip           ip+1
C     xp(nxc)  <--  xp(2)

      IF (ipx.GT.0 .AND. ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshPoints: MPI_Sendrecv')")  ip
#endif
         CALL MPI_Sendrecv (
     &     xpc(2),  1,MM_REAL,nbr(XMD),150+XMD,
     &     xpc(nxc),1,MM_REAL,nbr(XPD),150+XMD,
     &     comm,status,ierr)
      ELSEIF (ipx.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshPoints: MPI_Send')")  ip
#endif
         CALL MPI_Send (
     &     xpc(2),  1,MM_REAL,nbr(XMD),150+XMD,
     &     comm,ierr)
      ELSEIF (ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshPoints: MPI_Recv')")  ip
#endif
         CALL MPI_Recv (
     &     xpc(nxc),1,MM_REAL,nbr(XPD),150+XMD,
     &     comm,status,ierr)
      ENDIF
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmGetCoarseMeshPoints: MPI_Barrier')")  ip
#endif
      CALL MPI_Barrier (comm,ierr)
     
      IF (ipy.GT.0 .AND. ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshPoints: MPI_Sendrecv')")  ip
#endif
         CALL MPI_Sendrecv (
     &     ypc(2),  1,MM_REAL,nbr(YMD),150+YMD,
     &     ypc(nyc),1,MM_REAL,nbr(YPD),150+YMD,
     &     comm,status,ierr)
      ELSEIF (ipy.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshPoints: MPI_Send')")  ip
#endif
         CALL MPI_Send (
     &     ypc(2),  1,MM_REAL,nbr(YMD),150+YMD,
     &     comm,ierr)
      ELSEIF (ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshPoints: MPI_Recv')")  ip
#endif
         CALL MPI_Recv (
     &     ypc(nyc),1,MM_REAL,nbr(YPD),150+YMD,
     &     comm,status,ierr)
      ENDIF
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmGetCoarseMeshPoints: MPI_Barrier')")  ip
#endif
      CALL MPI_Barrier (comm,ierr)

      IF (ipz.GT.0 .AND. ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshPoints: MPI_Sendrecv')")  ip
#endif
         CALL MPI_Sendrecv (
     &     zpc(2),  1,MM_REAL,nbr(ZMD),150+ZMD,
     &     zpc(nzc),1,MM_REAL,nbr(ZPD),150+ZMD,
     &     comm,status,ierr)
      ELSEIF (ipz.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshPoints: MPI_Send')")  ip
#endif
         CALL MPI_Send (
     &     zpc(2),  1,MM_REAL,nbr(ZMD),150+ZMD,
     &     comm,ierr)
      ELSEIF (ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshPoints: MPI_Recv')")  ip
#endif
         CALL MPI_Recv (
     &     zpc(nzc),1,MM_REAL,nbr(ZPD),150+ZMD,
     &     comm,status,ierr)
      ENDIF
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmGetCoarseMeshPoints: MPI_Barrier')")  ip
#endif
      CALL MPI_Barrier (comm,ierr)
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmGetCoarseMeshWidths 
C     
C     Purpose: Compute the mesh widths for a coarse mesh given
C     Purpose: ones for the next finer level
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmGetCoarseMeshWidths (
     &     hxf,hyf,hzf,hxc,hyc,hzc,
     &     nxf,nyf,nzf,nxc,nyc,nzc,
     &     comm,nbr,bnd
     &     ,level)

      IMPLICIT NONE

      
      INTEGER comm,nbr(6),bnd(3)

      REAL hxf(*),hyf(*),hzf(*)
      REAL hxc(*),hyc(*),hzc(*)
      INTEGER nxf,nyf,nzf,nxc,nyc,nzc
      INTEGER level

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

*=======================================================================
*     Determine coarse-grid hx,hy,hz from fine-grid hx,hy,hz
*=======================================================================

#ifdef MPI_USED
      INTEGER status(MPI_STATUS_SIZE)
      INTEGER ierr
#endif

      INTEGER ixf,iyf,izf,ixc,iyc,izc

      IF (gmmlc1) THEN
         ixf=2
         DO ixc=2,nxc-1
            hxc(ixc)=hxf(ixf)+hxf(ixf+1)
            ixf=ixf+2
         END DO
      ELSE
         DO ixc=2,nxc-1
            hxc(ixc)=hxf(ixc)
         END DO
      END IF

      IF (gmmlc2) THEN
         iyf=2
         DO iyc=2,nyc-1
            hyc(iyc)=hyf(iyf)+hyf(iyf+1)
            iyf=iyf+2
         END DO
      ELSE
         DO iyc=2,nyc-1
            hyc(iyc)=hyf(iyc)
         END DO
      END IF

      IF (gmmlc3) THEN
         izf=2
         DO izc=2,nzc-1
            hzc(izc)=hzf(izf)+hzf(izf+1)
            izf=izf+2
         END DO
      ELSE
         DO izc=2,nzc-1
            hzc(izc)=hzf(izc)
         END DO
      END IF

#ifdef MPI_USED

C       ip             ip+1
C     xp(nxc-1)  -->  xp(1)

      IF (ipx.GT.0 .AND. ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Sendrecv')")  ip
#endif
         CALL MPI_Sendrecv (
     &     hxc(nxc-1),1,MM_REAL,nbr(XPD),150+XPD,
     &     hxc(1),    1,MM_REAL,nbr(XMD),150+XPD,
     &     comm,status,ierr)
      ELSEIF (ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &     hxc(nxc-1),1,MM_REAL,nbr(XPD),150+XPD,
     &     comm,ierr)
      ELSEIF (ipx.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &     hxc(1),    1,MM_REAL,nbr(XMD),150+XPD,
     &     comm,status,ierr)
      ENDIF
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Barrier')") ip
#endif
      CALL MPI_Barrier (comm,ierr)
     
      IF (ipy.GT.0 .AND. ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &     hyc(nyc-1),1,MM_REAL,nbr(YPD),150+YPD,
     &     hyc(1),    1,MM_REAL,nbr(YMD),150+YPD,
     &     comm,status,ierr)
      ELSEIF (ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &     hyc(nyc-1),1,MM_REAL,nbr(YPD),150+YPD,
     &     comm,ierr)
      ELSEIF (ipy.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &     hyc(1),    1,MM_REAL,nbr(YMD),150+YPD,
     &     comm,status,ierr)
      ENDIF
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Barrier')") ip
#endif
      CALL MPI_Barrier (comm,ierr)

      IF (ipz.GT.0 .AND. ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &     hzc(nzc-1),1,MM_REAL,nbr(ZPD),150+ZPD,
     &     hzc(1),    1,MM_REAL,nbr(ZMD),150+ZPD,
     &     comm,status,ierr)
      ELSEIF (ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &     hzc(nzc-1),1,MM_REAL,nbr(ZPD),150+ZPD,
     &     comm,ierr)
      ELSEIF (ipz.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &     hzc(1),    1,MM_REAL,nbr(ZMD),150+ZPD,
     &     comm,status,ierr)
      ENDIF
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Barrier')") ip
#endif
      CALL MPI_Barrier (comm,ierr)

C       ip           ip+1
C     xp(nxc)  <--  xp(2)

      IF (ipx.GT.0 .AND. ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &     hxc(2),  1,MM_REAL,nbr(XMD),150+XMD,
     &     hxc(nxc),1,MM_REAL,nbr(XPD),150+XMD,
     &     comm,status,ierr)
      ELSEIF (ipx.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &     hxc(2),  1,MM_REAL,nbr(XMD),150+XMD,
     &     comm,ierr)
      ELSEIF (ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &     hxc(nxc),1,MM_REAL,nbr(XPD),150+XMD,
     &     comm,status,ierr)
      ENDIF
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Barrier')") ip
#endif
      CALL MPI_Barrier (comm,ierr)
     
      IF (ipy.GT.0 .AND. ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &     hyc(2),  1,MM_REAL,nbr(YMD),150+YMD,
     &     hyc(nyc),1,MM_REAL,nbr(YPD),150+YMD,
     &     comm,status,ierr)
      ELSEIF (ipy.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &     hyc(2),  1,MM_REAL,nbr(YMD),150+YMD,
     &     comm,ierr)
      ELSEIF (ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &     hyc(nyc),1,MM_REAL,nbr(YPD),150+YMD,
     &     comm,status,ierr)
      ENDIF
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Barrier')") ip
#endif
      CALL MPI_Barrier (comm,ierr)

      IF (ipz.GT.0 .AND. ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &     hzc(2),  1,MM_REAL,nbr(ZMD),150+ZMD,
     &     hzc(nzc),1,MM_REAL,nbr(ZPD),150+ZMD,
     &     comm,status,ierr)
      ELSEIF (ipz.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &     hzc(2),  1,MM_REAL,nbr(ZMD),150+ZMD,
     &     comm,ierr)
      ELSEIF (ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &     hzc(nzc),1,MM_REAL,nbr(ZPD),150+ZMD,
     &     comm,status,ierr)
      ENDIF
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmGetCoarseMeshWidths: MPI_Barrier')") ip
#endif
      CALL MPI_Barrier (comm,ierr)

C       DO i=0,npx*npy*npz-1
C          ip = ipx+ipy*npx+ipz*npy*npx
C          IF (ip.EQ.i) THEN
C             DO ixc=1,nxc
C                print*, ip,'hx',ixc,hxc(ixc)
C             END DO
C             DO iyc=1,nyc
C                print*, ip,'hy',iyc,hyc(iyc)
C             END DO
C             DO izc=1,nzc
C                print*, ip,'hz',izc,hzc(izc)
C             END DO
C          END IF
C       CALL MPI_Barrier (comm,ierr)
C       END DO

*      write(*,1000) 'hxc',ipx,ipy,ipz,hxc(1),hyc(1),hzc(1)
* 1000 format(1x,a4,3i4,3e15.8)

#endif
      RETURN
      END

C=======================================================================
C
C     MODULE:  SUBROUTINE mmGetGlobalIndices 
C
C     PURPOSE: Determine the mapping between processor-local indices and 
C     PURPOSE: processor-global indices
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmGetGlobalIndices (ix1,iy1,iz1,nx,ny,nz,
     &     comm,nbr,bnd
     &     )

      IMPLICIT NONE

      INTEGER ix1,iy1,iz1,nx,ny,nz
      
      INTEGER comm,nbr(6),bnd(3)

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

*=======================================================================
*     Determine local x1,y1,z1 by summing hx,hy,hz's
*=======================================================================

#ifdef MPI_USED
      INTEGER status(MPI_STATUS_SIZE)
      INTEGER ierr
#endif
      INTEGER ixs1,iys1,izs1

      ix1 = 0
#ifdef MPI_USED
      IF (ipx.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetGlobalIndices: MPI_Recv')") ip
#endif
         CALL MPI_Recv (ix1,1,MPI_INTEGER,nbr(XMD),120+XPD,
     &        comm,status,ierr)
      END IF
      IF (ipx.LT.npx-1) THEN
         ixs1 = ix1 + nx - 2
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetGlobalIndices: MPI_Send')") ip
#endif
         CALL MPI_Send (ixs1,1,MPI_INTEGER,nbr(XPD),120+XPD,
     &        comm,ierr)
      END IF
#endif

      iy1 = 0
#ifdef MPI_USED
      IF (ipy.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetGlobalIndices: MPI_Recv')") ip
#endif
         CALL MPI_Recv (iy1,1,MPI_INTEGER,nbr(YMD),120+YPD,
     &        comm,status,ierr)
      END IF
      IF (ipy.LT.npy-1) THEN
         iys1 = iy1 + ny - 2
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetGlobalIndices: MPI_Send')") ip
#endif
         CALL MPI_Send (iys1,1,MPI_INTEGER,nbr(YPD),120+YPD,
     &        comm,ierr)
      END IF
#endif

      iz1 = 0
#ifdef MPI_USED
      IF (ipz.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetGlobalIndices: MPI_Recv')") ip
#endif
         CALL MPI_Recv (iz1,1,MPI_INTEGER,nbr(ZMD),120+ZPD,
     &        comm,status,ierr)
      END IF
      IF (ipz.LT.npz-1) THEN
         izs1 = iz1 + nz - 2
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetGlobalIndices: MPI_Send')") ip
#endif
         CALL MPI_Send (izs1,1,MPI_INTEGER,nbr(ZPD),120+ZPD,
     &        comm,ierr)
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C
C     MODULE:  SUBROUTINE mmGetGlobalSize 
C
C     PURPOSE: Determine the size of the problem
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmGetGlobalSize (nxg,nyg,nzg,nx,ny,nz,
     &     comm,nbr,bnd
     &     )

      IMPLICIT NONE

      INTEGER nxg,nyg,nzg,nx,ny,nz
      
      INTEGER comm,nbr(6),bnd(3)

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

*=======================================================================
*     Determine local x1,y1,z1 by summing hx,hy,hz's
*=======================================================================

#ifdef MPI_USED
      INTEGER status(MPI_STATUS_SIZE)
      INTEGER ierr
#endif
      INTEGER ixs1,iys1,izs1

      nxg = 0
#ifdef MPI_USED
      IF (npx.GT.1) THEN	
         IF (ipy.EQ.npy-1 .AND. ipz.EQ.npz-1) THEN
            IF (ipx.EQ.0) THEN
               ixs1 = nxg + nx - 2
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmGetGlobalSize: MPI_Send')") ip
#endif
               CALL MPI_Send (
     &              ixs1,1,MPI_INTEGER,nbr(XPD),140+XPD,
     &              comm,ierr)
            ELSE IF (ipx.GT.0 .AND. ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmGetGlobalSize: MPI_Recv')") ip
#endif
               CALL MPI_Recv (
     &              nxg,1,MPI_INTEGER,nbr(XMD),140+XPD,
     &              comm,status,ierr)
               ixs1 = nxg + nx - 2
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmGetGlobalSize: MPI_Send')") ip
#endif
               CALL MPI_Send (
     &              ixs1,1,MPI_INTEGER,nbr(XPD),140+XPD,
     &              comm,ierr)
            ELSE IF (ipx.EQ.npx-1) THEN
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmGetGlobalSize: MPI_Recv')") ip
#endif
               CALL MPI_Recv (
     &              nxg,1,MPI_INTEGER,nbr(XMD),140+XPD,
     &              comm,status,ierr)
               nxg = nxg + nx	
            END IF
         END IF
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetGlobalSize: MPI_BCAS')") ip
#endif
         CALL MPI_BCAST(nxg,1,MPI_INTEGER,npx*npy*npz-1,comm,
     &       ierr)
      ELSE
#endif
         nxg = nxg + nx	
#ifdef MPI_USED
      END IF
#endif

      nyg = 0
#ifdef MPI_USED
      IF (npy.GT.1) THEN 
         IF (ipz.EQ.npz-1 .AND. ipx.EQ.npx-1) THEN
            IF (ipy.EQ.0) THEN
               iys1 = nyg + ny - 2
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmGetGlobalSize: MPI_Send')") ip
#endif
               CALL MPI_Send (
     &               iys1,1,MPI_INTEGER,nbr(YPD),140+YPD,
     &               comm,ierr)
            ELSE IF (ipy.GT.0 .AND. ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmGetGlobalSize: MPI_Recv')") ip
#endif
               CALL MPI_Recv (
     &               nyg,1,MPI_INTEGER,nbr(YMD),140+YPD,
     &               comm,status,ierr)
               iys1 = nyg + ny - 2
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmGetGlobalSize: MPI_Send')") ip
#endif
               CALL MPI_Send (
     &               iys1,1,MPI_INTEGER,nbr(YPD),140+YPD,
     &               comm,ierr)
            ELSE IF (ipy.EQ.npy-1) THEN
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmGetGlobalSize: MPI_Recv')") ip
#endif
               CALL MPI_Recv (
     &               nyg,1,MPI_INTEGER,nbr(YMD),140+YPD,
     &               comm,status,ierr)
               nyg = nyg + ny
            END IF
         END IF
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetGlobalSize: MPI_BCAS')") ip
#endif
         CALL MPI_BCAST(nyg,1,MPI_INTEGER,npx*npy*npz-1,comm,
     &       ierr)
      ELSE
#endif
         nyg = nyg + ny	
#ifdef MPI_USED
      END IF
#endif

      nzg = 0
#ifdef MPI_USED
      IF (npz.GT.1) THEN 
         IF (ipx.EQ.npx-1 .AND. ipy.EQ.npy-1) THEN
            IF (ipz.EQ.0) THEN
               izs1 = nzg + nz - 2
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmGetGlobalSize: MPI_Send')") ip
#endif
               CALL MPI_Send (
     &               izs1,1,MPI_INTEGER,nbr(ZPD),140+ZPD,
     &               comm,ierr)
            ELSE IF (ipz.GT.0 .AND. ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmGetGlobalSize: MPI_Recv')") ip
#endif
               CALL MPI_Recv (
     &               nzg,1,MPI_INTEGER,nbr(ZMD),140+ZPD,
     &               comm,status,ierr)
               izs1 = nzg + nz - 2
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmGetGlobalSize: MPI_Send')") ip
#endif
               CALL MPI_Send (
     &               izs1,1,MPI_INTEGER,nbr(ZPD),140+ZPD,
     &               comm,ierr)
            ELSE IF (ipz.EQ.npz-1) THEN
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmGetGlobalSize: MPI_Recv')") ip
#endif
               CALL MPI_Recv (
     &               nzg,1,MPI_INTEGER,nbr(ZMD),140+ZPD,
     &               comm,status,ierr)
                nzg = nzg + nz
            END IF
         END IF
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetGlobalSize: MPI_BCAST')") ip
#endif
         CALL MPI_BCAST(nzg,1,MPI_INTEGER,npx*npy*npz-1,comm,
     &       ierr)
      ELSE
#endif
         nzg = nzg + nz
#ifdef MPI_USED
      END IF
#endif
*      write(*,5000) 'nxg',ipx,ipy,ipz,nxg,nyg,nzg
*      write(*,5000) 'nx',ipx,ipy,ipz,nx,ny,nz
* 5000 format(1x,a4,6i4)

      RETURN
      END
C=======================================================================
C     
C     Module:  mmGetGhost
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmGetGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      
      INTEGER comm,nbr(6),bnd(3)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"

      INTEGER status(MPI_STATUS_SIZE)
      INTEGER ierr
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_GHOST
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef MPI_USED
      IF (ipx.GT.0 .AND. ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetX1Ghost: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &        X(nx-1,1,1),1,bndx(XD),nbr(XPD),70+XPD,
     &        X(   1,1,1),1,bndx(XD),nbr(XMD),70+XPD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_x1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & 2*nydx*nzdx
#endif
      ELSE IF (ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetX1Ghost: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &        X(nx-1,1,1),1,bndx(XD),nbr(XPD),70+XPD,
     &        comm,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_x1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nzdx
#endif
      ELSE IF (ipx.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetX1Ghost: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &        X(   1,1,1),1,bndx(XD),nbr(XMD),70+XPD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_x1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nzdx
#endif
      END IF
#endif

#ifdef MPI_USED
      IF (ipx.GT.0 .AND. ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetXnGhost: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &        X( 2,1,1),1,bndx(XD),nbr(XMD),70+XMD,
     &        X(nx,1,1),1,bndx(XD),nbr(XPD),70+XMD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_xn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & 2*nydx*nzdx
#endif
      ELSE IF (ipx.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetXnGhost: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &        X( 2,1,1),1,bndx(XD),nbr(XMD),70+XMD,
     &        comm,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_xn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nzdx
#endif
      ELSE IF (ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetXnGhost: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &        X(nx,1,1),1,bndx(XD),nbr(XPD),70+XMD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_xn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nzdx
#endif
      END IF
#endif

#ifdef MPI_USED
      IF (ipy.GT.0 .AND. ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetY1Ghost: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &        X(1,ny-1,1),1,bndx(YD),nbr(YPD),70+YPD,
     &        X(1,   1,1),1,bndx(YD),nbr(YMD),70+YPD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_y1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nzdx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & 2*nzdx*nxdx
#endif
      ELSE IF (ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetY1Ghost: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &        X(1,ny-1,1),1,bndx(YD),nbr(YPD),70+YPD,
     &        comm,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_y1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nzdx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & nzdx*nxdx
#endif
      ELSE IF (ipy.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetY1Ghost: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &        X(1,   1,1),1,bndx(YD),nbr(YMD),70+YPD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_y1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nzdx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & nzdx*nxdx
#endif
      END IF
#endif


#ifdef MPI_USED
      IF (ipy.GT.0 .AND. ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetYnGhost: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &        X(1, 2,1),1,bndx(YD),nbr(YMD),70+YMD,
     &        X(1,ny,1),1,bndx(YD),nbr(YPD),70+YMD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_yn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nzdx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & 2*nzdx*nxdx
#endif
      ELSE IF (ipy.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetYnGhost: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &        X(1, 2,1),1,bndx(YD),nbr(YMD),70+YMD,
     &        comm,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_yn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nzdx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & nzdx*nxdx
#endif
      ELSE IF (ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetYnGhost: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &        X(1,ny,1),1,bndx(YD),nbr(YPD),70+YMD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_yn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nzdx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & nzdx*nxdx
#endif
      END IF
#endif
#ifdef MPI_USED
      IF (ipz.GT.0 .AND. ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZ1Ghost: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &        X(1,1,nz-1),1,bndx(ZD),nbr(ZPD),70+ZPD,
     &        X(1,1,   1),1,bndx(ZD),nbr(ZMD),70+ZPD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_z1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & 2*nxdx*nydx
#endif
      ELSE IF (ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZ1Ghost: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &        X(1,1,nz-1),1,bndx(ZD),nbr(ZPD),70+ZPD,
     &        comm,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_z1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nydx
#endif
      ELSE IF (ipz.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZ1Ghost: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &        X(1,1,   1),1,bndx(ZD),nbr(ZMD),70+ZPD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_z1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nydx
#endif
      END IF
#endif

#ifdef MPI_USED
      IF (ipz.GT.0 .AND. ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZnGhost: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &        X(1,1, 2),1,bndx(ZD),nbr(ZMD),70+ZMD,
     &        X(1,1,nz),1,bndx(ZD),nbr(ZPD),70+ZMD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_zn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & 2*nxdx*nydx
#endif
      ELSE IF (ipz.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZnGhost: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &        X(1,1, 2),1,bndx(ZD),nbr(ZMD),70+ZMD,
     &        comm,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_zn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nydx
#endif
      ELSE IF (ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZnGhost: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &        X(1,1,nz),1,bndx(ZD),nbr(ZPD),70+ZMD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_zn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nydx
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C
C     MODULE:  SUBROUTINE mmGetX1Ghost 
C
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from Y(nx-1,*,*) on (ipx,  ipy,ipz)
C     PURPOSE: to   X(1,   *,*) on (ipx+1,ipy,ipz)
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmGetX1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      
      INTEGER comm,nbr(6),bnd(3)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"

      INTEGER status(MPI_STATUS_SIZE)
      INTEGER ierr
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_GHOST
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef MPI_USED
      IF (ipx.GT.0 .AND. ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetX1Ghost: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &        X(nx-1,1,1),1,bndx(XD),nbr(XPD),70+XPD,
     &        X(   1,1,1),1,bndx(XD),nbr(XMD),70+XPD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_x1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & 2*nydx*nzdx
#endif
      ELSE IF (ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetX1Ghost: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &        X(nx-1,1,1),1,bndx(XD),nbr(XPD),70+XPD,
     &        comm,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_x1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nzdx
#endif
      ELSE IF (ipx.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetX1Ghost: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &        X(   1,1,1),1,bndx(XD),nbr(XMD),70+XPD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_x1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nzdx
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C
C     MODULE:  SUBROUTINE mmGetXnGhost 
C
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from Y(2, *,*) on (ipx,  ipy,ipz)
C     PURPOSE: to   X(nx,*,*) on (ipx-1,ipy,ipz)
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmGetXnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      
      INTEGER comm,nbr(6),bnd(3)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"

      INTEGER status(MPI_STATUS_SIZE)
      INTEGER ierr
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_GHOST
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef MPI_USED
      IF (ipx.GT.0 .AND. ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetXnGhost: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &        X( 2,1,1),1,bndx(XD),nbr(XMD),70+XMD,
     &        X(nx,1,1),1,bndx(XD),nbr(XPD),70+XMD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_xn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & 2*nydx*nzdx
#endif
      ELSE IF (ipx.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetXnGhost: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &        X( 2,1,1),1,bndx(XD),nbr(XMD),70+XMD,
     &        comm,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_xn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nzdx
#endif
      ELSE IF (ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetXnGhost: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &        X(nx,1,1),1,bndx(XD),nbr(XPD),70+XMD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_xn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nzdx
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmGetY1Ghost 
C
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from Y(*,ny-1,*) on (ipx,ipy,  ipz)
C     PURPOSE: to   X(*,1,   *) on (ipx,ipy+1,ipz)
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmGetY1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      
      INTEGER comm,nbr(6),bnd(3)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"

      INTEGER status(MPI_STATUS_SIZE)
      INTEGER ierr
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_GHOST
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef MPI_USED
      IF (ipy.GT.0 .AND. ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetY1Ghost: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &        X(1,ny-1,1),1,bndx(YD),nbr(YPD),70+YPD,
     &        X(1,   1,1),1,bndx(YD),nbr(YMD),70+YPD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_y1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nzdx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & 2*nzdx*nxdx
#endif
      ELSE IF (ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetY1Ghost: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &        X(1,ny-1,1),1,bndx(YD),nbr(YPD),70+YPD,
     &        comm,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_y1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nzdx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & nzdx*nxdx
#endif
      ELSE IF (ipy.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetY1Ghost: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &        X(1,   1,1),1,bndx(YD),nbr(YMD),70+YPD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_y1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nzdx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & nzdx*nxdx
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmGetYnGhost 
C     
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from Y(*,2, *) on (ipx,ipy,  ipz)
C     PURPOSE: to   X(*,ny,*) on (ipx,ipy-1,ipz)
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmGetYnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      
      INTEGER comm,nbr(6),bnd(3)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"

      INTEGER status(MPI_STATUS_SIZE)
      INTEGER ierr
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_GHOST
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef MPI_USED
      IF (ipy.GT.0 .AND. ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetYnGhost: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &        X(1, 2,1),1,bndx(YD),nbr(YMD),70+YMD,
     &        X(1,ny,1),1,bndx(YD),nbr(YPD),70+YMD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_yn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nzdx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & 2*nzdx*nxdx
#endif
      ELSE IF (ipy.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetYnGhost: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &        X(1, 2,1),1,bndx(YD),nbr(YMD),70+YMD,
     &        comm,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_yn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nzdx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & nzdx*nxdx
#endif
      ELSE IF (ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetYnGhost: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &        X(1,ny,1),1,bndx(YD),nbr(YPD),70+YMD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_yn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nzdx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & nzdx*nxdx
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmGetZ1Ghost 
C     
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from Y(*,*,nz-1) on (ipx,ipy,ipz)
C     PURPOSE: to   X(*,*,1)    on (ipx,ipy,ipz+1)
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmGetZ1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      
      INTEGER comm,nbr(6),bnd(3)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"

      INTEGER status(MPI_STATUS_SIZE)
      INTEGER ierr
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_GHOST
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef MPI_USED
      IF (ipz.GT.0 .AND. ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZ1Ghost: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &        X(1,1,nz-1),1,bndx(ZD),nbr(ZPD),70+ZPD,
     &        X(1,1,   1),1,bndx(ZD),nbr(ZMD),70+ZPD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_z1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & 2*nxdx*nydx
#endif
      ELSE IF (ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZ1Ghost: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &        X(1,1,nz-1),1,bndx(ZD),nbr(ZPD),70+ZPD,
     &        comm,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_z1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nydx
#endif
      ELSE IF (ipz.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZ1Ghost: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &        X(1,1,   1),1,bndx(ZD),nbr(ZMD),70+ZPD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_z1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nydx
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmGetZnGhost 
C
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from Y(*,*,2)  on (ipx,ipy,ipz)
C     PURPOSE: to   X(*,*,nz) on (ipx,ipy,ipz-1)
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmGetZnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      
      INTEGER comm,nbr(6),bnd(3)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"

      INTEGER status(MPI_STATUS_SIZE)
      INTEGER ierr
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_GHOST
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef MPI_USED
      IF (ipz.GT.0 .AND. ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZnGhost: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &        X(1,1, 2),1,bndx(ZD),nbr(ZMD),70+ZMD,
     &        X(1,1,nz),1,bndx(ZD),nbr(ZPD),70+ZMD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_zn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & 2*nxdx*nydx
#endif
      ELSE IF (ipz.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZnGhost: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &        X(1,1, 2),1,bndx(ZD),nbr(ZMD),70+ZMD,
     &        comm,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_zn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nydx
#endif
      ELSE IF (ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZnGhost: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &        X(1,1,nz),1,bndx(ZD),nbr(ZPD),70+ZMD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_zn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nydx
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C
C     MODULE:  SUBROUTINE mmGetX1Ghost2
C
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from Y(nx-1,*,*) on (ipx,  ipy,ipz)
C     PURPOSE: to   X(1,   *,*) on (ipx+1,ipy,ipz)
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmGetX1Ghost2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      
      INTEGER comm,nbr(6),bnd(3)
      LOGICAL lx,ly,lz

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
      INTEGER status(MPI_STATUS_SIZE)
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      INTEGER ierr,ix,iy,iz

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_GHOST2
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      IF (.NOT.lx) ix=0
      IF (lx) ix=1
      IF (.NOT.ly) iy=0
      IF (ly) iy=1
      IF (.NOT.lz) iz=0
      IF (lz) iz=1

#ifdef MPI_USED
      IF (lx.AND.ipx.GT.0 .AND. ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetX1Ghost2: MPI_Sendrecv',
     &        i2,i2,i2,'...')") ip,ix,iy,iz
#endif
         CALL MPI_Sendrecv (
     &        X(nx-1,1,1),1,bndx(XD),gsubnbr(ix,iy,iz,XPD),70+XPD,
     &        X( 1,1,1),1,bndx(XD),gsubnbr(ix,iy,iz,XMD),70+XPD,
     &        gsubcomm(ix,iy,iz),status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmGetX1Ghost2: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & 2*nydx*nzdx
#endif
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetX1Ghost2: ...MPI_Sendrecv',
     &        i2,i2,i2)") ip,ix,iy,iz
#endif
      ELSE IF (lx.AND.ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetX1Ghost2: MPI_Send',
     &        i2,i2,i2,'...')") ip,ix,iy,iz
#endif
         CALL MPI_Send (
     &        X(nx-1,1,1),1,bndx(XD),gsubnbr(ix,iy,iz,XPD),70+XPD,
     &        gsubcomm(ix,iy,iz),ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmGetX1Ghost2: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nzdx
#endif
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetX1Ghost2: ...MPI_Send',
     &        i2,i2,i2)") ip,ix,iy,iz
#endif
      ELSE IF (lx.AND.ipx.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetX1Ghost2: MPI_Recv',
     &        i2,i2,i2,'...')") ip,ix,iy,iz
#endif
         CALL MPI_Recv (
     &        X( 1,1,1),1,bndx(XD),gsubnbr(ix,iy,iz,XMD),70+XPD,
     &        gsubcomm(ix,iy,iz),status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmGetX1Ghost2: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nzdx
#endif
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetX1Ghost2: ...MPI_Recv',
     &        i2,i2,i2)") ip,ix,iy,iz
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C
C     MODULE:  SUBROUTINE mmGetXnGhost2
C
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from Y(2, *,*) on (ipx,  ipy,ipz)
C     PURPOSE: to   X(nx,*,*) on (ipx-1,ipy,ipz)
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmGetXnGhost2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      
      INTEGER comm,nbr(6),bnd(3)
      LOGICAL lx,ly,lz

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
      INTEGER status(MPI_STATUS_SIZE)
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      INTEGER ierr,icx,icy,icz

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_GHOST2
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      IF (.NOT.lx) icx=0
      IF (lx) icx=1
      IF (.NOT.ly) icy=0
      IF (ly) icy=1
      IF (.NOT.lz) icz=0
      IF (lz) icz=1

#ifdef MPI_USED
      IF (lx.AND.ipx.GT.0 .AND. ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetXnGhost2: MPI_Sendrecv',
     &        i2,i2,i2,'...')") ip,icx,icy,icz
#endif
         CALL MPI_Sendrecv (
     &        X( 2,1,1),1,bndx(XD),gsubnbr(icx,icy,icz,XMD),70+XMD,
     &        X(nx,1,1),1,bndx(XD),gsubnbr(icx,icy,icz,XPD),70+XMD,
     &        gsubcomm(icx,icy,icz),status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmGetXnGhost2: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & 2*nydx*nzdx
#endif
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetXnGhost2: ...MPI_Sendrecv',
     &        i2,i2,i2)") ip,icx,icy,icz
#endif
      ELSE IF (lx.AND.ipx.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetXnGhost2: MPI_Send',
     &        i2,i2,i2,'...')") ip,icx,icy,icz
#endif
         CALL MPI_Send (
     &        X( 2,1,1),1,bndx(XD),gsubnbr(icx,icy,icz,XMD),70+XMD,
     &        gsubcomm(icx,icy,icz),ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmGetXnGhost2: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nzdx
#endif
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetXnGhost2: ...MPI_Send',
     &        i2,i2,i2)") ip,icx,icy,icz
#endif
      ELSE IF (lx.AND.ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetXnGhost2: MPI_Recv',
     &        i2,i2,i2,'...')") ip,icx,icy,icz
#endif
         CALL MPI_Recv (
     &        X(nx,1,1),1,bndx(XD),gsubnbr(icx,icy,icz,XPD),70+XMD,
     &        gsubcomm(icx,icy,icz),status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmGetXnGhost2: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nzdx
#endif
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetXnGhost2: ...MPI_Recv',
     &        i2,i2,i2)") ip,icx,icy,icz
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmGetY1Ghost2
C
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from Y(*,ny-1,*) on (ipx,ipy,  ipz)
C     PURPOSE: to   X(*,1,   *) on (ipx,ipy+1,ipz)
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmGetY1Ghost2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      
      INTEGER comm,nbr(6),bnd(3)
      LOGICAL lx,ly,lz

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"

      INTEGER status(MPI_STATUS_SIZE)
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      INTEGER ierr,ix,iy,iz

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_GHOST2
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      IF (.NOT.lx) ix=0
      IF (lx) ix=1
      IF (.NOT.ly) iy=0
      IF (ly) iy=1
      IF (.NOT.lz) iz=0
      IF (lz) iz=1

#ifdef MPI_USED
      IF (ly.AND.ipy.GT.0 .AND. ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetY1Ghost2: MPI_Sendrecv',
     &        i2,i2,i2,'...')") ip,ix,iy,iz
#endif
         CALL MPI_Sendrecv (
     &        X(1,ny-1,1),1,bndx(YD),gsubnbr(ix,iy,iz,YPD),70+YPD,
     &        X(1,   1,1),1,bndx(YD),gsubnbr(ix,iy,iz,YMD),70+YPD,
     &        gsubcomm(ix,iy,iz),status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmGetY1Ghost2: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nzdx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & 2*nzdx*nxdx
#endif
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetY1Ghost2: ...MPI_Sendrecv',
     &        i2,i2,i2)") ip,ix,iy,iz
#endif
      ELSE IF (ly.AND.ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetY1Ghost2: MPI_Send',
     &        i2,i2,i2,'...')") ip,ix,iy,iz
#endif
         CALL MPI_Send (
     &        X(1,ny-1,1),1,bndx(YD),gsubnbr(ix,iy,iz,YPD),70+YPD,
     &        gsubcomm(ix,iy,iz),ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmGetY1Ghost2: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nzdx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & nzdx*nxdx
#endif
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetY1Ghost2: ...MPI_Send',
     &        i2,i2,i2)") ip,ix,iy,iz
#endif
      ELSE IF (ly.AND.ipy.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetY1Ghost2: MPI_Recv',
     &        i2,i2,i2,'...')") ip,ix,iy,iz
#endif
         CALL MPI_Recv (
     &        X(1,   1,1),1,bndx(YD),gsubnbr(ix,iy,iz,YMD),70+YPD,
     &        gsubcomm(ix,iy,iz),status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmGetY1Ghost2: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nzdx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & nzdx*nxdx
#endif
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetY1Ghost2: ...MPI_Recv',
     &        i2,i2,i2)") ip,ix,iy,iz
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmGetYnGhost2
C     
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from Y(*,2, *) on (ipx,ipy,  ipz)
C     PURPOSE: to   X(*,ny,*) on (ipx,ipy-1,ipz)
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmGetYnGhost2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      
      INTEGER comm,nbr(6),bnd(3)
      LOGICAL lx,ly,lz

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"

      INTEGER status(MPI_STATUS_SIZE)
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      INTEGER ierr,ix,iy,iz

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_GHOST2
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      IF (.NOT.lx) ix=0
      IF (lx) ix=1
      IF (.NOT.ly) iy=0
      IF (ly) iy=1
      IF (.NOT.lz) iz=0
      IF (lz) iz=1

#ifdef MPI_USED
      IF (ly.AND.ipy.GT.0 .AND. ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetYnGhost2: MPI_Sendrecv',
     &        i2,i2,i2,'...')") ip,ix,iy,iz
#endif
         CALL MPI_Sendrecv (
     &        X(1, 2,1),1,bndx(YD),gsubnbr(ix,iy,iz,YMD),70+YMD,
     &        X(1,ny,1),1,bndx(YD),gsubnbr(ix,iy,iz,YPD),70+YMD,
     &        gsubcomm(ix,iy,iz),status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmGetYnGhost2: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nzdx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & 2*nzdx*nxdx
#endif
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetYnGhost2: ...MPI_Sendrecv',
     &        i2,i2,i2)") ip,ix,iy,iz
#endif
      ELSE IF (ly.AND.ipy.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetYnGhost2: MPI_Send',
     &        i2,i2,i2,'...')") ip,ix,iy,iz
#endif
         CALL MPI_Send (
     &        X(1, 2,1),1,bndx(YD),gsubnbr(ix,iy,iz,YMD),70+YMD,
     &        gsubcomm(ix,iy,iz),ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmGetYnGhost2: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nzdx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & nzdx*nxdx
#endif
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetYnGhost2: ...MPI_Send',
     &        i2,i2,i2)") ip,ix,iy,iz
#endif
      ELSE IF (ly.AND.ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetYnGhost2: MPI_Recv',
     &        i2,i2,i2,'...')") ip,ix,iy,iz
#endif
         CALL MPI_Recv (
     &        X(1,ny,1),1,bndx(YD),gsubnbr(ix,iy,iz,YPD),70+YMD,
     &        gsubcomm(ix,iy,iz),status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmGetYnGhost2: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nzdx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & nzdx*nxdx
#endif
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetYnGhost2: ...MPI_Recv',
     &        i2,i2,i2)") ip,ix,iy,iz
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmGetZ1Ghost2 
C     
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from Y(*,*,nz-1) on (ipx,ipy,ipz)
C     PURPOSE: to   X(*,*,1)    on (ipx,ipy,ipz+1)
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmGetZ1Ghost2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      
      INTEGER comm,nbr(6),bnd(3)
      LOGICAL lx,ly,lz

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"

      INTEGER status(MPI_STATUS_SIZE)
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      INTEGER ierr,ix,iy,iz

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_GHOST2
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      IF (.NOT.lx) ix=0
      IF (lx) ix=1
      IF (.NOT.ly) iy=0
      IF (ly) iy=1
      IF (.NOT.lz) iz=0
      IF (lz) iz=1

#ifdef MPI_USED
      IF (lz.AND.ipz.GT.0 .AND. ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZ1Ghost2: MPI_Sendrecv',
     &        i2,i2,i2,'...')") ip,ix,iy,iz
#endif
         CALL MPI_Sendrecv (
     &        X(1,1,nz-1),1,bndx(ZD),gsubnbr(ix,iy,iz,ZPD),70+ZPD,
     &        X(1,1,   1),1,bndx(ZD),gsubnbr(ix,iy,iz,ZMD),70+ZPD,
     &        gsubcomm(ix,iy,iz),status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmGetZ1Ghost2:',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & 2*nxdx*nydx
#endif
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZ1Ghost2: ...MPI_Sendrecv',
     &        i2,i2,i2)") ip,ix,iy,iz
#endif
      ELSE IF (lz.AND.ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZ1Ghost2: MPI_Send',
     &        i2,i2,i2,'...')") ip,ix,iy,iz
#endif
         CALL MPI_Send (
     &        X(1,1,nz-1),1,bndx(ZD),gsubnbr(ix,iy,iz,ZPD),70+ZPD,
     &        gsubcomm(ix,iy,iz),ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmGetZ1Ghost2: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nydx
#endif
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZ1Ghost2: ...MPI_Send',
     &        i2,i2,i2)") ip,ix,iy,iz
#endif
      ELSE IF (lz.AND.ipz.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZ1Ghost2: MPI_Recv',
     &        i2,i2,i2,'...')") ip,ix,iy,iz
#endif
         CALL MPI_Recv (
     &        X(1,1,   1),1,bndx(ZD),gsubnbr(ix,iy,iz,ZMD),70+ZPD,
     &        gsubcomm(ix,iy,iz),status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmGetZ1Ghost2: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nydx
#endif
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZ1Ghost2: ...MPI_Recv',
     &        i2,i2,i2)") ip,ix,iy,iz
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmGetZnGhost2
C
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from Y(*,*,2)  on (ipx,ipy,ipz)
C     PURPOSE: to   X(*,*,nz) on (ipx,ipy,ipz-1)
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmGetZnGhost2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      
      INTEGER comm,nbr(6),bnd(3)
      LOGICAL lx,ly,lz

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"

      INTEGER status(MPI_STATUS_SIZE)
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      INTEGER ierr,ix,iy,iz

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_GHOST2
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      IF (.NOT.lx) ix=0
      IF (lx) ix=1
      IF (.NOT.ly) iy=0
      IF (ly) iy=1
      IF (.NOT.lz) iz=0
      IF (lz) iz=1

#ifdef MPI_USED
      IF (lz.AND.ipz.GT.0 .AND. ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZnGhost2: MPI_Sendrecv',
     &        i2,i2,i2,'...')") ip,ix,iy,iz
#endif
         CALL MPI_Sendrecv (
     &        X(1,1, 2),1,bndx(ZD),gsubnbr(ix,iy,iz,ZMD),70+ZMD,
     &        X(1,1,nz),1,bndx(ZD),gsubnbr(ix,iy,iz,ZPD),70+ZMD,
     &        gsubcomm(ix,iy,iz),status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmGetZnGhost2: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & 2*nxdx*nydx
#endif
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZnGhost2: ...MPI_Sendrecv',
     &        i2,i2,i2)") ip,ix,iy,iz
#endif
      ELSE IF (lz.AND.ipz.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZnGhost2: MPI_Send',
     &        i2,i2,i2,'...')") ip,ix,iy,iz
#endif
         CALL MPI_Send (
     &        X(1,1, 2),1,bndx(ZD),gsubnbr(ix,iy,iz,ZMD),70+ZMD,
     &        gsubcomm(ix,iy,iz),ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmGetZnGhost2: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nydx
#endif
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZnGhost2: ...MPI_Send',
     &        i2,i2,i2)") ip,ix,iy,iz
#endif
      ELSE IF (lz.AND.ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZnGhost2: MPI_Recv',
     &        i2,i2,i2,'...')") ip,ix,iy,iz
#endif
         CALL MPI_Recv (
     &        X(1,1,nz),1,bndx(ZD),gsubnbr(ix,iy,iz,ZPD),70+ZMD,
     &        gsubcomm(ix,iy,iz),status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in mmGetZnGhost2: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nydx
#endif
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmGetZnGhost2: ...MPI_Recv',
     &        i2,i2,i2)") ip,ix,iy,iz
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  mmInitCoords
C     
C     Purpose: Initialize global variables for coordinate system
C     
C     Algorithm:
C     
C     Inputs:
C     
C     Outputs:
C     
C     Globals accessed: 
C     
C     Globals modified:
C     
C     Calls:
C     
C     Called by:
C     
C     Flop count estimate:
C     
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C     
C     Modification history: 
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmInitCoords (nx,ny,nz,xp,yp,zp,icoords,level)
      IMPLICIT NONE

      INTEGER nx,ny,nz,level,icoords
      REAL xp(*),yp(*),zp(*)

#include "mgmpi_internal.h"

      REAL mmDxA,mmDyA,mmDzA,mmG2A,mmG31A,mmG32A
      REAL mmDxC,mmDyC,mmDzC,mmG2C,mmG31C,mmG32C
      REAL mmDxS,mmDyS,mmDzS,mmG2S,mmG31S,mmG32S
      EXTERNAL mmDxA,mmDyA,mmDzA,mmG2A,mmG31A,mmG32A
      EXTERNAL mmDxC,mmDyC,mmDzC,mmG2C,mmG31C,mmG32C
      EXTERNAL mmDxS,mmDyS,mmDzS,mmG2S,mmG31S,mmG32S

      INTEGER i,k

      k = level
      IF (icoords.EQ.MM_CARTESIAN) THEN
         DO i=1,nx-1
            gdx(i,k)=mmDxA(xp(i),xp(i+1))
            gdxi(i,k) = o1/gdx(i,k)
            gg2on(i,k)=mmG2A(xp(i))
            gg2off(i,k)=mmG2A(o2*(xp(i)+xp(i+1)))
            gg31on(i,k)=mmG31A(xp(i))
            gg31off(i,k)=mmG31A(o2*(xp(i)+xp(i+1)))
         END DO
         DO i=1,ny-1
            gdy(i,k)=mmDyA(yp(i),yp(i+1))
            gdyi(i,k) = o1/gdy(i,k)
            gg32on(i,k)=mmG32A(yp(i))
            gg32off(i,k)=mmG32A(o2*(yp(i)+yp(i+1)))
         END DO
         DO i=1,nz-1
            gdz(i,k)=mmDzA(zp(i),zp(i+1))
            gdzi(i,k) = o1/gdz(i,k)
         END DO
      ELSE IF (icoords.EQ.MM_CYLINDRICAL) THEN
         DO i=1,nx-1
            gdx(i,k)=mmDxC(xp(i),xp(i+1))
            gdxi(i,k) = o1/gdx(i,k)
            gg2on(i,k)=mmG2C(xp(i))
            gg2off(i,k)=mmG2C(o2*(xp(i)+xp(i+1)))
            gg31on(i,k)=mmG31C(xp(i))
            gg31off(i,k)=mmG31C(o2*(xp(i)+xp(i+1)))
         END DO
         DO i=1,ny-1
            gdy(i,k)=mmDyC(yp(i),yp(i+1))
            gdyi(i,k) = o1/gdy(i,k)
            gg32on(i,k)=mmG32C(yp(i))
            gg32off(i,k)=mmG32C(o2*(yp(i)+yp(i+1)))
         END DO
         DO i=1,nz-1
            gdz(i,k)=mmDzC(zp(i),zp(i+1))
            gdzi(i,k) = o1/gdz(i,k)
         END DO
      ELSE IF (icoords.EQ.MM_SPHERICAL) THEN
         DO i=1,nx-1
            gdx(i,k)=mmDxS(xp(i),xp(i+1))
            gdxi(i,k) = o1/gdx(i,k)
            gg2on(i,k)=mmG2S(xp(i))
            gg2off(i,k)=mmG2S(o2*(xp(i)+xp(i+1)))
            gg31on(i,k)=mmG31S(xp(i))
            gg31off(i,k)=mmG31S(o2*(xp(i)+xp(i+1)))
         END DO
         DO i=1,ny-1
            gdy(i,k)=mmDyS(yp(i),yp(i+1))
            gdyi(i,k) = o1/gdy(i,k)
            gg32on(i,k)=mmG32S(yp(i))
            gg32off(i,k)=mmG32S(o2*(yp(i)+yp(i+1)))
         END DO
         DO i=1,nz-1
            gdz(i,k)=mmDzS(zp(i),zp(i+1))
            gdzi(i,k) = o1/gdz(i,k)
         END DO
      END IF

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmInitFlops
C     
C     PURPOSE: Initialize the flop counters
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmInitFlops
      IMPLICIT NONE
#include "mgmpi_internal.h"
      INTEGER i

      DO i=1,NTIMER
         gmflops(i)=0.0
      END DO

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmInitTimers
C     
C     PURPOSE: Initialize the timers
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmInitTimers
      IMPLICIT NONE
      INTEGER k

#include "mgmpi_internal.h"

      DO k=1,NTIMER
          gtimecount(k)=0
      END DO

      gtimename(SB_MG3)   ='mg3ns/mg3s/mg3p   '
      gtimename(SB_MGSOLVE)   ='MgSolve           '
      gtimename(SB_DISTRIBUTE)   ='Distribute        '
      gtimename(SB_BUILD+0)   ='BuildFine/Coarse  '
      gtimename(SB_BUILD+1)   ='BuildFine/Coarse 1'
      gtimename(SB_BUILD+2)   ='BuildFine/Coarse 2'
      gtimename(SB_BUILD+3)   ='BuildFine/Coarse 3'
      gtimename(SB_BUILD+4)   ='BuildFine/Coarse 4'
      gtimename(SB_BUILD+5)   ='BuildFine/Coarse 5'

C     Main 3D solvers

      gtimename(SB_MG)     ='MgVCycle          '
      gtimename(SB_CGMG)   ='CgMg              '
      gtimename(SB_CG+0)   ='Cg (BiCgStab)     '
      gtimename(SB_CG+1)   ='Cg (BiCgStab) 1   '
      gtimename(SB_CG+2)   ='Cg (BiCgStab) 2   '
      gtimename(SB_CG+3)   ='Cg (BiCgStab) 3   '
      gtimename(SB_CG+4)   ='Cg (BiCgStab) 4   '
      gtimename(SB_CG+5)   ='Cg (BiCgStab) 5   '
      gtimename(SB_PRECON) ='Precon            '
      gtimename(SB_FMG)    ='Full MG           '

C     Smoother 2D solvers

      gtimename(SB_MG2+0)   ='MgVCycle2         '
      gtimename(SB_MG2+1)   ='MgVCycle2 1       '
      gtimename(SB_MG2+2)   ='MgVCycle2 2       '
      gtimename(SB_MG2+3)   ='MgVCycle2 3       '
      gtimename(SB_MG2+4)   ='MgVCycle2 4       '
      gtimename(SB_MG2+5)   ='MgVCycle2 5       '
      gtimename(SB_CG2+0)   ='Cg2               '
      gtimename(SB_CG2+1)   ='Cg2 1             '
      gtimename(SB_CG2+2)   ='Cg2 2             '
      gtimename(SB_CG2+3)   ='Cg2 3             '
      gtimename(SB_CG2+4)   ='Cg2 4             '
      gtimename(SB_CG2+5)   ='Cg2 5             '

C     3D linear algebra

      gtimename(SB_MATVEC+0)   ='Matvec            '
      gtimename(SB_MATVEC+1)   ='Matvec 1          '
      gtimename(SB_MATVEC+2)   ='Matvec 2          '
      gtimename(SB_MATVEC+3)   ='Matvec 3          '
      gtimename(SB_MATVEC+4)   ='Matvec 4          '
      gtimename(SB_MATVEC+5)   ='Matvec 5          '
      gtimename(SB_RESIDUAL+0)   ='Residual          '
      gtimename(SB_RESIDUAL+1)   ='Residual 1        '
      gtimename(SB_RESIDUAL+2)   ='Residual 2        '
      gtimename(SB_RESIDUAL+3)   ='Residual 3        '
      gtimename(SB_RESIDUAL+4)   ='Residual 4        '
      gtimename(SB_RESIDUAL+5)   ='Residual 5        '
      gtimename(SB_ZAXPY+0)   ='Zaxpy             '
      gtimename(SB_ZAXPY+1)   ='Zaxpy 1           '
      gtimename(SB_ZAXPY+2)   ='Zaxpy 2           '
      gtimename(SB_ZAXPY+3)   ='Zaxpy 3           '
      gtimename(SB_ZAXPY+4)   ='Zaxpy 4           '
      gtimename(SB_ZAXPY+5)   ='Zaxpy 5           '
      gtimename(SB_DOTPROD+0)   ='DotProd           '
      gtimename(SB_DOTPROD+1)   ='DotProd 1         '
      gtimename(SB_DOTPROD+2)   ='DotProd 2         '
      gtimename(SB_DOTPROD+3)   ='DotProd 3         '
      gtimename(SB_DOTPROD+4)   ='DotProd 4         '
      gtimename(SB_DOTPROD+5)   ='DotProd 5         '
      gtimename(SB_SUM+0)   ='Sum               '
      gtimename(SB_SUM+1)   ='Sum 1             '
      gtimename(SB_SUM+2)   ='Sum 2             '
      gtimename(SB_SUM+3)   ='Sum 3             '
      gtimename(SB_SUM+4)   ='Sum 4             '
      gtimename(SB_SUM+5)         ='Sum 5             '
      gtimename(SB_NORMALIZE+0)   ='Normalize         '
      gtimename(SB_NORMALIZE+1)   ='Normalize 1       '
      gtimename(SB_NORMALIZE+2)   ='Normalize 2       '
      gtimename(SB_NORMALIZE+3)   ='Normalize 3       '
      gtimename(SB_NORMALIZE+4)   ='Normalize 4       '
      gtimename(SB_NORMALIZE+5)   ='Normalize 5       '
      gtimename(SB_SHIFT+0)       ='Shift             '
      gtimename(SB_SHIFT+1)       ='Shift 1           '
      gtimename(SB_SHIFT+2)       ='Shift 2           '
      gtimename(SB_SHIFT+3)       ='Shift 3           '
      gtimename(SB_SHIFT+4)       ='Shift 4           '
      gtimename(SB_SHIFT+5)       ='Shift 5           '
      gtimename(SB_DRIVER)        ='Driver            '
      
C     2D linear algebra 

      gtimename(SB_MATVEC2+0)   ='Matvec2           '
      gtimename(SB_MATVEC2+1)   ='Matvec2 1         '
      gtimename(SB_MATVEC2+2)   ='Matvec2 2         '
      gtimename(SB_MATVEC2+3)   ='Matvec2 3         '
      gtimename(SB_MATVEC2+4)   ='Matvec2 4         '
      gtimename(SB_MATVEC2+5)   ='Matvec2 5         '
      gtimename(SB_RESIDUAL2+0)   ='Residual2         '
      gtimename(SB_RESIDUAL2+1)   ='Residual2 1       '
      gtimename(SB_RESIDUAL2+2)   ='Residual2 2       '
      gtimename(SB_RESIDUAL2+3)   ='Residual2 3       '
      gtimename(SB_RESIDUAL2+4)   ='Residual2 4       '
      gtimename(SB_RESIDUAL2+5)   ='Residual2 5       '
      gtimename(SB_ZAXPY2+0)   ='Zaxpy2            '
      gtimename(SB_ZAXPY2+1)   ='Zaxpy2 1          '
      gtimename(SB_ZAXPY2+2)   ='Zaxpy2 2          '
      gtimename(SB_ZAXPY2+3)   ='Zaxpy2 3          '
      gtimename(SB_ZAXPY2+4)   ='Zaxpy2 4          '
      gtimename(SB_ZAXPY2+5)   ='Zaxpy2 5          '
      gtimename(SB_DOTPROD2+0)   ='DotProd2          '
      gtimename(SB_DOTPROD2+1)   ='DotProd2 1        '
      gtimename(SB_DOTPROD2+2)   ='DotProd2 2        '
      gtimename(SB_DOTPROD2+3)   ='DotProd2 3        '
      gtimename(SB_DOTPROD2+4)   ='DotProd2 4        '
      gtimename(SB_DOTPROD2+5)   ='DotProd2 5        '

C     3D multigrid

      gtimename(SB_RESTRICT+0)   ='Restrict          '
      gtimename(SB_RESTRICT+1)   ='Restrict 1        '
      gtimename(SB_RESTRICT+2)   ='Restrict 2        '
      gtimename(SB_RESTRICT+3)   ='Restrict 3        '
      gtimename(SB_RESTRICT+4)   ='Restrict 4        '
      gtimename(SB_RESTRICT+5)   ='Restrict 5        '
      gtimename(SB_PROLONG+0)   ='Prolong           '
      gtimename(SB_PROLONG+1)   ='Prolong 1         '
      gtimename(SB_PROLONG+2)   ='Prolong 2         '
      gtimename(SB_PROLONG+3)   ='Prolong 3         '
      gtimename(SB_PROLONG+4)   ='Prolong 4         '
      gtimename(SB_PROLONG+5)   ='Prolong 5         '
      gtimename(SB_SMOOTH+0)   ='Smooth            '
      gtimename(SB_SMOOTH+1)   ='Smooth 1          '
      gtimename(SB_SMOOTH+2)   ='Smooth 2          '
      gtimename(SB_SMOOTH+3)   ='Smooth 3          '
      gtimename(SB_SMOOTH+4)   ='Smooth 4          '
      gtimename(SB_SMOOTH+5)   ='Smooth 5          '
      gtimename(SB_CORRECT+0)  ='Correct           '
      gtimename(SB_CORRECT+1)  ='Correct 1         '
      gtimename(SB_CORRECT+2)  ='Correct 2         '
      gtimename(SB_CORRECT+3)  ='Correct 3         '
      gtimename(SB_CORRECT+4)  ='Correct 4         '
      gtimename(SB_CORRECT+5)  ='Correct 5         '

C     2D multigrid

      gtimename(SB_RESTRICT2+0)   ='Restrict2         '
      gtimename(SB_RESTRICT2+1)   ='Restrict2 1       '
      gtimename(SB_RESTRICT2+2)   ='Restrict2 2       '
      gtimename(SB_RESTRICT2+3)   ='Restrict2 3       '
      gtimename(SB_RESTRICT2+4)   ='Restrict2 4       '
      gtimename(SB_RESTRICT2+5)   ='Restrict2 5       '
      gtimename(SB_PROLONG2+0)   ='Prolong2          '
      gtimename(SB_PROLONG2+1)   ='Prolong2 1        '
      gtimename(SB_PROLONG2+2)   ='Prolong2 2        '
      gtimename(SB_PROLONG2+3)   ='Prolong2 3        '
      gtimename(SB_PROLONG2+4)   ='Prolong2 4        '
      gtimename(SB_PROLONG2+5)   ='Prolong2 5        '

C     Communication

      gtimename(SB_GHOST+0)   ='Send/Get          '
      gtimename(SB_GHOST+1)   ='Send/Get 1        '
      gtimename(SB_GHOST+2)   ='Send/Get 2        '
      gtimename(SB_GHOST+3)   ='Send/Get 3        '
      gtimename(SB_GHOST+4)   ='Send/Get 4        '
      gtimename(SB_GHOST+5)   ='Send/Get 5        '
      gtimename(SB_GHOST2+0)   ='Send2/Get2        '
      gtimename(SB_GHOST2+1)   ='Send2/Get2 1      '
      gtimename(SB_GHOST2+2)   ='Send2/Get2 2      '
      gtimename(SB_GHOST2+3)   ='Send2/Get2 3      '
      gtimename(SB_GHOST2+4)   ='Send2/Get2 4      '
      gtimename(SB_GHOST2+5)   ='Send2/Get2 5      '
      gtimename(SB_IGHOST+0)   ='ISend/IGet        '
      gtimename(SB_IGHOST+1)   ='ISend/IGet 1      '
      gtimename(SB_IGHOST+2)   ='ISend/IGet 2      '
      gtimename(SB_IGHOST+3)   ='ISend/IGet 3      '
      gtimename(SB_IGHOST+4)   ='ISend/IGet 4      '
      gtimename(SB_IGHOST+5)   ='ISend/IGet 5      '
      gtimename(SB_MATVEC_WAIT)   ='IMatvec wait      '
      gtimename(SB_SMOOTH_WAIT)   ='ISmooth wait      '
      gtimename(SB_RESIDUAL_WAIT)   ='IResidual wait    '
      gtimename(SB_REFRESH+0)   ='Refresh BC        '
      gtimename(SB_REFRESH+1)   ='Refresh BC 1      '
      gtimename(SB_REFRESH+2)   ='Refresh BC 2      '
      gtimename(SB_REFRESH+3)   ='Refresh BC 3      '
      gtimename(SB_REFRESH+4)   ='Refresh BC 4      '
      gtimename(SB_REFRESH+5)   ='Refresh BC 5      '

C     convergence tests

      gtimename(SB_CONVERGE)   ='Convergence A     '
      gtimename(SB_CONVERGE)   ='Convergence B     '

      RETURN
      END
C=======================================================================
C     
C     Module:  mmLocalLineJacobi
C     
C     Purpose: Perform Jacobi processor-local alternating line smoothing
C
C     Inputs:  A,X,B: Linear system  
C     Inputs:  Y,R: Storage vectors
C
C     Outputs: X: Smoothed solution
C
C     Calls:   mmResidual 
C     Calls:   MPI_Barrier
C
C     Called by:
C
C     Flop count estimate:     nu (3*residual + 24*(nx-2)(ny-2)(nz-2))
C
C     Communication estimate:  3*residual (+ 3 barriers)
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmLocalLineJacobi (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,
     &     wgt,nu,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,level,icall)

      IMPLICIT NONE

      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER comm,nbr(6),bnd(3)
      
      INTEGER nx,ny,nz
      INTEGER nu
      INTEGER level
      REAL wgt
      REAL scaleA,scaleB

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz,j
      REAL m

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif
#ifdef DEBUG
      INTEGER ierr
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_SMOOTH+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmLocalLineJacobi'
#endif

      DO j=1,nu

*-----------------------------------------------------------------------
*     Compute the residual R = B - A*X 
*-----------------------------------------------------------------------

         CALL mmResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,0,level,isub)
#ifdef DEBUG
#ifdef MPI_USED
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmLineJacobi: MPI_BARRIER')") ip
#endif
         CALL MPI_BARRIER (comm,ierr)
#endif
#endif
*-----------------------------------------------------------------------
*     Do tridiagonal solves in X-direction
*-----------------------------------------------------------------------

         DO iz=2,nz-1
            DO iy=2,ny-1
               Y(2,iy,iz) = AD(2,iy,iz)
               DO ix=3,nx-1
                  m = A4(ix-1,iy,iz) / AD(ix-1,iy,iz)
                  Y(ix,iy,iz) = AD(ix,iy,iz) - m*A4(ix-1,iy,iz)
                  R(ix,iy,iz) = R(ix,iy,iz) - m*R(ix-1,iy,iz)
               END DO
               R(nx-1,iy,iz) = R(nx-1,iy,iz) / Y(nx-1,iy,iz)
               DO ix=nx-2,2,-1
                  R(ix,iy,iz) = (R(ix,iy,iz) 
     &                 - A1(ix,iy,iz)*R(ix+1,iy,iz)) / Y(ix,iy,iz)
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 8*(nx-3)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 8*(nx-3)*(ny-2)*(nz-2)
#endif
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & (ny-2)*(nz-2)
#endif

*-----------------------------------------------------------------------
*     Update X
*-----------------------------------------------------------------------

         DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
                  X(ix,iy,iz) = X(ix,iy,iz) + wgt*R(ix,iy,iz)
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif

*-----------------------------------------------------------------------
*     Compute the residual R = B - A*X 
*-----------------------------------------------------------------------

         CALL mmResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,0,level,isub)
#ifdef DEBUG
#ifdef MPI_USED
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmLineJacobi: MPI_BARRIER')") ip
#endif
         CALL MPI_BARRIER (comm,ierr)
#endif
#endif

*-----------------------------------------------------------------------
*     Do tridiagonal solves in Y-direction
*-----------------------------------------------------------------------

         DO iz=2,nz-1
            DO ix=2,nx-1
               Y(ix,2,iz) = AD(ix,2,iz)
               DO iy=3,ny-1
                  m = A5(ix,iy-1,iz) / AD(ix,iy-1,iz)
                  Y(ix,iy,iz) = AD(ix,iy,iz) - m*A5(ix,iy-1,iz)
                  R(ix,iy,iz) = R(ix,iy,iz) - m*R(ix,iy-1,iz)
               END DO
               R(ix,ny-1,iz) = R(ix,ny-1,iz) / Y(ix,ny-1,iz)
               DO iy=ny-2,2,-1
                  R(ix,iy,iz) = (R(ix,iy,iz) 
     &                 - A2(ix,iy,iz)*R(ix,iy+1,iz)) / Y(ix,iy,iz)
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 8*(nx-2)*(ny-3)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 8*(nx-2)*(ny-3)*(nz-2)
#endif
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & (nx-2)*(nz-2)
#endif

*-----------------------------------------------------------------------
*     Update Y
*-----------------------------------------------------------------------

         DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
                  X(ix,iy,iz) = X(ix,iy,iz) + wgt*R(ix,iy,iz)
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif

*-----------------------------------------------------------------------
*     Compute the residual R = B - A*X 
*-----------------------------------------------------------------------

         CALL mmResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,0,level,isub)
#ifdef DEBUG
#ifdef MPI_USED
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmLineJacobi: MPI_BARRIER')") ip
#endif
         CALL MPI_BARRIER (comm,ierr)
#endif
#endif

*-----------------------------------------------------------------------
*     Do tridiagonal solves in Z-direction
*-----------------------------------------------------------------------

         DO iy=2,ny-1
            DO ix=2,nx-1
               Y(ix,iy,2) = AD(ix,iy,2)
               DO iz=3,nz-1
                  m = A6(ix,iy,iz-1) / AD(ix,iy,iz-1)
                  Y(ix,iy,iz) = AD(ix,iy,iz) - m*A6(ix,iy,iz-1)
                  R(ix,iy,iz) = R(ix,iy,iz) - m*R(ix,iy,iz-1)
               END DO
               R(ix,iy,nz-1) = R(ix,iy,nz-1) / Y(ix,iy,nz-1)
               DO iz=nz-2,2,-1
                  R(ix,iy,iz) = (R(ix,iy,iz) 
     &                 - A3(ix,iy,iz)*R(ix,iy,iz+1)) / Y(ix,iy,iz)
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 8*(nx-2)*(ny-2)*(nz-3)
      gmflops(isub)=gmflops(isub) + 
     & 8*(nx-2)*(ny-2)*(nz-3)
#endif
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (ny-2)*(nx-2)
      gmflops(isub)=gmflops(isub) + 
     & (ny-2)*(nx-2)
#endif

*-----------------------------------------------------------------------
*     Update Z
*-----------------------------------------------------------------------

         DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
                  X(ix,iy,iz) = X(ix,iy,iz) + wgt*R(ix,iy,iz)
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif


      END DO

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmLocalLineJacobi'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmPlaneJacobi
C     
C     PURPOSE: Plane Jacobi using conjugate gradient plane solvers
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C     GLOBALS: gtimestart,gtimecount,gtime
C     GLOBALS: gmflops
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmPlaneJacobi (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &     nx,ny,nz,nxg,nyg,nzg,wgt,nu,iSmoothIt,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,scaleA,scaleB,
     &     level,icall)

      IMPLICIT NONE

      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxdrsm,nydrsm,nzdrsm
      INTEGER bndrsm(3)
      REAL RSM(nxdrsm,nydrsm,nzdrsm)
      
      INTEGER nxdpsm,nydpsm,nzdpsm
      INTEGER bndpsm(3)
      REAL PSM(nxdpsm,nydpsm,nzdpsm)
      
      INTEGER nxdwsm,nydwsm,nzdwsm
      INTEGER bndzsm(3)
      REAL WSM(nxdwsm,nydwsm,nzdwsm)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER comm,nbr(6),bnd(3)
      
      INTEGER nx,ny,nz
      INTEGER nxg,nyg,nzg
      INTEGER nu,iSmoothIt
      INTEGER level
      REAL wgt
      REAL scaleA,scaleB

#include "mgmpi_internal.h"

#ifdef DEBUG_SMOOTH
      REAL s,mmVectorSum
      EXTERNAL mmVectorSum
#endif

#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz,j
      INTEGER numIterCg,outputLevel,iStat
      REAL rTol,rnorm(NMAXLIM),resRedFine
      LOGICAL lx,ly,lz,mask(NMAXLIM)

      INTEGER tempCount
      DATA tempCount/0/

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_SMOOTH+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmPlaneJacobi enter'
#endif

C     -------------------------------
C     Initialize CG solver parameters
C     -------------------------------

      rTol = 1e-12
      outputLevel = 0

C     ---------------
C     Smooth nu times
C     ---------------

      DO j=1,nu

*-----------------------------------------------------------------------
*     Compute the residual R = B - A*X 
*-----------------------------------------------------------------------

         lx = .FALSE.
         ly = .TRUE.
         lz = .TRUE.

         mask(1)=.FALSE.
         DO ix=2,nx-1
            mask(ix) = .TRUE.
         END DO
         mask(nx)=.FALSE.

         CALL mmVectorClear (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz)

#ifdef DEBUG_SMOOTH
      if (ip.EQ.0) PRINT*, 'mmPlaneJacobi Residual FTT: level=',level
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,' mmPlaneJacobi Residual FTT')") ip
#endif
#endif 
         CALL mmResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,0,level,isub)

#ifdef DEBUG_SMOOTH
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(R) = ',s
#endif

*-----------------------------------------------------------------------
*     Do plane solves in Y-Z planes
*-----------------------------------------------------------------------

         numIterCg = 0

#ifdef DEBUG_SMOOTH
      if (ip.EQ.0) PRINT*, 'mmPlaneJacobi CG2 FTT'
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,' mmPlaneJacobi GC2 FTT')") ip
#endif
#endif 
         CALL mmCg2 (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,nx,ny,nz,nxg,nyg,nzg,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        numIterCg,iSmoothIt,rtol,outputLevel,
     &        
     &     comm,nbr,bnd
     &     ,scaleA,scaleB,lx,ly,lz,mask,level,
     &        rnorm,resRedFine,iStat,isub)

#ifdef DEBUG_SMOOTH
      s = mmVectorSum (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(Y) = ',s
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(R) = ',s
#endif

         IF (iStat.NE.0 .AND.outputLevel.NE.0 .AND. ip.EQ.0) THEN
            PRINT*, '-- mgmpi warning -- Coarse solver returned ',
     &           iStat
         END IF

*-----------------------------------------------------------------------
*     Update X
*-----------------------------------------------------------------------

#ifdef DEBUG_SMOOTH
      if (ip.EQ.0) PRINT*, 'mmPlaneJacobi update FTT'
#endif 
         DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
                  X(ix,iy,iz) = X(ix,iy,iz) + wgt*Y(ix,iy,iz)
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif

#ifdef DEBUG_SMOOTH
      s = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(X) = ',s
      s = mmVectorSum (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(Y) = ',s
#endif

*-----------------------------------------------------------------------
*     Compute the residual R = B - A*X 
*-----------------------------------------------------------------------

         lx = .TRUE.
         ly = .FALSE.
         lz = .TRUE.

         mask(1)=.FALSE.
         DO iy=2,ny-1
            mask(iy) = .TRUE.
         END DO
         mask(ny)=.FALSE.

#ifdef DEBUG_SMOOTH
      if (ip.EQ.0) PRINT*, 'mmPlaneJacobi Residual TFT: level=',level
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,' mmPlaneJacobi Residual TFT')") ip
#endif
#endif 
         CALL mmResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,0,level,isub)

#ifdef DEBUG_SMOOTH
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(R) = ',s
#endif

*-----------------------------------------------------------------------
*     Do plane solves in Z-X planes
*-----------------------------------------------------------------------

         numIterCg = 0

#ifdef DEBUG_SMOOTH
      if (ip.EQ.0) PRINT*, 'mmPlaneJacobi CG2 TFT'
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,' mmPlaneJacobi GC2 TFT')") ip
#endif
#endif 
         CALL mmCg2 (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,nx,ny,nz,nxg,nyg,nzg,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        numIterCg,iSmoothIt,rtol,outputLevel,
     &        
     &     comm,nbr,bnd
     &     ,scaleA,scaleB,lx,ly,lz,mask,level,
     &        rnorm,resRedFine,iStat,isub)
#ifdef DEBUG_SMOOTH
      s = mmVectorSum (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(Y) = ',s
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(R) = ',s
#endif
         IF (iStat.NE.0 .AND.outputLevel.NE.0 .AND. ip.EQ.0) THEN
            PRINT*, '-- mgmpi warning -- Coarse solver returned ',
     &           iStat
         END IF

*-----------------------------------------------------------------------
*     Update Y
*-----------------------------------------------------------------------

#ifdef DEBUG_SMOOTH
      if (ip.EQ.0) PRINT*, 'mmPlaneJacobi update TFT'
#endif 
         DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
                  X(ix,iy,iz) = X(ix,iy,iz) + wgt*Y(ix,iy,iz)
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif

#ifdef DEBUG_SMOOTH
      s = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(X) = ',s
      s = mmVectorSum (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(Y) = ',s
#endif

*-----------------------------------------------------------------------
*     Compute the residual R = B - A*X 
*-----------------------------------------------------------------------

         lx = .TRUE.
         ly = .TRUE.
         lz = .FALSE.

         mask(1)=.FALSE.
         DO iz=2,nz-1
            mask(iz) = .TRUE.
         END DO
         mask(nz)=.FALSE.

#ifdef DEBUG_SMOOTH
      if (ip.EQ.0) PRINT*, 'mmPlaneJacobi Residual TTF: level=',level
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,' mmPlaneJacobi Residual TTF')") ip
#endif
#endif 
         CALL mmResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,0,level,isub)

#ifdef DEBUG_SMOOTH
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(R) = ',s
#endif

*-----------------------------------------------------------------------
*     Do plane solves in X-Y planes
*-----------------------------------------------------------------------

         numIterCg = 0

#ifdef DEBUG_SMOOTH
      if (ip.EQ.0) PRINT*, 'mmPlaneJacobi CG2 TTF'
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,' mmPlaneJacobi GC2 TTF')") ip
#endif
#endif 
         CALL mmCg2 (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,nx,ny,nz,nxg,nyg,nzg,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        numIterCg,iSmoothIt,rtol,outputLevel,
     &        
     &     comm,nbr,bnd
     &     ,scaleA,scaleB,lx,ly,lz,mask,level,
     &        rnorm,resRedFine,iStat,isub)

#ifdef DEBUG_SMOOTH
      s = mmVectorSum (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(Y) = ',s
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(R) = ',s
#endif

         IF (iStat.NE.0 .AND.outputLevel.NE.0 .AND. ip.EQ.0) THEN
            PRINT*, '-- mgmpi warning -- Coarse solver returned ',
     &           iStat
         END IF

*-----------------------------------------------------------------------
*     Update Z
*-----------------------------------------------------------------------

#ifdef DEBUG_SMOOTH
      if (ip.EQ.0) PRINT*, 'mmPlaneJacobi update TTF'
#endif 
         DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
                  X(ix,iy,iz) = X(ix,iy,iz) + wgt*Y(ix,iy,iz)
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif

#ifdef DEBUG_SMOOTH
      s = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(X) = ',s
      s = mmVectorSum (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(Y) = ',s
#endif

      END DO

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmPlaneJacobi enter'
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmPlaneJacobiMg
C     
C     Purpose: Plane Jacobi using 2D multigrid plane solvers
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmPlaneJacobiMg (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &     nx,ny,nz,nxg,nyg,nzg,wgt,nu,iSmoothIt,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,scaleA,scaleB,
     &     level,icall)

      IMPLICIT NONE

      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxdrsm,nydrsm,nzdrsm
      INTEGER bndrsm(3)
      REAL RSM(nxdrsm,nydrsm,nzdrsm)
      
      INTEGER nxdpsm,nydpsm,nzdpsm
      INTEGER bndpsm(3)
      REAL PSM(nxdpsm,nydpsm,nzdpsm)
      
      INTEGER nxdwsm,nydwsm,nzdwsm
      INTEGER bndzsm(3)
      REAL WSM(nxdwsm,nydwsm,nzdwsm)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER comm,nbr(6),bnd(3)
      
      INTEGER nx,ny,nz
      INTEGER nxg,nyg,nzg
      INTEGER nu,iSmoothIt
      INTEGER level
      REAL wgt
      REAL scaleA,scaleB

#include "mgmpi_internal.h"

#ifdef DEBUG_SMOOTH
      REAL s,mmVectorSum
      EXTERNAL mmVectorSum
#endif

#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER j
      LOGICAL lx,ly,lz
      INTEGER numIterCg,outputLevel,iStat
      REAL rTol,rnorm(NMAXLIM),resRedFine
      LOGICAL mask(NMAXLIM)
      INTEGER ix,iy,iz

      INTEGER tempCount
      DATA tempCount/0/

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_SMOOTH+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmPlaneJacobiMg'
#endif

C     -------------------------------
C     Initialize CG solver parameters
C     -------------------------------

      rTol = 1e-12
      outputLevel = 0

C     ---------------
C     Smooth nu times
C     ---------------

      DO j=1,nu

*-----------------------------------------------------------------------
*     Compute the residual R = B - A*X 
*-----------------------------------------------------------------------

         lx = .FALSE.
         ly = .TRUE.
         lz = .TRUE.

         mask(1)=.FALSE.
         DO ix=2,nx-1
            mask(ix) = .TRUE.
         END DO
         mask(nx)=.FALSE.

         CALL mmVectorClear (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz)

#ifdef DEBUG_SMOOTH
      if (ip.EQ.0) PRINT*, 'mmPlaneJacobi Residual 1: level=',level
#endif 
         CALL mmResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,0,level,isub)

#ifdef DEBUG_SMOOTH
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(R) = ',s
#endif

*-----------------------------------------------------------------------
*     Do plane solves in Y-Z planes
*-----------------------------------------------------------------------

         numIterCg = 0

#ifdef DEBUG_SMOOTH
      if (ip.EQ.0) PRINT*, 'mmPlaneJacobi CG2 FTT'
#endif 
         CALL mmCg2 (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,nx,ny,nz,nxg,nyg,nzg,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        numIterCg,iSmoothIt,rtol,outputLevel,
     &        
     &     comm,nbr,bnd
     &     ,scaleA,scaleB,lx,ly,lz,mask,level,
     &        rnorm,resRedFine,iStat,isub)

#ifdef DEBUG_SMOOTH
      s = mmVectorSum (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(Y) = ',s
#endif

         IF (iStat.NE.0 .AND.outputLevel.NE.0 .AND. ip.EQ.0) THEN
            PRINT*, '-- mgmpi warning -- Coarse solver returned ',
     &           iStat
         END IF

*-----------------------------------------------------------------------
*     Update X
*-----------------------------------------------------------------------

#ifdef DEBUG_SMOOTH
      if (ip.EQ.0) PRINT*, 'mmPlaneJacobi update FTT'
#endif 
         DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
                  X(ix,iy,iz) = X(ix,iy,iz) + wgt*Y(ix,iy,iz)
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif

#ifdef DEBUG_SMOOTH
      s = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(Y) = ',s
#endif

*-----------------------------------------------------------------------
*     Compute the residual R = B - A*X 
*-----------------------------------------------------------------------

         lx = .TRUE.
         ly = .FALSE.
         lz = .TRUE.

         mask(1)=.FALSE.
         DO iy=2,ny-1
            mask(iy) = .TRUE.
         END DO
         mask(ny)=.FALSE.

#ifdef DEBUG_SMOOTH
      if (ip.EQ.0) PRINT*, 'mmPlaneJacobi Residual 2: level=',level
#endif 
         CALL mmResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,0,level,isub)

#ifdef DEBUG_SMOOTH
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(R) = ',s
#endif

*-----------------------------------------------------------------------
*     Do plane solves in Z-X planes
*-----------------------------------------------------------------------

         numIterCg = 0

#ifdef DEBUG_SMOOTH
      if (ip.EQ.0) PRINT*, 'mmPlaneJacobi CG2 TFT'
#endif 
         CALL mmCg2 (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,nx,ny,nz,nxg,nyg,nzg,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        numIterCg,iSmoothIt,rtol,outputLevel,
     &        
     &     comm,nbr,bnd
     &     ,scaleA,scaleB,lx,ly,lz,mask,level,
     &        rnorm,resRedFine,iStat,isub)
#ifdef DEBUG_SMOOTH
      s = mmVectorSum (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(Y) = ',s
#endif
         IF (iStat.NE.0 .AND.outputLevel.NE.0 .AND. ip.EQ.0) THEN
            PRINT*, '-- mgmpi warning -- Coarse solver returned ',
     &           iStat
         END IF

*-----------------------------------------------------------------------
*     Update Y
*-----------------------------------------------------------------------

#ifdef DEBUG_SMOOTH
      if (ip.EQ.0) PRINT*, 'mmPlaneJacobi update TFT'
#endif 
         DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
                  X(ix,iy,iz) = X(ix,iy,iz) + wgt*Y(ix,iy,iz)
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif

#ifdef DEBUG_SMOOTH
      s = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(Y) = ',s
#endif

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif

*-----------------------------------------------------------------------
*     Compute the residual R = B - A*X 
*-----------------------------------------------------------------------

         lx = .TRUE.
         ly = .TRUE.
         lz = .FALSE.

         mask(1)=.FALSE.
         DO iz=2,nz-1
            mask(iz) = .TRUE.
         END DO
         mask(nz)=.FALSE.

#ifdef DEBUG_SMOOTH
      if (ip.EQ.0) PRINT*, 'mmPlaneJacobi Residual 3: level=',level
#endif 
         CALL mmResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,0,level,isub)

#ifdef DEBUG_SMOOTH
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(R) = ',s
#endif

*-----------------------------------------------------------------------
*     Do plane solves in X-Y planes
*-----------------------------------------------------------------------

         numIterCg = 0

#ifdef DEBUG_SMOOTH
      if (ip.EQ.0) PRINT*, 'mmPlaneJacobi CG2 TTF'
#endif 
         CALL mmCg2 (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,nx,ny,nz,nxg,nyg,nzg,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        numIterCg,iSmoothIt,rtol,outputLevel,
     &        
     &     comm,nbr,bnd
     &     ,scaleA,scaleB,lx,ly,lz,mask,level,
     &        rnorm,resRedFine,iStat,isub)

#ifdef DEBUG_SMOOTH
      s = mmVectorSum (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(Y) = ',s
#endif

         IF (iStat.NE.0 .AND.outputLevel.NE.0 .AND. ip.EQ.0) THEN
            PRINT*, '-- mgmpi warning -- Coarse solver returned ',
     &           iStat
         END IF

*-----------------------------------------------------------------------
*     Update Z
*-----------------------------------------------------------------------

#ifdef DEBUG_SMOOTH
      if (ip.EQ.0) PRINT*, 'mmPlaneJacobi update TTF'
#endif 
         DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
                  X(ix,iy,iz) = X(ix,iy,iz) + wgt*Y(ix,iy,iz)
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif

#ifdef DEBUG_SMOOTH
      s = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '  sum(Y) = ',s
#endif

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif

      END DO

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmPlaneJacobiMg'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmMatrixClear
C     
C     PURPOSE: Clear the given matrix
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmMatrixClear (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,nx,ny,nz)

      IMPLICIT NONE

      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      INTEGER nx,ny,nz

#include "mgmpi_internal.h"

      INTEGER ix,iy,iz

      DO iz=1,nz
         DO iy=1,ny
            DO ix=1,nx
               AD(ix,iy,iz) = 0.0D0
               A1(ix,iy,iz) = 0.0D0
               A2(ix,iy,iz) = 0.0D0
               A3(ix,iy,iz) = 0.0D0
               A4(ix,iy,iz) = 0.0D0
               A5(ix,iy,iz) = 0.0D0
               A6(ix,iy,iz) = 0.0D0
            END DO
         END DO
      END DO

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmMatvec
C     
C     PURPOSE: Matrix-vector product
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmMatvec (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      LOGICAL isFirstCall
      DATA isFirstCall/.TRUE./

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MATVEC+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmMatvec'
#endif

      IF (gimatfree.EQ.1) THEN
#ifndef MPI_BLOCKING
         IF (ip.EQ.0.AND.isFirstCall) THEN 
            PRINT*, '-- mgmpi warning -- Matrix-free nonblocking version'
            PRINT*, '                    not implemented; continuing with'
            PRINT*, '                    blocking MPI calls.'
            isFirstCall = .FALSE.
         END IF
#endif
      END IF
      
      IF (gimatfree.EQ.1) THEN

         IF (gicoords.EQ.MM_CARTESIAN) THEN

            IF (giGridType.EQ.MM_IS_UNIFORM) THEN
               CALL mmBMFMatvecU (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &              
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &              level,isub)
            ELSE
               CALL mmBMFMatvecNU (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &              
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &              level,isub)
            END IF

         ELSE

            CALL mmBMFMatvecGen (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &           level,isub)

         END IF

      ELSE

#ifdef MPI_BLOCKING

         CALL mmBMatvec (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &        
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,level,isub)

#else 

         CALL mmIMatvec (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &        
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,level,isub)

#endif

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmMatvec'
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmBMatvec 
C     
C     Purpose: Matrix-vector product with blocking communication
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBMatvec (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      INTEGER ix,iy,iz

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MATVEC+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmBMatvec'
#endif

*-----------------------------------------------------------------------
*     Refresh the ghost points of X
*-----------------------------------------------------------------------

      CALL mmGetGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)

*-----------------------------------------------------------------------
*     Compute local Y = A*X
*-----------------------------------------------------------------------

      DO iz=2,nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               Y(ix,iy,iz) = AD(ix,iy,iz) * X(ix,iy,iz)
     &              + A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &              + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &              + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
            END DO
            DO ix=2,nx-1
               Y(ix,iy,iz) = Y(ix,iy,iz)
     &              + A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &              + A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &              + A3(ix,iy,iz)  *X(ix,iy,iz+1)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-2)*(ny-2)*(nz-2)
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmBMatvec'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmIMatvec
C     
C     PURPOSE: Matrix-free matrix-vector product using nonblocking 
C     PURPOSE: communication
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmIMatvec (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER i,ix,iy,iz

#ifdef MPI_USED
      INTEGER status(MPI_STATUS_SIZE,12),ierr
      INTEGER req(12)
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MATVEC+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmIMatvec'
#endif
*-----------------------------------------------------------------------
*     Start sending ghost points of X
*-----------------------------------------------------------------------

#ifdef MPI_USED

*     NONBLOCKING COMMUNICATION

      icommtag=icommtag+6
      IF (icommtag.GT.32757) icommtag=ICOMMSTART

      DO i=1,12
         req(i)=MPI_REQUEST_NULL
      END DO

      CALL mmIGetX1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(1),req(2),isub)
      CALL mmIGetY1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(3),req(4),isub)
      CALL mmIGetZ1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(5),req(6),isub)
      CALL mmIGetXnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(7),req(8),isub)
      CALL mmIGetYnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(9),req(10),isub)
      CALL mmIGetZnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(11),req(12),isub)

#endif

*-----------------------------------------------------------------------
*     Interior points
*-----------------------------------------------------------------------

      DO iz=3,nz-2
         DO iy=3,ny-2
            DO ix=3,nx-2
               Y(ix,iy,iz) = 
     &                A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &              + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &              + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &              + AD(ix,iy,iz)    *X(ix,iy,iz)
            END DO
            DO ix=3,nx-2
               Y(ix,iy,iz) = Y(ix,iy,iz)
     &              + A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &              + A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &              + A3(ix,iy,iz)  *X(ix,iy,iz+1)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-4)*(ny-4)*(nz-4)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-4)*(ny-4)*(nz-4)
#endif

*     wait for boundaries

      
#ifdef PERF_EVAL
      gtimestart(SB_MATVEC_WAIT) = MPI_WTIME()
#endif

#ifdef MPI_USED
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmIMatvec: MPI_WAITALL')") ip
#endif
      CALL MPI_WAITALL (12,req,status,ierr)
      IF (ierr.NE.0) print*, 'MPI Error in mmIMatvec wait: ',ierr
#endif

      
#ifdef PERF_EVAL
      gtime(SB_MATVEC_WAIT) = gtime(SB_MATVEC_WAIT) + MPI_WTIME() - gtimestart(SB_MATVEC_WAIT)
      gtimecount(SB_MATVEC_WAIT) = gtimecount(SB_MATVEC_WAIT) + 1
#endif

*-----------------------------------------------------------------------
*     X-facets
*-----------------------------------------------------------------------

      DO iz=2,nz-1
         DO iy=2,ny-1
            ix=2
            Y(ix,iy,iz) = 
     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            ix=3
*            Y(ix,iy,iz) = 
*     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            ix=nx-2
*            Y(ix,iy,iz) = 
*     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
            ix=nx-1
            Y(ix,iy,iz) = 
     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
         END DO
      END DO
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(ny-2)*(nz-2)
#endif
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(ny-2)*(nz-2)
#endif


*-----------------------------------------------------------------------
*     Y-facets
*-----------------------------------------------------------------------

      DO iz=2,nz-1
         DO ix=2,nx-1
            iy=2
            Y(ix,iy,iz) = 
     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            iy=3
*            Y(ix,iy,iz) = 
*     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            iy=ny-2
*            Y(ix,iy,iz) = 
*     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
            iy=ny-1
            Y(ix,iy,iz) = 
     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
         END DO
      END DO
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-2)*(nz-2)
#endif
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-2)*(nz-2)
#endif

*-----------------------------------------------------------------------
*     Z-facets
*-----------------------------------------------------------------------

      DO iy=2,ny-1
         DO ix=2,nx-1
            iz=2
            Y(ix,iy,iz) = 
     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            iz=3
*            Y(ix,iy,iz) = 
*     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            iz=nz-2
*            Y(ix,iy,iz) = 
*     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
            iz=nz-1
            Y(ix,iy,iz) = 
     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
         END DO
      END DO
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-2)*(ny-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-2)*(ny-2)
#endif
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-2)*(ny-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-2)*(ny-2)
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmIMatvec'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmBMFMatvecGen 
C     
C     PURPOSE: Matrix-free matrix-vector product for general problems
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBMFMatvecGen (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz
      REAL hxm,hym,hzm,hxp,hyp,hzp
      REAL xm2,ym2,zm2,xp2,yp2,zp2
      REAL dxs,dys,dzs
      REAL x0,y0

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MATVEC+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmBMFMatvecGen'
#endif

      IF (level.GT.1) THEN

         CALL mmBMatvec (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &        
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,level,isub)

      ELSE

*-----------------------------------------------------------------------
*     Refresh the ghost points of X
*-----------------------------------------------------------------------

         CALL mmGetGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)

         DO iz=2,nz-1
            dzs = gdz(iz-1,1)+gdz(iz,1)
            hzm=zp(iz)-zp(iz-1)
            hzp=zp(iz+1)-zp(iz)
            zm2 = o2*(zp(iz-1)+zp(iz))
            zp2 = o2*(zp(iz)+zp(iz+1))
            DO iy=2,ny-1
               dys = gdy(iy-1,1)+gdy(iy,1)
               hym=yp(iy)-yp(iy-1)
               hyp=yp(iy+1)-yp(iy)
               ym2 = o2*(yp(iy-1)+yp(iy))
               yp2 = o2*(yp(iy)+yp(iy+1))
               y0 = yp(iy)
               DO ix=2,nx-1
                  dxs = gdx(ix-1,1)+gdx(ix,1)
                  hxm=xp(ix)-xp(ix-1)
                  hxp=xp(ix+1)-xp(ix)
                  xm2 = o2*(xp(ix-1)+xp(ix))
                  xp2 = o2*(xp(ix)+xp(ix+1))
                  x0 = xp(ix)
                  Y(ix,iy,iz) = o4*gScaleA*(
     &                      dys*dzs/hxp*gg2off(ix,1)*gg31off(ix,1)
     &                 * (X(ix+1,iy,iz) - X(ix,iy,iz))
     &                 +    dzs*dxs/hyp*gg32off(iy,1)/gg2on(ix,1)**2
     &                 * (X(ix,iy+1,iz) - X(ix,iy,iz))
     &                 +    dxs*dys/hzp/(gg31on(ix,1)**2
     &                 * gg32on(iy,1)**2)
     &                 * (X(ix,iy,iz+1) - X(ix,iy,iz))
     &                 +    dys*dzs/hxm * gg2off(ix-1,1)
     &                 * gg31off(ix-1,1)   
     &                 * (X(ix-1,iy,iz) - X(ix,iy,iz))
     &                 +    dzs*dxs/hym*gg32off(iy-1,1)
     &                 / gg2on(ix,1)**2
     &                 * (X(ix,iy-1,iz) - X(ix,iy,iz))
     &                 +    dxs*dys/hzm/(gg31on(ix,1)**2
     &                 * gg32on(iy,1)**2)
     &                 * (X(ix,iy,iz-1) - X(ix,iy,iz)))
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 49*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 49*(nx-2)*(ny-2)*(nz-2)
#endif

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmBMFMatvecGen'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmBMFMatvecNU 
C     
C     PURPOSE: Matrix-free matrix-vector product for nonuniform meshes
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBMFMatvecNU (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz
      REAL hxm,hym,hzm,hxp,hyp,hzp
      REAL dxs,dys,dzs,dxp,dyp,dzp,dxm,dym,dzm

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MATVEC+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      IF (level.GT.1) THEN

         CALL mmBMatvec (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &        
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,level,isub)

      ELSE

*-----------------------------------------------------------------------
*     Refresh the ghost points of X
*-----------------------------------------------------------------------

         CALL mmGetGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)

         dzs = zp(3)-zp(1)
         hzm = zp(2)-zp(1)
         hzp = zp(3)-zp(2)
         DO iz=2,nz-1
            dys = yp(3)-yp(1)
            hym = yp(2)-yp(1)
            hyp = yp(3)-yp(2)
            DO iy=2,ny-1
               dxs = xp(3)-xp(1)
               hxm = xp(2)-xp(1)
               hxp = xp(3)-xp(2)
               DO ix=2,nx-1
                  dxp=dys*dzs/hxp
                  dxm=dys*dzs/hxm
                  dyp=dzs*dxs/hyp
                  dym=dzs*dxs/hym
                  dzp=dxs*dys/hzp
                  dzm=dxs*dys/hzm
                  Y(ix,iy,iz) = o4*gScaleA*(
     &                 -(dxp+dyp+dzp+dxm+dym+dzm)*X(ix,iy,iz)
     &                 + dxp * X(ix+1,iy,iz)
     &                 + dyp * X(ix,iy+1,iz)
     &                 + dzp * X(ix,iy,iz+1)
     &                 + dxm * X(ix-1,iy,iz)
     &                 + dym * X(ix,iy-1,iz)
     &                 + dzm * X(ix,iy,iz-1))
                  dxs = xp(ix+2)-xp(ix)
                  hxm=hxp
                  hxp=xp(ix+2)-xp(ix+1)
               END DO
               dys = yp(iy+2)-yp(iy)
               hym=hyp
               hyp=yp(iy+2)-yp(iy+1)
            END DO
            dzs = zp(iz+2)-zp(iz)
            hzm=hzp
            hzp=zp(iz+2)-zp(iz+1)
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 34*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 34*(nx-2)*(ny-2)*(nz-2)
#endif

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmBMFMatvecU 
C     
C     Purpose: Matrix-free matrix-vector product for uniform meshes
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBMFMatvecU (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER comm,nbr(6),bnd(3)


      INTEGER nx,ny,nz
      INTEGER level

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz
      REAL dx,dy,dz,dsum

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MATVEC+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmBMFMatvecU'
#endif

      IF (level.GT.1) THEN

         CALL mmBMatvec (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &        
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,level,isub)

      ELSE

*-----------------------------------------------------------------------
*     Refresh the ghost points of X
*-----------------------------------------------------------------------

         CALL mmGetGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)

         dx = gScaleA*hy(2)*hz(2)/hx(2)
         dy = gScaleA*hz(2)*hx(2)/hy(2)
         dz = gScaleA*hx(2)*hy(2)/hz(2)
         dsum = -2.0D0*(dx+dy+dz)

         DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
                  Y(ix,iy,iz) = dsum*X(ix,iy,iz)
     &                 + dx*(X(ix+1,iy,iz) + X(ix-1,iy,iz))
     &                 + dy*(X(ix,iy+1,iz) + X(ix,iy-1,iz))
     &                 + dz*(X(ix,iy,iz+1) + X(ix,iy,iz-1))
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 10*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 10*(nx-2)*(ny-2)*(nz-2)
#endif

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmBMFMatvecU'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmMatvec
C     
C     PURPOSE: Matrix-vector product
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmMatvec2 (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,mask,
     &     level,icall)

      IMPLICIT NONE

      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level
      LOGICAL lx,ly,lz,mask(*)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      LOGICAL isFirstCall
      DATA isFirstCall/.TRUE./

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MATVEC2+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef ASSERT
      IF (.NOT.(((.NOT.lx).AND.ly.AND.lz) .OR.
     &          (lx.AND.(.NOT.ly).AND.lz) .OR.
     &          (lx.AND.ly.AND.(.NOT.lz)))) THEN
         PRINT*, 'mmMatvec2 ASSERTION FAILED!'
         PRINT*, 'lx ly lz =',lx,ly,lz
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> Entering mmMatvec2'
#endif

      IF (gimatfree.EQ.1) THEN
#ifndef MPI_BLOCKING
         IF (ip.EQ.0.AND.isFirstCall) THEN 
          PRINT*, '-- mgmpi warning -- Matrix-free nonblocking version'
          PRINT*, '                    not implemented; continuing with'
          PRINT*, '                    blocking MPI calls.'
            isFirstCall = .FALSE.
         END IF
#endif
      END IF
      
      IF (gimatfree.EQ.1) THEN

         IF (gicoords.EQ.MM_CARTESIAN) THEN

            IF (giGridType.EQ.MM_IS_UNIFORM) THEN
               CALL mmBMFMatvecU2 (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &              
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &              lx,ly,lz,mask,level,isub)
            ELSE
               CALL mmBMFMatvecNU2 (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &              
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &              lx,ly,lz,mask,level,isub)
            END IF

         ELSE

            CALL mmBMFMatvecGen2 (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &           lx,ly,lz,mask,level,isub)

         END IF

      ELSE

#ifdef MPI_BLOCKING

         CALL mmBMatvec2 (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &        
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        lx,ly,lz,mask,level,isub)

#else 

         CALL mmIMatvec2 (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &        
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        lx,ly,lz,mask,level,isub)

#endif

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< Entering mmMatvec2'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C
C     MODULE:  SUBROUTINE mmBMatvec 
C
C     PURPOSE: Matrix-vector product with blocking communication
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBMatvec2 (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,mask,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level
      LOGICAL lx,ly,lz,mask(*)

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      INTEGER ix,iy,iz

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MATVEC2+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmBMatvec2: level=',level
#endif

*-----------------------------------------------------------------------
*     Refresh the ghost points of X
*-----------------------------------------------------------------------

      IF (lx) THEN
         CALL mmGetX1Ghost2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,isub)
         CALL mmGetXnGhost2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,isub)
      END IF
      IF (ly) THEN
         CALL mmGetY1Ghost2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,isub)
         CALL mmGetYnGhost2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,isub)
      END IF
      IF (lz) THEN
         CALL mmGetZ1Ghost2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,isub)
         CALL mmGetZnGhost2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,isub)
      END IF

*-----------------------------------------------------------------------
*     Compute local Y = A*X
*-----------------------------------------------------------------------

      IF (.NOT.lx) THEN
         DO ix=2,nx-1
            IF (mask(ix)) THEN
               DO iz=2,nz-1
                  DO iy=2,ny-1
                     Y(ix,iy,iz) = 
     &                      AD(ix,iy,iz)  *X(ix,iy,iz)
     &                    + A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &                    + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
                  END DO
                  DO iy=2,ny-1
                     Y(ix,iy,iz) = Y(ix,iy,iz) 
     &                    + A3(ix,iy,iz)  *X(ix,iy,iz+1)
     &                    + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 9*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 9*(ny-2)*(nz-2)
#endif
            END IF
         END DO
      ELSE IF (.NOT.ly) THEN
         DO iy=2,ny-1
            IF (mask(iy)) THEN
               DO iz=2,nz-1
                  DO ix=2,nx-1
                     Y(ix,iy,iz) = 
     &                    AD(ix,iy,iz)  *X(ix,iy,iz)
     &                    + A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &                    + A4(ix-1,iy,iz)*X(ix-1,iy,iz)
                  END DO
                  DO ix=2,nx-1
                     Y(ix,iy,iz) = Y(ix,iy,iz) 
     &                    + A3(ix,iy,iz)  *X(ix,iy,iz+1)
     &                    + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 9*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 9*(nx-2)*(nz-2)
#endif
            END IF
         END DO
      ELSE IF (.NOT.lz) THEN
         DO iz=2,nz-1
            IF (mask(iz)) THEN
               DO iy=2,ny-1
                  DO ix=2,nx-1
                     Y(ix,iy,iz) = 
     &                    AD(ix,iy,iz)  *X(ix,iy,iz)
     &                    + A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &                    + A4(ix-1,iy,iz)*X(ix-1,iy,iz)
                  END DO
                  DO ix=2,nx-1
                     Y(ix,iy,iz) = Y(ix,iy,iz) 
     &                    + A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &                    + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 9*(nx-2)*(ny-2)
      gmflops(isub)=gmflops(isub) + 
     & 9*(nx-2)*(ny-2)
#endif
            END IF
         END DO
      END IF            

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmBMatvec2: level=',level
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmIMatvec
C     
C     PURPOSE: Matrix-free matrix-vector product using nonblocking 
C     PURPOSE: communication
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmIMatvec2 (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level
      LOGICAL lx,ly,lz

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER i,ix,iy,iz

#ifdef MPI_USED
      INTEGER status(MPI_STATUS_SIZE,12),ierr
      INTEGER req(12)
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MATVEC2+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmIMatvec2'
#endif
*-----------------------------------------------------------------------
*     Start sending ghost points of X
*-----------------------------------------------------------------------

#ifdef MPI_USED

*     NONBLOCKING COMMUNICATION

      icommtag=icommtag+6
      IF (icommtag.GT.32757) icommtag=ICOMMSTART

      DO i=1,12
         req(i)=MPI_REQUEST_NULL
      END DO

      CALL mmIGetX1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(1),req(2),isub)
      CALL mmIGetY1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(3),req(4),isub)
      CALL mmIGetZ1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(5),req(6),isub)
      CALL mmIGetXnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(7),req(8),isub)
      CALL mmIGetYnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(9),req(10),isub)
      CALL mmIGetZnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(11),req(12),isub)

#endif

*-----------------------------------------------------------------------
*     Interior points
*-----------------------------------------------------------------------

      DO iz=3,nz-2
         DO iy=3,ny-2
            DO ix=3,nx-2
               Y(ix,iy,iz) = 
     &                A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &              + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &              + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &              + AD(ix,iy,iz)    *X(ix,iy,iz)
            END DO
            DO ix=3,nx-2
               Y(ix,iy,iz) = Y(ix,iy,iz)
     &              + A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &              + A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &              + A3(ix,iy,iz)  *X(ix,iy,iz+1)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-4)*(ny-4)*(nz-4)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-4)*(ny-4)*(nz-4)
#endif

*     wait for boundaries

      
#ifdef PERF_EVAL
      gtimestart(SB_MATVEC_WAIT) = MPI_WTIME()
#endif

#ifdef MPI_USED
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmIMatvec2: MPI_WAITALL')") ip
#endif
      CALL MPI_WAITALL (12,req,status,ierr)
      IF (ierr.NE.0) print*, 'MPI Error in mmIMatvec2 wait: ',ierr
#endif

      
#ifdef PERF_EVAL
      gtime(SB_MATVEC_WAIT) = gtime(SB_MATVEC_WAIT) + MPI_WTIME() - gtimestart(SB_MATVEC_WAIT)
      gtimecount(SB_MATVEC_WAIT) = gtimecount(SB_MATVEC_WAIT) + 1
#endif

*-----------------------------------------------------------------------
*     X-facets
*-----------------------------------------------------------------------

      DO iz=2,nz-1
         DO iy=2,ny-1
            ix=2
            Y(ix,iy,iz) = 
     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            ix=3
*            Y(ix,iy,iz) = 
*     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            ix=nx-2
*            Y(ix,iy,iz) = 
*     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
            ix=nx-1
            Y(ix,iy,iz) = 
     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(ny-2)*(nz-2)
#endif
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(ny-2)*(nz-2)
#endif

*-----------------------------------------------------------------------
*     Y-facets
*-----------------------------------------------------------------------

      DO iz=2,nz-1
         DO ix=2,nx-1
            iy=2
            Y(ix,iy,iz) = 
     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            iy=3
*            Y(ix,iy,iz) = 
*     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            iy=ny-2
*            Y(ix,iy,iz) = 
*     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
            iy=ny-1
            Y(ix,iy,iz) = 
     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
         END DO
      END DO
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-2)*(nz-2)
#endif
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-2)*(nz-2)
#endif

*-----------------------------------------------------------------------
*     Z-facets
*-----------------------------------------------------------------------

      DO iy=2,ny-1
         DO ix=2,nx-1
            iz=2
            Y(ix,iy,iz) = 
     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            iz=3
*            Y(ix,iy,iz) = 
*     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            iz=nz-2
*            Y(ix,iy,iz) = 
*     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
            iz=nz-1
            Y(ix,iy,iz) = 
     &           A4  (ix-1,iy,iz)*X(ix-1,iy,iz)
     &           + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           + AD(ix,iy,iz)  *X(ix,iy,iz)
     &           + A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           + A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           + A3(ix,iy,iz)  *X(ix,iy,iz+1)
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(ny-2)*(nx-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(ny-2)*(nx-2)
#endif
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(ny-2)*(nx-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(ny-2)*(nx-2)
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmIMatvec2'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C
C     MODULE:  SUBROUTINE mmBMFMatvecGen 
C
C     PURPOSE: Matrix-free matrix-vector product for general problems
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBMFMatvecGen2 (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,mask,level,
     &     icall)

      IMPLICIT NONE

      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level
      LOGICAL lx,ly,lz,mask(*)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz
      REAL hxm,hym,hzm,hxp,hyp,hzp
      REAL xm2,ym2,zm2,xp2,yp2,zp2
      REAL dxs,dys,dzs
      REAL x0,y0

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MATVEC2+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> BMFMatvecGen2'
#endif

      IF (level.GT.1) THEN

         CALL mmBMatvec2 (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &        
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        lx,ly,lz,mask,level,isub)

      ELSE

*-----------------------------------------------------------------------
*     Refresh the ghost points of X
*-----------------------------------------------------------------------

         IF (lx) THEN
            CALL mmGetX1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
            CALL mmGetXnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         END IF
         IF (ly) THEN
            CALL mmGetY1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
            CALL mmGetYnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         END IF
         IF (lz) THEN
            CALL mmGetZ1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
            CALL mmGetZnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         END IF

         IF ((.NOT.lx).AND.ly.AND.lz) THEN
            DO ix=2,nx-1
               IF (mask(ix)) THEN
                  dxs = gdx(ix-1,1)+gdx(ix,1)
                  hxm=xp(ix)-xp(ix-1)
                  hxp=xp(ix+1)-xp(ix)
                  xm2 = o2*(xp(ix-1)+xp(ix))
                  xp2 = o2*(xp(ix)+xp(ix+1))
                  x0 = xp(ix)
                  DO iz=2,nz-1
                     dzs = gdz(iz-1,1)+gdz(iz,1)
                     hzm=zp(iz)-zp(iz-1)
                     hzp=zp(iz+1)-zp(iz)
                     zm2 = o2*(zp(iz-1)+zp(iz))
                     zp2 = o2*(zp(iz)+zp(iz+1))
                     DO iy=2,ny-1
                        dys = gdy(iy-1,1)+gdy(iy,1)
                        hym=yp(iy)-yp(iy-1)
                        hyp=yp(iy+1)-yp(iy)
                        ym2 = o2*(yp(iy-1)+yp(iy))
                        yp2 = o2*(yp(iy)+yp(iy+1))
                        y0 = yp(iy)
                        Y(ix,iy,iz) = o4*gScaleA*(
     &                            dys*dzs/hxp*gg2off(ix,1)*gg31off(ix,1)
     &                       * (              - X(ix,iy,iz))
     &                       +    dzs*dxs/hyp*gg32off(iy,1)
     &                       / gg2on(ix,1)**2
     &                       * (X(ix,iy+1,iz) - X(ix,iy,iz))
     &                       +    dxs*dys/hzp/(gg31on(ix,1)**2
     &                       * gg32on(iy,1)**2)
     &                       * (X(ix,iy,iz+1) - X(ix,iy,iz))
     &                       +    dys*dzs/hxm * gg2off(ix-1,1)
     &                       * gg31off(ix-1,1)   
     &                       * (              - X(ix,iy,iz))
     &                       +    dzs*dxs/hym*gg32off(iy-1,1)
     &                       / gg2on(ix,1)**2
     &                       * (X(ix,iy-1,iz) - X(ix,iy,iz))
     &                       +    dxs*dys/hzm/(gg31on(ix,1)**2
     &                       * gg32on(iy,1)**2)
     &                       * (X(ix,iy,iz-1) - X(ix,iy,iz)))
                     END DO
                  END DO
                  
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 47*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 47*(ny-2)*(nz-2)
#endif
               END IF
            END DO
            

         ELSE IF (lx.AND.(.NOT.ly).AND.lz) THEN
            DO iy=2,ny-1
               IF (mask(iy)) THEN
                  dys = gdy(iy-1,1)+gdy(iy,1)
                  hym=yp(iy)-yp(iy-1)
                  hyp=yp(iy+1)-yp(iy)
                  ym2 = o2*(yp(iy-1)+yp(iy))
                  yp2 = o2*(yp(iy)+yp(iy+1))
                  y0 = yp(iy)
                  DO iz=2,nz-1
                     dzs = gdz(iz-1,1)+gdz(iz,1)
                     hzm=zp(iz)-zp(iz-1)
                     hzp=zp(iz+1)-zp(iz)
                     zm2 = o2*(zp(iz-1)+zp(iz))
                     zp2 = o2*(zp(iz)+zp(iz+1))
                     DO ix=2,nx-1
                        dxs = gdx(ix-1,1)+gdx(ix,1)
                        hxm=xp(ix)-xp(ix-1)
                        hxp=xp(ix+1)-xp(ix)
                        xm2 = o2*(xp(ix-1)+xp(ix))
                        xp2 = o2*(xp(ix)+xp(ix+1))
                        x0 = xp(ix)
                        Y(ix,iy,iz) = o4*gScaleA*(
     &                            dys*dzs/hxp*gg2off(ix,1)*gg31off(ix,1)
     &                       * (X(ix+1,iy,iz) - X(ix,iy,iz))
     &                       +    dzs*dxs/hyp*gg32off(iy,1)/
     &                       gg2on(ix,1)**2
     &                       * (              - X(ix,iy,iz))
     &                       +    dxs*dys/hzp/(gg31on(ix,1)**2
     &                       * gg32on(iy,1)**2)
     &                       * (X(ix,iy,iz+1) - X(ix,iy,iz))
     &                       +    dys*dzs/hxm * gg2off(ix-1,1)
     &                       * gg31off(ix-1,1)   
     &                       * (X(ix-1,iy,iz) - X(ix,iy,iz))
     &                       +    dzs*dxs/hym*gg32off(iy-1,1)
     &                       / gg2on(ix,1)**2
     &                       * (              - X(ix,iy,iz))
     &                       +    dxs*dys/hzm/(gg31on(ix,1)**2
     &                       * gg32on(iy,1)**2)
     &                       * (X(ix,iy,iz-1) - X(ix,iy,iz)))
                     END DO
                  END DO
                  
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 47*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 47*(nx-2)*(nz-2)
#endif
               END IF
            END DO


         ELSE IF (lx.AND.ly.AND.(.NOT.lz)) THEN
            DO iz=2,nz-1
               IF (mask(iz)) THEN
                  dzs = gdz(iz-1,1)+gdz(iz,1)
                  hzm=zp(iz)-zp(iz-1)
                  hzp=zp(iz+1)-zp(iz)
                  zm2 = o2*(zp(iz-1)+zp(iz))
                  zp2 = o2*(zp(iz)+zp(iz+1))
                  DO iy=2,ny-1
                     dys = gdy(iy-1,1)+gdy(iy,1)
                     hym=yp(iy)-yp(iy-1)
                     hyp=yp(iy+1)-yp(iy)
                     ym2 = o2*(yp(iy-1)+yp(iy))
                     yp2 = o2*(yp(iy)+yp(iy+1))
                     y0 = yp(iy)
                     DO ix=2,nx-1
                        dxs = gdx(ix-1,1)+gdx(ix,1)
                        hxm=xp(ix)-xp(ix-1)
                        hxp=xp(ix+1)-xp(ix)
                        xm2 = o2*(xp(ix-1)+xp(ix))
                        xp2 = o2*(xp(ix)+xp(ix+1))
                        x0 = xp(ix)
                        Y(ix,iy,iz) = o4*gScaleA*(
     &                       +    dys*dzs/hxp*gg2off(ix,1)*gg31off(ix,1)
     &                       * (X(ix+1,iy,iz) - X(ix,iy,iz))
     &                       +    dzs*dxs/hyp*gg32off(iy,1)/
     &                       gg2on(ix,1)**2
     &                       * (X(ix,iy+1,iz) - X(ix,iy,iz))
     &                       +    dxs*dys/hzp/(gg31on(ix,1)**2
     &                       * gg32on(iy,1)**2)
     &                       * (              - X(ix,iy,iz))
     &                       +    dys*dzs/hxm * gg2off(ix-1,1)
     &                       * gg31off(ix-1,1)   
     &                       * (X(ix-1,iy,iz) - X(ix,iy,iz))
     &                       +    dzs*dxs/hym*gg32off(iy-1,1)
     &                       / gg2on(ix,1)**2
     &                       * (X(ix,iy-1,iz) - X(ix,iy,iz))
     &                       +    dxs*dys/hzm/(gg31on(ix,1)**2
     &                       * gg32on(iy,1)**2)
     &                       * (              - X(ix,iy,iz)))
                     END DO
                  END DO
                  
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 47*(nx-2)*(ny-2)
      gmflops(isub)=gmflops(isub) + 
     & 47*(nx-2)*(ny-2)
#endif
               END IF
            END DO


         ELSE
            PRINT*, 'MPI Error in mmBMFMatvecGen2: lx,ly,lz=',lx,ly,lz
            CALL mmTerminate(comm)
         END IF
      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< BMFMatvecGen2'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmBMFMatvecNU 
C     
C     PURPOSE: Matrix-free matrix-vector product for nonuniform meshes
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBMFMatvecNU2 (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,mask,level,
     &     icall)

      IMPLICIT NONE

      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level
      LOGICAL lx,ly,lz,mask(*)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz
      REAL hxm,hym,hzm,hxp,hyp,hzp
      REAL dxs,dys,dzs,dxp,dyp,dzp,dxm,dym,dzm

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MATVEC2+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      IF (level.GT.1) THEN

         CALL mmBMatvec2 (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &        
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        lx,ly,lz,mask,level,isub)

      ELSE

*-----------------------------------------------------------------------
*     Refresh the ghost points of X
*-----------------------------------------------------------------------

         IF (lx) THEN
            CALL mmGetX1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
            CALL mmGetXnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         END IF
         IF (ly) THEN
            CALL mmGetY1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
            CALL mmGetYnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         END IF
         IF (lz) THEN
            CALL mmGetZ1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
            CALL mmGetZnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         END IF

         IF ((.NOT.lx).AND.ly.AND.lz) THEN
            dxs = xp(3)-xp(1)
            hxm = xp(2)-xp(1)
            hxp = xp(3)-xp(2)
            DO ix=2,nx-1
               IF (mask(ix)) THEN
                  dzs = zp(3)-zp(1)
                  hzm = zp(2)-zp(1)
                  hzp = zp(3)-zp(2)
                  DO iz=2,nz-1
                     dys = yp(3)-yp(1)
                     hym = yp(2)-yp(1)
                     hyp = yp(3)-yp(2)
                     DO iy=2,ny-1
                        dxp=dys*dzs/hxp
                        dxm=dys*dzs/hxm
                        dyp=dzs*dxs/hyp
                        dym=dzs*dxs/hym
                        dzp=dxs*dys/hzp
                        dzm=dxs*dys/hzm
                        Y(ix,iy,iz) = o4*gScaleA*(
     &                       -(dxp+dyp+dzp+dxm+dym+dzm)*X(ix,iy,iz)
     &                       + dyp * X(ix,iy+1,iz)
     &                       + dzp * X(ix,iy,iz+1)
     &                       + dym * X(ix,iy-1,iz)
     &                       + dzm * X(ix,iy,iz-1))
                        dxs = xp(ix+2)-xp(ix)
                        hxm=hxp
                        hxp=xp(ix+2)-xp(ix+1)
                     END DO
                     dys = yp(iy+2)-yp(iy)
                     hym=hyp
                     hyp=yp(iy+2)-yp(iy+1)
                  END DO
                  dzs = zp(iz+2)-zp(iz)
                  hzm=hzp
                  hzp=zp(iz+2)-zp(iz+1)
                  
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 30*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 30*(ny-2)*(nz-2)
#endif
               END IF
            END DO


         ELSE IF (lx.AND.(.NOT.ly).AND.lz) THEN
            dys = yp(3)-yp(1)
            hym = yp(2)-yp(1)
            hyp = yp(3)-yp(2)
            DO iy=2,ny-1
               IF (mask(iy)) THEN
                  dzs = zp(3)-zp(1)
                  hzm = zp(2)-zp(1)
                  hzp = zp(3)-zp(2)
                  DO iz=2,nz-1
                     dxs = xp(3)-xp(1)
                     hxm = xp(2)-xp(1)
                     hxp = xp(3)-xp(2)
                     DO ix=2,nx-1
                        dxp=dys*dzs/hxp
                        dxm=dys*dzs/hxm
                        dyp=dzs*dxs/hyp
                        dym=dzs*dxs/hym
                        dzp=dxs*dys/hzp
                        dzm=dxs*dys/hzm
                        Y(ix,iy,iz) = o4*gScaleA*(
     &                       -(dxp+dyp+dzp+dxm+dym+dzm)*X(ix,iy,iz)
     &                       + dxp * X(ix+1,iy,iz)
     &                       + dzp * X(ix,iy,iz+1)
     &                       + dxm * X(ix-1,iy,iz)
     &                       + dzm * X(ix,iy,iz-1))
                        dxs = xp(ix+2)-xp(ix)
                        hxm=hxp
                        hxp=xp(ix+2)-xp(ix+1)
                     END DO
                     dys = yp(iy+2)-yp(iy)
                     hym=hyp
                     hyp=yp(iy+2)-yp(iy+1)
                  END DO
                  dzs = zp(iz+2)-zp(iz)
                  hzm=hzp
                  hzp=zp(iz+2)-zp(iz+1)
                  
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 30*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 30*(nx-2)*(nz-2)
#endif
               END IF
            END DO

         ELSE IF (lx.AND.ly.AND.(.NOT.lz)) THEN
            dzs = zp(3)-zp(1)
            hzm = zp(2)-zp(1)
            hzp = zp(3)-zp(2)
            DO iz=2,nz-1
               IF (mask(iz)) THEN
                  dys = yp(3)-yp(1)
                  hym = yp(2)-yp(1)
                  hyp = yp(3)-yp(2)
                  DO iy=2,ny-1
                     dxs = xp(3)-xp(1)
                     hxm = xp(2)-xp(1)
                     hxp = xp(3)-xp(2)
                     DO ix=2,nx-1
                        dxp=dys*dzs/hxp
                        dxm=dys*dzs/hxm
                        dyp=dzs*dxs/hyp
                        dym=dzs*dxs/hym
                        dzp=dxs*dys/hzp
                        dzm=dxs*dys/hzm
                        Y(ix,iy,iz) = o4*gScaleA*(
     &                       -(dxp+dyp+dzp+dxm+dym+dzm)*X(ix,iy,iz)
     &                       + dxp * X(ix+1,iy,iz)
     &                       + dyp * X(ix,iy+1,iz)
     &                       + dxm * X(ix-1,iy,iz)
     &                       + dym * X(ix,iy-1,iz))
                        dxs = xp(ix+2)-xp(ix)
                        hxm=hxp
                        hxp=xp(ix+2)-xp(ix+1)
                     END DO
                     dys = yp(iy+2)-yp(iy)
                     hym=hyp
                     hyp=yp(iy+2)-yp(iy+1)
                  END DO
                  dzs = zp(iz+2)-zp(iz)
                  hzm=hzp
                  hzp=zp(iz+2)-zp(iz+1)
                  
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 30*(nx-2)*(ny-2)
      gmflops(isub)=gmflops(isub) + 
     & 30*(nx-2)*(ny-2)
#endif
               END IF
            END DO
         ELSE
            PRINT*, 'MPI Error in mmBMFMatvecNU2: lx,ly,lz=',lx,ly,lz
            CALL mmTerminate(comm)
         END IF
      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmBMFMatvecU 
C     
C     Purpose: Matrix-free matrix-vector product for uniform meshes
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBMFMatvecU2 (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,mask,level,
     &     icall)

      IMPLICIT NONE

      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER comm,nbr(6),bnd(3)


      INTEGER nx,ny,nz
      INTEGER level
      LOGICAL lx,ly,lz,mask(*)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz
      REAL dx,dy,dz,dsum

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MATVEC2+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmBMFMatvecU2'
#endif

      IF (level.GT.1) THEN

         CALL mmBMatvec2 (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &        
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        lx,ly,lz,mask,level,isub)

      ELSE

*-----------------------------------------------------------------------
*     Refresh the ghost points of X
*-----------------------------------------------------------------------

         IF (lx) THEN
            CALL mmGetX1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
            CALL mmGetXnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         END IF
         IF (ly) THEN
            CALL mmGetY1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
            CALL mmGetYnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         END IF
         IF (lz) THEN
            CALL mmGetZ1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
            CALL mmGetZnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         END IF

         dx = gScaleA*hy(2)*hz(2)/hx(2)
         dy = gScaleA*hz(2)*hx(2)/hy(2)
         dz = gScaleA*hx(2)*hy(2)/hz(2)
         dsum = -2.0D0*(dx+dy+dz)

         IF ((.NOT.lx).AND.ly.AND.lz) THEN
            DO ix=2,nx-1
               IF (mask(ix)) THEN
                  DO iz=2,nz-1
                     DO iy=2,ny-1
                        Y(ix,iy,iz) = dsum*X(ix,iy,iz)
     &                       + dy*(X(ix,iy+1,iz) + X(ix,iy-1,iz))
     &                       + dz*(X(ix,iy,iz+1) + X(ix,iy,iz-1))
                     END DO
                  END DO
                  
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 7*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 7*(ny-2)*(nz-2)
#endif
               END IF
            END DO
         ELSE IF (lx.AND.(.NOT.ly).AND.lz) THEN
            DO iy=2,ny-1
               IF (mask(iy)) THEN
                  DO iz=2,nz-1
                     DO ix=2,nx-1
                        Y(ix,iy,iz) = dsum*X(ix,iy,iz)
     &                       + dx*(X(ix+1,iy,iz) + X(ix-1,iy,iz))
     &                       + dz*(X(ix,iy,iz+1) + X(ix,iy,iz-1))
                     END DO
                  END DO
                  
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 7*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 7*(nx-2)*(nz-2)
#endif
               END IF
            END DO
         ELSE IF (lx.AND.ly.AND.(.NOT.lz)) THEN
            DO iz=2,nz-1
               IF (mask(iz)) THEN
                  DO iy=2,ny-1
                     DO ix=2,nx-1
                        Y(ix,iy,iz) = dsum*X(ix,iy,iz)
     &                       + dx*(X(ix+1,iy,iz) + X(ix-1,iy,iz))
     &                       + dy*(X(ix,iy+1,iz) + X(ix,iy-1,iz))
                     END DO
                  END DO
                  
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 7*(nx-2)*(ny-2)
      gmflops(isub)=gmflops(isub) + 
     & 7*(nx-2)*(ny-2)
#endif
               END IF
            END DO
         ELSE
            PRINT*, 'MPI Error in mmBMFMatvecU2: lx,ly,lz=',lx,ly,lz
            CALL mmTerminate(comm)
         END IF
      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmBMFMatvecU2'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmMatDiag
C     
C     PURPOSE: Return the specified fine-level matrix diagonal
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmMatDiag (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,idiag,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,scale,
     &     icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      INTEGER nx,ny,nz,idiag
      REAL scale
      INTEGER comm

      LOGICAL isFirstCall
      DATA isFirstCall/.TRUE./

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MAT_DIAG
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmMatDiag'
#endif

      IF (gimatfree.EQ.0) THEN
         IF (ip.EQ.0.AND.isFirstCall) THEN 
            PRINT*, '-- mgmpi error -- mmMatDiag called from stored'
            PRINT*, '                  matrix version'
         END IF
#ifdef MPI_USED
         CALL mmTerminate (comm)
#else 
         STOP
#endif
      END IF

      IF (gimatfree.EQ.1) THEN
#ifndef MPI_BLOCKING
         IF (ip.EQ.0.AND.isFirstCall) THEN 
            PRINT*, '-- mgmpi warning -- Matrix-free nonblocking version'
            PRINT*, '                    not implemented; continuing with'
            PRINT*, '                    blocking MPI calls.'
            isFirstCall = .FALSE.
         END IF
#endif
      END IF

      IF (gicoords.EQ.MM_CARTESIAN) THEN

         IF (giGridType.EQ.MM_IS_UNIFORM) THEN
            CALL mmBMFMatDiagU (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,idiag,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,scale,
     &           isub)
         ELSE
            CALL mmBMFMatDiagNU (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,idiag,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &           comm,scale,isub)
         END IF

      ELSE

         CALL mmBMFMatDiagGen (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,idiag,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,scale,
     &        isub)

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmMatDiag'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmBMFMatDiagGen 
C     
C     PURPOSE: Compute and return a specified matrix diagonal for
C     PURPOSE: general problems      
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBMFMatDiagGen (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,idiag,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,scale,icall)
      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)

      INTEGER nx,ny,nz,idiag
      REAL scale
      INTEGER comm

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz
      REAL hxm,hym,hzm,hxp,hyp,hzp
      REAL xm2,ym2,zm2,xp2,yp2,zp2
      REAL dxs,dys,dzs
      REAL x0,y0

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MAT_DIAG
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmBMFMatDiagGen'
#endif

      IF (idiag.EQ.0) THEN
         DO iz=2,nz-1
            dzs = gdz(iz-1,1)+gdz(iz,1)
            hzm=zp(iz)-zp(iz-1)
            hzp=zp(iz+1)-zp(iz)
            zm2 = o2*(zp(iz-1)+zp(iz))
            zp2 = o2*(zp(iz)+zp(iz+1))
            DO iy=2,ny-1
               dys = gdy(iy-1,1)+gdy(iy,1)
               hym=yp(iy)-yp(iy-1)
               hyp=yp(iy+1)-yp(iy)
               ym2 = o2*(yp(iy-1)+yp(iy))
               yp2 = o2*(yp(iy)+yp(iy+1))
               y0 = yp(iy)
               DO ix=2,nx-1
                  dxs = gdx(ix-1,1)+gdx(ix,1)
                  hxm=xp(ix)-xp(ix-1)
                  hxp=xp(ix+1)-xp(ix)
                  xm2 = o2*(xp(ix-1)+xp(ix))
                  xp2 = o2*(xp(ix)+xp(ix+1))
                  x0 = xp(ix)
                  X(ix,iy,iz) = -o4*scale*(
     &                 dys*dzs*(gg2off(ix,1)*gg31off(ix,1)/hxp
     &                 +gg2off(ix-1,1)*gg31off(ix-1,1)/hxm)
     &                 +dzs*dxs*((gg32off(iy,1)/gg2on(ix,1)**2)/hyp
     &                 +(gg32off(iy-1,1)/gg2on(ix,1)**2)/hym)
     &                 +dxs*dys*(o1/((gg31on(ix,1)**2*gg32on(iy,1)**2)))
     &                 *(o1/hzp + o1/hzm))
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 30*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 30*(nx-2)*(ny-2)*(nz-2)
#endif
      ELSE IF (idiag.EQ.1) THEN
         DO iz=1,nz-1
            dzs = gdz(iz-1,1)+gdz(iz,1)
            hzm=zp(iz)-zp(iz-1)
            hzp=zp(iz+1)-zp(iz)
            zm2 = o2*(zp(iz-1)+zp(iz))
            zp2 = o2*(zp(iz)+zp(iz+1))
            DO iy=1,ny-1
               dys = gdy(iy-1,1)+gdy(iy,1)
               hym=yp(iy)-yp(iy-1)
               hyp=yp(iy+1)-yp(iy)
               ym2 = o2*(yp(iy-1)+yp(iy))
               yp2 = o2*(yp(iy)+yp(iy+1))
               DO ix=1,nx-1
                  dxs = gdx(ix-1,1)+gdx(ix,1)
                  hxm=xp(ix)-xp(ix-1)
                  hxp=xp(ix+1)-xp(ix)
                  xm2 = o2*(xp(ix-1)+xp(ix))
                  xp2 = o2*(xp(ix)+xp(ix+1))
                  X(ix,iy,iz) = o4*scale*(dys*dzs/hxp*gg2off(ix,1)
     &                 * gg31off(ix,1))
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 6*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 6*(nx-2)*(ny-2)*(nz-2)
#endif
      ELSE IF (idiag.EQ.2) THEN
         DO iz=1,nz-1
            dzs = gdz(iz-1,1)+gdz(iz,1)
            hzm=zp(iz)-zp(iz-1)
            hzp=zp(iz+1)-zp(iz)
            zm2 = o2*(zp(iz-1)+zp(iz))
            zp2 = o2*(zp(iz)+zp(iz+1))
            DO iy=1,ny-1
               dys = gdy(iy-1,1)+gdy(iy,1)
               hym=yp(iy)-yp(iy-1)
               hyp=yp(iy+1)-yp(iy)
               ym2 = o2*(yp(iy-1)+yp(iy))
               yp2 = o2*(yp(iy)+yp(iy+1))
               DO ix=1,nx-1
                  dxs = gdx(ix-1,1)+gdx(ix,1)
                  hxm=xp(ix)-xp(ix-1)
                  hxp=xp(ix+1)-xp(ix)
                  xm2 = o2*(xp(ix-1)+xp(ix))
                  xp2 = o2*(xp(ix)+xp(ix+1))
                  x0 = xp(ix)
                  X(ix,iy,iz) = o4*scale*
     &                 (dzs*dxs/hyp * gg32off(iy,1)/gg2on(ix,1)**2)
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 7*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 7*(nx-2)*(ny-2)*(nz-2)
#endif
      ELSE IF (idiag.EQ.3) THEN
         DO iz=1,nz-1
            dzs = gdz(iz-1,1)+gdz(iz,1)
            hzm=zp(iz)-zp(iz-1)
            hzp=zp(iz+1)-zp(iz)
            zm2 = o2*(zp(iz-1)+zp(iz))
            zp2 = o2*(zp(iz)+zp(iz+1))
            DO iy=1,ny-1
               dys = gdy(iy-1,1)+gdy(iy,1)
               hym=yp(iy)-yp(iy-1)
               hyp=yp(iy+1)-yp(iy)
               ym2 = o2*(yp(iy-1)+yp(iy))
               yp2 = o2*(yp(iy)+yp(iy+1))
               y0 = yp(iy)
               DO ix=1,nx-1
                  dxs = gdx(ix-1,1)+gdx(ix,1)
                  hxm=xp(ix)-xp(ix-1)
                  hxp=xp(ix+1)-xp(ix)
                  xm2 = o2*(xp(ix-1)+xp(ix))
                  xp2 = o2*(xp(ix)+xp(ix+1))
                  x0 = xp(ix)
                  X(ix,iy,iz) = o4*scale*
     &                 (dxs*dys/hzp/(gg31on(ix,1)**2*gg32on(iy,1)**2))
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 8*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 8*(nx-2)*(ny-2)*(nz-2)
#endif
      ELSE IF (idiag.EQ.4) THEN
         DO iz=1,nz-1
            dzs = gdz(iz-1,1)+gdz(iz,1)
            hzm=zp(iz)-zp(iz-1)
            hzp=zp(iz+1)-zp(iz)
            zm2 = o2*(zp(iz-1)+zp(iz))
            zp2 = o2*(zp(iz)+zp(iz+1))
            DO iy=1,ny-1
               dys = gdy(iy-1,1)+gdy(iy,1)
               hym=yp(iy)-yp(iy-1)
               hyp=yp(iy+1)-yp(iy)
               ym2 = o2*(yp(iy-1)+yp(iy))
               yp2 = o2*(yp(iy)+yp(iy+1))
               DO ix=1,nx-1
                  dxs = gdx(ix-1,1)+gdx(ix,1)
                  hxm=xp(ix)-xp(ix-1)
                  hxp=xp(ix+1)-xp(ix)
                  xm2 = o2*(xp(ix-1)+xp(ix))
                  xp2 = o2*(xp(ix)+xp(ix+1))
                  X(ix,iy,iz) = o4*scale*
     &                 (dys*dzs/hxm * gg2off(ix-1,1)*gg31off(ix-1,1))
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 6*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 6*(nx-2)*(ny-2)*(nz-2)
#endif
      ELSE IF (idiag.EQ.5) THEN
         DO iz=1,nz-1
            dzs = gdz(iz-1,1)+gdz(iz,1)
            hzm=zp(iz)-zp(iz-1)
            hzp=zp(iz+1)-zp(iz)
            zm2 = o2*(zp(iz-1)+zp(iz))
            zp2 = o2*(zp(iz)+zp(iz+1))
            DO iy=1,ny-1
               dys = gdy(iy-1,1)+gdy(iy,1)
               hym=yp(iy)-yp(iy-1)
               hyp=yp(iy+1)-yp(iy)
               ym2 = o2*(yp(iy-1)+yp(iy))
               yp2 = o2*(yp(iy)+yp(iy+1))
               DO ix=1,nx-1
                  dxs = gdx(ix-1,1)+gdx(ix,1)
                  hxm=xp(ix)-xp(ix-1)
                  hxp=xp(ix+1)-xp(ix)
                  xm2 = o2*(xp(ix-1)+xp(ix))
                  xp2 = o2*(xp(ix)+xp(ix+1))
                  x0 = xp(ix)
                  X(ix,iy,iz) = o4*scale*
     &                 (dzs*dxs/hym * gg32off(iy-1,1)/gg2on(ix,1)**2)
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 7*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 7*(nx-2)*(ny-2)*(nz-2)
#endif
      ELSE IF (idiag.EQ.6) THEN
         DO iz=1,nz-1
            dzs = gdz(iz-1,1)+gdz(iz,1)
            hzm=zp(iz)-zp(iz-1)
            hzp=zp(iz+1)-zp(iz)
            zm2 = o2*(zp(iz-1)+zp(iz))
            zp2 = o2*(zp(iz)+zp(iz+1))
            DO iy=1,ny-1
               dys = gdy(iy-1,1)+gdy(iy,1)
               hym=yp(iy)-yp(iy-1)
               hyp=yp(iy+1)-yp(iy)
               ym2 = o2*(yp(iy-1)+yp(iy))
               yp2 = o2*(yp(iy)+yp(iy+1))
               y0 = yp(iy)
               DO ix=1,nx-1
                  dxs = gdx(ix-1,1)+gdx(ix,1)
                  hxm=xp(ix)-xp(ix-1)
                  hxp=xp(ix+1)-xp(ix)
                  xm2 = o2*(xp(ix-1)+xp(ix))
                  xp2 = o2*(xp(ix)+xp(ix+1))
                  x0 = xp(ix)
                  X(ix,iy,iz) = o4*scale*
     &                 (dxs*dys/hzm/(gg31on(ix,1)**2*gg32on(iy,1)**2))
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 8*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 8*(nx-2)*(ny-2)*(nz-2)
#endif
      END IF


#ifdef DEBUG
      IF (ip.EQ.0) PRINT*, 'mmBMFMatDiagGen: exiting'
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmBMFMatDiagGen'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C
C     MODULE:  SUBROUTINE mmBMFMatDiagNU 
C
C     PURPOSE: Compute and return a specified matrix diagonal for
C     PURPOSE: nonuniform problems
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBMFMatDiagNU (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,idiag,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,scale,icall)
      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)

      INTEGER nx,ny,nz,idiag
      REAL scale
      INTEGER comm

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz
      REAL hxm,hym,hzm,hxp,hyp,hzp
      REAL dxs,dys,dzs,dxp,dyp,dzp,dxm,dym,dzm
      REAL scaleO4

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MAT_DIAG
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmBMFMatDiagNU'
#endif

      scaleO4 = o4 * scale

     
      IF (idiag.EQ.0) THEN
         dzs = zp(3)-zp(1)
         hzm = zp(2)-zp(1)
         hzp = zp(3)-zp(2)
         DO iz=2,nz-1
            dys = yp(3)-yp(1)
            hym = yp(2)-yp(1)
            hyp = yp(3)-yp(2)
            DO iy=2,ny-1
               dxs = xp(3)-xp(1)
               hxm = xp(2)-xp(1)
               hxp = xp(3)-xp(2)
               DO ix=2,nx-1
                  dzp=dxs*dys/hzp
                  dxp=dys*dzs/hxp
                  dyp=dzs*dxs/hyp
                  dzm=dxs*dys/hzm
                  dxm=dys*dzs/hxm
                  dym=dzs*dxs/hym
                  X(ix,iy,iz) = -scaleO4*
     &                 (dxp + dyp + dzp + dxm + dym + dzm)
                  dxs = xp(ix+2)-xp(ix)
                  hxm=hxp
                  hxp=xp(ix+2)-xp(ix+1)
               END DO
               dys = yp(iy+2)-yp(iy)
               hym=hyp
               hyp=yp(iy+2)-yp(iy+1)
            END DO
            dzs = zp(iz+2)-zp(iz)
            hzm=hzp
            hzp=zp(iz+2)-zp(iz+1)
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 20*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 20*(nx-2)*(ny-2)*(nz-2)
#endif
      ELSE IF (idiag.EQ.1.OR.idiag.EQ.4) THEN
         dzs = zp(3)-zp(1)
         DO iz=1,nz-1
            dys = yp(3)-yp(1)
            DO iy=1,ny-1
               hxp = xp(3)-xp(2)
               DO ix=1,nx-1
                  dxp=dys*dzs/hxp
                  X(ix,iy,iz) = scaleO4*dxp
                  hxp=xp(ix+2)-xp(ix+1)
               END DO
               dys = yp(iy+2)-yp(iy)
            END DO
            dzs = zp(iz+2)-zp(iz)
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 4*(nx-1)*(ny-1)*(nz-1)
      gmflops(isub)=gmflops(isub) + 
     & 4*(nx-1)*(ny-1)*(nz-1)
#endif
      ELSE IF (idiag.EQ.2.OR.idiag.EQ.5) THEN
         dzs = zp(3)-zp(1)
         DO iz=1,nz-1
            hyp = yp(3)-yp(2)
            DO iy=1,ny-1
               dxs = xp(3)-xp(1)
               DO ix=1,nx-1
                  dyp=dzs*dxs/hyp
                  X(ix,iy,iz) = scaleO4*dyp
                  dxs = xp(ix+2)-xp(ix)
               END DO
               hyp = yp(iy+2)-yp(iy+1)
            END DO
            dzs = zp(iz+2)-zp(iz)
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 4*(nx-1)*(ny-1)*(nz-1)
      gmflops(isub)=gmflops(isub) + 
     & 4*(nx-1)*(ny-1)*(nz-1)
#endif
      ELSE IF (idiag.EQ.3.OR.idiag.EQ.6) THEN
         hzp = zp(3)-zp(2)
         DO iz=1,nz-1
            dys = yp(3)-yp(1)
            DO iy=1,ny-1
               dxs = xp(3)-xp(1)
               DO ix=1,nx-1
                  dzp=dxs*dys/hzp
                  X(ix,iy,iz) = scaleO4*dzp
                  dxs = xp(ix+2)-xp(ix)
               END DO
               dys = yp(iy+2)-yp(iy)
            END DO
            hzp=zp(iz+2)-zp(iz+1)
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 4*(nx-1)*(ny-1)*(nz-1)
      gmflops(isub)=gmflops(isub) + 
     & 4*(nx-1)*(ny-1)*(nz-1)
#endif
      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmBMFMatDiagNU'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C
C     MODULE:  SUBROUTINE mmBMFMatDiagU 
C
C     PURPOSE: Compute and return a specified matrix diagonal for
C     PURPOSE: uniform problems
C 
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBMFMatDiagU (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,idiag,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,scale,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      INTEGER nx,ny,nz,idiag
      REAL scale
      INTEGER comm

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz
      REAL dx,dy,dz,dsum

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MAT_DIAG
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmBMFMatDiagU'
#endif

      dx = scale*hy(2)*hz(2)/hx(2)
      dy = scale*hz(2)*hx(2)/hy(2)
      dz = scale*hx(2)*hy(2)/hz(2)
      dsum = -2.0D0*(dx+dy+dz)

      IF (idiag.EQ.0) THEN
         DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
                  X(ix,iy,iz) = dsum
               END DO
            END DO
         END DO
      ELSE IF (idiag.EQ.1.OR.idiag.EQ.4) THEN
         DO iz=1,nz-1
            DO iy=1,ny-1
               DO ix=1,nx-1
                  X(ix,iy,iz) = dx
               END DO
            END DO
         END DO
      ELSE IF (idiag.EQ.2.OR.idiag.EQ.5) THEN
         DO iz=1,nz-1
            DO iy=1,ny-1
               DO ix=1,nx-1
                  X(ix,iy,iz) = dy
               END DO
            END DO
         END DO
      ELSE IF (idiag.EQ.3.OR.idiag.EQ.6) THEN
         DO iz=1,nz-1
            DO iy=1,ny-1
               DO ix=1,nx-1
                  X(ix,iy,iz) = dz
               END DO
            END DO
         END DO
      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmBMFMatDiagU'
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmMgPrecon
C     
C     Purpose: Multigrid V-cycle preconditioner
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C               Sep 9 -- Changed i?1(k) to gmmi?(k,ip)
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmMgPrecon (
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     BC,nxdbc,nydbc,nzdbc,bndbc
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &     indh,inda,indx
     &     ,
     &     m,nx,ny,nz,
     &     nxg,nyg,nzg,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver,
     &     wgt,iterFine,iterCoarse,
     &     maxItFine,maxItCoarse,
     &     resTolFine,resTolCoarse,outputLevel,
     &     comm,nbr,bnd,bnd2
     &     ,
     &     scaleA,scaleB,rnorm,resRedFine,istat,icall)

      IMPLICIT NONE

      
      INTEGER nxdaf,nydaf,nzdaf
      INTEGER bndaf(3)  
      REAL AFD(nxdaf,nydaf,nzdaf)
      REAL AF1(nxdaf,nydaf,nzdaf)
      REAL AF2(nxdaf,nydaf,nzdaf)
      REAL AF3(nxdaf,nydaf,nzdaf)
      REAL AF4(nxdaf,nydaf,nzdaf)
      REAL AF5(nxdaf,nydaf,nzdaf)
      REAL AF6(nxdaf,nydaf,nzdaf)
      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      
      INTEGER nxdbf,nydbf,nzdbf
      INTEGER bndbf(3)	
      REAL BF(nxdbf,nydbf,nzdbf)
      
      INTEGER nxda(*),nyda(*),nzda(*)
      INTEGER bnda(3,*)  
      REAL AD(*)	
      REAL A1(*)
      REAL A2(*)
      REAL A3(*)
      REAL A4(*)
      REAL A5(*)
      REAL A6(*)
      
      INTEGER nxdxc(*),nydxc(*),nzdxc(*)
      INTEGER bndxc(3,*)  
      REAL XC(*)
      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxdbc(*),nydbc(*),nzdbc(*)
      INTEGER bndbc(3,*)
      REAL BC(*)
      
      INTEGER nxdr(*),nydr(*),nzdr(*)
      INTEGER bndr(3,*)	
      REAL R(*)
      
      INTEGER nxdrcg,nydrcg,nzdrcg
      INTEGER bndrcg(3)
      REAL RCG(nxdrcg,nydrcg,nzdrcg)
      
      INTEGER nxdpcg,nydpcg,nzdpcg
      INTEGER bndpcg(3)
      REAL PCG(nxdpcg,nydpcg,nzdpcg)
      
      INTEGER nxdwcg,nydwcg,nzdwcg
      INTEGER bndwcg(3)
      REAL WCG(nxdwcg,nydwcg,nzdwcg)
      
      INTEGER nxdrsm,nydrsm,nzdrsm
      INTEGER bndrsm(3)
      REAL RSM(nxdrsm,nydrsm,nzdrsm)
      
      INTEGER nxdpsm,nydpsm,nzdpsm
      INTEGER bndpsm(3)
      REAL PSM(nxdpsm,nydpsm,nzdpsm)
      
      INTEGER nxdwsm,nydwsm,nzdwsm
      INTEGER bndzsm(3)
      REAL WSM(nxdwsm,nydwsm,nzdwsm)
      INTEGER indh(*),inda(*),indx(*)
      INTEGER m
      INTEGER nx(m),ny(m),nz(m)
      INTEGER nxg(m),nyg(m),nzg(m)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      INTEGER numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver
      REAL resTolFine,resTolCoarse,wgt
      INTEGER iterFine,iterCoarse,outputLevel
      INTEGER maxItFine,maxItCoarse
      
      INTEGER comm,nbr(6),bnd(3,*),bnd2(3)
      REAL resRedFine
      REAL scaleA, scaleB,rnorm
      
      INTEGER istat
      INTEGER mmIsNan
      EXTERNAL mmIsNan      

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

#ifdef DEBUG_MG
      REAL mmVectorSum,s
      EXTERNAL mmVectorSum
#endif

      REAL mmDotProd,mmVectorNormalize,tmp
      REAL resCoarse
      REAL bnorm
      REAL rnormPrev,rnormFirst,relResCoarse
      INTEGER k,kc,iter,nu
      INTEGER ica,ifa,icx,ifx,ich,ifh,istatCoarse

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MG
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

C       print*, m
C       print*, nx
C       print*, ny
C       print*, nz
C       print*, nxg
C       print*, nyg
C       print*, nzg
C       print*, numPreSmooth
C       print*, numPostSmooth
C       print*, iSmoothIt
C       print*, iCoarseSolver
C       print*, wgt
C       print*, iterFine
C       print*, iterCoarse
C       print*, maxItFine
C       print*, maxItCoarse
C       print*, resTolFine
C       print*, resTolCoarse
C       print*, outputLevel
C       print*, scaleA
C       print*, scaleB
C       print*, rnorm
C       print*, resRedFine
C       print*, istat
C       print*, icall

      istat = 0

      iterCoarse = 0

*-----------------------------------------------------------------------
*     Initialize Dirichlet b.c.'s on all grids
*-----------------------------------------------------------------------
      

      CALL mmDirichletBC (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nx(1),ny(1),nz(1),
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     scaleA,scaleB,1,.TRUE.)
      DO k=2,m
         ifa = inda(k)
         CALL mmDirichletBC (
     &        
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,   nx(k),ny(k),nz(k),
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        scaleA,scaleB,k,.TRUE.)
      END DO

*-----------------------------------------------------------------------
*     Compute residual R = B - A*X
*-----------------------------------------------------------------------

#ifdef DEBUG_MG
      if (ip.EQ.0) PRINT*, '   mmMgPrecon Residual 1'
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,'    mmMgPrecon Residual 1')") ip
#endif
#endif 

      CALL mmResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,0,1,isub)

#ifdef DEBUG_MG
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '    mmMgPrecon sum(R) = ',s
#endif

*-----------------------------------------------------------------------
*     Compute norms for convergence test
*-----------------------------------------------------------------------

      ica = inda(2)

      bnorm = DSQRT(mmDotProd (
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx(1),ny(1),nz(1),comm,1,isub))
      rnorm = DSQRT(mmDotProd (
     &     R(1),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,
     &     R(1),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,
     &     nx(1),ny(1),nz(1),comm,1,isub))

#ifdef DEBUG_MG
      IF (ip.EQ.0) THEN
         print*, '   mmMgPrecon norms'
         print*, '     norm(b) = ',bnorm
         print*, '     norm(r) = ',rnorm
      END IF
#endif

      rnormFirst = rnorm

      iter = 0

      IF (outputLevel.NE.0 .AND. ip.EQ.0)
     &     WRITE(*,1000) iter, MPI_WTIME() - gtimestart(SB_MG3),
     &     rnorm/rnormFirst

*-----------------------------------------------------------------------
*     Check for convergence
*-----------------------------------------------------------------------

 10   CONTINUE

*-----------------------------------------------------------------------
*     Smooth and restrict residual to coarser grids
*-----------------------------------------------------------------------

      nu = numPreSmooth
      
      k = 1
      ifx = indx(k)
      ifa = inda(k)
      ifh = indh(k)
      kc = 2
      icx = indx(kc)
      ica = inda(kc)
      ich = indh(kc)

      IF (m.GT.k) THEN 

#ifdef DEBUG_MG
      if (ip.EQ.0) PRINT*, '   mmMgPrecon pre-smooth 1'
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,'    mmMgPrecon pre-smooth 1')") ip
#endif
#endif

         CALL mmSmooth (
     &        
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &        nx,ny,nz,nxg,nyg,nzg,wgt,nu,iSmoothIt,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,1,isub)

         nu = nu + numPreSmooth
C         print*, nxdxf,nydxf,nzdxf,nx(1),ny(1),nz(1)
C******************************************
C            CALL mmWriteVector ('X1.out    ',XF,
C     &           nxdxf,nydxf,nzdxf,nx,ny,nz,
C     &           gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
C     &           comm,nbr)
C******************************************
C             CALL mmWriteVector ('BF1.out   ',BF,
C      &           nxdbf,nydbf,nzdbf,nx,ny,nz,
C      &           gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
C      &           comm,nbr)
C             CALL mmTerminate(comm)

#ifdef DEBUG_MG
      s = mmVectorSum (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '    mmMgPrecon sum(X) = ',s
#endif

#ifdef DEBUG_MG
      if (ip.EQ.0) PRINT*, '   mmMgPrecon Residual 2'
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,'    mmMgPrecon Residual 2')") ip
#endif
#endif

         CALL mmResidual (
     &        
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,0,1,isub)

C******************************************
C            CALL mmWriteVector ('R1.out    ',R,
C     &           nxdr,nydr,nzdr,nx,ny,nz,
C     &        gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
C     &           comm,nbr)
C******************************************

#ifdef DEBUG_MG
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '    mmMgPrecon sum(R) = ',s
#endif

#ifdef DEBUG_MG
      if (ip.EQ.0) WRITE(*,"('   mmMgPrecon Restrict',i2,' -->',i2)") 
     &     k,kc
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,'    mmMgPrecon Restrict')") ip
#endif
#endif
         CALL mmRestrict (
     &        
     &     BC(ica),nxdbc(kc),nydbc(kc),nzdbc(kc),bndbc(1,kc)
     &     ,
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &        nx(kc),ny(kc),nz(kc),nx(k),ny(k),nz(k),
     &        hx(ifh),hy(ifh),hz(ifh),
     &        comm,nbr,bnd(1,kc),bnd(1,k),k,isub)

C******************************************
C            CALL mmWriteVector ('RC.out    ',BC,
C     &           nxdbc,nydbc,nzdbc,nx(2),ny(2),nz(2),
C     &        gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
C     &           comm,nbr)
C******************************************
#ifdef DEBUG_MG
      s = mmVectorSum (
     &     BC(ica),nxdbc(kc),nydbc(kc),nzdbc(kc),bndbc(1,kc)
     &     ,nx(kc),ny(kc),nz(kc),comm,isub)
      if (ip.EQ.0) PRINT*, '    mmMgPrecon sum(B) = ',s
#endif

      END IF

      DO k=2,m-1

         ifx = indx(k)
         ifa = inda(k)
         ifh = indh(k)
         kc = k+1
         icx = indx(kc)
         ica = inda(kc)
         ich = indh(kc)

#ifdef DEBUG_MG
      if (ip.EQ.0) PRINT*, '   mmMgPrecon pre-smooth',k
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,'    mmMgPrecon pre-smooth')") ip
#endif
      s = mmVectorSum (
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,nx(k),ny(k),nz(k),comm,isub)
      if (ip.EQ.0) PRINT*, '    mmMgPrecon sum(Xin) = ',s
#endif
         CALL mmSmooth (
     &        
     &     AD(ifa),A1(ifa),A2(ifa),A3(ifa),
     &     A4(ifa),A5(ifa),A6(ifa),nxda(k),nyda(k),nzda(k),
     &     bnda(1,k)
     &     ,
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,
     &     BC(ifa),nxdbc(k),nydbc(k),nzdbc(k),bndbc(1,k)
     &     ,
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &        
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &        nx(k),ny(k),nz(k),nxg(k),nyg(k),nzg(k),wgt,nu,iSmoothIt,
     &        
     &     xp(ifh),yp(ifh),zp(ifh),
     &     hx(ifh),hy(ifh),hz(ifh)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd(1,k)
     &     ,
     &        scaleA,scaleB,k,isub)

         nu = nu + numPreSmooth

#ifdef DEBUG_MG
      s = mmVectorSum (
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,nx(k),ny(k),nz(k),comm,isub)
      if (ip.EQ.0) PRINT*, '    mmMgPrecon sum(Xout) = ',s
#endif

#ifdef DEBUG_MG
      if (ip.EQ.0) PRINT*, '   mmMgPrecon Residual 3:',k
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,'    mmMgPrecon Residual 3')") ip
#endif
#endif
         CALL mmResidual (
     &        
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &     AD(ifa),A1(ifa),A2(ifa),A3(ifa),
     &     A4(ifa),A5(ifa),A6(ifa),nxda(k),nyda(k),nzda(k),
     &     bnda(1,k)
     &     ,
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,
     &     BC(ifa),nxdbc(k),nydbc(k),nzdbc(k),bndbc(1,k)
     &     ,
     &        nx(k),ny(k),nz(k),
     &        
     &     xp(ifh),yp(ifh),zp(ifh),
     &     hx(ifh),hy(ifh),hz(ifh)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd(1,k)
     &     ,
     &        scaleA,scaleB,0,k,isub)

#ifdef DEBUG_MG
      s = mmVectorSum (
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,nx(k),ny(k),nz(k),comm,isub)
      if (ip.EQ.0) PRINT*, '    mmMgPrecon sum(R) = ',s
#endif

#ifdef DEBUG_MG
      if (ip.EQ.0) WRITE(*,"('   mmMgPrecon Restrict',i2,' -->',i2)") 
     &     k,kc
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,'    mmMgPrecon Restrict')") ip
#endif
#endif
         CALL mmRestrict (
     &        
     &     BC(ica),nxdbc(kc),nydbc(kc),nzdbc(kc),bndbc(1,kc)
     &     ,
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &        nx(kc),ny(kc),nz(kc),nx(k),ny(k),nz(k),
     &        hx(ifh),hy(ifh),hz(ifh),
     &        comm,nbr,bnd(1,kc),bnd(1,k),k,isub)

#ifdef DEBUG_MG
      s = mmVectorSum (
     &     BC(ica),nxdbc(kc),nydbc(kc),nzdbc(kc),bndbc(1,kc)
     &     ,nx(kc),ny(kc),nz(kc),comm,isub)
      if (ip.EQ.0) PRINT*, '    mmMgPrecon sum(B) = ',s
#endif

      END DO

*-----------------------------------------------------------------------
*     Solve the coarse-grid problem
*-----------------------------------------------------------------------

      ica = inda(m)
      icx = indx(m)
      ich = indh(m)

#ifdef DEBUG_MG
      if (ip.EQ.0) PRINT*, '   mmMgPrecon coarse grid solve',m
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,'    mmMgPrecon coarse grid solve')") ip
#endif
#endif

      iStatCoarse=0
      IF (iCoarseSolver.EQ.MM_JACOBI_COARSE_SOLVER) THEN

         IF (m.NE.1) THEN

            CALL mmSmooth (
     &           
     &     AD(ica),A1(ica),A2(ica),A3(ica),
     &     A4(ica),A5(ica),A6(ica),nxda(m),nyda(m),nzda(m),
     &     bnda(1,m)
     &     ,
     &     XC(ica),nxdxc(m),nydxc(m),nzdxc(m),bndxc(1,m)
     &     ,
     &     BC(ica),nxdbc(m),nydbc(m),nzdbc(m),bndbc(1,m)
     &     ,
     &           
     &     R(icx),nxdr(m),nydr(m),nzdr(m),bndr(1,m)
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     , 
     &           
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &           nx(m),ny(m),nz(m),nxg(m),nyg(m),nzg(m),wgt,nu,
     &           iSmoothIt,
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &           
     &     comm,nbr,bnd(1,m)
     &     ,scaleA,scaleB,m,isub)

         ELSE

            CALL mmSmooth (
     &           
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     R(icx),nxdr(m),nydr(m),nzdr(m),bndr(1,m)
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     , 
     &           
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &           nx(m),ny(m),nz(m),nxg(m),nyg(m),nzg(m),wgt,nu,
     &           iSmoothIt,
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &           
     &     comm,nbr,bnd(1,m)
     &     ,scaleA,scaleB,m,isub)

         END IF

         iterCoarse = iterCoarse + maxItCoarse
      END IF

      IF (iCoarseSolver.EQ.MM_CG_COARSE_SOLVER) THEN

         IF (m.NE.1) THEN

            CALL mmCg (
     &           
     &     AD(ica),A1(ica),A2(ica),A3(ica),
     &     A4(ica),A5(ica),A6(ica),nxda(m),nyda(m),nzda(m),
     &     bnda(1,m)
     &     ,
     &     XC(ica),nxdxc(m),nydxc(m),nzdxc(m),bndxc(1,m)
     &     ,
     &     BC(ica),nxdbc(m),nydbc(m),nzdbc(m),bndbc(1,m)
     &     ,
     &           
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &           nx(m),ny(m),nz(m),nxg(m),nyg(m),nzg(m),
     &           
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,0,
     &           iterCoarse,maxItCoarse,resTolCoarse,0,
     &           
     &     comm,nbr,bnd(1,m)
     &     ,scaleA,scaleB,m,
     &           resCoarse,relResCoarse,istatCoarse,isub)

         ELSE 

            CALL mmCg (
     &           
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &           nx(m),ny(m),nz(m),nxg(m),nyg(m),nzg(m),
     &           
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,0,
     &           iterCoarse,maxItCoarse,resTolCoarse,0,
     &           
     &     comm,nbr,bnd(1,m)
     &     ,scaleA,scaleB,m,
     &           resCoarse,relResCoarse,istatCoarse,isub)

         END IF

      END IF

      IF (iStatCoarse.NE.0 .AND.outputLevel.NE.0 .AND. ip.EQ.0) THEN
         PRINT*, '   -- mgmpi warning -- Coarse solver returned ',
     &        iStatCoarse
      END IF
      
#ifdef DEBUG_MG
      s = mmVectorSum (
     &     XC(ica),nxdxc(m),nydxc(m),nzdxc(m),bndxc(1,m)
     &     ,nx(m),ny(m),nz(m),comm,isub)
      if (ip.EQ.0) PRINT*, '    mmMgPrecon sum(X) = ',s
#endif
      

*-----------------------------------------------------------------------
*     Prolong correction and smooth to finer grids
*-----------------------------------------------------------------------

#define NORMALIZE .TRUE.
#define REFRESH .TRUE.

      nu = m*numPostSmooth
      DO k=m-1,2,-1

         kc = k+1
         ica = inda(kc)
         icx = indx(kc)
         ich = indh(kc)
         ifa = inda(k)
         ifx = indx(k)
         ifh = indh(k)

***********(2) ***************
#ifdef DEBUG_MG
      if (ip.EQ.0) PRINT*, '   mmMgPrecon normalize',kc
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,'    mmMgPrecon normalize')") ip
#endif
#endif
         IF (NORMALIZE) THEN
            tmp = mmVectorNormalize (
     &           
     &     XC(ica),nxdxc(kc),nydxc(kc),nzdxc(kc),bndxc(1,kc)
     &     ,
     &           nx(kc),ny(kc),nz(kc),
     &           nxg(kc),nyg(kc),nzg(kc),
     &           
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &           comm,isub)

         END IF
            
***********(1) ***************

         IF (REFRESH) THEN
            CALL mmRefreshBC (
     &           
     &     XC(ica),nxdxc(kc),nydxc(kc),nzdxc(kc),bndxc(1,kc)
     &     ,nx(kc),ny(kc),nz(kc),
     &           
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &           
     &     comm,nbr,bnd(1,kc)
     &     ,scaleA,scaleB,kc,isub)
         END IF

#ifdef DEBUG_MG
      s = mmVectorSum (
     &     XC(ica),nxdxc(kc),nydxc(kc),nzdxc(kc),bndxc(1,kc)
     &     ,nx(kc),ny(kc),nz(kc),comm,isub)
      if (ip.EQ.0) PRINT*, '   mmMgPrecon  sum(X) = ',s
#endif

#ifdef DEBUG_MG
      if (ip.EQ.0) WRITE(*,"('   mmMgPrecon Prolong',i2,' <--',i2)") k,kc
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,'    mmMgPrecon Prolong')") ip
#endif
#endif
         CALL mmProlong (
     &        
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &     XC(ica),nxdxc(kc),nydxc(kc),nzdxc(kc),bndxc(1,kc)
     &     ,
     &     AD(ifa),A1(ifa),A2(ifa),A3(ifa),
     &     A4(ifa),A5(ifa),A6(ifa),nxda(k),nyda(k),nzda(k),
     &     bnda(1,k)
     &     ,
     &        nx(k),ny(k),nz(k),nx(kc),ny(kc),nz(kc),
     &        hx(ifh),hy(ifh),hz(ifh),
     &     comm,nbr,bnd(1,kc)
     &     ,k,.FALSE.,isub)

#ifdef DEBUG_MG
      s = mmVectorSum (
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,nx(k),ny(k),nz(k),comm,isub)
      if (ip.EQ.0) PRINT*, '    mmMgPrecon sum(R) = ',s
#endif

#ifdef DEBUG_MG
      if (ip.EQ.0) PRINT*, '   mmMgPrecon correct',k
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,'    mmMgPrecon correct')") ip
#endif
#endif
         CALL mmCorrect (
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,nx(k),ny(k),nz(k)
     &     ,isub)

#ifdef DEBUG_MG
      s = mmVectorSum (
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,nx(k),ny(k),nz(k),comm,isub)
      if (ip.EQ.0) PRINT*, '    mmMgPrecon sum(X) = ',s
#endif
         nu = nu - numPostSmooth

#ifdef DEBUG_MG
      if (ip.EQ.0) PRINT*, '   mmMgPrecon post-smooth',k
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,'    mmMgPrecon post-smooth')") ip
#endif
#endif
         CALL mmSmooth (
     &        
     &     AD(ifa),A1(ifa),A2(ifa),A3(ifa),
     &     A4(ifa),A5(ifa),A6(ifa),nxda(k),nyda(k),nzda(k),
     &     bnda(1,k)
     &     ,
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,
     &     BC(ifa),nxdbc(k),nydbc(k),nzdbc(k),bndbc(1,k)
     &     ,
     &        
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &        
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &        nx(k),ny(k),nz(k),nxg(k),nyg(k),nzg(k),wgt,nu,iSmoothIt,
     &        
     &     xp(ifh),yp(ifh),zp(ifh),
     &     hx(ifh),hy(ifh),hz(ifh)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd(1,k)
     &     ,
     &        scaleA,scaleB,k,isub)

#ifdef DEBUG_MG
      s = mmVectorSum (
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,nx(k),ny(k),nz(k),comm,isub)
      if (ip.EQ.0) PRINT*, '   mmMgPrecon sum(X) = ',s
#endif

      END DO

      IF (m.GT.1) THEN

         kc = 2
         ica = inda(2)
         icx = indx(2)
         ich = indh(2)
         k = 1
         ifa = inda(1)
         ifx = indx(1)
         ifh = indh(1)

***********(2) ***************
#ifdef DEBUG_MG
      if (ip.EQ.0) PRINT*, '   mmMgPrecon normalize',2
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,'    mmMgPrecon normalize')") ip
#endif
#endif
         IF (NORMALIZE) THEN
            tmp = mmVectorNormalize (
     &           
     &     XC(ica),nxdxc(2),nydxc(2),nzdxc(2),bndxc(1,2)
     &     ,
     &           nx(2),ny(2),nz(2),
     &           nxg(2),nyg(2),nzg(2),
     &           
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &           comm,isub)
         END IF

         IF (REFRESH) THEN
            CALL mmRefreshBC (
     &           
     &     XC(ica),nxdxc(2),nydxc(2),nzdxc(2),bndxc(1,2)
     &     ,nx(2),ny(2),nz(2),
     &           
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &           
     &     comm,nbr,bnd(1,2)
     &     ,scaleA,scaleB,2,isub)
         END IF

#ifdef DEBUG_MG
      s = mmVectorSum (
     &     XC(ica),nxdxc(2),nydxc(2),nzdxc(2),bndxc(1,2)
     &     ,nx(2),ny(2),nz(2),comm,isub)
      if (ip.EQ.0) PRINT*, '   mmMgPrecon sum(X) = ',s
#endif

#ifdef DEBUG_MG
      if (ip.EQ.0) WRITE(*,"('   mmMgPrecon Prolong',i2,' <--',i2)") k,kc
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,'    mmMgPrecon Prolong')") ip
#endif
#endif

      CALL mmProlong (
     &     R(ifx),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,
     &     XC(ica),nxdxc(2),nydxc(2),nzdxc(2),bndxc(1,2)
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     nx(1),ny(1),nz(1),nx(2),ny(2),nz(2),
     &     hx,hy,hz,
     &     comm,nbr,bnd(1,2)
     &     ,1,.FALSE.,isub)
      
C           IF (gisymmetric.EQ.0) THEN
C             CALL mmWriteVector ('   Pns.out     ',R(ifx),
C      &           nxdr,nydr,nzdr,nx,ny,nz,
C      &        gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
C      &           comm,nbr)
C          ELSE 
C             CALL mmWriteVector ('   Ps.out      ',R(ifx),
C      &           nxdr,nydr,nzdr,nx,ny,nz,
C       &        gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
C      &           comm,nbr)
C          END IF


#ifdef DEBUG_MG
      s = mmVectorSum (
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,nx(k),ny(k),nz(k),comm,isub)
      if (ip.EQ.0) PRINT*, '   mmMgPrecon sum(R) = ',s
#endif

#ifdef DEBUG_MG
      if (ip.EQ.0) PRINT*, '   mmMgPrecon correct',k
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,'    mmMgPrecon correct')") ip
#endif
#endif

         CALL mmCorrect (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     R(ifx),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,nx(1),ny(1),nz(1),isub)

C           IF (gisymmetric.EQ.0) THEN
C             CALL mmWriteVector ('Xns.out     ',XF,
C      &           nxdxf,nydxf,nzdxf,nx,ny,nz,
C      &        gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
C      &           comm,nbr)
C          ELSE 
C             CALL mmWriteVector ('Xs.out      ',XF,
C      &           nxdxf,nydxf,nzdxf,nx,ny,nz,
C      &        gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
C      &           comm,nbr)
C          END IF

#ifdef DEBUG_MG
      s = mmVectorSum (
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,nx(k),ny(k),nz(k),comm,isub)
      if (ip.EQ.0) PRINT*, '   mmMgPrecon sum(X) = ',s
#endif

         nu = nu - numPostSmooth

#ifdef DEBUG_MG
      if (ip.EQ.0) PRINT*, '   mmMgPrecon post-smooth 1'
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,'    mmMgPrecon post-smooth')") ip
#endif
#endif
         CALL mmSmooth (
     &        
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &        nx,ny,nz,nxg,nyg,nzg,wgt,nu,iSmoothIt,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,1,isub)

#ifdef DEBUG_MG
      s = mmVectorSum (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, '   mmMgPrecon sum(X) = ',s
#endif

      END IF

      iter = iter + 1
      iterFine = iterFine + 1

#ifdef DEBUG_MG
      if (ip.EQ.0) PRINT*, '   mmMgPrecon Smooth 1'
#ifdef DEBUG_PROCS
      WRITE(*,"(i2,'    mmMgPrecon Smooth')") ip
#endif
#endif
      CALL mmResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,0,1,isub)

#ifdef DEBUG_MG
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      if (ip.EQ.0) PRINT*, 1,'   mmMgPrecon sum(RF) = ',s
#endif

      rnormPrev = rnorm
      rnorm = DSQRT (mmDotProd 
     &     (
     &     R(1),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,
     &     R(1),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,nx(1),ny(1),nz(1),comm,1,isub))

      IF (outputLevel.EQ.2 .AND. ip.EQ.0)
     &     WRITE(*,1000) iter,MPI_WTIME() - gtimestart(SB_MG3),
     &     rnorm/rnormFirst

      IF (iter.LT.maxItFine) GO TO 10
      

*-----------------------------------------------------------------------
*     Exit subroutine, printing warning message if applicable
*-----------------------------------------------------------------------

      IF (ip.EQ.0.AND.outputLevel.GT.0) THEN
         IF (istat.EQ.1) print*,
     &        '== mgmpi ERROR == aborting MG: apparently diverging'
         IF (istat.EQ.2) print*,
     &        '== mgmpi ERROR == aborting MG: residual norm is NaN'
         IF (istat.EQ.3) print*,
     &        '-- mgmpi warning -- exiting MG: iteration limit reached'
      END IF

      IF (ip.EQ.0 .AND. outputLevel.EQ.1)
     &     print*, 'MG: ',iter,rnorm/rnormFirst
      IF (ip.EQ.0 .AND. outputLevel.GT.0)
     &     print*, 'R.R.F.: ',(rnorm/rnormFirst)**(1.0D0/iter)

      resRedFine = rnorm / rnormFirst

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN

 1000 FORMAT (5x,i4,f8.2,e14.3)

      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmMgSolve2
C     
C     Purpose: Initialize and call a specified solver
C
C     Algorithm: 
C
C     Inputs: 
C
C     Outputs: 
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate: 
C
C     Communication estimate: 
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmMgSolve2 (
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,nx,ny,nz,nxg,nyg,nzg,
     &     indh,inda,indx
     &     ,iGridTypeX,iGridTypeY,iGridTypeZ,
     &     comm,nbr,bnd,bnd2
     &     ,icoords,
     &     m,wgt,numPreSmooth,numPostSmooth,iSmoothIt,
     &     iFineSolver,iCoarseSolver,
     &     maxItFine,maxItCoarse,
     &     resTolFine,resTolCoarse,totalTime,
     &     outputLevel,
     &     iterFine,resFine,resRedFine,istat,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     ifng,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     P,nxdp,nydp,nzdp,bndp
     &     ,
     &     W,nxdw,nydw,nzdw,bndw
     &     ,
     &     RMG,nxdrmg,nydrmg,nzdrmg,bndrmg
     &     ,
     &     ZMG,nxdzmg,nydzmg,nzdzmg,bndzmg
     &     ,
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &     RT,nxdrt,nydrt,nzdrt,bndrt
     &     ,
     &     V,nxdv,nydv,nzdv,bndv
     &     ,
     &     PT,nxdpt,nydpt,nzdpt,bndpt
     &     ,
     &     S,nxds,nyds,nzds,bnds
     &     ,
     &     ST,nxdst,nydst,nzdst,bndst
     &     ,
     &     T,nxdt,nydt,nzdt,bndt
     &     ,
     &     xp,yp,zp,hx,hy,hz
     &     ,nzl,icall)

      IMPLICIT NONE

      INTEGER m
      
      INTEGER nxdaf,nydaf,nzdaf
      INTEGER bndaf(3)  
      REAL AFD(*)
      REAL AF1(*)
      REAL AF2(*)
      REAL AF3(*)
      REAL AF4(*)
      REAL AF5(*)
      REAL AF6(*)
      INTEGER nx(m),ny(m),nz(m)
      INTEGER nxg(m),nyg(m),nzg(m)
      INTEGER indh(*),inda(*),indx(*)
      INTEGER iGridTypeX,iGridTypeY,iGridTypeZ
      
      INTEGER comm,nbr(6),bnd(3,*),bnd2(3)
      INTEGER icoords
      REAL wgt,resTolFine,resTolCoarse,totalTime
      INTEGER numPreSmooth,numPostSmooth,iSmoothIt
      INTEGER istat
      INTEGER iFineSolver,iCoarseSolver
      INTEGER maxItFine,maxItCoarse
      REAL resFine, resRedFine
      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      
      INTEGER nxdbf,nydbf,nzdbf
      INTEGER bndbf(3)	
      REAL BF(nxdbf,nydbf,nzdbf)
      INTEGER ifng	
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER nxda(*),nyda(*),nzda(*)
      INTEGER bnda(3,*)  
      REAL AD(*)	
      REAL A1(*)
      REAL A2(*)
      REAL A3(*)
      REAL A4(*)
      REAL A5(*)
      REAL A6(*)
      
      INTEGER nxdx(*),nydx(*),nzdx(*)
      INTEGER bndx(3,*)  
      REAL X(*)
      
      INTEGER nxdy(*),nydy(*),nzdy(*)
      INTEGER bndy(3,*)
      REAL Y(*)
      
      INTEGER nxdb(*),nydb(*),nzdb(*)
      INTEGER bndb(3,*)
      REAL B(*)
      
      INTEGER nxdr(*),nydr(*),nzdr(*)
      INTEGER bndr(3,*)	
      REAL R(*)
      
      INTEGER nxdp(*),nydp(*),nzdp(*)
      INTEGER bndp(3,*)	
      REAL P(*)
      
      INTEGER nxdw(*),nydw(*),nzdw(*)
      INTEGER bndw(3,*)
      REAL W(*)
      
      INTEGER nxdrmg(*),nydrmg(*),nzdrmg(*)
      INTEGER bndrmg(3,*)
      REAL RMG(*)
      
      INTEGER nxdzmg(*),nydzmg(*),nzdzmg(*)
      INTEGER bndzmg(3,*)
      REAL ZMG(*)
      
      INTEGER nxdrcg,nydrcg,nzdrcg
      INTEGER bndrcg(3)
      REAL RCG(nxdrcg,nydrcg,nzdrcg)
      
      INTEGER nxdpcg,nydpcg,nzdpcg
      INTEGER bndpcg(3)
      REAL PCG(nxdpcg,nydpcg,nzdpcg)
      
      INTEGER nxdwcg,nydwcg,nzdwcg
      INTEGER bndwcg(3)
      REAL WCG(nxdwcg,nydwcg,nzdwcg)
      
      INTEGER nxdrsm,nydrsm,nzdrsm
      INTEGER bndrsm(3)
      REAL RSM(nxdrsm,nydrsm,nzdrsm)
      
      INTEGER nxdpsm,nydpsm,nzdpsm
      INTEGER bndpsm(3)
      REAL PSM(nxdpsm,nydpsm,nzdpsm)
      
      INTEGER nxdwsm,nydwsm,nzdwsm
      INTEGER bndzsm(3)
      REAL WSM(nxdwsm,nydwsm,nzdwsm)
      
      INTEGER nxdrt(*),nydrt(*),nzdrt(*)
      INTEGER bndrt(3,*)	
      REAL RT(*)
      
      INTEGER nxdv(*),nydv(*),nzdv(*)
      INTEGER bndv(3,*)	
      REAL V(*)
      
      INTEGER nxdpt(*),nydpt(*),nzdpt(*)
      INTEGER bndpt(3,*)	
      REAL PT(*)
      
      INTEGER nxds(*),nyds(*),nzds(*)
      INTEGER bnds(3,*)	
      REAL S(*)
      
      INTEGER nxdst(*),nydst(*),nzdst(*)
      INTEGER bndst(3,*)	
      REAL ST(*)
      
      INTEGER nxdt(*),nydt(*),nzdt(*)
      INTEGER bndt(3,*)	
      REAL T(*)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      INTEGER nzl
      INTEGER outputLevel,iterFine,iterCoarse

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      REAL mmVectorSum,mmVectorMax,mmVectorNormalize
*
      REAL x1,y1,z1,scaleA(MLIM),scaleB(MLIM),tmp,shft
      INTEGER i,j,ix,iy,iz,k,kc,kf
      INTEGER ica,ifa,icx,ifx,ich,ifh

#ifdef MPI_USED
      INTEGER ierr, status(MPI_STATUS_SIZE)
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MGSOLVE
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif


C       X(1) = 1
C       Y(1) = 2
C       B(1) = 3
C       R(1) = 4
C       P(1) = 5
C       W(1) = 6
C       RMG(1) = 7
C       ZMG(1) = 8
C       RT(1) = 9
C       V(1) = 10
C       PT(1) = 11
C       S(1) = 12
C       ST(1) = 13
C       T(1) = 14
C 
C       PRINT*, 'mmMgSolve2 X1=',X(1)
C       PRINT*, 'mmMgSolve2 Y1=',Y(1)
C       PRINT*, 'mmMgSolve2 B1=',B(1)
C       PRINT*, 'mmMgSolve2 R1=',R(1)
C       PRINT*, 'mmMgSolve2 P1=',P(1)
C       PRINT*, 'mmMgSolve2 W1=',W(1)
C       PRINT*, 'mmMgSolve2 RMG1=',RMG(1)
C       PRINT*, 'mmMgSolve2 ZMG1=',ZMG(1)
C       PRINT*, 'mmMgSolve2 RT1=',RT(1)
C       PRINT*, 'mmMgSolve2 V1=',V(1)
C       PRINT*, 'mmMgSolve2 PT1=',PT(1)
C       PRINT*, 'mmMgSolve2 S1=',S(1)
C       PRINT*, 'mmMgSolve2 ST1=',ST(1)
C       PRINT*, 'mmMgSolve2 T1=',T(1)
      
#ifdef DEBUG
      if (ip.EQ.0) print*, ip, 'mmMgSolve2: enter'
      if (ip.EQ.0) print*, 'mmMgSolve2: nx(1:2) = ',nx(1),nx(2)
      if (ip.EQ.0) print*, 'mmMgSolve2: nxdaf nxda(1:2) = ',nxdaf,
     &     nxda(1),nxda(2)
      if (ip.EQ.0) print*, 'mmMgSolve2: nxdb(1:2) = ',nxdb(1),nxdb(2)
#endif

#ifdef DEBUG
      tmp = mmVectorSum (
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,comm,isub) 
      IF (ip.EQ.0) print*,  'mmMgsolve2:sum(BF)=', tmp
      IF (gimatfree.EQ.0) THEN
CCCCC @@ Macro dependent parameters
         tmp = mmVectorSum (AFD,nxdaf,nydaf,nzdaf,bndaf,
     &        nx,ny,nz,comm,isub) 
         IF (ip.EQ.0) print*,  'mmMgsolve2:sum(AFD)=', tmp
      END IF
      IF (ip.EQ.0) print*, 'nxdaf...=',nxdaf,nydaf,nzdaf
      IF (ip.EQ.0) print*, 'bndaf=',bndaf
      IF (ip.EQ.0) print*, 'nx(1),ny(1),nz(1)=',nx(1),ny(1),nz(1)
      IF (ip.EQ.0) print*, 'nx(2),ny(2),nz(2)=',nx(2),ny(2),nz(2)
      CALL mmResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx(1),ny(1),nz(1),
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     1.0D0,1.0D0,0,1,isub)
      CALL mmVectorInfo (' R  ',R,nxdr,nydr,nzdr,bndr,
     &     nx(1),ny(1),nz(1),comm)
#endif

*-----------------------------------------------------------------------
*     If problem is singular, remove null-space component from R.H.S.
*-----------------------------------------------------------------------

C @@@@ Why is this commented out???
C
C       IF (ABS(ixmb).NE.MM_DIRICHLET .AND.
C      &    ABS(iymb).NE.MM_DIRICHLET .AND. 
C      &    ABS(izmb).NE.MM_DIRICHLET .AND.
C      &    ABS(ixpb).NE.MM_DIRICHLET .AND.
C      &    ABS(iypb).NE.MM_DIRICHLET .AND.
C      &    ABS(izpb).NE.MM_DIRICHLET) THEN
C       
C          shft = mmVectorSum (BF,nxd2,nyd2,nzd2,nx,ny,nz,comm,isub) 
C      &        / ((nxg(1)-2)*(nyg(1)-2)*(nzg(1)-2))
C 
C          DO iz=1,nz(1)
C             DO iy=1,ny(1)
C                DO ix=1,nx(1)
C                   BF(ix,iy,iz) = BF(ix,iy,iz) - shft
C                END DO
C             END DO
C          END DO
C       END IF

*-----------------------------------------------------------------------
*     Get hx,hy,hz for fine grid
*-----------------------------------------------------------------------
*         |       |          |       | 
*         o o o o o o        o o o o o
*         |       |          |       |  
*         O   O   O   O      O   O   O 
*         |       |          |       |  
*
*         o o o o o o        o o o o o
*  xpf:   1 2 3 4 5 6        1 2 3 4 5
*         O   O   O   O      O   O   O 
*  xpc:   1   2   3   4      1   2   3
*
*         o o o o o o        o o o o o
*  hxf:  1 2 3 4 5 6        1 2 3 4 5
*         O   O   O   O      O   O   O 
*  hxc: 1   2   3   4      1   2   3
*
*

*          ip-1            ip
*    hx:        | 1 : 2   3  ...  nx-1 : nx |
*               /   /   /   /          /
*    xp:   nx-1 | 1 : 2   3  ...  nx-1 : nx |
*-----------------------------------------------------------------------

#ifdef DEBUG
      if (ip.EQ.0) print*, ip, 'mmMgSolve2: get hx for fine grid'
      if (ip.EQ.0) print*, nx(1),ny(1),nz(1)
#endif
      
      DO ix=2,nx(1)
          hx(ix) = xp(ix) - xp(ix-1)
      END DO
      DO iy=2,ny(1)
          hy(iy) = yp(iy) - yp(iy-1)
      END DO
      DO iz=2,nz(1)
          hz(iz) = zp(iz) - zp(iz-1)
      END DO

#ifdef MPI_USED

      IF (ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmMgSolve2: MPI_Send')") ip
#endif
         CALL MPI_Send (hx(nx(1)-1),1,MM_REAL,
     &        nbr(XPD),XPD,comm,ierr)
      END IF
      IF (ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmMgSolve2: MPI_Send')") ip
#endif
         CALL MPI_Send (hy(ny(1)-1),1,MM_REAL,
     &        nbr(YPD),YPD,comm,ierr)
      END IF
      IF (ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmMgSolve2: MPI_Send')") ip
#endif
         CALL MPI_Send (hz(nz(1)-1),1,MM_REAL,
     &        nbr(ZPD),ZPD,comm,ierr)
      END IF

      IF (ipx.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmMgSolve2: MPI_Recv')") ip
#endif
         CALL MPI_Recv (hx(1),1,MM_REAL,
     &        nbr(XMD),XPD,comm,status,ierr)
      END IF
      IF (ipy.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmMgSolve2: MPI_Recv')") ip
#endif
         CALL MPI_Recv (hy(1),1,MM_REAL,
     &        nbr(YMD),YPD,comm,status,ierr)
      END IF
      IF (ipz.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmMgSolve2: MPI_Recv')") ip
#endif
         CALL MPI_Recv (hz(1),1,MM_REAL,
     &        nbr(ZMD),ZPD,comm,status,ierr)
      END IF
#endif

*-----------------------------------------------------------------------
*     Get hx,hy,hz for coarse grids
*-----------------------------------------------------------------------

#ifdef DEBUG
      if (ip.EQ.0) print*, ip, 'mmMgSolve2: get coarse mesh info'
#endif
      DO k=2,m
         ifh=indh(k-1)
         ich=indh(k)
         CALL mmGetCoarseMeshWidths (
     &        hx(ifh),hy(ifh),hz(ifh),hx(ich),hy(ich),hz(ich),
     &        nx(k-1),ny(k-1),nz(k-1),nx(k),ny(k),nz(k),
     &     comm,nbr,bnd
     &     ,k)
         CALL mmGetCoarseMeshPoints (
     &        xp(ifh),yp(ifh),zp(ifh),xp(ich),yp(ich),zp(ich),
     &        nx(k-1),ny(k-1),nz(k-1),nx(k),ny(k),nz(k),
     &     comm,nbr,bnd
     &     ,k)
      END DO

* ----------------------------------------------------------------------
*     Initialize level-dependent global variables
* ----------------------------------------------------------------------

#ifdef DEBUG
      if (ip.EQ.0) print*, ip, 'mmMgSolve2: set globals'
#endif

      DO k=2,m
         ifh = indh(k)
         CALL mmInitCoords (nx(k),ny(k),nz(k),xp(ifh),yp(ifh),zp(ifh),
     &        icoords,k)
      END DO

*=======================================================================
*     Get starting x,y,z
*=======================================================================

      x1 = xp(1)
      y1 = yp(1)
      z1 = zp(1)

*=======================================================================
*     Scale problem
*=======================================================================

#ifdef DEBUG
      if (ip.EQ.0) print*, ip, 'mmMgSolve2: find scaling'
#endif
      scaleA(1) = 1.0D0
      scaleB(1) = 1.0D0

*-----------------------------------------------------------------------
*     Scale input vector F
*-----------------------------------------------------------------------

      IF (ifng.EQ.-1) THEN
         DO iz=1,nz(1)
            DO iy=1,ny(1)
               DO ix=1,nx(1)
                  BF(ix,iy,iz) = -BF(ix,iy,iz)
               END DO
            END DO
         END DO
      END IF

#ifdef DEBUG
      tmp = mmVectorSum (
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) print*,  'mmMgsolve2:B:', tmp
#endif

      shft = mmVectorNormalize (
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,nxg,nyg,nzg,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,comm,isub)

      IF (gimatfree.EQ.1) THEN
         CALL mmMatDiag (
     &     R(1),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,0, nx(1),ny(1),nz(1),
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,comm,1.0D0,isub)
         scaleA(1) = 1.0D0/mmVectorMax(
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm)
      ELSE
CCCCC @@ Macro dependent parameters
         scaleA(1) = 1.0D0/mmVectorMax(AFD,nxdaf,nydaf,nzdaf,bndaf,
     &        nx,ny,nz,comm)
      END IF

      IF (mmVectorMax(
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,comm).GT.0.0D0) THEN
         scaleB(1) = 1.0D0/mmVectorMax(
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,comm)
      ELSE
         scaleB(1) = 1.0D0
      END IF

      scaleA(1) = 2.0D0**ANINT(LOG(ScaleA(1))/LOG(2.0D0))
      scaleB(1) = 2.0D0**ANINT(LOG(ScaleB(1))/LOG(2.0D0))

      CALL mmScale (
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,scaleA(1),scaleB(1),comm)

#ifdef DEBUG
      if (ip.EQ.0) print*, 'mmMgsolve2:scale=',scaleA(1),scaleB(1)
#endif

      gScaleA = scaleA(1)
      gScaleB = scaleB(1)

#ifdef DEBUG
      if (ip.EQ.0) print*, 'mmMgSolve2: 3'
      if (ip.EQ.0) print*, 'hx(1)=',hx(1)
      if (ip.EQ.0) print*, 'hx(indh(2))=',hx(indh(2))
      if (ip.EQ.0) print*, 'hx(indh(m))=',hx(indh(m))
#endif

*=======================================================================
*     Generate the coarse-grid problems
*=======================================================================

#ifdef DEBUG
      if (ip.EQ.0) print*, ip, 
     &     'mmMgSolve2: generate coarse-grid problems'
#endif

      IF ( iFineSolver.EQ.MM_MG_SOLVER .OR.
     &     iFineSolver.EQ.MM_BICGMG_SOLVER .OR.
     &     iFineSolver.EQ.MM_CGMG_SOLVER .OR.
     &     iFineSolver.EQ.MM_FMG_SOLVER) THEN

         DO k=2,m

            kc = k
            kf = k-1
            ifa=inda(kf)
            ifx=indx(kf)
            ifh=indh(kf)
            ica=inda(kc)
            icx=indx(kc)
            ich=indh(kc)

            scaleA(kc)=scaleA(kf)
            scaleB(kc)=scaleA(kf)

            IF (kf.EQ.1) THEN

#ifdef DEBUG
CCCCC @@ Macro dependent parameters
               IF (gimatfree.EQ.0) THEN
                  tmp = mmVectorSum (AFD,nxdaf,nydaf,nzdaf,bndaf,
     &                 nx,ny,nz,comm,isub) 
                  IF (ip.EQ.0) print*,  'mmMgsolve2:sum(AFD)=', tmp
CCCCC @@ Macro dependent parameters
                  tmp = mmVectorSum (AF1,nxdaf,nydaf,nzdaf,bndaf,
     &                 nx,ny,nz,comm,isub) 
                  IF (ip.EQ.0) print*,  'mmMgsolve2:sum(AF1)=', tmp
               END IF
               if (ip.EQ.0) print*, 'mmMgSolve2 calling mmBuildCoarse'
               if (ip.EQ.0) print*, 
     &              'mmMgSolve2: AC nxd,nyd,nzd,nx,ny,nz=',
     &              nxda(kc),nyda(kc),nzda(kc),nx(kc),ny(kc),nz(kc)
               if (ip.EQ.0) print*, 
     &              'mmMgSolve2: AF nxd,nyd,nzd,nx,ny,nz=',
     &              nxdaf,nydaf,nzdaf,nx(kf),ny(kf),nz(kf)
#endif
               CALL mmBuildCoarse (
     &              
     &     AD(ica),A1(ica),A2(ica),A3(ica),
     &     A4(ica),A5(ica),A6(ica),nxda(kc),nyda(kc),nzda(kc),
     &     bnda(1,kc)
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     R(ifx),nxdr(kf),nydr(kf),nzdr(kf),bndr(1,kf)
     &     ,
     &              nx(kc),ny(kc),nz(kc),nx(kf),ny(kf),nz(kf),
     &              
     &     xp(ifh),yp(ifh),zp(ifh),
     &     hx(ifh),hy(ifh),hz(ifh)
     &     ,
     &              comm,nbr,bnd(1,kc),bnd2(1),kc,isub)
            ELSE
               CALL mmBuildCoarse (
     &              
     &     AD(ica),A1(ica),A2(ica),A3(ica),
     &     A4(ica),A5(ica),A6(ica),nxda(kc),nyda(kc),nzda(kc),
     &     bnda(1,kc)
     &     ,
     &     AD(ifa),A1(ifa),A2(ifa),A3(ifa),
     &     A4(ifa),A5(ifa),A6(ifa),nxda(kf),nyda(kf),nzda(kf),
     &     bnda(1,kf)
     &     ,
     &     R(ifx),nxdr(kf),nydr(kf),nzdr(kf),bndr(1,kf)
     &     ,
     &              nx(kc),ny(kc),nz(kc),nx(kf),ny(kf),nz(kf),
     &              
     &     xp(ifh),yp(ifh),zp(ifh),
     &     hx(ifh),hy(ifh),hz(ifh)
     &     ,
     &              comm,nbr,bnd(1,kc),bnd(1,kf),kc,isub)

            END IF

            CALL mmVectorClear (
     &     X(ica),nxdx(kc),nydx(kc),nzdx(kc),bndx(1,kc)
     &     ,nx(kc),ny(kc),nz(kc))

         END DO
      END IF

#ifdef DEBUG
      if (ip.EQ.0) print*, 'mmMgSolve2: 4'
      if (ip.EQ.0) print*, 'hx(1)=',hx(1)
      if (ip.EQ.0) print*, 'hx(indh(2))=',hx(indh(2))
      if (ip.EQ.0) print*, 'hx(indh(m))=',hx(indh(m))
      if (ip.EQ.0) print*, 'iFineSolver=',iFineSolver
#endif

*=======================================================================
*     Solve the linear system
*=======================================================================

      iterFine = 0

#ifdef DEBUG
      if (ip.EQ.0) print*, ip, 'mmMgSolve2: call the linear solver'
#endif
      IF (iFineSolver.EQ.MM_CG_SOLVER) THEN

#ifdef DEBUG
      if (ip.EQ.0) print*, ip, 'mmMgSolve2: calling mmCg'
#endif
      CALL mmCg (
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     R(1),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,
     &     P(1),nxdp(1),nydp(1),nzdp(1),bndp(1,1)
     &     ,
     &     W(1),nxdw(1),nydw(1),nzdw(1),bndw(1,1)
     &     ,
     &     nx(1),ny(1),nz(1),nxg(1),nyg(1),nzg(1),
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,1,
     &     iterFine,maxItFine,resTolFine,outputLevel,
     &     comm,nbr,bnd
     &     ,scaleA(1),scaleB(1),1,
     &     resFine,resRedFine,istat,isub)

      ELSE IF (iFineSolver.EQ.MM_MG_SOLVER) THEN

#ifdef DEBUG
         if (ip.EQ.0) print*, ip,'mmMgSolve2: calling mmMgVcycle'
#endif
         CALL mmMgVcycle (
     &        
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &        
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &        
     &     indh,inda,indx
     &     ,1,m,nx,ny,nz,nxg,nyg,nzg,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver,
     &        wgt,iterFine,iterCoarse,maxItFine,maxItCoarse,
     &        resTolFine,resTolCoarse,outputLevel,
     &        
     &     comm,nbr,bnd,bnd2
     &     ,scaleA,scaleB,resFine,resRedFine,istat,isub)

      ELSE IF (iFineSolver.EQ.MM_BICG_SOLVER) THEN

#ifdef DEBUG
         if (ip.EQ.0) print*, ip,'mmMgSolve2: calling mmBiCgStab'
#endif

         CALL mmBiCgStab (
     &        
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &        
     &     R(1),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,
     &     RT(1),nxdrt(1),nydrt(1),nzdrt(1),bndrt(1,1)
     &     ,
     &     P(1),nxdp(1),nydp(1),nzdp(1),bndp(1,1)
     &     ,
     &        
     &     S(1),nxds(1),nyds(1),nzds(1),bnds(1,1)
     &     ,
     &     V(1),nxdv(1),nydv(1),nzdv(1),bndv(1,1)
     &     ,
     &     T(1),nxdt(1),nydt(1),nzdt(1),bndt(1,1)
     &     ,
     &        nx(1),ny(1),nz(1),nxg(1),nyg(1),nzg(1),
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        iterFine,maxItFine,resTolFine,outputLevel,
     &        
     &     comm,nbr,bnd
     &     ,scaleA(1),scaleB(1),1,
     &        resFine,resRedFine,istat,isub)

      ELSE IF (iFineSolver.EQ.MM_BICGMG_SOLVER) THEN
         
#ifdef DEBUG
         if (ip.EQ.0) print*, ip, 'mmMgSolve2: calling mmCgMg'
#endif
         CALL mmBiCgStabMg (
     &        
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &        
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &        
     &     R(1),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,
     &     RT(1),nxdrt(1),nydrt(1),nzdrt(1),bndrt(1,1)
     &     ,
     &     P(1),nxdp(1),nydp(1),nzdp(1),bndp(1,1)
     &     ,
     &     PT(1),nxdpt(1),nydpt(1),nzdpt(1),bndpt(1,1)
     &     ,
     &        
     &     S(1),nxds(1),nyds(1),nzds(1),bnds(1,1)
     &     ,
     &     ST(1),nxdst(1),nydst(1),nzdst(1),bndst(1,1)
     &     ,
     &     V(1),nxdv(1),nydv(1),nzdv(1),bndv(1,1)
     &     ,
     &     T(1),nxdt(1),nydt(1),nzdt(1),bndt(1,1)
     &     ,
     &        
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &        
     &     indh,inda,indx
     &     ,
     &        m,nx,ny,nz,
     &        nxg,nyg,nzg,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver,
     &        wgt,iterFine,iterCoarse,maxItFine,maxItCoarse,
     &        resTolFine,resTolCoarse,outputLevel,
     &        
     &     comm,nbr,bnd,bnd2
     &     ,scaleA,scaleB,resFine,resRedFine,istat,isub)

      ELSE IF (iFineSolver.EQ.MM_FMG_SOLVER) THEN
         
#ifdef DEBUG
         if (ip.EQ.0) print*, ip, 'mmMgSolve2: calling mmFmg'
#endif

         CALL mmFmg (
     &        
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &        
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &        
     &     indh,inda,indx
     &     ,m,nx,ny,nz,nxg,nyg,nzg,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver,
     &        wgt,iterFine,iterCoarse,maxItFine,maxItCoarse,
     &        resTolFine,resTolCoarse,outputLevel,
     &        
     &     comm,nbr,bnd,bnd2
     &     ,scaleA,scaleB,resFine,resRedFine,istat,isub)

      ELSE IF (iFineSolver.EQ.MM_CGMG_SOLVER) THEN
         
#ifdef DEBUG
         if (ip.EQ.0) print*, ip, 'mmMgSolve2: calling mmCgMg'
#endif

         CALL mmCgMg (
     &        
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &        
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &        
     &     R(1),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,
     &     P(1),nxdp(1),nydp(1),nzdp(1),bndp(1,1)
     &     ,
     &     W(1),nxdw(1),nydw(1),nzdw(1),bndw(1,1)
     &     ,
     &     ZMG(1),nxdzmg(1),nydzmg(1),nzdzmg(1),bndzmg(1,1)
     &     ,
     &        
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &        
     &     indh,inda,indx
     &     ,
     &        m,nx,ny,nz,
     &        nxg,nyg,nzg,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver,
     &        wgt,iterFine,iterCoarse,maxItFine,maxItCoarse,
     &        resTolFine,resTolCoarse,outputLevel,
     &        
     &     comm,nbr,bnd,bnd2
     &     ,scaleA,scaleB,resFine,resRedFine,istat,isub)

      END IF

*     -------------------------------------------------
*     Restore right-hand side vector
*     -------------------------------------------------

      CALL mmUnScale (
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,scaleA(1),scaleB(1),comm)

      CALL mmVectorShift (
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,-shft,comm,isub)

      IF (ifng.EQ.-1) THEN
         DO iz=1,nz(1)
            DO iy=1,ny(1)
               DO ix=1,nx(1)
                  BF(ix,iy,iz) = -BF(ix,iy,iz)
               END DO
            END DO
         END DO
      END IF

*=======================================================================
*     Output
*=======================================================================

      CALL mmGetGhost (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nx(1),ny(1),nz(1),
     &     comm,nbr,bnd
     &     ,isub)

      CALL mmRefreshBC (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,1,isub)

#ifdef DEBUG
      if (ip.EQ.0) print*, ip, 'mmMgSolve2: exit'
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmMgVcycle
C     
C     Purpose: Multigrid V-cycle solver
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C               Sep 9 -- Changed i?1(k) to gmmi?(k,ip)
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmMgVcycle (
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     BC,nxdbc,nydbc,nzdbc,bndbc
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &     indh,inda,indx
     &     ,
     &     m0,m,nx,ny,nz,
     &     nxg,nyg,nzg,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver,
     &     wgt,iterFine,iterCoarse,
     &     maxItFine,maxItCoarse,
     &     resTolFine,resTolCoarse,outputLevel,
     &     comm,nbr,bnd,bnd2
     &     ,
     &     scaleA,scaleB,rnorm,resRedFine,istat,icall)

      IMPLICIT NONE

      
      INTEGER nxdaf,nydaf,nzdaf
      INTEGER bndaf(3)  
      REAL AFD(nxdaf,nydaf,nzdaf)
      REAL AF1(nxdaf,nydaf,nzdaf)
      REAL AF2(nxdaf,nydaf,nzdaf)
      REAL AF3(nxdaf,nydaf,nzdaf)
      REAL AF4(nxdaf,nydaf,nzdaf)
      REAL AF5(nxdaf,nydaf,nzdaf)
      REAL AF6(nxdaf,nydaf,nzdaf)
      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      
      INTEGER nxdbf,nydbf,nzdbf
      INTEGER bndbf(3)	
      REAL BF(nxdbf,nydbf,nzdbf)
      
      INTEGER nxda(*),nyda(*),nzda(*)
      INTEGER bnda(3,*)  
      REAL AD(*)	
      REAL A1(*)
      REAL A2(*)
      REAL A3(*)
      REAL A4(*)
      REAL A5(*)
      REAL A6(*)
      
      INTEGER nxdxc(*),nydxc(*),nzdxc(*)
      INTEGER bndxc(3,*)  
      REAL XC(*)
      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxdbc(*),nydbc(*),nzdbc(*)
      INTEGER bndbc(3,*)
      REAL BC(*)
      
      INTEGER nxdr(*),nydr(*),nzdr(*)
      INTEGER bndr(3,*)	
      REAL R(*)
      
      INTEGER nxdrcg,nydrcg,nzdrcg
      INTEGER bndrcg(3)
      REAL RCG(nxdrcg,nydrcg,nzdrcg)
      
      INTEGER nxdpcg,nydpcg,nzdpcg
      INTEGER bndpcg(3)
      REAL PCG(nxdpcg,nydpcg,nzdpcg)
      
      INTEGER nxdwcg,nydwcg,nzdwcg
      INTEGER bndwcg(3)
      REAL WCG(nxdwcg,nydwcg,nzdwcg)
      
      INTEGER nxdrsm,nydrsm,nzdrsm
      INTEGER bndrsm(3)
      REAL RSM(nxdrsm,nydrsm,nzdrsm)
      
      INTEGER nxdpsm,nydpsm,nzdpsm
      INTEGER bndpsm(3)
      REAL PSM(nxdpsm,nydpsm,nzdpsm)
      
      INTEGER nxdwsm,nydwsm,nzdwsm
      INTEGER bndzsm(3)
      REAL WSM(nxdwsm,nydwsm,nzdwsm)
      INTEGER indh(*),inda(*),indx(*)
      INTEGER m0,m
      INTEGER nx(m),ny(m),nz(m)
      INTEGER nxg(m),nyg(m),nzg(m)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      INTEGER numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver
      REAL resTolFine,resTolCoarse,wgt
      INTEGER iterFine,iterCoarse,outputLevel
      INTEGER maxItFine,maxItCoarse
      
      INTEGER comm,nbr(6),bnd(3,*),bnd2(3)
      REAL resRedFine
      REAL scaleA, scaleB,rnorm
      INTEGER istat

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

#ifdef DEBUG_MG
      REAL mmVectorSum,s
      EXTERNAL mmVectorSum
#endif

      REAL mmDotProd,mmVectorMax,mmVectorNormalize,tmp
      REAL resCoarse
      REAL anorm,xnorm,bnorm
      REAL rnormPrev,rnormFirst,relResCoarse
      INTEGER k,kc,iter,nu
      INTEGER ica,ifa,icx,ifx,ich,ifh,istatCoarse
      LOGICAL dumpVectors
      INTEGER dumpIter

      INTEGER mmIsNan
      LOGICAL mmIsConverged
      EXTERNAL mmIsNan,mmIsConverged

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_MG
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmMgVcycle'
#endif

      iter = 0
      istat = 0
      dumpVectors = .FALSE.
      dumpIter = 0

      iterCoarse = 0


C****************************
      IF (dumpVectors) THEN
         CALL mmWriteMatrix ('AC.out    ',AD,A1,A2,A3,A4,A5,A6,
     &        nxda(2),nyda(2),nzda(2),nx(2),ny(2),nz(2),
     &        comm,nbr,2)
      END IF
C****************************

C****************************
      IF (dumpVectors) THEN
         CALL mmWriteMatrix ('AF.out    ',AFD,AF1,AF2,AF3,AF4,AF5,AF6,
     &        nxdaf,nydaf,nzdaf,nx(1),ny(1),nz(1),
     &        gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
     &        comm,nbr)
      END IF
C****************************


*-----------------------------------------------------------------------
*     Initialize Dirichlet b.c.'s on all grids
*-----------------------------------------------------------------------

      DO k=m0,m
         ifa = inda(k)
         IF (k.EQ.1) THEN
            CALL mmDirichletBC (
     &           
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nx(1),ny(1),nz(1),
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &           scaleA,scaleB,1,.FALSE.)
         ELSE
            IF (k.EQ.m0) THEN
               CALL mmDirichletBC (
     &              
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,   nx(k),ny(k),nz(k),
     &              
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &              scaleA,scaleB,k,.FALSE.)
            ELSE
               CALL mmDirichletBC (
     &              
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,   nx(k),ny(k),nz(k),
     &              
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &              scaleA,scaleB,k,.TRUE.)
            END IF
         END IF
      END DO

      IF (m0.EQ.1) THEN
*-----------------------------------------------------------------------
*     Compute residual R = B - A*X
*-----------------------------------------------------------------------

#ifdef DEBUG_MG
         if (ip.EQ.0) PRINT*, 'mmMgVcycle Residual #1'
#ifdef DEBUG_PROCS
         WRITE(*,"(i2,' mmMgVcycle Residual #1')") ip
#endif
#endif

         CALL mmResidual (
     &        
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,0,1,isub)


C****************************
         IF (dumpVectors.AND.iter.EQ.dumpIter) THEN
            CALL mmWriteVector ('X1A.out   ',XF,
     &           nxdxf,nydxf,nzdxf,nx(1),ny(1),nz(1),
     &           gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
     &           comm,nbr)
            ifx = indx(1)
            CALL mmWriteVector ('R1A.out   ',R(ifx),
     &           nxdr,nydr,nzdr,nx,ny,nz,
     &           gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
     &           comm,nbr)
         END IF
C****************************

#ifdef DEBUG_MG
         s = SQRT(mmDotProd(
     &     R(1),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,
     &     R(1),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,nx,ny,nz,comm,1,isub))
         if (ip.EQ.0) PRINT*, '  sum(R) = ',s
#endif

*-----------------------------------------------------------------------
*     Compute norms for convergence test
*-----------------------------------------------------------------------

         ica = inda(2)

         xnorm = DSQRT(mmDotProd (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &        nx(1),ny(1),nz(1),comm,1,isub))
         anorm = mmVectorMax (AD(ica),nxda(2),nyda(2),nzda(2),
     &        bnda(1,2),nx(2),ny(2),nz(2),comm)
         bnorm = DSQRT(mmDotProd (
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx(1),ny(1),nz(1),comm,1,
     &        isub))
         rnorm = DSQRT(mmDotProd (
     &     R(1),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,
     &     R(1),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,
     &        nx(1),ny(1),nz(1),comm,1,isub))

#ifdef DEBUG_MG
         IF (ip.EQ.0) THEN
            print*, 'mmMgVcycle norms'
            print*, '  norm(a) = ',anorm
            print*, '  norm(b) = ',bnorm
            print*, '  norm(r) = ',rnorm
            print*, '  norm(x) = ',xnorm
         END IF
#endif

         rnormFirst = rnorm

         iter = 0

*-----------------------------------------------------------------------
*     Check for convergence
*-----------------------------------------------------------------------

 10      CONTINUE

         IF (mmIsConverged(istat,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     R(1),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,nx(1),ny(1),
     &        nz(1),anorm,bnorm,rnorm,rnormFirst,resTolFine,iter,
     &        maxItFine,
     &     comm,nbr,bnd
     &     ,1,isub)) GO TO 20

*-----------------------------------------------------------------------
*     Smooth and restrict residual to coarser grids
*-----------------------------------------------------------------------

         nu = numPreSmooth
         
         k = 1
         ifx = indx(k)
         ifa = inda(k)
         ifh = indh(k)
         kc = 2
         icx = indx(kc)
         ica = inda(kc)
         ich = indh(kc)

         IF (m.GT.k) THEN 

#ifdef DEBUG_MG
            if (ip.EQ.0) PRINT*, 'mmMgVcycle Smooth 1'
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,' mmMgVcycle Smooth 1')") ip
#endif
#endif

            CALL mmSmooth (
     &           
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &           nx,ny,nz,nxg,nyg,nzg,wgt,nu,iSmoothIt,
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &           scaleA,scaleB,1,isub)

            nu = nu + numPreSmooth

C****************************
            IF (dumpVectors.AND.iter.EQ.dumpIter) THEN
               CALL mmWriteVector ('X1AS.out  ',XF,
     &              nxdxf,nydxf,nzdxf,nx(1),ny(1),nz(1),
     &              gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
     &              comm,nbr)
            END IF
C****************************

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nx,ny,nz,comm,isub)
            if (ip.EQ.0) PRINT*, '  sum(X) = ',s
#endif

#ifdef DEBUG_MG
            if (ip.EQ.0) PRINT*, 'mmMgVcycle Residual 1'
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,' mmMgVcycle Residual #2')") ip
#endif
#endif

            CALL mmResidual (
     &           
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &           scaleA,scaleB,0,1,isub)

C****************************
            IF (dumpVectors.AND.iter.EQ.dumpIter) THEN
               CALL mmWriteVector ('R1AS.out  ',R(ifx),
     &              nxdr,nydr,nzdr,nx,ny,nz,
     &              gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
     &              comm,nbr)
            END IF
C****************************

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
            if (ip.EQ.0) PRINT*, '  sum(R) = ',s
#endif

#ifdef DEBUG_MG
            if (ip.EQ.0) WRITE(*,
     &           "('mmMgVcycle Restrict',i2,' -->',i2)") k,kc
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,' mmMgVcycle Restrict')") ip
#endif
#endif
            CALL mmRestrict (
     &           
     &     BC(ica),nxdbc(kc),nydbc(kc),nzdbc(kc),bndbc(1,kc)
     &     ,
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &           nx(kc),ny(kc),nz(kc),nx(k),ny(k),nz(k),
     &           hx(ifh),hy(ifh),hz(ifh),
     &           comm,nbr,bnd(1,kc),bnd(1,k),k,isub)

C****************************
            IF (dumpVectors.AND.iter.EQ.dumpIter) THEN
               CALL mmWriteVector ('R2.out    ',BC(ica),
     &              nxdbc(kc),nydbc(kc),nzdbc(kc),nx(kc),ny(kc),
     &              gmmix(kc,ipx),gmmiy(kc,ipy),gmmiz(kc,ipz),
     &              nz(kc),comm,nbr)
            END IF
C****************************

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     BC(ica),nxdbc(kc),nydbc(kc),nzdbc(kc),bndbc(1,kc)
     &     ,nx(kc),ny(kc),nz(kc),
     &           comm,isub)
            if (ip.EQ.0) PRINT*, '  sum(B) = ',s
#endif

         END IF

         DO k=2,m-1

            ifx = indx(k)
            ifa = inda(k)
            ifh = indh(k)
            kc = k+1
            icx = indx(kc)
            ica = inda(kc)
            ich = indh(kc)

#ifdef DEBUG_MG
            if (ip.EQ.0) PRINT*, 'mmMgVcycle Smooth',k
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,' mmMgVcycle Smooth')") ip
#endif
            s = mmVectorSum (
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,nx(k),ny(k),nz(k),comm,
     &           isub)
            if (ip.EQ.0) PRINT*, '  sum(Xin) = ',s
#endif
            CALL mmSmooth (
     &           
     &     AD(ifa),A1(ifa),A2(ifa),A3(ifa),
     &     A4(ifa),A5(ifa),A6(ifa),nxda(k),nyda(k),nzda(k),
     &     bnda(1,k)
     &     ,
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,
     &     BC(ifa),nxdbc(k),nydbc(k),nzdbc(k),bndbc(1,k)
     &     ,
     &           
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &           
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &           nx(k),ny(k),nz(k),nxg(k),nyg(k),nzg(k),wgt,nu,
     &           iSmoothIt,
     &           
     &     xp(ifh),yp(ifh),zp(ifh),
     &     hx(ifh),hy(ifh),hz(ifh)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd(1,k)
     &     ,
     &           scaleA,scaleB,k,isub)

            nu = nu + numPreSmooth

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,nx(k),ny(k),nz(k),comm,
     &           isub)
            if (ip.EQ.0) PRINT*, '  sum(Xout) = ',s
#endif

#ifdef DEBUG_MG
            if (ip.EQ.0) PRINT*, 'mmMgVcycle Residual ',k
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,' mmMgVcycle Residual #3')") ip
#endif
#endif
            CALL mmResidual (
     &           
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &     AD(ifa),A1(ifa),A2(ifa),A3(ifa),
     &     A4(ifa),A5(ifa),A6(ifa),nxda(k),nyda(k),nzda(k),
     &     bnda(1,k)
     &     ,
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,
     &           
     &     BC(ifa),nxdbc(k),nydbc(k),nzdbc(k),bndbc(1,k)
     &     ,
     &           nx(k),ny(k),nz(k),
     &           
     &     xp(ifh),yp(ifh),zp(ifh),
     &     hx(ifh),hy(ifh),hz(ifh)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd(1,k)
     &     ,
     &           scaleA,scaleB,0,k,isub)

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,nx(k),ny(k),nz(k),comm,
     &           isub)
            if (ip.EQ.0) PRINT*, '  sum(R) = ',s
#endif

#ifdef DEBUG_MG
            if (ip.EQ.0) WRITE(*,
     &           "('mmMgVcycle Restrict',i2,' -->',i2)") k,kc
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,' mmMgVcycle Restrict')") ip
#endif
#endif
            CALL mmRestrict (
     &           
     &     BC(ica),nxdbc(kc),nydbc(kc),nzdbc(kc),bndbc(1,kc)
     &     ,
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &           nx(kc),ny(kc),nz(kc),nx(k),ny(k),nz(k),
     &           hx(ifh),hy(ifh),hz(ifh),
     &           comm,nbr,bnd(1,kc),bnd(1,k),k,isub)

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     BC(ica),nxdbc(kc),nydbc(kc),nzdbc(kc),bndbc(1,kc)
     &     ,nx(kc),ny(kc),nz(kc),
     &           comm,isub)
            if (ip.EQ.0) PRINT*, '  sum(B) = ',s
#endif

         END DO

*-----------------------------------------------------------------------
*     Solve the coarse-grid problem
*-----------------------------------------------------------------------

         ica = inda(m)
         icx = indx(m)
         ich = indh(m)

#ifdef DEBUG_MG
         if (ip.EQ.0) PRINT*, 'mmMgVcycle coarse grid solve',m
#ifdef DEBUG_PROCS
         WRITE(*,"(i2,' mmMgVcycle coarse grid solve')") ip
#endif
#endif

         iStatCoarse=0
         IF (iCoarseSolver.EQ.MM_JACOBI_COARSE_SOLVER) THEN

            IF (m.NE.1) THEN

               CALL mmSmooth (
     &              
     &     AD(ica),A1(ica),A2(ica),A3(ica),
     &     A4(ica),A5(ica),A6(ica),nxda(m),nyda(m),nzda(m),
     &     bnda(1,m)
     &     ,
     &     XC(ica),nxdxc(m),nydxc(m),nzdxc(m),bndxc(1,m)
     &     ,
     &     BC(ica),nxdbc(m),nydbc(m),nzdbc(m),bndbc(1,m)
     &     ,
     &              
     &     R(icx),nxdr(m),nydr(m),nzdr(m),bndr(1,m)
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     , 
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &              nx(m),ny(m),nz(m),nxg(m),nyg(m),nzg(m),1.0D0,
     &              maxItCoarse,
     &              iSmoothIt,
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &              
     &     comm,nbr,bnd(1,m)
     &     ,scaleA,scaleB,m,isub)

            ELSE

               CALL mmSmooth (
     &              
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     R(icx),nxdr(m),nydr(m),nzdr(m),bndr(1,m)
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &              nx(m),ny(m),nz(m),nxg(m),nyg(m),nzg(m),1.0D0,
     &              maxItCoarse,
     &              iSmoothIt,
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &              
     &     comm,nbr,bnd(1,m)
     &     ,scaleA,scaleB,m,isub)

            END IF

            iterCoarse = iterCoarse + maxItCoarse
         END IF

         IF (iCoarseSolver.EQ.MM_CG_COARSE_SOLVER) THEN
            
            IF (gisymmetric.EQ.0) THEN

C     @@@ Replace with mmBiCgStab here, but need extra temp vectors first

               IF (m.NE.1) THEN
                  CALL mmCg (
     &                 
     &     AD(ica),A1(ica),A2(ica),A3(ica),
     &     A4(ica),A5(ica),A6(ica),nxda(m),nyda(m),nzda(m),
     &     bnda(1,m)
     &     ,
     &     XC(ica),nxdxc(m),nydxc(m),nzdxc(m),bndxc(1,m)
     &     ,
     &     BC(ica),nxdbc(m),nydbc(m),nzdbc(m),bndbc(1,m)
     &     ,
     &                 
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &                 nx(m),ny(m),nz(m),nxg(m),nyg(m),nzg(m),
     &                 
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,0,
     &                 iterCoarse,maxItCoarse,resTolCoarse,0,
     &                 
     &     comm,nbr,bnd(1,m)
     &     ,scaleA,scaleB,m,
     &                 resCoarse,relResCoarse,istatCoarse,isub)
               ELSE 
                  CALL mmCg (
     &                 
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &                 nx(m),ny(m),nz(m),nxg(m),nyg(m),nzg(m),
     &                 
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,0,
     &                 iterCoarse,maxItCoarse,resTolCoarse,0,
     &                 
     &     comm,nbr,bnd(1,m)
     &     ,scaleA,scaleB,m,
     &                 resCoarse,relResCoarse,istatCoarse,isub)

               END IF
            ELSE
               IF (m.NE.1) THEN

                 CALL mmCg (
     &                 
     &     AD(ica),A1(ica),A2(ica),A3(ica),
     &     A4(ica),A5(ica),A6(ica),nxda(m),nyda(m),nzda(m),
     &     bnda(1,m)
     &     ,
     &     XC(ica),nxdxc(m),nydxc(m),nzdxc(m),bndxc(1,m)
     &     ,
     &     BC(ica),nxdbc(m),nydbc(m),nzdbc(m),bndbc(1,m)
     &     ,
     &                 
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &                 nx(m),ny(m),nz(m),nxg(m),nyg(m),nzg(m),
     &                 
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,0,
     &                 iterCoarse,maxItCoarse,resTolCoarse,0,
     &                 
     &     comm,nbr,bnd(1,m)
     &     ,scaleA,scaleB,m,
     &                 resCoarse,relResCoarse,istatCoarse,isub)
               ELSE 
                  CALL mmCg (
     &                 
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &                 nx(m),ny(m),nz(m),nxg(m),nyg(m),nzg(m),
     &                 
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,0,
     &                 iterCoarse,maxItCoarse,resTolCoarse,0,
     &                 
     &     comm,nbr,bnd(1,m)
     &     ,scaleA,scaleB,m,
     &                 resCoarse,relResCoarse,istatCoarse,isub)

               END IF
            END IF

         END IF

         IF (iStatCoarse.NE.0 .AND.
     &        outputLevel.NE.0 .AND. ip.EQ.0) THEN
            PRINT*, '-- mgmpi warning -- Coarse solver returned ',
     &           iStatCoarse
         END IF
         
#ifdef DEBUG_MG
         s = mmVectorSum (
     &     XC(ica),nxdxc(m),nydxc(m),nzdxc(m),bndxc(1,m)
     &     ,nx(m),ny(m),nz(m),comm,isub)
         if (ip.EQ.0) PRINT*, '  sum(X) = ',s
#endif
         

C****************************
         IF (dumpVectors.AND.iter.EQ.dumpIter) THEN
            CALL mmWriteVector ('C2.out    ',XC(ica),
     &           nxdxc(m),nydxc(m),nzdxc(m),nx(m),ny(m),nz(m),
     &           gmmix(m,ipx),gmmiy(m,ipy),gmmiz(m,ipz),
     &           comm,nbr)
         END IF
C****************************

*-----------------------------------------------------------------------
*     Prolong correction and smooth to finer grids
*-----------------------------------------------------------------------

#define NORMALIZE .TRUE.
#define REFRESH .TRUE.

         nu = m*numPostSmooth
         DO k=m-1,2,-1

            kc = k+1
            ica = inda(kc)
            icx = indx(kc)
            ich = indh(kc)
            ifa = inda(k)
            ifx = indx(k)
            ifh = indh(k)

***********(2) ***************
#ifdef DEBUG_MG
            if (ip.EQ.0) PRINT*, 'mmMgVcycle normalize',kc
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,' mmMgVcycle normalize')") ip
#endif
#endif
            IF (NORMALIZE) THEN
               tmp = mmVectorNormalize (
     &              
     &     XC(ica),nxdxc(kc),nydxc(kc),nzdxc(kc),bndxc(1,kc)
     &     ,
     &              nx(kc),ny(kc),nz(kc),
     &              nxg(kc),nyg(kc),nzg(kc),
     &              
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &              comm,isub)

            END IF
            
***********(1) ***************

            IF (REFRESH) THEN
               CALL mmRefreshBC (
     &              
     &     XC(ica),nxdxc(kc),nydxc(kc),nzdxc(kc),bndxc(1,kc)
     &     ,nx(kc),ny(kc),nz(kc),
     &              
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &              
     &     comm,nbr,bnd(1,kc)
     &     ,scaleA,scaleB,kc,isub)
            END IF

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     XC(ica),nxdxc(kc),nydxc(kc),nzdxc(kc),bndxc(1,kc)
     &     ,nx(kc),ny(kc),nz(kc),
     &           comm,isub)
            if (ip.EQ.0) PRINT*, '  sum(X) = ',s
#endif

#ifdef DEBUG_MG
            if (ip.EQ.0) WRITE(*,
     &           "('mmMgVcycle Prolong',i2,' <--',i2)") k,kc
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,' mmMgVcycle Prolong')") ip
#endif
#endif
            CALL mmProlong (
     &           
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &     XC(ica),nxdxc(kc),nydxc(kc),nzdxc(kc),bndxc(1,kc)
     &     ,
     &     AD(ifa),A1(ifa),A2(ifa),A3(ifa),
     &     A4(ifa),A5(ifa),A6(ifa),nxda(k),nyda(k),nzda(k),
     &     bnda(1,k)
     &     ,
     &           nx(k),ny(k),nz(k),nx(kc),ny(kc),nz(kc),
     &           hx(ifh),hy(ifh),hz(ifh),
     &     comm,nbr,bnd(1,kc)
     &     ,k,.FALSE.,isub)

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,nx(k),ny(k),nz(k),comm,
     &           isub)
            if (ip.EQ.0) PRINT*, '  sum(R) = ',s
#endif

#ifdef DEBUG_MG
            if (ip.EQ.0) PRINT*, 'mmMgVcycle correct',k
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,' mmMgVcycle correct')") ip
#endif
#endif
            CALL mmCorrect (
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,nx(k),ny(k),
     &           nz(k),isub)

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,nx(k),ny(k),nz(k),comm,
     &           isub)
            if (ip.EQ.0) PRINT*, '  sum(X) = ',s
#endif
            nu = nu - numPostSmooth

#ifdef DEBUG_MG
            if (ip.EQ.0) PRINT*, 'mmMgVcycle Smooth',k
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,' mmMgVcycle Smooth')") ip
#endif
#endif
            CALL mmSmooth (
     &           
     &     AD(ifa),A1(ifa),A2(ifa),A3(ifa),
     &     A4(ifa),A5(ifa),A6(ifa),nxda(k),nyda(k),nzda(k),
     &     bnda(1,k)
     &     ,
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,
     &     BC(ifa),nxdbc(k),nydbc(k),nzdbc(k),bndbc(1,k)
     &     ,
     &           
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &           
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &           nx(k),ny(k),nz(k),nxg(k),nyg(k),nzg(k),wgt,nu,
     &           iSmoothIt,
     &           
     &     xp(ifh),yp(ifh),zp(ifh),
     &     hx(ifh),hy(ifh),hz(ifh)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd(1,k)
     &     ,
     &           scaleA,scaleB,k,isub)

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,nx(k),ny(k),nz(k),comm,
     &           isub)
            if (ip.EQ.0) PRINT*, '  sum(X) = ',s
#endif

         END DO

         IF (m.GT.1) THEN

            kc = 2
            ica = inda(2)
            icx = indx(2)
            ich = indh(2)
            k = 1
            ifa = inda(1)
            ifx = indx(1)
            ifh = indh(1)

***********(2) ***************
#ifdef DEBUG_MG
            if (ip.EQ.0) PRINT*, 'mmMgVcycle normalize',2
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,' mmMgVcycle normalize')") ip
#endif
#endif
            IF (NORMALIZE) THEN
               tmp = mmVectorNormalize (
     &              
     &     XC(ica),nxdxc(2),nydxc(2),nzdxc(2),bndxc(1,2)
     &     ,
     &              nx(2),ny(2),nz(2),
     &              nxg(2),nyg(2),nzg(2),
     &              
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &              comm,isub)
            END IF

            IF (REFRESH) THEN
               CALL mmRefreshBC (
     &              
     &     XC(ica),nxdxc(2),nydxc(2),nzdxc(2),bndxc(1,2)
     &     ,nx(2),ny(2),nz(2),
     &              
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &              
     &     comm,nbr,bnd(1,2)
     &     ,scaleA,scaleB,2,isub)
            END IF

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     XC(ica),nxdxc(2),nydxc(2),nzdxc(2),bndxc(1,2)
     &     ,nx(2),ny(2),nz(2),comm,
     &           isub)
            if (ip.EQ.0) PRINT*, '  sum(X) = ',s
#endif

#ifdef DEBUG_MG
            if (ip.EQ.0) WRITE(*,
     &           "('mmMgVcycle Prolong',i2,' <--',i2)") k,kc
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,' mmMgVcycle Prolong')") ip
#endif
#endif

            CALL mmProlong (
     &           
     &     R(ifx),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,
     &     XC(ica),nxdxc(2),nydxc(2),nzdxc(2),bndxc(1,2)
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &           nx(1),ny(1),nz(1),nx(2),ny(2),nz(2),
     &           hx,hy,hz,
     &     comm,nbr,bnd(1,2)
     &     ,1,.FALSE.,isub)

C****************************
            IF (dumpVectors.AND.iter.EQ.dumpIter) THEN
               CALL mmWriteVector ('C1.out    ',R(ifx),
     &              nxdr(1),nydr(1),nzdr(1),nx(1),ny(1),nz(1),
     &              gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
     &              comm,nbr)
            END IF
C****************************


#ifdef DEBUG_MG
            s = mmVectorSum (
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,nx(k),ny(k),nz(k),comm,
     &           isub)
            if (ip.EQ.0) PRINT*, '  sum(R) = ',s
#endif

#ifdef DEBUG_MG
            if (ip.EQ.0) PRINT*, 'mmMgVcycle correct',k
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,' mmMgVcycle correct')") ip
#endif
#endif

            CALL mmCorrect (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     R(ifx),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,nx(1),ny(1),nz(1),isub)

C****************************
            IF (dumpVectors.AND.iter.EQ.dumpIter) THEN
               CALL mmWriteVector ('X1B.out   ',XF,
     &              nxdxf,nydxf,nzdxf,nx(1),ny(1),nz(1),
     &              gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
     &              comm,nbr)
            END IF
C****************************

C     IF (gisymmetric.EQ.0) THEN
C     CALL mmWriteVector ('Xns.out     ',XF,
C     &           nxdxf,nydxf,nzdxf,nx,ny,nz,
C     &        gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
C     &           comm,nbr)
C     ELSE 
C     CALL mmWriteVector ('Xs.out      ',XF,
C     &           nxdxf,nydxf,nzdxf,nx,ny,nz,
C     &        gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
C     &           comm,nbr)
C     END IF

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,nx(k),ny(k),nz(k),comm,
     &           isub)
            if (ip.EQ.0) PRINT*, '  sum(X) = ',s
#endif

            nu = nu - numPostSmooth

#ifdef DEBUG_MG
            if (ip.EQ.0) PRINT*, 'mmMgVcycle Smooth 1'
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,' mmMgVcycle Smooth')") ip
#endif
#endif
            CALL mmSmooth (
     &           
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &           nx,ny,nz,nxg,nyg,nzg,wgt,nu,iSmoothIt,
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &           scaleA,scaleB,1,isub)

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nx,ny,nz,comm,isub)
            if (ip.EQ.0) PRINT*, '  sum(X) = ',s
#endif

         END IF

#ifdef DEBUG_MG
         if (ip.EQ.0) PRINT*, 'mmMgVcycle Smooth 1'
#ifdef DEBUG_PROCS
         WRITE(*,"(i2,' mmMgVcycle Smooth')") ip
#endif
#endif
         CALL mmResidual (
     &        
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,0,1,isub)

C****************************
         IF (dumpVectors.AND.iter.EQ.dumpIter) THEN
            CALL mmWriteVector ('X1BS.out  ',XF,
     &           nxdxf,nydxf,nzdxf,nx(1),ny(1),nz(1),
     &           gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
     &           comm,nbr)
            CALL mmWriteVector ('R1BS.out  ',R(ifx),
     &           nxdr,nydr,nzdr,nx,ny,nz,
     &           gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
     &           comm,nbr)
            CALL mmTerminate(comm)
         END IF
C****************************

#ifdef DEBUG_MG
         s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
         if (ip.EQ.0) PRINT*, 1,'mmMgVcycle sum(RF) = ',s
#endif

         iter = iter + 1
         iterFine = iterFine + 1

         rnormPrev = rnorm
         rnorm = DSQRT (mmDotProd 
     &        (
     &     R(1),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,
     &     R(1),nxdr(1),nydr(1),nzdr(1),bndr(1,1)
     &     ,nx(1),ny(1),nz(1),comm,1,
     &        isub))

         IF (outputLevel.EQ.2 .AND. ip.EQ.0)
     &        WRITE(*,1000) iter,MPI_WTIME() - gtimestart(SB_MG3),
     &        rnorm/rnormFirst

         GO TO 10

      ELSE

****************************************************************************
****************************************************************************
*     (m0 .NE. 1)
****************************************************************************
****************************************************************************

*-----------------------------------------------------------------------
*     Compute residual R = B - A*X
*-----------------------------------------------------------------------

#ifdef DEBUG_MG
         if (ip.EQ.0) PRINT*, 'm0: mmMgVcycle Residual ',m0
#ifdef DEBUG_PROCS
         WRITE(*,"(i2,'m0: mmMgVcycle Residual ',i2)") ip,m0
#endif
#endif

         k = m0

         ifx = indx(k)
         ifa = inda(k)
         ifh = indh(k)
         kc = k+1
         icx = indx(kc)
         ica = inda(kc)
         ich = indh(kc)


         CALL mmResidual (
     &        
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &     AD(ifa),A1(ifa),A2(ifa),A3(ifa),
     &     A4(ifa),A5(ifa),A6(ifa),nxda(k),nyda(k),nzda(k),
     &     bnda(1,k)
     &     ,
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,
     &        
     &     BC(ifa),nxdbc(k),nydbc(k),nzdbc(k),bndbc(1,k)
     &     ,
     &        nx(k),ny(k),nz(k),
     &        
     &     xp(ifh),yp(ifh),zp(ifh),
     &     hx(ifh),hy(ifh),hz(ifh)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd(1,k)
     &     ,
     &        scaleA,scaleB,0,k,isub)

*-----------------------------------------------------------------------
*     Compute norms for convergence test
*-----------------------------------------------------------------------

         xnorm = DSQRT(mmDotProd (
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,
     &        nx(k),ny(k),nz(k),comm,k,isub))
         anorm = mmVectorMax (AD(ifa),nxda(k),nyda(k),nzda(K),
     &        bnda(1,k),nx(k),ny(k),nz(k),comm)
         bnorm = DSQRT(mmDotProd (
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,nx(k),ny(k),nz(k),comm,k,
     &        isub))
         rnorm = DSQRT(mmDotProd (
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &        nx(k),ny(k),nz(k),comm,k,isub))

#ifdef DEBUG_MG
         IF (ip.EQ.0) THEN
            print*, 'm0: mmMgVcycle norms'
            print*, '  norm(a) = ',anorm
            print*, '  norm(b) = ',bnorm
            print*, '  norm(r) = ',rnorm
         END IF
#endif

         rnormFirst = rnorm

         iter = 0

*-----------------------------------------------------------------------
*     Check for convergence
*-----------------------------------------------------------------------

 15      CONTINUE

         IF (mmIsConverged(istat,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,
     &        
     &     BC(ifa),nxdbc(k),nydbc(k),nzdbc(k),bndbc(1,k)
     &     ,
     &     R(ifa),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,nx(k),ny(k),nz(k),
     &        anorm,bnorm,rnorm,rnormFirst,resTolFine,iter,
     &        maxItFine,
     &     comm,nbr,bnd
     &     ,k,isub)) GO TO 20

         nu = m0*numPreSmooth

         DO k=m0,m-1

            ifx = indx(k)
            ifa = inda(k)
            ifh = indh(k)
            kc = k+1
            icx = indx(kc)
            ica = inda(kc)
            ich = indh(kc)

#ifdef DEBUG_MG
            if (ip.EQ.0) PRINT*, 'm0: mmMgVcycle Smooth',k
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,'m0:  mmMgVcycle Smooth')") ip
#endif
            s = mmVectorSum (
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,nx(k),ny(k),nz(k),comm,
     &           isub)
            if (ip.EQ.0) PRINT*, 'm0:   sum(Xin) = ',s
#endif
            CALL mmSmooth (
     &           
     &     AD(ifa),A1(ifa),A2(ifa),A3(ifa),
     &     A4(ifa),A5(ifa),A6(ifa),nxda(k),nyda(k),nzda(k),
     &     bnda(1,k)
     &     ,
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,
     &     BC(ifa),nxdbc(k),nydbc(k),nzdbc(k),bndbc(1,k)
     &     ,
     &           
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &           
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &           nx(k),ny(k),nz(k),nxg(k),nyg(k),nzg(k),wgt,nu,
     &           iSmoothIt,
     &           
     &     xp(ifh),yp(ifh),zp(ifh),
     &     hx(ifh),hy(ifh),hz(ifh)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd(1,k)
     &     ,
     &           scaleA,scaleB,k,isub)

            nu = nu + numPreSmooth

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,nx(k),ny(k),nz(k),comm,
     &           isub)
            if (ip.EQ.0) PRINT*, 'm0:   sum(Xout) = ',s
#endif

#ifdef DEBUG_MG
            if (ip.EQ.0) PRINT*, 'm0: mmMgVcycle Residual 3:',k
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,'m0:  mmMgVcycle Residual 3')") ip
#endif
#endif
            CALL mmResidual (
     &           
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &     AD(ifa),A1(ifa),A2(ifa),A3(ifa),
     &     A4(ifa),A5(ifa),A6(ifa),nxda(k),nyda(k),nzda(k),
     &     bnda(1,k)
     &     ,
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,
     &           
     &     BC(ifa),nxdbc(k),nydbc(k),nzdbc(k),bndbc(1,k)
     &     ,
     &           nx(k),ny(k),nz(k),
     &           
     &     xp(ifh),yp(ifh),zp(ifh),
     &     hx(ifh),hy(ifh),hz(ifh)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd(1,k)
     &     ,
     &           scaleA,scaleB,0,k,isub)

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,nx(k),ny(k),nz(k),comm,
     &           isub)
            if (ip.EQ.0) PRINT*, 'm0:   sum(R) = ',s
#endif

#ifdef DEBUG_MG
            if (ip.EQ.0) WRITE(*,
     &           "('m0: mmMgVcycle Restrict',i2,' -->',i2)") k,kc
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,'m0:  mmMgVcycle Restrict')") ip
#endif
#endif
            CALL mmRestrict (
     &           
     &     BC(ica),nxdbc(kc),nydbc(kc),nzdbc(kc),bndbc(1,kc)
     &     ,
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &           nx(kc),ny(kc),nz(kc),nx(k),ny(k),nz(k),
     &           hx(ifh),hy(ifh),hz(ifh),
     &           comm,nbr,bnd(1,kc),bnd(1,k),k,isub)

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     BC(ica),nxdbc(kc),nydbc(kc),nzdbc(kc),bndbc(1,kc)
     &     ,nx(kc),ny(kc),nz(kc),
     &           comm,isub)
            if (ip.EQ.0) PRINT*, 'm0:   sum(B) = ',s
#endif

         END DO

*-----------------------------------------------------------------------
*     Solve the coarse-grid problem
*-----------------------------------------------------------------------

         ica = inda(m)
         icx = indx(m)
         ich = indh(m)

#ifdef DEBUG_MG
         if (ip.EQ.0) PRINT*, 'm0: mmMgVcycle coarse grid solve',m
#ifdef DEBUG_PROCS
         WRITE(*,"(i2,'m0:  mmMgVcycle coarse grid solve')") ip
#endif
#endif

         iStatCoarse=0
         IF (iCoarseSolver.EQ.MM_JACOBI_COARSE_SOLVER) THEN

            CALL mmSmooth (
     &           
     &     AD(ica),A1(ica),A2(ica),A3(ica),
     &     A4(ica),A5(ica),A6(ica),nxda(m),nyda(m),nzda(m),
     &     bnda(1,m)
     &     ,
     &     XC(ica),nxdxc(m),nydxc(m),nzdxc(m),bndxc(1,m)
     &     ,
     &     BC(ica),nxdbc(m),nydbc(m),nzdbc(m),bndbc(1,m)
     &     ,
     &           
     &     R(icx),nxdr(m),nydr(m),nzdr(m),bndr(1,m)
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     , 
     &           
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &           nx(m),ny(m),nz(m),nxg(m),nyg(m),nzg(m),wgt,nu,
     &           iSmoothIt,
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &           
     &     comm,nbr,bnd(1,m)
     &     ,scaleA,scaleB,m,isub)


            iterCoarse = iterCoarse + maxItCoarse
         ELSE IF (iCoarseSolver.EQ.MM_CG_COARSE_SOLVER) THEN
            
            IF (gisymmetric.EQ.0) THEN

C     @@@ Replace with mmBiCgStab here, but need extra temp vectors first

               CALL mmCg (
     &              
     &     AD(ica),A1(ica),A2(ica),A3(ica),
     &     A4(ica),A5(ica),A6(ica),nxda(m),nyda(m),nzda(m),
     &     bnda(1,m)
     &     ,
     &     XC(ica),nxdxc(m),nydxc(m),nzdxc(m),bndxc(1,m)
     &     ,
     &     BC(ica),nxdbc(m),nydbc(m),nzdbc(m),bndbc(1,m)
     &     ,
     &              
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &              nx(m),ny(m),nz(m),nxg(m),nyg(m),nzg(m),
     &              
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,0,
     &              iterCoarse,maxItCoarse,resTolCoarse,0,
     &              
     &     comm,nbr,bnd(1,m)
     &     ,scaleA,scaleB,m,
     &              resCoarse,relResCoarse,istatCoarse,isub)
            ELSE

               CALL mmCg (
     &              
     &     AD(ica),A1(ica),A2(ica),A3(ica),
     &     A4(ica),A5(ica),A6(ica),nxda(m),nyda(m),nzda(m),
     &     bnda(1,m)
     &     ,
     &     XC(ica),nxdxc(m),nydxc(m),nzdxc(m),bndxc(1,m)
     &     ,
     &     BC(ica),nxdbc(m),nydbc(m),nzdbc(m),bndbc(1,m)
     &     ,
     &              
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &              nx(m),ny(m),nz(m),nxg(m),nyg(m),nzg(m),
     &              
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,0,
     &              iterCoarse,maxItCoarse,resTolCoarse,0,
     &              
     &     comm,nbr,bnd(1,m)
     &     ,scaleA,scaleB,m,
     &              resCoarse,relResCoarse,istatCoarse,isub)
            END IF

         END IF

         IF (iStatCoarse.NE.0 .AND.
     &        outputLevel.NE.0 .AND. ip.EQ.0) THEN
            PRINT*, 'm0: -- mgmpi warning -- Coarse solver returned ',
     &           iStatCoarse
         END IF
         
#ifdef DEBUG_MG
         s = mmVectorSum (
     &     XC(ica),nxdxc(m),nydxc(m),nzdxc(m),bndxc(1,m)
     &     ,nx(m),ny(m),nz(m),comm,isub)
         if (ip.EQ.0) PRINT*, 'm0:   sum(X) = ',s
#endif
         

C****************************
         IF (dumpVectors.AND.iter.EQ.dumpIter) THEN
            CALL mmWriteVector ('C2.out    ',XC(ica),
     &           nxdxc(m),nydxc(m),nzdxc(m),nx(m),ny(m),nz(m),
     &           gmmix(m,ipx),gmmiy(m,ipy),gmmiz(m,ipz),
     &           comm,nbr)
         END IF
C****************************

*-----------------------------------------------------------------------
*     Prolong correction and smooth to finer grids
*-----------------------------------------------------------------------

#define NORMALIZE .TRUE.
#define REFRESH .TRUE.

         nu = (m-m0+1)*numPostSmooth

         DO k=m-1,m0,-1

            kc = k+1
            ica = inda(kc)
            icx = indx(kc)
            ich = indh(kc)
            ifa = inda(k)
            ifx = indx(k)
            ifh = indh(k)

***********(2) ***************
#ifdef DEBUG_MG
            if (ip.EQ.0) PRINT*, 'm0: mmMgVcycle normalize',kc
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,'m0:  mmMgVcycle normalize')") ip
#endif
#endif
            IF (NORMALIZE) THEN
               tmp = mmVectorNormalize (
     &              
     &     XC(ica),nxdxc(kc),nydxc(kc),nzdxc(kc),bndxc(1,kc)
     &     ,
     &              nx(kc),ny(kc),nz(kc),
     &              nxg(kc),nyg(kc),nzg(kc),
     &              
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &              comm,isub)

            END IF
            
***********(1) ***************

            IF (REFRESH) THEN
               CALL mmRefreshBC (
     &              
     &     XC(ica),nxdxc(kc),nydxc(kc),nzdxc(kc),bndxc(1,kc)
     &     ,nx(kc),ny(kc),nz(kc),
     &              
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &              
     &     comm,nbr,bnd(1,kc)
     &     ,scaleA,scaleB,kc,isub)
            END IF

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     XC(ica),nxdxc(kc),nydxc(kc),nzdxc(kc),bndxc(1,kc)
     &     ,nx(kc),ny(kc),nz(kc),
     &           comm,isub)
            if (ip.EQ.0) PRINT*, 'm0:   sum(X) = ',s
#endif

#ifdef DEBUG_MG
            if (ip.EQ.0) WRITE(*,
     &           "('m0: mmMgVcycle Prolong',i2,' <--',i2)") k,kc
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,'m0:  mmMgVcycle Prolong')") ip
#endif
#endif
            CALL mmProlong (
     &           
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &     XC(ica),nxdxc(kc),nydxc(kc),nzdxc(kc),bndxc(1,kc)
     &     ,
     &     AD(ifa),A1(ifa),A2(ifa),A3(ifa),
     &     A4(ifa),A5(ifa),A6(ifa),nxda(k),nyda(k),nzda(k),
     &     bnda(1,k)
     &     ,
     &           nx(k),ny(k),nz(k),nx(kc),ny(kc),nz(kc),
     &           hx(ifh),hy(ifh),hz(ifh),
     &     comm,nbr,bnd(1,kc)
     &     ,k,.FALSE.,isub)

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,nx(k),ny(k),nz(k),comm,
     &           isub)
            if (ip.EQ.0) PRINT*, 'm0:   sum(R) = ',s
#endif

#ifdef DEBUG_MG
            if (ip.EQ.0) PRINT*, 'm0: mmMgVcycle correct',k
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,'m0:  mmMgVcycle correct')") ip
#endif
#endif
            CALL mmCorrect (
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,nx(k),ny(k),
     &           nz(k),isub)

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,nx(k),ny(k),nz(k),comm,
     &           isub)
            if (ip.EQ.0) PRINT*, 'm0:   sum(X) = ',s
#endif
            nu = nu - numPostSmooth

#ifdef DEBUG_MG
            if (ip.EQ.0) PRINT*, 'm0: mmMgVcycle Smooth',k
#ifdef DEBUG_PROCS
            WRITE(*,"(i2,'m0:  mmMgVcycle Smooth')") ip
#endif
#endif
            CALL mmSmooth (
     &           
     &     AD(ifa),A1(ifa),A2(ifa),A3(ifa),
     &     A4(ifa),A5(ifa),A6(ifa),nxda(k),nyda(k),nzda(k),
     &     bnda(1,k)
     &     ,
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,
     &     BC(ifa),nxdbc(k),nydbc(k),nzdbc(k),bndbc(1,k)
     &     ,
     &           
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &           
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &           nx(k),ny(k),nz(k),nxg(k),nyg(k),nzg(k),wgt,nu,
     &           iSmoothIt,
     &           
     &     xp(ifh),yp(ifh),zp(ifh),
     &     hx(ifh),hy(ifh),hz(ifh)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd(1,k)
     &     ,
     &           scaleA,scaleB,k,isub)

#ifdef DEBUG_MG
            s = mmVectorSum (
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,nx(k),ny(k),nz(k),comm,
     &           isub)
            if (ip.EQ.0) PRINT*, 'm0:   sum(X) = ',s
#endif

         END DO

         k = m0

         ifx = indx(k)
         ifa = inda(k)
         ifh = indh(k)
         kc = k+1
         icx = indx(kc)
         ica = inda(kc)
         ich = indh(kc)

         iter = iter + 1
         iterFine = iterFine + 1

#ifdef DEBUG_MG
         if (ip.EQ.0) PRINT*, 'm0: mmMgVcycle Smooth 1'
#ifdef DEBUG_PROCS
         WRITE(*,"(i2,'m0:  mmMgVcycle Smooth')") ip
#endif
#endif


         CALL mmResidual (
     &        
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &     AD(ifa),A1(ifa),A2(ifa),A3(ifa),
     &     A4(ifa),A5(ifa),A6(ifa),nxda(k),nyda(k),nzda(k),
     &     bnda(1,k)
     &     ,
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,
     &        
     &     BC(ifa),nxdbc(k),nydbc(k),nzdbc(k),bndbc(1,k)
     &     ,
     &        nx(k),ny(k),nz(k),
     &        
     &     xp(ifh),yp(ifh),zp(ifh),
     &     hx(ifh),hy(ifh),hz(ifh)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd(1,k)
     &     ,
     &        scaleA,scaleB,0,k,isub)


#ifdef DEBUG_MG
         s = mmVectorSum (
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,nx(k),ny(k),nz(k),comm,isub)
         if (ip.EQ.0) PRINT*, 1,'m0: mmMgVcycle sum(R) = ',s
#endif

         rnormPrev = rnorm
         rnorm = DSQRT (mmDotProd 
     &        (
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,
     &     R(ifx),nxdr(k),nydr(k),nzdr(k),bndr(1,k)
     &     ,nx(k),ny(k),nz(k),comm,k,
     &        isub))

         IF (outputLevel.EQ.2 .AND. ip.EQ.0)
     &        WRITE(*,1000) iter,MPI_WTIME() - gtimestart(SB_MG3),
     &        rnorm/rnormFirst

         GO TO 15

      END IF

*-----------------------------------------------------------------------
*     Exit subroutine, printing warning message if applicable
*-----------------------------------------------------------------------

 20   CONTINUE

      IF (ip.EQ.0.AND.outputLevel.GT.0) THEN
         IF (istat.EQ.1) print*,
     &        '== mgmpi ERROR == aborting MG: apparently diverging'
         IF (istat.EQ.2) print*,
     &        '== mgmpi ERROR == aborting MG: residual norm is NaN'
         IF (istat.EQ.3) print*,
     &        '-- mgmpi warning -- exiting MG: iteration limit reached'
      END IF

      IF (ip.EQ.0 .AND. outputLevel.EQ.1)
     &     print*, 'MG: ',iter,rnorm/rnormFirst
      IF (ip.EQ.0 .AND. outputLevel.GT.0)
     &     print*, 'R.R.F.: ',(rnorm/rnormFirst)**(1.0D0/iter)

      resRedFine = rnorm / rnormFirst

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmMgVcycle'
#endif

      RETURN

 1000 FORMAT (1x,i4,f8.2,e14.3)

      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmFmg
C     
C     Purpose: Full-Multigrid solver
C
C     Algorithm: 
C
C     Inputs: 
C
C     Outputs: 
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate: 
C
C     Communication estimate: 
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmFmg (
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     BC,nxdbc,nydbc,nzdbc,bndbc
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &     indh,inda,indx
     &     ,
     &     m,nx,ny,nz,
     &     nxg,nyg,nzg,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver,
     &     wgt,iterFine,iterCoarse,
     &     maxItFine,maxItCoarse,
     &     resTolFine,resTolCoarse,outputLevel,
     &     comm,nbr,bnd,bnd2
     &     ,
     &     scaleA,scaleB,rnorm,resRedFine,istat,icall)

      IMPLICIT NONE

      
      INTEGER nxdaf,nydaf,nzdaf
      INTEGER bndaf(3)  
      REAL AFD(nxdaf,nydaf,nzdaf)
      REAL AF1(nxdaf,nydaf,nzdaf)
      REAL AF2(nxdaf,nydaf,nzdaf)
      REAL AF3(nxdaf,nydaf,nzdaf)
      REAL AF4(nxdaf,nydaf,nzdaf)
      REAL AF5(nxdaf,nydaf,nzdaf)
      REAL AF6(nxdaf,nydaf,nzdaf)
      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      
      INTEGER nxdbf,nydbf,nzdbf
      INTEGER bndbf(3)	
      REAL BF(nxdbf,nydbf,nzdbf)
      
      INTEGER nxda(*),nyda(*),nzda(*)
      INTEGER bnda(3,*)  
      REAL AD(*)	
      REAL A1(*)
      REAL A2(*)
      REAL A3(*)
      REAL A4(*)
      REAL A5(*)
      REAL A6(*)
      
      INTEGER nxdxc(*),nydxc(*),nzdxc(*)
      INTEGER bndxc(3,*)  
      REAL XC(*)
      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxdbc(*),nydbc(*),nzdbc(*)
      INTEGER bndbc(3,*)
      REAL BC(*)
      
      INTEGER nxdr(*),nydr(*),nzdr(*)
      INTEGER bndr(3,*)	
      REAL R(*)
      
      INTEGER nxdrcg,nydrcg,nzdrcg
      INTEGER bndrcg(3)
      REAL RCG(nxdrcg,nydrcg,nzdrcg)
      
      INTEGER nxdpcg,nydpcg,nzdpcg
      INTEGER bndpcg(3)
      REAL PCG(nxdpcg,nydpcg,nzdpcg)
      
      INTEGER nxdwcg,nydwcg,nzdwcg
      INTEGER bndwcg(3)
      REAL WCG(nxdwcg,nydwcg,nzdwcg)
      
      INTEGER nxdrsm,nydrsm,nzdrsm
      INTEGER bndrsm(3)
      REAL RSM(nxdrsm,nydrsm,nzdrsm)
      
      INTEGER nxdpsm,nydpsm,nzdpsm
      INTEGER bndpsm(3)
      REAL PSM(nxdpsm,nydpsm,nzdpsm)
      
      INTEGER nxdwsm,nydwsm,nzdwsm
      INTEGER bndzsm(3)
      REAL WSM(nxdwsm,nydwsm,nzdwsm)
      INTEGER indh(*),inda(*),indx(*)
      INTEGER m
      INTEGER nx(m),ny(m),nz(m)
      INTEGER nxg(m),nyg(m),nzg(m)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      INTEGER numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver
      REAL resTolFine,resTolCoarse,wgt
      INTEGER iterFine,iterCoarse,outputLevel
      INTEGER maxItFine,maxItCoarse
      
      INTEGER comm,nbr(6),bnd(3,*),bnd2(3)
      REAL resRedFine
      REAL scaleA, scaleB,rnorm
      

      INTEGER istat
      INTEGER mmIsNan
      EXTERNAL mmIsNan      

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      LOGICAL dumpVectors
      REAL resCoarse
      REAL relResCoarse
      INTEGER k,kc
      INTEGER ica,ifa,icx,ifx,ich,ifh,istatCoarse
      REAL mmVectorNormalize,tmp

#ifdef DEBUG_MG
      REAL s
#endif

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_FMG
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      dumpVectors = .FALSE.

      k=1

      ifx = indx(k)
      ifa = inda(k)
      ifh = indh(k)
      kc = k+1
      icx = indx(kc)
      ica = inda(kc)
      ich = indh(kc)

      CALL mmRestrict (
     &     BC(ica),nxdbc(kc),nydbc(kc),nzdbc(kc),bndbc(1,kc)
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     nx(kc),ny(kc),nz(kc),nx(k),ny(k),nz(k),
     &     hx(ifh),hy(ifh),hz(ifh),
     &     comm,nbr,bnd(1,kc),bnd(1,k),k,isub)

      DO k=2,m-1
         ifx = indx(k)
         ifa = inda(k)
         ifh = indh(k)
         kc = k+1
         icx = indx(kc)
         ica = inda(kc)
         ich = indh(kc)

         CALL mmRestrict (
     &        
     &     BC(ica),nxdbc(kc),nydbc(kc),nzdbc(kc),bndbc(1,kc)
     &     ,
     &     BC(ifa),nxdbc(k),nydbc(k),nzdbc(k),bndbc(1,k)
     &     ,
     &        nx(kc),ny(kc),nz(kc),nx(k),ny(k),nz(k),
     &        hx(ifh),hy(ifh),hz(ifh),
     &        comm,nbr,bnd(1,kc),bnd(1,k),k,isub)

      END DO

*-----------------------------------------------------------------------
*     Solve the coarse-grid problem
*-----------------------------------------------------------------------

      ica = inda(m)
      icx = indx(m)
      ich = indh(m)

      CALL mmVectorClear (
     &     XC(ica),nxdxc(m),nydxc(m),nzdxc(m),bndxc(1,m)
     &     ,nx(m),ny(m),nz(m))

      iStatCoarse=0

      IF (iCoarseSolver.EQ.MM_CG_COARSE_SOLVER) THEN
         
         IF (gisymmetric.EQ.0) THEN

C     @@@ Replace with mmBiCgStab here, but need extra temp vectors first

            IF (m.NE.1) THEN
               CALL mmCg (
     &              
     &     AD(ica),A1(ica),A2(ica),A3(ica),
     &     A4(ica),A5(ica),A6(ica),nxda(m),nyda(m),nzda(m),
     &     bnda(1,m)
     &     ,
     &     XC(ica),nxdxc(m),nydxc(m),nzdxc(m),bndxc(1,m)
     &     ,
     &     BC(ica),nxdbc(m),nydbc(m),nzdbc(m),bndbc(1,m)
     &     ,
     &              
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &              nx(m),ny(m),nz(m),nxg(m),nyg(m),nzg(m),
     &              
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,1,
     &              iterCoarse,maxItCoarse,resTolCoarse,0,
     &              
     &     comm,nbr,bnd(1,m)
     &     ,scaleA,scaleB,m,
     &              resCoarse,relResCoarse,istatCoarse,isub)
            ELSE 
               CALL mmCg (
     &              
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &              nx(m),ny(m),nz(m),nxg(m),nyg(m),nzg(m),
     &              
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,1,
     &              iterCoarse,maxItCoarse,resTolCoarse,0,
     &              
     &     comm,nbr,bnd(1,m)
     &     ,scaleA,scaleB,m,
     &              resCoarse,relResCoarse,istatCoarse,isub)

            END IF
         ELSE
            IF (m.NE.1) THEN

               CALL mmCg (
     &              
     &     AD(ica),A1(ica),A2(ica),A3(ica),
     &     A4(ica),A5(ica),A6(ica),nxda(m),nyda(m),nzda(m),
     &     bnda(1,m)
     &     ,
     &     XC(ica),nxdxc(m),nydxc(m),nzdxc(m),bndxc(1,m)
     &     ,
     &     BC(ica),nxdbc(m),nydbc(m),nzdbc(m),bndbc(1,m)
     &     ,
     &              
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &              nx(m),ny(m),nz(m),nxg(m),nyg(m),nzg(m),
     &              
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,1,
     &              iterCoarse,maxItCoarse,resTolCoarse,0,
     &              
     &     comm,nbr,bnd(1,m)
     &     ,scaleA,scaleB,m,
     &              resCoarse,relResCoarse,istatCoarse,isub)
            ELSE 
               CALL mmCg (
     &              
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &              nx(m),ny(m),nz(m),nxg(m),nyg(m),nzg(m),
     &              
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,1,
     &              iterCoarse,maxItCoarse,resTolCoarse,0,
     &              
     &     comm,nbr,bnd(1,m)
     &     ,scaleA,scaleB,m,
     &              resCoarse,relResCoarse,istatCoarse,isub)

            END IF
         END IF
      END IF
      
      DO k=m-1,1,-1
         ifx = indx(k)
         ifa = inda(k)
         ifh = indh(k)
         kc = k+1
         icx = indx(kc)
         ica = inda(kc)
         ich = indh(kc)

         tmp = mmVectorNormalize (
     &        
     &     XC(ica),nxdxc(kc),nydxc(kc),nzdxc(kc),bndxc(1,kc)
     &     ,
     &        nx(kc),ny(kc),nz(kc),
     &        nxg(kc),nyg(kc),nzg(kc),
     &        
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        comm,isub)
         CALL mmRefreshBC (
     &        
     &     XC(ica),nxdxc(kc),nydxc(kc),nzdxc(kc),bndxc(1,kc)
     &     ,nx(kc),ny(kc),nz(kc),
     &        
     &     xp(ich),yp(ich),zp(ich),
     &     hx(ich),hy(ich),hz(ich)
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        
     &     comm,nbr,bnd(1,kc)
     &     ,scaleA,scaleB,kc,isub)
         IF (k.NE.1) THEN
            CALL mmProlong (
     &           
     &     XC(ifa),nxdxc(k),nydxc(k),nzdxc(k),bndxc(1,k)
     &     ,
     &     XC(ica),nxdxc(kc),nydxc(kc),nzdxc(kc),bndxc(1,kc)
     &     ,
     &     AD(ifa),A1(ifa),A2(ifa),A3(ifa),
     &     A4(ifa),A5(ifa),A6(ifa),nxda(k),nyda(k),nzda(k),
     &     bnda(1,k)
     &     ,
     &           nx(k),ny(k),nz(k),nx(kc),ny(kc),nz(kc),
     &           hx(ifh),hy(ifh),hz(ifh),
     &     comm,nbr,bnd(1,kc)
     &     ,k,.FALSE.,isub)
         ELSE
            CALL mmProlong (
     &           
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC(ica),nxdxc(kc),nydxc(kc),nzdxc(kc),bndxc(1,kc)
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &           nx(k),ny(k),nz(k),nx(kc),ny(kc),nz(kc),
     &           hx(ifh),hy(ifh),hz(ifh),
     &     comm,nbr,bnd(1,kc)
     &     ,k,.FALSE.,isub)
         END IF

         IF (dumpVectors.AND.k.EQ.1) THEN
            CALL mmWriteVector ('XFMG0.out ',XF,
     &           nxdxf,nydxf,nzdxf,nx(1),ny(1),nz(1),
     &           gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
     &           comm,nbr)
         END IF

         CALL mmMgVcycle (
     &        
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     BF,nxdbf,nydbf,nzdbf,bndbf
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     BC,nxdbc,nydbc,nzdbc,bndbc
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &        
     &     RCG,nxdrcg,nydrcg,nzdrcg,bndrcg
     &     ,
     &     PCG,nxdpcg,nydpcg,nzdpcg,bndpcg
     &     ,
     &     WCG,nxdwcg,nydwcg,nzdwcg,bndwcg
     &     ,
     &        
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &        
     &     indh,inda,indx
     &     ,
     &        k,m,nx,ny,nz,
     &        nxg,nyg,nzg,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &        numPreSmooth,numPostSmooth,iSmoothIt,iCoarseSolver,
     &        wgt,iterFine,iterCoarse,
     &        maxItFine,maxItCoarse,
     &        resTolFine,resTolCoarse,outputLevel,
     &        
     &     comm,nbr,bnd,bnd2
     &     ,
     &        scaleA,scaleB,rnorm,resRedFine,istat,icall)
         IF (dumpVectors.AND.k.EQ.1) THEN
            CALL mmWriteVector ('XFMG1.out ',XF,
     &           nxdxf,nydxf,nzdxf,nx(1),ny(1),nz(1),
     &           gmmix(1,ipx),gmmiy(1,ipy),gmmiz(1,ipz),
     &           comm,nbr)
            CALL mmTerminate(comm)
         END IF
      END DO
*-----------------------------------------------------------------------
*     Exit subroutine, printing warning message if applicable
*-----------------------------------------------------------------------

      IF (ip.EQ.0.AND.outputLevel.GT.0) THEN
         IF (istat.EQ.1) print*,
     &        '== mgmpi ERROR == aborting FMG: MG apparently diverging'
         IF (istat.EQ.2) print*,
     &        '== mgmpi ERROR == aborting FMG: MG residual norm is NaN'
         IF (istat.EQ.3) print*,
     &     '-- mgmpi warning -- exiting FMG: MG iteration limit reached'
      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN

 1000 FORMAT (1x,i4,f8.2,e14.3)

      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmOutput
C     
C     Purpose: Write parameters defining solver configuration
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmOutput (ifng,icoords,
     &     ixmb,ixpb,iymb,iypb,izmb,izpb,
     &     nxd2,nyd2,nzd2,nx,ny,nz,nxg,nyg,nzg,
     &     iGridTypeX,iGridTypeY,iGridTypeZ,
     &     maxItFine,maxItCoarse,resTolFine,resTolCoarse,m,
     &     wgt,numPreSmooth,numPostSmooth,iSmoothIt,
     &     iProblem,iFineSolver,iCoarseSolver,
     &     outputLevel,numItFine,resFine,resRedFine,istat,
     &     totalTime,nzl)
      IMPLICIT NONE

#include "mgmpi_internal.h"

      INTEGER ifng,icoords
      INTEGER ixmb,ixpb,iymb,iypb,izmb,izpb
      INTEGER nxd2,nyd2,nzd2,nx,ny,nz,nxg,nyg,nzg
      INTEGER iGridTypeX,iGridTypeY,iGridTypeZ
      INTEGER maxItFine,maxItCoarse,m
      REAL resTolFine,resTolCoarse,wgt
      INTEGER numPreSmooth,numPostSmooth,iSmoothIt
      INTEGER iProblem,iFineSolver,iCoarseSolver
      INTEGER outputLevel,numItFine
      REAL resFine,resRedFine,totalTime
      INTEGER istat,nzl

      IF (outputLevel.GT.1) THEN

*=========================================================

      write(*,"('PROBLEM SPECIFICATION')")

*---------------------------------------------------------

      IF (ifng.EQ.-1) THEN
          write(*,"(3x,'Forcing function is scaled by',i3)") ifng
      END IF

*---------------------------------------------------------
*     Coordinate system
*---------------------------------------------------------

      IF (icoords.EQ.MM_CARTESIAN) THEN
          write(*,"(3x,'Cartesian coordinates')")
      END IF
      IF (icoords.EQ.MM_CYLINDRICAL) THEN
          write(*,"(3x,'Cylindrical coordinates')")
      END IF
      IF (icoords.EQ.MM_SPHERICAL) THEN
          write(*,"(3x,'Spherical coordinates')")
      END IF

*---------------------------------------------------------
*     Boundary conditions
*---------------------------------------------------------

      IF (ixmb.EQ.MM_DIRICHLET) THEN
          write(*,"(3x,'Lower X1 face boundary conditions: Dirichlet')")
      END IF
      IF (ixmb.EQ.-MM_DIRICHLET) THEN
          write(*,"(3x,
     &        'Lower X1 face boundary conditions: Negated Dirichlet')")
      END IF
      IF (ixmb.EQ.MM_NEUMANN) THEN
          write(*,"(3x,'Lower X1 face boundary conditions: Neumann')")
      END IF
      IF (ixmb.EQ.-MM_NEUMANN) THEN
          write(*,"(3x,
     &        'Lower X1 face boundary conditions: Negated Neumann')")
      END IF
      IF (ixmb.EQ.MM_PERIODIC) THEN
          write(*,"(3x,'Lower X1 face boundary conditions: Periodic')")
      END IF

      IF (ixpb.EQ.MM_DIRICHLET) THEN
          write(*,"(3x,'Upper X1 face boundary conditions: Dirichlet')")
      END IF
      IF (ixpb.EQ.-MM_DIRICHLET) THEN
          write(*,"(3x,
     &        'Upper X1 face boundary conditions: Negated Dirichlet')")
      END IF
      IF (ixpb.EQ.MM_NEUMANN) THEN
          write(*,"(3x,'Upper X1 face boundary conditions: Neumann')")
      END IF
      IF (ixpb.EQ.-MM_NEUMANN) THEN
          write(*,"(3x,
     &        'Upper X1 face boundary conditions: Negated Neumann')")
      END IF
      IF (ixpb.EQ.MM_PERIODIC) THEN
          write(*,"(3x,'Upper X1 face boundary conditions: Periodic')")
      END IF

      IF (iymb.EQ.MM_DIRICHLET) THEN
          write(*,"(3x,'Lower X2 face boundary conditions: Dirichlet')")
      END IF
      IF (iymb.EQ.-MM_DIRICHLET) THEN
          write(*,"(3x,
     &        'Lower X2 face boundary conditions: Negated Dirichlet')")
      END IF
      IF (iymb.EQ.MM_NEUMANN) THEN
          write(*,"(3x,'Lower X2 face boundary conditions: Neumann')")
      END IF
      IF (iymb.EQ.-MM_NEUMANN) THEN
          write(*,"(3x,
     &        'Lower X2 face boundary conditions: Negated Neumann')")
      END IF
      IF (iymb.EQ.MM_PERIODIC) THEN
          write(*,"(3x,'Lower X2 face boundary conditions: Periodic')")
      END IF

      IF (iypb.EQ.MM_DIRICHLET) THEN
          write(*,"(3x,'Upper X2 face boundary conditions: Dirichlet')")
      END IF
      IF (iypb.EQ.-MM_DIRICHLET) THEN
          write(*,"(3x,
     &        'Upper X2 face boundary conditions: Negated Dirichlet')")
      END IF
      IF (iypb.EQ.MM_NEUMANN) THEN
          write(*,"(3x,'Upper X2 face boundary conditions: Neumann')")
      END IF
      IF (iypb.EQ.-MM_NEUMANN) THEN
          write(*,"(3x,
     &        'Upper X2 face boundary conditions: Negated Neumann')")
      END IF
      IF (iypb.EQ.MM_PERIODIC) THEN
          write(*,"(3x,'Upper X2 face boundary conditions: Periodic')")
      END IF

      IF (izmb.EQ.MM_DIRICHLET) THEN
          write(*,"(3x,'Lower X3 face boundary conditions: Dirichlet')")
      END IF
      IF (izmb.EQ.-MM_DIRICHLET) THEN
          write(*,"(3x,
     &        'Lower X3 face boundary conditions: Negated Dirichlet')")
      END IF
      IF (izmb.EQ.MM_NEUMANN) THEN
          write(*,"(3x,'Lower X3 face boundary conditions: Neumann')")
      END IF
      IF (izmb.EQ.-MM_NEUMANN) THEN
          write(*,"(3x,
     &        'Lower X3 face boundary conditions: Negated Neumann')")
      END IF
      IF (izmb.EQ.MM_PERIODIC) THEN
          write(*,"(3x,'Lower X3 face boundary conditions: Periodic')")
      END IF

      IF (izpb.EQ.MM_DIRICHLET) THEN
          write(*,"(3x,'Upper X3 face boundary conditions: Dirichlet')")
      END IF
      IF (izpb.EQ.-MM_DIRICHLET) THEN
          write(*,"(3x,
     &        'Upper X3 face boundary conditions: Negated Dirichlet')")
      END IF
      IF (izpb.EQ.MM_NEUMANN) THEN
          write(*,"(3x,'Upper X3 face boundary conditions: Neumann')")
      END IF
      IF (izpb.EQ.-MM_NEUMANN) THEN
          write(*,"(3x,
     &        'Upper X3 face boundary conditions: Negated Neumann')")
      END IF
      IF (izpb.EQ.MM_PERIODIC) THEN
          write(*,"(3x,'Upper X3 face boundary conditions: Periodic')")
      END IF

*---------------------------------------------------------
*     Array and grid sizes
*---------------------------------------------------------

      write(*,"(3x,'Input array dimensions:',i4,i4,i4)") nxd2,nyd2,nzd2
      write(*,"(3x,'Local grid size:',i4,i4,i4)") nx-2,ny-2,nz-2
      write(*,"(3x,'Global grid size:',i4,i4,i4)") nxg-2,nyg-2,nzg-2

*=========================================================

      write(*,"('DISCRETIZATION')")

*---------------------------------------------------------
*     Matrix storage scheme
*---------------------------------------------------------

      IF (iProblem.EQ.POISSON) THEN
         write(*,"(3x,'Implicit 7-point Poisson discretization')")
      ELSE IF (iProblem.EQ.SYMMETRIC) THEN
         write(*,"(3x,'Explicit 7-point symmetric discretization')")
      ELSE IF (iProblem.EQ.NONSYMMETRIC) THEN
         write(*,"(3x,'Explicit 7-point nonsymmetric discretization')")
      END IF

*---------------------------------------------------------
*     Grid spacing
*---------------------------------------------------------

      IF (iGridTypeX.EQ.1) THEN
          write(*,"(3x,'X1-axis grid point spacing: Uniform')")
      END IF
      IF (iGridTypeX.EQ.2) THEN
          write(*,"(3x,'X1-axis grid point spacing: Nonuniform')")
      END IF
      IF (iGridTypeY.EQ.1) THEN
          write(*,"(3x,'X2-axis grid point spacing: Uniform')")
      END IF
      IF (iGridTypeY.EQ.2) THEN
          write(*,"(3x,'X2-axis grid point spacing: Nonuniform')")
      END IF
      IF (iGridTypeZ.EQ.1) THEN
          write(*,"(3x,'X3-axis grid point spacing: Uniform')")
      END IF
      IF (iGridTypeZ.EQ.2) THEN
          write(*,"(3x,'X3-axis grid point spacing: Nonuniform')")
      END IF

*=========================================================

      write(*,"('CONCURRENCY')")

*---------------------------------------------------------
*     Processor topology
*---------------------------------------------------------

      write(*,"(3x,'Number of processors along X1-axis: ',i3)") npx
      write(*,"(3x,'Number of processors along X2-axis: ',i3)") npy
      write(*,"(3x,'Number of processors along X3-axis: ',i3)") npz

*---------------------------------------------------------
*     Interprocessor communication type
*---------------------------------------------------------

#ifdef MPI_BLOCKING
      write(*,"(3x,'Blocking communication used')")
#else
      write(*,"(3x,'Nonblocking communication used')")
#endif
*=========================================================

      write(*,"('SOLVER PARAMETERS')")

*---------------------------------------------------------
*     Solver
*---------------------------------------------------------

      IF (iFineSolver.EQ.MM_CG_SOLVER) THEN
         WRITE (*,"(3x,'CG solver')")
      ELSE IF (iFineSolver.EQ.MM_BICG_SOLVER) THEN
         WRITE (*,"(3x,'BiCG-STAB solver')")
      ELSE IF (iFineSolver.EQ.MM_BICGMG_SOLVER) THEN
         WRITE (*,"(3x,'V-cycle accerelated BiCG-STAB solver')")
      ELSE IF (iFineSolver.EQ.MM_MG_SOLVER) THEN
         WRITE (*,"(3x,'MG V-cycle solver')")
      ELSE IF (iFineSolver.EQ.MM_FMG_SOLVER) THEN
         WRITE (*,"(3x,'FMG solver')")
      ELSE IF (iFineSolver.EQ.MM_CGMG_SOLVER) THEN
         WRITE (*,"(3x,'V-cycle accelerated CG solver')")
      END IF

*---------------------------------------------------------
*     Solver parameters
*---------------------------------------------------------

      write(*,"(3x,'Iteration limit for fine-grid solver: ',i6)")
     &    maxItFine
      write(*,"(3x,'Iteration limit for coarse-grid solver: ',i6)")
     &    maxItCoarse

*---------------------------------------------------------
*     Solver parameters
*---------------------------------------------------------
      write(*,"(3x,'Tolerance on fine-grid solver residual: ',e8.3)")
     &    resTolFine
      write(*,"(3x,'Tolerance on coarse-grid solver residual: ',e8.3)")
     &    resTolCoarse

*---------------------------------------------------------
*     Solver parameters
*---------------------------------------------------------
      write(*,"(3x,'Number of multigrid levels: ',i3)") m

*---------------------------------------------------------
*     Solver parameters
*---------------------------------------------------------
      write(*,"(3x,'Relaxation parameter for the smoother: ',e8.3)")wgt

*---------------------------------------------------------
*     Solver parameters
*---------------------------------------------------------
      write(*,"(3x,'Number of pre-smoothings on fine-grid: ',i4)") 
     &    numPreSmooth
      write(*,"(3x,'Number of post-smoothings on fine-grid: ',i4)") 
     &    numPostSmooth

*---------------------------------------------------------
*     Solver parameters
*---------------------------------------------------------
      IF (iFineSolver.EQ.1) THEN
         write(*,"(3x,'Fine-grid solver: CG')")
      ELSE IF (iFineSolver.EQ.2) THEN
          write(*,"(3x,'Fine-grid solver: MG')")
      ELSE IF (iFineSolver.EQ.3) THEN
          write(*,"(3x,'Fine-grid solver: BiCg-STAB')")
      ELSE IF (iFineSolver.EQ.4) THEN
          write(*,"(3x,'Fine-grid solver: PBiCg-STAB')")
      ELSE IF (iFineSolver.EQ.5) THEN
          write(*,"(3x,'Fine-grid solver: FMG')")
      END IF

*---------------------------------------------------------
*     Solver parameters
*---------------------------------------------------------
      IF (iCoarseSolver.EQ.1) THEN
          write(*,"(3x,'Coarse-grid solver: CG')")
      END IF
      IF (iCoarseSolver.EQ.2) THEN
          write(*,"(3x,'Coarse-grid solver: smoother')")
      END IF

*---------------------------------------------------------
*     Solver parameters
*---------------------------------------------------------

      IF (gismoother.EQ.MM_POINT_JACOBI) THEN
          write(*,"(3x,'Smoother: point Jacobi')")
      END IF
      IF (gismoother.EQ.MM_LOCAL_LINE_JACOBI) THEN
          write(*,
     &      "(3x,'Smoother: processor-local alternating line Jacobi')")
      END IF
      IF (gismoother.EQ.MM_PLANE_JACOBI) THEN
          write(*,"(3x,'Smoother: alternating plane Jacobi')")
          write(*,"(3x,'Smoother: cg2 iterations: ',i4)") iSmoothIt
      END IF
      IF (gismoother.EQ.MM_LOCAL_POINT_RBGS) THEN
          write(*,"(3x,'Smoother: local point R/B Gauss-Seidel')")
      END IF

*---------------------------------------------------------
*     Solver parameters
*---------------------------------------------------------

      IF (gmmlc1.AND.gmmlc2.AND.gmmlc3) THEN
         write(*,"(3x,'Coarsening: Full-coarsening')")
      ELSE
         IF (gmmlc1) 
     &        write(*,"(3x,'Coarsening: Semi-coarsening: X1-axis')")
         IF (gmmlc2) 
     &        write(*,"(3x,'Coarsening: Semi-coarsening: X2-axis')")
         IF (gmmlc3) 
     &        write(*,"(3x,'Coarsening: Semi-coarsening: X3-axis')")
      END IF

*=========================================================
      write(*,"('OUTPUT')")
      write(*,"(3x,'Number of fine-grid iterations: ',i5)")
     &    numItFine  
      write(*,"(3x,'Final residual 2-norm: ',e15.8)")
     &    resFine
      write(*,"(3x,'Residual 2-norm reduction: ',e15.8)")
     &    resRedFine
      IF (numItFine.NE.0) THEN
         write(*,"(3x,'2-norm reduction rate: ',f12.5)")
     &        resRedFine**(1.0D0/numItFine)
      END IF
      write(*,"(3x,'Solver output status: ',i3)") istat
      write(*,"(3x,'Total time: ',f15.2)") totalTime
*=========================================================
      write(*,"('STORAGE')")
      write(*,"(3x,i15)") gistorage
*=========================================================
      END IF

      IF (outputLevel.GT.0) THEN
      write(*,"('SUMMARY')")

      WRITE(*,"(3x,'Summary: iterations         ',i15)") numItFine  
      WRITE(*,"(3x,'Summary: unknowns           ',i15)") 
     &     (nxg-1)*(nyg-1)*(nzg-1)
      WRITE(*,"(3x,'Summary: processors         ',i15)") npx*npy*npz
      WRITE(*,"(3x,'Summary: residual reduction ',f15.8)") 
     &     resRedFine**(1.0D0/numItFine)
      WRITE(*,"(3x,'Summary: iteration time     ',f15.8)") 
     &     totalTime/numItFine
      WRITE(*,"(3x,'Summary: mflop rate         ',f15.8)") 
     &     (gmflops(1)*1.0D-6)/gtime(1)
      WRITE(*,"(3x,'Summary: storage            ',i15)") gistorage
      WRITE(*,"(3x,'Summary: total time         ',f15.8)") totalTime
      
      OPEN (UNIT=9, FILE = 'summary')
      WRITE(9,"(3x,'Summary: iterations         ',i15)") numItFine  
      WRITE(9,"(3x,'Summary: unknowns           ',i15)") nxg*nyg*nzg
      WRITE(9,"(3x,'Summary: processors         ',i15)") npx*npy*npz
      WRITE(9,"(3x,'Summary: residual reduction ',f15.8)") 
     &     resRedFine**(1.0D0/numItFine)
      WRITE(9,"(3x,'Summary: iteration time     ',f15.8)") 
     &     totalTime/numItFine
      WRITE(9,"(3x,'Summary: mflop rate         ',f15.8)") 
     &     (gmflops(1)*1.0D-6)/gtime(1)
      WRITE(9,"(3x,'Summary: storage            ',i15)") gistorage
      WRITE(9,"(3x,'Summary: total time         ',f15.8)") totalTime
      CLOSE (9)
      END IF


C      problem size
C      number of processors
C      residual reduction per iteration
C      time per iteration
C      flops per iteration
C      storage
C      discretization error (if known)

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmProlong
C     
C     Purpose: Transfer a grid function to the next-finer grid level
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmProlong (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,nxf,nyf,nzf,nxc,nyc,nzc,
     &     hxf,hyf,hzf,comm,nbr,bndc,level,lUpdate,icall)

      IMPLICIT NONE

      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      
      INTEGER nxdxc,nydxc,nzdxc
      INTEGER bndxc(3)  
      REAL XC(nxdxc,nydxc,nzdxc)
      
      INTEGER nxdaf,nydaf,nzdaf
      INTEGER bndaf(3)  
      REAL AFD(nxdaf,nydaf,nzdaf)
      REAL AF1(nxdaf,nydaf,nzdaf)
      REAL AF2(nxdaf,nydaf,nzdaf)
      REAL AF3(nxdaf,nydaf,nzdaf)
      REAL AF4(nxdaf,nydaf,nzdaf)
      REAL AF5(nxdaf,nydaf,nzdaf)
      REAL AF6(nxdaf,nydaf,nzdaf)
      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      REAL hxf(nxf),hyf(nyf),hzf(nzf)
      INTEGER comm,nbr(6),bndc(3),level
      LOGICAL lUpdate

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      INTEGER idim

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_PROLONG
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> entering mmProlong'
#endif

      idim = 0

      IF (gmmlc1) idim = idim + 1
      IF (gmmlc2) idim = idim + 1
      IF (gmmlc3) idim = idim + 1

      IF (idim .EQ. 3) THEN

         IF (gimatfree.EQ.1) THEN

#ifdef DEBUG_PROLONG
            print*, 'Calling mmProlongLin'
#endif
            CALL mmProlongLin (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &           nxf,nyf,nzf,nxc,nyc,nzc,
     &           hxf,hyf,hzf,comm,nbr,bndc,level,lUpdate,isub)

         ELSE 

            IF (gicoords.EQ.MM_CARTESIAN) THEN

               IF (gisymmetric.EQ.1) THEN
#ifdef DEBUG_PROLONG
            print*, 'Calling mmProlongU (AF_SYMM)'
#endif
                  CALL mmProlongU (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     AFD,AF1,AF2,AF3,AF1,AF2,AF3,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &                 nxf,nyf,nzf,nxc,nyc,nzc,
     &                 hxf,hyf,hzf,comm,nbr,bndc,level,lUpdate,isub)
               ELSE
#ifdef DEBUG_PROLONG
            print*, 'Calling mmProlongU (AF)'
#endif
                  CALL mmProlongU (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &                 nxf,nyf,nzf,nxc,nyc,nzc,
     &                 hxf,hyf,hzf,comm,nbr,bndc,level,lUpdate,isub)
               END IF

            ELSE 

               IF (gisymmetric.EQ.1) THEN
#ifdef DEBUG_PROLONG
            print*, 'Calling mmProlongGen(AF_SYMM)'
#endif
                  CALL mmProlongGen (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     AFD,AF1,AF2,AF3,AF1,AF2,AF3,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &                 nxf,nyf,nzf,nxc,nyc,nzc,
     &                 hxf,hyf,hzf,comm,nbr,bndc,level,lUpdate,isub)
               ELSE
#ifdef DEBUG_PROLONG
            print*, 'Calling mmProlongGen(AF)'
#endif
                  CALL mmProlongGen (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &                 nxf,nyf,nzf,nxc,nyc,nzc,
     &                 hxf,hyf,hzf,comm,nbr,bndc,level,lUpdate,isub)
               END IF
               
            END IF
            
         END IF

      ELSE

         IF (gimatfree.EQ.1) THEN
            
#ifdef DEBUG_PROLONG
            print*, 'Calling mmProlongSCLin'
#endif
            CALL mmProlongSCLin (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &           nxf,nyf,nzf,nxc,nyc,nzc,
     &           hxf,hyf,hzf,comm,nbr,bndc,
     &           level,lUpdate,isub)

         ELSE 

            IF (gicoords.EQ.MM_CARTESIAN) THEN

               IF (gisymmetric.EQ.1) THEN
#ifdef DEBUG_PROLONG
                  print*, 'Calling mmProlongSCU(AF_SYMM)'
#endif
                  CALL mmProlongSCU (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     AFD,AF1,AF2,AF3,AF1,AF2,AF3,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &                 nxf,nyf,nzf,nxc,nyc,nzc,
     &                 hxf,hyf,hzf,comm,nbr,bndc,
     &                 level,lUpdate,isub)
               ELSE
#ifdef DEBUG_PROLONG
                  print*, 'Calling mmProlongSCU(AF)'
#endif
                  CALL mmProlongSCU (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &                 nxf,nyf,nzf,nxc,nyc,nzc,
     &                 hxf,hyf,hzf,comm,nbr,bndc,
     &                 level,lUpdate,isub)
               END IF

            ELSE 

               IF (gisymmetric.EQ.1) THEN
#ifdef DEBUG_PROLONG
                  print*, 'Calling mmProlongSCGen(AF_SYMM)'
#endif
                  CALL mmProlongSCGen (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     AFD,AF1,AF2,AF3,AF1,AF2,AF3,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &                 nxf,nyf,nzf,nxc,nyc,nzc,
     &                 hxf,hyf,hzf,comm,nbr,bndc,
     &                 level,lUpdate,isub)
               ELSE
#ifdef DEBUG_PROLONG
                  print*, 'Calling mmProlongSCGen(AF)'
#endif
                  CALL mmProlongSCGen (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,
     &                 nxf,nyf,nzf,nxc,nyc,nzc,
     &                 hxf,hyf,hzf,comm,nbr,bndc,
     &                 level,lUpdate,isub)
               END IF
               
            END IF
            
         END IF

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< entering mmProlong'
#endif

      RETURN
      END


C=======================================================================
C     
C     Module:  SUBROUTINE mmProlongU
C     
C     Purpose: Transfer a grid function to the next-finer grid level
C     Purpose: for uniform meshes
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmProlongU (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,nxf,nyf,nzf,nxc,nyc,nzc,
     &     hxf,hyf,hzf,comm,nbr,bndc,level,lUpdate,icall)

      IMPLICIT NONE

      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      
      INTEGER nxdxc,nydxc,nzdxc
      INTEGER bndxc(3)  
      REAL XC(nxdxc,nydxc,nzdxc)
      
      INTEGER nxdaf,nydaf,nzdaf
      INTEGER bndaf(3)  
      REAL AFD(nxdaf,nydaf,nzdaf)
      REAL AF1(nxdaf,nydaf,nzdaf)
      REAL AF2(nxdaf,nydaf,nzdaf)
      REAL AF3(nxdaf,nydaf,nzdaf)
      REAL AF4(nxdaf,nydaf,nzdaf)
      REAL AF5(nxdaf,nydaf,nzdaf)
      REAL AF6(nxdaf,nydaf,nzdaf)
      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      REAL hxf(nxf),hyf(nyf),hzf(nzf)
      INTEGER comm,nbr(6),bndc(3),level
      LOGICAL lUpdate

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

************************************************************************
*     Operator-based prolong XC to XF.  
*     Assumes a uniform grid (h(i) .EQ. h(i+1))
*************************************************************************

      INTEGER ixc,iyc,izc,ixf,iyf,izf
      REAL wxm,wxp,wym,wyp,wzm,wzp
      INTEGER CX1,CY1,CZ1,CX2,CY2,CZ2
      INTEGER DX1,DY1,DZ1,DX2,DY2,DZ2
      INTEGER MX1,MY1,MZ1,MX2,MY2,MZ2

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_PROLONG+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmProlongU'
#endif

      DX1=1
      DY1=1
      DZ1=1
      DX2=nxc-1
      DY2=nyc-1
      DZ2=nzc-1

      CX1=1
      CY1=1
      CZ1=1
      CX2=nxc-1
      CY2=nyc-1
      CZ2=nzc-1
      
      MX1=2
      MY1=2
      MZ1=2
      MX2=nxc-1
      MY2=nyc-1
      MZ2=nzc-1

      IF (ipx.EQ.npx-1) THEN
         CX2 = CX2 + 1
         MX2 = MX2 + 1
         DX2 = DX2 + 1
      END IF
      IF (ipy.EQ.npy-1) THEN
         CY2 = CY2 + 1
         MY2 = MY2 + 1
         DY2 = DY2 + 1
      END IF
      IF (ipz.EQ.npz-1) THEN
         CZ2 = CZ2 + 1
         MZ2 = MZ2 + 1
         DZ2 = DZ2 + 1
      END IF

*-----------------------------------------------------------------------
*     Refresh coarse-points on open boundaries
*-----------------------------------------------------------------------

      CALL mmGetX1Ghost (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
      CALL mmGetY1Ghost (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
      CALL mmGetZ1Ghost (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

*-----------------------------------------------------------------------
*     Fine-grid points that match coarse-grid points
*-----------------------------------------------------------------------

      DO izc=DZ1,DZ2
         izf = (izc-1)*2+1
         DO iyc=DY1,DY2
            iyf = (iyc-1)*2+1
            DO ixc=DX1,DX2
               ixf = (ixc-1)*2+1
               XF(ixf,iyf,izf) = XC(ixc,iyc,izc)
            END DO
         END DO
      END DO

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along X-axis
*-----------------------------------------------------------------------

      DO izc=CZ1,CZ2
         izf = (izc-1)*2+1
         DO iyc=CY1,CY2
            iyf = (iyc-1)*2+1
            DO ixc=MX1,MX2
               ixf = (ixc-1)*2
               wxm = ABS(AF4(ixf-1,iyf,izf))
               wxp = ABS(AF1(ixf,  iyf,izf))
               IF (wxm.LT.mmSmall) wxm = 1.0D0
               IF (wxp.LT.mmSmall) wxp = 1.0D0
               XF(ixf,iyf,izf) = 
     &              ( wxm * XF(ixf-1,iyf,izf) 
     &              + wxp * XF(ixf+1,iyf,izf) )
     &              / ( wxm + wxp )
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 5*(MX2-MX1-1)*(CY2-CY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 5*(MX2-MX1-1)*(CY2-CY1-1)*(CZ2-CZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along Y-axis
*-----------------------------------------------------------------------

      DO izc=CZ1,CZ2
         izf = (izc-1)*2+1
         DO iyc=MY1,MY2
            iyf = (iyc-1)*2
            DO ixc=CX1,CX2
               ixf = (ixc-1)*2+1
               wym = ABS(AF5(ixf,iyf-1,izf))
               wyp = ABS(AF2(ixf,iyf,  izf))
               IF (wym.LT.mmSmall) wym = 1.0D0
               IF (wyp.LT.mmSmall) wyp = 1.0D0
               XF(ixf,iyf,izf) =
     &              ( wym * XF(ixf,iyf-1,izf)
     &              + wyp * XF(ixf,iyf+1,izf) )
     &              / ( wym + wyp)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 5*(CX2-CX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 5*(CX2-CX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along Z-axis
*-----------------------------------------------------------------------

      DO izc=MZ1,MZ2
         izf = (izc-1)*2
         DO iyc=CY1,CY2
            iyf = (iyc-1)*2+1
            DO ixc=CX1,CX2
               ixf = (ixc-1)*2+1
               wzm = ABS(AF6(ixf,iyf,izf-1))
               wzp = ABS(AF3(ixf,iyf,izf))
               IF (wzm.LT.mmSmall) wzm = 1.0D0
               IF (wzp.LT.mmSmall) wzp = 1.0D0
               XF(ixf,iyf,izf) =
     &              ( wzm * XF(ixf,iyf,izf-1)
     &              + wzp * XF(ixf,iyf,izf+1) )
     &              / ( wzm + wzp )
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 5*(CX2-CX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 5*(CX2-CX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in X-Y plane
*-----------------------------------------------------------------------

      DO izc=CZ1,CZ2
         izf = (izc-1)*2+1
         DO iyc=MY1,MY2
            iyf = (iyc-1)*2
            DO ixc=MX1,MX2
               ixf = (ixc-1)*2
               wxm = ABS(AF4(ixf-1,iyf,  izf))
               wxp = ABS(AF1(ixf,  iyf,  izf))
               wym = ABS(AF5(ixf,  iyf-1,izf))
               wyp = ABS(AF2(ixf,  iyf  ,izf))
               IF (wxm.LT.mmSmall) wxm = 1.0D0
               IF (wxp.LT.mmSmall) wxp = 1.0D0
               IF (wym.LT.mmSmall) wym = 1.0D0
               IF (wyp.LT.mmSmall) wyp = 1.0D0
               XF(ixf,iyf,izf) =
     &              ( wxm * XF(ixf-1,iyf,  izf)
     &              + wxp * XF(ixf+1,iyf,  izf)
     &              + wym * XF(ixf,  iyf-1,izf)
     &              + wyp * XF(ixf,  iyf+1,izf) )
     &              /( wxm + wxp + wym + wyp )
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 11*(MX2-MX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 11*(MX2-MX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in Y-Z plane
*-----------------------------------------------------------------------

      DO izc=MZ1,MZ2
         izf = (izc-1)*2
         DO iyc=MY1,MY2
            iyf = (iyc-1)*2
            DO ixc=CX1,CX2
               ixf = (ixc-1)*2+1
               wym = ABS(AF5(ixf,iyf-1,izf))
               wyp = ABS(AF2(ixf,iyf,  izf))
               wzm = ABS(AF6(ixf,iyf,  izf-1))
               wzp = ABS(AF3(ixf,iyf,  izf))
               IF (wym.LT.mmSmall) wym = 1.0D0
               IF (wyp.LT.mmSmall) wyp = 1.0D0
               IF (wzm.LT.mmSmall) wzm = 1.0D0
               IF (wzp.LT.mmSmall) wzp = 1.0D0
               XF(ixf,iyf,izf) =
     &              ( wym * XF(ixf,iyf-1,izf)
     &              + wyp * XF(ixf,iyf+1,izf)
     &              + wzm * XF(ixf,iyf,  izf-1)
     &              + wzp * XF(ixf,iyf,  izf+1))
     &              /( wym + wyp + wzm + wzp )
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 11*(CX2-CX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 11*(CX2-CX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in X-Z plane
*-----------------------------------------------------------------------

      DO izc=MZ1,MZ2
         izf = (izc-1)*2
         DO iyc=CY1,CY2
            iyf = (iyc-1)*2+1
            DO ixc=MX1,MX2
               ixf = (ixc-1)*2
               wxm = ABS(AF4(ixf-1,iyf,izf))
               wxp = ABS(AF1(ixf,  iyf,izf))
               wzm = ABS(AF6(ixf,  iyf,izf-1))
               wzp = ABS(AF3(ixf,  iyf,izf))
               IF (wxm.LT.mmSmall) wxm = 1.0D0
               IF (wxp.LT.mmSmall) wxp = 1.0D0
               IF (wzm.LT.mmSmall) wzm = 1.0D0
               IF (wzp.LT.mmSmall) wzp = 1.0D0
               XF(ixf,iyf,izf) =
     &              ( wxm * XF(ixf-1,iyf,izf)
     &              + wxp * XF(ixf+1,iyf,izf)
     &              + wzm * XF(ixf,  iyf,izf-1)
     &              + wzp * XF(ixf,  iyf,izf+1))
     &              /( wxm + wxp + wzm + wzp )
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 11*(MX2-MX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 11*(MX2-MX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between eight coarse-grid points
*-----------------------------------------------------------------------

      DO izc=MZ1,MZ2
         izf = (izc-1)*2
         DO iyc=MY1,MY2
            iyf = (iyc-1)*2
            DO ixc=MX1,MX2
               ixf = (ixc-1)*2
               wxm = ABS(AF4(ixf-1,iyf,izf))
               wxp = ABS(AF1(ixf,  iyf,izf))
               wym = ABS(AF5(ixf,iyf-1,izf))
               wyp = ABS(AF2(ixf,iyf,  izf))
               wzm = ABS(AF6(ixf,  iyf,izf-1))
               wzp = ABS(AF3(ixf,  iyf,izf))
               IF (wxm.LT.mmSmall) wxm = 1.0D0
               IF (wxp.LT.mmSmall) wxp = 1.0D0
               IF (wym.LT.mmSmall) wym = 1.0D0
               IF (wyp.LT.mmSmall) wyp = 1.0D0
               IF (wzm.LT.mmSmall) wzm = 1.0D0
               IF (wzp.LT.mmSmall) wzp = 1.0D0
               XF(ixf,iyf,izf) =
     &              ( wxm * XF(ixf-1,iyf,  izf)
     &              + wxp * XF(ixf+1,iyf,  izf)
     &              + wym * XF(ixf,  iyf-1,izf)
     &              + wyp * XF(ixf,  iyf+1,izf)
     &              + wzm * XF(ixf,  iyf,  izf-1)
     &              + wzp * XF(ixf,  iyf,  izf+1))
     &              /( wxm + wxp + wym + wyp + wzm + wzp )
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 17*(MX2-MX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 17*(MX2-MX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmProlongU'
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmProlongGen
C     
C     Purpose: Transfer a grid function to the next-finer grid level 
C     Purpose: for uniform meshes
C     
C     Algorithm:
C     
C     Inputs:
C     
C     Outputs:
C     
C     Globals accessed:
C     
C     Globals modified:
C     
C     Calls:
C     
C     Called by:
C     
C     Flop count estimate:
C     
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C     
C     Modification history: 
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmProlongGen (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,nxf,nyf,nzf,nxc,nyc,nzc,
     &     hxf,hyf,hzf,comm,nbr,bndc,level,lUpdate,icall)

      IMPLICIT NONE

      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      
      INTEGER nxdxc,nydxc,nzdxc
      INTEGER bndxc(3)  
      REAL XC(nxdxc,nydxc,nzdxc)
      
      INTEGER nxdaf,nydaf,nzdaf
      INTEGER bndaf(3)  
      REAL AFD(nxdaf,nydaf,nzdaf)
      REAL AF1(nxdaf,nydaf,nzdaf)
      REAL AF2(nxdaf,nydaf,nzdaf)
      REAL AF3(nxdaf,nydaf,nzdaf)
      REAL AF4(nxdaf,nydaf,nzdaf)
      REAL AF5(nxdaf,nydaf,nzdaf)
      REAL AF6(nxdaf,nydaf,nzdaf)
      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      REAL hxf(nxf),hyf(nyf),hzf(nzf)
      INTEGER comm,nbr(6),bndc(3),level
      LOGICAL lUpdate

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

************************************************************************
*     Operator-based prolong XC to XF.  
*     Assumes a uniform grid (h(i) .EQ. h(i+1))
*************************************************************************

      INTEGER ixc,iyc,izc,ixf,iyf,izf
      REAL wxm,wxp,wym,wyp,wzm,wzp
      INTEGER CX1,CY1,CZ1,CX2,CY2,CZ2
      INTEGER DX1,DY1,DZ1,DX2,DY2,DZ2
      INTEGER MX1,MY1,MZ1,MX2,MY2,MZ2

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_PROLONG+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmProlongGen'
#endif

      DX1=1
      DY1=1
      DZ1=1
      DX2=nxc-1
      DY2=nyc-1
      DZ2=nzc-1

      CX1=1
      CY1=1
      CZ1=1
      CX2=nxc-1
      CY2=nyc-1
      CZ2=nzc-1
      
      MX1=2
      MY1=2
      MZ1=2
      MX2=nxc-1
      MY2=nyc-1
      MZ2=nzc-1

      IF (ipx.EQ.npx-1) THEN
         CX2 = CX2 + 1
         MX2 = MX2 + 1
         DX2 = DX2 + 1
      END IF
      IF (ipy.EQ.npy-1) THEN
         CY2 = CY2 + 1
         MY2 = MY2 + 1
         DY2 = DY2 + 1
      END IF
      IF (ipz.EQ.npz-1) THEN
         CZ2 = CZ2 + 1
         MZ2 = MZ2 + 1
         DZ2 = DZ2 + 1
      END IF

*-----------------------------------------------------------------------
*     Refresh coarse-points on open boundaries
*-----------------------------------------------------------------------

      CALL mmGetX1Ghost (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
      CALL mmGetY1Ghost (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
      CALL mmGetZ1Ghost (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

*-----------------------------------------------------------------------
*     PROLONG AND UPDATE
*-----------------------------------------------------------------------

      IF (lUpdate) THEN 

*-----------------------------------------------------------------------
*     Fine-grid points that match coarse-grid points
*-----------------------------------------------------------------------

         DO izc=DZ1,DZ2
            izf = (izc-1)*2+1
            DO iyc=DY1,DY2
               iyf = (iyc-1)*2+1
               DO ixc=DX1,DX2
                  ixf = (ixc-1)*2+1
                  XF(ixf,iyf,izf) = XF(ixf,iyf,izf) + XC(ixc,iyc,izc)
               END DO
            END DO
         END DO

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along X-axis
*-----------------------------------------------------------------------

         DO izc=CZ1,CZ2
            izf = (izc-1)*2+1
            DO iyc=CY1,CY2
               iyf = (iyc-1)*2+1
               DO ixc=MX1,MX2
                  ixf = (ixc-1)*2
                  wxm = ABS(AF4(ixf-1,iyf,izf) * gdxi(ixf-1,level))
                  wxp = ABS(AF1(ixf,  iyf,izf) * gdxi(ixf,  level))
                  IF (wxm.LT.mmSmall) wxm = 1.0D0
                  IF (wxp.LT.mmSmall) wxp = 1.0D0
                  XF(ixf,iyf,izf) = XF(ixf,iyf,izf) + 
     &                 ( wxm * XC(ixc-1,iyf,izf) 
     &                 + wxp * XC(ixc  ,iyf,izf) )
     &                 / ( wxm + wxp )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 7*(MX2-MX1-1)*(CY2-CY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 7*(MX2-MX1-1)*(CY2-CY1-1)*(CZ2-CZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along Y-axis
*-----------------------------------------------------------------------

         DO izc=CZ1,CZ2
            izf = (izc-1)*2+1
            DO iyc=MY1,MY2
               iyf = (iyc-1)*2
               DO ixc=CX1,CX2
                  ixf = (ixc-1)*2+1
                  wym = ABS(AF5(ixf,iyf-1,izf) * gdyi(iyf-1,level))
                  wyp = ABS(AF2(ixf,iyf,  izf) * gdyi(iyf,  level))
                  IF (wym.LT.mmSmall) wym = 1.0D0
                  IF (wyp.LT.mmSmall) wyp = 1.0D0
                  XF(ixf,iyf,izf) = XF(ixf,iyf,izf) +
     &                 ( wym * XF(ixf,iyf-1,izf)
     &                 + wyp * XF(ixf,iyf+1,izf) )
     &                 / ( wym + wyp)
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 7*(CX2-CX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 7*(CX2-CX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along Z-axis
*-----------------------------------------------------------------------

         DO izc=MZ1,MZ2
            izf = (izc-1)*2
            DO iyc=CY1,CY2
               iyf = (iyc-1)*2+1
               DO ixc=CX1,CX2
                  ixf = (ixc-1)*2+1
                  wzm = ABS(AF6(ixf,iyf,izf-1) * gdzi(izf-1,level))
                  wzp = ABS(AF3(ixf,iyf,izf)   * gdzi(izf,  level))
                  IF (wzm.LT.mmSmall) wzm = 1.0D0
                  IF (wzp.LT.mmSmall) wzp = 1.0D0
                  XF(ixf,iyf,izf) = XF(ixf,iyf,izf) +
     &                 ( wzm * XF(ixf,iyf,izf-1)
     &                 + wzp * XF(ixf,iyf,izf+1) )
     &                 / ( wzm + wzp )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 7*(CX2-CX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 7*(CX2-CX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in X-Y plane
*-----------------------------------------------------------------------

         DO izc=CZ1,CZ2
            izf = (izc-1)*2+1
            DO iyc=MY1,MY2
               iyf = (iyc-1)*2
               DO ixc=MX1,MX2
                  ixf = (ixc-1)*2
                  wxm = ABS(AF4(ixf-1,iyf,  izf) * gdxi(ixf-1,level))
                  wxp = ABS(AF1(ixf,  iyf,  izf) * gdxi(ixf,  level))
                  wym = ABS(AF5(ixf,  iyf-1,izf) * gdyi(iyf-1,level))
                  wyp = ABS(AF2(ixf,  iyf  ,izf) * gdyi(iyf,  level))
                  IF (wxm.LT.mmSmall) wxm = 1.0D0
                  IF (wxp.LT.mmSmall) wxp = 1.0D0
                  IF (wym.LT.mmSmall) wym = 1.0D0
                  IF (wyp.LT.mmSmall) wyp = 1.0D0
                  XF(ixf,iyf,izf) = XF(ixf,iyf,izf) +
     &                 ( wxm * XF(ixf-1,iyf,  izf)
     &                 + wxp * XF(ixf+1,iyf,  izf)
     &                 + wym * XF(ixf,  iyf-1,izf)
     &                 + wyp * XF(ixf,  iyf+1,izf) )
     &                 /( wxm + wxp + wym + wyp )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 15*(MX2-MX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 15*(MX2-MX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in Y-Z plane
*-----------------------------------------------------------------------

         DO izc=MZ1,MZ2
            izf = (izc-1)*2
            DO iyc=MY1,MY2
               iyf = (iyc-1)*2
               DO ixc=CX1,CX2
                  ixf = (ixc-1)*2+1
                  wym = ABS(AF5(ixf,iyf-1,izf)   * gdyi(iyf-1,level))
                  wyp = ABS(AF2(ixf,iyf,  izf)   * gdyi(iyf,  level))
                  wzm = ABS(AF6(ixf,iyf,  izf-1) * gdzi(izf-1,level))
                  wzp = ABS(AF3(ixf,iyf,  izf)   * gdzi(izf,  level))
                  IF (wym.LT.mmSmall) wym = 1.0D0
                  IF (wyp.LT.mmSmall) wyp = 1.0D0
                  IF (wzm.LT.mmSmall) wzm = 1.0D0
                  IF (wzp.LT.mmSmall) wzp = 1.0D0
                  XF(ixf,iyf,izf) = XF(ixf,iyf,izf) +
     &                 ( wym * XF(ixf,iyf-1,izf)
     &                 + wyp * XF(ixf,iyf+1,izf)
     &                 + wzm * XF(ixf,iyf,  izf-1)
     &                 + wzp * XF(ixf,iyf,  izf+1))
     &                 /( wym + wyp + wzm + wzp )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 15*(CX2-CX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 15*(CX2-CX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in X-Z plane
*-----------------------------------------------------------------------

         DO izc=MZ1,MZ2
            izf = (izc-1)*2
            DO iyc=CY1,CY2
               iyf = (iyc-1)*2+1
               DO ixc=MX1,MX2
                  ixf = (ixc-1)*2
                  wxm = ABS(AF4(ixf-1,iyf,izf)   * gdxi(ixf-1,level))
                  wxp = ABS(AF1(ixf,  iyf,izf)   * gdxi(ixf,  level))
                  wzm = ABS(AF6(ixf,  iyf,izf-1) * gdzi(izf-1,level))
                  wzp = ABS(AF3(ixf,  iyf,izf)   * gdzi(izf,  level))
                  IF (wxm.LT.mmSmall) wxm = 1.0D0
                  IF (wxp.LT.mmSmall) wxp = 1.0D0
                  IF (wzm.LT.mmSmall) wzm = 1.0D0
                  IF (wzp.LT.mmSmall) wzp = 1.0D0
                  XF(ixf,iyf,izf) = XF(ixf,iyf,izf) +
     &                 ( wxm * XF(ixf-1,iyf,izf)
     &                 + wxp * XF(ixf+1,iyf,izf)
     &                 + wzm * XF(ixf,  iyf,izf-1)
     &                 + wzp * XF(ixf,  iyf,izf+1))
     &                 /( wxm + wxp + wzm + wzp )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 15*(MX2-MX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 15*(MX2-MX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between eight coarse-grid points
*-----------------------------------------------------------------------

         DO izc=MZ1,MZ2
            izf = (izc-1)*2
            DO iyc=MY1,MY2
               iyf = (iyc-1)*2
               DO ixc=MX1,MX2
                  ixf = (ixc-1)*2
                  wxm = ABS(AF4(ixf-1,iyf,izf)   * gdxi(ixf-1,level))
                  wxp = ABS(AF1(ixf,  iyf,izf)   * gdxi(ixf,  level))
                  wym = ABS(AF5(ixf,iyf-1,izf)   * gdyi(iyf-1,level))
                  wyp = ABS(AF2(ixf,iyf,  izf)   * gdyi(iyf,  level))
                  wzm = ABS(AF6(ixf,  iyf,izf-1) * gdzi(izf-1,level))
                  wzp = ABS(AF3(ixf,  iyf,izf)   * gdzi(izf,  level))
                  IF (wxm.LT.mmSmall) wxm = 1.0D0
                  IF (wxp.LT.mmSmall) wxp = 1.0D0
                  IF (wym.LT.mmSmall) wym = 1.0D0
                  IF (wyp.LT.mmSmall) wyp = 1.0D0
                  IF (wzm.LT.mmSmall) wzm = 1.0D0
                  IF (wzp.LT.mmSmall) wzp = 1.0D0
                  XF(ixf,iyf,izf) = XF(ixf,iyf,izf) +
     &                 ( wxm * XF(ixf-1,iyf,  izf)
     &                 + wxp * XF(ixf+1,iyf,  izf)
     &                 + wym * XF(ixf,  iyf-1,izf)
     &                 + wyp * XF(ixf,  iyf+1,izf)
     &                 + wzm * XF(ixf,  iyf,  izf-1)
     &                 + wzp * XF(ixf,  iyf,  izf+1))
     &                 /( wxm + wxp + wym + wyp + wzm + wzp )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 23*(MX2-MX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 23*(MX2-MX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
#endif

      ELSE

*-----------------------------------------------------------------------
*     PROLONG ONLY 
*-----------------------------------------------------------------------

*-----------------------------------------------------------------------
*     Fine-grid points that match coarse-grid points
*-----------------------------------------------------------------------

         DO izc=DZ1,DZ2
            izf = (izc-1)*2+1
            DO iyc=DY1,DY2
               iyf = (iyc-1)*2+1
               DO ixc=DX1,DX2
                  ixf = (ixc-1)*2+1
                  XF(ixf,iyf,izf) = XC(ixc,iyc,izc)
               END DO
            END DO
         END DO

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along X-axis
*-----------------------------------------------------------------------

         DO izc=CZ1,CZ2
            izf = (izc-1)*2+1
            DO iyc=CY1,CY2
               iyf = (iyc-1)*2+1
               DO ixc=MX1,MX2
                  ixf = (ixc-1)*2
                  wxm = ABS(AF4(ixf-1,iyf,izf) * gdxi(ixf-1,level))
                  wxp = ABS(AF1(ixf,  iyf,izf) * gdxi(ixf,  level))
                  IF (wxm.LT.mmSmall) wxm = 1.0D0
                  IF (wxp.LT.mmSmall) wxp = 1.0D0
                  XF(ixf,iyf,izf) = 
     &                 ( wxm * XF(ixf-1,iyf,izf) 
     &                 + wxp * XF(ixf+1,iyf,izf) )
     &                 / ( wxm + wxp )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 7*(MX2-MX1-1)*(CY2-CY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 7*(MX2-MX1-1)*(CY2-CY1-1)*(CZ2-CZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along Y-axis
*-----------------------------------------------------------------------

         DO izc=CZ1,CZ2
            izf = (izc-1)*2+1
            DO iyc=MY1,MY2
               iyf = (iyc-1)*2
               DO ixc=CX1,CX2
                  ixf = (ixc-1)*2+1
                  wym = ABS(AF5(ixf,iyf-1,izf) * gdyi(iyf-1,level))
                  wyp = ABS(AF2(ixf,iyf,  izf) * gdyi(iyf,  level))
                  IF (wym.LT.mmSmall) wym = 1.0D0
                  IF (wyp.LT.mmSmall) wyp = 1.0D0
                  XF(ixf,iyf,izf) =
     &                 ( wym * XF(ixf,iyf-1,izf)
     &                 + wyp * XF(ixf,iyf+1,izf) )
     &                 / ( wym + wyp)
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 7*(CX2-CX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 7*(CX2-CX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along Z-axis
*-----------------------------------------------------------------------

         DO izc=MZ1,MZ2
            izf = (izc-1)*2
            DO iyc=CY1,CY2
               iyf = (iyc-1)*2+1
               DO ixc=CX1,CX2
                  ixf = (ixc-1)*2+1
                  wzm = ABS(AF6(ixf,iyf,izf-1) * gdzi(izf-1,level))
                  wzp = ABS(AF3(ixf,iyf,izf)   * gdzi(izf,  level))
                  IF (wzm.LT.mmSmall) wzm = 1.0D0
                  IF (wzp.LT.mmSmall) wzp = 1.0D0
                  XF(ixf,iyf,izf) =
     &                 ( wzm * XF(ixf,iyf,izf-1)
     &                 + wzp * XF(ixf,iyf,izf+1) )
     &                 / ( wzm + wzp )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 7*(CX2-CX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 7*(CX2-CX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in X-Y plane
*-----------------------------------------------------------------------

         DO izc=CZ1,CZ2
            izf = (izc-1)*2+1
            DO iyc=MY1,MY2
               iyf = (iyc-1)*2
               DO ixc=MX1,MX2
                  ixf = (ixc-1)*2
                  wxm = ABS(AF4(ixf-1,iyf,  izf) * gdxi(ixf-1,level))
                  wxp = ABS(AF1(ixf,  iyf,  izf) * gdxi(ixf,  level))
                  wym = ABS(AF5(ixf,  iyf-1,izf) * gdyi(iyf-1,level))
                  wyp = ABS(AF2(ixf,  iyf  ,izf) * gdyi(iyf,  level))
                  IF (wxm.LT.mmSmall) wxm = 1.0D0
                  IF (wxp.LT.mmSmall) wxp = 1.0D0
                  IF (wym.LT.mmSmall) wym = 1.0D0
                  IF (wyp.LT.mmSmall) wyp = 1.0D0
                  XF(ixf,iyf,izf) =
     &                 ( wxm * XF(ixf-1,iyf,  izf)
     &                 + wxp * XF(ixf+1,iyf,  izf)
     &                 + wym * XF(ixf,  iyf-1,izf)
     &                 + wyp * XF(ixf,  iyf+1,izf) )
     &                 /( wxm + wxp + wym + wyp )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 15*(MX2-MX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 15*(MX2-MX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in Y-Z plane
*-----------------------------------------------------------------------

         DO izc=MZ1,MZ2
            izf = (izc-1)*2
            DO iyc=MY1,MY2
               iyf = (iyc-1)*2
               DO ixc=CX1,CX2
                  ixf = (ixc-1)*2+1
                  wym = ABS(AF5(ixf,iyf-1,izf)   * gdyi(iyf-1,level))
                  wyp = ABS(AF2(ixf,iyf,  izf)   * gdyi(iyf,  level))
                  wzm = ABS(AF6(ixf,iyf,  izf-1) * gdzi(izf-1,level))
                  wzp = ABS(AF3(ixf,iyf,  izf)   * gdzi(izf,  level))
                  IF (wym.LT.mmSmall) wym = 1.0D0
                  IF (wyp.LT.mmSmall) wyp = 1.0D0
                  IF (wzm.LT.mmSmall) wzm = 1.0D0
                  IF (wzp.LT.mmSmall) wzp = 1.0D0
                  XF(ixf,iyf,izf) =
     &                 ( wym * XF(ixf,iyf-1,izf)
     &                 + wyp * XF(ixf,iyf+1,izf)
     &                 + wzm * XF(ixf,iyf,  izf-1)
     &                 + wzp * XF(ixf,iyf,  izf+1))
     &                 /( wym + wyp + wzm + wzp )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 15*(CX2-CX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 15*(CX2-CX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in X-Z plane
*-----------------------------------------------------------------------

         DO izc=MZ1,MZ2
            izf = (izc-1)*2
            DO iyc=CY1,CY2
               iyf = (iyc-1)*2+1
               DO ixc=MX1,MX2
                  ixf = (ixc-1)*2
                  wxm = ABS(AF4(ixf-1,iyf,izf)   * gdxi(ixf-1,level))
                  wxp = ABS(AF1(ixf,  iyf,izf)   * gdxi(ixf,  level))
                  wzm = ABS(AF6(ixf,  iyf,izf-1) * gdzi(izf-1,level))
                  wzp = ABS(AF3(ixf,  iyf,izf)   * gdzi(izf,  level))
                  IF (wxm.LT.mmSmall) wxm = 1.0D0
                  IF (wxp.LT.mmSmall) wxp = 1.0D0
                  IF (wzm.LT.mmSmall) wzm = 1.0D0
                  IF (wzp.LT.mmSmall) wzp = 1.0D0
                  XF(ixf,iyf,izf) =
     &                 ( wxm * XF(ixf-1,iyf,izf)
     &                 + wxp * XF(ixf+1,iyf,izf)
     &                 + wzm * XF(ixf,  iyf,izf-1)
     &                 + wzp * XF(ixf,  iyf,izf+1))
     &                 /( wxm + wxp + wzm + wzp )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 15*(MX2-MX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 15*(MX2-MX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between eight coarse-grid points
*-----------------------------------------------------------------------

         DO izc=MZ1,MZ2
            izf = (izc-1)*2
            DO iyc=MY1,MY2
               iyf = (iyc-1)*2
               DO ixc=MX1,MX2
                  ixf = (ixc-1)*2
                  wxm = ABS(AF4(ixf-1,iyf,izf)   * gdxi(ixf-1,level))
                  wxp = ABS(AF1(ixf,  iyf,izf)   * gdxi(ixf,  level))
                  wym = ABS(AF5(ixf,iyf-1,izf)   * gdyi(iyf-1,level))
                  wyp = ABS(AF2(ixf,iyf,  izf)   * gdyi(iyf,  level))
                  wzm = ABS(AF6(ixf,  iyf,izf-1) * gdzi(izf-1,level))
                  wzp = ABS(AF3(ixf,  iyf,izf)   * gdzi(izf,  level))
                  IF (wxm.LT.mmSmall) wxm = 1.0D0
                  IF (wxp.LT.mmSmall) wxp = 1.0D0
                  IF (wym.LT.mmSmall) wym = 1.0D0
                  IF (wyp.LT.mmSmall) wyp = 1.0D0
                  IF (wzm.LT.mmSmall) wzm = 1.0D0
                  IF (wzp.LT.mmSmall) wzp = 1.0D0
                  XF(ixf,iyf,izf) =
     &                 ( wxm * XF(ixf-1,iyf,  izf)
     &                 + wxp * XF(ixf+1,iyf,  izf)
     &                 + wym * XF(ixf,  iyf-1,izf)
     &                 + wyp * XF(ixf,  iyf+1,izf)
     &                 + wzm * XF(ixf,  iyf,  izf-1)
     &                 + wzp * XF(ixf,  iyf,  izf+1))
     &                 /( wxm + wxp + wym + wyp + wzm + wzp )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 23*(MX2-MX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 23*(MX2-MX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
#endif

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmProlongGen'
#endif

      RETURN
      END

C=======================================================================
C     
C     Module:  SUBROUTINE mmProlongLin
C     
C     Purpose: Transfer a grid function to the next-finer grid level
C     Purpose: for uniform meshes
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmProlongLin (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxf,nyf,nzf,nxc,nyc,nzc,
     &     hxf,hyf,hzf,comm,nbr,bndc,level,lUpdate,icall)

      IMPLICIT NONE

      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      
      INTEGER nxdxc,nydxc,nzdxc
      INTEGER bndxc(3)  
      REAL XC(nxdxc,nydxc,nzdxc)
      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      REAL hxf(nxf),hyf(nyf),hzf(nzf)
      INTEGER comm,nbr(6),bndc(3),level
      LOGICAL lUpdate

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

************************************************************************
*     Operator-based prolong XC to XF.  
*     Assumes a uniform grid (h(i) .EQ. h(i+1))
*************************************************************************

      INTEGER ixc,iyc,izc,ixf,iyf,izf
      INTEGER CX1,CY1,CZ1,CX2,CY2,CZ2
      INTEGER DX1,DY1,DZ1,DX2,DY2,DZ2
      INTEGER MX1,MY1,MZ1,MX2,MY2,MZ2

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_PROLONG+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmProlongLin'
#endif

      DX1=1
      DY1=1
      DZ1=1
      DX2=nxc-1
      DY2=nyc-1
      DZ2=nzc-1

      CX1=1
      CY1=1
      CZ1=1
      CX2=nxc-1
      CY2=nyc-1
      CZ2=nzc-1
      
      MX1=2
      MY1=2
      MZ1=2
      MX2=nxc-1
      MY2=nyc-1
      MZ2=nzc-1

      IF (ipx.EQ.npx-1) THEN
         CX2 = CX2 + 1
         MX2 = MX2 + 1
         DX2 = DX2 + 1
      END IF
      IF (ipy.EQ.npy-1) THEN
         CY2 = CY2 + 1
         MY2 = MY2 + 1
         DY2 = DY2 + 1
      END IF
      IF (ipz.EQ.npz-1) THEN
         CZ2 = CZ2 + 1
         MZ2 = MZ2 + 1
         DZ2 = DZ2 + 1
      END IF

*-----------------------------------------------------------------------
*     Refresh coarse-points on open boundaries
*-----------------------------------------------------------------------

      CALL mmGetX1Ghost (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
      CALL mmGetY1Ghost (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
      CALL mmGetZ1Ghost (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

*-----------------------------------------------------------------------
*     Fine-grid points that match coarse-grid points
*-----------------------------------------------------------------------

      DO izc=DZ1,DZ2
         izf = (izc-1)*2+1
         DO iyc=DY1,DY2
            iyf = (iyc-1)*2+1
            DO ixc=DX1,DX2
               ixf = (ixc-1)*2+1
               XF(ixf,iyf,izf) = XC(ixc,iyc,izc)
            END DO
         END DO
      END DO

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along X-axis
*-----------------------------------------------------------------------

      DO izc=CZ1,CZ2
         izf = (izc-1)*2+1
         DO iyc=CY1,CY2
            iyf = (iyc-1)*2+1
            DO ixc=MX1,MX2
               ixf = (ixc-1)*2
               XF(ixf,iyf,izf) = 
     &              o2*( XF(ixf-1,iyf,izf) 
     &              +    XF(ixf+1,iyf,izf) )
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 5*(MX2-MX1-1)*(CY2-CY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 5*(MX2-MX1-1)*(CY2-CY1-1)*(CZ2-CZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along Y-axis
*-----------------------------------------------------------------------

      DO izc=CZ1,CZ2
         izf = (izc-1)*2+1
         DO iyc=MY1,MY2
            iyf = (iyc-1)*2
            DO ixc=CX1,CX2
               ixf = (ixc-1)*2+1
               XF(ixf,iyf,izf) =
     &              o2*( XF(ixf,iyf-1,izf)
     &              +    XF(ixf,iyf+1,izf) )
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 5*(CX2-CX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 5*(CX2-CX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along Z-axis
*-----------------------------------------------------------------------

      DO izc=MZ1,MZ2
         izf = (izc-1)*2
         DO iyc=CY1,CY2
            iyf = (iyc-1)*2+1
            DO ixc=CX1,CX2
               ixf = (ixc-1)*2+1
               XF(ixf,iyf,izf) =
     &              o2*( XF(ixf,iyf,izf-1)
     &              +    XF(ixf,iyf,izf+1) )
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 5*(CX2-CX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 5*(CX2-CX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in X-Y plane
*-----------------------------------------------------------------------

      DO izc=CZ1,CZ2
         izf = (izc-1)*2+1
         DO iyc=MY1,MY2
            iyf = (iyc-1)*2
            DO ixc=MX1,MX2
               ixf = (ixc-1)*2
               XF(ixf,iyf,izf) =
     &              o4*( XF(ixf-1,iyf,  izf)
     &              +    XF(ixf+1,iyf,  izf)
     &              +    XF(ixf,  iyf-1,izf)
     &              +    XF(ixf,  iyf+1,izf) )
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 11*(MX2-MX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 11*(MX2-MX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in Y-Z plane
*-----------------------------------------------------------------------

      DO izc=MZ1,MZ2
         izf = (izc-1)*2
         DO iyc=MY1,MY2
            iyf = (iyc-1)*2
            DO ixc=CX1,CX2
               ixf = (ixc-1)*2+1
               XF(ixf,iyf,izf) =
     &              o4*( XF(ixf,iyf-1,izf)
     &              +    XF(ixf,iyf+1,izf)
     &              +    XF(ixf,iyf,  izf-1)
     &              +    XF(ixf,iyf,  izf+1))
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 11*(CX2-CX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 11*(CX2-CX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in X-Z plane
*-----------------------------------------------------------------------

      DO izc=MZ1,MZ2
         izf = (izc-1)*2
         DO iyc=CY1,CY2
            iyf = (iyc-1)*2+1
            DO ixc=MX1,MX2
               ixf = (ixc-1)*2
               XF(ixf,iyf,izf) =
     &              o4*( XF(ixf-1,iyf,izf)
     &              +    XF(ixf+1,iyf,izf)
     &              +    XF(ixf,  iyf,izf-1)
     &              +    XF(ixf,  iyf,izf+1))
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 11*(MX2-MX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 11*(MX2-MX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
#endif

*-----------------------------------------------------------------------
*     Fine-grid points between eight coarse-grid points
*-----------------------------------------------------------------------

      DO izc=MZ1,MZ2
         izf = (izc-1)*2
         DO iyc=MY1,MY2
            iyf = (iyc-1)*2
            DO ixc=MX1,MX2
               ixf = (ixc-1)*2
               XF(ixf,iyf,izf) =
     &              o6*( XF(ixf-1,iyf,  izf)
     &              +    XF(ixf+1,iyf,  izf)
     &              +    XF(ixf,  iyf-1,izf)
     &              +    XF(ixf,  iyf+1,izf)
     &              +    XF(ixf,  iyf,  izf-1)
     &              +    XF(ixf,  iyf,  izf+1))
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 17*(MX2-MX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 17*(MX2-MX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmProlongLin'
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  mmProlongSCU
C     
C     Purpose: Transfer a 3-D grid function to the next semi-refined grid 
C     
C     Algorithm: 5-point or 3-point operator-based prolongation
C     
C     Inputs:  XC: coarse grid function
C     
C     Outputs: XF: fine grid function
C     
C     Globals accessed:
C     
C     Globals modified:
C     
C     Calls:
C     
C     Called by:
C     
C     Flop count estimate:
C     
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C     
C     Modification history: 
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmProlongSCU (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,nxf,nyf,nzf,nxc,nyc,nzc,
     &     hxf,hyf,hzf,comm,nbr,bndc,level,lUpdate,icall)

      IMPLICIT NONE

      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      
      INTEGER nxdxc,nydxc,nzdxc
      INTEGER bndxc(3)  
      REAL XC(nxdxc,nydxc,nzdxc)
      
      INTEGER nxdaf,nydaf,nzdaf
      INTEGER bndaf(3)  
      REAL AFD(nxdaf,nydaf,nzdaf)
      REAL AF1(nxdaf,nydaf,nzdaf)
      REAL AF2(nxdaf,nydaf,nzdaf)
      REAL AF3(nxdaf,nydaf,nzdaf)
      REAL AF4(nxdaf,nydaf,nzdaf)
      REAL AF5(nxdaf,nydaf,nzdaf)
      REAL AF6(nxdaf,nydaf,nzdaf)
      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      REAL hxf(nxf),hyf(nyf),hzf(nzf)
      INTEGER comm,nbr(6),bndc(3),level
      LOGICAL lUpdate

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

************************************************************************
*     Operator-based prolong XC to XF.  
*     Assumes a uniform grid (h(i) .EQ. h(i+1))
*************************************************************************

      INTEGER ixc,iyc,izc,ixf,iyf,izf
      INTEGER idx,idy,idz
      INTEGER CX1,CY1,CZ1,CX2,CY2,CZ2
      INTEGER DX1,DY1,DZ1,DX2,DY2,DZ2
      INTEGER MX1,MY1,MZ1,MX2,MY2,MZ2
      INTEGER idim
      REAL wxm,wxp,wym,wyp,wzm,wzp

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_PROLONG+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmProlongSCU'
#endif

      DX1=1
      DY1=1
      DZ1=1
      DX2=nxc-1
      DY2=nyc-1
      DZ2=nzc-1

      CX1=1
      CY1=1
      CZ1=1
      CX2=nxc-1
      CY2=nyc-1
      CZ2=nzc-1
      
      MX1=2
      MY1=2
      MZ1=2
      MX2=nxc-1
      MY2=nyc-1
      MZ2=nzc-1

      IF (ipx.EQ.npx-1) THEN
         CX2 = CX2 + 1
         MX2 = MX2 + 1
         DX2 = DX2 + 1
      END IF
      IF (ipy.EQ.npy-1) THEN
         CY2 = CY2 + 1
         MY2 = MY2 + 1
         DY2 = DY2 + 1
      END IF
      IF (ipz.EQ.npz-1) THEN
         CZ2 = CZ2 + 1
         MZ2 = MZ2 + 1
         DZ2 = DZ2 + 1
      END IF

      idim = 0

      IF (gmmlc1) idim = idim + 1
      IF (gmmlc2) idim = idim + 1
      IF (gmmlc3) idim = idim + 1

      IF (idim.EQ.3) THEN

         PRINT*, '*** MGMPI ***: mmProlongSCU called as full'
         PRINT*, '               coarsening restriction!'
         PRINT*, '               Calling mmProlongU...'
         CALL mmProlongU (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,nxf,nyf,nzf,nxc,nyc,nzc,
     &        hxf,hyf,hzf,comm,nbr,bndc,level,lUpdate,icall)

         GOTO 10

      END IF
      
*-----------------------------------------------------------------------
*     Refresh coarse-points on open boundaries
*-----------------------------------------------------------------------

      IF (gmmlc1) CALL mmGetX1Ghost (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
      IF (gmmlc2) CALL mmGetY1Ghost (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
      IF (gmmlc3) CALL mmGetZ1Ghost (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

*-----------------------------------------------------------------------
*     Fine-grid points that match coarse-grid points
*-----------------------------------------------------------------------

      idx = 1
      idy = 1
      idz = 1
      IF (gmmlc1) idx = 2 
      IF (gmmlc2) idy = 2 
      IF (gmmlc3) idz = 2

      DO izc=DZ1,DZ2
         izf = (izc-1)*idz+1
         DO iyc=DY1,DY2
            iyf = (iyc-1)*idy+1
            DO ixc=DX1,DX2
               ixf = (ixc-1)*idx+1
               XF(ixf,iyf,izf) = XC(ixc,iyc,izc)
            END DO
         END DO
      END DO

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along X-axis
*-----------------------------------------------------------------------

      IF (gmmlc1) THEN

         DO izc=CZ1,CZ2
            izf = (izc-1)*idz+1
            DO iyc=CY1,CY2
               iyf = (iyc-1)*idy+1
               DO ixc=MX1,MX2
                  ixf = (ixc-1)*idx
                  wxm = ABS(AF4(ixf-1,iyf,izf))
                  wxp = ABS(AF1(ixf,  iyf,izf))
                  IF (wxm.LT.mmSmall) wxm = 1.0D0
                  IF (wxp.LT.mmSmall) wxp = 1.0D0
                  XF(ixf,iyf,izf) = 
     &                 ( wxm * XF(ixf-1,iyf,izf) 
     &                 + wxp * XF(ixf+1,iyf,izf) )
     &                 / ( wxm + wxp )
               END DO
            END DO
         END DO
         
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 5*(MX2-MX1-1)*(CY2-CY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 5*(MX2-MX1-1)*(CY2-CY1-1)*(CZ2-CZ1-1)
#endif

      END IF

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along Y-axis
*-----------------------------------------------------------------------

      IF (gmmlc2) THEN

         DO izc=CZ1,CZ2
            izf = (izc-1)*idz+1
            DO iyc=MY1,MY2
               iyf = (iyc-1)*idy
               DO ixc=CX1,CX2
                  ixf = (ixc-1)*idx+1
                  wym = ABS(AF5(ixf,iyf-1,izf))
                  wyp = ABS(AF2(ixf,iyf,  izf))
                  IF (wym.LT.mmSmall) wym = 1.0D0
                  IF (wyp.LT.mmSmall) wyp = 1.0D0
                  XF(ixf,iyf,izf) =
     &                 ( wym * XF(ixf,iyf-1,izf)
     &                 + wyp * XF(ixf,iyf+1,izf) )
     &                 / ( wym + wyp)
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 5*(CX2-CX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 5*(CX2-CX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
#endif

      END IF

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along Z-axis
*-----------------------------------------------------------------------

      IF (gmmlc3) THEN

         DO izc=MZ1,MZ2
            izf = (izc-1)*idz
            DO iyc=CY1,CY2
               iyf = (iyc-1)*idy+1
               DO ixc=CX1,CX2
                  ixf = (ixc-1)*idx+1
                  wzm = ABS(AF6(ixf,iyf,izf-1))
                  wzp = ABS(AF3(ixf,iyf,izf))
                  IF (wzm.LT.mmSmall) wzm = 1.0D0
                  IF (wzp.LT.mmSmall) wzp = 1.0D0
                  XF(ixf,iyf,izf) =
     &                 ( wzm * XF(ixf,iyf,izf-1)
     &                 + wzp * XF(ixf,iyf,izf+1) )
     &                 / ( wzm + wzp )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 5*(CX2-CX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 5*(CX2-CX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
#endif

      END IF

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in X-Y plane
*-----------------------------------------------------------------------

      IF (gmmlc1 .AND. gmmlc2) THEN

         DO izc=CZ1,CZ2
            izf = (izc-1)*idz+1
            DO iyc=MY1,MY2
               iyf = (iyc-1)*idy
               DO ixc=MX1,MX2
                  ixf = (ixc-1)*idx
                  wxm = ABS(AF4(ixf-1,iyf,  izf))
                  wxp = ABS(AF1(ixf,  iyf,  izf))
                  wym = ABS(AF5(ixf,  iyf-1,izf))
                  wyp = ABS(AF2(ixf,  iyf  ,izf))
                  IF (wxm.LT.mmSmall) wxm = 1.0D0
                  IF (wxp.LT.mmSmall) wxp = 1.0D0
                  IF (wym.LT.mmSmall) wym = 1.0D0
                  IF (wyp.LT.mmSmall) wyp = 1.0D0
                  XF(ixf,iyf,izf) =
     &                 ( wxm * XF(ixf-1,iyf,  izf)
     &                 + wxp * XF(ixf+1,iyf,  izf)
     &                 + wym * XF(ixf,  iyf-1,izf)
     &                 + wyp * XF(ixf,  iyf+1,izf) )
     &                 /( wxm + wxp + wym + wyp )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 11*(MX2-MX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 11*(MX2-MX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
#endif

      END IF

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in Y-Z plane
*-----------------------------------------------------------------------

      IF (gmmlc2 .AND. gmmlc3) THEN

         DO izc=MZ1,MZ2
            izf = (izc-1)*idz
            DO iyc=MY1,MY2
               iyf = (iyc-1)*idy
               DO ixc=CX1,CX2
                  ixf = (ixc-1)*idx+1
                  wym = ABS(AF5(ixf,iyf-1,izf))
                  wyp = ABS(AF2(ixf,iyf,  izf))
                  wzm = ABS(AF6(ixf,iyf,  izf-1))
                  wzp = ABS(AF3(ixf,iyf,  izf))
                  IF (wym.LT.mmSmall) wym = 1.0D0
                  IF (wyp.LT.mmSmall) wyp = 1.0D0
                  IF (wzm.LT.mmSmall) wzm = 1.0D0
                  IF (wzp.LT.mmSmall) wzp = 1.0D0
                  XF(ixf,iyf,izf) =
     &                 ( wym * XF(ixf,iyf-1,izf)
     &                 + wyp * XF(ixf,iyf+1,izf)
     &                 + wzm * XF(ixf,iyf,  izf-1)
     &                 + wzp * XF(ixf,iyf,  izf+1))
     &                 /( wym + wyp + wzm + wzp )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 11*(CX2-CX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 11*(CX2-CX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
#endif

      END IF

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in X-Z plane
*-----------------------------------------------------------------------

      IF (gmmlc1 .AND. gmmlc3) THEN

         DO izc=MZ1,MZ2
            izf = (izc-1)*idz
            DO iyc=CY1,CY2
               iyf = (iyc-1)*idy+1
               DO ixc=MX1,MX2
                  ixf = (ixc-1)*idx
                  wxm = ABS(AF4(ixf-1,iyf,izf))
                  wxp = ABS(AF1(ixf,  iyf,izf))
                  wzm = ABS(AF6(ixf,  iyf,izf-1))
                  wzp = ABS(AF3(ixf,  iyf,izf))
                  IF (wxm.LT.mmSmall) wxm = 1.0D0
                  IF (wxp.LT.mmSmall) wxp = 1.0D0
                  IF (wzm.LT.mmSmall) wzm = 1.0D0
                  IF (wzp.LT.mmSmall) wzp = 1.0D0
                  XF(ixf,iyf,izf) =
     &                 ( wxm * XF(ixf-1,iyf,izf)
     &                 + wxp * XF(ixf+1,iyf,izf)
     &                 + wzm * XF(ixf,  iyf,izf-1)
     &                 + wzp * XF(ixf,  iyf,izf+1))
     &                 /( wxm + wxp + wzm + wzp )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 11*(MX2-MX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 11*(MX2-MX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
#endif

      END IF

*-----------------------------------------------------------------------
*     Fine-grid points between eight coarse-grid points
*-----------------------------------------------------------------------

      IF (gmmlc1 .AND. gmmlc2 .AND. gmmlc3) THEN

         DO izc=MZ1,MZ2
            izf = (izc-1)*idz
            DO iyc=MY1,MY2
               iyf = (iyc-1)*idy
               DO ixc=MX1,MX2
                  ixf = (ixc-1)*idx
                  wxm = ABS(AF4(ixf-1,iyf,izf))
                  wxp = ABS(AF1(ixf,  iyf,izf))
                  wym = ABS(AF5(ixf,iyf-1,izf))
                  wyp = ABS(AF2(ixf,iyf,  izf))
                  wzm = ABS(AF6(ixf,  iyf,izf-1))
                  wzp = ABS(AF3(ixf,  iyf,izf))
                  IF (wxm.LT.mmSmall) wxm = 1.0D0
                  IF (wxp.LT.mmSmall) wxp = 1.0D0
                  IF (wym.LT.mmSmall) wym = 1.0D0
                  IF (wyp.LT.mmSmall) wyp = 1.0D0
                  IF (wzm.LT.mmSmall) wzm = 1.0D0
                  IF (wzp.LT.mmSmall) wzp = 1.0D0
                  XF(ixf,iyf,izf) =
     &                 ( wxm * XF(ixf-1,iyf,  izf)
     &                 + wxp * XF(ixf+1,iyf,  izf)
     &                 + wym * XF(ixf,  iyf-1,izf)
     &                 + wyp * XF(ixf,  iyf+1,izf)
     &                 + wzm * XF(ixf,  iyf,  izf-1)
     &                 + wzp * XF(ixf,  iyf,  izf+1))
     &                 /( wxm + wxp + wym + wyp + wzm + wzp )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 17*(MX2-MX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 17*(MX2-MX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
#endif

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

 10   CONTINUE

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmProlongSCU'
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  mmProlongSCLin
C     
C     Purpose: Transfer a 3-D grid function to the next semi-refined grid 
C     
C     Algorithm: 5-point or 3-point operator-based prolongation
C     
C     Inputs:  XC: coarse grid function
C     
C     Outputs: XF: fine grid function
C     
C     Globals accessed:
C     
C     Globals modified:
C     
C     Calls:
C     
C     Called by:
C     
C     Flop count estimate:
C     
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C     
C     Modification history: 
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmProlongSCLin (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxf,nyf,nzf,nxc,nyc,nzc,
     &     hxf,hyf,hzf,comm,nbr,bndc,level,lUpdate,icall)

      IMPLICIT NONE

      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      
      INTEGER nxdxc,nydxc,nzdxc
      INTEGER bndxc(3)  
      REAL XC(nxdxc,nydxc,nzdxc)
      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      REAL hxf(nxf),hyf(nyf),hzf(nzf)
      INTEGER comm,nbr(6),bndc(3),level
      LOGICAL lUpdate

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

************************************************************************
*     Operator-based prolong XC to XF.  
*     Assumes a uniform grid (h(i) .EQ. h(i+1))
*************************************************************************

      INTEGER ixc,iyc,izc,ixf,iyf,izf
      INTEGER idx,idy,idz
      INTEGER CX1,CY1,CZ1,CX2,CY2,CZ2
      INTEGER DX1,DY1,DZ1,DX2,DY2,DZ2
      INTEGER MX1,MY1,MZ1,MX2,MY2,MZ2
      INTEGER idim

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_PROLONG+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmProlongSCLin'
#endif

      DX1=1
      DY1=1
      DZ1=1
      DX2=nxc-1
      DY2=nyc-1
      DZ2=nzc-1

      CX1=1
      CY1=1
      CZ1=1
      CX2=nxc-1
      CY2=nyc-1
      CZ2=nzc-1
      
      MX1=2
      MY1=2
      MZ1=2
      MX2=nxc-1
      MY2=nyc-1
      MZ2=nzc-1

      IF (ipx.EQ.npx-1) THEN
         CX2 = CX2 + 1
         MX2 = MX2 + 1
         DX2 = DX2 + 1
      END IF
      IF (ipy.EQ.npy-1) THEN
         CY2 = CY2 + 1
         MY2 = MY2 + 1
         DY2 = DY2 + 1
      END IF
      IF (ipz.EQ.npz-1) THEN
         CZ2 = CZ2 + 1
         MZ2 = MZ2 + 1
         DZ2 = DZ2 + 1
      END IF

      idim = 0

      IF (gmmlc1) idim = idim + 1
      IF (gmmlc2) idim = idim + 1
      IF (gmmlc3) idim = idim + 1

      IF (idim.EQ.3) THEN

         PRINT*, '*** MGMPI ***: mmProlongSCLin called as full'
         PRINT*, '               coarsening restriction!'
         PRINT*, '               Calling mmProlongLin...'
         CALL mmProlongLin (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxf,nyf,nzf,nxc,nyc,nzc,
     &        hxf,hyf,hzf,comm,nbr,bndc,level,lUpdate,icall)

         GOTO 10

      END IF
      
*-----------------------------------------------------------------------
*     Refresh coarse-points on open boundaries
*-----------------------------------------------------------------------

      IF (gmmlc1) CALL mmGetX1Ghost (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
      IF (gmmlc2) CALL mmGetY1Ghost (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
      IF (gmmlc3) CALL mmGetZ1Ghost (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

*-----------------------------------------------------------------------
*     Fine-grid points that match coarse-grid points
*-----------------------------------------------------------------------

      idx = 1
      idy = 1
      idz = 1
      IF (gmmlc1) idx = 2 
      IF (gmmlc2) idy = 2 
      IF (gmmlc3) idz = 2

      DO izc=DZ1,DZ2
         izf = (izc-1)*idz+1
         DO iyc=DY1,DY2
            iyf = (iyc-1)*idy+1
            DO ixc=DX1,DX2
               ixf = (ixc-1)*idx+1
               XF(ixf,iyf,izf) = XC(ixc,iyc,izc)
            END DO
         END DO
      END DO

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along X-axis
*-----------------------------------------------------------------------

      IF (gmmlc1) THEN

         DO izc=CZ1,CZ2
            izf = (izc-1)*idz+1
            DO iyc=CY1,CY2
               iyf = (iyc-1)*idy+1
               DO ixc=MX1,MX2
                  ixf = (ixc-1)*idx
                  XF(ixf,iyf,izf) = 
     &                 o2*( XF(ixf-1,iyf,izf) 
     &                 +    XF(ixf+1,iyf,izf) )
               END DO
            END DO
         END DO
         
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(MX2-MX1-1)*(CY2-CY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 2*(MX2-MX1-1)*(CY2-CY1-1)*(CZ2-CZ1-1)
#endif

      END IF

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along Y-axis
*-----------------------------------------------------------------------

      IF (gmmlc2) THEN

         DO izc=CZ1,CZ2
            izf = (izc-1)*idz+1
            DO iyc=MY1,MY2
               iyf = (iyc-1)*idy
               DO ixc=CX1,CX2
                  ixf = (ixc-1)*idx+1
                  XF(ixf,iyf,izf) =
     &                 o2*( XF(ixf,iyf-1,izf)
     &                 +    XF(ixf,iyf+1,izf) )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(CX2-CX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 2*(CX2-CX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
#endif

      END IF

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along Z-axis
*-----------------------------------------------------------------------

      IF (gmmlc3) THEN

         DO izc=MZ1,MZ2
            izf = (izc-1)*idz
            DO iyc=CY1,CY2
               iyf = (iyc-1)*idy+1
               DO ixc=CX1,CX2
                  ixf = (ixc-1)*idx+1
                  XF(ixf,iyf,izf) =
     &                 o2*( XF(ixf,iyf,izf-1)
     &                 +    XF(ixf,iyf,izf+1) )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(CX2-CX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 2*(CX2-CX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
#endif

      END IF

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in X-Y plane
*-----------------------------------------------------------------------

      IF (gmmlc1 .AND. gmmlc2) THEN

         DO izc=CZ1,CZ2
            izf = (izc-1)*idz+1
            DO iyc=MY1,MY2
               iyf = (iyc-1)*idy
               DO ixc=MX1,MX2
                  ixf = (ixc-1)*idx
                  XF(ixf,iyf,izf) =
     &                 o4*( XF(ixf-1,iyf,  izf)
     &                 +    XF(ixf+1,iyf,  izf)
     &                 +    XF(ixf,  iyf-1,izf)
     &                 +    XF(ixf,  iyf+1,izf) )
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 4*(MX2-MX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 4*(MX2-MX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
#endif

      END IF

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in Y-Z plane
*-----------------------------------------------------------------------

      IF (gmmlc2 .AND. gmmlc3) THEN

         DO izc=MZ1,MZ2
            izf = (izc-1)*idz
            DO iyc=MY1,MY2
               iyf = (iyc-1)*idy
               DO ixc=CX1,CX2
                  ixf = (ixc-1)*idx+1
                  XF(ixf,iyf,izf) =
     &                 o4*( XF(ixf,iyf-1,izf)
     &                 +    XF(ixf,iyf+1,izf)
     &                 +    XF(ixf,iyf,  izf-1)
     &                 +    XF(ixf,iyf,  izf+1))
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 4*(CX2-CX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 4*(CX2-CX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
#endif

      END IF

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in X-Z plane
*-----------------------------------------------------------------------

      IF (gmmlc1 .AND. gmmlc3) THEN

         DO izc=MZ1,MZ2
            izf = (izc-1)*idz
            DO iyc=CY1,CY2
               iyf = (iyc-1)*idy+1
               DO ixc=MX1,MX2
                  ixf = (ixc-1)*idx
                  XF(ixf,iyf,izf) =
     &                 o4*( XF(ixf-1,iyf,izf)
     &                 +    XF(ixf+1,iyf,izf)
     &                 +    XF(ixf,  iyf,izf-1)
     &                 +    XF(ixf,  iyf,izf+1))
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 4*(MX2-MX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 4*(MX2-MX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
#endif

      END IF

*-----------------------------------------------------------------------
*     Fine-grid points between eight coarse-grid points
*-----------------------------------------------------------------------

      IF (gmmlc1 .AND. gmmlc2 .AND. gmmlc3) THEN

         DO izc=MZ1,MZ2
            izf = (izc-1)*idz
            DO iyc=MY1,MY2
               iyf = (iyc-1)*idy
               DO ixc=MX1,MX2
                  ixf = (ixc-1)*idx
                  XF(ixf,iyf,izf) =
     &                 o6*( XF(ixf-1,iyf,  izf)
     &                 +    XF(ixf+1,iyf,  izf)
     &                 +    XF(ixf,  iyf-1,izf)
     &                 +    XF(ixf,  iyf+1,izf)
     &                 +    XF(ixf,  iyf,  izf-1)
     &                 +    XF(ixf,  iyf,  izf+1))
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 6*(MX2-MX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 6*(MX2-MX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
#endif

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

 10   CONTINUE

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmProlongSCLin'
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  mmProlongSCGen
C     
C     Purpose: Transfer a 3-D grid function to the next semi-refined grid 
C     
C     Algorithm: 5-point or 3-point operator-based prolongation
C     
C     Inputs:  XC: coarse grid function
C     
C     Outputs: XF: fine grid function
C     
C     Globals accessed:
C     
C     Globals modified:
C     
C     Calls:
C     
C     Called by:
C     
C     Flop count estimate:
C     
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C     
C     Modification history: 
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmProlongSCGen (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,nxf,nyf,nzf,nxc,nyc,nzc,
     &     hxf,hyf,hzf,comm,nbr,bndc,level,lUpdate,icall)

      IMPLICIT NONE

      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      
      INTEGER nxdxc,nydxc,nzdxc
      INTEGER bndxc(3)  
      REAL XC(nxdxc,nydxc,nzdxc)
      
      INTEGER nxdaf,nydaf,nzdaf
      INTEGER bndaf(3)  
      REAL AFD(nxdaf,nydaf,nzdaf)
      REAL AF1(nxdaf,nydaf,nzdaf)
      REAL AF2(nxdaf,nydaf,nzdaf)
      REAL AF3(nxdaf,nydaf,nzdaf)
      REAL AF4(nxdaf,nydaf,nzdaf)
      REAL AF5(nxdaf,nydaf,nzdaf)
      REAL AF6(nxdaf,nydaf,nzdaf)
      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      REAL hxf(nxf),hyf(nyf),hzf(nzf)
      INTEGER comm,nbr(6),bndc(3),level
      LOGICAL lUpdate

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

************************************************************************
*     Operator-based prolong XC to XF.  
*     Assumes a uniform grid (h(i) .EQ. h(i+1))
*************************************************************************

      INTEGER ixc,iyc,izc,ixf,iyf,izf
      INTEGER idx,idy,idz
      INTEGER CX1,CY1,CZ1,CX2,CY2,CZ2
      INTEGER DX1,DY1,DZ1,DX2,DY2,DZ2
      INTEGER MX1,MY1,MZ1,MX2,MY2,MZ2
      INTEGER idim
      REAL wxm,wxp,wym,wyp,wzm,wzp

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_PROLONG+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmProlongSCGen'
#endif

      DX1=1
      DY1=1
      DZ1=1
      DX2=nxc-1
      DY2=nyc-1
      DZ2=nzc-1

      CX1=1
      CY1=1
      CZ1=1
      CX2=nxc-1
      CY2=nyc-1
      CZ2=nzc-1
      
      MX1=2
      MY1=2
      MZ1=2
      MX2=nxc-1
      MY2=nyc-1
      MZ2=nzc-1

      IF (ipx.EQ.npx-1) THEN
         CX2 = CX2 + 1
         MX2 = MX2 + 1
         DX2 = DX2 + 1
      END IF
      IF (ipy.EQ.npy-1) THEN
         CY2 = CY2 + 1
         MY2 = MY2 + 1
         DY2 = DY2 + 1
      END IF
      IF (ipz.EQ.npz-1) THEN
         CZ2 = CZ2 + 1
         MZ2 = MZ2 + 1
         DZ2 = DZ2 + 1
      END IF

      idim = 0

      IF (gmmlc1) idim = idim + 1
      IF (gmmlc2) idim = idim + 1
      IF (gmmlc3) idim = idim + 1

      IF (idim.EQ.3) THEN

         PRINT*, '*** MGMPI ***: mmProlongSCGen called as full'
         PRINT*, '               coarsening restriction!'
         PRINT*, '               Calling mmProlongGen...'
         CALL mmProlongGen (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     AFD,AF1,AF2,AF3,AF4,AF5,AF6,nxdaf,nydaf,nzdaf,bndaf
     &     ,nxf,nyf,nzf,nxc,nyc,nzc,
     &        hxf,hyf,hzf,comm,nbr,bndc,level,lUpdate,icall)

         GOTO 10

      END IF
      
*-----------------------------------------------------------------------
*     Refresh coarse-points on open boundaries
*-----------------------------------------------------------------------

      IF (gmmlc1) CALL mmGetX1Ghost (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
      IF (gmmlc2) CALL mmGetY1Ghost (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)
      IF (gmmlc3) CALL mmGetZ1Ghost (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,
     &     comm,nbr,bndc
     &     ,isub)

*-----------------------------------------------------------------------
*     Fine-grid points that match coarse-grid points
*-----------------------------------------------------------------------

      idx = 1
      idy = 1
      idz = 1
      IF (gmmlc1) idx = 2 
      IF (gmmlc2) idy = 2 
      IF (gmmlc3) idz = 2

      DO izc=DZ1,DZ2
         izf = (izc-1)*idz+1
         DO iyc=DY1,DY2
            iyf = (iyc-1)*idy+1
            DO ixc=DX1,DX2
               ixf = (ixc-1)*idx+1
               XF(ixf,iyf,izf) = XC(ixc,iyc,izc)
            END DO
         END DO
      END DO

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along X-axis
*-----------------------------------------------------------------------

      IF (gmmlc1) THEN

      DO izc=CZ1,CZ2
         izf = (izc-1)*idz+1
         DO iyc=CY1,CY2
            iyf = (iyc-1)*idy+1
            DO ixc=MX1,MX2
               ixf = (ixc-1)*idx
               wxm = ABS(AF4(ixf-1,iyf,izf) * gdxi(ixf-1,level))
               wxp = ABS(AF1(ixf,  iyf,izf) * gdxi(ixf,  level))
               IF (wxm.LT.mmSmall) wxm = 1.0D0
               IF (wxp.LT.mmSmall) wxp = 1.0D0
               XF(ixf,iyf,izf) = 
     &              ( wxm * XF(ixf-1,iyf,izf) 
     &              + wxp * XF(ixf+1,iyf,izf) )
     &              / ( wxm + wxp )
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 7*(MX2-MX1-1)*(CY2-CY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 7*(MX2-MX1-1)*(CY2-CY1-1)*(CZ2-CZ1-1)
#endif

      END IF

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along Y-axis
*-----------------------------------------------------------------------

      IF (gmmlc2) THEN

      DO izc=CZ1,CZ2
         izf = (izc-1)*idz+1
         DO iyc=MY1,MY2
            iyf = (iyc-1)*idy
            DO ixc=CX1,CX2
               ixf = (ixc-1)*idx+1
               wym = ABS(AF5(ixf,iyf-1,izf) * gdyi(iyf-1,level))
               wyp = ABS(AF2(ixf,iyf,  izf) * gdyi(iyf,  level))
               IF (wym.LT.mmSmall) wym = 1.0D0
               IF (wyp.LT.mmSmall) wyp = 1.0D0
               XF(ixf,iyf,izf) =
     &              ( wym * XF(ixf,iyf-1,izf)
     &              + wyp * XF(ixf,iyf+1,izf) )
     &              / ( wym + wyp)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 7*(CX2-CX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 7*(CX2-CX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
#endif

      END IF

*-----------------------------------------------------------------------
*     Fine-grid points between two coarse-grid points along Z-axis
*-----------------------------------------------------------------------

      IF (gmmlc3) THEN

      DO izc=MZ1,MZ2
         izf = (izc-1)*idz
         DO iyc=CY1,CY2
            iyf = (iyc-1)*idy+1
            DO ixc=CX1,CX2
               ixf = (ixc-1)*idx+1
               wzm = ABS(AF6(ixf,iyf,izf-1) * gdzi(izf-1,level))
               wzp = ABS(AF3(ixf,iyf,izf)   * gdzi(izf,  level))
               IF (wzm.LT.mmSmall) wzm = 1.0D0
               IF (wzp.LT.mmSmall) wzp = 1.0D0
               XF(ixf,iyf,izf) =
     &              ( wzm * XF(ixf,iyf,izf-1)
     &              + wzp * XF(ixf,iyf,izf+1) )
     &              / ( wzm + wzp )
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 7*(CX2-CX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 7*(CX2-CX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
#endif

      END IF

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in X-Y plane
*-----------------------------------------------------------------------

      IF (gmmlc1 .AND. gmmlc2) THEN

         DO izc=CZ1,CZ2
         izf = (izc-1)*idz+1
         DO iyc=MY1,MY2
            iyf = (iyc-1)*idy
            DO ixc=MX1,MX2
               ixf = (ixc-1)*idx
               wxm = ABS(AF4(ixf-1,iyf,  izf) * gdxi(ixf-1,level))
               wxp = ABS(AF1(ixf,  iyf,  izf) * gdxi(ixf,  level))
               wym = ABS(AF5(ixf,  iyf-1,izf) * gdyi(iyf-1,level))
               wyp = ABS(AF2(ixf,  iyf  ,izf) * gdyi(iyf,  level))
               IF (wxm.LT.mmSmall) wxm = 1.0D0
               IF (wxp.LT.mmSmall) wxp = 1.0D0
               IF (wym.LT.mmSmall) wym = 1.0D0
               IF (wyp.LT.mmSmall) wyp = 1.0D0
               XF(ixf,iyf,izf) =
     &              ( wxm * XF(ixf-1,iyf,  izf)
     &              + wxp * XF(ixf+1,iyf,  izf)
     &              + wym * XF(ixf,  iyf-1,izf)
     &              + wyp * XF(ixf,  iyf+1,izf) )
     &              /( wxm + wxp + wym + wyp )
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 15*(MX2-MX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 15*(MX2-MX1-1)*(MY2-MY1-1)*(CZ2-CZ1-1)
#endif

      END IF

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in Y-Z plane
*-----------------------------------------------------------------------

      IF (gmmlc2 .AND. gmmlc3) THEN

      DO izc=MZ1,MZ2
         izf = (izc-1)*idz
         DO iyc=MY1,MY2
            iyf = (iyc-1)*idy
            DO ixc=CX1,CX2
               ixf = (ixc-1)*idx+1
               wym = ABS(AF5(ixf,iyf-1,izf)   * gdyi(iyf-1,level))
               wyp = ABS(AF2(ixf,iyf,  izf)   * gdyi(iyf,  level))
               wzm = ABS(AF6(ixf,iyf,  izf-1) * gdzi(izf-1,level))
               wzp = ABS(AF3(ixf,iyf,  izf)   * gdzi(izf,  level))
               IF (wym.LT.mmSmall) wym = 1.0D0
               IF (wyp.LT.mmSmall) wyp = 1.0D0
               IF (wzm.LT.mmSmall) wzm = 1.0D0
               IF (wzp.LT.mmSmall) wzp = 1.0D0
               XF(ixf,iyf,izf) =
     &              ( wym * XF(ixf,iyf-1,izf)
     &              + wyp * XF(ixf,iyf+1,izf)
     &              + wzm * XF(ixf,iyf,  izf-1)
     &              + wzp * XF(ixf,iyf,  izf+1))
     &              /( wym + wyp + wzm + wzp )
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 15*(CX2-CX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 15*(CX2-CX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
#endif

      END IF

*-----------------------------------------------------------------------
*     Fine-grid points between four coarse-grid points in X-Z plane
*-----------------------------------------------------------------------

      IF (gmmlc1 .AND. gmmlc3) THEN

      DO izc=MZ1,MZ2
         izf = (izc-1)*idz
         DO iyc=CY1,CY2
            iyf = (iyc-1)*idy+1
            DO ixc=MX1,MX2
               ixf = (ixc-1)*idx
               wxm = ABS(AF4(ixf-1,iyf,izf)   * gdxi(ixf-1,level))
               wxp = ABS(AF1(ixf,  iyf,izf)   * gdxi(ixf,  level))
               wzm = ABS(AF6(ixf,  iyf,izf-1) * gdzi(izf-1,level))
               wzp = ABS(AF3(ixf,  iyf,izf)   * gdzi(izf,  level))
               IF (wxm.LT.mmSmall) wxm = 1.0D0
               IF (wxp.LT.mmSmall) wxp = 1.0D0
               IF (wzm.LT.mmSmall) wzm = 1.0D0
               IF (wzp.LT.mmSmall) wzp = 1.0D0
               XF(ixf,iyf,izf) =
     &              ( wxm * XF(ixf-1,iyf,izf)
     &              + wxp * XF(ixf+1,iyf,izf)
     &              + wzm * XF(ixf,  iyf,izf-1)
     &              + wzp * XF(ixf,  iyf,izf+1))
     &              /( wxm + wxp + wzm + wzp )
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 15*(MX2-MX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 15*(MX2-MX1-1)*(CY2-CY1-1)*(MZ2-MZ1-1)
#endif

      END IF

*-----------------------------------------------------------------------
*     Fine-grid points between eight coarse-grid points
*-----------------------------------------------------------------------

      IF (gmmlc1 .AND. gmmlc2 .AND. gmmlc3) THEN

      DO izc=MZ1,MZ2
         izf = (izc-1)*idz
         DO iyc=MY1,MY2
            iyf = (iyc-1)*idy
            DO ixc=MX1,MX2
               ixf = (ixc-1)*idx
               wxm = ABS(AF4(ixf-1,iyf,izf)   * gdxi(ixf-1,level))
               wxp = ABS(AF1(ixf,  iyf,izf)   * gdxi(ixf,  level))
               wym = ABS(AF5(ixf,iyf-1,izf)   * gdyi(iyf-1,level))
               wyp = ABS(AF2(ixf,iyf,  izf)   * gdyi(iyf,  level))
               wzm = ABS(AF6(ixf,  iyf,izf-1) * gdzi(izf-1,level))
               wzp = ABS(AF3(ixf,  iyf,izf)   * gdzi(izf,  level))
               IF (wxm.LT.mmSmall) wxm = 1.0D0
               IF (wxp.LT.mmSmall) wxp = 1.0D0
               IF (wym.LT.mmSmall) wym = 1.0D0
               IF (wyp.LT.mmSmall) wyp = 1.0D0
               IF (wzm.LT.mmSmall) wzm = 1.0D0
               IF (wzp.LT.mmSmall) wzp = 1.0D0
               XF(ixf,iyf,izf) =
     &              ( wxm * XF(ixf-1,iyf,  izf)
     &              + wxp * XF(ixf+1,iyf,  izf)
     &              + wym * XF(ixf,  iyf-1,izf)
     &              + wyp * XF(ixf,  iyf+1,izf)
     &              + wzm * XF(ixf,  iyf,  izf-1)
     &              + wzp * XF(ixf,  iyf,  izf+1))
     &              /( wxm + wxp + wym + wyp + wzm + wzp )
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 23*(MX2-MX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
      gmflops(isub)=gmflops(isub) + 
     & 23*(MX2-MX1-1)*(MY2-MY1-1)*(MZ2-MZ1-1)
#endif

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

 10   CONTINUE

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmProlongSCGen'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmRefreshBC
C     
C     PURPOSE: Refresh Neumann and periodic boundary conditions
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmRefreshBC (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz,level
      REAL scaleA,scaleB

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz,ierr
      REAL scale,t1,t2
      
#ifdef MPI_USED
      INTEGER status(MPI_STATUS_SIZE)
#else 
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_REFRESH
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      scale = scaleB / scaleA

*----------------------------------------
*     Lower X-boundary Neumann BCs
*----------------------------------------

      IF (ipx.EQ.0) THEN
         IF (ABS(ixmb).EQ.MM_NEUMANN .AND. level .GT. 1) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(1,iy,iz) = X(2,iy,iz)
               END DO
            END DO
         ELSE IF (ixmb.EQ.MM_NEUMANN) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(1,iy,iz) = X(2,iy,iz) - scale*hx(2)*XMB(iy,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*ny*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*ny*nz
#endif
         ELSE IF (ixmb.EQ.-MM_NEUMANN) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(1,iy,iz) = X(2,iy,iz) + scale*hx(2)*XMB(iy,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*ny*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*ny*nz
#endif
         END IF
      END IF

*----------------------------------------
*     Upper X-boundary Neumann BCs
*----------------------------------------

      IF (ipx.EQ.npx-1) THEN
         IF (ABS(ixpb).EQ.MM_NEUMANN .AND. level .GT. 1) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(nx,iy,iz) = X(nx-1,iy,iz)
               END DO
            END DO
         ELSE IF (ixpb.EQ.MM_NEUMANN) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(nx,iy,iz) = X(nx-1,iy,iz) + scale*hx(nx)*XPB(iy,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*ny*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*ny*nz
#endif
         ELSE IF (ixpb.EQ.-MM_NEUMANN) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(nx,iy,iz) = X(nx-1,iy,iz) - scale*hx(nx)*XPB(iy,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*ny*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*ny*nz
#endif
         END IF
      END IF

*----------------------------------------
*     Lower Y-boundary Neumann BCs
*----------------------------------------

      IF (ipy.EQ.0) THEN
         IF (ABS(iymb).EQ.MM_NEUMANN .AND. level .GT. 1) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,1,iz) = X(ix,2,iz)
               END DO
            END DO
         ELSE IF (iymb.EQ.MM_NEUMANN) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,1,iz) = X(ix,2,iz) - scale*hy(2)*YMB(ix,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*nx*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*nx*nz
#endif
         ELSE IF (iymb.EQ.-MM_NEUMANN) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,1,iz) = X(ix,2,iz) + scale*hy(2)*YMB(ix,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*nx*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*nx*nz
#endif
         END IF
      END IF

*----------------------------------------
*     Upper Y-boundary Neumann BCs
*----------------------------------------

      IF (ipy.EQ.npy-1) THEN
         IF (ABS(iypb).EQ.MM_NEUMANN .AND. level .GT. 1) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,ny,iz) = X(ix,ny-1,iz)
               END DO
            END DO
         ELSE IF (iypb.EQ.MM_NEUMANN) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,ny,iz) = X(ix,ny-1,iz) + scale*hy(ny)*YPB(ix,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*nx*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*nx*nz
#endif
         ELSE IF (iypb.EQ.-MM_NEUMANN) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,ny,iz) = X(ix,ny-1,iz) - scale*hy(ny)*YPB(ix,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*nx*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*nx*nz
#endif
         END IF
      END IF

*----------------------------------------
*     Lower Z-boundary Neumann BCs
*----------------------------------------

      IF (ipz.EQ.0) THEN
         IF (ABS(izmb).EQ.MM_NEUMANN .AND. level .GT. 1) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,1) = X(ix,iy,2)
               END DO
            END DO
         ELSE IF (izmb.EQ.MM_NEUMANN) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,1) = X(ix,iy,2) - scale*hz(2)*ZMB(ix,iy)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*ny*nx
      gmflops(isub)=gmflops(isub) + 
     & 3*ny*nx
#endif
         ELSE IF (izmb.EQ.-MM_NEUMANN) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,1) = X(ix,iy,2) + scale*hz(2)*ZMB(ix,iy)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*ny*nx
      gmflops(isub)=gmflops(isub) + 
     & 3*ny*nx
#endif
         END IF
      END IF

*----------------------------------------
*     Upper Z-boundary Neumann BCs
*----------------------------------------

      IF (ipz.EQ.npz-1) THEN
         IF (ABS(izpb).EQ.MM_NEUMANN .AND. level .GT. 1) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,nz) = X(ix,iy,nz-1)
               END DO
            END DO
         ELSE IF (izpb.EQ.MM_NEUMANN) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,nz) = X(ix,iy,nz-1) + scale*hz(nz)*ZPB(ix,iy)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*ny*nx
      gmflops(isub)=gmflops(isub) + 
     & 3*ny*nx
#endif
         ELSE IF (izpb.EQ.-MM_NEUMANN) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,nz) = X(ix,iy,nz-1) - scale*hz(nz)*ZPB(ix,iy)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*ny*nx
      gmflops(isub)=gmflops(isub) + 
     & 3*ny*nx
#endif
         END IF
      END IF

*------------------------------
*     X-boundary periodic BCs
*------------------------------

*      t1 = 1.0D0
      t1 = 2**(level-1.0)/(2**level-1.0)
      t2 = 1.0D0 - t1

      IF (ixmb.EQ.MM_PERIODIC) THEN

         IF (npx.EQ.1) THEN

            DO iz=1,nz
               DO iy=1,ny
                  X(1, iy,iz) = t1*X(nx-1,iy,iz) + t2*X(   2,iy,iz)
                  X(nx,iy,iz) = t1*X(   2,iy,iz) + t2*X(nx-1,iy,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 6*ny*nz
      gmflops(isub)=gmflops(isub) + 
     & 6*ny*nz
#endif

#ifdef MPI_USED

         ELSE

            IF (ipx.EQ.0) THEN
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmRefreshBC: MPI_Sendrecv')") ip
#endif
               CALL MPI_Sendrecv (
     &              X(2,1,1),1,bndx(XD),nbr(XMD),80+XMD,
     &              X(1,1,1),1,bndx(XD),nbr(XMD),80+XPD,
     &              comm,status,ierr)
               IF (ierr.NE.0) print*, 'MPI Error in mmRefreshBc: ',ierr
               DO iz=1,nz
                  DO iy=1,ny
                     X(1,iy,iz) = t1*X(1,iy,iz) + t2*X(2,iy,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*ny*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*ny*nz
#endif
            END IF
            IF (ipx.EQ.npx-1) THEN
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmRefreshBC: MPI_Sendrecv')") ip
#endif
               CALL MPI_Sendrecv (
     &              X(nx-1,1,1),1,bndx(XD),nbr(XPD),80+XPD,
     &              X(nx,  1,1),1,bndx(XD),nbr(XPD),80+XMD,
     &              comm,status,ierr)
               IF (ierr.NE.0) print*, 'MPI Error in mmRefreshBc: ',ierr
               DO iz=1,nz
                  DO iy=1,ny
                     X(nx,iy,iz) = t1*X(nx,iy,iz) + t2*X(nx-1,iy,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*ny*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*ny*nz
#endif
            END IF
#endif
         END IF
      END IF

*------------------------------
*     Y-boundary periodic BCs
*------------------------------

      IF (iymb.EQ.MM_PERIODIC) THEN

         IF (npy.EQ.1) THEN

            DO iz=1,nz
               DO ix=1,nx
                  X(ix, 1,iz) = t1*X(ix,ny-1,iz) + t2*X(ix,   2,iz)
                  X(ix,ny,iz) = t1*X(ix,   2,iz) + t2*X(ix,ny-1,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 6*nx*nz
      gmflops(isub)=gmflops(isub) + 
     & 6*nx*nz
#endif

#ifdef MPI_USED

         ELSE

            IF (ipy.EQ.0) THEN
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmRefreshBC: MPI_Sendrecv')") ip
#endif
               CALL MPI_Sendrecv (
     &              X(1,2,1),1,bndx(YD),nbr(YMD),80+YMD,
     &              X(1,1,1),1,bndx(YD),nbr(YMD),80+YPD,
     &              comm,status,ierr)
               IF (ierr.NE.0) print*, 'MPI Error in mmRefreshBc: ',ierr
               DO iz=1,nz
                  DO ix=1,nx
                     X(ix,1,iz) = t1*X(ix,1,iz) + t2*X(ix,2,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*nx*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*nx*nz
#endif
            END IF
            IF (ipy.EQ.npy-1) THEN
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmRefreshBC: MPI_Sendrecv')") ip
#endif
               CALL MPI_Sendrecv (
     &              X(1,ny-1,1),1,bndx(YD),nbr(YPD),80+YPD,
     &              X(1,  ny,1),1,bndx(YD),nbr(YPD),80+YMD,
     &              comm,status,ierr)
               IF (ierr.NE.0) print*, 'MPI Error in mmRefreshBc: ',ierr
               DO iz=1,nz
                  DO ix=1,nx
                     X(ix,ny,iz) = t1*X(ix,ny,iz) + t2*X(ix,ny-1,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*nx*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*nx*nz
#endif
            END IF
#endif
         END IF

      END IF

*------------------------------
*     Z-boundary periodic BCs
*------------------------------

      IF (izmb.EQ.MM_PERIODIC) THEN 

         IF (npz.EQ.1) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy, 1) = t1*X(ix,iy,nz-1) + t2*X(ix,iy,   2)
                  X(ix,iy,nz) = t1*X(ix,iy,   2) + t2*X(ix,iy,nz-1)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 6*ny*nx
      gmflops(isub)=gmflops(isub) + 
     & 6*ny*nx
#endif
#ifdef MPI_USED
         ELSE
            IF (ipz.EQ.0) THEN
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmRefreshBC: MPI_Sendrecv')") ip
#endif
               CALL MPI_Sendrecv (
     &              X(1,1,2),1,bndx(ZD),nbr(ZMD),80+ZPD,
     &              X(1,1,1),1,bndx(ZD),nbr(ZMD),80+ZPD,
     &              comm,status,ierr)
               IF (ierr.NE.0) print*, 'MPI Error in mmRefreshBc: ',ierr
               DO iy=1,ny
                  DO ix=1,nx
                     X(ix,iy,1) = t1*X(ix,iy,1) + t2*X(ix,iy,2)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*ny*nx
      gmflops(isub)=gmflops(isub) + 
     & 3*ny*nx
#endif
            END IF
            IF (ipz.EQ.npz-1) THEN
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmRefreshBC: MPI_Sendrecv')") ip
#endif
               CALL MPI_Sendrecv (
     &              X(1,1,nz-1),1,bndx(ZD),nbr(ZPD),80+ZPD,
     &              X(1,1,  nz),1,bndx(ZD),nbr(ZPD),80+ZPD,
     &              comm,status,ierr)
               IF (ierr.NE.0) print*, 'MPI Error in mmRefreshBc: ',ierr
               DO iy=1,ny
                  DO ix=1,nx
                     X(ix,iy,nz) = t1*X(ix,iy,nz) + t2*X(ix,iy,nz-1)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*ny*nx
      gmflops(isub)=gmflops(isub) + 
     & 3*ny*nx
#endif
            END IF
#endif
         END IF

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmRefreshBC2
C     
C     PURPOSE: Refresh Neumann and periodic boundary conditions for 2D
C     PURPOSE: solves
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmRefreshBC2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,lx,ly,lz,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz,level
      REAL scaleA,scaleB
      LOGICAL lx,ly,lz

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz,jx,jy,jz,ierr
      REAL scale,t1,t2
      
#ifdef MPI_USED
      INTEGER status(MPI_STATUS_SIZE)
#else 
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_REFRESH
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      scale = scaleB / scaleA

      IF (.NOT.lx) jx=0
      IF (lx) jx=1
      IF (.NOT.ly) jy=0
      IF (ly) jy=1
      IF (.NOT.lz) jz=0
      IF (lz) jz=1

*----------------------------------------
*     Lower X-boundary Neumann BCs
*----------------------------------------

      IF (lx.AND.ipx.EQ.0) THEN
         IF (ABS(ixmb).EQ.MM_NEUMANN .AND. level .GT. 1) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(1,iy,iz) = X(2,iy,iz)
               END DO
            END DO
         ELSE IF (ixmb.EQ.MM_NEUMANN) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(1,iy,iz) = X(2,iy,iz) - scale*hx(2)*XMB(iy,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*ny*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*ny*nz
#endif
         ELSE IF (ixmb.EQ.-MM_NEUMANN) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(1,iy,iz) = X(2,iy,iz) + scale*hx(2)*XMB(iy,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*ny*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*ny*nz
#endif
         END IF
      END IF

*----------------------------------------
*     Upper X-boundary Neumann BCs
*----------------------------------------

      IF (lx.AND.ipx.EQ.npx-1) THEN
         IF (ABS(ixpb).EQ.MM_NEUMANN .AND. level .GT. 1) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(nx,iy,iz) = X(nx-1,iy,iz)
               END DO
            END DO
         ELSE IF (ixpb.EQ.MM_NEUMANN) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(nx,iy,iz) = X(nx-1,iy,iz) + scale*hx(nx)*XPB(iy,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*ny*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*ny*nz
#endif
         ELSE IF (ixpb.EQ.-MM_NEUMANN) THEN
            DO iz=1,nz
               DO iy=1,ny
                  X(nx,iy,iz) = X(nx-1,iy,iz) - scale*hx(nx)*XPB(iy,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*ny*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*ny*nz
#endif
         END IF
      END IF

*----------------------------------------
*     Lower Y-boundary Neumann BCs
*----------------------------------------

      IF (ly.AND.ipy.EQ.0) THEN
         IF (ABS(iymb).EQ.MM_NEUMANN .AND. level .GT. 1) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,1,iz) = X(ix,2,iz)
               END DO
            END DO
         ELSE IF (iymb.EQ.MM_NEUMANN) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,1,iz) = X(ix,2,iz) - scale*hy(2)*YMB(ix,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*nx*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*nx*nz
#endif
         ELSE IF (iymb.EQ.-MM_NEUMANN) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,1,iz) = X(ix,2,iz) + scale*hy(2)*YMB(ix,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*nx*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*nx*nz
#endif
         END IF
      END IF

*----------------------------------------
*     Upper Y-boundary Neumann BCs
*----------------------------------------

      IF (ly.AND.ipy.EQ.npy-1) THEN
         IF (ABS(iypb).EQ.MM_NEUMANN .AND. level .GT. 1) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,ny,iz) = X(ix,ny-1,iz)
               END DO
            END DO
         ELSE IF (iypb.EQ.MM_NEUMANN) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,ny,iz) = X(ix,ny-1,iz) + scale*hy(ny)*YPB(ix,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*nx*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*nx*nz
#endif
         ELSE IF (iypb.EQ.-MM_NEUMANN) THEN
            DO iz=1,nz
               DO ix=1,nx
                  X(ix,ny,iz) = X(ix,ny-1,iz) - scale*hy(ny)*YPB(ix,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*nx*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*nx*nz
#endif
         END IF
      END IF

*----------------------------------------
*     Lower Z-boundary Neumann BCs
*----------------------------------------

      IF (lz.AND.ipz.EQ.0) THEN
         IF (ABS(izmb).EQ.MM_NEUMANN .AND. level .GT. 1) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,1) = X(ix,iy,2)
               END DO
            END DO
         ELSE IF (izmb.EQ.MM_NEUMANN) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,1) = X(ix,iy,2) - scale*hz(2)*ZMB(ix,iy)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*nx*ny
      gmflops(isub)=gmflops(isub) + 
     & 3*nx*ny
#endif
         ELSE IF (izmb.EQ.-MM_NEUMANN) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,1) = X(ix,iy,2) + scale*hz(2)*ZMB(ix,iy)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*nx*ny
      gmflops(isub)=gmflops(isub) + 
     & 3*nx*ny
#endif
         END IF
      END IF

*----------------------------------------
*     Upper Z-boundary Neumann BCs
*----------------------------------------

      IF (lz.AND.ipz.EQ.npz-1) THEN
         IF (ABS(izpb).EQ.MM_NEUMANN .AND. level .GT. 1) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,nz) = X(ix,iy,nz-1)
               END DO
            END DO
         ELSE IF (izpb.EQ.MM_NEUMANN) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,nz) = X(ix,iy,nz-1) + scale*hz(nz)*ZPB(ix,iy)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*nx*ny
      gmflops(isub)=gmflops(isub) + 
     & 3*nx*ny
#endif
         ELSE IF (izpb.EQ.-MM_NEUMANN) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy,nz) = X(ix,iy,nz-1) - scale*hz(nz)*ZPB(ix,iy)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*nx*ny
      gmflops(isub)=gmflops(isub) + 
     & 3*nx*ny
#endif
         END IF
      END IF

*------------------------------
*     X-boundary periodic BCs
*------------------------------

*      t1 = 1.0D0
      t1 = 2**(level-1.0)/(2**level-1.0)
      t2 = 1.0D0 - t1

      IF (lx.AND.ixmb.EQ.MM_PERIODIC) THEN

         IF (npx.EQ.1) THEN

            DO iz=1,nz
               DO iy=1,ny
                  X(1, iy,iz) = t1*X(nx-1,iy,iz) + t2*X(   2,iy,iz)
                  X(nx,iy,iz) = t1*X(   2,iy,iz) + t2*X(nx-1,iy,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 6*ny*nz
      gmflops(isub)=gmflops(isub) + 
     & 6*ny*nz
#endif

#ifdef MPI_USED

         ELSE

            IF (ipx.EQ.0) THEN
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmRefreshBC2: MPI_Sendrecv')") ip
#endif
               CALL MPI_Sendrecv (
     &              X(2,1,1),1,bndx(XD),gsubnbr(jx,jy,jz,XMD),80+XMD,
     &              X(1,1,1),1,bndx(XD),gsubnbr(jx,jy,jz,XMD),80+XPD,
     &              gsubcomm(jx,jy,jz),status,ierr)
               IF (ierr.NE.0) print*, 'MPI Error in mmRefreshBc2: ',ierr
               DO iz=1,nz
                  DO iy=1,ny
                     X(1,iy,iz) = t1*X(1,iy,iz) + t2*X(2,iy,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*ny*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*ny*nz
#endif
            END IF
            IF (ipx.EQ.npx-1) THEN
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmRefreshBC2: MPI_Sendrecv')") ip
#endif
               CALL MPI_Sendrecv (
     &              X(nx-1,1,1),1,bndx(XD),gsubnbr(jx,jy,jz,XPD),80+XPD,
     &              X(nx,  1,1),1,bndx(XD),gsubnbr(jx,jy,jz,XPD),80+XMD,
     &              gsubcomm(jx,jy,jz),status,ierr)
               IF (ierr.NE.0) print*, 'MPI Error in mmRefreshBc2: ',ierr
               DO iz=1,nz
                  DO iy=1,ny
                     X(nx,iy,iz) = t1*X(nx,iy,iz) + t2*X(nx-1,iy,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*ny*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*ny*nz
#endif
            END IF
#endif
         END IF
      END IF

*------------------------------
*     Y-boundary periodic BCs
*------------------------------

      IF (ly.AND.iymb.EQ.MM_PERIODIC) THEN

         IF (npy.EQ.1) THEN

            DO iz=1,nz
               DO ix=1,nx
                  X(ix, 1,iz) = t1*X(ix,ny-1,iz) + t2*X(ix,   2,iz)
                  X(ix,ny,iz) = t1*X(ix,   2,iz) + t2*X(ix,ny-1,iz)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 6*nx*nz
      gmflops(isub)=gmflops(isub) + 
     & 6*nx*nz
#endif

#ifdef MPI_USED

         ELSE

            IF (ipy.EQ.0) THEN
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmRefreshBC2: MPI_Sendrecv')") ip
#endif
               CALL MPI_Sendrecv (
     &              X(1,2,1),1,bndx(YD),gsubnbr(jx,jy,jz,YMD),80+YMD,
     &              X(1,1,1),1,bndx(YD),gsubnbr(jx,jy,jz,YMD),80+YPD,
     &              gsubcomm(jx,jy,jz),status,ierr)
               IF (ierr.NE.0) print*, 'MPI Error in mmRefreshBc2: ',ierr
               DO iz=1,nz
                  DO ix=1,nx
                     X(ix,1,iz) = t1*X(ix,1,iz) + t2*X(ix,2,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*nx*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*nx*nz
#endif
            END IF
            IF (ipy.EQ.npy-1) THEN
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmRefreshBC2: MPI_Sendrecv')") ip
#endif
               CALL MPI_Sendrecv (
     &              X(1,ny-1,1),1,bndx(YD),gsubnbr(jx,jy,jz,YPD),80+YPD,
     &              X(1,  ny,1),1,bndx(YD),gsubnbr(jx,jy,jz,YPD),80+YMD,
     &              gsubcomm(jx,jy,jz),status,ierr)
               IF (ierr.NE.0) print*, 'MPI Error in mmRefreshBc2: ',ierr
               DO iz=1,nz
                  DO ix=1,nx
                     X(ix,ny,iz) = t1*X(ix,ny,iz) + t2*X(ix,ny-1,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*nx*nz
      gmflops(isub)=gmflops(isub) + 
     & 3*nx*nz
#endif
            END IF
#endif
         END IF

      END IF

*------------------------------
*     Z-boundary periodic BCs
*------------------------------

      IF (lz.AND.izmb.EQ.MM_PERIODIC) THEN 

         IF (npz.EQ.1) THEN
            DO iy=1,ny
               DO ix=1,nx
                  X(ix,iy, 1) = t1*X(ix,iy,nz-1) + t2*X(ix,iy,   2)
                  X(ix,iy,nz) = t1*X(ix,iy,   2) + t2*X(ix,iy,nz-1)
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 6*nx*ny
      gmflops(isub)=gmflops(isub) + 
     & 6*nx*ny
#endif
#ifdef MPI_USED
         ELSE
            IF (ipz.EQ.0) THEN
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmRefreshBC2: MPI_Sendrecv')") ip
#endif
               CALL MPI_Sendrecv (
     &              X(1,1,2),1,bndx(ZD),gsubnbr(jx,jy,jz,ZMD),80+ZPD,
     &              X(1,1,1),1,bndx(ZD),gsubnbr(jx,jy,jz,ZMD),80+ZPD,
     &              gsubcomm(jx,jy,jz),status,ierr)
               IF (ierr.NE.0) print*, 'MPI Error in mmRefreshBc2: ',ierr
               DO iy=1,ny
                  DO ix=1,nx
                     X(ix,iy,1) = t1*X(ix,iy,1) + t2*X(ix,iy,2)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*nx*ny
      gmflops(isub)=gmflops(isub) + 
     & 3*nx*ny
#endif
            END IF
            IF (ipz.EQ.npz-1) THEN
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmRefreshBC2: MPI_Sendrecv')") ip
#endif
               CALL MPI_Sendrecv (
     &              X(1,1,nz-1),1,bndx(ZD),gsubnbr(jx,jy,jz,ZPD),80+ZPD,
     &              X(1,1,  nz),1,bndx(ZD),gsubnbr(jx,jy,jz,ZPD),80+ZPD,
     &              gsubcomm(jx,jy,jz),status,ierr)
               IF (ierr.NE.0) print*, 'MPI Error in mmRefreshBc2: ',ierr
               DO iy=1,ny
                  DO ix=1,nx
                     X(ix,iy,nz) = t1*X(ix,iy,nz) + t2*X(ix,iy,nz-1)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 3*nx*ny
      gmflops(isub)=gmflops(isub) + 
     & 3*nx*ny
#endif
            END IF
#endif
         END IF

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmResidual
C     
C     Purpose: Residual computation
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,diagScaled,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      INTEGER nx,ny,nz
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)
      REAL scaleA,scaleB
      INTEGER level
      INTEGER diagScaled
      LOGICAL lx,ly,lz
      

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif


#ifdef DEBUG
      REAL s,mmVectorSum
      EXTERNAL mmVectorSum
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESIDUAL+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

C@@@ Remove lx,ly,lz

      lx = .TRUE.
      ly = .TRUE.
      lz = .TRUE.

#ifdef PROC_TRACE
      IF (ip.EQ.0) print*, '>>> mmResidual'
#endif

      CALL mmRefreshBC (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,level,isub)

      IF (gimatfree.EQ.1) THEN

         IF ( gicoords.EQ.MM_CARTESIAN .AND.
     &        giGridType.EQ.MM_IS_UNIFORM) THEN
#ifdef DEBUG
            IF (ip.EQ.0) 
     &           print*, 'mmResidual: calling mmBMFResidualU ',level
#endif
            CALL mmBMFResidualU (
     &           
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &           diagScaled,lx,ly,lz,level,isub)
         ELSE
#ifdef DEBUG
            IF (ip.EQ.0) 
     &           print*, 'mmResidual: calling mmBMFResidualGen ',level
#endif
            CALL mmBMFResidualGen (
     &           
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &           diagScaled,lx,ly,lz,level,isub)
         END IF

      ELSE

#ifdef MPI_BLOCKING
#ifdef DEBUG
         IF (ip.EQ.0) 
     &           print*, 'mmResidual: calling mmBResidual ',level
#endif
         CALL mmBResidual (
     &        
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        diagScaled,lx,ly,lz,level,isub)
#else 
#ifdef DEBUG
         IF (ip.EQ.0) 
     &           print*, 'mmResidual: calling mmIResidualU ',level
#endif
         CALL mmIResidual (
     &        
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        diagScaled,lx,ly,lz,level,isub)
#endif

      END IF

#ifdef DEBUG
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, 'mmResidual: sum(R) = ',s,nx,ny,nz
      s = mmVectorSum (
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, 'mmResidual: sum(B) = ',s,nx,ny,nz
      s = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
      IF (ip.EQ.0) PRINT*, 'mmResidual: sum(X) = ',s,nx,ny,nz
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) print*, '<<< mmResidual'
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmBResidual 
C     
C     Purpose: Residual computation with blocking communication
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     diagScaled,lx,ly,lz,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level
      INTEGER diagScaled
      LOGICAL lx,ly,lz

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESIDUAL+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmBResidual'
#endif

*-----------------------------------------------------------------------
*     Refresh the ghost points of X
*-----------------------------------------------------------------------

       CALL mmGetGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)

*-----------------------------------------------------------------------
*     Compute local Y = A*X
*-----------------------------------------------------------------------

       DO iz=1,nz
          DO iy=1,ny
             DO ix=1,nx
                R(ix,iy,iz) = 0.0D0
             END DO
          END DO
       END DO
       
       IF (diagScaled.EQ.0) THEN

          DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
C     ideal counts for 33^3 m=3 nu=2: 10.1 %             
C     fpcsamp counts                   9.1 %
                  R(ix,iy,iz) = B(ix,iy,iz)
     &                 - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &                 - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &                 - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &                 - AD(ix,iy,iz)  *X(ix,iy,iz)
               END DO
C     ideal counts for 33^3 m=3 nu=2: 8.0 %
C     fpcsamp counts                  7.2 %
               DO ix=2,nx-1
                  R(ix,iy,iz) = R(ix,iy,iz)
     &                 - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &                 - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &                 - A3(ix,iy,iz)  *X(ix,iy,iz+1)
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 14*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 14*(nx-2)*(ny-2)*(nz-2)
#endif

      ELSE IF (diagScaled.EQ.1) THEN

         DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
C     ideal counts for 33^3 m=3 nu=2: 21.7 %
C     fpcsamp counts                  18.8 %
                  R(ix,iy,iz) = (B(ix,iy,iz)
     &                 - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &                 - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &                 - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &                 - AD(ix,iy,iz)  *X(ix,iy,iz))
               END DO
C     ideal counts for 33^3 m=3 nu=2: 19.4 %
C     fpcsamp counts                   16.9 %
               DO ix=2,nx-1
                  R(ix,iy,iz) = (R(ix,iy,iz)
     &                 - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &                 - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &                 - A3(ix,iy,iz)  *X(ix,iy,iz+1)) / AD(ix,iy,iz)
               END DO
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 15*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 15*(nx-2)*(ny-2)*(nz-2)
#endif
      END IF


      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmBResidual'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmIResidual
C     
C     PURPOSE: Residual computation using nonblocking communication
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmIResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     diagScaled,lx,ly,lz,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level
      INTEGER diagScaled
      LOGICAL lx,ly,lz

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER i,ix,iy,iz

#ifdef MPI_USED
      INTEGER status(MPI_STATUS_SIZE,12),ierr
      INTEGER req(12)
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESIDUAL+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmIResidual'
#endif
*-----------------------------------------------------------------------
*     Start sending ghost points of X
*-----------------------------------------------------------------------

#ifdef MPI_USED

*     NONBLOCKING COMMUNICATION

      icommtag=icommtag+6
      IF (icommtag.GT.32757) icommtag=ICOMMSTART

      DO i=1,12
         req(i)=MPI_REQUEST_NULL
      END DO

      CALL mmIGetX1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(1),req(2),isub)
      CALL mmIGetY1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(3),req(4),isub)
      CALL mmIGetZ1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(5),req(6),isub)
      CALL mmIGetXnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(7),req(8),isub)
      CALL mmIGetYnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(9),req(10),isub)
      CALL mmIGetZnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(11),req(12),isub)

#endif

*-----------------------------------------------------------------------
*     Interior points
*-----------------------------------------------------------------------

      DO iz=3,nz-2
         DO iy=3,ny-2
            DO ix=3,nx-2
               R(ix,iy,iz) = B(ix,iy,iz)
     &              - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &              - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &              - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &              - AD(ix,iy,iz)    *X(ix,iy,iz)
            END DO
            DO ix=3,nx-2
               R(ix,iy,iz) = R(ix,iy,iz)
     &              - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &              - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &              - A3(ix,iy,iz)  *X(ix,iy,iz+1)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-4)*(ny-4)*(nz-4)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-4)*(ny-4)*(nz-4)
#endif

*     wait for boundaries

      
#ifdef PERF_EVAL
      gtimestart(SB_RESIDUAL_WAIT) = MPI_WTIME()
#endif

#ifdef MPI_USED
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmIResidual: MPI_WAITALL')") ip
#endif
      CALL MPI_WAITALL (12,req,status,ierr)
      IF (ierr.NE.0) print*, 'MPI Error in mmIResidual wait: ',ierr
#endif

      
#ifdef PERF_EVAL
      gtime(SB_RESIDUAL_WAIT) = gtime(SB_RESIDUAL_WAIT) + MPI_WTIME() - gtimestart(SB_RESIDUAL_WAIT)
      gtimecount(SB_RESIDUAL_WAIT) = gtimecount(SB_RESIDUAL_WAIT) + 1
#endif

*-----------------------------------------------------------------------
*     X-facets
*-----------------------------------------------------------------------

      DO iz=2,nz-1
         DO iy=2,ny-1
            ix=2
            R(ix,iy,iz) = B(ix,iy,iz)
     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            ix=3
*            R(ix,iy,iz) =  B(ix,iy,iz)
*     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            ix=nx-2
*            R(ix,iy,iz) =  B(ix,iy,iz)
*     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
            ix=nx-1
            R(ix,iy,iz) =  B(ix,iy,iz)
     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(ny-2)*(nz-2)
#endif
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(ny-2)*(nz-2)
#endif

*-----------------------------------------------------------------------
*     Y-facets
*-----------------------------------------------------------------------

      DO iz=2,nz-1
         DO ix=2,nx-1
            iy=2
            R(ix,iy,iz) = B(ix,iy,iz)
     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            iy=3
*            R(ix,iy,iz) =  B(ix,iy,iz)
*     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            iy=ny-2
*            R(ix,iy,iz) =  B(ix,iy,iz)
*     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
            iy=ny-1
            R(ix,iy,iz) =  B(ix,iy,iz)
     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-2)*(nz-2)
#endif
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-2)*(nz-2)
#endif

*-----------------------------------------------------------------------
*     Z-facets
*-----------------------------------------------------------------------

      DO iy=2,ny-1
         DO ix=2,nx-1
            iz=2
            R(ix,iy,iz) =  B(ix,iy,iz)
     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            iz=3
*            R(ix,iy,iz) =  B(ix,iy,iz)
*     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            iz=nz-2
*            R(ix,iy,iz) =  B(ix,iy,iz)
*     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
            iz=nz-1
            R(ix,iy,iz) =  B(ix,iy,iz)
     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
         END DO
      END DO


      IF (diagScaled.EQ.1) THEN
         DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
                  R(ix,iy,iz) = R(ix,iy,iz) / AD(ix,iy,iz)
               END DO
            END DO
         END DO
      END IF

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(ny-2)*(nx-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(ny-2)*(nx-2)
#endif
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(ny-2)*(nx-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(ny-2)*(nx-2)
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmIResidual'
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmBMFResidualGen 
C     
C     Purpose: Matrix-free residual computation for general problems
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBMFResidualGen (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     diagScaled,lx,ly,lz,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level
      INTEGER diagScaled
      LOGICAL lx,ly,lz

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz
      REAL hxm,hym,hzm,hxp,hyp,hzp
      REAL dxs,dys,dzs
      REAL dsumi
      REAL d1,d2,d3,d4,d5,d6

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESIDUAL+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

C@@@  Remove lx,ly,lz

      lx = .TRUE.
      ly = .TRUE.
      lz = .TRUE.

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmBMFResidualGen'
#endif

      IF (level.GT.1) THEN

         CALL mmBResidual (
     &        
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        diagScaled,lx,ly,lz,level,isub)

      ELSE

*-----------------------------------------------------------------------
*     Refresh the ghost points of X
*-----------------------------------------------------------------------

         CALL mmGetGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)

         DO iz=1,nz
            DO iy=1,ny
               DO ix=1,nx
                  R(ix,iy,iz) = 0.0D0
               END DO
            END DO
         END DO

         IF (diagScaled.EQ.0) THEN
            DO iz=2,nz-1
               dzs = gdz(iz-1,1)+gdz(iz,1)
               hzm=zp(iz)-zp(iz-1)
               hzp=zp(iz+1)-zp(iz)
               DO iy=2,ny-1
                  dys = gdy(iy-1,1)+gdy(iy,1)
                  hym=yp(iy)-yp(iy-1)
                  hyp=yp(iy+1)-yp(iy)
                  DO ix=2,nx-1
                     dxs = gdx(ix-1,1)+gdx(ix,1)
                     hxm=xp(ix)-xp(ix-1)
                     hxp=xp(ix+1)-xp(ix)
                     d1=dys*dzs/hxp*gg2off(ix,1)*gg31off(ix,1)
                     d2=dzs*dxs/hyp*gg32off(iy,1)/gg2on(ix,1)**2
                     d3=dxs*dys/hzp/(gg31on(ix,1)**2*gg32on(iy,1)**2)
                     d4=dys*dzs/hxm * gg2off(ix-1,1)*gg31off(ix-1,1)   
                     d5=dzs*dxs/hym*gg32off(iy-1,1)/gg2on(ix,1)**2
                     d6=dxs*dys/hzm/(gg31on(ix,1)**2*gg32on(iy,1)**2)
                     R(ix,iy,iz) = B(ix,iy,iz) - o4*gScaleA*
     &                    ( d1 * (X(ix+1,iy,iz) - X(ix,iy,iz))
     &                    + d2 * (X(ix,iy+1,iz) - X(ix,iy,iz))
     &                    + d3 * (X(ix,iy,iz+1) - X(ix,iy,iz))
     &                    + d4 * (X(ix-1,iy,iz) - X(ix,iy,iz))
     &                    + d5 * (X(ix,iy-1,iz) - X(ix,iy,iz))
     &                    + d6 * (X(ix,iy,iz-1) - X(ix,iy,iz)))
                  END DO
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 53*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 53*(nx-2)*(ny-2)*(nz-2)
#endif
         ELSE
            DO iz=2,nz-1
               dzs = gdz(iz-1,1)+gdz(iz,1)
               hzm=zp(iz)-zp(iz-1)
               hzp=zp(iz+1)-zp(iz)
               DO iy=2,ny-1
                  dys = gdy(iy-1,1)+gdy(iy,1)
                  hym=yp(iy)-yp(iy-1)
                  hyp=yp(iy+1)-yp(iy)
                  DO ix=2,nx-1
                     dxs = gdx(ix-1,1)+gdx(ix,1)
                     hxm=xp(ix)-xp(ix-1)
                     hxp=xp(ix+1)-xp(ix)

                     d1=dys*dzs/hxp*gg2off(ix,1)*gg31off(ix,1)
                     d2=dzs*dxs/hyp*gg32off(iy,1)/gg2on(ix,1)**2
                     d3=dxs*dys/hzp/(gg31on(ix,1)**2*gg32on(iy,1)**2)
                     d4=dys*dzs/hxm * gg2off(ix-1,1)*gg31off(ix-1,1)   
                     d5=dzs*dxs/hym*gg32off(iy-1,1)/gg2on(ix,1)**2
                     d6=dxs*dys/hzm/(gg31on(ix,1)**2*gg32on(iy,1)**2)

                     dsumi = -o1/(o4*gScaleA*(d1+d2+d3+d4+d5+d6))

                     R(ix,iy,iz) = dsumi*(B(ix,iy,iz) - o4*gScaleA*
     &                    (  d1    * (X(ix+1,iy,iz) - X(ix,iy,iz))
     &                    +  d2 * (X(ix,iy+1,iz) - X(ix,iy,iz))
     &                    +  d3 * (X(ix,iy,iz+1) - X(ix,iy,iz))
     &                    +  d4 * (X(ix-1,iy,iz) - X(ix,iy,iz))
     &                    +  d5 * (X(ix,iy-1,iz) - X(ix,iy,iz))
     &                    +  d6 * (X(ix,iy,iz-1) - X(ix,iy,iz))))
                  END DO
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 62*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 62*(nx-2)*(ny-2)*(nz-2)
#endif
         END IF

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmBMFResidualGen'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmBMFResidualNU 
C     
C     PURPOSE: Matrix-free residual computation for nonuniform meshes
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBMFResidualNU (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     diagScaled,lx,ly,lz,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level
      INTEGER diagScaled
      LOGICAL lx,ly,lz

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz
      REAL hxm,hym,hzm,hxp,hyp,hzp
      REAL dxs,dys,dzs,dxp,dyp,dzp,dxm,dym,dzm,dsum,dsumi

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESIDUAL+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

C@@@  Remove lx,ly,lz
      lx = .TRUE.
      ly = .TRUE.
      lz = .TRUE.

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmBMFResidualNU'
#endif

      IF (level.GT.1) THEN

         CALL mmBResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        diagScaled,lx,ly,lz,level,isub)
      ELSE

*-----------------------------------------------------------------------
*     Refresh the ghost points of X
*-----------------------------------------------------------------------

         CALL mmGetGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)

         dzs = zp(3)-zp(1)
         hzm = zp(2)-zp(1)
         hzp = zp(3)-zp(2)
         IF (diagScaled.EQ.0) THEN
            DO iz=2,nz-1
               dys = yp(3)-yp(1)
               hym = yp(2)-yp(1)
               hyp = yp(3)-yp(2)
               DO iy=2,ny-1
                  dxs = xp(3)-xp(1)
                  hxm = xp(2)-xp(1)
                  hxp = xp(3)-xp(2)
                  DO ix=2,nx-1
                     dxp = dys*dzs/hxp
                     dyp = dzs*dxs/hyp
                     dzp = dxs*dys/hzp
                     dxm = dys*dzs/hxm
                     dym = dzs*dxs/hym
                     dzm = dxs*dys/hzm
                     dsum = dxp + dyp + dzp + dxm + dym + dzm
                     dzp = dzp/dsum
                     dxp = dxp/dsum
                     dyp = dyp/dsum
                     dzm = dzm/dsum
                     dxm = dxm/dsum
                     dym = dym/dsum
                     dsum = o4*dsum*gScaleA
                     R(ix,iy,iz) = B(ix,iy,iz) - dsum*(
     &                    - X(ix,iy,iz)
     &                    + dxp * X(ix+1,iy,iz)
     &                    + dyp * X(ix,iy+1,iz)
     &                    + dzp * X(ix,iy,iz+1)
     &                    + dxm * X(ix-1,iy,iz)
     &                    + dym * X(ix,iy-1,iz)
     &                    + dzm * X(ix,iy,iz-1))
                     dxs = xp(ix+2)-xp(ix)
                     hxm = hxp
                     hxp = xp(ix+1)-xp(ix)
                  END DO
                  dys = yp(iy+2)-yp(iy)
                  hym = hyp
                  hyp = yp(iy+1)-yp(iy)
               END DO
               dzs = zp(iz+2)-zp(iz)
               hzm = hzp
               hzp = zp(iz+1)-zp(iz)
            END DO

            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 41*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 41*(nx-2)*(ny-2)*(nz-2)
#endif

         ELSE                   ! diagScaled .EQ. 1
            DO iz=2,nz-1
               dys = yp(3)-yp(1)
               hym = yp(2)-yp(1)
               hyp = yp(3)-yp(2)
               DO iy=2,ny-1
                  dxs = xp(3)-xp(1)
                  hxm = xp(2)-xp(1)
                  hxp = xp(3)-xp(2)
                  DO ix=2,nx-1
                     dxp = dys*dzs/hxp
                     dyp = dzs*dxs/hyp
                     dzp = dxs*dys/hzp
                     dxm = dys*dzs/hxm
                     dym = dzs*dxs/hym
                     dzm = dxs*dys/hzm
                     dsum = dxp + dyp + dzp + dxm + dym + dzm
                     dzp = dzp/dsum
                     dxp = dxp/dsum
                     dyp = dyp/dsum
                     dzm = dzm/dsum
                     dxm = dxm/dsum
                     dym = dym/dsum
                     dsum = o4*dsum*gScaleA
                     dsumi = -o1 / dsum
                     R(ix,iy,iz) = dsumi*B(ix,iy,iz) + (
     &                    - X(ix,iy,iz)
     &                    + dxp * X(ix+1,iy,iz)
     &                    + dyp * X(ix,iy+1,iz)
     &                    + dzp * X(ix,iy,iz+1)
     &                    + dxm * X(ix-1,iy,iz)
     &                    + dym * X(ix,iy-1,iz)
     &                    + dzm * X(ix,iy,iz-1))
                     dxs = xp(ix+2)-xp(ix)
                     hxm = hxp
                     hxp = xp(ix+1)-xp(ix)
                  END DO
                  dys = yp(iy+2)-yp(iy)
                  hym = hyp
                  hyp = yp(iy+1)-yp(iy)
               END DO
               dzs = zp(iz+2)-zp(iz)
               hzm = hzp
               hzp = zp(iz+1)-zp(iz)
            END DO

            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 42*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 42*(nx-2)*(ny-2)*(nz-2)
#endif

         END IF

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif
      
#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmBMFResidualNU'
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmBMFResidualU 
C     
C     Purpose: Matrix-free residual computation for uniform meshes
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBMFResidualU (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     diagScaled,lx,ly,lz,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level
      INTEGER diagScaled
      LOGICAL lx,ly,lz

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz
      REAL hxm,hym,hzm,hxp,hyp,hzp
      REAL dxs,dys,dzs,dxp,dyp,dzp,dxm,dym,dzm,dsum,dsumi

#ifndef MPI_USED

      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESIDUAL+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

C@@@  Remove lx,ly,lz

      lx = .TRUE.
      ly = .TRUE.
      lz = .TRUE.


#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmBMFResidualU'
#endif

      IF (level.GT.1) THEN

         CALL mmBResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        diagScaled,lx,ly,lz,level,isub)

      ELSE

*-----------------------------------------------------------------------
*     Refresh the ghost points of X
*-----------------------------------------------------------------------

         CALL mmGetGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)

         DO iz=1,nz
            DO iy=1,ny
               DO ix=1,nx
                  R(ix,iy,iz) = 0.0D0
               END DO
            END DO
         END DO
         
         dzs = zp(3)-zp(1)
         hzm = zp(2)-zp(1)
         hzp = zp(3)-zp(2)
         dys = yp(3)-yp(1)
         hym = yp(2)-yp(1)
         hyp = yp(3)-yp(2)
         dxs = xp(3)-xp(1)
         hxm = xp(2)-xp(1)
         hxp = xp(3)-xp(2)
         dzp=dxs*dys/hzp
         dxp=dys*dzs/hxp
         dyp=dzs*dxs/hyp
         dzm=dxs*dys/hzm
         dxm=dys*dzs/hxm
         dym=dzs*dxs/hym
         dsum = dxp + dyp + dzp + dxm + dym + dzm
         dzp=dzp/dsum
         dxp=dxp/dsum
         dyp=dyp/dsum
         dzm=dzm/dsum
         dxm=dxm/dsum
         dym=dym/dsum
         dsum = o4*dsum*gScaleA
         dsumi = -1.0D0 / dsum
         IF (diagScaled.EQ.0) THEN
            DO iz=2,nz-1
               DO iy=2,ny-1
                  DO ix=2,nx-1
                     R(ix,iy,iz) = B(ix,iy,iz) - dsum*(
     &                    - X(ix,iy,iz)
     &                    + dxp * X(ix+1,iy,iz)
     &                    + dyp * X(ix,iy+1,iz)
     &                    + dzp * X(ix,iy,iz+1)
     &                    + dxm * X(ix-1,iy,iz)
     &                    + dym * X(ix,iy-1,iz)
     &                    + dzm * X(ix,iy,iz-1))
                  END DO
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 14*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 14*(nx-2)*(ny-2)*(nz-2)
#endif
         ELSE                   ! diagScaled .EQ. 1
            DO iz=2,nz-1
               DO iy=2,ny-1
                  DO ix=2,nx-1
                     R(ix,iy,iz) = dsumi*B(ix,iy,iz) + (
     &                    - X(ix,iy,iz)
     &                    + dxp * X(ix+1,iy,iz)
     &                    + dyp * X(ix,iy+1,iz)
     &                    + dzp * X(ix,iy,iz+1)
     &                    + dxm * X(ix-1,iy,iz)
     &                    + dym * X(ix,iy-1,iz)
     &                    + dzm * X(ix,iy,iz-1))
                  END DO
               END DO
            END DO
            
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 14*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 14*(nx-2)*(ny-2)*(nz-2)
#endif
         END IF
      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif


#ifdef DEBUG
      PRINT*, ip, 'mmBMFResidualU: exiting'
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmBMFResidualU'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmResidual2
C     
C     PURPOSE: Residual computation
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmResidual2 (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,diagScaled,lx,ly,lz,mask,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      INTEGER nx,ny,nz
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)
      REAL scaleA,scaleB
      INTEGER level
      INTEGER diagScaled
      LOGICAL lx,ly,lz,mask(*)
      

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

#ifdef DEBUG
      REAL s,mmVectorSum
      EXTERNAL mmVectorSum
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESIDUAL2+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

C DEBUG      R(12,12,13) = 3.141592
C DEBUG      print*, 'mmResidual2 X(12,12,13) = ',X(12,12,13)
C DEBUG      R(12,12,13) = 0.0D0

#ifdef PROC_TRACE
      IF (ip.EQ.0) print*, '>>> Entering mmResidual2'
#endif

#ifdef DEBUG
      print*, ip, 'mmResidual2: refresh b.c.''s'
#endif

      CALL mmRefreshBC2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,lx,ly,lz,level,isub)

#ifdef DEBUG
      s = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
      PRINT*, ip,'mmResidual2 sum(X) = ',s
#endif

      IF (gimatfree.EQ.1) THEN

         IF ( gicoords.EQ.MM_CARTESIAN .AND.
     &        giGridType.EQ.MM_IS_UNIFORM) THEN
#ifdef DEBUG
            print*, ip, 'mmResidual2: calling mmBMFResidualU2 ',level
#endif
            CALL mmBMFResidualU2 (
     &           
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &           diagScaled,lx,ly,lz,mask,level,isub)
         ELSE
#ifdef DEBUG
            print*, ip, 'mmResidual2: calling mmBMFResidualGen2 ',level
#endif
            CALL mmBMFResidualGen2 (
     &           
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &           diagScaled,lx,ly,lz,mask,level,isub)
         END IF

      ELSE

#ifdef MPI_BLOCKING
#ifdef DEBUG
         print*, ip, 'mmResidual2: calling mmBResidual2 ',level
#endif
         CALL mmBResidual2 (
     &        
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        diagScaled,lx,ly,lz,mask,level,isub)
#else 
#ifdef DEBUG
         print*, ip, 'mmResidual2: calling mmIResidual2 ',level
#endif
         CALL mmIResidual2 (
     &        
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        diagScaled,lx,ly,lz,mask,level,isub)
#endif

      END IF

#ifdef DEBUG
      s = mmVectorSum (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm,isub)
      PRINT*, ip,'mmResidual2: sum(R) = ',s
      s = mmVectorSum (
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,comm,isub)
      PRINT*, ip,'mmResidual2: sum(B) = ',s
      s = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
      PRINT*, ip,'mmResidual2: sum(X) = ',s
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) print*, '<<< Entering mmResidual2'
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmBResidual2 
C     
C     Purpose: Residual computation with blocking communication
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBResidual2 (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     diagScaled,lx,ly,lz,mask,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level
      INTEGER diagScaled
      LOGICAL lx,ly,lz,mask(*)


#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      INTEGER ix,iy,iz

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESIDUAL2+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> Entering mmBResidual2: level=',level
#endif

*-----------------------------------------------------------------------
*     Refresh the ghost points of X
*-----------------------------------------------------------------------

      IF (lx) THEN
         CALL mmGetX1Ghost2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,isub)
         CALL mmGetXnGhost2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,isub)
      END IF
      IF (ly) THEN
         CALL mmGetY1Ghost2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,isub)
         CALL mmGetYnGhost2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,isub)
      END IF
      IF (lz) THEN
         CALL mmGetZ1Ghost2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,isub)
         CALL mmGetZnGhost2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,lx,ly,lz,isub)
      END IF

*-----------------------------------------------------------------------
*     Compute local Y = A*X
*-----------------------------------------------------------------------

      IF (.NOT.lx) THEN
         DO ix=2,nx-1
            IF (mask(ix)) THEN
               DO iz=2,nz-1
                  DO iy=2,ny-1
                     R(ix,iy,iz) = B(ix,iy,iz)
     &                    - AD(ix,iy,iz)  *X(ix,iy,iz)
     &                    - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &                    - A3(ix,iy,iz)  *X(ix,iy,iz+1)
     &                    - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &                    - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 10*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 10*(ny-2)*(nz-2)
#endif
            END IF
         END DO
      ELSE IF (.NOT.ly) THEN
         DO iy=2,ny-1
            IF (mask(iy)) THEN
               DO iz=2,nz-1
                  DO ix=2,nx-1
                     R(ix,iy,iz) = B(ix,iy,iz)
     &                    - AD(ix,iy,iz)  *X(ix,iy,iz)
     &                    - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &                    - A3(ix,iy,iz)  *X(ix,iy,iz+1)
     &                    - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &                    - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 10*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 10*(nx-2)*(nz-2)
#endif
            END IF
         END DO
      ELSE IF (.NOT.lz) THEN
         DO iz=2,nz-1
            IF (mask(iz)) THEN
               DO iy=2,ny-1
                  DO ix=2,nx-1
                     R(ix,iy,iz) = B(ix,iy,iz)
     &                    - AD(ix,iy,iz)  *X(ix,iy,iz)
     &                    - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &                    - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &                    - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &                    - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 10*(ny-2)*(nx-2)
      gmflops(isub)=gmflops(isub) + 
     & 10*(ny-2)*(nx-2)
#endif
            END IF
         END DO
      END IF

      IF (diagScaled.EQ.1) THEN

         IF (.NOT.lx) THEN
            DO ix=2,nx-1
               IF (mask(ix)) THEN
                  DO iz=2,nz-1
                     DO iy=2,ny-1
                        R(ix,iy,iz) = R(ix,iy,iz) / AD(ix,iy,iz)
                     END DO
                  END DO
                  
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & (ny-2)*(nz-2)
#endif
               END IF
            END DO

         ELSE IF (.NOT.ly) THEN

            DO iy=2,ny-1
               IF (mask(iy)) THEN
                  DO iz=2,nz-1
                     DO ix=2,nx-1
                        R(ix,iy,iz) = R(ix,iy,iz) / AD(ix,iy,iz)
                     END DO
                  END DO
                  
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & (nx-2)*(nz-2)
#endif
               END IF
            END DO

         ELSE IF (.NOT.lz) THEN

            DO iz=2,nz-1
               IF (mask(iz)) THEN
                  DO iy=2,ny-1
                     DO ix=2,nx-1
                        R(ix,iy,iz) = R(ix,iy,iz) / AD(ix,iy,iz)
                     END DO
                  END DO
                  
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (ny-2)*(nx-2)
      gmflops(isub)=gmflops(isub) + 
     & (ny-2)*(nx-2)
#endif
               END IF
            END DO

         END IF

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< Entering mmBResidual2: level=',level
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmIResidual2
C     
C     PURPOSE: Residual computation using nonblocking communication
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmIResidual2 (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     diagScaled,lx,ly,lz,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level
      INTEGER diagScaled
      LOGICAL lx,ly,lz

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER i,ix,iy,iz

#ifdef MPI_USED
      INTEGER status(MPI_STATUS_SIZE,12),ierr
      INTEGER req(12)
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESIDUAL2+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> Entering mmIResidual2'
#endif
*-----------------------------------------------------------------------
*     Start sending ghost points of X
*-----------------------------------------------------------------------

#ifdef MPI_USED

*     NONBLOCKING COMMUNICATION

      icommtag=icommtag+6
      IF (icommtag.GT.32757) icommtag=ICOMMSTART

      DO i=1,12
         req(i)=MPI_REQUEST_NULL
      END DO

      CALL mmIGetX1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(1),req(2),isub)
      CALL mmIGetY1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(3),req(4),isub)
      CALL mmIGetZ1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(5),req(6),isub)
      CALL mmIGetXnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(7),req(8),isub)
      CALL mmIGetYnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(9),req(10),isub)
      CALL mmIGetZnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(11),req(12),isub)

#endif

*-----------------------------------------------------------------------
*     Interior points
*-----------------------------------------------------------------------

      DO iz=3,nz-2
         DO iy=3,ny-2
            DO ix=3,nx-2
               R(ix,iy,iz) = B(ix,iy,iz)
     &              - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &              - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &              - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &              - AD(ix,iy,iz)    *X(ix,iy,iz)
            END DO
            DO ix=3,nx-2
               R(ix,iy,iz) = R(ix,iy,iz)
     &              - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &              - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &              - A3(ix,iy,iz)  *X(ix,iy,iz+1)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-2)*(ny-2)*(nz-2)
#endif

*     wait for boundaries

      
#ifdef PERF_EVAL
      gtimestart(SB_RESIDUAL_WAIT) = MPI_WTIME()
#endif

#ifdef MPI_USED
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmIResidual2: MPI_WAITALL')") ip
#endif
      CALL MPI_WAITALL (12,req,status,ierr)
      IF (ierr.NE.0) print*, 'MPI Error in mmIResidual2 wait: ',ierr
#endif

      
#ifdef PERF_EVAL
      gtime(SB_RESIDUAL_WAIT) = gtime(SB_RESIDUAL_WAIT) + MPI_WTIME() - gtimestart(SB_RESIDUAL_WAIT)
      gtimecount(SB_RESIDUAL_WAIT) = gtimecount(SB_RESIDUAL_WAIT) + 1
#endif

*-----------------------------------------------------------------------
*     X-facets
*-----------------------------------------------------------------------

      DO iz=2,nz-1
         DO iy=2,ny-1
            ix=2
            R(ix,iy,iz) = B(ix,iy,iz)
     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            ix=3
*            R(ix,iy,iz) =  B(ix,iy,iz)
*     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            ix=nx-2
*            R(ix,iy,iz) =  B(ix,iy,iz)
*     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
            ix=nx-1
            R(ix,iy,iz) =  B(ix,iy,iz)
     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(ny-2)*(nz-2)
#endif
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(ny-2)*(nz-2)
#endif

*-----------------------------------------------------------------------
*     Y-facets
*-----------------------------------------------------------------------

      DO iz=2,nz-1
         DO ix=2,nx-1
            iy=2
            R(ix,iy,iz) = B(ix,iy,iz)
     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            iy=3
*            R(ix,iy,iz) =  B(ix,iy,iz)
*     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            iy=ny-2
*            R(ix,iy,iz) =  B(ix,iy,iz)
*     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
            iy=ny-1
            R(ix,iy,iz) =  B(ix,iy,iz)
     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-2)*(nz-2)
#endif
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-2)*(nz-2)
#endif

*-----------------------------------------------------------------------
*     Z-facets
*-----------------------------------------------------------------------

      DO iy=2,ny-1
         DO ix=2,nx-1
            iz=2
            R(ix,iy,iz) =  B(ix,iy,iz)
     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            iz=3
*            R(ix,iy,iz) =  B(ix,iy,iz)
*     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
*            iz=nz-2
*            R(ix,iy,iz) =  B(ix,iy,iz)
*     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
*     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
*     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
*     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
*     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
*     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
*     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
            iz=nz-1
            R(ix,iy,iz) =  B(ix,iy,iz)
     &           - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &           - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &           - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &           - AD(ix,iy,iz)  *X(ix,iy,iz)
     &           - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &           - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &           - A3(ix,iy,iz)  *X(ix,iy,iz+1)
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-2)*(ny-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-2)*(ny-2)
#endif
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-2)*(ny-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-2)*(ny-2)
#endif

      IF (diagScaled.EQ.1) THEN
         DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
                  R(ix,iy,iz) = R(ix,iy,iz) / AD(ix,iy,iz)
               END DO
            END DO
         END DO
      END IF

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & (nx-2)*(ny-2)*(nz-2)
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< Entering mmIResidual2'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmBMFResidualGen2
C     
C     PURPOSE: Matrix-free residual computation for general problems
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBMFResidualGen2 (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     diagScaled,lx,ly,lz,mask,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level
      INTEGER diagScaled
      LOGICAL lx,ly,lz,mask(*)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz
      REAL hxm,hym,hzm,hxp,hyp,hzp
      REAL xm2,ym2,zm2,xp2,yp2,zp2
      REAL dxs,dys,dzs
      REAL dsumi
      REAL d1,d2,d3,d4,d5,d6

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESIDUAL2+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmBMFResidualGen2'
#endif

      IF (level.GT.1) THEN

         CALL mmBResidual2 (
     &        
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        diagScaled,lx,ly,lz,mask,level,isub)

      ELSE

*-----------------------------------------------------------------------
*     Refresh the ghost points of X
*-----------------------------------------------------------------------

         IF (lx) THEN
            CALL mmGetX1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
            CALL mmGetXnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         END IF
         IF (ly) THEN
            CALL mmGetY1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
            CALL mmGetYnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         END IF
         IF (lz) THEN
            CALL mmGetZ1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
            CALL mmGetZnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         END IF

         IF (diagScaled.EQ.0) THEN

            IF ((.NOT.lx).AND.ly.AND.lz) THEN
               DO ix=2,nx-1
                  IF (mask(ix)) THEN
                     dxs = gdx(ix-1,1)+gdx(ix,1)
                     hxm = xp(ix)-xp(ix-1)
                     hxp = xp(ix+1)-xp(ix)
                     xm2 = o2*(xp(ix-1)+xp(ix))
                     xp2 = o2*(xp(ix)+xp(ix+1))
                     DO iz=2,nz-1
                        dzs = gdz(iz-1,1)+gdz(iz,1)
                        hzm = zp(iz)-zp(iz-1)
                        hzp = zp(iz+1)-zp(iz)
                        zm2 = o2*(zp(iz-1)+zp(iz))
                        zp2 = o2*(zp(iz)+zp(iz+1))
                        DO iy=2,ny-1
                           dys = gdy(iy-1,1)+gdy(iy,1)
                           hym = yp(iy)-yp(iy-1)
                           hyp = yp(iy+1)-yp(iy)
                           ym2 = o2*(yp(iy-1)+yp(iy))
                           yp2 = o2*(yp(iy)+yp(iy+1))

                           d1 = dys*dzs/hxp*gg2off(ix,1)*gg31off(ix,1)
                           d2 = dzs*dxs/hyp*gg32off(iy,1)/gg2on(ix,1)**2
                           d3 = dxs*dys/hzp/(gg31on(ix,1)**2
     &                          *gg32on(iy,1)**2)
                           d4 = dys*dzs/hxm * gg2off(ix-1,1)
     &                          *gg31off(ix-1,1)   
                           d5 = dzs*dxs/hym*gg32off(iy-1,1)
     &                          / gg2on(ix,1)**2
                           d6 = dxs*dys/hzm/(gg31on(ix,1)**2
     &                          *gg32on(iy,1)**2)

                           R(ix,iy,iz) = B(ix,iy,iz) - o4*gScaleA*
     &                          ( d1 * (              - X(ix,iy,iz))
     &                          + d2 * (X(ix,iy+1,iz) - X(ix,iy,iz))
     &                          + d3 * (X(ix,iy,iz+1) - X(ix,iy,iz))
     &                          + d4 * (              - X(ix,iy,iz))
     &                          + d5 * (X(ix,iy-1,iz) - X(ix,iy,iz))
     &                          + d6 * (X(ix,iy,iz-1) - X(ix,iy,iz)))
                        END DO
                     END DO
                     
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 55*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 55*(ny-2)*(nz-2)
#endif
                  END IF
               END DO
            ELSE IF (lx.AND.(.NOT.ly).AND.lz) THEN
               DO iy=2,ny-1
                  IF (mask(iy)) THEN
                     dys = gdy(iy-1,1)+gdy(iy,1)
                     hym = yp(iy)-yp(iy-1)
                     hyp = yp(iy+1)-yp(iy)
                     ym2 = o2*(yp(iy-1)+yp(iy))
                     yp2 = o2*(yp(iy)+yp(iy+1))
                     DO iz=2,nz-1
                        dzs = gdz(iz-1,1)+gdz(iz,1)
                        hzm = zp(iz)-zp(iz-1)
                        hzp = zp(iz+1)-zp(iz)
                        zm2 = o2*(zp(iz-1)+zp(iz))
                        zp2 = o2*(zp(iz)+zp(iz+1))
                        DO ix=2,nx-1
                           dxs = gdx(ix-1,1)+gdx(ix,1)
                           hxm = xp(ix)-xp(ix-1)
                           hxp = xp(ix+1)-xp(ix)
                           xm2 = o2*(xp(ix-1)+xp(ix))
                           xp2 = o2*(xp(ix)+xp(ix+1))

                           d1 = dys*dzs/hxp*gg2off(ix,1)*gg31off(ix,1)
                           d2 = dzs*dxs/hyp*gg32off(iy,1)/gg2on(ix,1)**2
                           d3 = dxs*dys/hzp/(gg31on(ix,1)**2
     &                          *gg32on(iy,1)**2)
                           d4 = dys*dzs/hxm * gg2off(ix-1,1)
     &                          *gg31off(ix-1,1)   
                           d5 = dzs*dxs/hym*gg32off(iy-1,1)
     &                          / gg2on(ix,1)**2
                           d6 = dxs*dys/hzm/(gg31on(ix,1)**2
     &                          *gg32on(iy,1)**2)

                           R(ix,iy,iz) = B(ix,iy,iz) - o4*gScaleA*
     &                          ( d1 * (X(ix+1,iy,iz) - X(ix,iy,iz))
     &                          + d2 * (              - X(ix,iy,iz))
     &                          + d3 * (X(ix,iy,iz+1) - X(ix,iy,iz))
     &                          + d4 * (X(ix-1,iy,iz) - X(ix,iy,iz))
     &                          + d5 * (              - X(ix,iy,iz))
     &                          + d6 * (X(ix,iy,iz-1) - X(ix,iy,iz)))
                        END DO
                     END DO
                     
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 55*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 55*(nx-2)*(nz-2)
#endif
                  END IF
               END DO
            ELSE IF (lx.AND.ly.AND.(.NOT.lz)) THEN
               DO iz=2,nz-1
                  IF (mask(iz)) THEN
                     dzs = gdz(iz-1,1)+gdz(iz,1)
                     hzm = zp(iz)-zp(iz-1)
                     hzp = zp(iz+1)-zp(iz)
                     zm2 = o2*(zp(iz-1)+zp(iz))
                     zp2 = o2*(zp(iz)+zp(iz+1))
                     DO iy=2,ny-1
                        dys = gdy(iy-1,1)+gdy(iy,1)
                        hym = yp(iy)-yp(iy-1)
                        hyp = yp(iy+1)-yp(iy)
                        ym2 = o2*(yp(iy-1)+yp(iy))
                        yp2 = o2*(yp(iy)+yp(iy+1))
                        DO ix=2,nx-1
                           dxs = gdx(ix-1,1)+gdx(ix,1)
                           hxm = xp(ix)-xp(ix-1)
                           hxp = xp(ix+1)-xp(ix)
                           xm2 = o2*(xp(ix-1)+xp(ix))
                           xp2 = o2*(xp(ix)+xp(ix+1))

                           d1 = dys*dzs/hxp*gg2off(ix,1)*gg31off(ix,1)
                           d2 = dzs*dxs/hyp*gg32off(iy,1)/gg2on(ix,1)**2
                           d3 = dxs*dys/hzp/(gg31on(ix,1)**2
     &                          *gg32on(iy,1)**2)
                           d4 = dys*dzs/hxm * gg2off(ix-1,1)
     &                          *gg31off(ix-1,1)   
                           d5 = dzs*dxs/hym*gg32off(iy-1,1)
     &                          / gg2on(ix,1)**2
                           d6 = dxs*dys/hzm/(gg31on(ix,1)**2
     &                          *gg32on(iy,1)**2)

                           R(ix,iy,iz) = B(ix,iy,iz) - o4*gScaleA*
     &                          ( d1 * (X(ix+1,iy,iz) - X(ix,iy,iz))
     &                          + d2 * (X(ix,iy+1,iz) - X(ix,iy,iz))
     &                          + d3 * (              - X(ix,iy,iz))
     &                          + d4 * (X(ix-1,iy,iz) - X(ix,iy,iz))
     &                          + d5 * (X(ix,iy-1,iz) - X(ix,iy,iz))
     &                          + d6 * (              - X(ix,iy,iz)))
                        END DO
                     END DO
                     
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 55*(ny-2)*(nx-2)
      gmflops(isub)=gmflops(isub) + 
     & 55*(ny-2)*(nx-2)
#endif
                  END IF
               END DO

            END IF


         ELSE
            IF ((.NOT.lx).AND.ly.AND.lz) THEN
               DO ix=2,nx-1
                  IF (mask(ix)) THEN
                     dxs = gdx(ix-1,1)+gdx(ix,1)
                     hxm = xp(ix)-xp(ix-1)
                     hxp = xp(ix+1)-xp(ix)
                     xm2 = o2*(xp(ix-1)+xp(ix))
                     xp2 = o2*(xp(ix)+xp(ix+1))
                     DO iz=2,nz-1
                        dzs = gdz(iz-1,1)+gdz(iz,1)
                        hzm = zp(iz)-zp(iz-1)
                        hzp = zp(iz+1)-zp(iz)
                        zm2 = o2*(zp(iz-1)+zp(iz))
                        zp2 = o2*(zp(iz)+zp(iz+1))
                        DO iy=2,ny-1
                           dys = gdy(iy-1,1)+gdy(iy,1)
                           hym = yp(iy)-yp(iy-1)
                           hyp = yp(iy+1)-yp(iy)
                           ym2 = o2*(yp(iy-1)+yp(iy))
                           yp2 = o2*(yp(iy)+yp(iy+1))

                           d1 = dys*dzs/hxp*gg2off(ix,1)*gg31off(ix,1)
                           d2 = dzs*dxs/hyp*gg32off(iy,1)/gg2on(ix,1)**2
                           d3 = dxs*dys/hzp/(gg31on(ix,1)**2
     &                          *gg32on(iy,1)**2)
                           d4 = dys*dzs/hxm * gg2off(ix-1,1)
     &                          *gg31off(ix-1,1)   
                           d5 = dzs*dxs/hym*gg32off(iy-1,1)
     &                          / gg2on(ix,1)**2
                           d6 = dxs*dys/hzm/(gg31on(ix,1)**2
     &                          *gg32on(iy,1)**2)

                           dsumi = -o1/(o4*gScaleA*(d1+d2+d3+d4+d5+d6))

                           R(ix,iy,iz) = dsumi*(B(ix,iy,iz) - o4*
     &                          gScaleA*
     &                          ( d1 * (              - X(ix,iy,iz))
     &                          + d2 * (X(ix,iy+1,iz) - X(ix,iy,iz))
     &                          + d3 * (X(ix,iy,iz+1) - X(ix,iy,iz))
     &                          + d4 * (              - X(ix,iy,iz))
     &                          + d5 * (X(ix,iy-1,iz) - X(ix,iy,iz))
     &                          + d6 * (X(ix,iy,iz-1) - X(ix,iy,iz))))
                        END DO
                     END DO
                     
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 64*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 64*(ny-2)*(nz-2)
#endif
                  END IF
               END DO
            ELSE IF (lx.AND.(.NOT.ly).AND.lz) THEN
               DO iy=2,ny-1
                  IF (mask(iy)) THEN
                     dys = gdy(iy-1,1)+gdy(iy,1)
                     hym = yp(iy)-yp(iy-1)
                     hyp = yp(iy+1)-yp(iy)
                     ym2 = o2*(yp(iy-1)+yp(iy))
                     yp2 = o2*(yp(iy)+yp(iy+1))
                     DO iz=2,nz-1
                        dzs = gdz(iz-1,1)+gdz(iz,1)
                        hzm = zp(iz)-zp(iz-1)
                        hzp = zp(iz+1)-zp(iz)
                        zm2 = o2*(zp(iz-1)+zp(iz))
                        zp2 = o2*(zp(iz)+zp(iz+1))
                        DO ix=2,nx-1
                           dxs = gdx(ix-1,1)+gdx(ix,1)
                           hxm = xp(ix)-xp(ix-1)
                           hxp = xp(ix+1)-xp(ix)
                           xm2 = o2*(xp(ix-1)+xp(ix))
                           xp2 = o2*(xp(ix)+xp(ix+1))

                           d1 = dys*dzs/hxp*gg2off(ix,1)*gg31off(ix,1)
                           d2 = dzs*dxs/hyp*gg32off(iy,1)/gg2on(ix,1)**2
                           d3 = dxs*dys/hzp/(gg31on(ix,1)**2
     &                          *gg32on(iy,1)**2)
                           d4 = dys*dzs/hxm * gg2off(ix-1,1)
     &                          *gg31off(ix-1,1)   
                           d5 = dzs*dxs/hym*gg32off(iy-1,1)
     &                          / gg2on(ix,1)**2
                           d6 = dxs*dys/hzm/(gg31on(ix,1)**2
     &                          *gg32on(iy,1)**2)

                           dsumi = -o1/(o4*gScaleA*(d1+d2+d3+d4+d5+d6))

                           R(ix,iy,iz) = dsumi*(B(ix,iy,iz) - o4*
     &                          gScaleA*
     &                          ( d1 * (X(ix+1,iy,iz) - X(ix,iy,iz))
     &                          + d2 * (              - X(ix,iy,iz))
     &                          + d3 * (X(ix,iy,iz+1) - X(ix,iy,iz))
     &                          + d4 * (X(ix-1,iy,iz) - X(ix,iy,iz))
     &                          + d5 * (              - X(ix,iy,iz))
     &                          + d6 * (X(ix,iy,iz-1) - X(ix,iy,iz))))
                        END DO
                     END DO
                     
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 64*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 64*(nx-2)*(nz-2)
#endif
                  END IF
               END DO
            ELSE IF (lx.AND.ly.AND.(.NOT.lz)) THEN
               DO iz=2,nz-1
                  IF (mask(iz)) THEN
                     dzs = gdz(iz-1,1)+gdz(iz,1)
                     hzm = zp(iz)-zp(iz-1)
                     hzp = zp(iz+1)-zp(iz)
                     zm2 = o2*(zp(iz-1)+zp(iz))
                     zp2 = o2*(zp(iz)+zp(iz+1))
                     DO iy=2,ny-1
                        dys = gdy(iy-1,1)+gdy(iy,1)
                        hym = yp(iy)-yp(iy-1)
                        hyp = yp(iy+1)-yp(iy)
                        ym2 = o2*(yp(iy-1)+yp(iy))
                        yp2 = o2*(yp(iy)+yp(iy+1))
                        DO ix=2,nx-1
                           dxs = gdx(ix-1,1)+gdx(ix,1)
                           hxm = xp(ix)-xp(ix-1)
                           hxp = xp(ix+1)-xp(ix)
                           xm2 = o2*(xp(ix-1)+xp(ix))
                           xp2 = o2*(xp(ix)+xp(ix+1))

                           d1 = dys*dzs/hxp*gg2off(ix,1)*gg31off(ix,1)
                           d2 = dzs*dxs/hyp*gg32off(iy,1)/gg2on(ix,1)**2
                           d3 = dxs*dys/hzp/(gg31on(ix,1)**2
     &                          *gg32on(iy,1)**2)
                           d4 = dys*dzs/hxm * gg2off(ix-1,1)
     &                          *gg31off(ix-1,1)   
                           d5 = dzs*dxs/hym*gg32off(iy-1,1)
     &                          / gg2on(ix,1)**2
                           d6 = dxs*dys/hzm/(gg31on(ix,1)**2
     &                          *gg32on(iy,1)**2)

                           dsumi = -o1/(o4*gScaleA*(d1+d2+d3+d4+d5+d6))

                           R(ix,iy,iz) = dsumi*(B(ix,iy,iz) - o4*
     &                          gScaleA*
     &                          ( d1 * (X(ix+1,iy,iz) - X(ix,iy,iz))
     &                          + d2 * (X(ix,iy+1,iz) - X(ix,iy,iz))
     &                          + d3 * (              - X(ix,iy,iz))
     &                          + d4 * (X(ix-1,iy,iz) - X(ix,iy,iz))
     &                          + d5 * (X(ix,iy-1,iz) - X(ix,iy,iz))
     &                          + d6 * (              - X(ix,iy,iz))))
                        END DO
                     END DO
                     
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 64*(ny-2)*(nx-2)
      gmflops(isub)=gmflops(isub) + 
     & 64*(ny-2)*(nx-2)
#endif
                  END IF
               END DO
            END IF

         END IF                 ! diagScalaed

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmBMFResidualGen2'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmBMFResidualNU2 
C     
C     PURPOSE: Matrix-free residual computation for nonuniform meshes
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBMFResidualNU2 (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     diagScaled,lx,ly,lz,mask,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level
      INTEGER diagScaled
      LOGICAL lx,ly,lz,mask(*)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz
      REAL hxm,hym,hzm,hxp,hyp,hzp
      REAL dxs,dys,dzs,dxp,dyp,dzp,dxm,dym,dzm,dsum,dsumi

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESIDUAL2+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmBMFResidualNU2'
#endif

      IF (level.GT.1) THEN

         CALL mmBResidual2 (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        diagScaled,lx,ly,lz,mask,level,isub)
      ELSE

*-----------------------------------------------------------------------
*     Refresh the ghost points of X
*-----------------------------------------------------------------------

         IF (lx) THEN
            CALL mmGetX1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
            CALL mmGetXnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         END IF
         IF (ly) THEN
            CALL mmGetY1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
            CALL mmGetYnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         END IF
         IF (lz) THEN
            CALL mmGetZ1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
            CALL mmGetZnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         END IF

         IF (diagScaled.EQ.0) THEN

            IF ((.NOT.lx).AND.ly.AND.lz) THEN
               dxs = xp(3)-xp(1)
               hxm = xp(2)-xp(1)
               hxp = xp(3)-xp(2)
               DO ix=2,nx-1
                  IF (mask(ix)) THEN
                     dzs = zp(3)-zp(1)
                     hzm = zp(2)-zp(1)
                     hzp = zp(3)-zp(2)
                     DO iz=2,nz-1
                        dys = yp(3)-yp(1)
                        hym = yp(2)-yp(1)
                        hyp = yp(3)-yp(2)
                        DO iy=2,ny-1
                           dxp=dys*dzs/hxp
                           dyp=dzs*dxs/hyp
                           dzp=dxs*dys/hzp
                           dxm=dys*dzs/hxm
                           dym=dzs*dxs/hym
                           dzm=dxs*dys/hzm
                           dsum = dxp + dyp + dzp + dxm + dym + dzm
                           dzp=dzp/dsum
                           dyp=dyp/dsum
                           dzm=dzm/dsum
                           dym=dym/dsum
                           dsum = o4*dsum*gScaleA
                           R(ix,iy,iz) = B(ix,iy,iz) - dsum*(
     &                          - X(ix,iy,iz)
     &                          + dyp * X(ix,iy+1,iz)
     &                          + dzp * X(ix,iy,iz+1)
     &                          + dym * X(ix,iy-1,iz)
     &                          + dzm * X(ix,iy,iz-1))
                           dxs = xp(ix+2)-xp(ix)
                           hxm=hxp
                           hxp=xp(ix+1)-xp(ix)
                        END DO
                        dys = yp(iy+2)-yp(iy)
                        hym=hyp
                        hyp=yp(iy+1)-yp(iy)
                     END DO
                     dzs = zp(iz+2)-zp(iz)
                     hzm=hzp
                     hzp=zp(iz+1)-zp(iz)
                     
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 35*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 35*(ny-2)*(nz-2)
#endif
                  END IF
               END DO
            ELSE IF (lx.AND.(.NOT.ly).AND.lz) THEN
               dys = yp(3)-yp(1)
               hym = yp(2)-yp(1)
               hyp = yp(3)-yp(2)
               DO iy=2,ny-1
                  IF (mask(iy)) THEN
                     dzs = zp(3)-zp(1)
                     hzm = zp(2)-zp(1)
                     hzp = zp(3)-zp(2)
                     DO iz=2,nz-1
                        dxs = xp(3)-xp(1)
                        hxm = xp(2)-xp(1)
                        hxp = xp(3)-xp(2)
                        DO ix=2,nx-1
                           dxp=dys*dzs/hxp
                           dyp=dzs*dxs/hyp
                           dzp=dxs*dys/hzp
                           dxm=dys*dzs/hxm
                           dym=dzs*dxs/hym
                           dzm=dxs*dys/hzm
                           dsum = dxp + dyp + dzp + dxm + dym + dzm
                           dzp=dzp/dsum
                           dxp=dxp/dsum
                           dzm=dzm/dsum
                           dxm=dxm/dsum
                           dsum = o4*dsum*gScaleA
                           R(ix,iy,iz) = B(ix,iy,iz) - dsum*(
     &                          - X(ix,iy,iz)
     &                          + dxp * X(ix+1,iy,iz)
     &                          + dzp * X(ix,iy,iz+1)
     &                          + dxm * X(ix-1,iy,iz)
     &                          + dzm * X(ix,iy,iz-1))
                           dxs = xp(ix+2)-xp(ix)
                           hxm=hxp
                           hxp=xp(ix+1)-xp(ix)
                        END DO
                        dys = yp(iy+2)-yp(iy)
                        hym=hyp
                        hyp=yp(iy+1)-yp(iy)
                     END DO
                     dzs = zp(iz+2)-zp(iz)
                     hzm=hzp
                     hzp=zp(iz+1)-zp(iz)
                     
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 35*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 35*(nx-2)*(nz-2)
#endif
                  END IF
               END DO
            ELSE IF (lx.AND.ly.AND.(.NOT.lz)) THEN
               dzs = zp(3)-zp(1)
               hzm = zp(2)-zp(1)
               hzp = zp(3)-zp(2)
               DO iz=2,nz-1
                  IF (mask(iz)) THEN
                     dys = yp(3)-yp(1)
                     hym = yp(2)-yp(1)
                     hyp = yp(3)-yp(2)
                     DO iy=2,ny-1
                        dxs = xp(3)-xp(1)
                        hxm = xp(2)-xp(1)
                        hxp = xp(3)-xp(2)
                        DO ix=2,nx-1
                           dxp=dys*dzs/hxp
                           dyp=dzs*dxs/hyp
                           dzp=dxs*dys/hzp
                           dxm=dys*dzs/hxm
                           dym=dzs*dxs/hym
                           dzm=dxs*dys/hzm
                           dsum = dxp + dyp + dzp + dxm + dym + dzm
                           dxp=dxp/dsum
                           dyp=dyp/dsum
                           dxm=dxm/dsum
                           dym=dym/dsum
                           dsum = o4*dsum*gScaleA
                           R(ix,iy,iz) = B(ix,iy,iz) - dsum*(
     &                          - X(ix,iy,iz)
     &                          + dxp * X(ix+1,iy,iz)
     &                          + dyp * X(ix,iy+1,iz)
     &                          + dxm * X(ix-1,iy,iz)
     &                          + dym * X(ix,iy-1,iz))
                           dxs = xp(ix+2)-xp(ix)
                           hxm=hxp
                           hxp=xp(ix+1)-xp(ix)
                        END DO
                        dys = yp(iy+2)-yp(iy)
                        hym=hyp
                        hyp=yp(iy+1)-yp(iy)
                     END DO
                     dzs = zp(iz+2)-zp(iz)
                     hzm=hzp
                     hzp=zp(iz+1)-zp(iz)
                     
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 35*(ny-2)*(nx-2)
      gmflops(isub)=gmflops(isub) + 
     & 35*(ny-2)*(nx-2)
#endif
                  END IF
               END DO
            END IF

         ELSE                   ! diagScaled .EQ. 1

            IF ((.NOT.lx).AND.ly.AND.lz) THEN
               dxs = xp(3)-xp(1)
               hxm = xp(2)-xp(1)
               hxp = xp(3)-xp(2)
               DO ix=2,nx-1
                  IF (mask(ix)) THEN
                     dzs = zp(3)-zp(1)
                     hzm = zp(2)-zp(1)
                     hzp = zp(3)-zp(2)
                     DO iz=2,nz-1
                        dys = yp(3)-yp(1)
                        hym = yp(2)-yp(1)
                        hyp = yp(3)-yp(2)
                        DO iy=2,ny-1
                           dxp=dys*dzs/hxp
                           dyp=dzs*dxs/hyp
                           dzp=dxs*dys/hzp
                           dxm=dys*dzs/hxm
                           dym=dzs*dxs/hym
                           dzm=dxs*dys/hzm
                           dsum = dxp + dyp + dzp + dxm + dym + dzm
                           dzp=dzp/dsum
                           dyp=dyp/dsum
                           dzm=dzm/dsum
                           dym=dym/dsum
                           dsum = o4*dsum*gScaleA
                           dsumi = -o1 / dsum
                           R(ix,iy,iz) = dsumi*B(ix,iy,iz) + (
     &                          - X(ix,iy,iz)
     &                          + dyp * X(ix,iy+1,iz)
     &                          + dzp * X(ix,iy,iz+1)
     &                          + dym * X(ix,iy-1,iz)
     &                          + dzm * X(ix,iy,iz-1))
                           dxs = xp(ix+2)-xp(ix)
                           hxm=hxp
                           hxp=xp(ix+1)-xp(ix)
                        END DO
                        dys = yp(iy+2)-yp(iy)
                        hym=hyp
                        hyp=yp(iy+1)-yp(iy)
                     END DO
                     dzs = zp(iz+2)-zp(iz)
                     hzm=hzp
                     hzp=zp(iz+1)-zp(iz)
                     
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 36*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 36*(ny-2)*(nz-2)
#endif
                  END IF
               END DO
            ELSE IF (lx.AND.(.NOT.ly).AND.lz) THEN
               dys = yp(3)-yp(1)
               hym = yp(2)-yp(1)
               hyp = yp(3)-yp(2)
               DO iy=2,ny-1
                  IF (mask(iy)) THEN
                     dzs = zp(3)-zp(1)
                     hzm = zp(2)-zp(1)
                     hzp = zp(3)-zp(2)
                     DO iz=2,nz-1
                        dxs = xp(3)-xp(1)
                        hxm = xp(2)-xp(1)
                        hxp = xp(3)-xp(2)
                        DO ix=2,nx-1
                           dxp=dys*dzs/hxp
                           dyp=dzs*dxs/hyp
                           dzp=dxs*dys/hzp
                           dxm=dys*dzs/hxm
                           dym=dzs*dxs/hym
                           dzm=dxs*dys/hzm
                           dsum = dxp + dyp + dzp + dxm + dym + dzm
                           dzp=dzp/dsum
                           dxp=dxp/dsum
                           dzm=dzm/dsum
                           dxm=dxm/dsum
                           dsum = o4*dsum*gScaleA
                           dsumi = -o1 / dsum
                           R(ix,iy,iz) = dsumi*B(ix,iy,iz) + (
     &                          - X(ix,iy,iz)
     &                          + dxp * X(ix+1,iy,iz)
     &                          + dzp * X(ix,iy,iz+1)
     &                          + dxm * X(ix-1,iy,iz)
     &                          + dzm * X(ix,iy,iz-1))
                           dxs = xp(ix+2)-xp(ix)
                           hxm=hxp
                           hxp=xp(ix+1)-xp(ix)
                        END DO
                        dys = yp(iy+2)-yp(iy)
                        hym=hyp
                        hyp=yp(iy+1)-yp(iy)
                     END DO
                     dzs = zp(iz+2)-zp(iz)
                     hzm=hzp
                     hzp=zp(iz+1)-zp(iz)
                     
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 36*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 36*(nx-2)*(nz-2)
#endif
                  END IF
               END DO
            ELSE IF (lx.AND.ly.AND.(.NOT.lz)) THEN
               dzs = zp(3)-zp(1)
               hzm = zp(2)-zp(1)
               hzp = zp(3)-zp(2)
               DO iz=2,nz-1
                  IF (mask(iz)) THEN
                     dys = yp(3)-yp(1)
                     hym = yp(2)-yp(1)
                     hyp = yp(3)-yp(2)
                     DO iy=2,ny-1
                        dxs = xp(3)-xp(1)
                        hxm = xp(2)-xp(1)
                        hxp = xp(3)-xp(2)
                        DO ix=2,nx-1
                           dxp=dys*dzs/hxp
                           dyp=dzs*dxs/hyp
                           dzp=dxs*dys/hzp
                           dxm=dys*dzs/hxm
                           dym=dzs*dxs/hym
                           dzm=dxs*dys/hzm
                           dsum = dxp + dyp + dzp + dxm + dym + dzm
                           dxp=dxp/dsum
                           dyp=dyp/dsum
                           dxm=dxm/dsum
                           dym=dym/dsum
                           dsum = o4*dsum*gScaleA
                           dsumi = -o1 / dsum
                           R(ix,iy,iz) = dsumi*B(ix,iy,iz) + (
     &                          - X(ix,iy,iz)
     &                          + dxp * X(ix+1,iy,iz)
     &                          + dyp * X(ix,iy+1,iz)
     &                          + dxm * X(ix-1,iy,iz)
     &                          + dym * X(ix,iy-1,iz))
                           dxs = xp(ix+2)-xp(ix)
                           hxm=hxp
                           hxp=xp(ix+1)-xp(ix)
                        END DO
                        dys = yp(iy+2)-yp(iy)
                        hym=hyp
                        hyp=yp(iy+1)-yp(iy)
                     END DO
                     dzs = zp(iz+2)-zp(iz)
                     hzm=hzp
                     hzp=zp(iz+1)-zp(iz)
                     
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 36*(ny-2)*(nx-2)
      gmflops(isub)=gmflops(isub) + 
     & 36*(ny-2)*(nx-2)
#endif
                  END IF
               END DO
            END IF 
         END IF                 ! diagScaled
      END IF                    ! level NE 1

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmBMFResidualNU2'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmBMFResidualU2 
C     
C     PURPOSE: Matrix-free residual computation for uniform meshes
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBMFResidualU2 (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     diagScaled,lx,ly,lz,mask,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      INTEGER comm,nbr(6),bnd(3)

      INTEGER nx,ny,nz
      INTEGER level
      INTEGER diagScaled
      LOGICAL lx,ly,lz,mask(*)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz
      REAL hxm,hym,hzm,hxp,hyp,hzp
      REAL dxs,dys,dzs,dxp,dyp,dzp,dxm,dym,dzm,dsum,dsumi

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESIDUAL2+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> Entering mmBMFResidualU'
#endif

      IF (level.GT.1) THEN

         CALL mmBResidual2 (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        diagScaled,lx,ly,lz,mask,level,isub)

      ELSE

*-----------------------------------------------------------------------
*     Refresh the ghost points of X
*-----------------------------------------------------------------------

         IF (lx) THEN
            CALL mmGetX1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
            CALL mmGetXnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         END IF
         IF (ly) THEN
            CALL mmGetY1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
            CALL mmGetYnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         END IF
         IF (lz) THEN
            CALL mmGetZ1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
            CALL mmGetZnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         END IF

         IF (diagScaled.EQ.0) THEN

            dzs = zp(3)-zp(1)
            hzm = zp(2)-zp(1)
            hzp = zp(3)-zp(2)
            dys = yp(3)-yp(1)
            hym = yp(2)-yp(1)
            hyp = yp(3)-yp(2)
            dxs = xp(3)-xp(1)
            hxm = xp(2)-xp(1)
            hxp = xp(3)-xp(2)
            dzp=dxs*dys/hzp
            dxp=dys*dzs/hxp
            dyp=dzs*dxs/hyp
            dzm=dxs*dys/hzm
            dxm=dys*dzs/hxm
            dym=dzs*dxs/hym
            dsum = dxp + dyp + dzp + dxm + dym + dzm
            dzp=dzp/dsum
            dxp=dxp/dsum
            dyp=dyp/dsum
            dzm=dzm/dsum
            dxm=dxm/dsum
            dym=dym/dsum
            dsum = o4*dsum*gScaleA
            dsumi = -1.0D0 / dsum
            IF ((.NOT.lx).AND.ly.AND.lz) THEN
               DO ix=2,nx-1
                  IF (mask(ix)) THEN
                     DO iz=2,nz-1
                        DO iy=2,ny-1
                           R(ix,iy,iz) = B(ix,iy,iz) - dsum*(
     &                          - X(ix,iy,iz)
     &                          + dyp * X(ix,iy+1,iz)
     &                          + dzp * X(ix,iy,iz+1)
     &                          + dym * X(ix,iy-1,iz)
     &                          + dzm * X(ix,iy,iz-1))
                        END DO
                     END DO
                     
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 10*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 10*(ny-2)*(nz-2)
#endif
                  END IF
               END DO
            ELSE IF (lx.AND.(.NOT.ly).AND.lz) THEN
               DO iy=2,ny-1
                  IF (mask(iy)) THEN
                     DO iz=2,nz-1
                        DO ix=2,nx-1
                           R(ix,iy,iz) = B(ix,iy,iz) - dsum*(
     &                          - X(ix,iy,iz)
     &                          + dxp * X(ix+1,iy,iz)
     &                          + dzp * X(ix,iy,iz+1)
     &                          + dxm * X(ix-1,iy,iz)
     &                          + dzm * X(ix,iy,iz-1))
                        END DO
                     END DO
                     
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 10*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 10*(nx-2)*(nz-2)
#endif
                  END IF
               END DO
            ELSE IF (lx.AND.ly.AND.(.NOT.lz)) THEN
               DO iz=2,nz-1
                  IF (mask(iz)) THEN
                     DO iy=2,ny-1
                        DO ix=2,nx-1
                           R(ix,iy,iz) = B(ix,iy,iz) - dsum*(
     &                          - X(ix,iy,iz)
     &                          + dxp * X(ix+1,iy,iz)
     &                          + dyp * X(ix,iy+1,iz)
     &                          + dxm * X(ix-1,iy,iz)
     &                          + dym * X(ix,iy-1,iz))
                        END DO
                     END DO
                     
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 10*(nx-2)*(ny-2)
      gmflops(isub)=gmflops(isub) + 
     & 10*(nx-2)*(ny-2)
#endif
                  END IF
               END DO
            END IF
         ELSE                   ! diagScaled .EQ. 1
            IF ((.NOT.lx).AND.ly.AND.lz) THEN
               DO ix=2,nx-1
                  IF (mask(ix)) THEN
                     DO iz=2,nz-1
                        DO iy=2,ny-1
                           R(ix,iy,iz) = dsumi*B(ix,iy,iz) + (
     &                          - X(ix,iy,iz)
     &                          + dyp * X(ix,iy+1,iz)
     &                          + dzp * X(ix,iy,iz+1)
     &                          + dym * X(ix,iy-1,iz)
     &                          + dzm * X(ix,iy,iz-1))
                        END DO
                     END DO
                     
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 10*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 10*(ny-2)*(nz-2)
#endif
                  END IF
               END DO
            ELSE IF (lx.AND.(.NOT.ly).AND.lz) THEN
               DO iy=2,ny-1
                  IF (mask(iy)) THEN
                     DO iz=2,nz-1
                        DO ix=2,nx-1
                           R(ix,iy,iz) = dsumi*B(ix,iy,iz) + (
     &                          - X(ix,iy,iz)
     &                          + dxp * X(ix+1,iy,iz)
     &                          + dzp * X(ix,iy,iz+1)
     &                          + dxm * X(ix-1,iy,iz)
     &                          + dzm * X(ix,iy,iz-1))
                        END DO
                     END DO
                     
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 10*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 10*(nx-2)*(nz-2)
#endif
                  END IF
               END DO
            ELSE IF (lx.AND.ly.AND.(.NOT.lz)) THEN
               DO iz=2,nz-1
                  IF (mask(iz)) THEN
                     DO iy=2,ny-1
                        DO ix=2,nx-1
                           R(ix,iy,iz) = dsumi*B(ix,iy,iz) + (
     &                          - X(ix,iy,iz)
     &                          + dxp * X(ix+1,iy,iz)
     &                          + dyp * X(ix,iy+1,iz)
     &                          + dxm * X(ix-1,iy,iz)
     &                          + dym * X(ix,iy-1,iz))
                        END DO
                     END DO
                     
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 10*(nx-2)*(ny-2)
      gmflops(isub)=gmflops(isub) + 
     & 10*(nx-2)*(ny-2)
#endif
                  END IF
               END DO
            END IF
         END IF                 ! diagScaled

      END IF                    ! level NE 1
      
      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< Entering mmBMFResidualU'
#endif

      RETURN
      END
      
C=======================================================================
C     
C     Module:  mmRestrict
C     
C     Purpose: Call appropriate restriction subroutine
C     
C     Inputs:  XF: fine grid function
C     
C     Outputs: XC: coarse grid function
C     
C     Globals accessed:
C     
C     Globals modified:
C     
C     Calls:
C     
C     Called by:
C     
C     Flop count estimate:
C     
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C     
C     Modification history: 
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmRestrict (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,
     &     nxc,nyc,nzc,nxf,nyf,nzf,
     &     hxf,hyf,hzf,comm,nbr,bndc,bndf,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdxc,nydxc,nzdxc
      INTEGER bndxc(3)  
      REAL XC(nxdxc,nydxc,nzdxc)
      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      REAL hxf(*),hyf(*),hzf(*)
      INTEGER comm,nbr(6),bndc(3),bndf(3),level

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER idim

#ifdef DEBUG
      REAL s,mmVectorSum
      EXTERNAL mmVectorSum
#endif

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESTRICT
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef DEBUG
      IF (ip.EQ.0) print*, ip, 'mmRestrict: entering'
#endif
#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmRestrict'
#endif

************************************************************************
*     Restrict XF to XC 
************************************************************************

      idim = 0

      IF (gmmlc1) idim = idim + 1
      IF (gmmlc2) idim = idim + 1
      IF (gmmlc3) idim = idim + 1

      IF (idim .EQ. 3) THEN

         IF (gicoords.EQ.MM_CARTESIAN) THEN
#ifdef DEBUG
            IF (ip.EQ.0) print*, ip, 'mmRestrict: calling mmRestrictU'
#endif
            CALL mmRestrictU (
     &           
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &           hxf,hyf,hzf,comm,nbr,bndc,bndf,level,isub)
         ELSE  
#ifdef DEBUG
            IF (ip.EQ.0) print*, ip, 'mmRestrict: calling mmRestrictGen'
#endif
            CALL mmRestrictGen (
     &           
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &           hxf,hyf,hzf,comm,nbr,bndc,bndf,level,isub)
         END IF

      ELSE

         IF (gicoords.EQ.MM_CARTESIAN) THEN

#ifdef DEBUG
            IF (ip.EQ.0) print*, ip, 'mmRestrict: calling mmRestrictSCU'
#endif
            CALL mmRestrictSCU (
     &           
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &           hxf,hyf,hzf,comm,nbr,bndc,bndf,level,isub)
         ELSE

#ifdef DEBUG
            IF (ip.EQ.0) print*, ip, 'mmRestrict: calling mmRestrictGen'
#endif
            CALL mmRestrictSCGen (
     &           
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &           hxf,hyf,hzf,comm,nbr,bndc,bndf,level,isub)
         END IF

      END IF

#ifdef DEBUG
      s = mmVectorSum (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,comm,isub)
      if (ip.EQ.0) PRINT*, 'mmRestrict sum(XC) = ',s
#endif
#ifdef DEBUG
      IF (ip.EQ.0) print*, ip, 'mmRestrict: exiting'
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmRestrict'
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  mmRestrictGen
C     
C     Purpose: Transfer a 3-D grid function to the next coarser grid 
C
C     Algorithm: Coordinate-transformed 7-point weighted injection
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     2.5 (nxf nyf nzf)
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmRestrictGen (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &     hxf,hyf,hzf,comm,nbr,bndc,bndf,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdxc,nydxc,nzdxc
      INTEGER bndxc(3)  
      REAL XC(nxdxc,nydxc,nzdxc)
      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      REAL hxf(*),hyf(*),hzf(*)
      INTEGER comm,nbr(6),bndc(3),bndf(3),level

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

************************************************************************
*     Restrict XF to XC using 7-point averaging.  Assumes a uniform grid
*     (h(i) .EQ. h(i+1))
**************************************************************************

      INTEGER ixc,iyc,izc,ixf,iyf,izf
      REAL wD,w0,w1,wxm,wxp,wym,wyp,wzm,wzp

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESTRICT+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmRestrictGen'
#endif
      w0 = 2.0D0
      w1 = 1.0D0

      wD = w0 + 6.0D0*w1
      w0 = w0 / wD
      w1 = w1 / wD

*-----------------------------------------------------------------------
*     Get fine-grid boundary elements
*-----------------------------------------------------------------------

      CALL mmGetXnGhost (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
      CALL mmGetYnGhost (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
      CALL mmGetZnGhost (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

*-----------------------------------------------------------------------
*     Restrict XC <-- XF
*-----------------------------------------------------------------------

      DO izc=2, nzc-1
         izf = 2*(izc-1)+1
         wzm = gdzi(izf-1,level)
         wzp = gdzi(izf,level)
         DO iyc=2, nyc-1
            iyf = 2*(iyc-1)+1
            wym = gdyi(iyf-1,level)
            wyp = gdyi(iyf,level)
            DO ixc=2, nxc-1
               ixf = 2*(ixc-1)+1
               wxm = gdxi(ixf-1,level)
               wxp = gdxi(ixf,level)
               XC(ixc,iyc,izc) =
     &              w0 *XF(ixf,iyf,izf) +
     &              w1*(wxm*XF(ixf-1,iyf,izf) 
     &              +   wxp*XF(ixf+1,iyf,izf)
     &              +   wym*XF(ixf,iyf-1,izf) 
     &              +   wyp*XF(ixf,iyf+1,izf)
     &              +   wzm*XF(ixf,iyf,izf-1) 
     &              +   wzp*XF(ixf,iyf,izf+1))
     &            / ( wxm + wxp + wym + wyp + wzm + wzp)
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 22*(nxc-2)*(nyc-2)*(nzc-2)
      gmflops(isub)=gmflops(isub) + 
     & 22*(nxc-2)*(nyc-2)*(nzc-2)
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmRestrictGen'
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmRestrictU (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &     hxf,hyf,hzf,comm,nbr,bndc,bndf,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdxc,nydxc,nzdxc
      INTEGER bndxc(3)  
      REAL XC(nxdxc,nydxc,nzdxc)
      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      REAL hxf(*),hyf(*),hzf(*)
      INTEGER comm,nbr(6),bndc(3),bndf(3),level

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ixc,iyc,izc,ixf,iyf,izf
      REAL WD,W0,W1

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif
      
      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESTRICT+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmRestrictU'
#endif
      W0 = 6.0D0
      W1 = 1.0D0

      WD = W0 + 6.0D0*W1
      W0 = W0 / WD
      W1 = W1 / WD

*-----------------------------------------------------------------------
*     Get fine-grid boundary elements
*-----------------------------------------------------------------------

      CALL mmGetXnGhost (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
      CALL mmGetYnGhost (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
      CALL mmGetZnGhost (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

*-----------------------------------------------------------------------
*     Restrict XC <-- XF
*-----------------------------------------------------------------------

      DO izc=2, nzc-1
         izf = 2*(izc-1)+1
         DO iyc=2, nyc-1
            iyf = 2*(iyc-1)+1
            DO ixc=2, nxc-1
               ixf = 2*(ixc-1)+1
               XC(ixc,iyc,izc) =
     &                W0* XF(ixf,iyf,izf)
     &              + W1*(XF(ixf-1,iyf,izf) + XF(ixf+1,iyf,izf)
     &              +     XF(ixf,iyf-1,izf) + XF(ixf,iyf+1,izf)
     &              +     XF(ixf,iyf,izf-1) + XF(ixf,iyf,izf+1))
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 8*(nxc-2)*(nyc-2)*(nzc-2)
      gmflops(isub)=gmflops(isub) + 
     & 8*(nxc-2)*(nyc-2)*(nzc-2)
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmRestrictU'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmRestrict2
C     
C     PURPOSE: Transfer a grid function to the next-coarser 2D grid level
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmRestrict2 (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &     hxf,hyf,hzf,comm,nbr,bndc,bndf,lx,ly,lz,mask,level,
     &     icall)

      IMPLICIT NONE

      
      INTEGER nxdxc,nydxc,nzdxc
      INTEGER bndxc(3)  
      REAL XC(nxdxc,nydxc,nzdxc)
      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      REAL hxf(*),hyf(*),hzf(*)
      INTEGER comm,nbr(6),bndc(3),bndf(3),level
      LOGICAL lx,ly,lz,mask(*)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

#ifdef DEBUG
      REAL s,mmVectorSum
      EXTERNAL mmVectorSum
#endif

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESTRICT2
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef DEBUG
      IF (ip.EQ.0) print*, ip, 'mmRestrict2: entering'
#endif
#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmRestrict2'
#endif

#ifdef ASSERT
      IF (.NOT.(((.NOT.lx).AND.ly.AND.lz) .OR.
     &          (lx.AND.(.NOT.ly).AND.lz) .OR.
     &          (lx.AND.ly.AND.(.NOT.lz)))) THEN
         PRINT*, 'mmRestrict2 ASSERTION FAILED!'
         PRINT*, 'lx ly lz =',lx,ly,lz
      END IF
#endif
************************************************************************
*     Restrict XF to XC 
************************************************************************

      IF (gicoords.EQ.MM_CARTESIAN) THEN
#ifdef DEBUG
      IF (ip.EQ.0) print*, ip, 'mmRestrict2: calling mmRestrictU2'
#endif
      CALL mmRestrictU2 (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &     hxf,hyf,hzf,comm,nbr,bndc,bndf,lx,ly,lz,mask,level,
     &     isub)
      ELSE  
#ifdef DEBUG
      IF (ip.EQ.0) print*, ip, 'mmRestrict2: calling mmRestrictGen2'
#endif
      CALL mmRestrictGen2 (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &     hxf,hyf,hzf,comm,nbr,bndc,bndf,lx,ly,lz,mask,level,
     &     isub)
      END IF
#ifdef DEBUG
         s = mmVectorSum (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,nxc,nyc,nzc,comm,isub)
         if (ip.EQ.0) PRINT*, 'mmRestrict2 sum(XC) = ',s
#endif
#ifdef DEBUG
      IF (ip.EQ.0) print*, ip, 'mmRestrict2: exiting'
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmRestrict2'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmRestrictGen2
C     
C     PURPOSE: Transfer a grid function to the next-coarser 2D grid level
C     PURPOSE: for general problems 
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmRestrictGen2 (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &     hxf,hyf,hzf,comm,nbr,bndc,bndf,lx,ly,lz,mask,level,
     &     icall)

      IMPLICIT NONE

      
      INTEGER nxdxc,nydxc,nzdxc
      INTEGER bndxc(3)  
      REAL XC(nxdxc,nydxc,nzdxc)
      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      REAL hxf(*),hyf(*),hzf(*)
      INTEGER comm,nbr(6),bndc(3),bndf(3),level
      LOGICAL lx,ly,lz,mask(*)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

************************************************************************
*     Restrict XF to XC in 2D using 5-point averaging.
**************************************************************************

      INTEGER ixc,iyc,izc,ixf,iyf,izf
      REAL wD,w0,w1,wxm,wxp,wym,wyp,wzm,wzp

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESTRICT2+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmRestrictGen2'
#endif

#ifdef ASSERT
      IF (.NOT.(((.NOT.lx).AND.ly.AND.lz) .OR.
     &          (lx.AND.(.NOT.ly).AND.lz) .OR.
     &          (lx.AND.ly.AND.(.NOT.lz)))) THEN
         PRINT*, 'mmRestrictGen2 ASSERTION FAILED!'
         PRINT*, 'lx ly lz =',lx,ly,lz
      END IF
#endif

      w0 = 2.0D0
      w1 = 1.0D0

      wD = w0 + 4.0D0*w1
      w0 = w0 / wD
      w1 = w1 / wD

*-----------------------------------------------------------------------
*     Get fine-grid boundary elements
*-----------------------------------------------------------------------

      IF (lx) THEN
         CALL mmGetXnGhost2 (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,lx,ly,lz,isub)
      END IF
      IF (ly) THEN
         CALL mmGetYnGhost2 (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,lx,ly,lz,isub)
      END IF
      IF (lz) THEN
         CALL mmGetZnGhost2 (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,lx,ly,lz,isub)
      END IF

*-----------------------------------------------------------------------
*     Restrict XC <-- XF
*-----------------------------------------------------------------------

      IF (.NOT.lx) THEN
         DO ixc=2, nxc-1
            IF (mask(ixc)) THEN
               ixf = ixc
               DO izc=2, nzc-1
                  izf = 2*(izc-1)+1
                  wzm = gdzi(izf-1,level)
                  wzp = gdzi(izf,level)
                  DO iyc=2, nyc-1
                     iyf = 2*(iyc-1)+1
                     wym = gdyi(iyf-1,level)
                     wyp = gdyi(iyf,level)
                     XC(ixc,iyc,izc) =
     &                    w0 *XF(ixf,iyf,izf) +
     &                    w1*(wym*XF(ixf,iyf-1,izf) 
     &                    +   wyp*XF(ixf,iyf+1,izf)
     &                    +   wzm*XF(ixf,iyf,izf-1) 
     &                    +   wzp*XF(ixf,iyf,izf+1))
     &                    / ( wym + wyp + wzm + wzp)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 16*(nzc-2)*(nyc-2)
      gmflops(isub)=gmflops(isub) + 
     & 16*(nzc-2)*(nyc-2)
#endif
            END IF
         END DO
      ELSE IF (.NOT.ly) THEN
         DO iyc=2, nyc-1
            IF (mask(iyc)) THEN
               iyf = iyc
               DO izc=2, nzc-1
                  izf = 2*(izc-1)+1
                  wzm = gdzi(izf-1,level)
                  wzp = gdzi(izf,level)
                  DO ixc=2, nxc-1
                     ixf = 2*(ixc-1)+1
                     wxm = gdxi(ixf-1,level)
                     wxp = gdxi(ixf,level)
                     XC(ixc,iyc,izc) =
     &                    w0 *XF(ixf,iyf,izf) +
     &                    w1*(wxm*XF(ixf-1,iyf,izf) 
     &                    +   wxp*XF(ixf+1,iyf,izf)
     &                    +   wzm*XF(ixf,iyf,izf-1) 
     &                    +   wzp*XF(ixf,iyf,izf+1))
     &                    / ( wxm + wxp + wzm + wzp)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 16*(nxc-2)*(nzc-2)
      gmflops(isub)=gmflops(isub) + 
     & 16*(nxc-2)*(nzc-2)
#endif
            END IF
         END DO
      ELSE IF (.NOT.ly) THEN
         DO izc=2, nzc-1
            IF (mask(izc)) THEN
               izf = izc
               DO iyc=2, nyc-1
                  iyf = 2*(iyc-1)+1
                  wym = gdyi(iyf-1,level)
                  wyp = gdyi(iyf,level)
                  DO ixc=2, nxc-1
                     ixf = 2*(ixc-1)+1
                     wxm = gdxi(ixf-1,level)
                     wxp = gdxi(ixf,level)
                     XC(ixc,iyc,izc) =
     &                    w0 *XF(ixf,iyf,izf) +
     &                    w1*(wxm*XF(ixf-1,iyf,izf) 
     &                    +   wxp*XF(ixf+1,iyf,izf)
     &                    +   wym*XF(ixf,iyf-1,izf) 
     &                    +   wyp*XF(ixf,iyf+1,izf))
     &                    / ( wxm + wxp + wym + wyp )
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 16*(nxc-2)*(nyc-2)
      gmflops(isub)=gmflops(isub) + 
     & 16*(nxc-2)*(nyc-2)
#endif
            END IF
         END DO
      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmRestrictGen2'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmRestrictU2
C     
C     PURPOSE: Transfer a grid function to the next-coarser 2D grid level
C     PURPOSE: for uniform meshes
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmRestrictU2 (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &     hxf,hyf,hzf,comm,nbr,bndc,bndf,lx,ly,lz,mask,level,
     &     icall)

      IMPLICIT NONE

      
      INTEGER nxdxc,nydxc,nzdxc
      INTEGER bndxc(3)  
      REAL XC(nxdxc,nydxc,nzdxc)
      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      REAL hxf(*),hyf(*),hzf(*)
      INTEGER comm,nbr(6),bndc(3),bndf(3),level
      LOGICAL lx,ly,lz,mask(*)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ixc,iyc,izc,ixf,iyf,izf
      REAL WD,W0,W1

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif
      
      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESTRICT2+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmRestrictU2'
#endif

#ifdef ASSERT
      IF (.NOT.(((.NOT.lx).AND.ly.AND.lz) .OR.
     &     (lx.AND.(.NOT.ly).AND.lz) .OR.
     &     (lx.AND.ly.AND.(.NOT.lz)))) THEN
         PRINT*, 'mmRestrictU2 ASSERTION FAILED!'
         PRINT*, 'lx ly lz =',lx,ly,lz
      END IF
#endif

      W0 = 2.0D0
      W1 = 1.0D0

      WD = W0 + 4.0D0*W1
      W0 = W0 / WD
      W1 = W1 / WD

*-----------------------------------------------------------------------
*     Get fine-grid boundary elements
*-----------------------------------------------------------------------

      IF (lx) THEN
         CALL mmGetXnGhost2 (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,lx,ly,lz,isub)
      END IF
      IF (ly) THEN
         CALL mmGetYnGhost2 (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,lx,ly,lz,isub)
      END IF
      IF (lz) THEN
         CALL mmGetZnGhost2 (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,lx,ly,lz,isub)
      END IF

*-----------------------------------------------------------------------
*     Restrict XC <-- XF
*-----------------------------------------------------------------------

      IF (.NOT.lx) THEN
         DO ixc=2, nxc-1
            IF (mask(ixc)) THEN
               ixf = ixc
               DO izc=2, nzc-1
                  izf = 2*(izc-1)+1
                  DO iyc=2, nyc-1
                     iyf = 2*(iyc-1)+1
                     XC(ixc,iyc,izc) = W0* XF(ixf,iyf,izf)
     &                    + W1*(XF(ixf,iyf-1,izf) + XF(ixf,iyf+1,izf)
     &                    +     XF(ixf,iyf,izf-1) + XF(ixf,iyf,izf+1))
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 6*(nyc-2)*(nzc-2)
      gmflops(isub)=gmflops(isub) + 
     & 6*(nyc-2)*(nzc-2)
#endif
            END IF
         END DO
      ELSE IF (.NOT.ly) THEN
         DO iyc=2, nyc-1
            IF (mask(iyc)) THEN
               iyf = iyc
               DO izc=2, nzc-1
                  izf = 2*(izc-1)+1
                  DO ixc=2, nxc-1
                     ixf = 2*(ixc-1)+1
                     XC(ixc,iyc,izc) = W0* XF(ixf,iyf,izf)
     &                    + W1*(XF(ixf-1,iyf,izf) + XF(ixf+1,iyf,izf)
     &                    +     XF(ixf,iyf,izf-1) + XF(ixf,iyf,izf+1))
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 6*(nxc-2)*(nzc-2)
      gmflops(isub)=gmflops(isub) + 
     & 6*(nxc-2)*(nzc-2)
#endif
            END IF
         END DO
      ELSE IF (.NOT.lz) THEN
         DO izc=2, nzc-1
            IF (mask(izc)) THEN
               izf = izc
               DO iyc=2, nyc-1
                  iyf = 2*(iyc-1)+1
                  DO ixc=2, nxc-1
                     ixf = 2*(ixc-1)+1
                     XC(ixc,iyc,izc) = W0*XF(ixf,iyf,izf)
     &                    + W1*(XF(ixf-1,iyf,izf) + XF(ixf+1,iyf,izf)
     &                    +     XF(ixf,iyf-1,izf) + XF(ixf,iyf+1,izf))
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 6*(nxc-2)*(nyc-2)
      gmflops(isub)=gmflops(isub) + 
     & 6*(nxc-2)*(nyc-2)
#endif
            END IF
         END DO
      END IF


      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmRestrictU2'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictSCU
C     
C     Purpose: Transfer a 3-D grid function to the next semi-coarsened grid 
C
C     Algorithm: 5-point or 3-point weighted average (2-D or 1-D 
C     Algorithm: half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmRestrictSCU (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &     hxf,hyf,hzf,comm,nbr,bndc,bndf,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdxc,nydxc,nzdxc
      INTEGER bndxc(3)  
      REAL XC(nxdxc,nydxc,nzdxc)
      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      REAL hxf(*),hyf(*),hzf(*)
      INTEGER comm,nbr(6),bndc(3),bndf(3),level

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ixc,iyc,izc,ixf,iyf,izf,idim
      REAL WD,W0,W1

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif
      
      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESTRICT+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmRestrictSCU'
#endif

      idim = 0

      IF (gmmlc1) idim = idim + 1
      IF (gmmlc2) idim = idim + 1
      IF (gmmlc3) idim = idim + 1

      IF (idim.EQ.3) THEN

         PRINT*, '*** MGMPI ***: mmRestrictSCU called as full'
         PRINT*, '               coarsening restriction!'
         PRINT*, '               Calling mmRestrictU...'
         CALL mmRestrictU (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &        hxf,hyf,hzf,comm,nbr,bndc,bndf,level,icall)

         GOTO 10

      ELSE IF (idim.EQ.2) THEN
                  
         W0 = 4.0D0
         W1 = 1.0D0

         WD = W0 + 4.0D0*W1
         W0 = W0 / WD
         W1 = W1 / WD

      ELSE IF (idim.EQ.1) THEN

         W0 = 2.0D0
         W1 = 1.0D0

         WD = W0 + 2.0D0*W1
         W0 = W0 / WD
         W1 = W1 / WD

      END IF

*-----------------------------------------------------------------------
*     Get required fine-grid boundary elements
*-----------------------------------------------------------------------

      IF (gmmlc1) CALL mmGetXnGhost (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
      IF (gmmlc2) CALL mmGetYnGhost (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
      IF (gmmlc3) CALL mmGetZnGhost (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

*-----------------------------------------------------------------------
*     2D Restrict XC <-- XF along Y- and Z- axes
*-----------------------------------------------------------------------

      IF ((.NOT.gmmlc1) .AND. gmmlc2 .AND .gmmlc3) THEN

         DO izc=2, nzc-1
            izf = 2*(izc-1)+1
            DO iyc=2, nyc-1
               iyf = 2*(iyc-1)+1
               DO ixc=2, nxc-1
                  ixf = ixc
                  XC(ixc,iyc,izc) =
     &                 W0* XF(ixf,iyf,izf)
     &                 + W1*(XF(ixf,iyf-1,izf) + XF(ixf,iyf+1,izf)
     &                 +     XF(ixf,iyf,izf-1) + XF(ixf,iyf,izf+1))
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 6*(nxc-2)*(nyc-2)*(nzc-2)
      gmflops(isub)=gmflops(isub) + 
     & 6*(nxc-2)*(nyc-2)*(nzc-2)
#endif

*-----------------------------------------------------------------------
*     2D Restrict XC <-- XF along X- and Z- axes
*-----------------------------------------------------------------------

      ELSE IF (gmmlc1 .AND. (.NOT.gmmlc2) .AND. gmmlc3)  THEN

         DO izc=2, nzc-1
            izf = 2*(izc-1)+1
            DO iyc=2, nyc-1
               iyf = iyc
               DO ixc=2, nxc-1
                  ixf = 2*(ixc-1)+1
                  XC(ixc,iyc,izc) =
     &                 W0* XF(ixf,iyf,izf)
     &                 + W1*(XF(ixf-1,iyf,izf) + XF(ixf+1,iyf,izf)
     &                 +     XF(ixf,iyf,izf-1) + XF(ixf,iyf,izf+1))
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 6*(nxc-2)*(nyc-2)*(nzc-2)
      gmflops(isub)=gmflops(isub) + 
     & 6*(nxc-2)*(nyc-2)*(nzc-2)
#endif

*-----------------------------------------------------------------------
*     2D Restrict XC <-- XF along X- and Y- axes
*-----------------------------------------------------------------------

      ELSE IF (gmmlc1 .AND. gmmlc2 .AND. (.NOT.gmmlc3))  THEN

         DO izc=2, nzc-1
            izf = izc
            DO iyc=2, nyc-1
               iyf = 2*(iyc-1)+1
               DO ixc=2, nxc-1
                  ixf = 2*(ixc-1)+1
                  XC(ixc,iyc,izc) =
     &                 W0* XF(ixf,iyf,izf)
     &                 + W1*(XF(ixf-1,iyf,izf) + XF(ixf+1,iyf,izf)
     &                 +     XF(ixf,iyf-1,izf) + XF(ixf,iyf+1,izf))
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 6*(nxc-2)*(nyc-2)*(nzc-2)
      gmflops(isub)=gmflops(isub) + 
     & 6*(nxc-2)*(nyc-2)*(nzc-2)
#endif

*-----------------------------------------------------------------------
*    1D Restrict XC <-- XF along X-
*-----------------------------------------------------------------------

      ELSE IF (gmmlc1 .AND. (.NOT.gmmlc2) .AND. (.NOT.gmmlc3)) THEN

         DO izc=2, nzc-1
            izf = izc
            DO iyc=2, nyc-1
               iyf = iyc
               DO ixc=2, nxc-1
                  ixf = 2*(ixc-1)+1
                  XC(ixc,iyc,izc) =
     &                 W0* XF(ixf,iyf,izf)
     &                 + W1*(XF(ixf-1,iyf,izf) + XF(ixf+1,iyf,izf))
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 4*(nxc-2)*(nyc-2)*(nzc-2)
      gmflops(isub)=gmflops(isub) + 
     & 4*(nxc-2)*(nyc-2)*(nzc-2)
#endif

*-----------------------------------------------------------------------
*    1D Restrict XC <-- XF along X-
*-----------------------------------------------------------------------

      ELSE IF ((.NOT.gmmlc1) .AND. gmmlc2 .AND. (.NOT.gmmlc3)) THEN

         DO izc=2, nzc-1
            izf = izc
            DO iyc=2, nyc-1
               iyf = 2*(iyc-1)+1
               DO ixc=2, nxc-1
                  ixf = ixc
                  XC(ixc,iyc,izc) =
     &                 W0* XF(ixf,iyf,izf)
     &                 + W1*(XF(ixf,iyf-1,izf) + XF(ixf,iyf+1,izf))
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 4*(nxc-2)*(nyc-2)*(nzc-2)
      gmflops(isub)=gmflops(isub) + 
     & 4*(nxc-2)*(nyc-2)*(nzc-2)
#endif

*-----------------------------------------------------------------------
*    1D Restrict XC <-- XF along X-
*-----------------------------------------------------------------------

      ELSE IF ((.NOT.gmmlc1) .AND. (.NOT.gmmlc2) .AND. gmmlc3) THEN

         DO izc=2, nzc-1
            izf = 2*(izc-1)+1
            DO iyc=2, nyc-1
               iyf = iyc
               DO ixc=2, nxc-1
                  ixf = ixc
                  XC(ixc,iyc,izc) =
     &                 W0* XF(ixf,iyf,izf)
     &                 + W1*(XF(ixf,iyf,izf-1) + XF(ixf,iyf,izf+1))
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 4*(nxc-2)*(nyc-2)*(nzc-2)
      gmflops(isub)=gmflops(isub) + 
     & 4*(nxc-2)*(nyc-2)*(nzc-2)
#endif

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

 10   CONTINUE

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmRestrictSCU'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictSCU
C     
C     Purpose: Transfer a 3-D grid function to the next semi-coarsened grid 
C
C     Algorithm: 5-point or 3-point weighted average (2-D or 1-D 
C     Algorithm: half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmRestrictSCGen (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &     hxf,hyf,hzf,comm,nbr,bndc,bndf,level,icall)

      IMPLICIT NONE

      
      INTEGER nxdxc,nydxc,nzdxc
      INTEGER bndxc(3)  
      REAL XC(nxdxc,nydxc,nzdxc)
      
      INTEGER nxdxf,nydxf,nzdxf
      INTEGER bndxf(3)
      REAL XF(nxdxf,nydxf,nzdxf)
      INTEGER nxc,nyc,nzc,nxf,nyf,nzf
      REAL hxf(*),hyf(*),hzf(*)
      INTEGER comm,nbr(6),bndc(3),bndf(3),level

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ixc,iyc,izc,ixf,iyf,izf,idim
      REAL wD,w0,w1,wxm,wxp,wym,wyp,wzm,wzp

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif
      
      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_RESTRICT+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmRestrictSCGen'
#endif

      idim = 0

      IF (gmmlc1) idim = idim + 1
      IF (gmmlc2) idim = idim + 1
      IF (gmmlc3) idim = idim + 1

      IF (idim.EQ.3) THEN

         PRINT*, '*** MGMPI ***: mmRestrictSCGen called as full'
         PRINT*, '               coarsening restriction!'
         PRINT*, '               Calling mmRestrictU...'
         CALL mmRestrictGen (
     &     XC,nxdxc,nydxc,nzdxc,bndxc
     &     ,
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxc,nyc,nzc,nxf,nyf,nzf,
     &        hxf,hyf,hzf,comm,nbr,bndc,bndf,level,icall)

         GOTO 10

      ELSE IF (idim.EQ.2) THEN
                  
         w0 = 4.0D0
         w1 = 1.0D0

         wD = w0 + 4.0D0*w1
         w0 = w0 / wD
         w1 = w1 / wD

      ELSE IF (idim.EQ.1) THEN

         w0 = 2.0D0
         w1 = 1.0D0

         wD = w0 + 2.0D0*w1
         w0 = w0 / wD
         w1 = w1 / wD

      END IF

*-----------------------------------------------------------------------
*     Get required fine-grid boundary elements
*-----------------------------------------------------------------------

      IF (gmmlc1) CALL mmGetXnGhost (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
      IF (gmmlc2) CALL mmGetYnGhost (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)
      IF (gmmlc3) CALL mmGetZnGhost (
     &     XF,nxdxf,nydxf,nzdxf,bndxf
     &     ,nxf,nyf,nzf,
     &     comm,nbr,bndf
     &     ,isub)

*-----------------------------------------------------------------------
*     2D Restrict XC <-- XF along Y- and Z- axes
*-----------------------------------------------------------------------

      IF ((.NOT.gmmlc1) .AND. gmmlc2 .AND .gmmlc3) THEN

         DO izc=2, nzc-1
            izf = 2*(izc-1)+1
            wzm = gdzi(izf-1,level)
            wzp = gdzi(izf,level)
            DO iyc=2, nyc-1
               iyf = 2*(iyc-1)+1
               wym = gdyi(iyf-1,level)
               wyp = gdyi(iyf,level)
               DO ixc=2, nxc-1
                  ixf = ixc
                  XC(ixc,iyc,izc) =
     &                 w0* XF(ixf,iyf,izf)
     &                 + w1*(wym*XF(ixf,iyf-1,izf) 
     &                 +     wyp*XF(ixf,iyf+1,izf)
     &                 +     wzm*XF(ixf,iyf,izf-1) 
     &                 +     wzp*XF(ixf,iyf,izf+1))
     &                 / (wym + wyp + wzm + wzp)
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 14*(nxc-2)*(nyc-2)*(nzc-2)
      gmflops(isub)=gmflops(isub) + 
     & 14*(nxc-2)*(nyc-2)*(nzc-2)
#endif

*-----------------------------------------------------------------------
*     2D Restrict XC <-- XF along X- and Z- axes
*-----------------------------------------------------------------------

      ELSE IF (gmmlc1 .AND. (.NOT.gmmlc2) .AND. gmmlc3)  THEN

         DO izc=2, nzc-1
            izf = 2*(izc-1)+1
            wzm = gdzi(izf-1,level)
            wzp = gdzi(izf,level)
            DO iyc=2, nyc-1
               iyf = iyc
               DO ixc=2, nxc-1
                  ixf = 2*(ixc-1)+1
                  wxm = gdxi(ixf-1,level)
                  wxp = gdxi(ixf,level)
                  XC(ixc,iyc,izc) =
     &                 w0* XF(ixf,iyf,izf)
     &                 + w1*(wxm*XF(ixf-1,iyf,izf) 
     &                 +     wxp*XF(ixf+1,iyf,izf)
     &                 +     wzm*XF(ixf,iyf,izf-1) 
     &                 +     wzp*XF(ixf,iyf,izf+1))
     &                 / (wxm + wxp + wzm + wzp)
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 16*(nxc-2)*(nyc-2)*(nzc-2)
      gmflops(isub)=gmflops(isub) + 
     & 16*(nxc-2)*(nyc-2)*(nzc-2)
#endif

*-----------------------------------------------------------------------
*     2D Restrict XC <-- XF along X- and Y- axes
*-----------------------------------------------------------------------

      ELSE IF (gmmlc1 .AND. gmmlc2 .AND. (.NOT.gmmlc3))  THEN

         DO izc=2, nzc-1
            izf = izc
            DO iyc=2, nyc-1
               iyf = 2*(iyc-1)+1
               wym = gdyi(iyf-1,level)
               wyp = gdyi(iyf,level)
               DO ixc=2, nxc-1
                  ixf = 2*(ixc-1)+1
                  wxm = gdxi(ixf-1,level)
                  wxp = gdxi(ixf,level)
                  XC(ixc,iyc,izc) =
     &                 w0* XF(ixf,iyf,izf)
     &                 + w1*(wym*XF(ixf,iyf-1,izf) 
     &                 +     wyp*XF(ixf,iyf+1,izf)
     &                 +     wxm*XF(ixf-1,iyf,izf) 
     &                 +     wxp*XF(ixf+1,iyf,izf))
     &                 / (wym + wyp + wxm + wxp)
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 16*(nxc-2)*(nyc-2)*(nzc-2)
      gmflops(isub)=gmflops(isub) + 
     & 16*(nxc-2)*(nyc-2)*(nzc-2)
#endif

*-----------------------------------------------------------------------
*    1D Restrict XC <-- XF along X-
*-----------------------------------------------------------------------

      ELSE IF (gmmlc1 .AND. (.NOT.gmmlc2) .AND. (.NOT.gmmlc3)) THEN

         DO izc=2, nzc-1
            izf = izc
            DO iyc=2, nyc-1
               iyf = iyc
               DO ixc=2, nxc-1
                  ixf = 2*(ixc-1)+1
                  wxm = gdxi(ixf-1,level)
                  wxp = gdxi(ixf,level)
                  XC(ixc,iyc,izc) =
     &                 w0* XF(ixf,iyf,izf)
     &                 + w1*(wxm*XF(ixf-1,iyf,izf) 
     &                 +     wxp*XF(ixf+1,iyf,izf))
     &                 / (wxm + wxp)
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 10*(nxc-2)*(nyc-2)*(nzc-2)
      gmflops(isub)=gmflops(isub) + 
     & 10*(nxc-2)*(nyc-2)*(nzc-2)
#endif

*-----------------------------------------------------------------------
*    1D Restrict XC <-- XF along X-
*-----------------------------------------------------------------------

      ELSE IF ((.NOT.gmmlc1) .AND. gmmlc2 .AND. (.NOT.gmmlc3)) THEN

         DO izc=2, nzc-1
            izf = izc
            DO iyc=2, nyc-1
               iyf = 2*(iyc-1)+1
               wym = gdyi(iyf-1,level)
               wyp = gdyi(iyf,level)
               DO ixc=2, nxc-1
                  ixf = ixc
                  XC(ixc,iyc,izc) =
     &                 w0* XF(ixf,iyf,izf)
     &                 + w1*(wym*XF(ixf,iyf-1,izf) 
     &                 +     wyp*XF(ixf,iyf+1,izf))
     &                 / (wym + wyp)
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 8*(nxc-2)*(nyc-2)*(nzc-2)
      gmflops(isub)=gmflops(isub) + 
     & 8*(nxc-2)*(nyc-2)*(nzc-2)
#endif

*-----------------------------------------------------------------------
*    1D Restrict XC <-- XF along X-
*-----------------------------------------------------------------------

      ELSE IF ((.NOT.gmmlc1) .AND. (.NOT.gmmlc2) .AND. gmmlc3) THEN

         DO izc=2, nzc-1
            izf = 2*(izc-1)+1
            wzm = gdzi(izf-1,level)
            wzp = gdzi(izf,level)
            DO iyc=2, nyc-1
               iyf = iyc
               DO ixc=2, nxc-1
                  ixf = ixc
                  XC(ixc,iyc,izc) =
     &                 w0* XF(ixf,iyf,izf)
     &                 + w1*(wzm*XF(ixf,iyf,izf-1) 
     &                 +     wzp*XF(ixf,iyf,izf+1))
     &                 / (wzm + wzp)
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 4*(nxc-2)*(nyc-2)*(nzc-2)
      gmflops(isub)=gmflops(isub) + 
     & 4*(nxc-2)*(nyc-2)*(nzc-2)
#endif

      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

 10   CONTINUE

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmRestrictSCGen'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmScale
C     
C     PURPOSE: Scale the matrix, solution, and right-hand side
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmScale (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,scaleA,scaleB,comm)

      IMPLICIT NONE

      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      INTEGER nx,ny,nz,comm
      REAL scaleA,scaleB,scale

#include "mgmpi_internal.h"

      INTEGER ix,iy,iz

      IF (gimatfree.EQ.0) THEN

         DO iz=1,nz-1
            DO iy=1,ny-1
               DO ix=1,nx-1
                  A1(ix,iy,iz) = scaleA*A1(ix,iy,iz)
                  A2(ix,iy,iz) = scaleA*A2(ix,iy,iz)
                  A3(ix,iy,iz) = scaleA*A3(ix,iy,iz)
               END DO
            END DO
         END DO
         IF (gisymmetric.EQ.0) THEN
            DO iz=1,nz-1
               DO iy=1,ny-1
                  DO ix=1,nx-1
                     A4(ix,iy,iz) = scaleA*A4(ix,iy,iz)
                     A5(ix,iy,iz) = scaleA*A5(ix,iy,iz)
                     A6(ix,iy,iz) = scaleA*A6(ix,iy,iz)
                  END DO
               END DO
            END DO
         END IF

      END IF

      scale = scaleB / scaleA

      DO iz=2,nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               B(ix,iy,iz) = scaleB*B(ix,iy,iz)
               X(ix,iy,iz) = scale*X(ix,iy,iz)
            END DO
         END DO
      END DO
      IF (gimatfree.EQ.0) THEN
         DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
                  AD(ix,iy,iz) = scaleA*AD(ix,iy,iz)
               END DO
            END DO
         END DO
      END IF

      RETURN
      END
C=======================================================================
C     
C     Module:  mmScaleRhs
C     
C     Purpose: Scale (iscale=1) or unscale (iscale=-1) the right-hand 
C     Purpose: side for Poisson problem
C
C     Algorithm: 
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmScaleRhs (
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,iScale)

      IMPLICIT NONE

      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      INTEGER nx,ny,nz,iScale

#include "mgmpi_internal.h"

      INTEGER ix,iy,iz
      REAL dxs,dys,dzs

      IF (iScale.EQ.1) THEN
         DO iz=2,nz-1
            dzs = gdz(iz-1,1)+gdz(iz,1)
            DO iy=2,ny-1
               dys = gdy(iy-1,1)+gdy(iy,1)
               DO ix=2,nx-1
                  dxs = gdx(ix-1,1)+gdx(ix,1)
                  B(ix,iy,iz) = o8*dxs*dys*dzs*B(ix,iy,iz)
               END DO
            END DO
         END DO
      ELSE IF (iScale.EQ.-1) THEN
         DO iz=2,nz-1
            dzs = gdz(iz-1,1)+gdz(iz,1)
            DO iy=2,ny-1
               dys = gdy(iy-1,1)+gdy(iy,1)
               DO ix=2,nx-1
                  dxs = gdx(ix-1,1)+gdx(ix,1)
                  B(ix,iy,iz) = 8.0D0*B(ix,iy,iz) / (dxs*dys*dzs)
               END DO
            END DO
         END DO
      END IF

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmSendX1Ghost
C     
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from Y(1,   *,*) on (ipx+1,ipy,ipz)
C     PURPOSE: to   X(nx-1,*,*) on (ipx,  ipy,ipz)
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmSendX1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      
      INTEGER comm,nbr(6),bnd(3)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

#ifdef MPI_USED
      INTEGER status(MPI_STATUS_SIZE)
      INTEGER ierr
#endif

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_GHOST
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef MPI_USED
      IF (ipx.GT.0 .AND. ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmSendX1Ghost: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &        X(   1,1,1),1,bndx(XD),nbr(XMD),70+XMD,
     &        X(nx-1,1,1),1,bndx(XD),nbr(XPD),70+XMD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in send_x1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & 2*nydx*nzdx
#endif
      ELSE IF (ipx.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmSendX1Ghost: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &        X(   1,1,1),1,bndx(XD),nbr(XMD),70+XMD,
     &        comm,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in send_x1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nzdx
#endif
      ELSE IF (ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmSendX1Ghost: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &        X(nx-1,1,1),1,bndx(XD),nbr(XPD),70+XMD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in send_x1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nzdx
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmSendY1Ghost
C     
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from X(*,1,   *) on (ipx,ipy+1,ipz)
C     PURPOSE: to   X(*,ny-1,*) on (ipx,ipy,ipz)
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmSendY1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      
      INTEGER comm,nbr(6),bnd(3)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

#ifdef MPI_USED
      INTEGER status(MPI_STATUS_SIZE)
      INTEGER ierr
#endif

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_GHOST
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef MPI_USED
      IF (ipy.GT.0 .AND. ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmSendY1Ghost: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &        X(1,   1,1),1,bndx(YD),nbr(YMD),70+YMD,
     &        X(1,ny-1,1),1,bndx(YD),nbr(YPD),70+YMD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in send_y1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nxdx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & 2*nxdx*nzdx
#endif
      ELSE IF (ipy.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmSendY1Ghost: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &        X(1,   1,1),1,bndx(YD),nbr(YMD),70+YMD,
     &        comm,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in send_y1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nzdx
#endif
      ELSE IF (ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmSendY1Ghost: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &        X(1,ny-1,1),1,bndx(YD),nbr(YPD),70+YMD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in send_y1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nzdx
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmSendZ1Ghost
C     
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from X(*,*,1,  ) on (ipx,ipy,ipz+1)
C     PURPOSE: to   X(*,*,nz-1) on (ipx,ipy,ipz)
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmSendZ1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      
      INTEGER comm,nbr(6),bnd(3)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

#ifdef MPI_USED
      INTEGER status(MPI_STATUS_SIZE)
      INTEGER ierr
#endif

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_GHOST
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef MPI_USED
      IF (ipz.GT.0 .AND. ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmSendZ1Ghost: MPI_Sendrecv')") ip
#endif
         CALL MPI_Sendrecv (
     &        X(1,1,   1),1,bndx(ZD),nbr(ZMD),70+ZMD,
     &        X(1,1,nz-1),1,bndx(ZD),nbr(ZPD),70+ZMD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in send_z1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & 2*nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & 2*nxdx*nydx
#endif
      ELSE IF (ipz.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmSendZ1Ghost: MPI_Send')") ip
#endif
         CALL MPI_Send (
     &        X(1,1,   1),1,bndx(ZD),nbr(ZMD),70+ZMD,
     &        comm,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in send_z1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nydx
#endif
      ELSE IF (ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmSendZ1Ghost: MPI_Recv')") ip
#endif
         CALL MPI_Recv (
     &        X(1,1,nz-1),1,bndx(ZD),nbr(ZPD),70+ZMD,
     &        comm,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in send_z1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nydx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nydx
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmIGetX1Ghost 
C     
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from Y(nx-1,*,*) on (ipx,  ipy,ipz)
C     PURPOSE: to   X(1,   *,*) on (ipx+1,ipy,ipz) using nonblocking
C     PURPOSE: communication
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmIGetX1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,sreq,rreq,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER comm,nbr(6),bnd(3)
      INTEGER nx,ny,nz
      INTEGER sreq,rreq

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"

      INTEGER ierr
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_IGHOST
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef MPI_USED
      IF (ipx.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmIGetX1Ghost: MPI_IRecv')") ip
#endif
         CALL MPI_IRecv (
     &        X(   1,1,1),1,bndx(XD),nbr(XMD),icommtag+XPD,
     &        comm,rreq,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_x1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nzdx
#endif
      END IF
      IF (ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmIGetX1Ghost: MPI_ISend')") ip
#endif
         CALL MPI_ISend (
     &        X(nx-1,1,1),1,bndx(XD),nbr(XPD),icommtag+XPD,
     &        comm,sreq,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_x1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nzdx
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmIGetXnGhost 
C     
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from Y(2, *,*) on (ipx,  ipy,ipz)
C     PURPOSE: to   X(nx,*,*) on (ipx-1,ipy,ipz) using nonblocking
C     PURPOSE: communication
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmIGetXnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,sreq,rreq,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER comm,nbr(6),bnd(3)
      INTEGER nx,ny,nz
      INTEGER sreq,rreq

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"

      INTEGER ierr
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_IGHOST
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef MPI_USED
      IF (ipx.LT.npx-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmIGetXnGhost: MPI_IRecv')") ip
#endif
         CALL MPI_IRecv (
     &        X(nx,1,1),1,bnd(XD),nbr(XPD),icommtag+XMD,
     &        comm,rreq,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_xn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nzdx
#endif
      END IF
      IF (ipx.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmIGetXnGhost: MPI_ISend')") ip
#endif
         CALL MPI_ISend (
     &        X( 2,1,1),1,bnd(XD),nbr(XMD),icommtag+XMD,
     &        comm,sreq,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_xn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nzdx
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmIGetY1Ghost 
C     
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from Y(*,ny-1,*) on (ipx,ipy,  ipz)
C     PURPOSE: to   X(*,1,   *) on (ipx,ipy+1,ipz) using nonblocking
C     PURPOSE: communication
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmIGetY1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,sreq,rreq,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER comm,nbr(6),bnd(3)
      INTEGER nx,ny,nz
      INTEGER sreq,rreq

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"

      INTEGER ierr
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_IGHOST
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef MPI_USED
      IF (ipy.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmIGetY1Ghost: MPI_IRecv')") ip
#endif
         CALL MPI_IRecv (
     &        X(1,   1,1),1,bndx(YD),nbr(YMD),icommtag+YPD,
     &        comm,rreq,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_y1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nzdx
#endif
      END IF
      IF (ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmIGetY1Ghost: MPI_ISend')") ip
#endif
         CALL MPI_ISend (
     &        X(1,ny-1,1),1,bndx(YD),nbr(YPD),icommtag+YPD,
     &        comm,sreq,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_y1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nzdx
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmIGetYnGhost 
C     
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from Y(*,2, *) on (ipx,ipy,  ipz)
C     PURPOSE: to   X(*,ny,*) on (ipx,ipy-1,ipz) using nonblocking
C     PURPOSE: communication
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmIGetYnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,sreq,rreq,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER comm,nbr(6),bnd(3)
      INTEGER nx,ny,nz
      INTEGER sreq,rreq

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"

      INTEGER ierr
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_IGHOST
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef MPI_USED
      IF (ipy.LT.npy-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmIGetYnGhost: MPI_IRecv')") ip
#endif
         CALL MPI_IRecv (
     &        X(1,ny,1),1,bnd(YD),nbr(YPD),icommtag+YMD,
     &        comm,rreq,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_yn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nzdx
#endif
      END IF
      IF (ipy.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmIGetYnGhost: MPI_ISend')") ip
#endif
         CALL MPI_ISend (
     &        X(1, 2,1),1,bnd(YD),nbr(YMD),icommtag+YMD,
     &        comm,sreq,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_yn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nxdx*nzdx
	gcomm(isub) = gcomm(isub) + 
     & nxdx*nzdx
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmIGetZ1Ghost 
C     
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from Y(*,*,nz-1) on (ipx,ipy,ipz)
C     PURPOSE: to   X(*,*,1)    on (ipx,ipy,ipz+1) using nonblocking
C     PURPOSE: communication
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmIGetZ1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,sreq,rreq,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER comm,nbr(6),bnd(3)
      INTEGER nx,ny,nz
      INTEGER sreq,rreq

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"

      INTEGER ierr
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_IGHOST
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef MPI_USED
      IF (ipz.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmIGetZ1Ghost: MPI_IRecv')") ip
#endif
         CALL MPI_IRecv (
     &        X(1,1,   1),1,bndx(ZD),nbr(ZMD),icommtag+ZPD,
     &        comm,rreq,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_z1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nxdx
#endif
      END IF
      IF (ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmIGetZ1Ghost: MPI_ISend')") ip
#endif
         CALL MPI_ISend (
     &        X(1,1,nz-1),1,bndx(ZD),nbr(ZPD),icommtag+ZPD,
     &        comm,sreq,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_z1_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nxdx
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmIGetZnGhost 
C     
C     PURPOSE: Transfer ghost values 
C     PURPOSE: from X(*,*,2)  on (ipx,ipy,ipz)
C     PURPOSE: to   X(*,*,nz) on (ipx,ipy,ipz-1) using nonblocking
C     PURPOSE: communication
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmIGetZnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,sreq,rreq,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER comm,nbr(6),bnd(3)
      INTEGER nx,ny,nz
      INTEGER sreq,rreq

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"

      INTEGER ierr
#else
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_IGHOST
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef MPI_USED
      IF (ipz.LT.npz-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmIGetZnGhost: MPI_IRecv')") ip
#endif
         CALL MPI_IRecv (
     &        X(1,1,nz),1,bnd(ZD),nbr(ZPD),icommtag+ZMD,
     &        comm,rreq,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_zn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nxdx
#endif
      END IF
      IF (ipz.GT.0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmIGetZnGhost: MPI_ISend')") ip
#endif
         CALL MPI_ISend (
     &        X(1,1, 2),1,bnd(ZD),nbr(ZMD),icommtag+ZMD,
     &        comm,sreq,ierr)
         IF (ierr.NE.0) print*, 'MPI Error in get_zn_ghost: ',ierr
         
#ifdef PERF_EVAL
	gcomm(1) = gcomm(1) + 
     & nydx*nxdx
	gcomm(isub) = gcomm(isub) + 
     & nydx*nxdx
#endif
      END IF
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmVectorShift
C     
C     Purpose: Shift B <-- B - t so that sum(B)=0
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmShiftRhs (
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,nxg,nyg,nzg,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,comm,icall)

      IMPLICIT NONE

      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      INTEGER nx,ny,nz,nxg,nyg,nzg
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      INTEGER comm

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      REAL mmVectorSum, t
      INTEGER ix,iy,iz

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif
      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_SHIFT
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      IF (ABS(ixmb).NE.MM_DIRICHLET .AND.
     &     ABS(iymb).NE.MM_DIRICHLET .AND. 
     &     ABS(izmb).NE.MM_DIRICHLET .AND.
     &     ABS(ixpb).NE.MM_DIRICHLET .AND.
     &     ABS(iypb).NE.MM_DIRICHLET .AND.
     &     ABS(izpb).NE.MM_DIRICHLET) THEN

         t = mmVectorSum (
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,comm,isub) 
     &        / ((nxg-2)*(nyg-2)*(nzg-2))

         DO iz=1,nz
            DO iy=1,ny
               DO ix=1,nx
                  B(ix,iy,iz) = B(ix,iy,iz) - t
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & nx*ny*nz
      gmflops(isub)=gmflops(isub) + 
     & nx*ny*nz
#endif
         
      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmSmooth
C     
C     PURPOSE: Call an error-smoothing procedure (e.g. Jacobi)
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmSmooth (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &     nx,ny,nz,nxg,nyg,nzg,wgt,nu,iSmoothIt,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,level,icall)

      IMPLICIT NONE

      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxdrsm,nydrsm,nzdrsm
      INTEGER bndrsm(3)
      REAL RSM(nxdrsm,nydrsm,nzdrsm)
      
      INTEGER nxdpsm,nydpsm,nzdpsm
      INTEGER bndpsm(3)
      REAL PSM(nxdpsm,nydpsm,nzdpsm)
      
      INTEGER nxdwsm,nydwsm,nzdwsm
      INTEGER bndzsm(3)
      REAL WSM(nxdwsm,nydwsm,nzdwsm)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER comm,nbr(6),bnd(3)
      
      INTEGER nx,ny,nz
      INTEGER nxg,nyg,nzg
      INTEGER nu,iSmoothIt
      INTEGER level
      REAL wgt
      REAL scaleA,scaleB

      LOGICAL isFirstCall
      DATA isFirstCall/.TRUE./

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_SMOOTH+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmSmooth'
#endif

 1000 CONTINUE
      
      IF (gismoother .EQ. MM_POINT_JACOBI) THEN
#ifdef MPI_BLOCKING
         CALL mmBJacobi (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,wgt,nu,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,level,isub)
#else 
         IF (gisymmetric.EQ.1) THEN
            CALL mmIJacobi (
     &     AD,A1,A2,A3,A1,A2,A3,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,wgt,nu,
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &           scaleA,scaleB,level,isub)
         ELSE
            CALL mmIJacobi (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,wgt,nu,
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &           scaleA,scaleB,level,isub)
         END IF
#endif
      ELSE IF (gismoother.EQ.MM_LOCAL_POINT_RBGS) THEN
         IF (gimatfree.EQ.1) THEN
            CALL mmBMFRBGaussSeidel (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,wgt,nu,
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &           scaleA,scaleB,level,isub)
         ELSE
            CALL mmBRBGaussSeidel (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,wgt,nu,
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &           scaleA,scaleB,level,isub)
         END IF
      ELSE IF (gismoother.EQ.MM_LOCAL_LINE_JACOBI) THEN

         IF (gimatfree.EQ.1) THEN
            IF (ip.EQ.0.AND.isFirstCall) THEN 
               PRINT*, '-- mgmpi warning -- Matrix-free local line'
               PRINT*, '                    smoothing not implemented;'
               PRINT*, '                    dropping to point smoother.'
            END IF
            gismoother=MM_POINT_JACOBI
            isFirstCall = .FALSE.
            GOTO 1000
         END IF

         IF (gisymmetric.EQ.1) THEN
            CALL mmLocalLineJacobi (
     &           
     &     AD,A1,A2,A3,A1,A2,A3,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,wgt,nu,
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &           scaleA,scaleB,level,isub)
         ELSE
            CALL mmLocalLineJacobi (
     &           
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,wgt,nu,
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &           scaleA,scaleB,level,isub)

         END IF

      ELSE IF (gismoother.EQ.MM_PLANE_JACOBI) THEN

         IF (gimatfree.EQ.1) THEN
            IF (ip.EQ.0.AND.isFirstCall) THEN 
               PRINT*, '-- mgmpi warning -- Matrix-free plane smoothing'
               PRINT*, '                    not implemented; dropping'
               PRINT*, '                    to local line smoother.'
            END IF
            gismoother=MM_LOCAL_LINE_JACOBI
            isFirstCall = .FALSE.
            GOTO 1000
         END IF

         IF (gisymmetric.EQ.1) THEN
            CALL mmPlaneJacobi (
     &           
     &     AD,A1,A2,A3,A1,A2,A3,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &           nx,ny,nz,nxg,nyg,nzg,wgt,nu,iSmoothIt,
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &           scaleA,scaleB,level,isub)
         ELSE
            CALL mmPlaneJacobi (
     &           
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     RSM,nxdrsm,nydrsm,nzdrsm,bndrsm
     &     ,
     &     PSM,nxdpsm,nydpsm,nzdpsm,bndpsm
     &     ,
     &     WSM,nxdwsm,nydwsm,nzdwsm,bndzsm
     &     ,
     &           nx,ny,nz,nxg,nyg,nzg,wgt,nu,iSmoothIt,
     &           
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &           scaleA,scaleB,level,isub)
         END IF
      END IF

      isFirstCall = .FALSE.

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      END

C=======================================================================
C     
C     Module:  mmBJacobi
C     
C     Purpose: Point weighted Jacobi smoother
C
C     Inputs:  A,X,B: linear system
C     Inputs:  R: residual storage
C
C     Outputs: X: smoothed solution
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================


      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBJacobi (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,wgt,nu,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,level,icall)

      IMPLICIT NONE

      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER comm,nbr(6),bnd(3)
      
      INTEGER nx,ny,nz
      INTEGER nu
      INTEGER level
      REAL wgt
      REAL scaleA,scaleB,s,mmVectorSum

#include "mgmpi_internal.h"

#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz,j

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_SMOOTH+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmBJacobi'
#endif
      
      DO j=1,nu

#ifdef DEBUG
         IF (ip.EQ.0) PRINT*, 'mmBJacobi iteration ',j
#endif

*-----------------------------------------------------------------------
*     Compute local X = X + w*(B-AX) ./ diag(A)
*-----------------------------------------------------------------------

#ifdef DEBUG
         s = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
         if (ip.EQ.0) PRINT*, level,'mmBJacobi: sum(X) = ',s
         s = mmVectorSum (
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,comm,isub)
         if (ip.EQ.0) PRINT*, level,'mmBJacobi: sum(B) = ',s
#endif
         CALL mmResidual (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,1,level,isub)
#ifdef DEBUG
         CALL mmVectorInfo ('R   ',
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,comm)
#endif

#ifdef DEBUG
         s = mmVectorSum (R,nxdr,nydr,nzdr,bndr,nx,ny,nz,comm,isub)
         if (ip.EQ.0) PRINT*, level,'mmBJacobi: sum(R) = ',s
#endif
         DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
                  X(ix,iy,iz) = X(ix,iy,iz) + wgt*R(ix,iy,iz)
               END DO
            END DO
         END DO
#ifdef DEBUG
         s = mmVectorSum (X,nxdx,nydx,nzdx,bndx,nx,ny,nz,comm,isub)
         if (ip.EQ.0) PRINT*, level,'mmBJacobi: sum(X) = ',s
#endif

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif

      END DO

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmBJacobi'
#endif

      RETURN
      END

C=======================================================================
C     
C     Module:  mmBRBGaussSeidel
C     
C     Purpose: Point weighted red-black Gauss Seidel
C
C     Algorithm: Processor-local RB GS, so convergence may change with
C     Algorithm: processor distributions (i.e. local X(2,2,2) always red)
C
C     Inputs:  A,X,B: linear system
C
C     Outputs: X: smoothed solution
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================


      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBRBGaussSeidel (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,wgt,nu,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,level,icall)

      IMPLICIT NONE

      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER comm,nbr(6),bnd(3)
      
      INTEGER nx,ny,nz
      INTEGER nu
      INTEGER level
      REAL wgt
      REAL scaleA,scaleB,s,mmVectorSum

#include "mgmpi_internal.h"

#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz,j,ix0

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_SMOOTH+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmBRBGaussSeidel'
#endif
      
      DO j=1,nu

#ifdef DEBUG
         IF (ip.EQ.0) PRINT*, 'mmBRBGaussSeidel iteration ',j
#endif

         CALL mmRefreshBC (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,level,isub)

*-----------------------------------------------------------------------
*     Compute red points
*-----------------------------------------------------------------------

#ifdef DEBUG
         s = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
         if (ip.EQ.0) PRINT*, level,'mmBRBGaussSeidel: sum(X) = ',s
         s = mmVectorSum (
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,comm,isub)
         if (ip.EQ.0) PRINT*, level,'mmBRBGaussSeidel: sum(B) = ',s
#endif

         DO iz=2,nz-1
            DO iy=2,ny-1
               IF ((iy*iz/2)*2.EQ.iy*iz) ix0=2
               IF ((iy*iz/2)*2.NE.iy*iz) ix0=3
               DO ix=ix0,nx-1,2
                  X(ix,iy,iz) = X(ix,iy,iz) + wgt*((B(ix,iy,iz)
     &                 - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &                 - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &                 - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &                 - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &                 - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &                 - A3(ix,iy,iz)  *X(ix,iy,iz+1)) / AD(ix,iy,iz)
     &                 - X(ix,iy,iz))
               END DO
            END DO
         END DO

#ifdef DEBUG
         s = mmVectorSum (X,nxdx,nydx,nzdx,bndx,nx,ny,nz,comm,isub)
         if (ip.EQ.0) PRINT*, level,'mmBRBGaussSeidel: sum(X) = ',s
#endif

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif

*-----------------------------------------------------------------------
*     Compute black points
*-----------------------------------------------------------------------

         DO iz=2,nz-1
            DO iy=2,ny-1
               IF ((iy*iz/2)*2.EQ.iy*iz) ix0=3
               IF ((iy*iz/2)*2.NE.iy*iz) ix0=2
               DO ix=ix0,nx-1,2
                  X(ix,iy,iz) = X(ix,iy,iz) + wgt*((B(ix,iy,iz)
     &                 - A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &                 - A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &                 - A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &                 - A1(ix,iy,iz)  *X(ix+1,iy,iz)
     &                 - A2(ix,iy,iz)  *X(ix,iy+1,iz)
     &                 - A3(ix,iy,iz)  *X(ix,iy,iz+1)) / AD(ix,iy,iz)
     &                 - X(ix,iy,iz))
               END DO
            END DO
         END DO

#ifdef DEBUG
         s = mmVectorSum (X,nxdx,nydx,nzdx,bndx,nx,ny,nz,comm,isub)
         if (ip.EQ.0) PRINT*, level,'mmBRBGaussSeidel: sum(X) = ',s
#endif

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif

      END DO


      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmBRBGaussSeidel'
#endif

      RETURN
      END

C=======================================================================
C     
C     Module:  mmBRBGaussSeidel
C     
C     Purpose: Point weighted red-black Gauss Seidel
C
C     Algorithm: Processor-local RB GS, so convergence may change with
C     Algorithm: processor distributions (i.e. local X(2,2,2) always red)
C
C     Inputs:  A,X,B: linear system
C
C     Outputs: X: smoothed solution
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================


      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmBMFRBGaussSeidel (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,wgt,nu,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,level,icall)

      IMPLICIT NONE

      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER comm,nbr(6),bnd(3)
      
      INTEGER nx,ny,nz
      INTEGER nu
      INTEGER level
      REAL wgt
      REAL scaleA,scaleB,s,mmVectorSum

#include "mgmpi_internal.h"

#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz,j,ix0
      REAL hxm,hym,hzm,hxp,hyp,hzp
      REAL dxs,dys,dzs,dxp,dyp,dzp,dxm,dym,dzm,dsum,dsumi

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_SMOOTH+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      IF (level.GT.1) THEN

         CALL mmBRBGaussSeidel (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,wgt,nu,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,level,icall)
      END IF

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmBRBGaussSeidel'
#endif
      
      DO j=1,nu

#ifdef DEBUG
         IF (ip.EQ.0) PRINT*, 'mmBRBGaussSeidel iteration ',j
#endif

         CALL mmGetGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)

         dzs = zp(3)-zp(1)
         hzm = zp(2)-zp(1)
         hzp = zp(3)-zp(2)
         dys = yp(3)-yp(1)
         hym = yp(2)-yp(1)
         hyp = yp(3)-yp(2)
         dxs = xp(3)-xp(1)
         hxm = xp(2)-xp(1)
         hxp = xp(3)-xp(2)
         dzp=dxs*dys/hzp
         dxp=dys*dzs/hxp
         dyp=dzs*dxs/hyp
         dzm=dxs*dys/hzm
         dxm=dys*dzs/hxm
         dym=dzs*dxs/hym
         dsum = dxp + dyp + dzp + dxm + dym + dzm
         dzp=dzp/dsum
         dxp=dxp/dsum
         dyp=dyp/dsum
         dzm=dzm/dsum
         dxm=dxm/dsum
         dym=dym/dsum
         dsum = o4*dsum*gScaleA
         dsumi = -1.0D0 / dsum

*-----------------------------------------------------------------------
*     Compute red points
*-----------------------------------------------------------------------

#ifdef DEBUG
         s = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
         if (ip.EQ.0) PRINT*, level,'mmBRBGaussSeidel: sum(X) = ',s
         s = mmVectorSum (
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,comm,isub)
         if (ip.EQ.0) PRINT*, level,'mmBRBGaussSeidel: sum(B) = ',s
#endif

         DO iz=2,nz-1
            DO iy=2,ny-1
               IF ((iy*iz/2)*2.EQ.iy*iz) ix0=2
               IF ((iy*iz/2)*2.NE.iy*iz) ix0=3
               DO ix=ix0,nx-1,2
                  X(ix,iy,iz) = X(ix,iy,iz) -(B(ix,iy,iz) / dsum
     &                 - dxm*X(ix-1,iy,iz)
     &                 - dym*X(ix,iy-1,iz)
     &                 - dzm*X(ix,iy,iz-1)
     &                 - dxp  *X(ix+1,iy,iz)
     &                 - dyp  *X(ix,iy+1,iz)
     &                 - dzp  *X(ix,iy,iz+1)
     &                 +       X(ix,iy,iz))
               END DO
            END DO
         END DO

#ifdef DEBUG
         s = mmVectorSum (X,nxdx,nydx,nzdx,bndx,nx,ny,nz,comm,isub)
         if (ip.EQ.0) PRINT*, level,'mmBRBGaussSeidel: sum(X) = ',s
#endif

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif

*-----------------------------------------------------------------------
*     Compute black points
*-----------------------------------------------------------------------

         DO iz=2,nz-1
            DO iy=2,ny-1
               IF ((iy*iz/2)*2.EQ.iy*iz) ix0=3
               IF ((iy*iz/2)*2.NE.iy*iz) ix0=2
               DO ix=ix0,nx-1,2
                  X(ix,iy,iz) = X(ix,iy,iz) -(B(ix,iy,iz) / dsum
     &                 - dxm*X(ix-1,iy,iz)
     &                 - dym*X(ix,iy-1,iz)
     &                 - dzm*X(ix,iy,iz-1)
     &                 - dxp  *X(ix+1,iy,iz)
     &                 - dyp  *X(ix,iy+1,iz)
     &                 - dzp  *X(ix,iy,iz+1)
     &                 +       X(ix,iy,iz))
               END DO
            END DO
         END DO

#ifdef DEBUG
         s = mmVectorSum (X,nxdx,nydx,nzdx,bndx,nx,ny,nz,comm,isub)
         if (ip.EQ.0) PRINT*, level,'mmBRBGaussSeidel: sum(X) = ',s
#endif

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(ny-2)*(nz-2)
#endif

      END DO


      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmBRBGaussSeidel'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C=======================================================================
C     
C     MODULE:  SUBROUTINE mmIJacobi 
C     
C     PURPOSE: Point Jacobi with nonblocking communication
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmIJacobi (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,
     &     wgt,nu,
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &     scaleA,scaleB,level,icall)

      IMPLICIT NONE

      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      
      REAL xp(*),yp(*),zp(*)
      REAL hx(*),hy(*),hz(*)
      
      INTEGER comm,nbr(6),bnd(3)
      
      INTEGER nx,ny,nz
      INTEGER nu
      INTEGER level
      REAL wgt
      REAL scaleA,scaleB

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz,i,j

#ifdef MPI_USED
      INTEGER status(MPI_STATUS_SIZE,12),ierr
      INTEGER req(12)
#else 
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_SMOOTH
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '>>> mmIJacobi'
#endif
      DO j=1,nu

         CALL mmRefreshBC (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &        
     &     xp,yp,zp,hx,hy,hz
     &     ,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,
     &     comm,nbr,bnd
     &     ,
     &        scaleA,scaleB,level,isub)

*-----------------------------------------------------------------------
*     Refresh the ghost points of X
*-----------------------------------------------------------------------

#ifdef MPI_USED
         icommtag=icommtag+6
         IF (icommtag.GT.32757) icommtag=ICOMMSTART

         DO i=1,12
            req(i)=MPI_REQUEST_NULL
         END DO

         CALL mmIGetX1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(1),req(2),isub)
         CALL mmIGetY1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(3),req(4),isub)
         CALL mmIGetZ1Ghost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(5),req(6),isub)
         CALL mmIGetXnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(7),req(8),isub)
         CALL mmIGetYnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(9),req(10),isub)
         CALL mmIGetZnGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,req(11),req(12),isub)
#endif

*-----------------------------------------------------------------------
*     Compute local interior of R = A*X
*-----------------------------------------------------------------------

         DO iz=3,nz-2
            DO iy=3,ny-2
               DO ix=3,nx-2
                  R(ix,iy,iz) = A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &                 + A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &                 + A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &                 + AD(ix,iy,iz)  *X(ix,iy,iz)
               END DO
               DO ix=3,nx-2
                  R(ix,iy,iz) = R(ix,iy,iz)
     &                 + A1(ix,iy,iz)*X(ix+1,iy,iz)
     &                 + A2(ix,iy,iz)*X(ix,iy+1,iz)
     &                 + A3(ix,iy,iz)*X(ix,iy,iz+1)
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-4)*(ny-4)*(nz-4)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-4)*(ny-4)*(nz-4)
#endif

*-----------------------------------------------------------------------
*     wait for boundaries
*-----------------------------------------------------------------------

         
#ifdef PERF_EVAL
      gtimestart(SB_SMOOTH_WAIT) = MPI_WTIME()
#endif

#ifdef MPI_USED
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmIJacobi: MPI_WAITALL')") ip
#endif
         CALL MPI_WAITALL (12,req,status,ierr)
         IF (ierr.NE.0) print*, 'MPI Error n mmIMatvec wait: ',ierr
#endif

         
#ifdef PERF_EVAL
      gtime(SB_SMOOTH_WAIT) = gtime(SB_SMOOTH_WAIT) + MPI_WTIME() - gtimestart(SB_SMOOTH_WAIT)
      gtimecount(SB_SMOOTH_WAIT) = gtimecount(SB_SMOOTH_WAIT) + 1
#endif

*-----------------------------------------------------------------------
*     X-facets
*-----------------------------------------------------------------------

         ix=2
         DO iz=2,nz-1
            DO iy=2,ny-1
               R(ix,iy,iz) = A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &              +        A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &              +        A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &              +        AD(ix,iy,iz)  *X(ix,iy,iz)
     &              +        A1(ix,iy,iz)*X(ix+1,iy,iz)
     &              +        A2(ix,iy,iz)*X(ix,iy+1,iz)
     &              +        A3(ix,iy,iz)*X(ix,iy,iz+1)
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(ny-2)*(nz-2)
#endif
         ix=nx-1
         DO iz=2,nz-1
            DO iy=2,ny-1
               r(ix,iy,iz) = a4(ix-1,iy,iz)*x(ix-1,iy,iz)
     &              +        a5(ix,iy-1,iz)*x(ix,iy-1,iz)
     &              +        a6(ix,iy,iz-1)*x(ix,iy,iz-1)
     &              +        ad(ix,iy,iz)  *x(ix,iy,iz)
     &              +        a1(ix,iy,iz)*x(ix+1,iy,iz)
     &              +        a2(ix,iy,iz)*x(ix,iy+1,iz)
     &              +        a3(ix,iy,iz)*x(ix,iy,iz+1)
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(ny-2)*(nz-2)
#endif

*-----------------------------------------------------------------------
*     y-facets
*-----------------------------------------------------------------------

         iy=2
         DO iz=2,nz-1
            DO ix=2,nx-1
               r(ix,iy,iz) = a4(ix-1,iy,iz)*x(ix-1,iy,iz)
     &              +        a5(ix,iy-1,iz)*x(ix,iy-1,iz)
     &              +        a6(ix,iy,iz-1)*x(ix,iy,iz-1)
     &              +        ad(ix,iy,iz)  *x(ix,iy,iz)
     &              +        a1(ix,iy,iz)*x(ix+1,iy,iz)
     &              +        a2(ix,iy,iz)*x(ix,iy+1,iz)
     &              +        a3(ix,iy,iz)*x(ix,iy,iz+1)
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-2)*(nz-2)
#endif
         iy=ny-1
         DO iz=2,nz-1
            DO ix=2,nx-1
               R(ix,iy,iz) = A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &              +        A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &              +        A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &              +        AD(ix,iy,iz)  *X(ix,iy,iz)
     &              +        A1(ix,iy,iz)*X(ix+1,iy,iz)
     &              +        A2(ix,iy,iz)*X(ix,iy+1,iz)
     &              +        A3(ix,iy,iz)*X(ix,iy,iz+1)
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(nx-2)*(nz-2)
#endif

*-----------------------------------------------------------------------
*     Z-facets
*-----------------------------------------------------------------------

         iz=2
         DO iy=2,ny-1
            DO ix=2,nx-1
               R(ix,iy,iz) = A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &              +        A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &              +        A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &              +        AD(ix,iy,iz)  *X(ix,iy,iz)
     &              +        A1(ix,iy,iz)*X(ix+1,iy,iz)
     &              +        A2(ix,iy,iz)*X(ix,iy+1,iz)
     &              +        A3(ix,iy,iz)*X(ix,iy,iz+1)
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(ny-2)*(nx-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(ny-2)*(nx-2)
#endif
         iz=nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               R(ix,iy,iz) = A4(ix-1,iy,iz)*X(ix-1,iy,iz)
     &              +        A5(ix,iy-1,iz)*X(ix,iy-1,iz)
     &              +        A6(ix,iy,iz-1)*X(ix,iy,iz-1)
     &              +        AD(ix,iy,iz)  *X(ix,iy,iz)
     &              +        A1(ix,iy,iz)*X(ix+1,iy,iz)
     &              +        A2(ix,iy,iz)*X(ix,iy+1,iz)
     &              +        A3(ix,iy,iz)*X(ix,iy,iz+1)
            END DO
         END DO
         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 13*(ny-2)*(nx-2)
      gmflops(isub)=gmflops(isub) + 
     & 13*(ny-2)*(nx-2)
#endif


*-----------------------------------------------------------------------
*     Compute X = X + w*(B-R) ./ diag(A)
*-----------------------------------------------------------------------

         DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
                  R(ix,iy,iz) = B(ix,iy,iz) - R(ix,iy,iz)
                  X(ix,iy,iz) = X(ix,iy,iz) + wgt*R(ix,iy,iz)
     &                 / AD(ix,iy,iz)
               END DO
            END DO
         END DO

         
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 4*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 4*(nx-2)*(ny-2)*(nz-2)
#endif

      END DO

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

#ifdef PROC_TRACE
      IF (ip.EQ.0) PRINT*, '<<< mmIJacobi'
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmTerminate
C     
C     PURPOSE: Halt execution
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmTerminate(comm)
      IMPLICIT NONE
      INTEGER comm
#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"
      INTEGER ierr
#ifdef MPI_USED
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmTerminate: MPI_BARRIER')") -1
#endif
      CALL MPI_BARRIER (comm,ierr)
      if (ierr.NE.0) print*, 'MPI Error in terminate: ',ierr
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmTerminate: MPI_COMM_FREE')") -1
#endif
      CALL MPI_COMM_FREE (comm,ierr)
      if (ierr.NE.0) print*, 'MPI Error in terminate: ',ierr
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmTerminate: MPI_FINALIZE')") -1
#endif
      CALL MPI_FINALIZE (ierr)
      if (ierr.NE.0) print*, 'MPI Error in terminate: ',ierr
#endif
      STOP
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmUnScale
C     
C     PURPOSE: Unscale the matrix, solution, and right-hand side
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmUnScale (
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,scaleA,scaleB,comm)

      IMPLICIT NONE

      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      INTEGER nx,ny,nz,comm
      REAL scaleA,scaleB,scale

#include "mgmpi_internal.h"

      INTEGER ix,iy,iz

      IF (gimatfree.EQ.0) THEN

         DO iz=1,nz-1
            DO iy=1,ny-1
               DO ix=1,nx-1
                  A1(ix,iy,iz) = A1(ix,iy,iz)/scaleA
                  A2(ix,iy,iz) = A2(ix,iy,iz)/scaleA
                  A3(ix,iy,iz) = A3(ix,iy,iz)/scaleA
                  A4(ix,iy,iz) = A4(ix,iy,iz)/scaleA
                  A5(ix,iy,iz) = A5(ix,iy,iz)/scaleA
                  A6(ix,iy,iz) = A6(ix,iy,iz)/scaleA
               END DO
            END DO
         END DO
         IF (gisymmetric.EQ.0) THEN
            DO iz=1,nz-1
               DO iy=1,ny-1
                  DO ix=1,nx-1
                     A4(ix,iy,iz) = A4(ix,iy,iz)/scaleA
                     A5(ix,iy,iz) = A5(ix,iy,iz)/scaleA
                     A6(ix,iy,iz) = A6(ix,iy,iz)/scaleA
                  END DO
               END DO
            END DO
         END IF
      END IF

      scale = scaleA / scaleB

      DO iz=2,nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               B(ix,iy,iz) = B(ix,iy,iz)/scaleB
               X(ix,iy,iz) = scale*X(ix,iy,iz)
            END DO
         END DO
      END DO

      IF (gimatfree.EQ.0) THEN
         DO iz=2,nz-1
            DO iy=2,ny-1
               DO ix=2,nx-1
                  AD(ix,iy,iz) = AD(ix,iy,iz)/scaleA
               END DO
            END DO
         END DO
      END IF

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmVectorClear
C     
C     PURPOSE: Set a vector to 0
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmVectorClear (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz

#include "mgmpi_internal.h"

      INTEGER ix,iy,iz

      DO iz=1,nz
         DO iy=1,ny
            DO ix=1,nx
               X(ix,iy,iz)=(0.0D0)
            END DO
         END DO
      END DO

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmVectorCollapse
C     
C     Purpose: Copy vector distributed among processors to processor 0
C
C     Algorithm: 3-phase : z-axis, y-axis, then x-axis collapse
C
C     Inputs:  X distributed among all processors
C
C     Outputs: Y on processor 0
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmVectorCollapse (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,level,
     &     comm,icall)

      IMPLICIT NONE
      
      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz,level
      INTEGER comm

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      INTEGER ix,iy,iz,ierr,j
      INTEGER status(MPI_STATUS_SIZE)

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_SUM
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      IF (ip.EQ.0) THEN
         DO j=0,ip-1
            CALL MPI_Recv (Y,1,gmmcgrid(level,j),j,j,comm,status,ierr)
         END DO
      ELSE
         CALL MPI_Send (X,1,gmmgrid(level),0,ip,comm,ierr)
      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmVectorExpand
C     
C     Purpose: Copy vector distributed among processors to processor 0
C
C     Algorithm: 3-phase : z-axis, y-axis, then x-axis collapse
C
C     Inputs:  X distributed among all processors
C
C     Outputs: Y on processor 0
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmVectorExpand (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,level,
     &     comm,icall)

      IMPLICIT NONE
      
      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz,level
      INTEGER comm

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      INTEGER ix,iy,iz,ierr,j
      INTEGER status(MPI_STATUS_SIZE)

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_SUM
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      IF (ip.EQ.0) THEN
         DO j=0,ip-1
            CALL MPI_Send (Y,1,gmmcgrid(level,j),j,j,comm,ierr)
         END DO
      ELSE
         CALL MPI_Recv (X,1,gmmgrid(level),0,ip,comm,status,ierr)
      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmVectorCopy
C     
C     PURPOSE: Copy one vector to another
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmVectorCopy (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz)

      IMPLICIT NONE

      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz

#include "mgmpi_internal.h"

      INTEGER ix,iy,iz

      DO iz=1,nz
         DO iy=1,ny
            DO ix=1,nx
               Y(ix,iy,iz) = X(ix,iy,iz)
            END DO
         END DO
      END DO

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmVectorCopy2
C     
C     PURPOSE: Masked copy of multiple planar (2D) vectors 
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmVectorCopy2 (
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,lx,ly,lz,mask)

      IMPLICIT NONE

      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      LOGICAL lx,ly,lz,mask(*)

#include "mgmpi_internal.h"

      INTEGER ix,iy,iz

      IF (.NOT.lx) THEN
         DO iz=1,nz
            DO iy=1,ny
               DO ix=1,nx
                  IF (mask(ix)) Y(ix,iy,iz) = X(ix,iy,iz)
               END DO
            END DO
         END DO
      ELSE IF (.NOT.ly) THEN
         DO iz=1,nz
            DO iy=1,ny
               DO ix=1,nx
                  IF (mask(iy)) Y(ix,iy,iz) = X(ix,iy,iz)
               END DO
            END DO
         END DO
      ELSE IF (.NOT.lz) THEN
         DO iz=1,nz
            DO iy=1,ny
               DO ix=1,nx
                  IF (mask(iz)) Y(ix,iy,iz) = X(ix,iy,iz)
               END DO
            END DO
         END DO
      END IF

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmVectorInfo
C     
C     PURPOSE: Print out various properties of a vector
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmVectorInfo (varname,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm)

      IMPLICIT NONE

      CHARACTER*4 varname
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      INTEGER comm 

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      INTEGER ix,iy,iz,ierr
      REAL amaxax,aminax,asumax,amaxx,aminx,asumx
      REAL bmaxax,bminax,bsumax,bmaxx,bminx,bsumx

      amaxax = 0.0D0
      aminax = 1D100
      asumax = 0.0D0
      amaxx = -1D100
      aminx = 1D100
      asumx = 0.0D0
      DO iz=2,nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               amaxax = MAX(amaxax,ABS(X(ix,iy,iz)))
               aminax = MIN(aminax,ABS(X(ix,iy,iz)))
               asumax = asumax + ABS(X(ix,iy,iz))
               amaxx = MAX(amaxx,X(ix,iy,iz))
               aminx = MIN(aminx,X(ix,iy,iz))
               asumx = asumx + X(ix,iy,iz)
            END DO
         END DO
      END DO

#ifdef MPI_USED
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmVectorInfo: MPI_Allreduce')") ip
#endif
      bmaxax = amaxax ! (in case allreduce is not called for one processor)
      CALL MPI_Allreduce (amaxax,bmaxax,1,MM_REAL,MPI_MAX,comm,ierr)
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmVectorInfo: MPI_Allreduce')") ip
#endif
      bminax = aminax ! (in case allreduce is not called for one processor)
      CALL MPI_Allreduce (aminax,bminax,1,MM_REAL,MPI_MIN,comm,ierr)
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmVectorInfo: MPI_Allreduce')") ip
#endif
      bsumax = asumax ! (in case allreduce is not called for one processor)
      CALL MPI_Allreduce (asumax,bsumax,1,MM_REAL,MPI_SUM,comm,ierr)
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmVectorInfo: MPI_Allreduce')") ip
#endif
      bmaxx = amaxx ! (in case allreduce is not called for one processor)
      CALL MPI_Allreduce (amaxx,bmaxx,1,MM_REAL,MPI_MAX,comm,ierr)
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmVectorInfo: MPI_Allreduce')") ip
#endif
      bminx = aminx ! (in case allreduce is not called for one processor)
      CALL MPI_Allreduce (aminx,bminx,1,MM_REAL,MPI_MIN,comm,ierr)
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmVectorInfo: MPI_Allreduce')") ip
#endif
      bsumx = asumx ! (in case allreduce is not called for one processor)
      CALL MPI_Allreduce (asumx,bsumx,1,MM_REAL,MPI_SUM,comm,ierr)
#else
      bmaxax = amaxax
      bminax = aminax
      bsumax = asumax
      bmaxx = amaxx
      bminx = aminx
      bsumx = asumx
#endif

      IF (ip.EQ.0) THEN
         print*, '---- Vector ',varname,' info: -------------'
         WRITE (*,"(5x,'|min| |max| |sum| = ',3e12.3)") 
     &        bminax,bmaxax, bsumax
         WRITE (*,"(5x,' min   max   sum  = ',3e12.3)") 
     &        bminx,bmaxx, bsumx
      END IF
#ifdef MPI_USED
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmVectorInfo: MPI_Barrier')") ip
#endif
      CALL MPI_Barrier (comm,ierr)
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmVectorIsNan
C     
C     PURPOSE: Display processor and indices of vector elements which
C     PURPOSE: are NaN's 
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmVectorIsNan (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz,comm

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      INTEGER mmIsNan,mmIsFinite
      EXTERNAL mmIsNan,mmIsFinite

      INTEGER ix,iy,iz,ierr

#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmVectorIsNan: MPI_BARRIER')") ip
#endif
#ifdef MPI_USED
      CALL MPI_BARRIER (comm,ierr)
#endif
      DO iz=2,nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               IF (mmIsNan(X(ix,iy,iz)).NE.0) THEN
                  write(*,1000) ipx,ipy,ipz,ix,iy,iz
 1000             FORMAT (1x,'nan: ',6i4)
               ELSE IF (mmIsFinite(X(ix,iy,iz)).EQ.0) THEN
                  write(*,2000) ipx,ipy,ipz,ix,iy,iz
 2000             FORMAT (1x,'inf: ',6i4)
               END IF
            END DO
         END DO
      END DO
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmVectorIsNan: MPI_BARRIER')") ip
#endif
#ifdef MPI_USED
      CALL MPI_BARRIER (comm,ierr)
#endif


      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  REAL FUNCTION mmConverge4
C     
C     PURPOSE: Return value for componentwise relative backward error
C     PURPOSE: (See "Templates", Dongarra et. al., p.55, Criterium 4)
C
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      REAL FUNCTION mmConverge4(
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,icall)

      IMPLICIT NONE

      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER comm,nbr(6),bnd(3)
      INTEGER nx,ny,nz

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      INTEGER ix,iy,iz,ierr
      REAL s,t

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_CONVERGE+1
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef DEBUG
      IF (ip.EQ.0) print*, 'mmConverge4 1',nx,ny,nz
#endif

      CALL mmGetGhost (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)

      s=0.0D0
      DO iz=2,nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               s = MAX(s,ABS(R(ix,iy,iz)) / (
     &              ABS(B(ix,iy,iz))
     &              + ABS(AD(ix,iy,iz) * X(ix,iy,iz))
     &              + ABS(A1(ix,iy,iz)  *X(ix+1,iy,iz))
     &              + ABS(A2(ix,iy,iz)  *X(ix,iy+1,iz))
     &              + ABS(A3(ix,iy,iz)  *X(ix,iy,iz+1))
     &              + ABS(A4(ix-1,iy,iz)*X(ix-1,iy,iz))
     &              + ABS(A5(ix,iy-1,iz)*X(ix,iy-1,iz))
     &              + ABS(A6(ix,iy,iz-1)*X(ix,iy,iz-1))))
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 15*(nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 15*(nx-2)*(ny-2)*(nz-2)
#endif

#ifdef DEBUG
      IF (ip.EQ.0) print*, 'mmConverge4 2',nx,ny,nz
#endif

#ifdef MPI_USED
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,'mmConverge4: MPI_Allreduce')")  ip
#endif
      t = s ! (in case allreduce is not called for one processor)
      CALL MPI_Allreduce (s,t,1,MM_REAL,MPI_MAX,comm,ierr)
      mmConverge4 = t
#else
      mmConverge4 = s
#endif

#ifdef DEBUG
      IF (ip.EQ.0) print*, 'mmConverge4 3',nx,ny,nz
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmConvergeTest
C     
C     Purpose: Return result of a convergence criterium
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      LOGICAL FUNCTION mmIsConverged(iStat,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     R,nxdr,nydr,nzdr,bndr
     &     ,nx,ny,nz,
     &     anorm,bnorm,rnorm,rnorm0,resTolFine,iter,maxIter,
     &     comm,nbr,bnd
     &     ,level,icall)

      IMPLICIT NONE

      INTEGER iStat
      
      INTEGER nxda,nyda,nzda
      INTEGER bnda(3)  
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdb,nydb,nzdb
      INTEGER bndb(3)
      REAL B(nxdb,nydb,nzdb)
      
      INTEGER nxdr,nydr,nzdr
      INTEGER bndr(3)	
      REAL R(nxdr,nydr,nzdr)
      INTEGER nx,ny,nz
      REAL bnorm,rnorm,rnorm0,resTolFine
      INTEGER iter,maxIter
      
      INTEGER comm,nbr(6),bnd(3)
      INTEGER level

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

#ifdef DEBUG
      REAL tmp
#endif

      REAL anorm,xnorm
      
      INTEGER mmIsNan
      REAL mmDotProd,mmVectorMaxRatio,mmConverge4
      EXTERNAL mmIsNan,mmDotProd,mmVectorMaxRatio,mmConverge4

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_CONVERGE
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef DEBUG
      IF (ip.EQ.0) print*, 'entering mmConvergeTest'
#endif

*-----------------------------------------------------------------------
*     Check for "divergence"
*-----------------------------------------------------------------------

C     ( These can cause problems if values are different on different)
C     ( processors. )

      IF (rnorm/rnorm0.GT.10.0) THEN
         istat = 1
         mmIsConverged = .TRUE.  ! [sic]: just to exit loop
         GO TO 10
      END IF

C@@@      IF (mmIsNan(rnorm).NE.0) THEN
      IF (.FALSE.) THEN
         istat = 2
         mmIsConverged = .TRUE.  ! [sic]: just to exit loop
         GO TO 10
      END IF

      IF (iter .GE. maxIter) THEN
         istat = 3
         mmIsConverged = .TRUE.  ! [sic]: just to exit loop
         GOTO 10
      END IF

*-----------------------------------------------------------------------
*     Check for "convergence"
*-----------------------------------------------------------------------

C     Set default convergence to false

      istat = 0
      mmIsConverged = .FALSE.

*------------------------------------------------
*     ||R|| <= epsilon * ( ||A|| * ||X|| + ||B||)
*------------------------------------------------

      IF (CONV_CRIT.EQ.1) THEN

         xnorm = DSQRT (mmDotProd (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,level,isub))

#ifdef DEBUG
         IF (ip.EQ.0) THEN
            print*, 'Convergence criterion 1:'
            print*, '||R|| <= tol * (||A|| ||X|| + ||B||)'
            print*, 'rnorm = ',rnorm
            print*, 'tol = ',resTolFine
            print*, 'anorm = ',anorm
            print*, 'xnorm = ',xnorm
            print*, 'bnorm = ',bnorm
         END IF
#endif
            
         IF (rnorm.LE.resTolFine*(anorm*xnorm+bnorm)) THEN
            mmIsConverged = .TRUE.
            GO TO 10
         END IF

*-------------------------------------------
*     ||R|| <= epsilon * ( ||B|| )
*-------------------------------------------

      ELSE IF (CONV_CRIT.EQ.2) THEN 

#ifdef DEBUG
         IF (ip.EQ.0) THEN
            print*, 'Convergence criterion 2:'
            print*, '||R|| <= tol * ||B||'
            print*, 'rnorm = ',rnorm
            print*, 'tol = ',resTolFine
            print*, 'bnorm = ',bnorm
         END IF
#endif
         IF (rnorm.LE.resTolFine*bnorm) THEN
            mmIsConverged = .TRUE.
            GO TO 10
         END IF

*-------------------------------------------
*     Componentwise |R(i)| <= epsilon * ( |B(i)| ) for all i
*-------------------------------------------

      ELSE IF (CONV_CRIT.EQ.3) THEN

C-------------------------------------------------------------
C  |R(i)/B(i)| <= eps for all i               (Most stringent)
C-------------------------------------------------------------

#ifdef DEBUG
         tmp = mmVectorMaxRatio (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,comm)
         IF (ip.EQ.0) THEN
            print*, 'Convergence criterion 3:'
            print*, 'max |R(i)/B(i)| <= tol '
            print*, 'rnorm = ',rnorm
            print*, 'bnorm = ',bnorm
            print*, 'tol = ',resTolFine
            print*, 'max |R(i)/B(i)| = ', tmp
         END IF
#endif

         IF (mmVectorMaxRatio (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,nx,ny,nz,comm)
     &        .LE. resTolFine)  THEN
            mmIsConverged = .TRUE.
            GO TO 10
         END IF

      ELSE IF (CONV_CRIT.EQ.4) THEN

#ifdef DEBUG
         tmp = mmConverge4 (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
         IF (ip.EQ.0) THEN
            print*, 'Convergence criterion 4: '
            print*, 'max |R(i)|/(|B|+|A|*|X|)(i) <= tol'
            print*, 'rnorm = ',rnorm
            print*, 'tol = ',resTolFine
            print*, 'bnorm = ',bnorm
            print*, 'max |R(i)|/(|B|+|A|*|X|)(i) = ', tmp
         END IF
#endif
         IF (mmConverge4 (
     &     R,nxdr,nydr,nzdr,bndr
     &     ,
     &     B,nxdb,nydb,nzdb,bndb
     &     ,
     &     AD,A1,A2,A3,A4,A5,A6,nxda,nyda,nzda,bnda
     &     ,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,
     &     comm,nbr,bnd
     &     ,isub)
     &        .LE. resTolFine) THEN
            mmIsConverged = .TRUE.
            GO TO 10
         END IF

      ELSE

         IF (ip.EQ.0) THEN
            PRINT*, '== mgmpi ERROR == unknown value ',CONV_CRIT,
     &           ' for CONV_CRIT'
            PRINT*, '                  Should be between 1 and 4'
         END IF

      END IF

C--------------------------------------------------------------------

 10   CONTINUE

#ifdef DEBUG
      IF (ip.EQ.0) print*, 'exiting mmConvergeTest'
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  REAL FUNCTION mmVectorMax
C     
C     PURPOSE: Return the size of the vector element of largest magnitude
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      REAL FUNCTION mmVectorMax (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      INTEGER comm 

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      INTEGER ix,iy,iz,ierr
      REAL a,b

      a=0.0D0
      DO iz=2,nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               a = MAX(a,ABS(X(ix,iy,iz)))
            END DO
         END DO
      END DO

#ifdef MPI_USED
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmVectorMax: MPI_Allreduce')") -1
#endif
      b = a ! (in case allreduce is not called for one processor)
      CALL MPI_Allreduce (a,b,1,MM_REAL,MPI_MAX,comm,ierr)
      mmVectorMax = b
#else
      mmVectorMax = a
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  REAL FUNCTION mmVectorMax2
C     
C     PURPOSE: Return the sizes of the 2-D-vector elements of largest magnitude
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmVectorMax2 (b,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,lx,ly,lz,mask)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      INTEGER comm 
      LOGICAL lx,ly,lz,mask(*)

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      INTEGER ix,iy,iz,ierr
      REAL a,b(*)

      IF (.NOT.lx) THEN
         DO ix = 2,nx-1
            IF (mask(ix)) THEN
               a=(0.0D0)
               DO iz=2,nz-1
                  DO iy=2,ny-1
                     a = MAX(a,ABS(X(ix,iy,iz)))
                  END DO
               END DO
#ifdef MPI_USED
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmVectorMax2: MPI_Allreduce')")  -1
#endif
               b(ix)=a ! (in case allreduce is not called for one processor) 
               CALL MPI_Allreduce(a,b(ix),1,MM_REAL,MPI_MAX,
     &              gsubcomm(0,1,1),ierr)
#else 
               b(ix)=a
#endif
            END IF
         END DO
      ELSE IF (.NOT.ly) THEN
         DO iy = 2,ny-1
            IF (mask(iy)) THEN
               a=(0.0D0)
               DO iz=2,nz-1
                  DO ix=2,nx-1
                     a = MAX(a,ABS(X(ix,iy,iz)))
                  END DO
               END DO
#ifdef MPI_USED
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmVectorMax2: MPI_Allreduce')")  -1
#endif
               b(iy)=a ! (in case allreduce is not called for one processor)
               CALL MPI_Allreduce(a,b(iy),1,MM_REAL,MPI_MAX,
     &              gsubcomm(1,0,1),ierr)
#else
               b(iy)=a
#endif
            END IF
         END DO
      ELSE IF (.NOT.lz) THEN
         DO iz = 2,nz-1
            IF (mask(iz)) THEN
               a=(0.0D0)
               DO iy=2,ny-1
                  DO ix=2,nx-1
                     a = MAX(a,ABS(X(ix,iy,iz)))
                  END DO
               END DO
#ifdef MPI_USED
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmVectorMax2: MPI_Allreduce')")  -1
#endif
               b(iz)=a ! (in case allreduce is not called for one processor)
               CALL MPI_Allreduce(a,b(iz),1,MM_REAL,MPI_MAX,
     &              gsubcomm(1,1,0),ierr)
#else
               b(iz)=a
#endif
            END IF
         END DO
      ELSE
         PRINT*, 'MPI Error in mmVectorMax2: lx,ly,lz=',lx,ly,lz
         CALL mmTerminate(comm)
      END IF
      
      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  REAL FUNCTION mmVectorMaxRatio
C     
C     PURPOSE: Return the size of the largest magnitude ratio X(i)/Y(i)
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      REAL FUNCTION mmVectorMaxRatio (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,nx,ny,nz,comm)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      INTEGER nx,ny,nz
      INTEGER comm 

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      INTEGER ix,iy,iz,ierr
      REAL a,b

#ifdef DEBUG
      print*, 'mmVectorMaxRatio 1',nx,ny,nz
#endif

      a=0.0D0
      DO iz=2,nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               IF (ABS(Y(ix,iy,iz)).GT.mmSmall) 
     &              a = MAX(a,ABS(X(ix,iy,iz)/Y(ix,iy,iz)))
            END DO
         END DO
      END DO

#ifdef DEBUG
      print*, 'mmVectorMaxRatio 2',nx,ny,nz
#endif

#ifdef MPI_USED
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmVectorMaxRatio: MPI_Allreduce')") -1
#endif
      b = a ! (in case allreduce is not called for one processor)
      CALL MPI_Allreduce (a,b,1,MM_REAL,MPI_MAX,comm,ierr)
      mmVectorMaxRatio = b
#else
      mmVectorMaxRatio = a
#endif

#ifdef DEBUG
      print*, 'mmVectorMaxRatio 3',nx,ny,nz
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  REAL FUNCTION mmVectorNormalize
C     
C     PURPOSE: If no boundary conditions are Dirichlet, scale the vector
C     PURPOSE: such that its constant component is 0.  I.e. so sum(x)=0
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      REAL FUNCTION mmVectorNormalize (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,nxg,nyg,nzg,
     &     XPB,XMB,YPB,YMB,ZPB,ZMB,nxdbnd,nydbnd,nzdbnd,
     &     ixpb,ixmb,iypb,iymb,izpb,izmb
     &     ,comm,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      INTEGER nxg,nyg,nzg
      
      INTEGER nxdbnd,nydbnd,nzdbnd
      REAL XPB(nydbnd,nzdbnd),XMB(nydbnd,nzdbnd)
      REAL YPB(nxdbnd,nzdbnd),YMB(nxdbnd,nzdbnd)
      REAL ZPB(nxdbnd,nydbnd),ZMB(nxdbnd,nydbnd)
      INTEGER ixpb,ixmb,iypb,iymb,izpb,izmb
      INTEGER comm

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      REAL s,mmVectorSum

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif
      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_NORMALIZE
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      s = 0

      IF (ABS(ixmb).NE.MM_DIRICHLET .AND.
     &    ABS(iymb).NE.MM_DIRICHLET .AND. 
     &    ABS(izmb).NE.MM_DIRICHLET .AND.
     &    ABS(ixpb).NE.MM_DIRICHLET .AND.
     &    ABS(iypb).NE.MM_DIRICHLET .AND.
     &    ABS(izpb).NE.MM_DIRICHLET) THEN

         s = mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,isub)
         s = -s / ((nxg-2)*(nyg-2)*(nzg-2))
         CALL mmVectorShift (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,s,comm,isub)

      END IF

      mmVectorNormalize = s

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmVectorShift
C     
C     PURPOSE: Add the given scalar  to each vector element
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmVectorShift (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,s,comm,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      REAL s
      INTEGER comm

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      INTEGER ix,iy,iz

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif
      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_SHIFT
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      DO iz=2,nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               X(ix,iy,iz) = X(ix,iy,iz) + s
            END DO
         END DO
      END DO

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmVectorShift2
C     
C     PURPOSE: Add the given scalar shifts to each planar vector 
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmVectorShift2 (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,s,comm,lx,ly,lz,mask,icall)

      IMPLICIT NONE

      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      REAL s(*)
      INTEGER comm
      LOGICAL lx,ly,lz,mask(*)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif
      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_SHIFT
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef ASSERT
      IF (.NOT.(((.NOT.lx).AND.ly.AND.lz) .OR.
     &          (lx.AND.(.NOT.ly).AND.lz) .OR.
     &          (lx.AND.ly.AND.(.NOT.lz)))) THEN
         PRINT*, 'mmVectorShift2 ASSERTION FAILED!'
         PRINT*, 'lx ly lz =',lx,ly,lz
      END IF
#endif

      IF (.NOT.lx) THEN
         DO ix=2,nx-1
            IF (mask(ix)) THEN
               DO iz=2,nz-1
                  DO iy=2,ny-1
                     X(ix,iy,iz) = X(ix,iy,iz) + s(ix)
                  END DO
               END DO
            END IF
         END DO
      ELSE IF (.NOT.ly) THEN
         DO iy=2,ny-1
            IF (mask(iy)) THEN
               DO iz=2,nz-1
                  DO ix=2,nx-1
                     X(ix,iy,iz) = X(ix,iy,iz) + s(iy)
                  END DO
               END DO
            END IF
         END DO
      ELSE IF (.NOT.lz) THEN
         DO iz=2,nz-1
            IF (mask(iz)) THEN
               DO iy=2,ny-1
                  DO ix=2,nx-1
                     X(ix,iy,iz) = X(ix,iy,iz) + s(iz)
                  END DO
               END DO
            END IF
         END DO
      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  REAL FUNCTION mmVectorSum
C     
C     PURPOSE: Return the sum of vector elements
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      REAL FUNCTION mmVectorSum (
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,icall)

      IMPLICIT NONE
      
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      INTEGER comm

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      INTEGER ix,iy,iz,ierr
      REAL a,b

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif
      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_SUM
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      a=0.0D0
      DO iz=2,nz-1
         DO iy=2,ny-1
            DO ix=2,nx-1
               a = a + X(ix,iy,iz)
            END DO
         END DO
      END DO
      
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (nx-2)*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & (nx-2)*(ny-2)*(nz-2)
#endif

#ifdef MPI_USED
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmVectorSum: MPI_Allreduce')") -1
#endif
      b = a ! (in case allreduce is not called for one processor)
      CALL MPI_Allreduce (a,b,1,MM_REAL,MPI_SUM,comm,ierr)
      mmVectorSum = b
#else
      mmVectorSum = a
#endif

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  REAL FUNCTION mmVectorSum2
C     
C     PURPOSE: Return the sum of planar (2D) vector elements
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmVectorSum2 (b,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,nx,ny,nz,comm,lx,ly,lz,mask,icall)

      IMPLICIT NONE
      
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      INTEGER nx,ny,nz
      INTEGER comm
      LOGICAL lx,ly,lz,mask(*)
      REAL b(*)

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

      INTEGER ix,iy,iz,ierr
      REAL a

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif
      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_SUM
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

#ifdef ASSERT
      IF (.NOT.(((.NOT.lx).AND.ly.AND.lz) .OR.
     &          (lx.AND.(.NOT.ly).AND.lz) .OR.
     &          (lx.AND.ly.AND.(.NOT.lz)))) THEN
         PRINT*, 'mmVectorSum2 ASSERTION FAILED!'
         PRINT*, 'lx ly lz =',lx,ly,lz
      END IF
#endif

      IF (.NOT.lx) THEN
         DO ix = 2,nx-1
            IF (mask(ix)) THEN
               a=(0.0D0)
               DO iz=2,nz-1
                  DO iy=2,ny-1
                     a = a + X(ix,iy,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & (ny-2)*(nz-2)
#endif
#if defined(MPI_USED)
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmVectorSum2: MPI_Allreduce')") -1
#endif
               b(ix) = a ! (in case allreduce is not called for one processor)
               CALL MPI_Allreduce(a,b(ix),1,MM_REAL,MPI_SUM,
     &              gsubcomm(0,1,1),ierr)
#ifdef ASSERT
               IF (a.NE.b(ix)) THEN
                  PRINT*, 'mmVectorSum2 ASSERTION FAILED!'
                  PRINT*, 'ix a b(ix) =',ix,a,b(ix)
               END IF
#endif
#else
               b(ix)=a
#endif
            END IF
         END DO
      ELSE IF (.NOT.ly) THEN
         DO iy = 2,ny-1
            IF (mask(iy)) THEN
               a=(0.0D0)
               DO iz=2,nz-1
                  DO ix=2,nx-1
                     a = a + X(ix,iy,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & (nx-2)*(nz-2)
#endif
#if defined(MPI_USED)
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmVectorSum2: MPI_Allreduce')") -1
#endif
               b(iy) = a ! (in case allreduce is not called for one processor)
               CALL MPI_Allreduce(a,b(iy),1,MM_REAL,MPI_SUM,
     &              gsubcomm(1,0,1),ierr)
#ifdef ASSERT
               IF (a.NE.b(iy)) THEN
                  PRINT*, 'mmVectorSum2 ASSERTION FAILED!'
                  PRINT*, 'iy a b(iy) =',iy,a,b(iy)
               END IF
#endif
#else
               b(iy)=a
#endif
            END IF
         END DO
      ELSE IF (.NOT.lz) THEN
         DO iz = 2,nz-1
            IF (mask(iz)) THEN
               a=(0.0D0)
               DO iy=2,ny-1
                  DO ix=2,nx-1
                     a = a + X(ix,iy,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & (ny-2)*(nx-2)
      gmflops(isub)=gmflops(isub) + 
     & (ny-2)*(nx-2)
#endif
#if defined(MPI_USED)
#ifdef MPI_TRACE
               WRITE(*,"(1x,i3,' mmVectorSum2: MPI_Allreduce')") -1
#endif
               b(iz) = a ! (in case allreduce is not called for one processor)
               CALL MPI_Allreduce(a,b(iz),1,MM_REAL,MPI_SUM,
     &              gsubcomm(1,1,0),ierr)
#ifdef ASSERT
               IF (a.NE.b(iz)) THEN
                  PRINT*, 'mmVectorSum2 ASSERTION FAILED!'
                  PRINT*, 'iz a b(iz) =',iz,a,b(iz)
               END IF
#endif
#else
               b(iz)=a
#endif
            END IF
         END DO
      ELSE
         PRINT*, 'MPI Error in mmVectorSum2: lx,ly,lz=',lx,ly,lz
         CALL mmTerminate(comm)
      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmWriteFlopRate
C     
C     PURPOSE: Output the flop rate for each instrumented routine
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmWriteFlopRate
      IMPLICIT NONE
#include "mgmpi_internal.h"
      INTEGER i
      DO i=1,NTIMER
         IF(gmflops(i).NE.0 .AND. gtime(i).NE.0) THEN
             WRITE(*,3000) gtimename(i), (gmflops(i)*1.0D-6)/gtime(i)
         END IF
 3000    FORMAT (3x,a18,'MFlop rate: ',f7.2)
      END DO
      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmWriteFlops
C     
C     PURPOSE: Output the flop counts for each instrumented routine
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmWriteFlops
      IMPLICIT NONE
#include "mgmpi_internal.h"
      INTEGER i
      DO i=1,NTIMER
         IF(gmflops(i).NE.0) THEN
             WRITE(*,2000) gtimename(i), gmflops(i)/1.0D6
         END IF
 2000    FORMAT (3x,a18,' MFlops: ',f15.4)
      END DO
      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmWriteMatrix
C     
C     Purpose: Dump the matrix to the given file
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmWriteMatrix (filename,
     &     AD,A1,A2,A3,A4,A5,A6,
     &     nxda,nyda,nzda,nx,ny,nz,
     &     ix1,iy1,iz1,comm,nbr)
      IMPLICIT NONE
      CHARACTER*10 filename
      INTEGER nxda,nyda,nzda,nx,ny,nz,ix1,iy1,iz1,comm,nbr(6)
      REAL AD(nxda,nyda,nzda)
      REAL A1(nxda,nyda,nzda)
      REAL A2(nxda,nyda,nzda)
      REAL A3(nxda,nyda,nzda)
      REAL A4(nxda,nyda,nzda)
      REAL A5(nxda,nyda,nzda)
      REAL A6(nxda,nyda,nzda)

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

*-----------------------------------------------------------------------
*     Write a matrix to a file.  Note--may not output elements in the
*     correct order if more than 1 processor is used.
*-----------------------------------------------------------------------

      INTEGER ix,iy,iz,wait
      CHARACTER*18 fullFilename
      INTEGER n,nx1,nx2,ny1,ny2,nz1,nz2

#ifdef MPI_USED
      INTEGER status(MPI_STATUS_SIZE)
      INTEGER ierr
#endif

      IF (nx.GT.33) return      ! safety to prevent huge files

      nx1=2
      ny1=2
      nz1=2
      IF (ipx.EQ.0) nx1=1
      IF (ipy.EQ.0) ny1=1
      IF (ipz.EQ.0) nz1=1

      nx2=nx-1
      ny2=ny-1
      nz2=nz-1
      IF (ipx.EQ.npx-1) nx2=nx
      IF (ipy.EQ.npy-1) ny2=ny
      IF (ipz.EQ.npz-1) nz2=nz

#ifdef MPI_USED
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmWriteMatrix: MPI_Barrier')") ip
#endif
      CALL MPI_Barrier (comm,ierr)
      if (ierr.NE.0) print*, 'MPI Error in write_matrix: ',ierr
      IF (ip .GT. 0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmWriteMatrix: MPI_Recv')") ip
#endif
         CALL MPI_Recv (wait,1,MPI_INTEGER,ip-1,100,comm,status,ierr)
         if (ierr.NE.0) print*, 'MPI Error in write_matrix: ',ierr
      END IF
#endif

      n = 10
      DO n=10,1,-1
         IF (filename(n:n).NE.' ') GOTO 10
      END DO
 10   CONTINUE
      fullFilename = filename(1:n) // '.' 
     &     // CHAR(ICHAR('0')+ipx)
     &     // CHAR(ICHAR('0')+ipy)
     &     // CHAR(ICHAR('0')+ipz)
      
      OPEN (UNIT=9, FILE = fullFilename)

      DO iz=nz1,nz2
         DO iy=ny1,ny2
            DO ix=nx1,nx2
               write(9,100) ix+ix1,iy+iy1,iz+iz1,
     &              A6(ix,iy,iz-1),
     &              A5(ix,iy-1,iz),
     &              A4(ix-1,iy,iz),
     &              AD(ix,iy,iz),
     &              A1(ix,iy,iz),
     &              A2(ix,iy,iz),
     &              A3(ix,iy,iz)
            END DO
         END DO
      END DO
  100 FORMAT (1x,3i3,7f8.4)

      CLOSE (9)

#ifdef MPI_USED
      IF (ip.LT.np-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmWriteMatrix: MPI_Send')") ip
#endif
         CALL MPI_Send (wait,1,MPI_INTEGER,ip+1,100,comm,ierr)
         if (ierr.NE.0) print*, 'MPI Error in write_matrix: ',ierr
      END IF

#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmWriteMatrix: MPI_Barrier')") ip
#endif
      CALL MPI_Barrier (comm,ierr)
      if (ierr.NE.0) print*, 'MPI Error in write_matrix: ',ierr
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmWritePercents
C     
C     PURPOSE: Output the percent times for each instrumented routine
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmWritePercents
      IMPLICIT NONE
#include "mgmpi_internal.h"
      INTEGER i
      DO i=1,NTIMER
         IF(gtimecount(i).NE.0) THEN
             WRITE(*,1000) gtimename(i), 100.0*gtime(i)/gtime(1)
         END IF
 1000    FORMAT (3x,a18,' Percent time: ',f8.3)
      END DO
      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmWriteSlice
C     
C     PURPOSE: Dump a slice of the vector to the given file
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmWriteSlice (filename,X,
     &     nxd,nyd,nzd,nx,ny,nz,
     &     x1,y1,z1,ix1,iy1,iz1,hx,hy,hz,
     &     comm,nbr)
      IMPLICIT NONE
      CHARACTER*10 filename
      INTEGER nxd,nyd,nzd,nx,ny,nz,ix1,iy1,iz1
      INTEGER comm,nbr(6)
      REAL X(nxd,nyd,nzd)
      REAL hx,hy,hz,x1,y1,z1

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

*-----------------------------------------------------------------------
*     Write a vector to a file.  Note--may not output elements in the
*     correct order if more than 1 processor is used.
*-----------------------------------------------------------------------

      INTEGER ix,iy,iz,wait,nx1,nx2,ny1,ny2,nz1,nz2
#ifdef MPI_USED
      INTEGER status(MPI_STATUS_SIZE)
      INTEGER ierr
#endif

      REAL xval1,yval1,zval1

      nx1=2
      ny1=2
      nz1=2
C      xval1=x1+hx(2)
C      yval1=y1+hy(2)
C      zval1=z1+hz(2)
      IF (ipx.EQ.0) nx1=1
      IF (ipy.EQ.0) ny1=1
      IF (ipz.EQ.0) nz1=1
      IF (ipx.EQ.0) xval1=x1
      IF (ipy.EQ.0) yval1=y1
      IF (ipz.EQ.0) zval1=z1

      nx2=nx-1
      ny2=ny-1
      nz2=nz-1
      IF (ipx.EQ.npx-1) nx2=nx
      IF (ipy.EQ.npy-1) ny2=ny
      IF (ipz.EQ.npz-1) nz2=nz

#ifdef MPI_USED
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmWriteSlice: MPI_Barrier')") ip
#endif
      CALL MPI_Barrier (comm,ierr)
      if (ierr.NE.0) print*, 'MPI Error in write_vector: ',ierr
      IF (ip .GT. 0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmWriteSlice: MPI_Recv')") ip
#endif
         CALL MPI_Recv (wait,1,MPI_INTEGER,ip-1,100,comm,status,ierr)
         if (ierr.NE.0) print*, 'MPI Error in write_vector: ',ierr
      END IF
#endif

      OPEN (UNIT=9, FILE = filename)
C     zval = zval1
      iz=(nz1+nz2)/2
C         yval = yval1
         DO iy=ny1,ny2
C            xval = xval1
            DO ix=nx1,nx2
               write(9,*) ix1+ix,iy1+iy,iz1+iz,X(ix,iy,iz)
*               write(9,1000) xval,yval,zval,X(ix,iy,iz)
* 1000          format (e11.4,e11.4,e11.4,e18.10)
C               xval = xval + hx(ix+1)
            END DO
C            yval = yval + hy(iy+1)
         END DO
C         zval = zval + hz(iz+1)

      CLOSE (9)

#ifdef MPI_USED
      IF (ip.LT.np-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmWriteSlice: MPI_Send')") ip
#endif
         CALL MPI_Send (wait,1,MPI_INTEGER,ip+1,100,comm,ierr)
         if (ierr.NE.0) print*, 'MPI Error in write_vector: ',ierr
      END IF

#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmWriteSlice: MPI_Barrier')") ip
#endif
      CALL MPI_Barrier (comm,ierr)
      if (ierr.NE.0) print*, 'MPI Error in write_vector: ',ierr
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmWriteTimes
C     
C     PURPOSE: Output the execution time for each instrumented routine
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmWriteTimes
      IMPLICIT NONE
#include "mgmpi_internal.h"
      INTEGER i
      DO i=1,NTIMER
         IF(gtimecount(i).NE.0) THEN
             WRITE(*,1000) gtimename(i), gtime(i)
         END IF
 1000    FORMAT (3x,a18,' Time: ',f15.8)
      END DO
      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmWriteTotalFlopRate
C     
C     PURPOSE: Output the overall flop rate for each instrumented routine
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmWriteTotalFlopRate (comm)
      IMPLICIT NONE
*     sum of flops / max of time
#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"
      INTEGER i,comm,ierr
      DOUBLE PRECISION gtimemax,gmflopssum
      
      DO i=1,NTIMER

#ifdef MPI_USED
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmWriteTotalFlopRate: MPI_Allreduce')") ip
#endif
         gmflopssum = gmflops(i) ! (in case allreduce is not called for one processor)
         CALL MPI_Allreduce (gmflops(i),gmflopssum,1,
     &        MPI_REAL8,MPI_SUM,comm,ierr)
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmWriteTotalFlopRate: MPI_Allreduce')") ip
#endif
         gtimemax = gtime(i)
         CALL MPI_Allreduce (gtime(i),gtimemax,1,
     &        MPI_REAL8,MPI_MAX,comm,ierr)
#else 
         gmflopssum = gmflops(i)
         gtimemax = gtime(i)
#endif
         IF(gmflopssum.NE.0 .AND. gtimemax.NE.0 .AND. ip.EQ.0) THEN
            WRITE(*,3000) gtimename(i), (gmflopssum*1.0D-6)/gtimemax
         ELSE IF (gmflopssum.NE.0 .OR. gtimemax.NE.0 .AND. ip.EQ.0) THEN
         END IF
 3000    FORMAT (3x,a18,'Total MFlop rate: ',f7.2)
      END DO
       
      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmWriteVector
C     
C     Purpose: Write the given vector to the file "filename.<ipx><ipy><ipz>"
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmWriteVector (filename,X,
     &     nxd,nyd,nzd,nx,ny,nz,
     &     ix1,iy1,iz1,
     &     comm,nbr)

      IMPLICIT NONE

      CHARACTER*10 filename
      INTEGER nxd,nyd,nzd,nx,ny,nz,ix1,iy1,iz1
      INTEGER comm,nbr(6)
      REAL X(nxd,nyd,nzd)

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

*-----------------------------------------------------------------------
*     Write a vector to files 'filename.<ipx><ipy><ipz>'.  
*-----------------------------------------------------------------------

      INTEGER ix,iy,iz,wait,nx1,nx2,ny1,ny2,nz1,nz2
      CHARACTER*18 fullFilename
      INTEGER n
      REAL zmin,zmax

#ifdef MPI_USED
      INTEGER status(MPI_STATUS_SIZE)
      INTEGER ierr
#endif

      PRINT*, 'dumping <',filename,'> ...'

      nx1=2
      ny1=2
      nz1=2
      IF (ipx.EQ.0) nx1=1
      IF (ipy.EQ.0) ny1=1
      IF (ipz.EQ.0) nz1=1

      nx2=nx-1
      ny2=ny-1
      nz2=nz-1
      IF (ipx.EQ.npx-1) nx2=nx
      IF (ipy.EQ.npy-1) ny2=ny
      IF (ipz.EQ.npz-1) nz2=nz

#ifdef MPI_USED
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmWriteVector: MPI_Barrier')") ip
#endif
      CALL MPI_Barrier (comm,ierr)
      if (ierr.NE.0) print*, 'MPI Error in write_vector: ',ierr
      IF (ip .GT. 0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmWriteVector: MPI_Recv')") ip
#endif
         CALL MPI_Recv (wait,1,MPI_INTEGER,ip-1,100,comm,status,ierr)
         if (ierr.NE.0) print*, 'MPI Error in write_vector: ',ierr
      END IF
#endif

      n = 10
      DO n=10,1,-1
         IF (filename(n:n).NE.' ') GOTO 10
      END DO
 10   CONTINUE
      fullFilename = filename(1:n) // '.' 
     &     // CHAR(ICHAR('0')+ipx)
     &     // CHAR(ICHAR('0')+ipy)
     &     // CHAR(ICHAR('0')+ipz)
      
      OPEN (UNIT=9, FILE = fullFilename)

      zmin=X(nx1,ny1,nz1)
      zmax=X(nx1,ny1,nz1)
      DO iz=nz1,nz2
         DO iy=ny1,ny2
            DO ix=nx1,nx2
               write(9,1000) ix1+ix,iy1+iy,iz1+iz,X(ix,iy,iz)
 1000          format (3i3,e18.9)
               zmin = MIN(zmin,X(ix,iy,iz))
               zmax = MAX(zmax,X(ix,iy,iz))
            END DO
         END DO
      END DO

      print*, 'ZMIN,ZMAX = ',zmin,zmax
      CLOSE (9)

#ifdef MPI_USED
      IF (ip.LT.np-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmWriteVector: MPI_Send')") ip
#endif
         CALL MPI_Send (wait,1,MPI_INTEGER,ip+1,100,comm,ierr)
         if (ierr.NE.0) print*, 'MPI Error in write_vector: ',ierr
      END IF

#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmWriteVector: MPI_Barrier')") ip
#endif
      CALL MPI_Barrier (comm,ierr)
      if (ierr.NE.0) print*, 'MPI Error in write_vector: ',ierr
#endif

      RETURN
      END
C=======================================================================
C     
C     Module:  SUBROUTINE mmWriteBoundary
C     
C     Purpose: Write the given vector boundary to the file 
C     Purpose: "filename.<ipx><ipy><ipz>"
C
C     Algorithm:
C
C     Inputs:
C
C     Outputs:
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:
C
C     Communication estimate:
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 
C
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmWriteBoundary (filename,X,
     &     nxd,nyd,nzd,nx,ny,nz,ix1,iy1,iz1,
     &     comm,nbr)

      IMPLICIT NONE

      CHARACTER*10 filename
      INTEGER nxd,nyd,nzd,nx,ny,nz,ix1,iy1,iz1
      INTEGER comm,nbr(6)
      REAL X(nxd,nyd,nzd)

#ifdef MPI_USED
      include "mpif.h"
#endif
#include "mgmpi_internal.h"

*-----------------------------------------------------------------------
*     Write a vector to files 'filename.<ipx><ipy><ipz>'.  
*-----------------------------------------------------------------------

      INTEGER ix,iy,iz,wait,nx1,nx2,ny1,ny2,nz1,nz2
      CHARACTER*18 fullFilename
      INTEGER n

#ifdef MPI_USED
      INTEGER status(MPI_STATUS_SIZE)
      INTEGER ierr
#endif

      PRINT*, 'dumping <',filename,'> ...'
      IF (nx.GT.40) return      ! safety to prevent huge files

      nx1=1
      ny1=1
      nz1=1

      nx2=nx
      ny2=ny
      nz2=nz

#ifdef MPI_USED
#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmWriteVector: MPI_Barrier')") ip
#endif
      CALL MPI_Barrier (comm,ierr)
      if (ierr.NE.0) print*, 'MPI Error in write_vector: ',ierr
      IF (ip .GT. 0) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmWriteVector: MPI_Recv')") ip
#endif
         CALL MPI_Recv (wait,1,MPI_INTEGER,ip-1,100,comm,status,ierr)
         if (ierr.NE.0) print*, 'MPI Error in write_vector: ',ierr
      END IF
#endif

      n = 10
      DO n=10,1,-1
         IF (filename(n:n).NE.' ') GOTO 10
      END DO
 10   CONTINUE
      fullFilename = filename(1:n) // '.' 
     &     // CHAR(ICHAR('0')+ipx)
     &     // CHAR(ICHAR('0')+ipy)
     &     // CHAR(ICHAR('0')+ipz)
      
      OPEN (UNIT=9, FILE = fullFilename)

      DO iz=nz1,nz2
         DO iy=ny1,ny2
            ix=nx1
            write(9,1000) ix1+ix,iy1+iy,iz1+iz,X(ix,iy,iz)
            ix=nx2
            write(9,1000) ix1+ix,iy1+iy,iz1+iz,X(ix,iy,iz)
         END DO
      END DO
      DO iz=nz1,nz2
         DO ix=nx1,nx2
            iy=ny1
            write(9,1000) ix1+ix,iy1+iy,iz1+iz,X(ix,iy,iz)
            iy=ny2
            write(9,1000) ix1+ix,iy1+iy,iz1+iz,X(ix,iy,iz)
         END DO
      END DO
      DO iy=ny1,ny2
         DO ix=nx1,nx2
            iz=nz1
            write(9,1000) ix1+ix,iy1+iy,iz1+iz,X(ix,iy,iz)
            iz=nz2
            write(9,1000) ix1+ix,iy1+iy,iz1+iz,X(ix,iy,iz)
         END DO
      END DO

      CLOSE (9)

#ifdef MPI_USED
      IF (ip.LT.np-1) THEN
#ifdef MPI_TRACE
         WRITE(*,"(1x,i3,' mmWriteVector: MPI_Send')") ip
#endif
         CALL MPI_Send (wait,1,MPI_INTEGER,ip+1,100,comm,ierr)
         if (ierr.NE.0) print*, 'MPI Error in write_vector: ',ierr
      END IF

#ifdef MPI_TRACE
      WRITE(*,"(1x,i3,' mmWriteVector: MPI_Barrier')") ip
#endif
      CALL MPI_Barrier (comm,ierr)
      if (ierr.NE.0) print*, 'MPI Error in write_vector: ',ierr
#endif

 1000          format (3i3,e24.15)

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted averagie (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmZaxpy2
C     
C     PURPOSE: Compute Z = a X + Y for vectors X,Y,Z and 1-D array a
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmZaxpy2 (
     &     Z,nxdz,nydz,nzdz,bndz
     &     ,a,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,nx,ny,nz,lx,ly,lz,mask,level,
     &     icall)

      IMPLICIT NONE

      
      INTEGER nxdz,nydz,nzdz
      INTEGER bndz(3)
      REAL Z(nxdz,nydz,nzdz)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      REAL a(*)
      INTEGER nx,ny,nz
      INTEGER level
      LOGICAL lx,ly,lz,mask(*)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_ZAXPY2+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      IF (.NOT.lx) THEN
         DO ix=2,nx-1
            IF (mask(ix)) THEN
               DO iz=2,nz-1
                  DO iy=2,ny-1
                     Z(ix,iy,iz) = a(ix) * X(ix,iy,iz) + Y(ix,iy,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(ny-2)*(nz-2)
#endif
            END IF
         END DO
      ELSE IF (.NOT.ly) THEN
         DO iy=2,ny-1
            IF (mask(iy)) THEN
               DO iz=2,nz-1
                  DO ix=2,nx-1
                     Z(ix,iy,iz) = a(iy) * X(ix,iy,iz) + Y(ix,iy,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(nz-2)
#endif
            END IF
         END DO
      ELSE IF (.NOT.lz) THEN
         DO iz=2,nz-1
            IF (mask(iz)) THEN
               DO iy=2,ny-1
                  DO ix=2,nx-1
                     Z(ix,iy,iz) = a(iz) * X(ix,iy,iz) + Y(ix,iy,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(ny-2)*(nx-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(ny-2)*(nx-2)
#endif
            END IF
         END DO
      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
C     
C     Module:  mmRestrictU
C     
C     Purpose: Transfer a 3-D grid function to the next-coarser grid 
C
C     Algorithm: 7-point weighted average (half-weighting)
C
C     Inputs:  XF: fine grid function
C
C     Outputs: XC: coarse grid function
C
C     Globals accessed:
C
C     Globals modified:
C
C     Calls:
C
C     Called by:
C
C     Flop count estimate:     nxf nyf nzf
C
C     Communication estimate:  nxf nyf + nyf nzf + nzf nxf
C     
C     Author:  James Bordner (bordner@ncsa.uiuc.edu)
C
C     Modification history: 

C=======================================================================
C     
C     MODULE:  SUBROUTINE mmZmaxpy
C     
C     PURPOSE: Compute Z = - a X + Y for vectors X,Y,Z and 1-D array a
C     
C     AUTHOR:  James Bordner (bordner@ncsa.uiuc.edu)
C     AUTHOR:  National Center for Supercomputing Applications
C     
C=======================================================================

      






















































































































































































#include "mgmpi.cpp"

      SUBROUTINE mmZmaxpy2 (
     &     Z,nxdz,nydz,nzdz,bndz
     &     ,a,
     &     X,nxdx,nydx,nzdx,bndx
     &     ,
     &     Y,nxdy,nydy,nzdy,bndy
     &     ,nx,ny,nz,lx,ly,lz,mask,level,
     &     icall)

      IMPLICIT NONE

      
      INTEGER nxdz,nydz,nzdz
      INTEGER bndz(3)
      REAL Z(nxdz,nydz,nzdz)
      
      INTEGER nxdx,nydx,nzdx
      INTEGER bndx(3)  
      REAL X(nxdx,nydx,nzdx)
      
      INTEGER nxdy,nydy,nzdy
      INTEGER bndy(3)
      REAL Y(nxdy,nydy,nzdy)
      REAL a(*)
      INTEGER nx,ny,nz
      INTEGER level
      LOGICAL lx,ly,lz,mask(*)

#include "mgmpi_internal.h"
#ifdef MPI_USED
      include "mpif.h"
#endif

      INTEGER ix,iy,iz

#ifndef MPI_USED
      DOUBLE PRECISION MPI_WTIME
      EXTERNAL MPI_WTIME
#endif

      
      INTEGER isub,icall
#ifdef PERF_EVAL
      isub = SB_ZAXPY2+level
      gtimestart(isub) = MPI_WTIME()
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   - gcomm(isub)
	gmflops(icall) = gmflops(icall) - gmflops(isub)
      END IF	
#endif

      IF (.NOT.lx) THEN
         DO ix=2,nx-1
            IF (mask(ix)) THEN
               DO iz=2,nz-1
                  DO iy=2,ny-1
                     Z(ix,iy,iz) = - a(ix) * X(ix,iy,iz) + Y(ix,iy,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(ny-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(ny-2)*(nz-2)
#endif
            END IF
         END DO
      ELSE IF (.NOT.ly) THEN
         DO iy=2,ny-1
            IF (mask(iy)) THEN
               DO iz=2,nz-1
                  DO ix=2,nx-1
                     Z(ix,iy,iz) = - a(iy) * X(ix,iy,iz) + Y(ix,iy,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(nx-2)*(nz-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(nx-2)*(nz-2)
#endif
            END IF
         END DO
      ELSE IF (.NOT.lz) THEN
         DO iz=2,nz-1
            IF (mask(iz)) THEN
               DO iy=2,ny-1
                  DO ix=2,nx-1
                     Z(ix,iy,iz) = - a(iz) * X(ix,iy,iz) + Y(ix,iy,iz)
                  END DO
               END DO
               
#ifdef PERF_EVAL
      gmflops(1)     =gmflops(1)      + 
     & 2*(ny-2)*(nx-2)
      gmflops(isub)=gmflops(isub) + 
     & 2*(ny-2)*(nx-2)
#endif
            END IF
         END DO
      END IF

      
#ifdef PERF_EVAL
      IF (icall.NE.0 .AND. icall.NE.isub) THEN
	gcomm(icall)   = gcomm(icall)   + gcomm(isub)
	gmflops(icall) = gmflops(icall) + gmflops(isub)
      END IF	
      IF (icall.NE.isub) THEN
          gtime(isub) = gtime(isub) + MPI_WTIME() - gtimestart(isub)
          gtimecount(isub) = gtimecount(isub) + 1
      END IF
#endif

      RETURN
      END
